@@ -1,31 +1,29 @@
 #include "Autofire.hh"
-#include "IntegerSetting.hh"
-#include "memory.hh"
 #include <algorithm>
 #include <cassert>
 
 namespace openmsx {
 
 Autofire::Autofire(CommandController& commandController,
-                   unsigned newMinInts, unsigned newMaxInts, string_ref name)
+                   unsigned newMinInts, unsigned newMaxInts, std::string_view name)
 	: min_ints(std::max(newMinInts, 1u))
 	, max_ints(std::max(newMaxInts, min_ints + 1))
-	, speedSetting(make_unique<IntegerSetting>(commandController, name,
-		"controls the speed of this autofire circuit", 0, 0, 100))
-	, clock(EmuTime::zero)
+	, speedSetting(commandController, name,
+		"controls the speed of this autofire circuit", 0, 0, 100)
+	, clock(EmuTime::zero())
 {
 	setClock();
-	speedSetting->attach(*this);
+	speedSetting.attach(*this);
 }
 
 Autofire::~Autofire()
 {
-	speedSetting->detach(*this);
+	speedSetting.detach(*this);
 }
 
 void Autofire::setClock()
 {
-	int speed = speedSetting->getInt();
+	int speed = speedSetting.getInt();
 	clock.setFreq(
 	    (2 * 50 * 60) / (max_ints - (speed * (max_ints - min_ints)) / 100));
 }
@@ -33,13 +31,13 @@ void Autofire::setClock()
 void Autofire::update(const Setting& setting)
 {
 	(void)setting;
-	assert(&setting == speedSetting.get());
+	assert(&setting == &speedSetting);
 	setClock();
 }
 
 bool Autofire::getSignal(EmuTime::param time)
 {
-	return speedSetting->getInt() == 0 ?
+	return speedSetting.getInt() == 0 ?
 		false : clock.getTicksTill(time) & 1;
 }
 
