@@ -1,59 +1,25 @@
 #include "CartridgeSlotManager.hh"
 #include "MSXMotherBoard.hh"
 #include "HardwareConfig.hh"
-#include "RecordedCommand.hh"
 #include "CommandException.hh"
-#include "InfoTopic.hh"
 #include "FileContext.hh"
 #include "TclObject.hh"
 #include "MSXException.hh"
-#include "StringOp.hh"
 #include "CliComm.hh"
 #include "unreachable.hh"
-#include "memory.hh"
+#include "one_of.hh"
+#include "outer.hh"
+#include "StringOp.hh"
 #include "xrange.hh"
 #include <cassert>
+#include <memory>
 
 using std::string;
 using std::vector;
 
 namespace openmsx {
 
-class CartCmd final : public RecordedCommand
-{
-public:
-	CartCmd(CartridgeSlotManager& manager, MSXMotherBoard& motherBoard,
-	        string_ref commandName);
-	void execute(array_ref<TclObject> tokens, TclObject& result,
-	             EmuTime::param time) override;
-	string help(const vector<string>& tokens) const override;
-	void tabCompletion(vector<string>& tokens) const override;
-	bool needRecord(array_ref<TclObject> tokens) const override;
-private:
-	const HardwareConfig* getExtensionConfig(string_ref cartname);
-	CartridgeSlotManager& manager;
-	CliComm& cliComm;
-};
-
-class CartridgeSlotInfo final : public InfoTopic
-{
-public:
-	CartridgeSlotInfo(InfoCommand& machineInfoCommand,
-	                  CartridgeSlotManager& manger);
-	void execute(array_ref<TclObject> tokens,
-	             TclObject& result) const override;
-	string help(const vector<string>& tokens) const override;
-private:
-	CartridgeSlotManager& manager;
-};
-
-
 // CartridgeSlotManager::Slot
-CartridgeSlotManager::Slot::Slot()
-	: config(nullptr), useCount(0), ps(0), ss(0)
-{
-}
-
 CartridgeSlotManager::Slot::~Slot()
 {
 	assert(!config);
@@ -75,9 +41,8 @@ bool CartridgeSlotManager::Slot::used(const HardwareConfig* allowed) const
 // CartridgeSlotManager
 CartridgeSlotManager::CartridgeSlotManager(MSXMotherBoard& motherBoard_)
 	: motherBoard(motherBoard_)
-	, cartCmd(make_unique<CartCmd>(*this, motherBoard, "cart"))
-	, extSlotInfo(make_unique<CartridgeSlotInfo>(
-		motherBoard.getMachineInfoCommand(), *this))
+	, cartCmd(*this, motherBoard, "cart")
+	, extSlotInfo(motherBoard.getMachineInfoCommand())
 {
 }
 
@@ -90,29 +55,24 @@ CartridgeSlotManager::~CartridgeSlotManager()
 	}
 }
 
-int CartridgeSlotManager::getSlotNum(string_ref slot)
+int CartridgeSlotManager::getSlotNum(std::string_view slot)
 {
-	if ((slot.size() == 1) && ('a' <= slot[0]) && (slot[0] <= 'p')) {
-		return -(1 + slot[0] - 'a');
+	if (slot.size() == 1) {
+		if (('0' <= slot[0]) && (slot[0] <= '3')) {
+			return slot[0] - '0';
+		} else if (('a' <= slot[0]) && (slot[0] <= 'p')) {
+			return -(1 + slot[0] - 'a');
+		} else if (slot[0] == 'X') {
+			return -256;
+		}
+	} else if (slot.size() == 2) {
+		if ((slot[0] == '?') && ('0' <= slot[1]) && (slot[1] <= '3')) {
+			return slot[1] - '0' - 128;
+		}
 	} else if (slot == "any") {
 		return -256;
-	} else if (slot == "X") {
-		return -256;
-	} else if ((slot.size() == 2) && (slot[0] == '?')) {
-		int result = slot[1] - '0';
-		if ((result < 0) || (4 <= result)) {
-			throw MSXException(
-				"Invalid slot specification: " + slot);
-		}
-		return result - 128;
-	} else {
-		int result = stoi(slot);
-		if ((result < 0) || (4 <= result)) {
-			throw MSXException(
-				"Invalid slot specification: " + slot);
-		}
-		return result;
 	}
+	throw MSXException("Invalid slot specification: ", slot);
 }
 
 void CartridgeSlotManager::createExternalSlot(int ps)
@@ -135,9 +95,9 @@ void CartridgeSlotManager::createExternalSlot(int ps, int ss)
 			extName[3] += slot;
 			motherBoard.getMSXCliComm().update(
 				CliComm::HARDWARE, slotName, "add");
-			slots[slot].cartCommand = make_unique<CartCmd>(
+			slots[slot].cartCommand = std::make_unique<CartCmd>(
 				*this, motherBoard, slotName);
-			slots[slot].extCommand = make_unique<ExtCmd>(
+			slots[slot].extCommand = std::make_unique<ExtCmd>(
 				motherBoard, extName);
 			return;
 		}
@@ -193,17 +153,33 @@ void CartridgeSlotManager::getSpecificSlot(unsigned slot, int& ps, int& ss) cons
 {
 	assert(slot < MAX_SLOTS);
 	if (!slots[slot].exists()) {
-		throw MSXException(StringOp::Builder() <<
-			"slot-" << char('a' + slot) << " not defined.");
+		throw MSXException("slot-", char('a' + slot), " not defined.");
 	}
 	if (slots[slot].used()) {
-		throw MSXException(StringOp::Builder() <<
-			"slot-" << char('a' + slot) << " already in use.");
+		throw MSXException("slot-", char('a' + slot), " already in use.");
 	}
 	ps = slots[slot].ps;
 	ss = slots[slot].ss;
 }
 
+int CartridgeSlotManager::allocateSpecificPrimarySlot(unsigned slot, const HardwareConfig& hwConfig)
+{
+	assert(slot < MAX_SLOTS);
+	if (!slots[slot].exists()) {
+		throw MSXException("slot-", char('a' + slot), " not defined.");
+	}
+	if (slots[slot].used()) {
+		throw MSXException("slot-", char('a' + slot), " already in use.");
+	}
+	if (slots[slot].ss != -1) {
+		throw MSXException("slot-", char('a' + slot), " is not a primary slot.");
+	}
+	assert(slots[slot].useCount == 0);
+	slots[slot].config = &hwConfig;
+	slots[slot].useCount = 1;
+	return slots[slot].ps;
+}
+
 void CartridgeSlotManager::getAnyFreeSlot(int& ps, int& ss) const
 {
 	// search for the lowest free slot
@@ -223,17 +199,15 @@ void CartridgeSlotManager::getAnyFreeSlot(int& ps, int& ss) const
 	}
 }
 
-void CartridgeSlotManager::allocatePrimarySlot(
-		int& ps, const HardwareConfig& hwConfig)
+int CartridgeSlotManager::allocateAnyPrimarySlot(const HardwareConfig& hwConfig)
 {
 	for (auto slot : xrange(MAX_SLOTS)) {
-		ps = slots[slot].ps;
 		if (slots[slot].exists() && (slots[slot].ss == -1) &&
 		    !slots[slot].used()) {
 			assert(slots[slot].useCount == 0);
 			slots[slot].config = &hwConfig;
 			slots[slot].useCount = 1;
-			return;
+			return slots[slot].ps;
 		}
 	}
 	throw MSXException("No free primary slot");
@@ -259,10 +233,10 @@ void CartridgeSlotManager::allocateSlot(
 				slots[slot].config = &hwConfig;
 			} else {
 				if (slots[slot].config != &hwConfig) {
-					throw MSXException(StringOp::Builder()
-						<< "Slot " << ps << '-' << ss
-						<< " already in use by "
-						<< slots[slot].config->getName());
+					throw MSXException(
+						"Slot ", ps, '-', ss,
+						" already in use by ",
+						slots[slot].config->getName());
 				}
 			}
 			++slots[slot].useCount;
@@ -303,18 +277,20 @@ bool CartridgeSlotManager::isExternalSlot(int ps, int ss, bool convert) const
 
 
 // CartCmd
-CartCmd::CartCmd(CartridgeSlotManager& manager_, MSXMotherBoard& motherBoard,
-                 string_ref commandName)
-	: RecordedCommand(motherBoard.getCommandController(),
-	                  motherBoard.getStateChangeDistributor(),
-	                  motherBoard.getScheduler(),
+CartridgeSlotManager::CartCmd::CartCmd(
+		CartridgeSlotManager& manager_, MSXMotherBoard& motherBoard_,
+		std::string_view commandName)
+	: RecordedCommand(motherBoard_.getCommandController(),
+	                  motherBoard_.getStateChangeDistributor(),
+	                  motherBoard_.getScheduler(),
 	                  commandName)
 	, manager(manager_)
-	, cliComm(motherBoard.getMSXCliComm())
+	, cliComm(motherBoard_.getMSXCliComm())
 {
 }
 
-const HardwareConfig* CartCmd::getExtensionConfig(string_ref cartname)
+const HardwareConfig* CartridgeSlotManager::CartCmd::getExtensionConfig(
+	std::string_view cartname)
 {
 	if (cartname.size() != 5) {
 		throw SyntaxError();
@@ -323,98 +299,99 @@ const HardwareConfig* CartCmd::getExtensionConfig(string_ref cartname)
 	return manager.slots[slot].config;
 }
 
-void CartCmd::execute(array_ref<TclObject> tokens, TclObject& result,
-                      EmuTime::param /*time*/)
+void CartridgeSlotManager::CartCmd::execute(
+	span<const TclObject> tokens, TclObject& result, EmuTime::param /*time*/)
 {
-	string_ref cartname = tokens[0].getString();
+	std::string_view cartname = tokens[0].getString();
 
 	// strip namespace qualification
 	//  TODO investigate whether it's a good idea to strip namespace at a
 	//       higher level for all commands. How does that interact with
 	//       the event recording feature?
 	auto pos = cartname.rfind("::");
-	if (pos != string_ref::npos) {
+	if (pos != std::string_view::npos) {
 		cartname = cartname.substr(pos + 2);
 	}
 	if (tokens.size() == 1) {
 		// query name of cartridge
 		auto* extConf = getExtensionConfig(cartname);
-		result.addListElement(cartname + ':');
-		result.addListElement(extConf ? extConf->getName() : "");
+		result.addListElement(strCat(cartname, ':'),
+		                      extConf ? extConf->getName() : string{});
 		if (!extConf) {
-			TclObject options;
-			options.addListElement("empty");
+			TclObject options = makeTclList("empty");
 			result.addListElement(options);
 		}
-	} else if ((tokens[1].getString() ==  "eject") ||
-	           (tokens[1].getString() == "-eject")) {
+	} else if (tokens[1] == one_of("eject", "-eject")) {
 		// remove cartridge (or extension)
-		if (tokens[1].getString() == "-eject") {
-			result.setString(
+		if (tokens[1] == "-eject") {
+			result =
 				"Warning: use of '-eject' is deprecated, "
-				"instead use the 'eject' subcommand");
+				"instead use the 'eject' subcommand";
 		}
 		if (auto* extConf = getExtensionConfig(cartname)) {
 			try {
 				manager.motherBoard.removeExtension(*extConf);
-				cliComm.update(CliComm::MEDIA, cartname, "");
+				cliComm.update(CliComm::MEDIA, cartname, {});
 			} catch (MSXException& e) {
-				throw CommandException("Can't remove cartridge: " +
+				throw CommandException("Can't remove cartridge: ",
 				                       e.getMessage());
 			}
 		}
 	} else {
 		// insert cartridge
 		auto slotname = (cartname.size() == 5)
-			? string(1, cartname[4])
+			? cartname.substr(4, 1)
 			: "any";
 		size_t extensionNameToken = 1;
-		if (tokens[1].getString() == "insert") {
+		if (tokens[1] == "insert") {
 			if (tokens.size() > 2) {
 				extensionNameToken = 2;
 			} else {
 				throw CommandException("Missing argument to insert subcommand");
 			}
 		}
-		array_ref<TclObject> options(std::begin(tokens) + extensionNameToken + 1,
-		                             std::end(tokens));
+		auto options = tokens.subspan(extensionNameToken + 1);
 		try {
-			string_ref romname = tokens[extensionNameToken].getString();
+			std::string_view romname = tokens[extensionNameToken].getString();
 			auto extension = HardwareConfig::createRomConfig(
-				manager.motherBoard, romname, slotname, options);
+				manager.motherBoard, string(romname), string(slotname), options);
 			if (slotname != "any") {
 				if (auto* extConf = getExtensionConfig(cartname)) {
 					// still a cartridge inserted, (try to) remove it now
 					manager.motherBoard.removeExtension(*extConf);
 				}
 			}
-			result.setString(manager.motherBoard.insertExtension(
-				"ROM", std::move(extension)));
+			result = manager.motherBoard.insertExtension(
+				"ROM", std::move(extension));
 			cliComm.update(CliComm::MEDIA, cartname, romname);
 		} catch (MSXException& e) {
-			throw CommandException(e.getMessage());
+			throw CommandException(std::move(e).getMessage());
 		}
 	}
 }
 
-string CartCmd::help(const vector<string>& tokens) const
+string CartridgeSlotManager::CartCmd::help(const vector<string>& tokens) const
 {
-	return tokens[0] + " eject              : remove the ROM cartridge from this slot\n" +
-	       tokens[0] + " insert <filename>  : insert ROM cartridge with <filename>\n" +
-	       tokens[0] + " <filename>         : insert ROM cartridge with <filename>\n" +
-	       tokens[0] + "                    : show which ROM cartridge is in this slot";
+	return strCat(
+		tokens[0], " eject              : remove the ROM cartridge from this slot\n",
+		tokens[0], " insert <filename>  : insert ROM cartridge with <filename>\n",
+		tokens[0], " <filename>         : insert ROM cartridge with <filename>\n",
+		tokens[0], "                    : show which ROM cartridge is in this slot\n",
+		"The following options are supported when inserting a cartridge:\n"
+		"-ips <filename>    : apply the given IPS patch to the ROM image\n"
+		"-romtype <romtype> : specify the ROM mapper type\n");
 }
 
-void CartCmd::tabCompletion(vector<string>& tokens) const
+void CartridgeSlotManager::CartCmd::tabCompletion(vector<string>& tokens) const
 {
 	vector<const char*> extra;
 	if (tokens.size() < 3) {
 		extra = { "eject", "insert" };
 	}
-	completeFileName(tokens, UserFileContext(), extra);
+	completeFileName(tokens, userFileContext(), extra);
 }
 
-bool CartCmd::needRecord(array_ref<TclObject> tokens) const
+bool CartridgeSlotManager::CartCmd::needRecord(span<const TclObject> tokens) const
 {
 	return tokens.size() > 1;
 }
@@ -422,16 +399,17 @@ bool CartCmd::needRecord(array_ref<TclObject> tokens) const
 
 // class CartridgeSlotInfo
 
-CartridgeSlotInfo::CartridgeSlotInfo(InfoCommand& machineInfoCommand,
-                                   CartridgeSlotManager& manager_)
+CartridgeSlotManager::CartridgeSlotInfo::CartridgeSlotInfo(
+		InfoCommand& machineInfoCommand)
 	: InfoTopic(machineInfoCommand, "external_slot")
-	, manager(manager_)
 {
 }
 
-void CartridgeSlotInfo::execute(array_ref<TclObject> tokens,
-                                TclObject& result) const
+void CartridgeSlotManager::CartridgeSlotInfo::execute(
+	span<const TclObject> tokens, TclObject& result) const
 {
+	checkNumArgs(tokens, Between{2, 3}, Prefix{2}, "?slot?");
+	auto& manager = OUTER(CartridgeSlotManager, extSlotInfo);
 	switch (tokens.size()) {
 	case 2: {
 		// return list of slots
@@ -445,17 +423,17 @@ void CartridgeSlotInfo::execute(array_ref<TclObject> tokens,
 	}
 	case 3: {
 		// return info on a particular slot
-		const auto& name = tokens[2].getString();
-		if ((name.size() != 5) || (!name.starts_with("slot"))) {
-			throw CommandException("Invalid slot name: " + name);
+		const auto& slotName = tokens[2].getString();
+		if ((slotName.size() != 5) || (!StringOp::startsWith(slotName, "slot"))) {
+			throw CommandException("Invalid slot name: ", slotName);
 		}
-		unsigned num = name[4] - 'a';
+		unsigned num = slotName[4] - 'a';
 		if (num >= CartridgeSlotManager::MAX_SLOTS) {
-			throw CommandException("Invalid slot name: " + name);
+			throw CommandException("Invalid slot name: ", slotName);
 		}
 		auto& slot = manager.slots[num];
 		if (!slot.exists()) {
-			throw CommandException("Slot '" + name + "' doesn't currently exist in this msx machine.");
+			throw CommandException("Slot '", slotName, "' doesn't currently exist in this msx machine.");
 		}
 		result.addListElement(slot.ps);
 		if (slot.ss == -1) {
@@ -466,16 +444,15 @@ void CartridgeSlotInfo::execute(array_ref<TclObject> tokens,
 		if (slot.config) {
 			result.addListElement(slot.config->getName());
 		} else {
-			result.addListElement("");
+			result.addListElement(std::string_view{});
 		}
 		break;
 	}
-	default:
-		throw SyntaxError();
 	}
 }
 
-string CartridgeSlotInfo::help(const vector<string>& /*tokens*/) const
+string CartridgeSlotManager::CartridgeSlotInfo::help(
+	const vector<string>& /*tokens*/) const
 {
 	return "Without argument: show list of available external slots.\n"
 	       "With argument: show primary and secondary slot number for "
