@@ -1,9 +1,7 @@
 #include "DebugDevice.hh"
 #include "Clock.hh"
 #include "FileOperations.hh"
-#include "FilenameSetting.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include <iostream>
 #include <iomanip>
 
@@ -13,17 +11,13 @@ namespace openmsx {
 
 DebugDevice::DebugDevice(const DeviceConfig& config)
 	: MSXDevice(config)
-{
-	const auto& outputFile = config.getChildData("filename", "stdout");
-	fileNameSetting = make_unique<FilenameSetting>(
+	, fileNameSetting(
 		getCommandController(), "debugoutput",
-		"name of the file the debugdevice outputs to", outputFile);
-	openOutput(fileNameSetting->getString());
-	reset(EmuTime::dummy());
-}
-
-DebugDevice::~DebugDevice()
+		"name of the file the debugdevice outputs to",
+		config.getChildData("filename", "stdout"))
 {
+	openOutput(fileNameSetting.getString());
+	reset(EmuTime::dummy());
 }
 
 void DebugDevice::reset(EmuTime::param /*time*/)
@@ -34,7 +28,7 @@ void DebugDevice::reset(EmuTime::param /*time*/)
 
 void DebugDevice::writeIO(word port, byte value, EmuTime::param time)
 {
-	const auto& newName = fileNameSetting->getString();
+	const auto& newName = fileNameSetting.getString();
 	if (newName != fileNameString) {
 		openOutput(newName);
 	}
@@ -57,7 +51,7 @@ void DebugDevice::writeIO(word port, byte value, EmuTime::param time)
 			break;
 		}
 		if (!(value & 0x40)){
-			(*outputstrm) << std::endl;
+			(*outputstrm) << '\n' << std::flush;
 		}
 		break;
 	case 1:
@@ -69,6 +63,7 @@ void DebugDevice::writeIO(word port, byte value, EmuTime::param time)
 			break;
 		case MULTIBYTE:
 			outputMultiByte(value);
+			break;
 		default:
 			break;
 		}
@@ -93,12 +88,12 @@ void DebugDevice::outputSingleByte(byte value, EmuTime::param time)
 		displayByte(tmp, ASC);
 		(*outputstrm) << "' ";
 	}
-	Clock<3579545> zero(EmuTime::zero);
+	Clock<3579545> zero(EmuTime::zero());
 	(*outputstrm) << "emutime: " << std::dec << zero.getTicksTill(time);
 	if ((modeParameter & 0x08) && ((value < ' ') || (value == 127))) {
 		displayByte(value, ASC); // do special effects
 	}
-	(*outputstrm) << std::endl;
+	(*outputstrm) << '\n' << std::flush;
 }
 
 void DebugDevice::outputMultiByte(byte value)
@@ -149,7 +144,7 @@ void DebugDevice::displayByte(byte value, DisplayType type)
 	}
 }
 
-void DebugDevice::openOutput(const string& name)
+void DebugDevice::openOutput(std::string_view name)
 {
 	fileNameString = name;
 	debugOut.close();
@@ -164,7 +159,7 @@ void DebugDevice::openOutput(const string& name)
 	}
 }
 
-static enum_string<DebugDevice::DebugMode> debugModeInfo[] = {
+static std::initializer_list<enum_string<DebugDevice::DebugMode>> debugModeInfo = {
 	{ "OFF",        DebugDevice::OFF },
 	{ "SINGLEBYTE", DebugDevice::SINGLEBYTE },
 	{ "MULTIBYTE",  DebugDevice::MULTIBYTE },
@@ -176,8 +171,8 @@ template<typename Archive>
 void DebugDevice::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.template serializeBase<MSXDevice>(*this);
-	ar.serialize("mode", mode);
-	ar.serialize("modeParameter", modeParameter);
+	ar.serialize("mode",          mode,
+	             "modeParameter", modeParameter);
 }
 INSTANTIATE_SERIALIZE_METHODS(DebugDevice);
 REGISTER_MSXDEVICE(DebugDevice, "DebugDevice");
