@@ -1,5 +1,3 @@
-// $Id: DummyDevice.cc 12527 2012-05-17 17:34:11Z m9710797 $
-
 #include "DummyDevice.hh"
 #include "unreachable.hh"
 
@@ -15,4 +13,9 @@ void DummyDevice::reset(EmuTime::param /*time*/)
 	UNREACHABLE;
 }
 
+void DummyDevice::getNameList(TclObject& /*result*/) const
+{
+	// keep empty list
+}
+
 } // namespace openmsx
