@@ -1,9 +1,7 @@
-// $Id: I8255.cc 8352 2008-11-12 18:39:08Z m9710797 $
-
 #include "I8255.hh"
 #include "I8255Interface.hh"
 #include "serialize.hh"
-#include "CliComm.hh"
+#include "unreachable.hh"
 
 namespace openmsx {
 
@@ -24,10 +22,10 @@ const int BIT_NR       = 0x0E;
 const int SET_RESET    = 0x01;
 
 
-I8255::I8255(I8255Interface& interf, EmuTime::param time, CliComm& cliComm_)
+I8255::I8255(I8255Interface& interf, EmuTime::param time,
+             StringSetting& invalidPpiModeSetting)
 	: interface(interf)
-	, cliComm(cliComm_)
-	, warningPrinted(false)
+	, ppiModeCallback(invalidPpiModeSetting)
 {
 	reset(time);
 }
@@ -41,6 +39,60 @@ void I8255::reset(EmuTime::param time)
 	                            DIRECTION_C0 | DIRECTION_C1, time); // all input
 }
 
+byte I8255::read(byte port, EmuTime::param time)
+{
+	switch (port) {
+	case 0:
+		return readPortA(time);
+	case 1:
+		return readPortB(time);
+	case 2:
+		return readPortC(time);
+	case 3:
+		return readControlPort(time);
+	default:
+		UNREACHABLE;
+		return 0; // avoid warning
+	}
+}
+
+byte I8255::peek(byte port, EmuTime::param time) const
+{
+	switch (port) {
+	case 0:
+		return peekPortA(time);
+	case 1:
+		return peekPortB(time);
+	case 2:
+		return peekPortC(time);
+	case 3:
+		return readControlPort(time);
+	default:
+		UNREACHABLE;
+		return 0; // avoid warning
+	}
+}
+
+void I8255::write(byte port, byte value, EmuTime::param time)
+{
+	switch (port) {
+	case 0:
+		writePortA(value, time);
+		break;
+	case 1:
+		writePortB(value, time);
+		break;
+	case 2:
+		writePortC(value, time);
+		break;
+	case 3:
+		writeControlPort(value, time);
+		break;
+	default:
+		UNREACHABLE;
+	}
+}
+
 byte I8255::readPortA(EmuTime::param time)
 {
 	switch (control & MODE_A) {
@@ -265,11 +317,8 @@ void I8255::writeControlPort(byte value, EmuTime::param time)
 	if (value & SET_MODE) {
 		// set new control mode
 		control = value;
-		if ((control & (MODE_A | MODE_B)) && !warningPrinted) {
-			warningPrinted = true;
-			cliComm.printWarning("Invalid PPI mode selected. "
-			"This is not yet correctly emulated. "
-			"On a real MSX this will most likely hang.");
+		if ((control & (MODE_A | MODE_B))) {
+			ppiModeCallback.execute();
 		}
 		outputPortA(latchPortA, time);
 		outputPortB(latchPortB, time);
@@ -310,10 +359,10 @@ void I8255::writeControlPort(byte value, EmuTime::param time)
 template<typename Archive>
 void I8255::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("latchPortA", latchPortA);
-	ar.serialize("latchPortB", latchPortB);
-	ar.serialize("latchPortC", latchPortC);
-	ar.serialize("control",    control);
+	ar.serialize("latchPortA", latchPortA,
+	             "latchPortB", latchPortB,
+	             "latchPortC", latchPortC,
+	             "control",    control);
 
 	// note: don't write to any output ports (is handled elsewhere)
 	//       don't serialize 'warningPrinted'
@@ -321,4 +370,3 @@ void I8255::serialize(Archive& ar, unsigned /*version*/)
 INSTANTIATE_SERIALIZE_METHODS(I8255);
 
 } // namespace openmsx
-
