@@ -3,13 +3,13 @@
 #include "ReadOnlySetting.hh"
 #include "CommandController.hh"
 #include "Timer.hh"
-#include "memory.hh"
+#include <memory>
 
 namespace openmsx {
 
 static std::string getLedName(LedStatus::Led led)
 {
-	static const char* const names[LedStatus::NUM_LEDS] = {
+	static constexpr const char* const names[LedStatus::NUM_LEDS] = {
 		"power", "caps", "kana", "pause", "turbo", "FDD"
 	};
 	return names[led];
@@ -27,15 +27,14 @@ LedStatus::LedStatus(
 	for (int i = 0; i < NUM_LEDS; ++i) {
 		ledValue[i] = false;
 		std::string name = getLedName(static_cast<Led>(i));
-		ledStatus[i] = make_unique<ReadOnlySetting>(
+		ledStatus[i] = std::make_unique<ReadOnlySetting>(
 			commandController, "led_" + name,
-			"Current status for LED: " + name, "off");
+			"Current status for LED: " + name,
+			TclObject("off"));
 	}
 }
 
-LedStatus::~LedStatus()
-{
-}
+LedStatus::~LedStatus() = default;
 
 void LedStatus::setLed(Led led, bool status)
 {
@@ -60,13 +59,10 @@ void LedStatus::setLed(Led led, bool status)
 	}
 }
 
-void LedStatus::handleEvent(Led led)
+void LedStatus::handleEvent(Led led) noexcept
 {
-	static const std::string ON  = "on";
-	static const std::string OFF = "off";
-	const std::string& str = ledValue[led] ? ON : OFF;
-
-	ledStatus[led]->setReadOnlyValue(str);
+	std::string_view str = ledValue[led] ? "on": "off";
+	ledStatus[led]->setReadOnlyValue(TclObject(str));
 	msxCliComm.update(CliComm::LED, getLedName(led), str);
 }
 
