@@ -7,25 +7,22 @@
 #include "MSXCPU.hh"
 #include "CacheLine.hh"
 #include "TclObject.hh"
-#include "StringOp.hh"
+#include "Math.hh"
 #include "MSXException.hh"
+#include "one_of.hh"
+#include "ranges.hh"
 #include "serialize.hh"
 #include "stl.hh"
 #include "unreachable.hh"
-#include <algorithm>
+#include "view.hh"
 #include <cassert>
 #include <cstring>
 #include <iterator> // for back_inserter
 
 using std::string;
-using std::vector;
 
 namespace openmsx {
 
-byte MSXDevice::unmappedRead[0x10000];
-byte MSXDevice::unmappedWrite[0x10000];
-
-
 MSXDevice::MSXDevice(const DeviceConfig& config, const string& name)
 	: deviceConfig(config)
 {
@@ -44,7 +41,7 @@ void MSXDevice::initName(const string& name)
 	if (getMotherBoard().findDevice(deviceName)) {
 		unsigned n = 0;
 		do {
-			deviceName = StringOp::Builder() << name << " (" << ++n << ')';
+			deviceName = strCat(name, " (", ++n, ')');
 		} while (getMotherBoard().findDevice(deviceName));
 	}
 }
@@ -77,24 +74,22 @@ void MSXDevice::staticInit()
 
 MSXMotherBoard& MSXDevice::getMotherBoard() const
 {
-       return getHardwareConfig().getMotherBoard();
+	return getHardwareConfig().getMotherBoard();
 }
 
 void MSXDevice::testRemove(Devices removed) const
 {
 	auto all = referencedBy;
-	sort(begin(all),     end(all));
-	sort(begin(removed), end(removed));
+	ranges::sort(all);
+	ranges::sort(removed);
 	Devices rest;
-	set_difference(begin(all), end(all), begin(removed), end(removed),
-	               back_inserter(rest));
+	ranges::set_difference(all, removed, back_inserter(rest));
 	if (!rest.empty()) {
-		StringOp::Builder msg;
-		msg << "Still in use by ";
+		string msg = "Still in use by";
 		for (auto& d : rest) {
-			msg << d->getName() << ' ';
+			strAppend(msg, ' ', d->getName());
 		}
-		throw MSXException(msg);
+		throw MSXException(std::move(msg));
 	}
 }
 
@@ -111,9 +106,9 @@ void MSXDevice::lockDevices()
 		auto* dev = getMotherBoard().findDevice(name);
 		if (!dev) {
 			throw MSXException(
-				"Unsatisfied dependency: '" + getName() +
-				"' depends on unavailable device '" +
-				name + "'.");
+				"Unsatisfied dependency: '", getName(),
+				"' depends on unavailable device '",
+				name, "'.");
 		}
 		references.push_back(dev);
 		dev->referencedBy.push_back(this);
@@ -123,7 +118,7 @@ void MSXDevice::lockDevices()
 void MSXDevice::unlockDevices()
 {
 	for (auto& r : references) {
-		r->referencedBy.erase(find_unguarded(r->referencedBy, this));
+		move_pop_back(r->referencedBy, rfind_unguarded(r->referencedBy, this));
 	}
 }
 
@@ -173,15 +168,24 @@ PluggingController& MSXDevice::getPluggingController() const
 void MSXDevice::registerSlots()
 {
 	MemRegions tmpMemRegions;
+	unsigned align = getBaseSizeAlignment();
+	assert(Math::ispow2(align));
 	for (auto& m : getDeviceConfig().getChildren("mem")) {
 		unsigned base = m->getAttributeAsInt("base");
 		unsigned size = m->getAttributeAsInt("size");
-		if ((base >= 0x10000) || (size > 0x10000)) {
+		if ((base >= 0x10000) || (size > 0x10000) || ((base + size) > 0x10000)) {
 			throw MSXException(
-				"Invalid memory specification for device " +
-				getName() + " should be in range "
+				"Invalid memory specification for device ",
+				getName(), " should be in range "
 				"[0x0000,0x10000).");
 		}
+		if (((base & (align - 1)) || (size & (align - 1))) &&
+		    !allowUnaligned()) {
+			throw MSXException(
+				"invalid memory specification for device ",
+				getName(), " should be aligned on at least 0x",
+				hex_string<4>(align), '.');
+		}
 		tmpMemRegions.emplace_back(base, size);
 	}
 	if (tmpMemRegions.empty()) {
@@ -198,7 +202,13 @@ void MSXDevice::registerSlots()
 		throw MSXException("Invalid memory specification");
 	}
 	if (secondaryConfig) {
-		ss = slotManager.getSlotNum(secondaryConfig->getAttribute("slot"));
+		const auto& ss_str = secondaryConfig->getAttribute("slot");
+		ss = slotManager.getSlotNum(ss_str);
+		if ((-16 <= ss) && (ss <= -1) && (ss != ps)) {
+			throw MSXException(
+				"Invalid secondary slot specification: \"",
+				ss_str, "\".");
+		}
 	} else {
 		ss = 0;
 	}
@@ -254,15 +264,14 @@ void MSXDevice::registerSlots()
 	//  - Fix the slot number so that it remains the same after a
 	//    savestate/loadstate.
 	assert(primaryConfig);
-	primaryConfig->setAttribute("slot", StringOp::toString(ps));
+	primaryConfig->setAttribute("slot", strCat(ps));
 	if (secondaryConfig) {
-		string slot = (ss == -1) ? "X" : StringOp::toString(ss);
+		string slot = (ss == -1) ? "X" : strCat(ss);
 		secondaryConfig->setAttribute("slot", slot);
 	} else {
 		if (ss != -1) {
 			throw MSXException(
-				"Missing <secondary> tag for device" +
-				getName());
+				"Missing <secondary> tag for device", getName());
 		}
 	}
 
@@ -286,9 +295,9 @@ void MSXDevice::unregisterSlots()
 	if (memRegions.empty()) return;
 
 	int logicalSS = (ss == -1) ? 0 : ss;
-	for (auto& r : memRegions) {
+	for (const auto& [base, size] : memRegions) {
 		getCPUInterface().unregisterMemDevice(
-			*this, ps, logicalSS, r.first, r.second);
+			*this, ps, logicalSS, base, size);
 	}
 
 	// See comments above about allocateSlot() for more details:
@@ -305,13 +314,11 @@ void MSXDevice::getVisibleMemRegion(unsigned& base, unsigned& size) const
 		size = 0;
 		return;
 	}
-	auto it = begin(memRegions);
-	unsigned lowest  = it->first;
-	unsigned highest = it->first + it->second;
-	for (++it; it != end(memRegions); ++it) {
-		lowest  = std::min(lowest,  it->first);
-		highest = std::max(highest, it->first + it->second);
-	}
+
+	auto lowest  = min_value(view::transform(memRegions,
+		[](auto& r) { return r.first; }));
+	auto highest = max_value(view::transform(memRegions,
+		[](auto& r) { return r.first + r.second; }));
 	assert(lowest <= highest);
 	base = lowest;
 	size = highest - lowest;
@@ -320,19 +327,19 @@ void MSXDevice::getVisibleMemRegion(unsigned& base, unsigned& size) const
 void MSXDevice::registerPorts()
 {
 	for (auto& i : getDeviceConfig().getChildren("io")) {
-		unsigned base = StringOp::stringToInt(i->getAttribute("base"));
-		unsigned num  = StringOp::stringToInt(i->getAttribute("num"));
+		unsigned base = i->getAttributeAsInt("base");
+		unsigned num  = i->getAttributeAsInt("num");
 		const auto& type = i->getAttribute("type", "IO");
 		if (((base + num) > 256) || (num == 0) ||
-		    ((type != "I") && (type != "O") && (type != "IO"))) {
+		    (type != one_of("I", "O", "IO"))) {
 			throw MSXException("Invalid IO port specification");
 		}
 		for (unsigned port = base; port < base + num; ++port) {
-			if ((type == "I") || (type == "IO")) {
+			if (type == one_of("I", "IO")) {
 				getCPUInterface().register_IO_In(port, this);
 				inPorts.push_back(port);
 			}
-			if ((type == "O") || (type == "IO")) {
+			if (type == one_of("O", "IO")) {
 				getCPUInterface().register_IO_Out(port, this);
 				outPorts.push_back(port);
 			}
@@ -383,7 +390,7 @@ void MSXDevice::getNameList(TclObject& result) const
 
 void MSXDevice::getDeviceInfo(TclObject& result) const
 {
-	result.addListElement(getDeviceConfig().getName());
+	result.addDictKeyValue("type", getDeviceConfig().getName());
 	getExtraDeviceInfo(result);
 }
 
@@ -392,23 +399,21 @@ void MSXDevice::getExtraDeviceInfo(TclObject& /*result*/) const
 	// nothing
 }
 
-
-byte MSXDevice::readIO(word port, EmuTime::param /*time*/)
+unsigned MSXDevice::getBaseSizeAlignment() const
 {
-	(void)port;
-	PRT_DEBUG("MSXDevice::readIO (0x" << std::hex << int(port & 0xFF)
-	          << std::dec << ") : No device implementation.");
+	return CacheLine::SIZE;
+}
+
+
+byte MSXDevice::readIO(word /*port*/, EmuTime::param /*time*/)
+{
+	// read from unmapped IO
 	return 0xFF;
 }
 
-void MSXDevice::writeIO(word port, byte value, EmuTime::param /*time*/)
+void MSXDevice::writeIO(word /*port*/, byte /*value*/, EmuTime::param /*time*/)
 {
-	(void)port;
-	(void)value;
-	PRT_DEBUG("MSXDevice::writeIO(port 0x" << std::hex << int(port & 0xFF)
-	          << std::dec << ",value " << int(value)
-	          << ") : No device implementation.");
-	// do nothing
+	// write to unmapped IO, do nothing
 }
 
 byte MSXDevice::peekIO(word /*port*/, EmuTime::param /*time*/) const
@@ -417,11 +422,9 @@ byte MSXDevice::peekIO(word /*port*/, EmuTime::param /*time*/) const
 }
 
 
-byte MSXDevice::readMem(word address, EmuTime::param /*time*/)
+byte MSXDevice::readMem(word /*address*/, EmuTime::param /*time*/)
 {
-	(void)address;
-	PRT_DEBUG("MSXDevice: read from unmapped memory " << std::hex <<
-	          int(address) << std::dec);
+	// read from unmapped memory
 	return 0xFF;
 }
 
@@ -430,13 +433,10 @@ const byte* MSXDevice::getReadCacheLine(word /*start*/) const
 	return nullptr; // uncacheable
 }
 
-void MSXDevice::writeMem(word address, byte /*value*/,
-                            EmuTime::param /*time*/)
+void MSXDevice::writeMem(word /*address*/, byte /*value*/,
+                         EmuTime::param /*time*/)
 {
-	(void)address;
-	PRT_DEBUG("MSXDevice: write to unmapped memory " << std::hex <<
-	          int(address) << std::dec);
-	// do nothing
+	// write to unmapped memory, do nothing
 }
 
 byte MSXDevice::peekMem(word address, EmuTime::param /*time*/) const
@@ -446,7 +446,7 @@ byte MSXDevice::peekMem(word address, EmuTime::param /*time*/) const
 		word offset = address & CacheLine::LOW;
 		return cache[offset];
 	} else {
-		PRT_DEBUG("MSXDevice: peek not supported for this device");
+		// peek not supported for this device
 		return 0xFF;
 	}
 }
@@ -457,14 +457,75 @@ void MSXDevice::globalWrite(word /*address*/, byte /*value*/,
 	UNREACHABLE;
 }
 
+void MSXDevice::globalRead(word /*address*/, EmuTime::param /*time*/)
+{
+	UNREACHABLE;
+}
+
 byte* MSXDevice::getWriteCacheLine(word /*start*/) const
 {
 	return nullptr; // uncacheable
 }
 
-void MSXDevice::invalidateMemCache(word start, unsigned size)
+
+// calls 'action(<start2>, <size2>, args..., ps, ss)'
+// with 'start', 'size' clipped to each of the ranges in 'memRegions'
+template<typename Action, typename... Args>
+void MSXDevice::clip(unsigned start, unsigned size, Action action, Args... args)
 {
-	getCPU().invalidateMemCache(start, size);
+	int ss2 = (ss != -1) ? ss : 0;
+	unsigned end = start + size;
+	for (auto [base, fullBsize] : memRegions) {
+		// split on 16kB boundaries
+		while (fullBsize > 0) {
+			unsigned bsize = std::min(fullBsize, ((base + 0x4000) & ~0x3fff) - base);
+
+			unsigned baseEnd = base + bsize;
+			// intersect [start, end) with [base, baseEnd) -> [clipStart, clipEnd)
+			unsigned clipStart = std::max(start, base);
+			unsigned clipEnd   = std::min(end, baseEnd);
+			if (clipStart < clipEnd) { // non-empty
+				unsigned clipSize = clipEnd - clipStart;
+				action(clipStart, clipSize, args..., ps, ss2);
+			}
+
+			base += bsize;
+			fullBsize -= bsize;
+		}
+	}
+}
+
+void MSXDevice::invalidateDeviceRWCache(unsigned start, unsigned size)
+{
+	clip(start, size, [&](auto... args) { getCPUInterface().invalidateRWCache(args...); });
+}
+void MSXDevice::invalidateDeviceRCache(unsigned start, unsigned size)
+{
+	clip(start, size, [&](auto... args) { getCPUInterface().invalidateRCache(args...); });
+}
+void MSXDevice::invalidateDeviceWCache(unsigned start, unsigned size)
+{
+	clip(start, size, [&](auto... args) { getCPUInterface().invalidateWCache(args...); });
+}
+
+void MSXDevice::fillDeviceRWCache(unsigned start, unsigned size, byte* rwData)
+{
+	fillDeviceRWCache(start, size, rwData, rwData);
+}
+void MSXDevice::fillDeviceRWCache(unsigned start, unsigned size, const byte* rData, byte* wData)
+{
+	assert(!allowUnaligned());
+	clip(start, size, [&](auto... args) { getCPUInterface().fillRWCache(args...); }, rData, wData);
+}
+void MSXDevice::fillDeviceRCache(unsigned start, unsigned size, const byte* rData)
+{
+	assert(!allowUnaligned());
+	clip(start, size, [&](auto... args) { getCPUInterface().fillRCache(args...); }, rData);
+}
+void MSXDevice::fillDeviceWCache(unsigned start, unsigned size, byte* wData)
+{
+	assert(!allowUnaligned());
+	clip(start, size, [&](auto... args) { getCPUInterface().fillWCache(args...); }, wData);
 }
 
 template<typename Archive>
