@@ -1,9 +1,10 @@
 #include "MSXPrinterPort.hh"
 #include "DummyPrinterPortDevice.hh"
 #include "checked_cast.hh"
+#include "outer.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include "unreachable.hh"
+#include <memory>
 
 using std::string;
 
@@ -12,17 +13,14 @@ namespace openmsx {
 MSXPrinterPort::MSXPrinterPort(const DeviceConfig& config)
 	: MSXDevice(config)
 	, Connector(MSXDevice::getPluggingController(), "printerport",
-	            make_unique<DummyPrinterPortDevice>())
+	            std::make_unique<DummyPrinterPortDevice>())
+	, debuggable(getMotherBoard(), MSXDevice::getName())
 {
 	data = 255;     // != 0;
 	strobe = false; // != true;
 	reset(getCurrentTime());
 }
 
-MSXPrinterPort::~MSXPrinterPort()
-{
-}
-
 void MSXPrinterPort::reset(EmuTime::param time)
 {
 	writeData(0, time);    // TODO check this
@@ -70,12 +68,12 @@ void MSXPrinterPort::writeData(byte newData, EmuTime::param time)
 	}
 }
 
-const string MSXPrinterPort::getDescription() const
+std::string_view MSXPrinterPort::getDescription() const
 {
 	return "MSX Printer port";
 }
 
-string_ref MSXPrinterPort::getClass() const
+std::string_view MSXPrinterPort::getClass() const
 {
 	return "Printer Port";
 }
@@ -92,13 +90,32 @@ PrinterPortDevice& MSXPrinterPort::getPluggedPrintDev() const
 	return *checked_cast<PrinterPortDevice*>(&getPlugged());
 }
 
+
+MSXPrinterPort::Debuggable::Debuggable(MSXMotherBoard& motherBoard_, const std::string& name_)
+	: SimpleDebuggable(motherBoard_, name_, "Printer Port", 2)
+{
+}
+
+byte MSXPrinterPort::Debuggable::read(unsigned address)
+{
+	auto& pport = OUTER(MSXPrinterPort, debuggable);
+	return (address == 0) ? pport.strobe : pport.data;
+}
+
+void MSXPrinterPort::Debuggable::write(unsigned address, byte value)
+{
+	auto& pport = OUTER(MSXPrinterPort, debuggable);
+	pport.writeIO(address, value, pport.getCurrentTime());
+}
+
+
 template<typename Archive>
 void MSXPrinterPort::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.template serializeBase<MSXDevice>(*this);
 	ar.template serializeBase<Connector>(*this);
-	ar.serialize("strobe", strobe);
-	ar.serialize("data", data);
+	ar.serialize("strobe", strobe,
+	             "data",   data);
 	// TODO force writing data to port??
 }
 INSTANTIATE_SERIALIZE_METHODS(MSXPrinterPort);
