@@ -2,37 +2,20 @@
 #include "MSXCPU.hh"
 #include "MSXMotherBoard.hh"
 #include "PanasonicMemory.hh"
-#include "FirmwareSwitch.hh"
-#include "SimpleDebuggable.hh"
+#include "outer.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
-#include "memory.hh"
 
 namespace openmsx {
 
-class S1990Debuggable final : public SimpleDebuggable
-{
-public:
-	S1990Debuggable(MSXMotherBoard& motherBoard, MSXS1990& s1990);
-	byte read(unsigned address) override;
-	void write(unsigned address, byte value) override;
-private:
-	MSXS1990& s1990;
-};
-
-
 MSXS1990::MSXS1990(const DeviceConfig& config)
 	: MSXDevice(config)
-	, firmwareSwitch(make_unique<FirmwareSwitch>(config))
-	, debuggable(make_unique<S1990Debuggable>(getMotherBoard(), *this))
+	, debuggable(getMotherBoard(), getName())
+	, firmwareSwitch(config)
 {
 	reset(EmuTime::dummy());
 }
 
-MSXS1990::~MSXS1990()
-{
-}
-
 void MSXS1990::reset(EmuTime::param /*time*/)
 {
 	registerSelect = 0; // TODO check this
@@ -73,10 +56,9 @@ void MSXS1990::writeIO(word port, byte value, EmuTime::param /*time*/)
 
 byte MSXS1990::readRegister(byte reg) const
 {
-	PRT_DEBUG("S1990: read reg " << int(reg));
 	switch (reg) {
 	case 5:
-		return firmwareSwitch->getStatus() ? 0x40 : 0x00;
+		return firmwareSwitch.getStatus() ? 0x40 : 0x00;
 	case 6:
 		return cpuStatus;
 	case 13:
@@ -104,27 +86,27 @@ void MSXS1990::setCPUStatus(byte value)
 	cpuStatus = value & 0x60;
 	getCPU().setActiveCPU((cpuStatus & 0x20) ? MSXCPU::CPU_Z80 :
 	                                           MSXCPU::CPU_R800);
-	bool dram = (cpuStatus & 0x40) ? false : true;
+	bool dram = (cpuStatus & 0x40) == 0;
 	getCPU().setDRAMmode(dram);
 	getMotherBoard().getPanasonicMemory().setDRAM(dram);
 	// TODO bit 7 -> reset MSX ?????
 }
 
 
-S1990Debuggable::S1990Debuggable(MSXMotherBoard& motherBoard, MSXS1990& s1990_)
-	: SimpleDebuggable(motherBoard, s1990_.getName() + " regs",
-	                   "S1990 registers", 16)
-	, s1990(s1990_)
+MSXS1990::Debuggable::Debuggable(MSXMotherBoard& motherBoard_, const std::string& name_)
+	: SimpleDebuggable(motherBoard_, name_ + " regs", "S1990 registers", 16)
 {
 }
 
-byte S1990Debuggable::read(unsigned address)
+byte MSXS1990::Debuggable::read(unsigned address)
 {
+	auto& s1990 = OUTER(MSXS1990, debuggable);
 	return s1990.readRegister(address);
 }
 
-void S1990Debuggable::write(unsigned address, byte value)
+void MSXS1990::Debuggable::write(unsigned address, byte value)
 {
+	auto& s1990 = OUTER(MSXS1990, debuggable);
 	s1990.writeRegister(address, value);
 }
 
@@ -133,8 +115,8 @@ template<typename Archive>
 void MSXS1990::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.template serializeBase<MSXDevice>(*this);
-	ar.serialize("registerSelect", registerSelect);
-	ar.serialize("cpuStatus", cpuStatus);
+	ar.serialize("registerSelect", registerSelect,
+	             "cpuStatus",      cpuStatus);
 	if (ar.isLoader()) {
 		setCPUStatus(cpuStatus);
 	}
