@@ -8,13 +8,13 @@
 #include "Math.hh"
 #include "MemBuffer.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include "vla.hh"
 #include <algorithm>
-#include <vector>
 #include <cassert>
 #include <cmath>
 #include <cstring>
+#include <memory>
+#include <vector>
 
 using std::max;
 using std::min;
@@ -26,7 +26,6 @@ class Paper
 {
 public:
 	Paper(unsigned x, unsigned y, double dotSizeX, double dotSizeY);
-	~Paper();
 
 	std::string save() const;
 	void setDotSize(double sizeX, double sizeY);
@@ -47,15 +46,6 @@ private:
 };
 
 
-PrinterCore::PrinterCore()
-	: toPrint(0), prevStrobe(true)
-{
-}
-
-PrinterCore::~PrinterCore()
-{
-}
-
 bool PrinterCore::getStatus(EmuTime::param /*time*/)
 {
 	return false; // false = low = ready
@@ -122,16 +112,11 @@ ImagePrinter::ImagePrinter(MSXMotherBoard& motherBoard_, bool graphicsHiLo_)
 	: motherBoard(motherBoard_)
 	, graphicsHiLo(graphicsHiLo_)
 {
-	auto& info = motherBoard.getSharedStuff("print-resolution");
-	if (info.counter == 0) {
-		assert(!info.stuff);
-		info.stuff = new IntegerSetting(
-		    motherBoard.getCommandController(), "print-resolution",
-		    "resolution of the output image of emulated dot matrix printer in DPI",
-		    300, 72, 1200);
-	}
-	++info.counter;
-	dpiSetting = reinterpret_cast<IntegerSetting*>(info.stuff);
+	dpiSetting = motherBoard.getSharedStuff<IntegerSetting>(
+		"print-resolution",
+		motherBoard.getCommandController(), "print-resolution",
+		"resolution of the output image of emulated dot matrix printer in DPI",
+		300, 72, 1200);
 
 	letterQuality = false;
 	bold = false;
@@ -168,16 +153,6 @@ ImagePrinter::ImagePrinter(MSXMotherBoard& motherBoard_, bool graphicsHiLo_)
 ImagePrinter::~ImagePrinter()
 {
 	flushEmulatedPrinter();
-
-	auto& info = motherBoard.getSharedStuff("print-resolution");
-	assert(info.counter);
-	assert(dpiSetting);
-	assert(dpiSetting == info.stuff);
-	--info.counter;
-	if (info.counter == 0) {
-		delete dpiSetting;
-		info.stuff = nullptr;
-	}
 }
 
 void ImagePrinter::write(byte data)
@@ -277,16 +252,16 @@ void ImagePrinter::ensurePrintPage()
 		// A4 paper format (210mm x 297mm) at 300dpi
 		// TODO make this configurable
 		int dpi = dpiSetting->getInt();
-		unsigned paperSizeX = unsigned((210 / 25.4) * dpi);
-		unsigned paperSizeY = unsigned((297 / 25.4) * dpi);
+		auto paperSizeX = unsigned((210 / 25.4) * dpi);
+		auto paperSizeY = unsigned((297 / 25.4) * dpi);
 
 		unsigned dotsX, dotsY;
 		getNumberOfDots(dotsX, dotsY);
 		pixelSizeX = double(paperSizeX) / dotsX;
 		pixelSizeY = double(paperSizeY) / dotsY;
 
-		paper = make_unique<Paper>(paperSizeX, paperSizeY,
-		                           pixelSizeX, pixelSizeY);
+		paper = std::make_unique<Paper>(paperSizeX, paperSizeY,
+		                                pixelSizeX, pixelSizeY);
 	}
 }
 
@@ -297,10 +272,10 @@ void ImagePrinter::flushEmulatedPrinter()
 			try {
 				string filename = paper->save();
 				motherBoard.getMSXCliComm().printInfo(
-					"Printed to " + filename);
+					"Printed to ", filename);
 			} catch (MSXException& e) {
 				motherBoard.getMSXCliComm().printWarning(
-					"Failed to print: " + e.getMessage());
+					"Failed to print: ", e.getMessage());
 			}
 			printAreaTop = -1.0;
 			printAreaBottom = 0.0;
@@ -382,7 +357,7 @@ void ImagePrinter::printVisibleCharacter(byte data)
 // class ImagePrinterMSX
 
 // MSX-Font taken from NMS8250 BIOS ROM
-static const byte MSXFontRaw[256 * 8] = {
+constexpr byte MSXFontRaw[256 * 8] = {
 	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  // 0
 	0x3C, 0x42, 0xA5, 0x81, 0xA5, 0x99, 0x42, 0x3C,  // 1
 	0x3C, 0x7E, 0xDB, 0xFF, 0xFF, 0xDB, 0x66, 0x3C,  // 2
@@ -643,8 +618,8 @@ static const byte MSXFontRaw[256 * 8] = {
 
 static byte MSXFont[256 * 9];
 
-ImagePrinterMSX::ImagePrinterMSX(MSXMotherBoard& motherBoard)
-	: ImagePrinter(motherBoard, true)
+ImagePrinterMSX::ImagePrinterMSX(MSXMotherBoard& motherBoard_)
+	: ImagePrinter(motherBoard_, true)
 {
 	msxPrnSetFont(MSXFontRaw);
 	resetEmulatedPrinter();
@@ -656,7 +631,7 @@ const string& ImagePrinterMSX::getName() const
 	return name;
 }
 
-string_ref ImagePrinterMSX::getDescription() const
+std::string_view ImagePrinterMSX::getDescription() const
 {
 	// TODO which printer type
 	return "Emulate MSX printer, prints to image.";
@@ -911,7 +886,7 @@ void ImagePrinterMSX::processCharacter(byte data)
 					hpos = leftBorder;
 				}
 				break;
-			case 9: { // HAT: Horizontal tabulator
+			case 9: // HAT: Horizontal tabulator
 				// TODO: fix for other font-sizes
 				hpos = ((unsigned(hpos) + 64 - leftBorder) & ~63)
 				     + leftBorder;
@@ -919,8 +894,7 @@ void ImagePrinterMSX::processCharacter(byte data)
 					break;
 				}
 				hpos = leftBorder;
-				// fall thru: CR/LF
-			}
+				[[fallthrough]];
 			case 10: // LF: Carriage return + Line feed
 			case 11: // VT: Vertical tabulator (like LF)
 				//hpos = leftBorder;
@@ -966,7 +940,7 @@ REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, ImagePrinterMSX, "ImagePrinterMSX");
 
 // class ImagePrinterEpson
 
-static const byte EpsonFontRom[] = {
+constexpr byte EpsonFontRom[] = {
 	0x8b, 0x04, 0x0a, 0x20, 0x8a, 0x60, 0x0a, 0x20, 0x1c, 0x02, 0x00, 0x00, //   0
 	0x8b, 0x1c, 0x22, 0x08, 0xa2, 0x48, 0x22, 0x08, 0x22, 0x18, 0x00, 0x00, //   1
 	0x9b, 0x00, 0x3c, 0x00, 0x82, 0x40, 0x02, 0x00, 0x3c, 0x02, 0x00, 0x00, //   2
@@ -1225,8 +1199,8 @@ static const byte EpsonFontRom[] = {
 	0x8b, 0x1a, 0x24, 0x42, 0x08, 0x92, 0x20, 0x84, 0x48, 0xb0, 0x00, 0x00  // 255
 };
 
-ImagePrinterEpson::ImagePrinterEpson(MSXMotherBoard& motherBoard)
-	: ImagePrinter(motherBoard, false)
+ImagePrinterEpson::ImagePrinterEpson(MSXMotherBoard& motherBoard_)
+	: ImagePrinter(motherBoard_, false)
 {
 	resetEmulatedPrinter();
 }
@@ -1237,7 +1211,7 @@ const string& ImagePrinterEpson::getName() const
 	return name;
 }
 
-string_ref ImagePrinterEpson::getDescription() const
+std::string_view ImagePrinterEpson::getDescription() const
 {
 	return "Emulate Epson FX80 printer, prints to image.";
 }
@@ -1607,7 +1581,7 @@ void ImagePrinterEpson::processCharacter(byte data)
 				hpos = leftBorder;
 			}
 			break;
-		case 9: { // Horizontal TAB
+		case 9: // Horizontal TAB
 			// TODO: fix for other font-sizes
 			hpos = ((unsigned(hpos) + 64 - leftBorder) & ~63)
 			     + leftBorder;
@@ -1615,8 +1589,7 @@ void ImagePrinterEpson::processCharacter(byte data)
 				break;
 			}
 			hpos = leftBorder;
-			// fall thru: CR/LF
-		}
+			[[fallthrough]];
 		case 10: // Line Feed
 		case 11: // Vertical TAB
 			vpos += lineFeed;
@@ -1684,10 +1657,6 @@ Paper::Paper(unsigned x, unsigned y, double dotSizeX, double dotSizeY)
 	setDotSize(dotSizeX, dotSizeY);
 }
 
-Paper::~Paper()
-{
-}
-
 string Paper::save() const
 {
 	string filename = FileOperations::getNextNumberedFileName(
@@ -1700,10 +1669,10 @@ string Paper::save() const
 	return filename;
 }
 
-void Paper::setDotSize(double sizeX, double sizeY)
+void Paper::setDotSize(double dotSizeX, double dotSizeY)
 {
-	radiusX = sizeX / 2.0;
-	radiusY = sizeY / 2.0;
+	radiusX = dotSizeX / 2.0;
+	radiusY = dotSizeY / 2.0;
 
 	int rx = int(16 * radiusX);
 	int ry = int(16 * radiusY);
