@@ -1,18 +1,15 @@
 #include "PrinterPortLogger.hh"
 #include "PlugException.hh"
 #include "FileException.hh"
-#include "File.hh"
-#include "FilenameSetting.hh"
 #include "serialize.hh"
-#include "memory.hh"
 
 namespace openmsx {
 
 PrinterPortLogger::PrinterPortLogger(CommandController& commandController)
-	: logFilenameSetting(make_unique<FilenameSetting>(
+	: logFilenameSetting(
 		commandController, "printerlogfilename",
 		"filename of the file where the printer output is logged to",
-		"printer.log"))
+		"printer.log")
 	, toPrint(0) // Initialize to avoid a static analysis (cppcheck) warning.
 		     // For correctness it's not strictly needed to initialize
 		     // this variable. But understanding why exactly it's not
@@ -23,10 +20,6 @@ PrinterPortLogger::PrinterPortLogger(CommandController& commandController)
 {
 }
 
-PrinterPortLogger::~PrinterPortLogger()
-{
-}
-
 bool PrinterPortLogger::getStatus(EmuTime::param /*time*/)
 {
 	return false; // false = low = ready
@@ -34,10 +27,10 @@ bool PrinterPortLogger::getStatus(EmuTime::param /*time*/)
 
 void PrinterPortLogger::setStrobe(bool strobe, EmuTime::param /*time*/)
 {
-	if (file && !strobe && prevStrobe) {
+	if (file.is_open() && !strobe && prevStrobe) {
 		// falling edge
-		file->write(&toPrint, 1);
-		file->flush(); // optimize when it turns out flushing
+		file.write(&toPrint, 1);
+		file.flush(); // optimize when it turns out flushing
 		               // every time is too slow
 	}
 	prevStrobe = strobe;
@@ -52,17 +45,17 @@ void PrinterPortLogger::plugHelper(
 		Connector& /*connector*/, EmuTime::param /*time*/)
 {
 	try {
-		file = make_unique<File>(logFilenameSetting->getString(),
-		                         File::TRUNCATE);
+		file = File(logFilenameSetting.getString(),
+		            File::TRUNCATE);
 	} catch (FileException& e) {
-		throw PlugException("Couldn't plug printer logger: " +
+		throw PlugException("Couldn't plug printer logger: ",
 		                    e.getMessage());
 	}
 }
 
 void PrinterPortLogger::unplugHelper(EmuTime::param /*time*/)
 {
-	file.reset();
+	file.close();
 }
 
 const std::string& PrinterPortLogger::getName() const
@@ -71,7 +64,7 @@ const std::string& PrinterPortLogger::getName() const
 	return name;
 }
 
-string_ref PrinterPortLogger::getDescription() const
+std::string_view PrinterPortLogger::getDescription() const
 {
 	return	"Log everything that is sent to the printer port to a "
 		"file. The filename can be set with the "
