@@ -3,9 +3,7 @@
 #include "DeviceConfig.hh"
 #include "XMLElement.hh"
 #include "serialize.hh"
-#include "memory.hh"
-
-using std::unique_ptr;
+#include <memory>
 
 namespace openmsx {
 
@@ -14,9 +12,7 @@ PrinterPortSimpl::PrinterPortSimpl(const HardwareConfig& hwConf_)
 {
 }
 
-PrinterPortSimpl::~PrinterPortSimpl()
-{
-}
+PrinterPortSimpl::~PrinterPortSimpl() = default;
 
 bool PrinterPortSimpl::getStatus(EmuTime::param /*time*/)
 {
@@ -43,8 +39,8 @@ static XMLElement createXML()
 void PrinterPortSimpl::createDAC()
 {
 	static XMLElement xml = createXML();
-	dac = make_unique<DACSound8U>("simpl", getDescription(),
-	                              DeviceConfig(hwConf, xml));
+	dac = std::make_unique<DACSound8U>("simpl", getDescription(),
+	                                   DeviceConfig(hwConf, xml));
 }
 
 void PrinterPortSimpl::plugHelper(Connector& /*connector*/, EmuTime::param /*time*/)
@@ -63,7 +59,7 @@ const std::string& PrinterPortSimpl::getName() const
 	return name;
 }
 
-string_ref PrinterPortSimpl::getDescription() const
+std::string_view PrinterPortSimpl::getDescription() const
 {
 	return "Play samples via your printer port.";
 }
