@@ -1,8 +1,6 @@
 #include "RP5C01.hh"
-#include "EnumSetting.hh"
 #include "SRAM.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include <cassert>
 #include <ctime>
 
@@ -11,61 +9,51 @@ namespace openmsx {
 // TODO  ALARM is not implemented        (not connected on MSX)
 // TODO  1Hz 16Hz output not implemented (not connected on MSX)
 
-const nibble MODE_REG  = 13;
-const nibble TEST_REG  = 14;
-const nibble RESET_REG = 15;
+constexpr nibble MODE_REG  = 13;
+constexpr nibble TEST_REG  = 14;
+constexpr nibble RESET_REG = 15;
 
-const nibble TIME_BLOCK  = 0;
-const nibble ALARM_BLOCK = 1;
+constexpr nibble TIME_BLOCK  = 0;
+constexpr nibble ALARM_BLOCK = 1;
 
-const nibble MODE_BLOKSELECT  = 0x3;
-const nibble MODE_ALARMENABLE = 0x4;
-const nibble MODE_TIMERENABLE = 0x8;
+constexpr nibble MODE_BLOKSELECT  = 0x3;
+constexpr nibble MODE_ALARMENABLE = 0x4;
+constexpr nibble MODE_TIMERENABLE = 0x8;
 
-const nibble TEST_SECONDS = 0x1;
-const nibble TEST_MINUTES = 0x2;
-const nibble TEST_DAYS    = 0x4;
-const nibble TEST_YEARS   = 0x8;
+constexpr nibble TEST_SECONDS = 0x1;
+constexpr nibble TEST_MINUTES = 0x2;
+constexpr nibble TEST_DAYS    = 0x4;
+constexpr nibble TEST_YEARS   = 0x8;
 
-const nibble RESET_ALARM    = 0x1;
-const nibble RESET_FRACTION = 0x2;
+constexpr nibble RESET_ALARM    = 0x1;
+constexpr nibble RESET_FRACTION = 0x2;
 
 
 // 0-bits are ignored on writing and return 0 on reading
-static const nibble mask[4][13] = {
+constexpr nibble mask[4][13] = {
 	{ 0xf, 0x7, 0xf, 0x7, 0xf, 0x3, 0x7, 0xf, 0x3, 0xf, 0x1, 0xf, 0xf},
 	{ 0x0, 0x0, 0xf, 0x7, 0xf, 0x3, 0x7, 0xf, 0x3, 0x0, 0x1, 0x3, 0x0},
 	{ 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf},
 	{ 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf}
 };
 
-static std::unique_ptr<EnumSetting<RP5C01::RTCMode>> createModeSetting(
-	CommandController& commandController, const std::string& name)
-{
-	return make_unique<EnumSetting<RP5C01::RTCMode>>(
+RP5C01::RP5C01(CommandController& commandController, SRAM& regs_,
+               EmuTime::param time, const std::string& name)
+	: regs(regs_)
+	, modeSetting(
 		commandController,
 		((name == "Real time clock") ? "rtcmode" // bw-compat
 		                             : (name + " mode")),
 		"Real Time Clock mode", RP5C01::EMUTIME,
 		EnumSetting<RP5C01::RTCMode>::Map{
 			{"EmuTime",  RP5C01::EMUTIME},
-			{"RealTime", RP5C01::REALTIME}});
-}
-
-RP5C01::RP5C01(CommandController& commandController, SRAM& regs_,
-               EmuTime::param time, const std::string& name)
-	: regs(regs_)
-	, modeSetting(createModeSetting(commandController, name))
+			{"RealTime", RP5C01::REALTIME}})
 	, reference(time)
 {
 	initializeTime();
 	reset(time);
 }
 
-RP5C01::~RP5C01()
-{
-}
-
 void RP5C01::reset(EmuTime::param time)
 {
 	modeReg = MODE_TIMERENABLE;
@@ -75,6 +63,23 @@ void RP5C01::reset(EmuTime::param time)
 }
 
 nibble RP5C01::readPort(nibble port, EmuTime::param time)
+{
+	switch (port) {
+	case MODE_REG:
+	case TEST_REG:
+	case RESET_REG:
+		// nothing
+		break;
+	default:
+		unsigned block = modeReg & MODE_BLOKSELECT;
+		if (block == TIME_BLOCK) {
+			updateTimeRegs(time);
+		}
+	}
+	return peekPort(port);
+}
+
+nibble RP5C01::peekPort(nibble port) const
 {
 	assert(port <= 0x0f);
 	switch (port) {
@@ -86,9 +91,6 @@ nibble RP5C01::readPort(nibble port, EmuTime::param time)
 		return 0x0f; // TODO check this
 	default:
 		unsigned block = modeReg & MODE_BLOKSELECT;
-		if (block == TIME_BLOCK) {
-			updateTimeRegs(time);
-		}
 		nibble tmp = regs[block * 13 + port];
 		return tmp & mask[block][port];
 	}
@@ -196,9 +198,9 @@ static int daysInMonth(int month, unsigned leapYear)
 
 void RP5C01::updateTimeRegs(EmuTime::param time)
 {
-	if (modeSetting->getEnum() == EMUTIME) {
+	if (modeSetting.getEnum() == EMUTIME) {
 		// sync with EmuTime, perfect emulation
-		unsigned elapsed = unsigned(reference.getTicksTill(time));
+		auto elapsed = unsigned(reference.getTicksTill(time));
 		reference.advance(time);
 
 		// in test mode increase sec/min/.. at a rate of 16384Hz
@@ -253,19 +255,19 @@ void RP5C01::resetAlarm()
 template<typename Archive>
 void RP5C01::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("reference", reference);
-	ar.serialize("fraction",  fraction);
-	ar.serialize("seconds",   seconds);
-	ar.serialize("minutes",   minutes);
-	ar.serialize("hours",     hours);
-	ar.serialize("dayWeek",   dayWeek);
-	ar.serialize("years",     years);
-	ar.serialize("leapYear",  leapYear);
-	ar.serialize("days",      days);
-	ar.serialize("months",    months);
-	ar.serialize("modeReg",   modeReg);
-	ar.serialize("testReg",   testReg);
-	ar.serialize("resetReg",  resetReg);
+	ar.serialize("reference", reference,
+	             "fraction",  fraction,
+	             "seconds",   seconds,
+	             "minutes",   minutes,
+	             "hours",     hours,
+	             "dayWeek",   dayWeek,
+	             "years",     years,
+	             "leapYear",  leapYear,
+	             "days",      days,
+	             "months",    months,
+	             "modeReg",   modeReg,
+	             "testReg",   testReg,
+	             "resetReg",  resetReg);
 }
 INSTANTIATE_SERIALIZE_METHODS(RP5C01);
 
