@@ -11,6 +11,7 @@
 #include "FilePool.hh"
 #include "UserSettings.hh"
 #include "RomDatabase.hh"
+#include "RomInfo.hh"
 #include "TclCallbackMessages.hh"
 #include "MSXMotherBoard.hh"
 #include "StateChangeDistributor.hh"
@@ -37,26 +38,33 @@
 #include "Thread.hh"
 #include "Timer.hh"
 #include "serialize.hh"
-#include "openmsx.hh"
 #include "checked_cast.hh"
+#include "ranges.hh"
 #include "statp.hh"
 #include "stl.hh"
+#include "StringOp.hh"
 #include "unreachable.hh"
-#include "memory.hh"
+#include "view.hh"
 #include "build-info.hh"
 #include <cassert>
+#include <memory>
 
-using std::string;
-using std::vector;
 using std::make_shared;
+using std::make_unique;
+using std::string;
+using std::string_view;
+using std::vector;
 
 namespace openmsx {
 
-class QuitCommand final : public Command
+// global variable to communicate the exit-code from the 'exit' command to main()
+int exitCode = 0;
+
+class ExitCommand final : public Command
 {
 public:
-	QuitCommand(CommandController& commandController, EventDistributor& distributor);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
+	ExitCommand(CommandController& commandController, EventDistributor& distributor);
+	void execute(span<const TclObject> tokens, TclObject& result) override;
 	string help(const vector<string>& tokens) const override;
 private:
 	EventDistributor& distributor;
@@ -66,7 +74,7 @@ class MachineCommand final : public Command
 {
 public:
 	MachineCommand(CommandController& commandController, Reactor& reactor);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
+	void execute(span<const TclObject> tokens, TclObject& result) override;
 	string help(const vector<string>& tokens) const override;
 	void tabCompletion(vector<string>& tokens) const override;
 private:
@@ -77,7 +85,7 @@ class TestMachineCommand final : public Command
 {
 public:
 	TestMachineCommand(CommandController& commandController, Reactor& reactor);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
+	void execute(span<const TclObject> tokens, TclObject& result) override;
 	string help(const vector<string>& tokens) const override;
 	void tabCompletion(vector<string>& tokens) const override;
 private:
@@ -88,7 +96,7 @@ class CreateMachineCommand final : public Command
 {
 public:
 	CreateMachineCommand(CommandController& commandController, Reactor& reactor);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
+	void execute(span<const TclObject> tokens, TclObject& result) override;
 	string help(const vector<string>& tokens) const override;
 private:
 	Reactor& reactor;
@@ -98,7 +106,7 @@ class DeleteMachineCommand final : public Command
 {
 public:
 	DeleteMachineCommand(CommandController& commandController, Reactor& reactor);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
+	void execute(span<const TclObject> tokens, TclObject& result) override;
 	string help(const vector<string>& tokens) const override;
 	void tabCompletion(vector<string>& tokens) const override;
 private:
@@ -109,7 +117,7 @@ class ListMachinesCommand final : public Command
 {
 public:
 	ListMachinesCommand(CommandController& commandController, Reactor& reactor);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
+	void execute(span<const TclObject> tokens, TclObject& result) override;
 	string help(const vector<string>& tokens) const override;
 private:
 	Reactor& reactor;
@@ -119,7 +127,7 @@ class ActivateMachineCommand final : public Command
 {
 public:
 	ActivateMachineCommand(CommandController& commandController, Reactor& reactor);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
+	void execute(span<const TclObject> tokens, TclObject& result) override;
 	string help(const vector<string>& tokens) const override;
 	void tabCompletion(vector<string>& tokens) const override;
 private:
@@ -130,7 +138,7 @@ class StoreMachineCommand final : public Command
 {
 public:
 	StoreMachineCommand(CommandController& commandController, Reactor& reactor);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
+	void execute(span<const TclObject> tokens, TclObject& result) override;
 	string help(const vector<string>& tokens) const override;
 	void tabCompletion(vector<string>& tokens) const override;
 private:
@@ -141,18 +149,34 @@ class RestoreMachineCommand final : public Command
 {
 public:
 	RestoreMachineCommand(CommandController& commandController, Reactor& reactor);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
+	void execute(span<const TclObject> tokens, TclObject& result) override;
 	string help(const vector<string>& tokens) const override;
 	void tabCompletion(vector<string>& tokens) const override;
 private:
 	Reactor& reactor;
 };
 
+class GetClipboardCommand final : public Command
+{
+public:
+	GetClipboardCommand(CommandController& commandController);
+	void execute(span<const TclObject> tokens, TclObject& result) override;
+	string help(const vector<string>& tokens) const override;
+};
+
+class SetClipboardCommand final : public Command
+{
+public:
+	SetClipboardCommand(CommandController& commandController);
+	void execute(span<const TclObject> tokens, TclObject& result) override;
+	string help(const vector<string>& tokens) const override;
+};
+
 class ConfigInfo final : public InfoTopic
 {
 public:
 	ConfigInfo(InfoCommand& openMSXInfoCommand, const string& configName);
-	void execute(array_ref<TclObject> tokens,
+	void execute(span<const TclObject> tokens,
 	             TclObject& result) const override;
 	string help(const vector<string>& tokens) const override;
 	void tabCompletion(vector<string>& tokens) const override;
@@ -163,27 +187,27 @@ private:
 class RealTimeInfo final : public InfoTopic
 {
 public:
-	RealTimeInfo(InfoCommand& openMSXInfoCommand);
-	void execute(array_ref<TclObject> tokens,
+	explicit RealTimeInfo(InfoCommand& openMSXInfoCommand);
+	void execute(span<const TclObject> tokens,
 	             TclObject& result) const override;
 	string help(const vector<string>& tokens) const override;
 private:
 	const uint64_t reference;
 };
 
-
-Reactor::Reactor()
-	: mbSem(1)
-	, activeBoard(nullptr)
-	, blockedCounter(0)
-	, paused(false)
-	, running(true)
-	, isInit(false)
+class SoftwareInfoTopic final : InfoTopic
 {
-#if UNIQUE_PTR_BUG
-	display = nullptr;
-#endif
-}
+public:
+	SoftwareInfoTopic(InfoCommand& openMSXInfoCommand, Reactor& reactor);
+	void execute(span<const TclObject> tokens,
+	             TclObject& result) const override;
+	std::string help(const std::vector<std::string>& tokens) const override;
+private:
+	Reactor& reactor;
+};
+
+
+Reactor::Reactor() = default;
 
 void Reactor::init()
 {
@@ -203,17 +227,13 @@ void Reactor::init()
 	diskManipulator = make_unique<DiskManipulator>(
 		*globalCommandController, *this);
 	virtualDrive = make_unique<DiskChanger>(
-		"virtual_drive", *globalCommandController,
-		*diskFactory, *diskManipulator, true);
-	filePool = make_unique<FilePool>(
-		*globalCommandController, *eventDistributor);
+		*this, "virtual_drive");
+	filePool = make_unique<FilePool>(*globalCommandController, *this);
 	userSettings = make_unique<UserSettings>(
 		*globalCommandController);
-	softwareDatabase = make_unique<RomDatabase>(
-		*globalCommandController, *globalCliComm);
 	afterCommand = make_unique<AfterCommand>(
 		*this, *eventDistributor, *globalCommandController);
-	quitCommand = make_unique<QuitCommand>(
+	exitCommand = make_unique<ExitCommand>(
 		*globalCommandController, *eventDistributor);
 	messageCommand = make_unique<MessageCommand>(
 		*globalCommandController);
@@ -233,6 +253,10 @@ void Reactor::init()
 		*globalCommandController, *this);
 	restoreMachineCommand = make_unique<RestoreMachineCommand>(
 		*globalCommandController, *this);
+	getClipboardCommand = make_unique<GetClipboardCommand>(
+		*globalCommandController);
+	setClipboardCommand = make_unique<SetClipboardCommand>(
+		*globalCommandController);
 	aviRecordCommand = make_unique<AviRecorder>(*this);
 	extensionInfo = make_unique<ConfigInfo>(
 		getOpenMSXInfoCommand(), "extensions");
@@ -240,6 +264,8 @@ void Reactor::init()
 		getOpenMSXInfoCommand(), "machines");
 	realTimeInfo = make_unique<RealTimeInfo>(
 		getOpenMSXInfoCommand());
+	softwareInfoTopic = make_unique<SoftwareInfoTopic>(
+		getOpenMSXInfoCommand(), *this);
 	tclCallbackMessages = make_unique<TclCallbackMessages>(
 		*globalCliComm, *globalCommandController);
 
@@ -248,7 +274,9 @@ void Reactor::init()
 	getGlobalSettings().getPauseSetting().attach(*this);
 
 	eventDistributor->registerEventListener(OPENMSX_QUIT_EVENT, *this);
+#if PLATFORM_ANDROID
 	eventDistributor->registerEventListener(OPENMSX_FOCUS_EVENT, *this);
+#endif
 	eventDistributor->registerEventListener(OPENMSX_DELETE_BOARDS, *this);
 	isInit = true;
 }
@@ -259,12 +287,22 @@ Reactor::~Reactor()
 	deleteBoard(activeBoard);
 
 	eventDistributor->unregisterEventListener(OPENMSX_QUIT_EVENT, *this);
+#if PLATFORM_ANDROID
 	eventDistributor->unregisterEventListener(OPENMSX_FOCUS_EVENT, *this);
+#endif
 	eventDistributor->unregisterEventListener(OPENMSX_DELETE_BOARDS, *this);
 
 	getGlobalSettings().getPauseSetting().detach(*this);
 }
 
+RomDatabase& Reactor::getSoftwareDatabase()
+{
+	if (!softwareDatabase) {
+		softwareDatabase = make_unique<RomDatabase>(*globalCliComm);
+	}
+	return *softwareDatabase;
+}
+
 CliComm& Reactor::getCliComm()
 {
 	return *globalCliComm;
@@ -285,31 +323,31 @@ InfoCommand& Reactor::getOpenMSXInfoCommand()
 	return globalCommandController->getOpenMSXInfoCommand();
 }
 
-vector<string> Reactor::getHwConfigs(string_ref type)
+vector<string> Reactor::getHwConfigs(string_view type)
 {
 	vector<string> result;
-	for (auto& p : SystemFileContext().getPaths()) {
+	for (auto& p : systemFileContext().getPaths()) {
 		const auto& path = FileOperations::join(p, type);
 		ReadDir configsDir(path);
 		while (auto* entry = configsDir.getEntry()) {
-			string_ref name = entry->d_name;
+			string_view name = entry->d_name;
 			const auto& fullname = FileOperations::join(path, name);
-			if (name.ends_with(".xml") &&
+			if (StringOp::endsWith(name, ".xml") &&
 			    FileOperations::isRegularFile(fullname)) {
 				name.remove_suffix(4);
-				result.push_back(name.str());
+				result.emplace_back(name);
 			} else if (FileOperations::isDirectory(fullname)) {
 				const auto& config = FileOperations::join(
 					fullname, "hardwareconfig.xml");
 				if (FileOperations::isRegularFile(config)) {
-					result.push_back(name.str());
+					result.emplace_back(name);
 				}
 			}
 		}
 	}
 	// remove duplicates
-	sort(begin(result), end(result));
-	result.erase(unique(begin(result), end(result)), end(result));
+	ranges::sort(result);
+	result.erase(ranges::unique(result), end(result));
 	return result;
 }
 
@@ -317,9 +355,8 @@ void Reactor::createMachineSetting()
 {
 	EnumSetting<int>::Map machines; // int's are unique dummy values
 	int count = 1;
-	for (auto& name : getHwConfigs("machines")) {
-		machines.emplace_back(name, count++);
-	}
+	append(machines, view::transform(getHwConfigs("machines"),
+		[&](auto& name) { return std::pair(name, count++); }));
 	machines.emplace_back("C-BIOS_MSX2+", 0); // default machine
 
 	machineSetting = make_unique<EnumSetting<int>>(
@@ -334,28 +371,25 @@ MSXMotherBoard* Reactor::getMotherBoard() const
 	return activeBoard;
 }
 
-string Reactor::getMachineID() const
+string_view Reactor::getMachineID() const
 {
-	return activeBoard ? activeBoard->getMachineID() : "";
+	return activeBoard ? activeBoard->getMachineID() : string_view{};
 }
 
-vector<string_ref> Reactor::getMachineIDs() const
+vector<string_view> Reactor::getMachineIDs() const
 {
-	vector<string_ref> result;
-	for (auto& b : boards) {
-		result.push_back(b->getMachineID());
-	}
-	return result;
+	return to_vector(view::transform(
+		boards, [](auto& b) { return b->getMachineID(); }));
 }
 
-MSXMotherBoard& Reactor::getMachine(string_ref machineID) const
+MSXMotherBoard& Reactor::getMachine(string_view machineID) const
 {
 	for (auto& b : boards) {
 		if (b->getMachineID() == machineID) {
 			return *b;
 		}
 	}
-	throw CommandException("No machine with ID: " + machineID);
+	throw CommandException("No machine with ID: ", machineID);
 }
 
 Reactor::Board Reactor::createEmptyMotherBoard()
@@ -372,11 +406,8 @@ void Reactor::replaceBoard(MSXMotherBoard& oldBoard_, Board newBoard_)
 	boards.push_back(move(newBoard_));
 
 	// Lookup old board (it must be present).
-	auto it = begin(boards);
-	while (it->get() != &oldBoard_) {
-		++it;
-		assert(it != end(boards));
-	}
+	auto it = find_if_unguarded(boards,
+		[&](auto& b) { return b.get() == &oldBoard_; });
 
 	// If the old board was the active board, then activate the new board
 	if (it->get() == activeBoard) {
@@ -387,18 +418,13 @@ void Reactor::replaceBoard(MSXMotherBoard& oldBoard_, Board newBoard_)
 	// Note that we don't use the 'garbageBoards' mechanism as used in
 	// deleteBoard(). This means oldBoard cannot be used  anymore right
 	// after this method returns.
-	boards.erase(it);
+	move_pop_back(boards, it);
 }
 
 void Reactor::switchMachine(const string& machine)
 {
 	if (!display) {
-#if UNIQUE_PTR_BUG
-		display2 = make_unique<Display>(*this);
-		display = display2.get();
-#else
 		display = make_unique<Display>(*this);
-#endif
 		// TODO: Currently it is not possible to move this call into the
 		//       constructor of Display because the call to createVideoSystem()
 		//       indirectly calls Reactor.getDisplay().
@@ -426,11 +452,9 @@ void Reactor::switchBoard(MSXMotherBoard* newBoard)
 {
 	assert(Thread::isMainThread());
 	assert(!newBoard ||
-	       (any_of(begin(boards), end(boards),
-	               [&](Boards::value_type& b) { return b.get() == newBoard; })));
+	       (ranges::any_of(boards, [&](auto& b) { return b.get() == newBoard; })));
 	assert(!activeBoard ||
-	       (any_of(begin(boards), end(boards),
-	               [&](Boards::value_type& b) { return b.get() == activeBoard; })));
+	       (ranges::any_of(boards, [&](auto& b) { return b.get() == activeBoard; })));
 	if (activeBoard) {
 		activeBoard->activate(false);
 	}
@@ -438,7 +462,7 @@ void Reactor::switchBoard(MSXMotherBoard* newBoard)
 		// Don't hold the lock for longer than the actual switch.
 		// In the past we had a potential for deadlocks here, because
 		// (indirectly) the code below still acquires other locks.
-		ScopedLock lock(mbSem);
+		std::lock_guard<std::mutex> lock(mbMutex);
 		activeBoard = newBoard;
 	}
 	eventDistributor->distributeEvent(
@@ -463,10 +487,10 @@ void Reactor::deleteBoard(MSXMotherBoard* board)
 		// delete active board -> there is no active board anymore
 		switchBoard(nullptr);
 	}
-	auto it = find_if_unguarded(boards,
-		[&](Boards::value_type& b) { return b.get() == board; });
+	auto it = rfind_if_unguarded(boards,
+		[&](auto& b) { return b.get() == board; });
 	auto board_ = move(*it);
-	boards.erase(it);
+	move_pop_back(boards, it);
 	// Don't immediately delete old boards because it's possible this
 	// routine is called via a code path that goes through the old
 	// board. Instead remember this board and delete it at a safe moment
@@ -485,7 +509,7 @@ void Reactor::enterMainLoop()
 			activeBoard->exitCPULoopSync();
 		}
 	} else {
-		ScopedLock lock(mbSem);
+		std::lock_guard<std::mutex> lock(mbMutex);
 		if (activeBoard) {
 			activeBoard->exitCPULoopAsync();
 		}
@@ -499,7 +523,7 @@ void Reactor::run(CommandLineParser& parser)
 	// execute init.tcl
 	try {
 		commandController.source(
-			PreferSystemFileContext().resolve("init.tcl"));
+			preferSystemFileContext().resolve("init.tcl"));
 	} catch (FileException&) {
 		// no init.tcl, ignore
 	}
@@ -507,12 +531,20 @@ void Reactor::run(CommandLineParser& parser)
 	// execute startup scripts
 	for (auto& s : parser.getStartupScripts()) {
 		try {
-			commandController.source(UserFileContext().resolve(s));
+			commandController.source(userFileContext().resolve(s));
 		} catch (FileException& e) {
-			throw FatalError("Couldn't execute script: " +
+			throw FatalError("Couldn't execute script: ",
 			                 e.getMessage());
 		}
 	}
+	for (auto& cmd : parser.getStartupCommands()) {
+		try {
+			commandController.executeCommand(cmd);
+		} catch (CommandException& e) {
+			throw FatalError("Couldn't execute command: ", cmd,
+			                 '\n', e.getMessage());
+		}
+	}
 
 	// At this point openmsx is fully started, it's OK now to start
 	// accepting external commands
@@ -531,23 +563,29 @@ void Reactor::run(CommandLineParser& parser)
 		}
 	}
 
-	while (running) {
-		eventDistributor->deliverEvents();
-		assert(garbageBoards.empty());
-		bool blocked = (blockedCounter > 0) || !activeBoard;
-		if (!blocked) blocked = !activeBoard->execute();
-		if (blocked) {
-			// At first sight a better alternative is to use the
-			// SDL_WaitEvent() function. Though when inspecting
-			// the implementation of that function, it turns out
-			// to also use a sleep/poll loop, with even shorter
-			// sleep periods as we use here. Maybe in future
-			// SDL implementations this will be improved.
-			eventDistributor->sleep(20 * 1000);
-		}
+	while (doOneIteration()) {
+		// nothing
 	}
 }
 
+bool Reactor::doOneIteration()
+{
+	eventDistributor->deliverEvents();
+	assert(garbageBoards.empty());
+	bool blocked = (blockedCounter > 0) || !activeBoard;
+	if (!blocked) blocked = !activeBoard->execute();
+	if (blocked) {
+		// At first sight a better alternative is to use the
+		// SDL_WaitEvent() function. Though when inspecting
+		// the implementation of that function, it turns out
+		// to also use a sleep/poll loop, with even shorter
+		// sleep periods as we use here. Maybe in future
+		// SDL implementations this will be improved.
+		eventDistributor->sleep(20 * 1000);
+	}
+	return running;
+}
+
 void Reactor::unpause()
 {
 	if (paused) {
@@ -601,8 +639,8 @@ int Reactor::signalEvent(const std::shared_ptr<const Event>& event)
 	if (type == OPENMSX_QUIT_EVENT) {
 		enterMainLoop();
 		running = false;
-	} else if (type == OPENMSX_FOCUS_EVENT) {
 #if PLATFORM_ANDROID
+	} else if (type == OPENMSX_FOCUS_EVENT) {
 		// Android SDL port sends a (un)focus event when an app is put in background
 		// by the OS for whatever reason (like an incoming phone call) and all screen
 		// resources are taken away from the app.
@@ -618,26 +656,12 @@ int Reactor::signalEvent(const std::shared_ptr<const Event>& event)
 		} else {
 			block();
 		}
-#else
-		// On other platforms, the user may specify if openMSX should be
-		// halted on loss of focus.
-		if (!getGlobalSettings().getPauseOnLostFocusSetting().getBoolean()) {
-			return 0;
-		}
-		auto& focusEvent = checked_cast<const FocusEvent&>(*event);
-		if (focusEvent.getGain()) {
-			// gained focus
-			if (!getGlobalSettings().getPauseSetting().getBoolean()) {
-				unpause();
-			}
-		} else {
-			// lost focus
-			pause();
-		}
 #endif
 	} else if (type == OPENMSX_DELETE_BOARDS) {
+		// Doesn't really matter which one we delete, just that we do
+		// one per event.
 		assert(!garbageBoards.empty());
-		garbageBoards.erase(begin(garbageBoards));
+		garbageBoards.pop_back();
 	} else {
 		UNREACHABLE; // we didn't subscribe to this event...
 	}
@@ -645,54 +669,63 @@ int Reactor::signalEvent(const std::shared_ptr<const Event>& event)
 }
 
 
-// class QuitCommand
+// class ExitCommand
 
-QuitCommand::QuitCommand(CommandController& commandController,
+ExitCommand::ExitCommand(CommandController& commandController_,
                          EventDistributor& distributor_)
-	: Command(commandController, "exit")
+	: Command(commandController_, "exit")
 	, distributor(distributor_)
 {
 }
 
-void QuitCommand::execute(array_ref<TclObject> /*tokens*/, TclObject& /*result*/)
+void ExitCommand::execute(span<const TclObject> tokens, TclObject& /*result*/)
 {
+	checkNumArgs(tokens, Between{1, 2}, Prefix{1}, "?exitcode?");
+	switch (tokens.size()) {
+	case 1:
+		exitCode = 0;
+		break;
+	case 2:
+		exitCode = tokens[1].getInt(getInterpreter());
+		break;
+	}
 	distributor.distributeEvent(make_shared<QuitEvent>());
 }
 
-string QuitCommand::help(const vector<string>& /*tokens*/) const
+string ExitCommand::help(const vector<string>& /*tokens*/) const
 {
-	return "Use this command to stop the emulator\n";
+	return "Use this command to stop the emulator.\n"
+	       "Optionally you can pass an exit-code.\n";
 }
 
 
 // class MachineCommand
 
-MachineCommand::MachineCommand(CommandController& commandController,
+MachineCommand::MachineCommand(CommandController& commandController_,
                                Reactor& reactor_)
-	: Command(commandController, "machine")
+	: Command(commandController_, "machine")
 	, reactor(reactor_)
 {
 }
 
-void MachineCommand::execute(array_ref<TclObject> tokens, TclObject& result)
+void MachineCommand::execute(span<const TclObject> tokens, TclObject& result)
 {
+	checkNumArgs(tokens, Between{1, 2}, Prefix{1}, "?machinetype?");
 	switch (tokens.size()) {
 	case 1: // get current machine
 		// nothing
 		break;
 	case 2:
 		try {
-			reactor.switchMachine(tokens[1].getString().str());
+			reactor.switchMachine(string(tokens[1].getString()));
 		} catch (MSXException& e) {
-			throw CommandException("Machine switching failed: " +
+			throw CommandException("Machine switching failed: ",
 			                       e.getMessage());
 		}
 		break;
-	default:
-		throw SyntaxError();
 	}
 	// Always return machineID (of current or of new machine).
-	result.setString(reactor.getMachineID());
+	result = reactor.getMachineID();
 }
 
 string MachineCommand::help(const vector<string>& /*tokens*/) const
@@ -708,24 +741,22 @@ void MachineCommand::tabCompletion(vector<string>& tokens) const
 
 // class TestMachineCommand
 
-TestMachineCommand::TestMachineCommand(CommandController& commandController,
+TestMachineCommand::TestMachineCommand(CommandController& commandController_,
                                        Reactor& reactor_)
-	: Command(commandController, "test_machine")
+	: Command(commandController_, "test_machine")
 	, reactor(reactor_)
 {
 }
 
-void TestMachineCommand::execute(array_ref<TclObject> tokens,
+void TestMachineCommand::execute(span<const TclObject> tokens,
                                  TclObject& result)
 {
-	if (tokens.size() != 2) {
-		throw SyntaxError();
-	}
+	checkNumArgs(tokens, 2, "machinetype");
 	try {
 		MSXMotherBoard mb(reactor);
-		mb.loadMachine(tokens[1].getString().str());
+		mb.loadMachine(string(tokens[1].getString()));
 	} catch (MSXException& e) {
-		result.setString(e.getMessage()); // error
+		result = e.getMessage(); // error
 	}
 }
 
@@ -745,19 +776,17 @@ void TestMachineCommand::tabCompletion(vector<string>& tokens) const
 // class CreateMachineCommand
 
 CreateMachineCommand::CreateMachineCommand(
-	CommandController& commandController, Reactor& reactor_)
-	: Command(commandController, "create_machine")
+	CommandController& commandController_, Reactor& reactor_)
+	: Command(commandController_, "create_machine")
 	, reactor(reactor_)
 {
 }
 
-void CreateMachineCommand::execute(array_ref<TclObject> tokens, TclObject& result)
+void CreateMachineCommand::execute(span<const TclObject> tokens, TclObject& result)
 {
-	if (tokens.size() != 1) {
-		throw SyntaxError();
-	}
+	checkNumArgs(tokens, 1, Prefix{1}, nullptr);
 	auto newBoard = reactor.createEmptyMotherBoard();
-	result.setString(newBoard->getMachineID());
+	result = newBoard->getMachineID();
 	reactor.boards.push_back(move(newBoard));
 }
 
@@ -777,18 +806,16 @@ string CreateMachineCommand::help(const vector<string>& /*tokens*/) const
 // class DeleteMachineCommand
 
 DeleteMachineCommand::DeleteMachineCommand(
-	CommandController& commandController, Reactor& reactor_)
-	: Command(commandController, "delete_machine")
+	CommandController& commandController_, Reactor& reactor_)
+	: Command(commandController_, "delete_machine")
 	, reactor(reactor_)
 {
 }
 
-void DeleteMachineCommand::execute(array_ref<TclObject> tokens,
+void DeleteMachineCommand::execute(span<const TclObject> tokens,
                                    TclObject& /*result*/)
 {
-	if (tokens.size() != 2) {
-		throw SyntaxError();
-	}
+	checkNumArgs(tokens, 2, "id");
 	reactor.deleteBoard(&reactor.getMachine(tokens[1].getString()));
 }
 
@@ -806,13 +833,13 @@ void DeleteMachineCommand::tabCompletion(vector<string>& tokens) const
 // class ListMachinesCommand
 
 ListMachinesCommand::ListMachinesCommand(
-	CommandController& commandController, Reactor& reactor_)
-	: Command(commandController, "list_machines")
+	CommandController& commandController_, Reactor& reactor_)
+	: Command(commandController_, "list_machines")
 	, reactor(reactor_)
 {
 }
 
-void ListMachinesCommand::execute(array_ref<TclObject> /*tokens*/,
+void ListMachinesCommand::execute(span<const TclObject> /*tokens*/,
                                   TclObject& result)
 {
 	result.addListElements(reactor.getMachineIDs());
@@ -827,26 +854,24 @@ string ListMachinesCommand::help(const vector<string>& /*tokens*/) const
 // class ActivateMachineCommand
 
 ActivateMachineCommand::ActivateMachineCommand(
-	CommandController& commandController, Reactor& reactor_)
-	: Command(commandController, "activate_machine")
+	CommandController& commandController_, Reactor& reactor_)
+	: Command(commandController_, "activate_machine")
 	, reactor(reactor_)
 {
 }
 
-void ActivateMachineCommand::execute(array_ref<TclObject> tokens,
+void ActivateMachineCommand::execute(span<const TclObject> tokens,
                                      TclObject& result)
 {
+	checkNumArgs(tokens, Between{1, 2}, Prefix{1}, "id");
 	switch (tokens.size()) {
 	case 1:
 		break;
-	case 2: {
+	case 2:
 		reactor.switchBoard(&reactor.getMachine(tokens[1].getString()));
 		break;
 	}
-	default:
-		throw SyntaxError();
-	}
-	result.setString(reactor.getMachineID());
+	result = reactor.getMachineID();
 }
 
 string ActivateMachineCommand::help(const vector<string>& /*tokens*/) const
@@ -865,16 +890,17 @@ void ActivateMachineCommand::tabCompletion(vector<string>& tokens) const
 // class StoreMachineCommand
 
 StoreMachineCommand::StoreMachineCommand(
-	CommandController& commandController, Reactor& reactor_)
-	: Command(commandController, "store_machine")
+	CommandController& commandController_, Reactor& reactor_)
+	: Command(commandController_, "store_machine")
 	, reactor(reactor_)
 {
 }
 
-void StoreMachineCommand::execute(array_ref<TclObject> tokens, TclObject& result)
+void StoreMachineCommand::execute(span<const TclObject> tokens, TclObject& result)
 {
+	checkNumArgs(tokens, Between{1, 3}, Prefix{1}, "?id? ?filename?");
 	string filename;
-	string_ref machineID;
+	string_view machineID;
 	switch (tokens.size()) {
 	case 1:
 		machineID = reactor.getMachineID();
@@ -886,17 +912,16 @@ void StoreMachineCommand::execute(array_ref<TclObject> tokens, TclObject& result
 		break;
 	case 3:
 		machineID = tokens[1].getString();
-		filename = tokens[2].getString().str();
+		filename = tokens[2].getString();
 		break;
-	default:
-		throw SyntaxError();
 	}
 
 	auto& board = reactor.getMachine(machineID);
 
 	XmlOutputArchive out(filename);
 	out.serialize("machine", board);
-	result.setString(filename);
+	out.close();
+	result = filename;
 }
 
 string StoreMachineCommand::help(const vector<string>& /*tokens*/) const
@@ -904,7 +929,7 @@ string StoreMachineCommand::help(const vector<string>& /*tokens*/) const
 	return
 		"store_machine                       Save state of current machine to file \"openmsxNNNN.xml.gz\"\n"
 		"store_machine machineID             Save state of machine \"machineID\" to file \"openmsxNNNN.xml.gz\"\n"
-                "store_machine machineID <filename>  Save state of machine \"machineID\" to indicated file\n"
+		"store_machine machineID <filename>  Save state of machine \"machineID\" to indicated file\n"
 		"\n"
 		"This is a low-level command, the 'savestate' script is easier to use.";
 }
@@ -918,15 +943,16 @@ void StoreMachineCommand::tabCompletion(vector<string>& tokens) const
 // class RestoreMachineCommand
 
 RestoreMachineCommand::RestoreMachineCommand(
-	CommandController& commandController, Reactor& reactor_)
-	: Command(commandController, "restore_machine")
+	CommandController& commandController_, Reactor& reactor_)
+	: Command(commandController_, "restore_machine")
 	, reactor(reactor_)
 {
 }
 
-void RestoreMachineCommand::execute(array_ref<TclObject> tokens,
+void RestoreMachineCommand::execute(span<const TclObject> tokens,
                                     TclObject& result)
 {
+	checkNumArgs(tokens, Between{1, 2}, Prefix{1}, "?filename?");
 	auto newBoard = reactor.createEmptyMotherBoard();
 
 	string filename;
@@ -935,11 +961,11 @@ void RestoreMachineCommand::execute(array_ref<TclObject> tokens,
 		// load last saved entry
 		struct stat st;
 		string dirName = FileOperations::getUserOpenMSXDir() + "/savestates/";
-		string lastEntry = "";
+		string lastEntry;
 		time_t lastTime = 0;
 		ReadDir dir(dirName);
 		while (dirent* d = dir.getEntry()) {
-			int res = stat((dirName + string(d->d_name)).c_str(), &st);
+			int res = stat(strCat(dirName, d->d_name).c_str(), &st);
 			if ((res == 0) && S_ISREG(st.st_mode)) {
 				time_t modTime = st.st_mtime;
 				if (modTime > lastTime) {
@@ -948,27 +974,26 @@ void RestoreMachineCommand::execute(array_ref<TclObject> tokens,
 				}
 			}
 		}
-		if (lastEntry == "") {
+		if (lastEntry.empty()) {
 			throw CommandException("Can't find last saved state.");
 		}
 		filename = dirName + lastEntry;
 		break;
 	}
 	case 2:
-		filename = tokens[1].getString().str();
+		filename = tokens[1].getString();
 		break;
-	default:
-		throw SyntaxError();
 	}
 
-	//std::cerr << "Loading " << filename << std::endl;
+	//std::cerr << "Loading " << filename << '\n';
 	try {
 		XmlInputArchive in(filename);
 		in.serialize("machine", *newBoard);
 	} catch (XMLException& e) {
-		throw CommandException("Cannot load state, bad file format: " + e.getMessage());
+		throw CommandException("Cannot load state, bad file format: ",
+		                       e.getMessage());
 	} catch (MSXException& e) {
-		throw CommandException("Cannot load state: " + e.getMessage());
+		throw CommandException("Cannot load state: ", e.getMessage());
 	}
 
 	// Savestate also contains stuff like the keyboard state at the moment
@@ -976,23 +1001,68 @@ void RestoreMachineCommand::execute(array_ref<TclObject> tokens,
 	// now we want the MSX to see the actual host keyboard state.
 	newBoard->getStateChangeDistributor().stopReplay(newBoard->getCurrentTime());
 
-	result.setString(newBoard->getMachineID());
+	result = newBoard->getMachineID();
 	reactor.boards.push_back(move(newBoard));
 }
 
 string RestoreMachineCommand::help(const vector<string>& /*tokens*/) const
 {
-	return
-		"restore_machine                       Load state from last saved state in default directory\n"
-		"restore_machine <filename>            Load state from indicated file\n"
-		"\n"
-		"This is a low-level command, the 'loadstate' script is easier to use.";
+	return "restore_machine                       Load state from last saved state in default directory\n"
+	       "restore_machine <filename>            Load state from indicated file\n"
+	       "\n"
+	       "This is a low-level command, the 'loadstate' script is easier to use.";
 }
 
 void RestoreMachineCommand::tabCompletion(vector<string>& tokens) const
 {
 	// TODO: add the default files (state files in user's savestates dir)
-	completeFileName(tokens, UserFileContext());
+	completeFileName(tokens, userFileContext());
+}
+
+
+// class GetClipboardCommand
+
+GetClipboardCommand::GetClipboardCommand(CommandController& commandController_)
+	: Command(commandController_, "get_clipboard_text")
+{
+}
+
+void GetClipboardCommand::execute(span<const TclObject> tokens, TclObject& result)
+{
+	checkNumArgs(tokens, 1, Prefix{1}, nullptr);
+	if (char* text = SDL_GetClipboardText()) {
+		result = text;
+		SDL_free(text);
+	}
+}
+
+string GetClipboardCommand::help(const vector<string>& /*tokens*/) const
+{
+	return "Returns the (text) content of the clipboard as a string.";
+}
+
+
+// class SetClipboardCommand
+
+SetClipboardCommand::SetClipboardCommand(CommandController& commandController_)
+	: Command(commandController_, "set_clipboard_text")
+{
+}
+
+void SetClipboardCommand::execute(span<const TclObject> tokens, TclObject& /*result*/)
+{
+	checkNumArgs(tokens, 2, "text");
+	string text(tokens[1].getString());
+	if (SDL_SetClipboardText(text.c_str()) != 0) {
+		const char* err = SDL_GetError();
+		SDL_ClearError();
+		throw CommandException(err);
+	}
+}
+
+string SetClipboardCommand::help(const vector<string>& /*tokens*/) const
+{
+	return "Send the given string to the clipboard.";
 }
 
 
@@ -1005,7 +1075,7 @@ ConfigInfo::ConfigInfo(InfoCommand& openMSXInfoCommand,
 {
 }
 
-void ConfigInfo::execute(array_ref<TclObject> tokens, TclObject& result) const
+void ConfigInfo::execute(span<const TclObject> tokens, TclObject& result) const
 {
 	// TODO make meta info available through this info topic
 	switch (tokens.size()) {
@@ -1019,13 +1089,12 @@ void ConfigInfo::execute(array_ref<TclObject> tokens, TclObject& result) const
 				configName, tokens[2].getString());
 			if (auto* info = config.findChild("info")) {
 				for (auto& i : info->getChildren()) {
-					result.addListElement(i.getName());
-					result.addListElement(i.getData());
+					result.addDictKeyValue(i.getName(), i.getData());
 				}
 			}
 		} catch (MSXException& e) {
 			throw CommandException(
-				"Couldn't get config info: " + e.getMessage());
+				"Couldn't get config info: ", e.getMessage());
 		}
 		break;
 	}
@@ -1036,8 +1105,8 @@ void ConfigInfo::execute(array_ref<TclObject> tokens, TclObject& result) const
 
 string ConfigInfo::help(const vector<string>& /*tokens*/) const
 {
-	return "Shows a list of available " + configName + ", "
-	       "or get meta information about the selected item.\n";
+	return strCat("Shows a list of available ", configName, ", "
+	              "or get meta information about the selected item.\n");
 }
 
 void ConfigInfo::tabCompletion(vector<string>& tokens) const
@@ -1054,11 +1123,11 @@ RealTimeInfo::RealTimeInfo(InfoCommand& openMSXInfoCommand)
 {
 }
 
-void RealTimeInfo::execute(array_ref<TclObject> /*tokens*/,
+void RealTimeInfo::execute(span<const TclObject> /*tokens*/,
                            TclObject& result) const
 {
 	auto delta = Timer::getTime() - reference;
-	result.setDouble(delta / 1000000.0);
+	result = delta / 1000000.0;
 }
 
 string RealTimeInfo::help(const vector<string>& /*tokens*/) const
@@ -1066,4 +1135,47 @@ string RealTimeInfo::help(const vector<string>& /*tokens*/) const
 	return "Returns the time in seconds since openMSX was started.";
 }
 
+
+// SoftwareInfoTopic
+
+SoftwareInfoTopic::SoftwareInfoTopic(InfoCommand& openMSXInfoCommand, Reactor& reactor_)
+	: InfoTopic(openMSXInfoCommand, "software")
+	, reactor(reactor_)
+{
+}
+
+void SoftwareInfoTopic::execute(
+	span<const TclObject> tokens, TclObject& result) const
+{
+	if (tokens.size() != 3) {
+		throw CommandException("Wrong number of parameters");
+	}
+
+	Sha1Sum sha1sum = Sha1Sum(tokens[2].getString());
+	auto& romDatabase = reactor.getSoftwareDatabase();
+	const RomInfo* romInfo = romDatabase.fetchRomInfo(sha1sum);
+	if (!romInfo) {
+		// no match found
+		throw CommandException(
+			"Software with sha1sum ", sha1sum.toString(), " not found");
+	}
+
+	const char* bufStart = romDatabase.getBufferStart();
+	result.addDictKeyValues("title",            romInfo->getTitle(bufStart),
+	                        "year",             romInfo->getYear(bufStart),
+	                        "company",          romInfo->getCompany(bufStart),
+	                        "country",          romInfo->getCountry(bufStart),
+	                        "orig_type",        romInfo->getOrigType(bufStart),
+	                        "remark",           romInfo->getRemark(bufStart),
+	                        "original",         romInfo->getOriginal(),
+	                        "mapper_type_name", RomInfo::romTypeToName(romInfo->getRomType()),
+	                        "genmsxid",         romInfo->getGenMSXid());
+}
+
+string SoftwareInfoTopic::help(const vector<string>& /*tokens*/) const
+{
+	return "Returns information about the software "
+	       "given its sha1sum, in a paired list.";
+}
+
 } // namespace openmsx
