@@ -8,21 +8,25 @@
 #include "MSXMixer.hh"
 #include "MSXCommandController.hh"
 #include "XMLException.hh"
+#include "TclArgParser.hh"
 #include "TclObject.hh"
 #include "FileOperations.hh"
 #include "FileContext.hh"
 #include "StateChange.hh"
+#include "Timer.hh"
+#include "CliComm.hh"
+#include "Display.hh"
 #include "Reactor.hh"
-#include "Command.hh"
 #include "CommandException.hh"
 #include "MemBuffer.hh"
-#include "StringOp.hh"
+#include "one_of.hh"
+#include "ranges.hh"
 #include "serialize.hh"
-#include "serialize_stl.hh"
-#include "memory.hh"
-#include "xrange.hh"
-#include <functional>
+#include "serialize_meta.hh"
+#include "view.hh"
 #include <cassert>
+#include <cmath>
+#include <iomanip>
 
 using std::string;
 using std::vector;
@@ -32,15 +36,18 @@ using std::move;
 namespace openmsx {
 
 // Time between two snapshots (in seconds)
-static const double SNAPSHOT_PERIOD = 1.0;
+constexpr double SNAPSHOT_PERIOD = 1.0;
 
-// Max number of snapshots in a replay
-static const unsigned MAX_NOF_SNAPSHOTS = 10;
+// Max number of snapshots in a replay file
+constexpr unsigned MAX_NOF_SNAPSHOTS = 10;
 
-// Min distance between snapshots in replay (in seconds)
-static const EmuDuration MIN_PARTITION_LENGTH = EmuDuration(60.0);
+// Min distance between snapshots in replay file (in seconds)
+constexpr auto MIN_PARTITION_LENGTH = EmuDuration(60.0);
 
-static const char* const REPLAY_DIR = "replays";
+// Max distance of one before last snapshot before the end time in replay file (in seconds)
+constexpr auto MAX_DIST_1_BEFORE_LAST_SNAPSHOT = EmuDuration(30.0);
+
+constexpr const char* const REPLAY_DIR = "replays";
 
 // A replay is a struct that contains a vector of motherboards and an MSX event
 // log. Those combined are a replay, because you can replay the events from an
@@ -51,7 +58,7 @@ static const char* const REPLAY_DIR = "replays";
 
 struct Replay
 {
-	Replay(Reactor& reactor_)
+	explicit Replay(Reactor& reactor_)
 		: reactor(reactor_), currentTime(EmuTime::dummy()) {}
 
 	Reactor& reactor;
@@ -93,17 +100,6 @@ struct Replay
 };
 SERIALIZE_CLASS_VERSION(Replay, 4);
 
-class ReverseCmd final : public Command
-{
-public:
-	ReverseCmd(ReverseManager& manager, CommandController& controller);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
-	string help(const vector<string>& tokens) const override;
-	void tabCompletion(vector<string>& tokens) const override;
-private:
-	ReverseManager& manager;
-};
-
 
 // struct ReverseHistory
 
@@ -121,36 +117,12 @@ void ReverseManager::ReverseHistory::clear()
 }
 
 
-// struct ReverseChunk
-
-ReverseManager::ReverseChunk::ReverseChunk()
-	: time(EmuTime::zero)
-{
-}
-
-ReverseManager::ReverseChunk::ReverseChunk(ReverseChunk&& rhs)
-	: time      (move(rhs.time))
-	, savestate (move(rhs.savestate))
-	, eventCount(move(rhs.eventCount))
-{
-}
-
-ReverseManager::ReverseChunk& ReverseManager::ReverseChunk::operator=(
-	ReverseChunk&& rhs)
-{
-	time       = move(rhs.time);
-	savestate  = move(rhs.savestate);
-	eventCount = move(rhs.eventCount);
-	return *this;
-}
-
-
 class EndLogEvent final : public StateChange
 {
 public:
-	EndLogEvent() {} // for serialize
-	EndLogEvent(EmuTime::param time)
-		: StateChange(time)
+	EndLogEvent() = default; // for serialize
+	explicit EndLogEvent(EmuTime::param time_)
+		: StateChange(time_)
 	{
 	}
 
@@ -163,17 +135,12 @@ REGISTER_POLYMORPHIC_CLASS(StateChange, EndLogEvent, "EndLog");
 
 // class ReverseManager
 
-enum SyncType {
-	NEW_SNAPSHOT,
-	INPUT_EVENT
-};
-
 ReverseManager::ReverseManager(MSXMotherBoard& motherBoard_)
-	: Schedulable(motherBoard_.getScheduler())
+	: syncNewSnapshot(motherBoard_.getScheduler())
+	, syncInputEvent (motherBoard_.getScheduler())
 	, motherBoard(motherBoard_)
 	, eventDistributor(motherBoard.getReactor().getEventDistributor())
-	, reverseCmd(make_unique<ReverseCmd>(
-		*this, motherBoard.getCommandController()))
+	, reverseCmd(motherBoard.getCommandController())
 	, keyboard(nullptr)
 	, eventDelay(nullptr)
 	, replayIndex(0)
@@ -204,6 +171,8 @@ void ReverseManager::start()
 		// create first snapshot
 		collecting = true;
 		takeSnapshot(getCurrentTime());
+		// schedule creation of next snapshot
+		schedule(getCurrentTime());
 		// start recording events
 		motherBoard.getStateChangeDistributor().registerRecorder(*this);
 	}
@@ -214,8 +183,8 @@ void ReverseManager::stop()
 {
 	if (isCollecting()) {
 		motherBoard.getStateChangeDistributor().unregisterRecorder(*this);
-		removeSyncPoint(NEW_SNAPSHOT); // don't schedule new snapshot takings
-		removeSyncPoint(INPUT_EVENT); // stop any pending replay actions
+		syncNewSnapshot.removeSyncPoint(); // don't schedule new snapshot takings
+		syncInputEvent .removeSyncPoint(); // stop any pending replay actions
 		history.clear();
 		replayIndex = 0;
 		collecting = false;
@@ -226,11 +195,11 @@ void ReverseManager::stop()
 	assert(!isReplaying());
 }
 
-EmuTime::param ReverseManager::getEndTime(const ReverseHistory& history) const
+EmuTime::param ReverseManager::getEndTime(const ReverseHistory& hist) const
 {
-	if (!history.events.empty()) {
+	if (!hist.events.empty()) {
 		if (auto* ev = dynamic_cast<const EndLogEvent*>(
-				history.events.back().get())) {
+				hist.events.back().get())) {
 			// last log element is EndLogEvent, use that
 			return ev->getTime();
 		}
@@ -242,83 +211,63 @@ EmuTime::param ReverseManager::getEndTime(const ReverseHistory& history) const
 
 void ReverseManager::status(TclObject& result) const
 {
-	result.addListElement("status");
-	if (!isCollecting()) {
-		result.addListElement("disabled");
-	} else if (isReplaying()) {
-		result.addListElement("replaying");
-	} else {
-		result.addListElement("enabled");
-	}
+	result.addDictKeyValue("status", !isCollecting() ? "disabled"
+	                               : isReplaying()   ? "replaying"
+	                                                 : "enabled");
 
-	result.addListElement("begin");
 	EmuTime b(isCollecting() ? begin(history.chunks)->second.time
-	                         : EmuTime::zero);
-	result.addListElement((b - EmuTime::zero).toDouble());
+	                         : EmuTime::zero());
+	result.addDictKeyValue("begin", (b - EmuTime::zero()).toDouble());
 
-	result.addListElement("end");
-	EmuTime end(isCollecting() ? getEndTime(history) : EmuTime::zero);
-	result.addListElement((end - EmuTime::zero).toDouble());
+	EmuTime end(isCollecting() ? getEndTime(history) : EmuTime::zero());
+	result.addDictKeyValue("end", (end - EmuTime::zero()).toDouble());
 
-	result.addListElement("current");
-	EmuTime current(isCollecting() ? getCurrentTime() : EmuTime::zero);
-	result.addListElement((current - EmuTime::zero).toDouble());
+	EmuTime current(isCollecting() ? getCurrentTime() : EmuTime::zero());
+	result.addDictKeyValue("current", (current - EmuTime::zero()).toDouble());
 
-	result.addListElement("snapshots");
 	TclObject snapshots;
-	for (auto& p : history.chunks) {
-		EmuTime time = p.second.time;
-		snapshots.addListElement((time - EmuTime::zero).toDouble());
-	}
-	result.addListElement(snapshots);
+	snapshots.addListElements(view::transform(history.chunks, [](auto& p) {
+		return (p.second.time - EmuTime::zero()).toDouble();
+	}));
+	result.addDictKeyValue("snapshots", snapshots);
 
-	result.addListElement("last_event");
-	auto lastEvent = history.events.rbegin();
-	if (lastEvent != history.events.rend() && dynamic_cast<const EndLogEvent*>(lastEvent->get())) {
+	auto lastEvent = rbegin(history.events);
+	if (lastEvent != rend(history.events) && dynamic_cast<const EndLogEvent*>(lastEvent->get())) {
 		++lastEvent;
 	}
-	EmuTime le(isCollecting() && (lastEvent != history.events.rend()) ? (*lastEvent)->getTime() : EmuTime::zero);
-	result.addListElement((le - EmuTime::zero).toDouble());
+	EmuTime le(isCollecting() && (lastEvent != rend(history.events)) ? (*lastEvent)->getTime() : EmuTime::zero());
+	result.addDictKeyValue("last_event", (le - EmuTime::zero()).toDouble());
 }
 
 void ReverseManager::debugInfo(TclObject& result) const
 {
 	// TODO this is useful during development, but for the end user this
 	// information means nothing. We should remove this later.
-	StringOp::Builder res;
+	string res;
 	size_t totalSize = 0;
-	for (auto& p : history.chunks) {
-		auto& chunk = p.second;
-		res << p.first << ' '
-		    << (chunk.time - EmuTime::zero).toDouble() << ' '
-		    << ((chunk.time - EmuTime::zero).toDouble() / (getCurrentTime() - EmuTime::zero).toDouble()) * 100 << '%'
-		    << " (" << chunk.savestate.size() << ')'
-		    << " (next event index: " << chunk.eventCount << ")\n";
-		totalSize += chunk.savestate.size();
+	for (const auto& [idx, chunk] : history.chunks) {
+		strAppend(res, idx, ' ',
+		          (chunk.time - EmuTime::zero()).toDouble(), ' ',
+		          ((chunk.time - EmuTime::zero()).toDouble() / (getCurrentTime() - EmuTime::zero()).toDouble()) * 100, "%"
+		          " (", chunk.size, ")"
+		          " (next event index: ", chunk.eventCount, ")\n");
+		totalSize += chunk.size;
 	}
-	res << "total size: " << totalSize << '\n';
-	result.setString(string(res));
+	strAppend(res, "total size: ", totalSize, '\n');
+	result = res;
 }
 
-static void parseGoTo(Interpreter& interp, array_ref<TclObject> tokens,
+static void parseGoTo(Interpreter& interp, span<const TclObject> tokens,
                       bool& novideo, double& time)
 {
 	novideo = false;
-	bool hasTime = false;
-	for (auto i : xrange(size_t(2), tokens.size())) {
-		if (tokens[i].getString() == "-novideo") {
-			novideo = true;
-		} else {
-			time = tokens[i].getDouble(interp);
-			hasTime = true;
-		}
-	}
-	if (!hasTime) {
-		throw SyntaxError();
-	}
+	ArgsInfo info[] = { flagArg("-novideo", novideo) };
+	auto args = parseTclArgs(interp, tokens.subspan(2), info);
+	if (args.size() != 1) throw SyntaxError();
+	time = args[0].getDouble(interp);
 }
 
-void ReverseManager::goBack(array_ref<TclObject> tokens)
+void ReverseManager::goBack(span<const TclObject> tokens)
 {
 	bool novideo;
 	double t;
@@ -329,10 +278,10 @@ void ReverseManager::goBack(array_ref<TclObject> tokens)
 	EmuTime target(EmuTime::dummy());
 	if (t >= 0) {
 		EmuDuration d(t);
-		if (d < (now - EmuTime::zero)) {
+		if (d < (now - EmuTime::zero())) {
 			target = now - d;
 		} else {
-			target = EmuTime::zero;
+			target = EmuTime::zero();
 		}
 	} else {
 		target = now + EmuDuration(-t);
@@ -340,14 +289,14 @@ void ReverseManager::goBack(array_ref<TclObject> tokens)
 	goTo(target, novideo);
 }
 
-void ReverseManager::goTo(array_ref<TclObject> tokens)
+void ReverseManager::goTo(span<const TclObject> tokens)
 {
 	bool novideo;
 	double t;
 	auto& interp = motherBoard.getReactor().getInterpreter();
 	parseGoTo(interp, tokens, novideo, t);
 
-	EmuTime target = EmuTime::zero + EmuDuration(t);
+	EmuTime target = EmuTime::zero() + EmuDuration(t);
 	goTo(target, novideo);
 }
 
@@ -361,8 +310,22 @@ void ReverseManager::goTo(EmuTime::param target, bool novideo)
 	goTo(target, novideo, history, true); // move in current time-line
 }
 
+// this function is used below, but factored out, because it's already way too long
+static void reportProgress(Reactor& reactor, const EmuTime& targetTime, int percentage)
+{
+	double targetTimeDisp = (targetTime - EmuTime::zero()).toDouble();
+	std::ostringstream sstr;
+	sstr << "Time warping to " <<
+		int(targetTimeDisp / 60) << ':' << std::setfill('0') <<
+		std::setw(5) << std::setprecision(2) << std::fixed <<
+		std::fmod(targetTimeDisp, 60.0) <<
+		"... " << percentage << '%';
+	reactor.getCliComm().printProgress(sstr.str());
+	reactor.getDisplay().repaintDelayed(0);
+}
+
 void ReverseManager::goTo(
-	EmuTime::param target, bool novideo, ReverseHistory& history,
+	EmuTime::param target, bool novideo, ReverseHistory& hist,
 	bool sameTimeLine)
 {
 	auto& mixer = motherBoard.getMSXMixer();
@@ -378,12 +341,12 @@ void ReverseManager::goTo(
 
 		// -- Locate destination snapshot --
 		// We can't go back further in the past than the first snapshot.
-		assert(!history.chunks.empty());
-		auto it = begin(history.chunks);
+		assert(!hist.chunks.empty());
+		auto it = begin(hist.chunks);
 		EmuTime firstTime = it->second.time;
 		EmuTime targetTime = std::max(target, firstTime);
 		// Also don't go further into the future than 'end time'.
-		targetTime = std::min(targetTime, getEndTime(history));
+		targetTime = std::min(targetTime, getEndTime(hist));
 
 		// Duration of 2 PAL frames. Possible improvement is to use the
 		// actual refresh rate (PAL/NTSC). But it should be the refresh
@@ -399,16 +362,17 @@ void ReverseManager::goTo(
 		// find oldest snapshot that is not newer than requested time
 		// TODO ATM we do a linear search, could be improved to do a binary search.
 		assert(it->second.time <= preTarget); // first one is not newer
-		assert(it != end(history.chunks)); // there are snapshots
+		assert(it != end(hist.chunks)); // there are snapshots
 		do {
 			++it;
-		} while (it != end(history.chunks) &&
+		} while (it != end(hist.chunks) &&
 			 it->second.time <= preTarget);
 		// We found the first one that's newer, previous one is last
 		// one that's not newer (thus older or equal).
-		assert(it != begin(history.chunks));
+		assert(it != begin(hist.chunks));
 		--it;
-		EmuTime snapshotTime = it->second.time;
+		ReverseChunk& chunk = it->second;
+		EmuTime snapshotTime = chunk.time;
 		assert(snapshotTime <= preTarget);
 
 		// IF current time is before the wanted time AND either
@@ -433,8 +397,9 @@ void ReverseManager::goTo(
 			// -- restore old snapshot --
 			newBoard_ = reactor.createEmptyMotherBoard();
 			newBoard = newBoard_.get();
-			MemInputArchive in(it->second.savestate.data(),
-					   it->second.savestate.size());
+			MemInputArchive in(chunk.savestate.data(),
+					   chunk.size,
+					   chunk.deltaBlocks);
 			in.serialize("machine", *newBoard);
 
 			if (eventDelay) {
@@ -445,9 +410,9 @@ void ReverseManager::goTo(
 			}
 
 			// terminate replay log with EndLogEvent (if not there already)
-			if (history.events.empty() ||
-			    !dynamic_cast<const EndLogEvent*>(history.events.back().get())) {
-				history.events.push_back(
+			if (hist.events.empty() ||
+			    !dynamic_cast<const EndLogEvent*>(hist.events.back().get())) {
+				hist.events.push_back(
 					std::make_shared<EndLogEvent>(currentTime));
 			}
 
@@ -455,7 +420,7 @@ void ReverseManager::goTo(
 			// Also we should stop collecting in this ReverseManager,
 			// and start collecting in the new one.
 			auto& newManager = newBoard->getReverseManager();
-			newManager.transferHistory(history, it->second.eventCount);
+			newManager.transferHistory(hist, chunk.eventCount);
 
 			// transfer (or copy) state from old to new machine
 			transferState(*newBoard);
@@ -466,8 +431,49 @@ void ReverseManager::goTo(
 		}
 
 		// -- goto correct time within snapshot --
-		// fast forward 2 frames before target time
-		newBoard->fastForward(preTarget, true);
+		// Fast forward 2 frames before target time.
+		// If we're short on snapshots, create them at intervals that are
+		// at least the usual interval, but the later, the more: each
+		// time divide the remaining time in half and make a snapshot
+		// there.
+		auto lastProgress = Timer::getTime();
+		auto startMSXTime = newBoard->getCurrentTime();
+		auto lastSnapshotTarget = startMSXTime;
+		bool everShowedProgress = false;
+		syncNewSnapshot.removeSyncPoint(); // don't schedule new snapshot takings during fast forward
+		while (true) {
+			auto currentTimeNewBoard = newBoard->getCurrentTime();
+			auto nextSnapshotTarget = std::min(
+				preTarget,
+				lastSnapshotTarget + std::max(
+					EmuDuration(SNAPSHOT_PERIOD),
+					(preTarget - lastSnapshotTarget) / 2
+					));
+			auto nextTarget = std::min(nextSnapshotTarget, currentTimeNewBoard + EmuDuration::sec(1));
+			newBoard->fastForward(nextTarget, true);
+			auto now = Timer::getTime();
+			if (((now - lastProgress) > 1000000) || ((currentTimeNewBoard >= preTarget) && everShowedProgress)) {
+				everShowedProgress = true;
+				lastProgress = now;
+				int percentage = ((currentTimeNewBoard - startMSXTime) * 100u) / (preTarget - startMSXTime);
+				reportProgress(newBoard->getReactor(), targetTime, percentage);
+			}
+			// note: fastForward does not always stop at
+			//       _exactly_ the requested time
+			if (currentTimeNewBoard >= preTarget) break;
+			if (currentTimeNewBoard >= nextSnapshotTarget) {
+				// NOTE: there used to be
+				//newBoard->getReactor().getEventDistributor().deliverEvents();
+				// here, but that has all kinds of nasty side effects: it enables
+				// processing of hotkeys, which can cause things like the machine
+				// being deleted, causing a crash. TODO: find a better way to support
+				// live updates of the UI whilst being in a reverse action...
+				newBoard->getReverseManager().takeSnapshot(currentTimeNewBoard);
+				lastSnapshotTarget = nextSnapshotTarget;
+			}
+		}
+		// re-enable automatic snapshots
+		schedule(getCurrentTime());
 
 		// switch to the new MSXMotherBoard
 		//  Note: this deletes the current MSXMotherBoard and
@@ -521,26 +527,29 @@ void ReverseManager::transferState(MSXMotherBoard& newBoard)
 	newBoard.getMSXCommandController().transferSettings(oldController);
 }
 
-void ReverseManager::saveReplay(array_ref<TclObject> tokens, TclObject& result)
+void ReverseManager::saveReplay(
+	Interpreter& interp, span<const TclObject> tokens, TclObject& result)
 {
 	const auto& chunks = history.chunks;
 	if (chunks.empty()) {
 		throw CommandException("No recording...");
 	}
 
-	string filename;
-	switch (tokens.size()) {
-	case 2:
-		// nothing
-		break;
-	case 3:
-		filename = tokens[2].getString().str();
-		break;
-	default:
-		throw SyntaxError();
+	std::string_view filenameArg;
+	int maxNofExtraSnapshots = MAX_NOF_SNAPSHOTS;
+	ArgsInfo info[] = { valueArg("-maxnofextrasnapshots", maxNofExtraSnapshots) };
+	auto args = parseTclArgs(interp, tokens.subspan(2), info);
+	switch (args.size()) {
+		case 0: break; // nothing
+		case 1: filenameArg = args[0].getString(); break;
+		default: throw SyntaxError();
 	}
-	filename = FileOperations::parseCommandFileArgument(
-		filename, REPLAY_DIR, "openmsx", ".omr");
+	if (maxNofExtraSnapshots < 0) {
+		throw CommandException("Maximum number of snapshots should be at least 0");
+	}
+
+	string filename = FileOperations::parseCommandFileArgument(
+		filenameArg, REPLAY_DIR, "openmsx", ".omr");
 
 	auto& reactor = motherBoard.getReactor();
 	Replay replay(reactor);
@@ -553,40 +562,49 @@ void ReverseManager::saveReplay(array_ref<TclObject> tokens, TclObject& result)
 	// restore first snapshot to be able to serialize it to a file
 	auto initialBoard = reactor.createEmptyMotherBoard();
 	MemInputArchive in(begin(chunks)->second.savestate.data(),
-	                   begin(chunks)->second.savestate.size());
+	                   begin(chunks)->second.size,
+			   begin(chunks)->second.deltaBlocks);
 	in.serialize("machine", *initialBoard);
 	replay.motherBoards.push_back(move(initialBoard));
 
-	// determine which extra snapshots to put in the replay
-	const auto& startTime = begin(chunks)->second.time;
-	const auto& endTime   = chunks.rbegin()->second.time;
-	EmuDuration totalLength = endTime - startTime;
-	EmuDuration partitionLength = totalLength.divRoundUp(MAX_NOF_SNAPSHOTS);
-	partitionLength = std::max(MIN_PARTITION_LENGTH, partitionLength);
-	EmuTime nextPartitionEnd = startTime + partitionLength;
-	auto it = begin(chunks);
-	auto lastAddedIt = begin(chunks); // already added
-	while (it != end(chunks)) {
-		++it;
-		if (it == end(chunks) || (it->second.time > nextPartitionEnd)) {
-			--it;
-			assert(it->second.time <= nextPartitionEnd);
-			if (it != lastAddedIt) {
-				// this is a new one, add it to the list of snapshots
-				Reactor::Board board = reactor.createEmptyMotherBoard();
-				MemInputArchive in(it->second.savestate.data(),
-				                   it->second.savestate.size());
-				in.serialize("machine", *board);
-				replay.motherBoards.push_back(move(board));
-				lastAddedIt = it;
-			}
+	if (maxNofExtraSnapshots > 0) {
+		// determine which extra snapshots to put in the replay
+		const auto& startTime = begin(chunks)->second.time;
+		// for the end time, try to take MAX_DIST_1_BEFORE_LAST_SNAPSHOT
+		// seconds before the normal end time so that we get an extra snapshot
+		// at that point, which is comfortable if you want to reverse from the
+		// last snapshot after loading the replay.
+		const auto& lastChunkTime = rbegin(chunks)->second.time;
+		const auto& endTime   = ((startTime + MAX_DIST_1_BEFORE_LAST_SNAPSHOT) < lastChunkTime) ? lastChunkTime - MAX_DIST_1_BEFORE_LAST_SNAPSHOT : lastChunkTime;
+		EmuDuration totalLength = endTime - startTime;
+		EmuDuration partitionLength = totalLength.divRoundUp(maxNofExtraSnapshots);
+		partitionLength = std::max(MIN_PARTITION_LENGTH, partitionLength);
+		EmuTime nextPartitionEnd = startTime + partitionLength;
+		auto it = begin(chunks);
+		auto lastAddedIt = begin(chunks); // already added
+		while (it != end(chunks)) {
 			++it;
-			while (it != end(chunks) && it->second.time > nextPartitionEnd) {
-				nextPartitionEnd += partitionLength;
+			if (it == end(chunks) || (it->second.time > nextPartitionEnd)) {
+				--it;
+				assert(it->second.time <= nextPartitionEnd);
+				if (it != lastAddedIt) {
+					// this is a new one, add it to the list of snapshots
+					Reactor::Board board = reactor.createEmptyMotherBoard();
+					MemInputArchive in2(it->second.savestate.data(),
+							    it->second.size,
+							    it->second.deltaBlocks);
+					in2.serialize("machine", *board);
+					replay.motherBoards.push_back(move(board));
+					lastAddedIt = it;
+				}
+				++it;
+				while (it != end(chunks) && it->second.time > nextPartitionEnd) {
+					nextPartitionEnd += partitionLength;
+				}
 			}
 		}
+		assert(lastAddedIt == std::prev(end(chunks))); // last snapshot must be included
 	}
-	assert(lastAddedIt == --end(chunks)); // last snapshot must be included
 
 	// add sentinel when there isn't one yet
 	bool addSentinel = history.events.empty() ||
@@ -600,6 +618,7 @@ void ReverseManager::saveReplay(array_ref<TclObject> tokens, TclObject& result)
 		XmlOutputArchive out(filename);
 		replay.events = &history.events;
 		out.serialize("replay", replay);
+		out.close();
 	} catch (MSXException&) {
 		if (addSentinel) {
 			history.events.pop_back();
@@ -615,37 +634,24 @@ void ReverseManager::saveReplay(array_ref<TclObject> tokens, TclObject& result)
 		history.events.pop_back();
 	}
 
-	result.setString("Saved replay to " + filename);
+	result = "Saved replay to " + filename;
 }
 
 void ReverseManager::loadReplay(
-	Interpreter& interp, array_ref<TclObject> tokens, TclObject& result)
+	Interpreter& interp, span<const TclObject> tokens, TclObject& result)
 {
-	if (tokens.size() < 3) throw SyntaxError();
-
-	vector<string> arguments;
-	const TclObject* whereArg = nullptr;
 	bool enableViewOnly = false;
-
-	for (size_t i = 2; i < tokens.size(); ++i) {
-		string_ref token = tokens[i].getString();
-		if (token == "-viewonly") {
-			enableViewOnly = true;
-		} else if (token == "-goto") {
-			if (++i == tokens.size()) {
-				throw CommandException("Missing argument");
-			}
-			whereArg = &tokens[i];
-		} else {
-			arguments.push_back(token.str());
-		}
-	}
-
+	std::optional<TclObject> where;
+	ArgsInfo info[] = {
+		flagArg("-viewonly", enableViewOnly),
+		valueArg("-goto", where),
+	};
+	auto arguments = parseTclArgs(interp, tokens.subspan(2), info);
 	if (arguments.size() != 1) throw SyntaxError();
 
 	// resolve the filename
-	UserDataFileContext context(REPLAY_DIR);
-	string fileNameArg = arguments[0];
+	auto context = userDataFileContext(REPLAY_DIR);
+	string fileNameArg(arguments[0].getString());
 	string filename;
 	try {
 		// Try filename as typed by user.
@@ -671,22 +677,22 @@ void ReverseManager::loadReplay(
 		XmlInputArchive in(filename);
 		in.serialize("replay", replay);
 	} catch (XMLException& e) {
-		throw CommandException("Cannot load replay, bad file format: " + e.getMessage());
+		throw CommandException("Cannot load replay, bad file format: ",
+		                       e.getMessage());
 	} catch (MSXException& e) {
-		throw CommandException("Cannot load replay: " + e.getMessage());
+		throw CommandException("Cannot load replay: ", e.getMessage());
 	}
 
 	// get destination time index
-	auto destination = EmuTime::zero;
-	string_ref where = whereArg ? whereArg->getString() : "begin";
-	if (where == "begin") {
-		destination = EmuTime::zero;
-	} else if (where == "end") {
-		destination = EmuTime::infinity;
-	} else if (where == "savetime") {
+	auto destination = EmuTime::zero();
+	if (!where || (*where == "begin")) {
+		destination = EmuTime::zero();
+	} else if (*where == "end") {
+		destination = EmuTime::infinity();
+	} else if (*where == "savetime") {
 		destination = replay.currentTime;
 	} else {
-		destination += EmuDuration(whereArg->getDouble(interp));
+		destination += EmuDuration(where->getDouble(interp));
 	}
 
 	// OK, we are going to be actually changing states now
@@ -711,22 +717,23 @@ void ReverseManager::loadReplay(
 	auto& newEvents = newHistory.events;
 
 	// Restore snapshots
-	unsigned replayIndex = 0;
+	unsigned replayIdx = 0;
 	for (auto& m : replay.motherBoards) {
 		ReverseChunk newChunk;
 		newChunk.time = m->getCurrentTime();
 
-		MemOutputArchive out;
+		MemOutputArchive out(newHistory.lastDeltaBlocks,
+		                     newChunk.deltaBlocks, false);
 		out.serialize("machine", *m);
-		newChunk.savestate = out.releaseBuffer();
+		newChunk.savestate = out.releaseBuffer(newChunk.size);
 
-		// update replayIndex
+		// update replayIdx
 		// TODO: should we use <= instead??
-		while (replayIndex < newEvents.size() &&
-		       (newEvents[replayIndex]->getTime() < newChunk.time)) {
-			replayIndex++;
+		while (replayIdx < newEvents.size() &&
+		       (newEvents[replayIdx]->getTime() < newChunk.time)) {
+			replayIdx++;
 		}
-		newChunk.eventCount = replayIndex;
+		newChunk.eventCount = replayIdx;
 
 		newHistory.chunks[newHistory.getNextSeqNum(newChunk.time)] =
 			move(newChunk);
@@ -738,7 +745,7 @@ void ReverseManager::loadReplay(
 	bool novideo = false;
 	goTo(destination, novideo, newHistory, false); // move to different time-line
 
-	result.setString("Loaded replay from " + filename);
+	result = "Loaded replay from " + filename;
 }
 
 void ReverseManager::transferHistory(ReverseHistory& oldHistory,
@@ -747,6 +754,9 @@ void ReverseManager::transferHistory(ReverseHistory& oldHistory,
 	assert(!isCollecting());
 	assert(history.chunks.empty());
 
+	// 'ids' for old and new serialize blobs don't match, so cleanup old cache
+	oldHistory.lastDeltaBlocks.clear();
+
 	// actual history transfer
 	history.swap(oldHistory);
 
@@ -762,56 +772,54 @@ void ReverseManager::transferHistory(ReverseHistory& oldHistory,
 	replayNextEvent();
 }
 
-void ReverseManager::executeUntil(EmuTime::param /*time*/, int userData)
+void ReverseManager::execNewSnapshot()
 {
-	switch (userData) {
-	case NEW_SNAPSHOT:
-		// During record we should take regular snapshots, and 'now'
-		// it's been a while since the last snapshot. But 'now' can be
-		// in the middle of a CPU instruction (1). However the CPU
-		// emulation code cannot handle taking snapshots at arbitrary
-		// moments in EmuTime (2)(3)(4). So instead we send out an
-		// event that indicates we want to take a snapshot (5).
-		// (1) Schedulables are executed at the exact requested
-		//     EmuTime, even in the middle of a Z80 instruction.
-		// (2) The CPU code serializes all registers, current time and
-		//     various other status info, but not enough info to be
-		//     able to resume in the middle of an instruction.
-		// (3) Only the CPU has this limitation of not being able to
-		//     take a snapshot at any EmuTime, all other devices can.
-		//     This is because in our emulation model the CPU 'drives
-		//     time forward'. It's the only device code that can be
-		//     interrupted by other emulation code (via Schedulables).
-		// (4) In the past we had a CPU core that could execute/resume
-		//     partial instructions (search SVN history). Though it was
-		//     much more complex and it also ran slower than the
-		//     current code.
-		// (5) Events are delivered from the Reactor code. That code
-		//     only runs when the CPU code has exited (meaning no
-		//     longer active in any stackframe). So it's executed right
-		//     after the CPU has finished the current instruction. And
-		//     that's OK, we only require regular snapshots here, they
-		//     should not be *exactly* equally far apart in time.
-		pendingTakeSnapshot = true;
-		eventDistributor.distributeEvent(
-			std::make_shared<SimpleEvent>(OPENMSX_TAKE_REVERSE_SNAPSHOT));
-		break;
-	case INPUT_EVENT:
-		auto event = history.events[replayIndex];
-		try {
-			// deliver current event at current time
-			motherBoard.getStateChangeDistributor().distributeReplay(event);
-		} catch (MSXException&) {
-			// can throw in case we replay a command that fails
-			// ignore
-		}
-		if (!dynamic_cast<const EndLogEvent*>(event.get())) {
-			++replayIndex;
-			replayNextEvent();
-		} else {
-			assert(!isReplaying()); // stopped by replay of EndLogEvent
-		}
-		break;
+	// During record we should take regular snapshots, and 'now'
+	// it's been a while since the last snapshot. But 'now' can be
+	// in the middle of a CPU instruction (1). However the CPU
+	// emulation code cannot handle taking snapshots at arbitrary
+	// moments in EmuTime (2)(3)(4). So instead we send out an
+	// event that indicates we want to take a snapshot (5).
+	// (1) Schedulables are executed at the exact requested
+	//     EmuTime, even in the middle of a Z80 instruction.
+	// (2) The CPU code serializes all registers, current time and
+	//     various other status info, but not enough info to be
+	//     able to resume in the middle of an instruction.
+	// (3) Only the CPU has this limitation of not being able to
+	//     take a snapshot at any EmuTime, all other devices can.
+	//     This is because in our emulation model the CPU 'drives
+	//     time forward'. It's the only device code that can be
+	//     interrupted by other emulation code (via Schedulables).
+	// (4) In the past we had a CPU core that could execute/resume
+	//     partial instructions (search SVN history). Though it was
+	//     much more complex and it also ran slower than the
+	//     current code.
+	// (5) Events are delivered from the Reactor code. That code
+	//     only runs when the CPU code has exited (meaning no
+	//     longer active in any stackframe). So it's executed right
+	//     after the CPU has finished the current instruction. And
+	//     that's OK, we only require regular snapshots here, they
+	//     should not be *exactly* equally far apart in time.
+	pendingTakeSnapshot = true;
+	eventDistributor.distributeEvent(
+		std::make_shared<SimpleEvent>(OPENMSX_TAKE_REVERSE_SNAPSHOT));
+}
+
+void ReverseManager::execInputEvent()
+{
+	auto event = history.events[replayIndex];
+	try {
+		// deliver current event at current time
+		motherBoard.getStateChangeDistributor().distributeReplay(event);
+	} catch (MSXException&) {
+		// can throw in case we replay a command that fails
+		// ignore
+	}
+	if (!dynamic_cast<const EndLogEvent*>(event.get())) {
+		++replayIndex;
+		replayNextEvent();
+	} else {
+		assert(!isReplaying()); // stopped by replay of EndLogEvent
 	}
 }
 
@@ -825,6 +833,8 @@ int ReverseManager::signalEvent(const shared_ptr<const Event>& event)
 	if (pendingTakeSnapshot) {
 		pendingTakeSnapshot = false;
 		takeSnapshot(getCurrentTime());
+		// schedule creation of next snapshot
+		schedule(getCurrentTime());
 	}
 	return 0;
 }
@@ -832,11 +842,11 @@ int ReverseManager::signalEvent(const shared_ptr<const Event>& event)
 unsigned ReverseManager::ReverseHistory::getNextSeqNum(EmuTime::param time) const
 {
 	if (chunks.empty()) {
-		return 1;
+		return 0;
 	}
 	const auto& startTime = begin(chunks)->second.time;
 	double duration = (time - startTime).toDouble();
-	return unsigned(duration / SNAPSHOT_PERIOD + 0.5) + 1;
+	return lrint(duration / SNAPSHOT_PERIOD);
 }
 
 void ReverseManager::takeSnapshot(EmuTime::param time)
@@ -853,22 +863,20 @@ void ReverseManager::takeSnapshot(EmuTime::param time)
 	// the same moment in time).
 
 	// actually create new snapshot
-	MemOutputArchive out;
-	out.serialize("machine", motherBoard);
 	ReverseChunk& newChunk = history.chunks[seqNum];
+	newChunk.deltaBlocks.clear();
+	MemOutputArchive out(history.lastDeltaBlocks, newChunk.deltaBlocks, true);
+	out.serialize("machine", motherBoard);
 	newChunk.time = time;
-	newChunk.savestate = out.releaseBuffer();
+	newChunk.savestate = out.releaseBuffer(newChunk.size);
 	newChunk.eventCount = replayIndex;
-
-	// schedule creation of next snapshot
-	schedule(getCurrentTime());
 }
 
 void ReverseManager::replayNextEvent()
 {
 	// schedule next event at its own time
 	assert(replayIndex < history.events.size());
-	setSyncPoint(history.events[replayIndex]->getTime(), INPUT_EVENT);
+	syncInputEvent.setSyncPoint(history.events[replayIndex]->getTime());
 }
 
 void ReverseManager::signalStateChange(const shared_ptr<StateChange>& event)
@@ -901,12 +909,13 @@ void ReverseManager::stopReplay(EmuTime::param time)
 {
 	if (isReplaying()) {
 		// if we're replaying, stop it and erase remainder of event log
-		removeSyncPoint(INPUT_EVENT);
+		syncInputEvent.removeSyncPoint();
 		Events& events = history.events;
 		events.erase(begin(events) + replayIndex, end(events));
 		// search snapshots that are newer than 'time' and erase them
-		auto it = find_if(begin(history.chunks), end(history.chunks),
-			[&](Chunks::value_type& p) { return p.second.time > time; });
+		auto it = ranges::find_if(history.chunks, [&](auto& p) {
+			return p.second.time > time;
+		});
 		history.chunks.erase(it, end(history.chunks));
 		// this also means someone is changing history, record that
 		reRecordCount++;
@@ -929,12 +938,12 @@ void ReverseManager::stopReplay(EmuTime::param time)
 template<unsigned N>
 void ReverseManager::dropOldSnapshots(unsigned count)
 {
-	unsigned y = (count + N - 1) ^ (count + N);
+	unsigned y = (count + N) ^ (count + N + 1);
 	unsigned d = N;
 	unsigned d2 = 2 * N + 1;
 	while (true) {
 		y >>= 1;
-		if ((y == 0) || (count <= d)) return;
+		if ((y == 0) || (count < d)) return;
 		history.chunks.erase(count - d);
 		d += d2;
 		d2 *= 2;
@@ -943,63 +952,50 @@ void ReverseManager::dropOldSnapshots(unsigned count)
 
 void ReverseManager::schedule(EmuTime::param time)
 {
-	setSyncPoint(time + EmuDuration(SNAPSHOT_PERIOD), NEW_SNAPSHOT);
+	syncNewSnapshot.setSyncPoint(time + EmuDuration(SNAPSHOT_PERIOD));
 }
 
 
 // class ReverseCmd
 
-ReverseCmd::ReverseCmd(ReverseManager& manager_, CommandController& controller)
+ReverseManager::ReverseCmd::ReverseCmd(CommandController& controller)
 	: Command(controller, "reverse")
-	, manager(manager_)
 {
 }
 
-void ReverseCmd::execute(array_ref<TclObject> tokens, TclObject& result)
+void ReverseManager::ReverseCmd::execute(span<const TclObject> tokens, TclObject& result)
 {
-	if (tokens.size() < 2) {
-		throw CommandException("Missing subcommand");
-	}
+	checkNumArgs(tokens, AtLeast{2}, "subcommand ?arg ...?");
+	auto& manager = OUTER(ReverseManager, reverseCmd);
 	auto& interp = getInterpreter();
-	string_ref subcommand = tokens[1].getString();
-	if        (subcommand == "start") {
-		manager.start();
-	} else if (subcommand == "stop") {
-		manager.stop();
-	} else if (subcommand == "status") {
-		manager.status(result);
-	} else if (subcommand == "debug") {
-		manager.debugInfo(result);
-	} else if (subcommand == "goback") {
-		manager.goBack(tokens);
-	} else if (subcommand == "goto") {
-		manager.goTo(tokens);
-	} else if (subcommand == "savereplay") {
-		return manager.saveReplay(tokens, result);
-	} else if (subcommand == "loadreplay") {
-		return manager.loadReplay(interp, tokens, result);
-	} else if (subcommand == "viewonlymode") {
-		auto& distributor = manager.motherBoard.getStateChangeDistributor();
-		switch (tokens.size()) {
-		case 2:
-			result.setString(distributor.isViewOnlyMode() ? "true" : "false");
-			break;
-		case 3:
-			distributor.setViewOnlyMode(tokens[2].getBoolean(interp));
-			break;
-		default:
-			throw SyntaxError();
-		}
-	} else if (subcommand == "truncatereplay") {
-		if (manager.isReplaying()) {
-			manager.signalStopReplay(manager.getCurrentTime());
-		}
-	} else {
-		throw CommandException("Invalid subcommand: " + subcommand);
-	}
+	executeSubCommand(tokens[1].getString(),
+		"start",      [&]{ manager.start(); },
+		"stop",       [&]{ manager.stop(); },
+		"status",     [&]{ manager.status(result); },
+		"debug",      [&]{ manager.debugInfo(result); },
+		"goback",     [&]{ manager.goBack(tokens); },
+		"goto",       [&]{ manager.goTo(tokens); },
+		"savereplay", [&]{ manager.saveReplay(interp, tokens, result); },
+		"loadreplay", [&]{ manager.loadReplay(interp, tokens, result); },
+		"viewonlymode", [&]{
+			auto& distributor = manager.motherBoard.getStateChangeDistributor();
+			switch (tokens.size()) {
+			case 2:
+				result = distributor.isViewOnlyMode();
+				break;
+			case 3:
+				distributor.setViewOnlyMode(tokens[2].getBoolean(interp));
+				break;
+			default:
+				throw SyntaxError();
+			}},
+		"truncatereplay", [&] {
+			if (manager.isReplaying()) {
+				manager.signalStopReplay(manager.getCurrentTime());
+			}});
 }
 
-string ReverseCmd::help(const vector<string>& /*tokens*/) const
+string ReverseManager::ReverseCmd::help(const vector<string>& /*tokens*/) const
 {
 	return "start               start collecting reverse data\n"
 	       "stop                stop collecting\n"
@@ -1012,25 +1008,24 @@ string ReverseCmd::help(const vector<string>& /*tokens*/) const
 	       "loadreplay [-goto <begin|end|savetime|<n>>] [-viewonly] <name>   load a replay (snapshot and replay data) with given name and start replaying\n";
 }
 
-void ReverseCmd::tabCompletion(vector<string>& tokens) const
+void ReverseManager::ReverseCmd::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		static const char* const subCommands[] = {
+		static constexpr const char* const subCommands[] = {
 			"start", "stop", "status", "goback", "goto",
 			"savereplay", "loadreplay", "viewonlymode",
 			"truncatereplay",
 		};
 		completeString(tokens, subCommands);
 	} else if ((tokens.size() == 3) || (tokens[1] == "loadreplay")) {
-		if (tokens[1] == "loadreplay" || tokens[1] == "savereplay") {
+		if (tokens[1] == one_of("loadreplay", "savereplay")) {
 			std::vector<const char*> cmds;
 			if (tokens[1] == "loadreplay") {
 				cmds = { "-goto", "-viewonly" };
 			}
-			UserDataFileContext context(REPLAY_DIR);
-			completeFileName(tokens, context, cmds);
+			completeFileName(tokens, userDataFileContext(REPLAY_DIR), cmds);
 		} else if (tokens[1] == "viewonlymode") {
-			static const char* const options[] = { "true", "false" };
+			static constexpr const char* const options[] = { "true", "false" };
 			completeString(tokens, options);
 		}
 	}
