@@ -1,9 +1,11 @@
 #include "CasImage.hh"
 #include "File.hh"
+#include "FilePool.hh"
 #include "Filename.hh"
 #include "CliComm.hh"
 #include "Clock.hh"
 #include "MSXException.hh"
+#include "stl.hh"
 #include "xrange.hh"
 #include <cstring> // for memcmp
 
@@ -18,8 +20,8 @@ namespace openmsx {
 // the max. Let's take 3760 then as a safe value.
 // UPDATE: that seems to break RUN"CAS:" type of programs. 3744 seems to work
 // for those as well (we don't understand why yet)
-static const unsigned BAUDRATE = 3744;
-static const unsigned OUTPUT_FREQUENCY = 4 * BAUDRATE; // 4 samples per bit
+constexpr unsigned BAUDRATE = 3744;
+constexpr unsigned OUTPUT_FREQUENCY = 4 * BAUDRATE; // 4 samples per bit
 // We oversample the audio signal for better sound quality (especially in
 // combination with the hq resampler). Without oversampling the audio output
 // could contain portions like this:
@@ -30,21 +32,21 @@ static const unsigned OUTPUT_FREQUENCY = 4 * BAUDRATE; // 4 samples per bit
 // lost after the hq resampler. After oversampling, the signal looks like this:
 //   -1, -1, -1, -1, +1, +1, +1, +1, -1, -1, -1, -1, ...
 // So every sample repeated 4 times.
-static const unsigned AUDIO_OVERSAMPLE = 4;
+constexpr unsigned AUDIO_OVERSAMPLE = 4;
 
 // number of output bytes for silent parts
-static const unsigned SHORT_SILENCE = OUTPUT_FREQUENCY * 1; // 1 second
-static const unsigned LONG_SILENCE  = OUTPUT_FREQUENCY * 2; // 2 seconds
+constexpr unsigned SHORT_SILENCE = OUTPUT_FREQUENCY * 1; // 1 second
+constexpr unsigned LONG_SILENCE  = OUTPUT_FREQUENCY * 2; // 2 seconds
 
 // number of 1-bits for headers
-static const unsigned LONG_HEADER  = 16000 / 2;
-static const unsigned SHORT_HEADER =  4000 / 2;
+constexpr unsigned LONG_HEADER  = 16000 / 2;
+constexpr unsigned SHORT_HEADER =  4000 / 2;
 
 // headers definitions
-static const byte CAS_HEADER   [ 8] = { 0x1F,0xA6,0xDE,0xBA,0xCC,0x13,0x7D,0x74 };
-static const byte ASCII_HEADER [10] = { 0xEA,0xEA,0xEA,0xEA,0xEA,0xEA,0xEA,0xEA,0xEA,0xEA };
-static const byte BINARY_HEADER[10] = { 0xD0,0xD0,0xD0,0xD0,0xD0,0xD0,0xD0,0xD0,0xD0,0xD0 };
-static const byte BASIC_HEADER [10] = { 0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3 };
+constexpr byte CAS_HEADER   [ 8] = { 0x1F,0xA6,0xDE,0xBA,0xCC,0x13,0x7D,0x74 };
+constexpr byte ASCII_HEADER [10] = { 0xEA,0xEA,0xEA,0xEA,0xEA,0xEA,0xEA,0xEA,0xEA,0xEA };
+constexpr byte BINARY_HEADER[10] = { 0xD0,0xD0,0xD0,0xD0,0xD0,0xD0,0xD0,0xD0,0xD0,0xD0 };
+constexpr byte BASIC_HEADER [10] = { 0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3 };
 
 
 CasImage::CasImage(const Filename& filename, FilePool& filePool, CliComm& cliComm)
@@ -53,16 +55,16 @@ CasImage::CasImage(const Filename& filename, FilePool& filePool, CliComm& cliCom
 	convert(filename, filePool, cliComm);
 }
 
-short CasImage::getSampleAt(EmuTime::param time)
+int16_t CasImage::getSampleAt(EmuTime::param time)
 {
-	static const Clock<OUTPUT_FREQUENCY> zero(EmuTime::zero);
+	constexpr Clock<OUTPUT_FREQUENCY> zero(EmuTime::zero());
 	unsigned pos = zero.getTicksTill(time);
 	return pos < output.size() ? output[pos] * 256 : 0;
 }
 
 EmuTime CasImage::getEndTime() const
 {
-	Clock<OUTPUT_FREQUENCY> clk(EmuTime::zero);
+	Clock<OUTPUT_FREQUENCY> clk(EmuTime::zero());
 	clk += unsigned(output.size());
 	return clk.getTime();
 }
@@ -72,14 +74,14 @@ unsigned CasImage::getFrequency() const
 	return OUTPUT_FREQUENCY * AUDIO_OVERSAMPLE;
 }
 
-void CasImage::fillBuffer(unsigned pos, int** bufs, unsigned num) const
+void CasImage::fillBuffer(unsigned pos, float** bufs, unsigned num) const
 {
 	size_t nbSamples = output.size();
 	if ((pos / AUDIO_OVERSAMPLE) < nbSamples) {
 		for (auto i : xrange(num)) {
 			bufs[0][i] = ((pos / AUDIO_OVERSAMPLE) < nbSamples)
-			           ? output[pos / AUDIO_OVERSAMPLE] * 256
-			           : 0;
+			           ? output[pos / AUDIO_OVERSAMPLE]
+			           : 0.0f;
 			++pos;
 		}
 	} else {
@@ -87,13 +89,18 @@ void CasImage::fillBuffer(unsigned pos, int** bufs, unsigned num) const
 	}
 }
 
+float CasImage::getAmplificationFactorImpl() const
+{
+	return 1.0f / 128;
+}
+
 void CasImage::write0()
 {
-	output.insert(end(output), { 127, 127, -127, -127 } );
+	append(output, {127, 127, -127, -127});
 }
 void CasImage::write1()
 {
-	output.insert(end(output), { 127, -127, 127, -127 } );
+	append(output, {127, -127, 127, -127});
 }
 
 // write a header signal
@@ -129,11 +136,11 @@ void CasImage::writeByte(byte b)
 }
 
 // write data until a header is detected
-bool CasImage::writeData(const byte* buf, size_t size, size_t& pos)
+bool CasImage::writeData(span<byte> buf, size_t& pos)
 {
 	bool eof = false;
-	while ((pos + 8) <= size) {
-		if (!memcmp(&buf[pos], CAS_HEADER, 8)) {
+	while ((pos + 8) <= buf.size()) {
+		if (memcmp(&buf[pos], CAS_HEADER, 8) == 0) {
 			return eof;
 		}
 		writeByte(buf[pos]);
@@ -142,7 +149,7 @@ bool CasImage::writeData(const byte* buf, size_t size, size_t& pos)
 		}
 		pos++;
 	}
-	while (pos < size) {
+	while (pos < buf.size()) {
 		writeByte(buf[pos++]);
 	}
 	return false;
@@ -151,16 +158,15 @@ bool CasImage::writeData(const byte* buf, size_t size, size_t& pos)
 void CasImage::convert(const Filename& filename, FilePool& filePool, CliComm& cliComm)
 {
 	File file(filename);
-	size_t size;
-	const byte* buf = file.mmap(size);
+	auto buf = file.mmap();
 
 	// search for a header in the .cas file
 	bool issueWarning = false;
 	bool headerFound = false;
 	bool firstFile = true;
 	size_t pos = 0;
-	while ((pos + 8) <= size) {
-		if (!memcmp(&buf[pos], CAS_HEADER, 8)) {
+	while ((pos + 8) <= buf.size()) {
+		if (memcmp(&buf[pos], CAS_HEADER, 8) == 0) {
 			// it probably works fine if a long header is used for every
 			// header but since the msx bios makes a distinction between
 			// them, we do also (hence a lot of code).
@@ -168,65 +174,62 @@ void CasImage::convert(const Filename& filename, FilePool& filePool, CliComm& cl
 			pos += 8;
 			writeSilence(LONG_SILENCE);
 			writeHeader(LONG_HEADER);
-			if ((pos + 10) <= size) {
+			if ((pos + 10) <= buf.size()) {
 				// determine file type
 				FileType type = CassetteImage::UNKNOWN;
-				if (!memcmp(&buf[pos], ASCII_HEADER, 10)) {
+				if (memcmp(&buf[pos], ASCII_HEADER, 10) == 0) {
 					type = CassetteImage::ASCII;
-				} else if (!memcmp(&buf[pos], BINARY_HEADER, 10)) {
+				} else if (memcmp(&buf[pos], BINARY_HEADER, 10) == 0) {
 					type = CassetteImage::BINARY;
-				} else if (!memcmp(&buf[pos], BASIC_HEADER, 10)) {
+				} else if (memcmp(&buf[pos], BASIC_HEADER, 10) == 0) {
 					type = CassetteImage::BASIC;
 				}
 				if (firstFile) setFirstFileType(type);
 				switch (type) {
 					case CassetteImage::ASCII:
-						writeData(buf, size, pos);
+						writeData(buf, pos);
 						bool eof;
 						do {
 							pos += 8;
 							writeSilence(SHORT_SILENCE);
 							writeHeader(SHORT_HEADER);
-							eof = writeData(buf, size, pos);
-						} while (!eof && ((pos + 8) <= size));
+							eof = writeData(buf, pos);
+						} while (!eof && ((pos + 8) <= buf.size()));
 						break;
 					case CassetteImage::BINARY:
 					case CassetteImage::BASIC:
-						writeData(buf, size, pos);
+						writeData(buf, pos);
 						writeSilence(SHORT_SILENCE);
 						writeHeader(SHORT_HEADER);
 						pos += 8;
-						writeData(buf, size, pos);
+						writeData(buf, pos);
 						break;
 					default:
 						// unknown file type: using long header
-						writeData(buf, size, pos);
+						writeData(buf, pos);
 						break;
 				}
 			} else {
 				// unknown file type: using long header
-				writeData(buf, size, pos);
+				writeData(buf, pos);
 			}
 			firstFile = false;
 		} else {
-			// should not occur
-			PRT_DEBUG("CAS2WAV: skipping unhandled data");
+			// skipping unhandled data, shouldn't occur in normal cas file
 			pos++;
 			issueWarning = true;
 		}
 	}
 	if (!headerFound) {
-		throw MSXException(filename.getOriginal() +
-		                   ": not a valid CAS image");
+		throw MSXException(filename.getOriginal(), ": not a valid CAS image");
 	}
 	if (issueWarning) {
-		 cliComm.printWarning("Skipped unhandled data in " +
+		 cliComm.printWarning("Skipped unhandled data in ",
 		                      filename.getOriginal());
 	}
 
 	// conversion successful, now calc sha1sum
-	file.setFilePool(filePool);
-	setSha1Sum(file.getSha1Sum());
+	setSha1Sum(filePool.getSha1Sum(file));
 }
 
 } // namespace openmsx
