@@ -1,5 +1,4 @@
 #include "Interpreter.hh"
-#include "EventDistributor.hh"
 #include "Command.hh"
 #include "TclObject.hh"
 #include "CommandException.hh"
@@ -9,16 +8,15 @@
 #include "InterpreterOutput.hh"
 #include "MSXCPUInterface.hh"
 #include "FileOperations.hh"
-#include "array_ref.hh"
+#include "ranges.hh"
+#include "span.hh"
 #include "stl.hh"
 #include "unreachable.hh"
-#include "xrange.hh"
 #include <iostream>
 #include <utility>
 #include <vector>
 #include <cstdint>
 //#include <tk.h>
-#include "openmsx.hh"
 
 using std::string;
 using std::vector;
@@ -26,7 +24,7 @@ using std::vector;
 namespace openmsx {
 
 // See comments in traceProc()
-static std::vector<std::pair<uintptr_t, BaseSetting*>> traces;
+static std::vector<std::pair<uintptr_t, BaseSetting*>> traces; // sorted on first
 static uintptr_t traceCount = 0;
 
 
@@ -72,8 +70,7 @@ void Interpreter::init(const char* programName)
 	Tcl_FindExecutable(programName);
 }
 
-Interpreter::Interpreter(EventDistributor& eventDistributor_)
-	: eventDistributor(eventDistributor_)
+Interpreter::Interpreter()
 {
 	interp = Tcl_CreateInterp();
 	Tcl_Preserve(interp);
@@ -81,13 +78,13 @@ Interpreter::Interpreter(EventDistributor& eventDistributor_)
 	// TODO need to investigate this: doesn't work on windows
 	/*
 	if (Tcl_Init(interp) != TCL_OK) {
-		std::cout << "Tcl_Init: " << interp->result << std::endl;
+		std::cout << "Tcl_Init: " << interp->result << '\n';
 	}
 	if (Tk_Init(interp) != TCL_OK) {
-		std::cout << "Tk_Init error: " << interp->result << std::endl;
+		std::cout << "Tk_Init error: " << interp->result << '\n';
 	}
 	if (Tcl_Eval(interp, "wm withdraw .") != TCL_OK) {
-		std::cout << "wm withdraw error: " << interp->result << std::endl;
+		std::cout << "wm withdraw error: " << interp->result << '\n';
 	}
 	*/
 
@@ -100,8 +97,10 @@ Interpreter::Interpreter(EventDistributor& eventDistributor_)
 	}
 	Tcl_SetStdChannel(channel, TCL_STDOUT);
 
-	setVariable("env(OPENMSX_USER_DATA)", FileOperations::getUserDataDir());
-	setVariable("env(OPENMSX_SYSTEM_DATA)", FileOperations::getSystemDataDir());
+	setVariable(TclObject("env(OPENMSX_USER_DATA)"),
+	            TclObject(FileOperations::getUserDataDir()));
+	setVariable(TclObject("env(OPENMSX_SYSTEM_DATA)"),
+	            TclObject(FileOperations::getSystemDataDir()));
 }
 
 Interpreter::~Interpreter()
@@ -114,7 +113,18 @@ Interpreter::~Interpreter()
 	}
 	Tcl_Release(interp);
 
-	Tcl_Finalize();
+	// Tcl_Finalize() should only be called once for the whole application
+	// tcl8.6 checks for this (tcl8.5 did not).
+	// Normally we only create/destroy exactly one Interpreter object for
+	// openMSX, and then simply calling Tcl_Finalize() here is fine. Though
+	// when running unittest we do create/destroy multiple Interpreter's.
+	// Another option is to not call Tcl_Finalize(), but that leaves some
+	// memory allocated, and makes memory-leak checkers report more errors.
+	static bool scheduled = false;
+	if (!scheduled) {
+		scheduled = true;
+		atexit(Tcl_Finalize);
+	}
 }
 
 int Interpreter::outputProc(ClientData clientData, const char* buf,
@@ -122,7 +132,7 @@ int Interpreter::outputProc(ClientData clientData, const char* buf,
 {
 	try {
 		auto* output = static_cast<Interpreter*>(clientData)->output;
-		string_ref text(buf, toWrite);
+		std::string_view text(buf, toWrite);
 		if (!text.empty() && output) {
 			output->output(text);
 		}
@@ -134,18 +144,15 @@ int Interpreter::outputProc(ClientData clientData, const char* buf,
 
 void Interpreter::registerCommand(const string& name, Command& command)
 {
-	assert(commandTokenMap.find(name) == end(commandTokenMap));
-	commandTokenMap[name] = Tcl_CreateObjCommand(
+	auto token = Tcl_CreateObjCommand(
 		interp, name.c_str(), commandProc,
 		static_cast<ClientData>(&command), nullptr);
+	command.setToken(token);
 }
 
-void Interpreter::unregisterCommand(string_ref name, Command& /*command*/)
+void Interpreter::unregisterCommand(Command& command)
 {
-	auto it = commandTokenMap.find(name);
-	assert(it != end(commandTokenMap));
-	Tcl_DeleteCommandFromToken(interp, it->second);
-	commandTokenMap.erase(it);
+	Tcl_DeleteCommandFromToken(interp, static_cast<Tcl_Command>(command.getToken()));
 }
 
 int Interpreter::commandProc(ClientData clientData, Tcl_Interp* interp,
@@ -153,7 +160,7 @@ int Interpreter::commandProc(ClientData clientData, Tcl_Interp* interp,
 {
 	try {
 		auto& command = *static_cast<Command*>(clientData);
-		auto tokens = make_array_ref(
+		span<const TclObject> tokens(
 			reinterpret_cast<TclObject*>(const_cast<Tcl_Obj**>(objv)),
 			objc);
 		int res = TCL_OK;
@@ -171,10 +178,7 @@ int Interpreter::commandProc(ClientData clientData, Tcl_Interp* interp,
 			}
 			command.execute(tokens, result);
 		} catch (MSXException& e) {
-			PRT_DEBUG(
-				"Interpreter: Got an exception while executing a command: "
-				<< e.getMessage());
-			result.setString(e.getMessage());
+			result = e.getMessage();
 			res = TCL_ERROR;
 		}
 		Tcl_SetObjResult(interp, result.getTclObject());
@@ -189,9 +193,9 @@ int Interpreter::commandProc(ClientData clientData, Tcl_Interp* interp,
 //   - build-in Tcl commands
 //   - openmsx commands
 //   - user-defined procs
-vector<string> Interpreter::getCommandNames()
+TclObject Interpreter::getCommandNames()
 {
-	return splitList(execute("openmsx::all_command_names"), interp);
+	return execute("openmsx::all_command_names");
 }
 
 bool Interpreter::isComplete(const string& command) const
@@ -199,70 +203,68 @@ bool Interpreter::isComplete(const string& command) const
 	return Tcl_CommandComplete(command.c_str()) != 0;
 }
 
-string Interpreter::execute(const string& command)
+TclObject Interpreter::execute(const string& command)
 {
 	int success = Tcl_Eval(interp, command.c_str());
-	string result =  Tcl_GetStringResult(interp);
 	if (success != TCL_OK) {
-		throw CommandException(result);
+		throw CommandException(Tcl_GetStringResult(interp));
 	}
-	return result;
+	return TclObject(Tcl_GetObjResult(interp));
 }
 
-string Interpreter::executeFile(const string& filename)
+TclObject Interpreter::executeFile(const string& filename)
 {
 	int success = Tcl_EvalFile(interp, filename.c_str());
-	string result =  Tcl_GetStringResult(interp);
 	if (success != TCL_OK) {
-		throw CommandException(result);
+		throw CommandException(Tcl_GetStringResult(interp));
 	}
-	return result;
+	return TclObject(Tcl_GetObjResult(interp));
 }
 
-static void setVar(Tcl_Interp* interp, const char* name, const char* value)
+static void setVar(Tcl_Interp* interp, const TclObject& name, const TclObject& value)
 {
-	if (!Tcl_SetVar(interp, name, value, TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG)) {
+	if (!Tcl_ObjSetVar2(interp, name.getTclObjectNonConst(), nullptr,
+		            value.getTclObjectNonConst(),
+		            TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG)) {
 		// might contain error message of a trace proc
-		std::cerr << Tcl_GetStringResult(interp) << std::endl;
+		std::cerr << Tcl_GetStringResult(interp) << '\n';
 	}
 }
-static const char* getVar(Tcl_Interp* interp, const char* name)
+static Tcl_Obj* getVar(Tcl_Interp* interp, const TclObject& name)
 {
-	return Tcl_GetVar(interp, name, TCL_GLOBAL_ONLY);
+	return Tcl_ObjGetVar2(interp, name.getTclObjectNonConst(), nullptr,
+	                      TCL_GLOBAL_ONLY);
 }
 
-void Interpreter::setVariable(const string& name, const string& value)
+void Interpreter::setVariable(const TclObject& name, const TclObject& value)
 {
-	if (!Tcl_SetVar(interp, name.c_str(), value.c_str(),
-		        TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG)) {
+	if (!Tcl_ObjSetVar2(interp, name.getTclObjectNonConst(), nullptr,
+		            value.getTclObjectNonConst(),
+		            TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG)) {
 		throw CommandException(Tcl_GetStringResult(interp));
 	}
 }
 
-void Interpreter::unsetVariable(const string& name)
+void Interpreter::unsetVariable(const char* name)
 {
-	Tcl_UnsetVar(interp, name.c_str(), TCL_GLOBAL_ONLY);
+	Tcl_UnsetVar(interp, name, TCL_GLOBAL_ONLY);
 }
 
-const char* Interpreter::getVariable(const string& name) const
-{
-	return getVar(interp, name.c_str());
-}
-
-static string getSafeValueString(BaseSetting& setting)
+static TclObject getSafeValue(BaseSetting& setting)
 {
 	try {
-		return setting.getString();
+		return setting.getValue();
 	} catch (MSXException&) {
-		return "0"; // 'safe' value, see comment in registerSetting()
+		return TclObject(0); // 'safe' value, see comment in registerSetting()
 	}
 }
-void Interpreter::registerSetting(BaseSetting& variable, const string& name)
+void Interpreter::registerSetting(BaseSetting& variable)
 {
-	if (const char* tclVarValue = getVariable(name)) {
+	const auto& name = variable.getFullNameObj();
+	if (Tcl_Obj* tclVarValue = getVar(interp, name)) {
 		// Tcl var already existed, use this value
 		try {
-			variable.setStringDirect(tclVarValue);
+			variable.setValueDirect(TclObject(tclVarValue));
 		} catch (MSXException&) {
 			// Ignore: can happen in case of proxy settings when
 			// the current machine doesn't have this setting.
@@ -274,7 +276,7 @@ void Interpreter::registerSetting(BaseSetting& variable, const string& name)
 		}
 	} else {
 		// define Tcl var
-		setVariable(name, getSafeValueString(variable));
+		setVariable(name, getSafeValue(variable));
 	}
 
 	// The call setVariable() above can already trigger traces on this
@@ -290,7 +292,7 @@ void Interpreter::registerSetting(BaseSetting& variable, const string& name)
 	//   create_machine
 	//   machine2::load_machine msx2
 	//
-	// Before changing the 'safe-value' (see getSafeValueString()) to '0',
+	// Before changing the 'safe-value' (see getSafeValue()) to '0',
 	// this gave errors because the load_icons script didn't expect to see
 	// 'proxy' (the old 'safe-value') as value.
 	//
@@ -303,18 +305,19 @@ void Interpreter::registerSetting(BaseSetting& variable, const string& name)
 
 	uintptr_t traceID = traceCount++;
 	traces.emplace_back(traceID, &variable); // still in sorted order
-	Tcl_TraceVar(interp, name.c_str(),
+	Tcl_TraceVar(interp, name.getString().data(), // 0-terminated
 	             TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
 	             traceProc, reinterpret_cast<ClientData>(traceID));
 }
 
-void Interpreter::unregisterSetting(BaseSetting& variable, const string& name)
+void Interpreter::unregisterSetting(BaseSetting& variable)
 {
-	auto it = find_if_unguarded(traces, EqualTupleValue<1>(&variable));
+	auto it = rfind_if_unguarded(traces, EqualTupleValue<1>(&variable));
 	uintptr_t traceID = it->first;
 	traces.erase(it);
 
-	Tcl_UntraceVar(interp, name.c_str(),
+	const char* name = variable.getFullName().data(); // 0-terminated
+	Tcl_UntraceVar(interp, name,
 	               TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
 	               traceProc, reinterpret_cast<ClientData>(traceID));
 	unsetVariable(name);
@@ -322,14 +325,21 @@ void Interpreter::unregisterSetting(BaseSetting& variable, const string& name)
 
 static BaseSetting* getTraceSetting(uintptr_t traceID)
 {
-	auto it = lower_bound(begin(traces), end(traces), traceID,
-	                      LessTupleElement<0>());
+	auto it = ranges::lower_bound(traces, traceID, LessTupleElement<0>());
 	return ((it != end(traces)) && (it->first == traceID))
 		? it->second : nullptr;
 }
 
+#ifndef NDEBUG
+static std::string_view removeColonColon(std::string_view s)
+{
+	if (StringOp::startsWith(s, "::")) s.remove_prefix(2);
+	return s;
+}
+#endif
+
 char* Interpreter::traceProc(ClientData clientData, Tcl_Interp* interp,
-                           const char* part1, const char* /*part2*/, int flags)
+                             const char* part1, const char* /*part2*/, int flags)
 {
 	try {
 		// Lookup Setting object that belongs to this Tcl variable.
@@ -366,27 +376,30 @@ char* Interpreter::traceProc(ClientData clientData, Tcl_Interp* interp,
 		auto* variable = getTraceSetting(traceID);
 		if (!variable) return nullptr;
 
+		const TclObject& part1Obj = variable->getFullNameObj();
+		assert(removeColonColon(part1) == removeColonColon(part1Obj.getString()));
+
 		static string static_string;
 		if (flags & TCL_TRACE_READS) {
 			try {
-				setVar(interp, part1, variable->getString().c_str());
+				setVar(interp, part1Obj, variable->getValue());
 			} catch (MSXException& e) {
-				static_string = e.getMessage();
+				static_string = std::move(e).getMessage();
 				return const_cast<char*>(static_string.c_str());
 			}
 		}
 		if (flags & TCL_TRACE_WRITES) {
 			try {
-				const char* v = getVar(interp, part1);
-				string newValue = v ? v : "";
-				variable->setStringDirect(newValue);
-				string newValue2 = variable->getString();
+				Tcl_Obj* v = getVar(interp, part1Obj);
+				TclObject newValue(v ? v : Tcl_NewObj());
+				variable->setValueDirect(newValue);
+				const TclObject& newValue2 = variable->getValue();
 				if (newValue != newValue2) {
-					setVar(interp, part1, newValue2.c_str());
+					setVar(interp, part1Obj, newValue2);
 				}
 			} catch (MSXException& e) {
-				setVar(interp, part1, getSafeValueString(*variable).c_str());
-				static_string = e.getMessage();
+				setVar(interp, part1Obj, getSafeValue(*variable));
+				static_string = std::move(e).getMessage();
 				return const_cast<char*>(static_string.c_str());
 			}
 		}
@@ -395,15 +408,15 @@ char* Interpreter::traceProc(ClientData clientData, Tcl_Interp* interp,
 				// note we cannot use restoreDefault(), because
 				// that goes via Tcl and the Tcl variable
 				// doesn't exist at this point
-				variable->setStringDirect(
-					variable->getRestoreValue());
+				variable->setValueDirect(TclObject(
+					variable->getRestoreValue()));
 			} catch (MSXException&) {
 				// for some reason default value is not valid ATM,
 				// keep current value (happened for videosource
 				// setting before turning on (set power on) the
 				// MSX machine)
 			}
-			setVar(interp, part1, getSafeValueString(*variable).c_str());
+			setVar(interp, part1Obj, getSafeValue(*variable));
 			Tcl_TraceVar(interp, part1, TCL_TRACE_READS |
 			                TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
 			             traceProc,
@@ -417,31 +430,12 @@ char* Interpreter::traceProc(ClientData clientData, Tcl_Interp* interp,
 
 void Interpreter::createNamespace(const std::string& name)
 {
-	execute("namespace eval ::" + name + " {}");
+	execute(strCat("namespace eval ", name, " {}"));
 }
 
 void Interpreter::deleteNamespace(const std::string& name)
 {
-	execute("namespace delete ::" + name);
-}
-
-vector<string> Interpreter::splitList(const std::string& list)
-{
-	return splitList(list, interp);
-}
-
-vector<string> Interpreter::splitList(const string& list, Tcl_Interp* interp)
-{
-	int argc;
-	const char** argv;
-	if (Tcl_SplitList(interp, list.c_str(), &argc, &argv) == TCL_ERROR) {
-		throw CommandException(
-			interp ? Tcl_GetStringResult(interp)
-			       : "splitList error");
-	}
-	vector<string> result(argv, argv + argc);
-	Tcl_Free(reinterpret_cast<char*>(argv));
-	return result;
+	execute("namespace delete " + name);
 }
 
 void Interpreter::poll()
@@ -450,9 +444,17 @@ void Interpreter::poll()
 	Tcl_DoOneEvent(TCL_DONT_WAIT);
 }
 
-TclParser Interpreter::parse(string_ref command)
+TclParser Interpreter::parse(std::string_view command)
 {
 	return TclParser(interp, command);
 }
 
+void Interpreter::wrongNumArgs(unsigned argc, span<const TclObject> tokens, const char* message)
+{
+	assert(argc <= tokens.size());
+	Tcl_WrongNumArgs(interp, argc, reinterpret_cast<Tcl_Obj* const*>(tokens.data()), message);
+	// not efficient, but anyway on an error path
+	throw CommandException(Tcl_GetStringResult(interp));
+}
+
 } // namespace openmsx
