@@ -3,19 +3,18 @@
 #include "Reactor.hh"
 #include "MSXEventDistributor.hh"
 #include "MSXMotherBoard.hh"
-#include "SettingsConfig.hh"
 #include "SettingsManager.hh"
 #include "InfoCommand.hh"
 #include "Interpreter.hh"
 #include "Setting.hh"
 #include "Event.hh"
 #include "MSXException.hh"
-#include "KeyRange.hh"
-#include "memory.hh"
+#include "stl.hh"
 #include <iostream>
+#include <memory>
 
 using std::string;
-using std::vector;
+using std::string_view;
 
 namespace openmsx {
 
@@ -29,11 +28,11 @@ MSXCommandController::MSXCommandController(
 	, reactor(reactor_)
 	, motherboard(motherboard_)
 	, msxEventDistributor(msxEventDistributor_)
-	, machineID(machineID_)
+	, machineID(strCat("::", machineID_, "::"))
 {
 	globalCommandController.getInterpreter().createNamespace(machineID);
 
-	machineInfoCommand = make_unique<InfoCommand>(*this, "machine_info");
+	machineInfoCommand = std::make_unique<InfoCommand>(*this, "machine_info");
 	machineInfoCommand->setAllowedInEmptyMachine(true);
 
 	msxEventDistributor.registerEventListener(*this);
@@ -46,28 +45,29 @@ MSXCommandController::~MSXCommandController()
 	machineInfoCommand.reset();
 
 	#ifndef NDEBUG
-	for (auto& p : commandMap) {
-		std::cout << "Command not unregistered: " << p.first() << std::endl;
+	for (auto* c : commandMap) {
+		std::cout << "Command not unregistered: " << c->getName() << '\n';
 	}
-	for (auto& p : settingMap) {
-		std::cout << "Setting not unregistered: " << p.first() << std::endl;
+	for (auto* s : settings) {
+		std::cout << "Setting not unregistered: " << s->getFullName() << '\n';
 	}
 	assert(commandMap.empty());
-	assert(settingMap.empty());
+	assert(settings.empty());
 	#endif
 
 	globalCommandController.getInterpreter().deleteNamespace(machineID);
 }
 
-string MSXCommandController::getFullName(string_ref name)
+string MSXCommandController::getFullName(string_view name)
 {
-	return "::" + machineID + "::" + name;
+	return strCat(machineID, name);
 }
 
 void MSXCommandController::registerCommand(Command& command, const string& str)
 {
 	assert(!hasCommand(str));
-	commandMap[str] = &command;
+	assert(command.getName() == str);
+	commandMap.insert_noDuplicateCheck(&command);
 
 	string fullname = getFullName(str);
 	globalCommandController.registerCommand(command, fullname);
@@ -76,9 +76,10 @@ void MSXCommandController::registerCommand(Command& command, const string& str)
 	command.setAllowedInEmptyMachine(false);
 }
 
-void MSXCommandController::unregisterCommand(Command& command, string_ref str)
+void MSXCommandController::unregisterCommand(Command& command, string_view str)
 {
 	assert(hasCommand(str));
+	assert(command.getName() == str);
 	commandMap.erase(str);
 
 	globalCommandController.unregisterProxyCommand(str);
@@ -87,14 +88,14 @@ void MSXCommandController::unregisterCommand(Command& command, string_ref str)
 }
 
 void MSXCommandController::registerCompleter(CommandCompleter& completer,
-                                             string_ref str)
+                                             string_view str)
 {
 	string fullname = getFullName(str);
 	globalCommandController.registerCompleter(completer, fullname);
 }
 
 void MSXCommandController::unregisterCompleter(CommandCompleter& completer,
-                                               string_ref str)
+                                               string_view str)
 {
 	string fullname = getFullName(str);
 	globalCommandController.unregisterCompleter(completer, fullname);
@@ -102,69 +103,41 @@ void MSXCommandController::unregisterCompleter(CommandCompleter& completer,
 
 void MSXCommandController::registerSetting(Setting& setting)
 {
-	const string& name = setting.getName();
-	assert(!findSetting(name));
-	settingMap[name] = &setting;
+	setting.setPrefix(machineID);
+
+	settings.push_back(&setting);
 
 	globalCommandController.registerProxySetting(setting);
-	string fullname = getFullName(name);
-	globalCommandController.getSettingsConfig().getSettingsManager()
-		.registerSetting(setting, fullname);
-	globalCommandController.getInterpreter().registerSetting(setting, fullname);
+	globalCommandController.getSettingsManager().registerSetting(setting);
+	globalCommandController.getInterpreter().registerSetting(setting);
 }
 
 void MSXCommandController::unregisterSetting(Setting& setting)
 {
-	const string& name = setting.getName();
-	assert(findSetting(name));
-	settingMap.erase(name);
+	move_pop_back(settings, rfind_unguarded(settings, &setting));
 
 	globalCommandController.unregisterProxySetting(setting);
-	string fullname = getFullName(name);
-	globalCommandController.getInterpreter().unregisterSetting(setting, fullname);
-	globalCommandController.getSettingsConfig().getSettingsManager()
-		.unregisterSetting(setting, fullname);
+	globalCommandController.getInterpreter().unregisterSetting(setting);
+	globalCommandController.getSettingsManager().unregisterSetting(setting);
 }
 
-void MSXCommandController::changeSetting(Setting& setting, const string& value)
-{
-	string fullname = getFullName(setting.getName());
-	globalCommandController.changeSetting(fullname, value);
-}
-
-Command* MSXCommandController::findCommand(string_ref name) const
+Command* MSXCommandController::findCommand(string_view name) const
 {
 	auto it = commandMap.find(name);
-	return (it != end(commandMap)) ? it->second : nullptr;
+	return (it != end(commandMap)) ? *it : nullptr;
 }
 
-BaseSetting* MSXCommandController::findSetting(string_ref name)
-{
-	auto it = settingMap.find(name);
-	return (it != end(settingMap)) ? it->second : nullptr;
-}
-
-const BaseSetting* MSXCommandController::findSetting(string_ref setting) const
-{
-	return const_cast<MSXCommandController*>(this)->findSetting(setting);
-}
-
-bool MSXCommandController::hasCommand(string_ref command) const
+bool MSXCommandController::hasCommand(string_view command) const
 {
 	return findCommand(command) != nullptr;
 }
 
-string MSXCommandController::executeCommand(const string& command,
-                                            CliConnection* connection)
+TclObject MSXCommandController::executeCommand(const string& command,
+                                               CliConnection* connection)
 {
 	return globalCommandController.executeCommand(command, connection);
 }
 
-vector<string> MSXCommandController::splitList(const string& list)
-{
-	return globalCommandController.splitList(list);
-}
-
 CliComm& MSXCommandController::getCliComm()
 {
 	return motherboard.getMSXCliComm();
@@ -175,15 +148,16 @@ Interpreter& MSXCommandController::getInterpreter()
 	return globalCommandController.getInterpreter();
 }
 
-void MSXCommandController::signalEvent(
+void MSXCommandController::signalMSXEvent(
 	const std::shared_ptr<const Event>& event, EmuTime::param /*time*/)
 {
 	if (event->getType() != OPENMSX_MACHINE_ACTIVATED) return;
 
 	// simple way to synchronize proxy settings
-	for (auto* s : values(settingMap)) {
+	for (auto* s : settings) {
 		try {
-			changeSetting(*s, s->getString());
+			getInterpreter().setVariable(
+				s->getFullNameObj(), s->getValue());
 		} catch (MSXException&) {
 			// ignore
 		}
@@ -197,11 +171,14 @@ bool MSXCommandController::isActive() const
 
 void MSXCommandController::transferSettings(const MSXCommandController& from)
 {
-	for (auto& p : settingMap) {
-		if (auto* fromSetting = from.findSetting(p.first())) {
+	const auto& fromPrefix = from.getPrefix();
+	auto& manager = globalCommandController.getSettingsManager();
+	for (auto* s : settings) {
+		if (auto* fromSetting = manager.findSetting(fromPrefix, s->getBaseName())) {
 			if (!fromSetting->needTransfer()) continue;
 			try {
-				changeSetting(*p.second, fromSetting->getString());
+				getInterpreter().setVariable(
+					s->getFullNameObj(), fromSetting->getValue());
 			} catch (MSXException&) {
 				// ignore
 			}
