@@ -1,132 +1,16 @@
 #include "TclObject.hh"
 #include "Interpreter.hh"
 #include "CommandException.hh"
-#include <tcl.h>
 
 namespace openmsx {
 
-TclObject::TclObject()
-{
-	init(Tcl_NewObj());
-}
-
-TclObject::TclObject(Tcl_Obj* obj_)
-{
-	init(obj_);
-}
-
-TclObject::TclObject(string_ref value)
-{
-	init(Tcl_NewStringObj(value.data(), int(value.size())));
-}
-
-TclObject::TclObject(const TclObject& object)
-{
-	init(object.obj);
-}
-
-void TclObject::init(Tcl_Obj* obj_)
-{
-	obj = obj_;
-	Tcl_IncrRefCount(obj);
-}
-
-TclObject::~TclObject()
-{
-	Tcl_DecrRefCount(obj);
-}
-
-TclObject& TclObject::operator=(const TclObject& other)
-{
-	if (&other != this) {
-		Tcl_DecrRefCount(obj);
-		init(other.obj);
-	}
-	return *this;
-}
-
 static void throwException(Tcl_Interp* interp)
 {
-	string_ref message = interp ? Tcl_GetStringResult(interp)
+	std::string_view message = interp ? Tcl_GetStringResult(interp)
 	                            : "TclObject error";
 	throw CommandException(message);
 }
 
-void TclObject::setString(string_ref value)
-{
-	if (Tcl_IsShared(obj)) {
-		Tcl_DecrRefCount(obj);
-		obj = Tcl_NewStringObj(value.data(), int(value.size()));
-		Tcl_IncrRefCount(obj);
-	} else {
-		Tcl_SetStringObj(obj, value.data(), int(value.size()));
-	}
-}
-
-void TclObject::setInt(int value)
-{
-	if (Tcl_IsShared(obj)) {
-		Tcl_DecrRefCount(obj);
-		obj = Tcl_NewIntObj(value);
-		Tcl_IncrRefCount(obj);
-	} else {
-		Tcl_SetIntObj(obj, value);
-	}
-}
-
-void TclObject::setBoolean(bool value)
-{
-	if (Tcl_IsShared(obj)) {
-		Tcl_DecrRefCount(obj);
-		obj = Tcl_NewBooleanObj(value);
-		Tcl_IncrRefCount(obj);
-	} else {
-		Tcl_SetBooleanObj(obj, value);
-	}
-}
-
-void TclObject::setDouble(double value)
-{
-	if (Tcl_IsShared(obj)) {
-		Tcl_DecrRefCount(obj);
-		obj = Tcl_NewDoubleObj(value);
-		Tcl_IncrRefCount(obj);
-	} else {
-		Tcl_SetDoubleObj(obj, value);
-	}
-}
-
-void TclObject::setBinary(byte* buf, unsigned length)
-{
-	if (Tcl_IsShared(obj)) {
-		Tcl_DecrRefCount(obj);
-		obj = Tcl_NewByteArrayObj(buf, length);
-		Tcl_IncrRefCount(obj);
-	} else {
-		Tcl_SetByteArrayObj(obj, buf, length);
-	}
-}
-
-void TclObject::addListElement(string_ref element)
-{
-	addListElement(Tcl_NewStringObj(element.data(), int(element.size())));
-}
-
-void TclObject::addListElement(int value)
-{
-	addListElement(Tcl_NewIntObj(value));
-}
-
-void TclObject::addListElement(double value)
-{
-	addListElement(Tcl_NewDoubleObj(value));
-}
-
-void TclObject::addListElement(const TclObject& element)
-{
-	addListElement(element.obj);
-}
-
 void TclObject::addListElement(Tcl_Obj* element)
 {
 	// Although it's theoretically possible that Tcl_ListObjAppendElement()
@@ -147,6 +31,44 @@ void TclObject::addListElement(Tcl_Obj* element)
 	}
 }
 
+void TclObject::addListElementsImpl(std::initializer_list<Tcl_Obj*> l)
+{
+	Tcl_Obj* const* objv = l.begin();
+	addListElementsImpl(int(l.size()), objv);
+}
+
+void TclObject::addListElementsImpl(int objc, Tcl_Obj* const* objv)
+{
+	Tcl_Interp* interp = nullptr; // see comment in addListElement
+	if (Tcl_IsShared(obj)) {
+		Tcl_DecrRefCount(obj);
+		obj = Tcl_DuplicateObj(obj);
+		Tcl_IncrRefCount(obj);
+	}
+	if (Tcl_ListObjReplace(interp, obj, INT_MAX, 0, objc, objv) != TCL_OK) {
+		throwException(interp);
+	}
+}
+
+void TclObject::addDictKeyValues(std::initializer_list<Tcl_Obj*> keyValuePairs)
+{
+	assert((keyValuePairs.size() % 2) == 0);
+	Tcl_Interp* interp = nullptr; // see comment in addListElement
+	if (Tcl_IsShared(obj)) {
+		Tcl_DecrRefCount(obj);
+		obj = Tcl_DuplicateObj(obj);
+		Tcl_IncrRefCount(obj);
+	}
+	auto it = keyValuePairs.begin(), et = keyValuePairs.end();
+	while (it != et) {
+		Tcl_Obj* key   = *it++;
+		Tcl_Obj* value = *it++;
+		if (Tcl_DictObjPut(interp, obj, key, value) != TCL_OK) {
+			throwException(interp);
+		}
+	}
+}
+
 int TclObject::getInt(Interpreter& interp_) const
 {
 	auto* interp = interp_.interp;
@@ -177,17 +99,18 @@ double TclObject::getDouble(Interpreter& interp_) const
 	return result;
 }
 
-string_ref TclObject::getString() const
+std::string_view TclObject::getString() const
 {
 	int length;
 	char* buf = Tcl_GetStringFromObj(obj, &length);
-	return string_ref(buf, length);
+	return std::string_view(buf, length);
 }
 
-const byte* TclObject::getBinary(unsigned& length) const
+span<const uint8_t> TclObject::getBinary() const
 {
-	return static_cast<const byte*>(Tcl_GetByteArrayFromObj(
-		obj, reinterpret_cast<int*>(&length)));
+	int length;
+	auto* buf = Tcl_GetByteArrayFromObj(obj, &length);
+	return {buf, size_t(length)};
 }
 
 unsigned TclObject::getListLength(Interpreter& interp_) const
@@ -199,6 +122,14 @@ unsigned TclObject::getListLength(Interpreter& interp_) const
 	}
 	return result;
 }
+unsigned TclObject::getListLengthUnchecked() const
+{
+	int result;
+	if (Tcl_ListObjLength(nullptr, obj, &result) != TCL_OK) {
+		return 0; // error
+	}
+	return result;
+}
 
 TclObject TclObject::getListIndex(Interpreter& interp_, unsigned index) const
 {
@@ -209,6 +140,14 @@ TclObject TclObject::getListIndex(Interpreter& interp_, unsigned index) const
 	}
 	return element ? TclObject(element) : TclObject();
 }
+TclObject TclObject::getListIndexUnchecked(unsigned index) const
+{
+	Tcl_Obj* element;
+	if (Tcl_ListObjIndex(nullptr, obj, index, &element) != TCL_OK) {
+		return TclObject();
+	}
+	return element ? TclObject(element) : TclObject();
+}
 
 TclObject TclObject::getDictValue(Interpreter& interp_, const TclObject& key) const
 {
@@ -230,16 +169,15 @@ bool TclObject::evalBool(Interpreter& interp_) const
 	return result != 0;
 }
 
-std::string TclObject::executeCommand(Interpreter& interp_, bool compile)
+TclObject TclObject::executeCommand(Interpreter& interp_, bool compile)
 {
 	auto* interp = interp_.interp;
 	int flags = compile ? 0 : TCL_EVAL_DIRECT;
 	int success = Tcl_EvalObjEx(interp, obj, flags);
-	std::string result =  Tcl_GetStringResult(interp);
 	if (success != TCL_OK) {
-		throw CommandException(result);
+		throw CommandException(Tcl_GetStringResult(interp));
 	}
-	return result;
+	return TclObject(Tcl_GetObjResult(interp));
 }
 
 } // namespace openmsx
