@@ -11,14 +11,14 @@
 #include "InputEvents.hh"
 #include "Display.hh"
 #include "EventDistributor.hh"
-#include "BooleanSetting.hh"
-#include "IntegerSetting.hh"
+#include "SDL.h"
 #include "Version.hh"
 #include "checked_cast.hh"
 #include "utf8_unchecked.hh"
 #include "StringOp.hh"
 #include "ScopedAssign.hh"
-#include "memory.hh"
+#include "scope_exit.hh"
+#include "view.hh"
 #include "xrange.hh"
 #include <algorithm>
 #include <fstream>
@@ -27,49 +27,46 @@
 using std::min;
 using std::max;
 using std::string;
+using std::string_view;
 
 namespace openmsx {
 
 // class ConsoleLine
 
-ConsoleLine::ConsoleLine()
-{
-}
-
-ConsoleLine::ConsoleLine(string_ref line_, unsigned rgb)
-	: line(line_.str())
+ConsoleLine::ConsoleLine(string line_, uint32_t rgb)
+	: line(std::move(line_))
 	, chunks(1, {rgb, 0})
 {
 }
 
-void ConsoleLine::addChunk(string_ref text, unsigned rgb)
+void ConsoleLine::addChunk(string_view text, uint32_t rgb)
 {
 	chunks.emplace_back(rgb, line.size());
 	line.append(text.data(), text.size());
 }
 
-unsigned ConsoleLine::numChars() const
+size_t ConsoleLine::numChars() const
 {
-	return unsigned(utf8::unchecked::size(line));
+	return utf8::unchecked::size(line);
 }
 
-unsigned ConsoleLine::chunkColor(unsigned i) const
+uint32_t ConsoleLine::chunkColor(size_t i) const
 {
 	assert(i < chunks.size());
 	return chunks[i].first;
 }
 
-string_ref ConsoleLine::chunkText(unsigned i) const
+string_view ConsoleLine::chunkText(size_t i) const
 {
 	assert(i < chunks.size());
 	auto pos = chunks[i].second;
 	auto len = ((i + 1) == chunks.size())
-	         ? string_ref::npos
+	         ? string_view::npos
 	         : chunks[i + 1].second - pos;
-	return string_ref(line).substr(pos, len);
+	return string_view(line).substr(pos, len);
 }
 
-ConsoleLine ConsoleLine::substr(unsigned pos, unsigned len) const
+ConsoleLine ConsoleLine::substr(size_t pos, size_t len) const
 {
 	ConsoleLine result;
 	if (chunks.empty()) {
@@ -103,9 +100,9 @@ ConsoleLine ConsoleLine::substr(unsigned pos, unsigned len) const
 
 // class CommandConsole
 
-static const char* const PROMPT_NEW  = "> ";
-static const char* const PROMPT_CONT = "| ";
-static const char* const PROMPT_BUSY = "*busy*";
+constexpr const char* const PROMPT_NEW  = "> ";
+constexpr const char* const PROMPT_CONT = "| ";
+constexpr const char* const PROMPT_BUSY = "*busy*";
 
 CommandConsole::CommandConsole(
 		GlobalCommandController& commandController_,
@@ -114,17 +111,17 @@ CommandConsole::CommandConsole(
 	: commandController(commandController_)
 	, eventDistributor(eventDistributor_)
 	, display(display_)
-	, consoleSetting(make_unique<BooleanSetting>(
+	, consoleSetting(
 		commandController, "console",
-		"turns console display on/off", false, Setting::DONT_SAVE))
-	, historySizeSetting(make_unique<IntegerSetting>(
+		"turns console display on/off", false, Setting::DONT_SAVE)
+	, historySizeSetting(
 		commandController, "console_history_size",
-		"amount of commands kept in console history", 100, 0, 10000))
-	, removeDoublesSetting(make_unique<BooleanSetting>(
+		"amount of commands kept in console history", 100, 0, 10000)
+	, removeDoublesSetting(
 		commandController, "console_remove_doubles",
 		"don't add the command to history if it's the same as the previous one",
-		true))
-	, history(std::max(1, historySizeSetting->getInt()))
+		true)
+	, history(std::max(1, historySizeSetting.getInt()))
 	, executingCommand(false)
 {
 	resetScrollBack();
@@ -139,7 +136,7 @@ CommandConsole::CommandConsole(
 	print(string(fullVersion.size(), '-'));
 	print("\n"
 	      "General information about openMSX is available at "
-	      "http://www.openmsx.org.\n"
+	      "http://openmsx.org.\n"
 	      "\n"
 	      "Type 'help' to see a list of available commands "
 	      "(use <PgUp>/<PgDn> to scroll).\n"
@@ -165,16 +162,15 @@ CommandConsole::~CommandConsole()
 void CommandConsole::saveHistory()
 {
 	try {
-		UserFileContext context("console");
 		std::ofstream outputfile;
 		FileOperations::openofstream(outputfile,
-		        context.resolveCreate("history.txt"));
+		        userFileContext("console").resolveCreate("history.txt"));
 		if (!outputfile) {
 			throw FileException(
 				"Error while saving the console history.");
 		}
 		for (auto& s : history) {
-			outputfile << string_ref(s).substr(prompt.size()) << '\n';
+			outputfile << s << '\n';
 		}
 	} catch (FileException& e) {
 		commandController.getCliComm().printWarning(e.getMessage());
@@ -184,36 +180,29 @@ void CommandConsole::saveHistory()
 void CommandConsole::loadHistory()
 {
 	try {
-		UserFileContext context("console");
 		std::ifstream inputfile(
-		        context.resolveCreate("history.txt").c_str());
-		if (!inputfile) {
-			throw FileException(
-				"Error while loading the console history.");
-		}
+		        userFileContext("console").
+				resolveCreate("history.txt").c_str());
 		string line;
 		while (inputfile) {
 			getline(inputfile, line);
-			if (!line.empty()) {
-				putCommandHistory(prompt + line);
-			}
+			putCommandHistory(line);
 		}
-	} catch (FileException& e) {
-		PRT_DEBUG(e.getMessage());
-		(void)&e; // Prevent warning
+	} catch (FileException&) {
+		// Error while loading the console history, ignore
 	}
 }
 
 void CommandConsole::getCursorPosition(unsigned& xPosition, unsigned& yPosition) const
 {
 	xPosition = cursorPosition % getColumns();
-	unsigned num = lines[0].numChars() / getColumns();
-	yPosition = num - (cursorPosition / getColumns());
+	auto num = lines[0].numChars() / getColumns();
+	yPosition = unsigned(num - (cursorPosition / getColumns()));
 }
 
 ConsoleLine CommandConsole::getLine(unsigned line) const
 {
-	unsigned count = 0;
+	size_t count = 0;
 	for (auto buf : xrange(lines.size())) {
 		count += (lines[buf].numChars() / getColumns()) + 1;
 		if (count > line) {
@@ -227,10 +216,8 @@ ConsoleLine CommandConsole::getLine(unsigned line) const
 
 int CommandConsole::signalEvent(const std::shared_ptr<const Event>& event)
 {
+	if (!consoleSetting.getBoolean()) return 0;
 	auto& keyEvent = checked_cast<const KeyEvent&>(*event);
-	if (!consoleSetting->getBoolean()) {
-		return 0;
-	}
 
 	// If the console is open then don't pass the event to the MSX
 	// (whetever the (keyboard) event is). If the event has a meaning for
@@ -273,11 +260,19 @@ bool CommandConsole::handleEvent(const KeyEvent& keyEvent)
 			cursorPosition = unsigned(prompt.size());
 			return true;
 		case Keys::K_E:
-			cursorPosition = lines[0].numChars();
+			cursorPosition = unsigned(lines[0].numChars());
 			return true;
 		case Keys::K_C:
 			clearCommand();
 			return true;
+		case Keys::K_L:
+			clearHistory();
+			return true;
+#ifndef __APPLE__
+		case Keys::K_V:
+			paste();
+			return true;
+#endif
 		}
 		break;
 	case Keys::KM_SHIFT:
@@ -290,6 +285,46 @@ bool CommandConsole::handleEvent(const KeyEvent& keyEvent)
 			return true;
 		}
 		break;
+	case Keys::KM_META:
+		switch (key) {
+#ifdef __APPLE__
+		case Keys::K_V:
+			paste();
+			return true;
+		case Keys::K_K:
+			clearHistory();
+			return true;
+		case Keys::K_LEFT:
+			cursorPosition = unsigned(prompt.size());
+			return true;
+		case Keys::K_RIGHT:
+			cursorPosition = unsigned(lines[0].numChars());
+			return true;
+#endif
+		}
+		break;
+	case Keys::KM_ALT:
+		switch (key) {
+		case Keys::K_BACKSPACE:
+			deleteToStartOfWord();
+			return true;
+#ifdef __APPLE__
+		case Keys::K_DELETE:
+			deleteToEndOfWord();
+			return true;
+#else
+		case Keys::K_D:
+			deleteToEndOfWord();
+			return true;
+#endif
+		case Keys::K_LEFT:
+			gotoStartOfWord();
+			return true;
+		case Keys::K_RIGHT:
+			gotoEndOfWord();
+			return true;
+		}
+		break;
 	case 0: // no modifier
 		switch (key) {
 		case Keys::K_PAGEUP:
@@ -329,16 +364,24 @@ bool CommandConsole::handleEvent(const KeyEvent& keyEvent)
 			}
 			return true;
 		case Keys::K_HOME:
+#ifdef __APPLE__
+			scroll(lines.size());
+#else
 			cursorPosition = unsigned(prompt.size());
+#endif
 			return true;
 		case Keys::K_END:
-			cursorPosition = lines[0].numChars();
+#ifdef __APPLE__
+			scroll(-lines.size());
+#else
+			cursorPosition = unsigned(lines[0].numChars());
+#endif
 			return true;
 		}
 		break;
 	}
 
-	uint16_t unicode = keyEvent.getUnicode();
+	auto unicode = keyEvent.getUnicode();
 	if (!unicode || (mod & Keys::KM_META)) {
 		// Disallow META modifer for 'normal' key presses because on
 		// MacOSX Cmd+L is used as a hotkey to toggle the console.
@@ -355,6 +398,17 @@ bool CommandConsole::handleEvent(const KeyEvent& keyEvent)
 		return false;
 	}
 
+	// Apparently on macOS keyboard events for keys like F1 have a non-zero
+	// unicode field. This confuses the console code (it thinks it's a
+	// printable character) and it prevents those keys from triggering
+	// hotkey bindings. See this bug report:
+	//   https://github.com/openMSX/openMSX/issues/1095
+	// As a workaround we ignore chars in the 'Private Use Area' (PUA).
+	//   https://en.wikipedia.org/wiki/Private_Use_Areas
+	if (utf8::is_pua(unicode)) {
+		return false;
+	}
+
 	if (unicode >= 0x20) {
 		normalKey(unicode);
 	} else {
@@ -363,7 +417,7 @@ bool CommandConsole::handleEvent(const KeyEvent& keyEvent)
 	return true;
 }
 
-void CommandConsole::output(string_ref text)
+void CommandConsole::output(string_view text)
 {
 	print(text);
 }
@@ -373,20 +427,20 @@ unsigned CommandConsole::getOutputColumns() const
 	return getColumns();
 }
 
-void CommandConsole::print(string_ref text, unsigned rgb)
+void CommandConsole::print(string_view text, unsigned rgb)
 {
 	while (true) {
 		auto pos = text.find('\n');
-		newLineConsole(ConsoleLine(text.substr(0, pos), rgb));
-		if (pos == string_ref::npos) return;
+		newLineConsole(ConsoleLine(string(text.substr(0, pos)), rgb));
+		if (pos == string_view::npos) return;
 		text = text.substr(pos + 1); // skip newline
 		if (text.empty()) return;
 	}
 }
 
-void CommandConsole::newLineConsole(string_ref line)
+void CommandConsole::newLineConsole(string line)
 {
-	newLineConsole(ConsoleLine(line));
+	newLineConsole(ConsoleLine(std::move(line)));
 }
 
 void CommandConsole::newLineConsole(ConsoleLine line)
@@ -394,47 +448,44 @@ void CommandConsole::newLineConsole(ConsoleLine line)
 	if (lines.isFull()) {
 		lines.removeBack();
 	}
-	ConsoleLine tmp = lines[0];
-	lines[0] = line;
-	lines.addFront(tmp);
+	ConsoleLine tmp = std::move(lines[0]);
+	lines[0] = std::move(line);
+	lines.addFront(std::move(tmp));
 }
 
 void CommandConsole::putCommandHistory(const string& command)
 {
-	// TODO don't store PROMPT as part of history
-	if (command == prompt) {
+	if (command.empty()) return;
+	if (removeDoublesSetting.getBoolean() && !history.empty() &&
+	    (history.back() == command)) {
 		return;
 	}
-	if (removeDoublesSetting->getBoolean() && !history.empty()
-		&& (history.back() == command)) {
-		return;
-	}
-
 	if (history.full()) history.pop_front();
 	history.push_back(command);
-
 }
 
 void CommandConsole::commandExecute()
 {
 	resetScrollBack();
-	putCommandHistory(lines[0].str());
+	string cmd0 = lines[0].str().substr(prompt.size());
+	putCommandHistory(cmd0);
 	saveHistory(); // save at this point already, so that we don't lose history in case of a crash
 
-	commandBuffer += lines[0].str().substr(prompt.size()) + '\n';
+	strAppend(commandBuffer, cmd0, '\n');
 	newLineConsole(lines[0]);
 	if (commandController.isComplete(commandBuffer)) {
-		// Normally the busy promt is NOT shown (not even very briefly
+		// Normally the busy prompt is NOT shown (not even very briefly
 		// because the screen is not redrawn), though for some commands
 		// that potentially take a long time to execute, we explictly
-		// send events, see also comment in handleEvent().
+		// send events, see also comment in signalEvent().
 		prompt = PROMPT_BUSY;
 		putPrompt();
 
 		try {
-			ScopedAssign<bool> sa(executingCommand, true);
-			string result = commandController.executeCommand(
+			ScopedAssign sa(executingCommand, true);
+			auto resultObj = commandController.executeCommand(
 				commandBuffer);
+			auto result = resultObj.getString();
 			if (!result.empty()) {
 				print(result);
 			}
@@ -449,16 +500,14 @@ void CommandConsole::commandExecute()
 	putPrompt();
 }
 
-ConsoleLine CommandConsole::highLight(string_ref line)
+ConsoleLine CommandConsole::highLight(string_view line)
 {
-	assert(line.starts_with(prompt));
-	string_ref command = line.substr(prompt.size());
 	ConsoleLine result;
 	result.addChunk(prompt, 0xffffff);
 
-	TclParser parser = commandController.getInterpreter().parse(command);
+	TclParser parser = commandController.getInterpreter().parse(line);
 	string colors = parser.getColors();
-	assert(colors.size() == command.size());
+	assert(colors.size() == line.size());
 
 	unsigned pos = 0;
 	while (pos != colors.size()) {
@@ -478,15 +527,15 @@ ConsoleLine CommandConsole::highLight(string_ref line)
 		case 'o': rgb = 0x00cdcd; break; // operator
 		default:  rgb = 0xffffff; break; // other
 		}
-		result.addChunk(command.substr(pos2, pos - pos2), rgb);
+		result.addChunk(line.substr(pos2, pos - pos2), rgb);
 	}
 	return result;
 }
 
 void CommandConsole::putPrompt()
 {
-	commandScrollBack = history.size();
-	currentLine = prompt;
+	commandScrollBack = unsigned(history.size());
+	currentLine.clear();
 	lines[0] = highLight(currentLine);
 	cursorPosition = unsigned(prompt.size());
 }
@@ -494,19 +543,120 @@ void CommandConsole::putPrompt()
 void CommandConsole::tabCompletion()
 {
 	resetScrollBack();
-	unsigned pl = unsigned(prompt.size());
-	string_ref front = utf8::unchecked::substr(lines[0].str(), pl, cursorPosition - pl);
-	string_ref back  = utf8::unchecked::substr(lines[0].str(), cursorPosition);
+	auto pl = unsigned(prompt.size());
+	string front(utf8::unchecked::substr(lines[0].str(), pl, cursorPosition - pl));
+	string back (utf8::unchecked::substr(lines[0].str(), cursorPosition));
 	string newFront = commandController.tabCompletion(front);
 	cursorPosition = pl + unsigned(utf8::unchecked::size(newFront));
-	currentLine = prompt + newFront + back;
+	currentLine = newFront + back;
 	lines[0] = highLight(currentLine);
 }
 
 void CommandConsole::scroll(int delta)
 {
-	consoleScrollBack = min(max(consoleScrollBack + delta, 0),
-	                        int(lines.size()));
+	consoleScrollBack = max(min(consoleScrollBack + delta, int(lines.size()) - int(rows)), 0);
+}
+
+// Returns the position of the start of the word relative to the current cursor position.
+// If the cursor is already located at the start of a word then return the start of the previous word.
+// If there is no previous word, then return the start of the line (position of the prompt).
+//
+// One complication is that the line is utf8 encoded and the positions are
+// given in character-counts instead of byte-counts. To solve this this
+// function returns a tuple:
+//  * an iterator to the start of the word (likely the new cursor position).
+//  * an iterator to the current cursor position.
+//  * the distance (in characters, not bytes) between these two iterators.
+// The first item in this tuple is the actual result. The other two are only
+// returned for efficiency (this function calculates them anyway, and it's
+// likely the caller will need them as well).
+static std::tuple<std::string::const_iterator, std::string::const_iterator, unsigned>
+	getStartOfWord(const std::string& line, unsigned cursorPos, unsigned promptSize)
+{
+	auto begin  = std::begin(line);
+	auto prompt = begin + promptSize; // assumes prompt only contains single-byte utf8 chars
+	auto cursor = begin; utf8::unchecked::advance(cursor, cursorPos);
+	auto pos    = cursor;
+
+	// search (backwards) for first non-white-space
+	unsigned distance = 0;
+	while (pos > prompt) {
+		auto pos2 = pos;
+		utf8::unchecked::prior(pos2);
+		if (*pos2 != one_of(' ', '\t')) break;
+		pos = pos2;
+		++distance;
+	}
+	// search (backwards) for first white-space
+	while (pos > prompt) {
+		auto pos2 = pos;
+		utf8::unchecked::prior(pos2);
+		if (*pos2 == one_of(' ', '\t')) break;
+		pos = pos2;
+		++distance;
+	}
+	return {pos, cursor, distance};
+}
+
+// Similar to getStartOfWord() but locates the end of the word instead.
+// The end of the word is the 2nd (instead of the 1st) element in the tuple.
+// This is done so that both getStartOfWord() and getEndOfWord() have the invariant:
+//  result = [begin, end, distance]
+//   -> begin <= end
+//   -> end - begin == distance
+static std::tuple<std::string::const_iterator, std::string::const_iterator, unsigned>
+	getEndOfWord(const std::string& line, unsigned cursorPos)
+{
+	auto begin  = std::begin(line);
+	auto end    = std::end  (line);
+	auto cursor = begin; utf8::unchecked::advance(cursor, cursorPos);
+	auto pos    = cursor;
+
+	// search (forwards) for first non-white-space
+	unsigned distance = 0;
+	while ((pos < end) && (*pos == one_of(' ', '\t'))) {
+		++pos; // single-byte utf8 character
+		++distance;
+	}
+	// search (forwards) for first white-space
+	while ((pos < end) && (*pos != one_of(' ', '\t'))) {
+		utf8::unchecked::next(pos);
+		++distance;
+	}
+	return {cursor, pos, distance};
+}
+
+void CommandConsole::gotoStartOfWord()
+{
+	auto [begin, end, distance] = getStartOfWord(lines[0].str(), cursorPosition, prompt.size());
+	cursorPosition -= distance;
+}
+
+void CommandConsole::deleteToStartOfWord()
+{
+	resetScrollBack();
+	currentLine = lines[0].str();
+	auto [begin, end, distance] = getStartOfWord(currentLine, cursorPosition, prompt.size());
+	currentLine.erase(begin, end);
+	currentLine.erase(0, prompt.size());
+	lines[0] = highLight(currentLine);
+	cursorPosition -= distance;
+}
+
+void CommandConsole::gotoEndOfWord()
+{
+	auto [begin, end, distance] = getEndOfWord(lines[0].str(), cursorPosition);
+	cursorPosition += distance;
+}
+
+void CommandConsole::deleteToEndOfWord()
+{
+	resetScrollBack();
+	currentLine = lines[0].str();
+	auto [begin, end, distance] = getEndOfWord(currentLine, cursorPosition);
+	currentLine.erase(begin, end);
+	currentLine.erase(0, prompt.size());
+	lines[0] = highLight(currentLine);
 }
 
 void CommandConsole::prevCommand()
@@ -524,7 +674,7 @@ void CommandConsole::prevCommand()
 	if (match) {
 		commandScrollBack = tmp;
 		lines[0] = highLight(history[commandScrollBack]);
-		cursorPosition = lines[0].numChars();
+		cursorPosition = unsigned(lines[0].numChars());
 	}
 }
 
@@ -544,10 +694,10 @@ void CommandConsole::nextCommand()
 		commandScrollBack = tmp;
 		lines[0] = highLight(history[commandScrollBack]);
 	} else {
-		commandScrollBack = history.size();
+		commandScrollBack = unsigned(history.size());
 		lines[0] = highLight(currentLine);
 	}
-	cursorPosition = lines[0].numChars();
+	cursorPosition = unsigned(lines[0].numChars());
 }
 
 void CommandConsole::clearCommand()
@@ -555,11 +705,19 @@ void CommandConsole::clearCommand()
 	resetScrollBack();
 	commandBuffer.clear();
 	prompt = PROMPT_NEW;
-	currentLine = prompt;
+	currentLine.clear();
 	lines[0] = highLight(currentLine);
 	cursorPosition = unsigned(prompt.size());
 }
 
+void CommandConsole::clearHistory()
+{
+	resetScrollBack();
+	while (lines.size() > 1) {
+		lines.removeBack();
+	}
+}
+
 void CommandConsole::backspace()
 {
 	resetScrollBack();
@@ -570,6 +728,7 @@ void CommandConsole::backspace()
 		auto e = b;
 		utf8::unchecked::advance(e, 1);
 		currentLine.erase(b, e);
+		currentLine.erase(0, prompt.size());
 		lines[0] = highLight(currentLine);
 		--cursorPosition;
 	}
@@ -585,18 +744,20 @@ void CommandConsole::delete_key()
 		auto e = b;
 		utf8::unchecked::advance(e, 1);
 		currentLine.erase(b, e);
+		currentLine.erase(0, prompt.size());
 		lines[0] = highLight(currentLine);
 	}
 }
 
-void CommandConsole::normalKey(uint16_t chr)
+void CommandConsole::normalKey(uint32_t chr)
 {
 	assert(chr);
 	resetScrollBack();
 	currentLine = lines[0].str();
 	auto pos = begin(currentLine);
 	utf8::unchecked::advance(pos, cursorPosition);
-	utf8::unchecked::append(uint32_t(chr), inserter(currentLine, pos));
+	utf8::unchecked::append(chr, inserter(currentLine, pos));
+	currentLine.erase(0, prompt.size());
 	lines[0] = highLight(currentLine);
 	++cursorPosition;
 }
@@ -606,4 +767,56 @@ void CommandConsole::resetScrollBack()
 	consoleScrollBack = 0;
 }
 
+static std::vector<std::string_view> splitLines(std::string_view str)
+{
+	// This differs from StringOp::split() in two ways:
+	// - If the input is an empty string, then the resulting vector
+	//   contains 1 element which is the empty string (StringOp::split()
+	//   would return an empty vector).
+	// - If the input ends on a newline character, then the final item in
+	//   the result vector is an empty string (StringOp::split() would not
+	//   include that last empty string).
+	// TODO can we come up with a good name for this function and move it
+	//      to StringOp?
+	std::vector<std::string_view> result;
+	while (true) {
+		auto pos = str.find_first_of('\n');
+		if (pos == std::string_view::npos) break;
+		result.push_back(str.substr(0, pos));
+		str = str.substr(pos + 1);
+	}
+	result.push_back(str);
+	return result;
+}
+
+void CommandConsole::paste()
+{
+	char* text = SDL_GetClipboardText();
+	if (!text) return;
+	scope_exit e([&]{ SDL_free(text); });
+
+	auto pastedLines = splitLines(text);
+	assert(!pastedLines.empty());
+
+	// helper function 'append()' to append to the existing text
+	std::string_view prefix = lines[0].str();
+	prefix.remove_prefix(prompt.size());
+	auto append = [&](std::string_view suffix) {
+		if (prefix.empty()) {
+			lines[0] = highLight(suffix);
+		} else {
+			lines[0] = highLight(strCat(prefix, suffix));
+			prefix = "";
+		}
+	};
+	// execute all but the last line
+	for (const auto& line : view::drop_back(pastedLines, 1)) {
+		append(line);
+		commandExecute();
+	}
+	// only enter (not execute) the last line
+	append(pastedLines.back());
+	cursorPosition = unsigned(lines[0].numChars());
+}
+
 } // namespace openmsx
