@@ -1,9 +1,6 @@
 #include "OSDConsoleRenderer.hh"
 #include "CommandConsole.hh"
-#include "EnumSetting.hh"
-#include "IntegerSetting.hh"
 #include "BooleanSetting.hh"
-#include "FilenameSetting.hh"
 #include "SDLImage.hh"
 #include "Display.hh"
 #include "InputEventGenerator.hh"
@@ -13,11 +10,12 @@
 #include "Reactor.hh"
 #include "MSXException.hh"
 #include "openmsx.hh"
-#include "memory.hh"
+#include "one_of.hh"
 #include "unreachable.hh"
 #include "xrange.hh"
 #include <algorithm>
 #include <cassert>
+#include <memory>
 
 #include "components.hh"
 #if COMPONENT_GL
@@ -25,6 +23,8 @@
 #endif
 
 using std::string;
+using std::string_view;
+using namespace gl;
 
 namespace openmsx {
 
@@ -32,99 +32,35 @@ namespace openmsx {
   * Note that when using a background image on the GLConsole,
   * that image's alpha channel is used instead.
   */
-static const int CONSOLE_ALPHA = 180;
-static const uint64_t BLINK_RATE = 500000; // us
-static const int CHAR_BORDER = 4;
-
-
-// class OSDConsoleRenderer::TextCacheElement
-
-OSDConsoleRenderer::TextCacheElement::TextCacheElement(
-		const std::string& text_, unsigned rgb_,
-		std::unique_ptr<BaseImage> image_, unsigned width_)
-	: text(text_), image(std::move(image_)), rgb(rgb_), width(width_)
-{
-}
-
-OSDConsoleRenderer::TextCacheElement::TextCacheElement(TextCacheElement&& rhs)
-	: text(std::move(rhs.text)), image(std::move(rhs.image))
-	, rgb(std::move(rhs.rgb)), width(std::move(rhs.width))
-{
-}
+constexpr int CONSOLE_ALPHA = 180;
+constexpr uint64_t BLINK_RATE = 500000; // us
+constexpr int CHAR_BORDER = 4;
 
 
 // class OSDConsoleRenderer
 
+constexpr string_view defaultFont = "skins/VeraMono.ttf.gz";
+
 OSDConsoleRenderer::OSDConsoleRenderer(
 		Reactor& reactor_, CommandConsole& console_,
 		unsigned screenW_, unsigned screenH_,
 		bool openGL_)
 	: Layer(COVER_NONE, Z_CONSOLE)
 	, reactor(reactor_)
+	, display(reactor.getDisplay()) // need to store because still needed during destructor
 	, console(console_)
 	, consoleSetting(console.getConsoleSetting())
 	, screenW(screenW_)
 	, screenH(screenH_)
 	, openGL(openGL_)
-{
-#if !COMPONENT_GL
-	assert(!openGL);
-#endif
-	destX = destY = destW = destH = 0; // recalc on first paint()
-	blink = false;
-	lastBlinkTime = Timer::getTime();
-	lastCursorX = lastCursorY = 0;
-
-	active = false;
-	activeTime = 0;
-	setCoverage(COVER_PARTIAL);
-
-	// font size
-	CommandController& commandController = reactor.getCommandController();
-	fontSizeSetting = make_unique<IntegerSetting>(commandController,
-		"consolefontsize", "Size of the console font", 12, 8, 32);
-
-	// font
-	const string& defaultFont = "skins/VeraMono.ttf.gz";
-	fontSetting = make_unique<FilenameSetting>(commandController,
-		"consolefont", "console font file", defaultFont);
-	fontSetting->setChecker([this](TclObject& value) {
-		loadFont(value.getString().str());
-	});
-	try {
-		loadFont(fontSetting->getString());
-	} catch (MSXException&) {
-		// This will happen when you upgrade from the old .png based
-		// fonts to the new .ttf fonts. So provide a smooth upgrade path.
-		reactor.getCliComm().printWarning(
-			"Loading selected font (" + fontSetting->getString() +
-			") failed. Reverting to default font (" + defaultFont + ").");
-		fontSetting->setString(defaultFont);
-		if (font.empty()) {
-			// we can't continue without font
-			throw FatalError("Couldn't load default console font.\n"
-			                 "Please check your installation.");
-		}
-	}
-
-	// rows / columns
-	int columns = (((screenW - CHAR_BORDER) / font.getWidth()) * 30) / 32;
-	int rows = ((screenH / font.getHeight()) * 6) / 15;
-	consoleColumnsSetting = make_unique<IntegerSetting>(commandController,
-		"consolecolumns", "number of columns in the console", columns,
-		32, 999);
-	consoleRowsSetting = make_unique<IntegerSetting>(commandController,
-		"consolerows", "number of rows in the console", rows, 1, 99);
-	adjustColRow();
-
-	// placement
-	// On Android, console must by default be placed on top, in order to prevent
-	// that it overlaps with the virtual Android keyboard, which is always placed
-	// at the bottom of the screen
-	consolePlacementSetting = make_unique<EnumSetting<Placement>>(
-		commandController, "consoleplacement",
+	, consolePlacementSetting(
+		reactor.getCommandController(), "consoleplacement",
 		"position of the console within the emulator",
-		PLATFORM_ANDROID ? CP_TOP : CP_BOTTOM, EnumSetting<Placement>::Map{
+		// On Android, console must by default be placed on top, in
+		// order to prevent that it overlaps with the virtual Android
+		// keyboard, which is always placed at the bottom of the screen
+		PLATFORM_ANDROID ? CP_TOP : CP_BOTTOM,
+		EnumSetting<Placement>::Map{
 			{"topleft",     CP_TOPLEFT},
 			{"top",         CP_TOP},
 			{"topright",    CP_TOPRIGHT},
@@ -133,27 +69,80 @@ OSDConsoleRenderer::OSDConsoleRenderer(
 			{"right",       CP_RIGHT},
 			{"bottomleft",  CP_BOTTOMLEFT},
 			{"bottom",      CP_BOTTOM},
-			{"bottomright", CP_BOTTOMRIGHT}});
+			{"bottomright", CP_BOTTOMRIGHT}})
+	, fontSizeSetting(reactor.getCommandController(),
+		"consolefontsize", "Size of the console font", 12, 8, 32)
+	, fontSetting(reactor.getCommandController(),
+		"consolefont", "console font file", defaultFont)
+	, consoleColumnsSetting(reactor.getCommandController(),
+		"consolecolumns", "number of columns in the console",
+		initFontAndGetColumns(), 32, 999)
+	, consoleRowsSetting(reactor.getCommandController(),
+		"consolerows", "number of rows in the console",
+		getRows(), 1, 99)
+	, backgroundSetting(reactor.getCommandController(),
+		"consolebackground", "console background file",
+		"skins/ConsoleBackgroundGrey.png")
+{
+#if !COMPONENT_GL
+	assert(!openGL);
+#endif
+	bgPos = bgSize = ivec2(); // recalc on first paint()
+	blink = false;
+	lastBlinkTime = Timer::getTime();
+	lastCursorX = lastCursorY = 0;
+
+	active = false;
+	activeTime = 0;
+	setCoverage(COVER_PARTIAL);
+
+	adjustColRow();
 
 	// background (only load backgound on first paint())
-	backgroundSetting = make_unique<FilenameSetting>(commandController,
-		"consolebackground", "console background file",
-		"skins/ConsoleBackgroundGrey.png");
-	backgroundSetting->setChecker([this](TclObject& value) {
-		loadBackground(value.getString().str());
+	backgroundSetting.setChecker([this](TclObject& value) {
+		loadBackground(value.getString());
 	});
 	// don't yet load background
 
 	consoleSetting.attach(*this);
-	fontSetting->attach(*this);
-	fontSizeSetting->attach(*this);
+	fontSetting.attach(*this);
+	fontSizeSetting.attach(*this);
 	setActive(consoleSetting.getBoolean());
 }
 
+int OSDConsoleRenderer::initFontAndGetColumns()
+{
+	// init font
+	fontSetting.setChecker([this](TclObject& value) {
+		loadFont(string(value.getString()));
+	});
+	try {
+		loadFont(fontSetting.getString());
+	} catch (MSXException&) {
+		// This will happen when you upgrade from the old .png based
+		// fonts to the new .ttf fonts. So provide a smooth upgrade path.
+		reactor.getCliComm().printWarning(
+			"Loading selected font (", fontSetting.getString(),
+			") failed. Reverting to default font (", defaultFont, ").");
+		fontSetting.setString(defaultFont);
+		if (font.empty()) {
+			// we can't continue without font
+			throw FatalError("Couldn't load default console font.\n"
+			                 "Please check your installation.");
+		}
+	}
+
+	return (((screenW - CHAR_BORDER) / font.getWidth()) * 30) / 32;
+}
+int OSDConsoleRenderer::getRows()
+{
+	// initFontAndGetColumns() must already be called
+	return ((screenH / font.getHeight()) * 6) / 15;
+}
 OSDConsoleRenderer::~OSDConsoleRenderer()
 {
-	fontSizeSetting->detach(*this);
-	fontSetting->detach(*this);
+	fontSizeSetting.detach(*this);
+	fontSetting.detach(*this);
 	consoleSetting.detach(*this);
 	setActive(false);
 }
@@ -161,10 +150,10 @@ OSDConsoleRenderer::~OSDConsoleRenderer()
 void OSDConsoleRenderer::adjustColRow()
 {
 	unsigned consoleColumns = std::min<unsigned>(
-		consoleColumnsSetting->getInt(),
+		consoleColumnsSetting.getInt(),
 		(screenW - CHAR_BORDER) / font.getWidth());
 	unsigned consoleRows = std::min<unsigned>(
-		consoleRowsSetting->getInt(),
+		consoleRowsSetting.getInt(),
 		screenH / font.getHeight());
 	console.setColumns(consoleColumns);
 	console.setRows(consoleRows);
@@ -174,9 +163,8 @@ void OSDConsoleRenderer::update(const Setting& setting)
 {
 	if (&setting == &consoleSetting) {
 		setActive(consoleSetting.getBoolean());
-	} else if ((&setting == fontSetting.get()) ||
-	           (&setting == fontSizeSetting.get())) {
-		loadFont(fontSetting->getString());
+	} else if (&setting == one_of(&fontSetting, &fontSizeSetting)) {
+		loadFont(fontSetting.getString());
 	} else {
 		UNREACHABLE;
 	}
@@ -187,11 +175,9 @@ void OSDConsoleRenderer::setActive(bool active_)
 	if (active == active_) return;
 	active = active_;
 
-	reactor.getDisplay().repaintDelayed(40000); // 25 fps
+	display.repaintDelayed(40000); // 25 fps
 
 	activeTime = Timer::getTime();
-
-	reactor.getInputEventGenerator().setKeyRepeat(active);
 }
 
 byte OSDConsoleRenderer::getVisibility() const
@@ -205,14 +191,14 @@ byte OSDConsoleRenderer::getVisibility() const
 		if (dur > FADE_IN_DURATION) {
 			return 255;
 		} else {
-			reactor.getDisplay().repaintDelayed(40000); // 25 fps
+			display.repaintDelayed(40000); // 25 fps
 			return byte((dur * 255) / FADE_IN_DURATION);
 		}
 	} else {
 		if (dur > FADE_OUT_DURATION) {
 			return 0;
 		} else {
-			reactor.getDisplay().repaintDelayed(40000); // 25 fps
+			display.repaintDelayed(40000); // 25 fps
 			return byte(255 - ((dur * 255) / FADE_OUT_DURATION));
 		}
 	}
@@ -222,114 +208,119 @@ bool OSDConsoleRenderer::updateConsoleRect()
 {
 	adjustColRow();
 
-	unsigned x, y, w, h;
-	h = font.getHeight() * console.getRows();
-	w = (font.getWidth() * console.getColumns()) + CHAR_BORDER;
+	ivec2 size((font.getWidth()  * console.getColumns()) + CHAR_BORDER,
+	            font.getHeight() * console.getRows());
 
 	// TODO use setting listener in the future
-	switch (consolePlacementSetting->getEnum()) {
+	ivec2 pos;
+	switch (consolePlacementSetting.getEnum()) {
 		case CP_TOPLEFT:
 		case CP_LEFT:
 		case CP_BOTTOMLEFT:
-			x = 0;
+			pos[0] = 0;
 			break;
 		case CP_TOPRIGHT:
 		case CP_RIGHT:
 		case CP_BOTTOMRIGHT:
-			x = (screenW - w);
+			pos[0] = (screenW - size[0]);
 			break;
 		case CP_TOP:
 		case CP_CENTER:
 		case CP_BOTTOM:
 		default:
-			x = (screenW - w) / 2;
+			pos[0] = (screenW - size[0]) / 2;
 			break;
 	}
-	switch (consolePlacementSetting->getEnum()) {
+	switch (consolePlacementSetting.getEnum()) {
 		case CP_TOPLEFT:
 		case CP_TOP:
 		case CP_TOPRIGHT:
-			y = 0;
+			pos[1] = 0;
 			break;
 		case CP_LEFT:
 		case CP_CENTER:
 		case CP_RIGHT:
-			y = (screenH - h) / 2;
+			pos[1] = (screenH - size[1]) / 2;
 			break;
 		case CP_BOTTOMLEFT:
 		case CP_BOTTOM:
 		case CP_BOTTOMRIGHT:
 		default:
-			y = (screenH - h);
+			pos[1] = (screenH - size[1]);
 			break;
 	}
 
-	bool result = (x != destX) || (y != destY) ||
-	              (w != destW) || (h != destH);
-	destX = x; destY = y; destW = w; destH = h;
+	bool result = (pos != bgPos) || (size != bgSize);
+	bgPos  = pos;
+	bgSize = size;
 	return result;
 }
 
-void OSDConsoleRenderer::loadFont(const string& value)
+void OSDConsoleRenderer::loadFont(string_view value)
 {
-	string filename = SystemFileContext().resolve(value);
-	auto newFont = TTFFont(filename, fontSizeSetting->getInt());
+	string filename = systemFileContext().resolve(value);
+	auto newFont = TTFFont(filename, fontSizeSetting.getInt());
 	if (!newFont.isFixedWidth()) {
-		throw MSXException(value + " is not a monospaced font");
+		throw MSXException(value, " is not a monospaced font");
 	}
 	font = std::move(newFont);
 	clearCache();
 }
 
-void OSDConsoleRenderer::loadBackground(const string& value)
+void OSDConsoleRenderer::loadBackground(string_view value)
 {
 	if (value.empty()) {
 		backgroundImage.reset();
 		return;
 	}
-	string filename = SystemFileContext().resolve(value);
+	auto* output = display.getOutputSurface();
+	if (!output) {
+		backgroundImage.reset();
+		return;
+	}
+	string filename = systemFileContext().resolve(value);
 	if (!openGL) {
-		backgroundImage = make_unique<SDLImage>(filename, destW, destH);
+		backgroundImage = std::make_unique<SDLImage>(*output, filename, bgSize);
 	}
 #if COMPONENT_GL
 	else {
-		backgroundImage = make_unique<GLImage>(filename, destW, destH);
+		backgroundImage = std::make_unique<GLImage>(*output, filename, bgSize);
 	}
 #endif
 }
 
 void OSDConsoleRenderer::drawText(OutputSurface& output, const ConsoleLine& line,
-                                  int x, int y, byte alpha)
+                                  ivec2 pos, byte alpha)
 {
-	unsigned chunks = line.numChunks();
-	for (unsigned i = 0; i < chunks; ++i) {
-		unsigned rgb = line.chunkColor(i);
-		string_ref text = line.chunkText(i);
-		drawText2(output, text, x, y, alpha, rgb);
+	for (auto i : xrange(line.numChunks())) {
+		auto rgb = line.chunkColor(i);
+		string_view text = line.chunkText(i);
+		drawText2(output, text, pos[0], pos[1], alpha, rgb);
 	}
 }
 
-void OSDConsoleRenderer::drawText2(OutputSurface& output, string_ref text,
+void OSDConsoleRenderer::drawText2(OutputSurface& output, string_view text,
                                    int& x, int y, byte alpha, unsigned rgb)
 {
 	unsigned width;
 	BaseImage* image;
 	if (!getFromCache(text, rgb, image, width)) {
-		string textStr = text.str();
+		string textStr(text);
 		SDLSurfacePtr surf;
+		unsigned rgb2 = openGL ? 0xffffff : rgb; // openGL -> always render white
 		try {
 			unsigned dummyHeight;
 			font.getSize(textStr, width, dummyHeight);
 			surf = font.render(textStr,
-			                   (rgb >> 16) & 0xff,
-			                   (rgb >>  8) & 0xff,
-			                   (rgb >>  0) & 0xff);
+			                   (rgb2 >> 16) & 0xff,
+			                   (rgb2 >>  8) & 0xff,
+			                   (rgb2 >>  0) & 0xff);
 		} catch (MSXException& e) {
 			static bool alreadyPrinted = false;
 			if (!alreadyPrinted) {
 				alreadyPrinted = true;
 				reactor.getCliComm().printWarning(
-					"Invalid console text (invalid UTF-8): " +
+					"Invalid console text (invalid UTF-8): ",
 					e.getMessage());
 			}
 			return; // don't cache negative results
@@ -338,21 +329,30 @@ void OSDConsoleRenderer::drawText2(OutputSurface& output, string_ref text,
 		if (!surf) {
 			// nothing was rendered, so do nothing
 		} else if (!openGL) {
-			image2 = make_unique<SDLImage>(std::move(surf));
+			image2 = std::make_unique<SDLImage>(output, std::move(surf));
 		}
 #if COMPONENT_GL
 		else {
-			image2 = make_unique<GLImage>(std::move(surf));
+			image2 = std::make_unique<GLImage>(output, std::move(surf));
 		}
 #endif
 		image = image2.get();
-		insertInCache(textStr, rgb, std::move(image2), width);
+		insertInCache(std::move(textStr), rgb, std::move(image2), width);
+	}
+	if (image) {
+		if (openGL) {
+			byte r = (rgb >> 16) & 0xff;
+			byte g = (rgb >>  8) & 0xff;
+			byte b = (rgb >>  0) & 0xff;
+			image->draw(output, ivec2(x, y), r, g, b, alpha);
+		} else {
+			image->draw(output, ivec2(x, y), alpha);
+		}
 	}
-	if (image) image->draw(output, x, y, alpha);
 	x += width; // in case of trailing whitespace width != image->getWidth()
 }
 
-bool OSDConsoleRenderer::getFromCache(string_ref text, unsigned rgb,
+bool OSDConsoleRenderer::getFromCache(string_view text, unsigned rgb,
                                       BaseImage*& image, unsigned& width)
 {
 	// Items are LRU sorted, so the next requested items will often be
@@ -360,7 +360,8 @@ bool OSDConsoleRenderer::getFromCache(string_ref text, unsigned rgb,
 	// duplicate items (e.g. the command prompt '> ') degrade this
 	// heuristic).
 	auto it = cacheHint;
-	if ((it->text == text) && (it->rgb  == rgb)) {
+	// For openGL ignore rgb
+	if ((it->text == text) && (openGL || (it->rgb  == rgb))) {
 		goto found;
 	}
 
@@ -369,7 +370,7 @@ bool OSDConsoleRenderer::getFromCache(string_ref text, unsigned rgb,
 	// in the N first positions in the cache (in approx reverse order).
 	for (it = begin(textCache); it != end(textCache); ++it) {
 		if (it->text != text) continue;
-		if (it->rgb  != rgb ) continue;
+		if (!openGL && (it->rgb  != rgb)) continue;
 found:		image = it->image.get();
 		width = it->width;
 		cacheHint = it;
@@ -384,31 +385,35 @@ found:		image = it->image.get();
 }
 
 void OSDConsoleRenderer::insertInCache(
-	const string& text, unsigned rgb, std::unique_ptr<BaseImage> image,
+	string text, unsigned rgb, std::unique_ptr<BaseImage> image,
 	unsigned width)
 {
-	static const unsigned MAX_TEXT_CACHE_SIZE = 250;
+	constexpr unsigned MAX_TEXT_CACHE_SIZE = 250;
 	if (textCache.size() == MAX_TEXT_CACHE_SIZE) {
 		// flush the least recently used entry
-		auto it = end(textCache);
-		--it;
+		auto it = std::prev(std::end(textCache));
 		if (it == cacheHint) {
 			cacheHint = begin(textCache);
 		}
 		textCache.pop_back();
 	}
-	textCache.push_front(TextCacheElement(
-		text, rgb, std::move(image), width));
+	textCache.emplace_front(std::move(text), rgb, std::move(image), width);
 }
 
 void OSDConsoleRenderer::clearCache()
 {
 	// cacheHint must always point to a valid item, so insert a dummy entry
 	textCache.clear();
-	textCache.emplace_back("", 0, nullptr, 0);
+	textCache.emplace_back(string{}, 0, nullptr, 0);
 	cacheHint = begin(textCache);
 }
 
+gl::ivec2 OSDConsoleRenderer::getTextPos(int cursorX, int cursorY)
+{
+	return bgPos + ivec2(CHAR_BORDER + cursorX * font.getWidth(),
+	                     bgSize[1] - (font.getHeight() * (cursorY + 1)) - 1);
+}
+
 void OSDConsoleRenderer::paint(OutputSurface& output)
 {
 	byte visibility = getVisibility();
@@ -416,7 +421,7 @@ void OSDConsoleRenderer::paint(OutputSurface& output)
 
 	if (updateConsoleRect()) {
 		try {
-			loadBackground(backgroundSetting->getString());
+			loadBackground(backgroundSetting.getString());
 		} catch (MSXException& e) {
 			reactor.getCliComm().printWarning(e.getMessage());
 		}
@@ -427,13 +432,13 @@ void OSDConsoleRenderer::paint(OutputSurface& output)
 		// no background image, try to create an empty one
 		try {
 			if (!openGL) {
-				backgroundImage = make_unique<SDLImage>(
-					destW, destH, CONSOLE_ALPHA);
+				backgroundImage = std::make_unique<SDLImage>(
+					output, bgSize, CONSOLE_ALPHA);
 			}
 #if COMPONENT_GL
 			else {
-				backgroundImage = make_unique<GLImage>(
-					destW, destH, CONSOLE_ALPHA);
+				backgroundImage = std::make_unique<GLImage>(
+					output, bgSize, CONSOLE_ALPHA);
 			}
 #endif
 		} catch (MSXException&) {
@@ -441,15 +446,13 @@ void OSDConsoleRenderer::paint(OutputSurface& output)
 		}
 	}
 	if (backgroundImage) {
-		backgroundImage->draw(output, destX, destY, visibility);
+		backgroundImage->draw(output, bgPos, visibility);
 	}
 
-	for (auto loop : xrange(destH / font.getHeight())) {
+	for (auto loop : xrange(bgSize[1] / font.getHeight())) {
 		drawText(output,
 		         console.getLine(loop + console.getScrollBack()),
-		         destX + CHAR_BORDER,
-		         destY + destH - (1 + loop) * font.getHeight() - 1,
-		         visibility);
+		         getTextPos(0, loop), visibility);
 	}
 
 	// Check if the blink period is over
@@ -469,9 +472,7 @@ void OSDConsoleRenderer::paint(OutputSurface& output)
 	}
 	if (blink && (console.getScrollBack() == 0)) {
 		drawText(output, ConsoleLine("_"),
-		         destX + CHAR_BORDER + cursorX * font.getWidth(),
-		         destY + destH - (font.getHeight() * (cursorY + 1)) - 1,
-		         visibility);
+		         getTextPos(cursorX, cursorY), visibility);
 	}
 }
 
