@@ -1,57 +1,29 @@
 #include "OSDGUI.hh"
 #include "OSDWidget.hh"
-#include "OSDTopWidget.hh"
 #include "OSDRectangle.hh"
 #include "OSDText.hh"
 #include "Display.hh"
-#include "Command.hh"
 #include "CommandException.hh"
 #include "TclObject.hh"
 #include "StringOp.hh"
-#include "memory.hh"
-#include <algorithm>
-#include <cassert>
+#include "one_of.hh"
+#include "outer.hh"
+#include <memory>
+#include <utility>
 
 using std::string;
+using std::string_view;
 using std::unique_ptr;
 using std::vector;
 
 namespace openmsx {
 
-class OSDCommand final : public Command
-{
-public:
-	OSDCommand(OSDGUI& gui, CommandController& commandController);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
-	string help(const vector<string>& tokens) const override;
-	void tabCompletion(vector<string>& tokens) const override;
-
-private:
-	void create   (array_ref<TclObject> tokens, TclObject& result);
-	void destroy  (array_ref<TclObject> tokens, TclObject& result);
-	void info     (array_ref<TclObject> tokens, TclObject& result);
-	void exists   (array_ref<TclObject> tokens, TclObject& result);
-	void configure(array_ref<TclObject> tokens, TclObject& result);
-	unique_ptr<OSDWidget> create(string_ref type, const string& name) const;
-	void configure(OSDWidget& widget, array_ref<TclObject> tokens,
-	               unsigned skip);
-
-	OSDWidget& getWidget(string_ref name) const;
-
-	OSDGUI& gui;
-};
-
-
 // class OSDGUI
 
 OSDGUI::OSDGUI(CommandController& commandController, Display& display_)
 	: display(display_)
-	, osdCommand(make_unique<OSDCommand>(*this, commandController))
-	, topWidget(make_unique<OSDTopWidget>(*this))
-{
-}
-
-OSDGUI::~OSDGUI()
+	, osdCommand(commandController)
+	, topWidget(display_)
 {
 }
 
@@ -63,108 +35,101 @@ void OSDGUI::refresh() const
 
 // class OSDCommand
 
-OSDCommand::OSDCommand(OSDGUI& gui_, CommandController& commandController)
-	: Command(commandController, "osd")
-	, gui(gui_)
+OSDGUI::OSDCommand::OSDCommand(CommandController& commandController_)
+	: Command(commandController_, "osd")
 {
 }
 
-void OSDCommand::execute(array_ref<TclObject> tokens, TclObject& result)
+void OSDGUI::OSDCommand::execute(span<const TclObject> tokens, TclObject& result)
 {
-	if (tokens.size() < 2) {
-		throw SyntaxError();
-	}
-	string_ref subCommand = tokens[1].getString();
-	if (subCommand == "create") {
-		create(tokens, result);
-		gui.refresh();
-	} else if (subCommand == "destroy") {
-		destroy(tokens, result);
-		gui.refresh();
-	} else if (subCommand == "info") {
-		info(tokens, result);
-	} else if (subCommand == "exists") {
-		exists(tokens, result);
-	} else if (subCommand == "configure") {
-		configure(tokens, result);
-		gui.refresh();
-	} else {
+	checkNumArgs(tokens, AtLeast{2}, "subcommand ?arg ...?");
+	executeSubCommand(tokens[1].getString(),
+		"create",    [&]{ create(tokens, result); },
+		"destroy",   [&]{ destroy(tokens, result); },
+		"info",      [&]{ info(tokens, result); },
+		"exists",    [&]{ exists(tokens, result); },
+		"configure", [&]{ configure(tokens, result); });
+}
+
+void OSDGUI::OSDCommand::create(span<const TclObject> tokens, TclObject& result)
+{
+	checkNumArgs(tokens, AtLeast{4}, Prefix{2}, "type name ?property value ...?");
+	string_view type = tokens[2].getString();
+	auto& fullname = tokens[3];
+	auto fullnameStr = fullname.getString();
+
+	auto& gui = OUTER(OSDGUI, osdCommand);
+	auto& top = gui.getTopWidget();
+	if (top.findByName(fullnameStr)) {
 		throw CommandException(
-			"Invalid subcommand '" + subCommand + "', expected "
-			"'create', 'destroy', 'info', 'exists' or 'configure'.");
+			"There already exists a widget with this name: ",
+			fullnameStr);
 	}
-}
 
-void OSDCommand::create(array_ref<TclObject> tokens, TclObject& result)
-{
-	if (tokens.size() < 4) {
-		throw SyntaxError();
-	}
-	string_ref type = tokens[2].getString();
-	string_ref fullname = tokens[3].getString();
-	string_ref parentname, name;
-	StringOp::splitOnLast(fullname, '.', parentname, name);
-	if (name.empty()) std::swap(parentname, name);
-
-	auto* parent = gui.getTopWidget().findSubWidget(parentname);
+	auto [parentname, childName] = StringOp::splitOnLast(fullnameStr, '.');
+	auto* parent = childName.empty() ? &top : top.findByName(parentname);
 	if (!parent) {
 		throw CommandException(
-			"Parent widget doesn't exist yet:" + parentname);
-	}
-	if (parent->findSubWidget(name)) {
-		throw CommandException(
-			"There already exists a widget with this name: " +
-			fullname);
+			"Parent widget doesn't exist yet:", parentname);
 	}
 
-	auto widget = create(type, name.str());
-	configure(*widget, tokens, 4);
+	auto widget = create(type, fullname);
+	configure(*widget, tokens.subspan(4));
+	top.addName(*widget);
 	parent->addWidget(std::move(widget));
 
-	result.setString(fullname);
+	result = fullname;
+	gui.refresh();
 }
 
-unique_ptr<OSDWidget> OSDCommand::create(
-		string_ref type, const string& name) const
+unique_ptr<OSDWidget> OSDGUI::OSDCommand::create(
+	string_view type, const TclObject& name) const
 {
+	auto& gui = OUTER(OSDGUI, osdCommand);
 	if (type == "rectangle") {
-		return make_unique<OSDRectangle>(gui, name);
+		return std::make_unique<OSDRectangle>(gui.display, name);
 	} else if (type == "text") {
-		return make_unique<OSDText>(gui, name);
+		return std::make_unique<OSDText>(gui.display, name);
 	} else {
 		throw CommandException(
-			"Invalid widget type '" + type + "', expected "
+			"Invalid widget type '", type, "', expected "
 			"'rectangle' or 'text'.");
 	}
 }
 
-void OSDCommand::destroy(array_ref<TclObject> tokens, TclObject& result)
+void OSDGUI::OSDCommand::destroy(span<const TclObject> tokens, TclObject& result)
 {
-	if (tokens.size() != 3) {
-		throw SyntaxError();
-	}
-	auto* widget = gui.getTopWidget().findSubWidget(tokens[2].getString());
+	checkNumArgs(tokens, 3, "name");
+	auto fullname = tokens[2].getString();
+
+	auto& gui = OUTER(OSDGUI, osdCommand);
+	auto& top = gui.getTopWidget();
+	auto* widget = top.findByName(fullname);
 	if (!widget) {
 		// widget not found, not an error
-		result.setBoolean(false);
+		result = false;
 		return;
 	}
+
 	auto* parent = widget->getParent();
 	if (!parent) {
 		throw CommandException("Can't destroy the top widget.");
 	}
+
+	top.removeName(*widget);
 	parent->deleteWidget(*widget);
-	result.setBoolean(true);
+	result = true;
+	gui.refresh();
 }
 
-void OSDCommand::info(array_ref<TclObject> tokens, TclObject& result)
+void OSDGUI::OSDCommand::info(span<const TclObject> tokens, TclObject& result)
 {
+	checkNumArgs(tokens, Between{2, 4}, Prefix{2}, "?name? ?property?");
+	auto& gui = OUTER(OSDGUI, osdCommand);
 	switch (tokens.size()) {
 	case 2: {
 		// list widget names
-		vector<string> names;
-		gui.getTopWidget().listWidgetNames("", names);
-		result.addListElements(names);
+		result.addListElements(gui.getTopWidget().getAllWidgetNames());
 		break;
 	}
 	case 3: {
@@ -179,46 +144,41 @@ void OSDCommand::info(array_ref<TclObject> tokens, TclObject& result)
 		widget.getProperty(tokens[3].getString(), result);
 		break;
 	}
-	default:
-		throw SyntaxError();
 	}
 }
 
-void OSDCommand::exists(array_ref<TclObject> tokens, TclObject& result)
+void OSDGUI::OSDCommand::exists(span<const TclObject> tokens, TclObject& result)
 {
-	if (tokens.size() != 3) {
-		throw SyntaxError();
-	}
-	auto* widget = gui.getTopWidget().findSubWidget(tokens[2].getString());
-	result.setBoolean(widget != nullptr);
+	checkNumArgs(tokens, 3, "name");
+	auto& gui = OUTER(OSDGUI, osdCommand);
+	auto* widget = gui.getTopWidget().findByName(tokens[2].getString());
+	result = widget != nullptr;
 }
 
-void OSDCommand::configure(array_ref<TclObject> tokens, TclObject& /*result*/)
+void OSDGUI::OSDCommand::configure(span<const TclObject> tokens, TclObject& /*result*/)
 {
-	if (tokens.size() < 3) {
-		throw SyntaxError();
-	}
-	configure(getWidget(tokens[2].getString()), tokens, 3);
+	checkNumArgs(tokens, AtLeast{3}, "name ?property value ...?");
+	configure(getWidget(tokens[2].getString()), tokens.subspan(3));
+	auto& gui = OUTER(OSDGUI, osdCommand);
+	gui.refresh();
 }
 
-void OSDCommand::configure(OSDWidget& widget, array_ref<TclObject> tokens,
-                           unsigned skip)
+void OSDGUI::OSDCommand::configure(OSDWidget& widget, span<const TclObject> tokens)
 {
-	assert(tokens.size() >= skip);
-	if ((tokens.size() - skip) & 1) {
+	if (tokens.size() & 1) {
 		// odd number of extra arguments
 		throw CommandException(
-			"Missing value for '" + tokens.back().getString() + "'.");
+			"Missing value for '", tokens.back().getString(), "'.");
 	}
 
 	auto& interp = getInterpreter();
-	for (size_t i = skip; i < tokens.size(); i += 2) {
-		const auto& name = tokens[i + 0].getString();
-		widget.setProperty(interp, name, tokens[i + 1]);
+	for (size_t i = 0; i < tokens.size(); i += 2) {
+		const auto& propName = tokens[i + 0].getString();
+		widget.setProperty(interp, propName, tokens[i + 1]);
 	}
 }
 
-string OSDCommand::help(const vector<string>& tokens) const
+string OSDGUI::OSDCommand::help(const vector<string>& tokens) const
 {
 	if (tokens.size() >= 2) {
 		if (tokens[1] == "create") {
@@ -281,29 +241,27 @@ string OSDCommand::help(const vector<string>& tokens) const
 	}
 }
 
-void OSDCommand::tabCompletion(vector<string>& tokens) const
+void OSDGUI::OSDCommand::tabCompletion(vector<string>& tokens) const
 {
+	auto& gui = OUTER(OSDGUI, osdCommand);
 	if (tokens.size() == 2) {
-		static const char* const cmds[] = {
+		static constexpr const char* const cmds[] = {
 			"create", "destroy", "info", "exists", "configure"
 		};
 		completeString(tokens, cmds);
 	} else if ((tokens.size() == 3) && (tokens[1] == "create")) {
-		static const char* const types[] = { "rectangle", "text" };
+		static constexpr const char* const types[] = { "rectangle", "text" };
 		completeString(tokens, types);
 	} else if ((tokens.size() == 3) ||
 	           ((tokens.size() == 4) && (tokens[1] == "create"))) {
-		vector<string> names;
-		gui.getTopWidget().listWidgetNames("", names);
-		completeString(tokens, names);
+		completeString(tokens, gui.getTopWidget().getAllWidgetNames());
 	} else {
 		try {
-			vector<string_ref> properties;
+			vector<string_view> properties;
 			if (tokens[1] == "create") {
-				auto widget = create(tokens[2], "");
+				auto widget = create(tokens[2], TclObject());
 				properties = widget->getProperties();
-			} else if ((tokens[1] == "configure") ||
-			           (tokens[1] == "info")) {
+			} else if (tokens[1] == one_of("configure", "info")) {
 				const auto& widget = getWidget(tokens[2]);
 				properties = widget.getProperties();
 			}
@@ -314,11 +272,12 @@ void OSDCommand::tabCompletion(vector<string>& tokens) const
 	}
 }
 
-OSDWidget& OSDCommand::getWidget(string_ref name) const
+OSDWidget& OSDGUI::OSDCommand::getWidget(string_view name) const
 {
-	auto* widget = gui.getTopWidget().findSubWidget(name);
+	auto& gui = OUTER(OSDGUI, osdCommand);
+	auto* widget = gui.getTopWidget().findByName(name);
 	if (!widget) {
-		throw CommandException("No widget with name " + name);
+		throw CommandException("No widget with name ", name);
 	}
 	return *widget;
 }
