@@ -2,13 +2,12 @@
 #include "LocalFileReference.hh"
 #include "MSXException.hh"
 #include "StringOp.hh"
-#include "memory.hh"
+#include "ranges.hh"
 #include "stl.hh"
 #include "xrange.hh"
 #include <SDL_ttf.h>
-#include <algorithm>
-#include <vector>
 #include <cassert>
+#include <vector>
 
 using std::string;
 
@@ -32,7 +31,7 @@ public:
 	void release(TTF_Font* font);
 
 private:
-	TTFFontPool();
+	TTFFontPool() = default;
 	~TTFFontPool();
 
 	// We want to keep the LocalFileReference object alive for as long as
@@ -50,22 +49,7 @@ private:
 	// of step 3 and 4. Though this has the disadvantage that if openMSX
 	// crashes between step 3 and 4 the temp file is still left behind.
 	struct FontInfo {
-		// TODO use compiler generated versions once VS supports this
-		FontInfo() {}
-		FontInfo(FontInfo&& rhs)
-			: file(std::move(rhs.file)), font(std::move(rhs.font))
-			, name(std::move(rhs.name)), size(std::move(rhs.size))
-			, count(std::move(rhs.count)) {}
-		FontInfo& operator=(FontInfo&& rhs) {
-			file  = std::move(rhs.file);
-			font  = std::move(rhs.font);
-			name  = std::move(rhs.name);
-			size  = std::move(rhs.size);
-			count = std::move(rhs.count);
-			return *this;
-		}
-
-		std::unique_ptr<LocalFileReference> file;
+		LocalFileReference file;
 		TTF_Font* font;
 		std::string name;
 		int size;
@@ -80,8 +64,7 @@ private:
 SDLTTF::SDLTTF()
 {
 	if (TTF_Init() < 0) {
-		throw FatalError(StringOp::Builder() <<
-			"Couldn't initialize SDL_ttf: " << TTF_GetError());
+		throw FatalError("Couldn't initialize SDL_ttf: ", TTF_GetError());
 	}
 }
 
@@ -99,10 +82,6 @@ SDLTTF& SDLTTF::instance()
 
 // class TTFFontPool
 
-TTFFontPool::TTFFontPool()
-{
-}
-
 TTFFontPool::~TTFFontPool()
 {
 	assert(pool.empty());
@@ -116,9 +95,9 @@ TTFFontPool& TTFFontPool::instance()
 
 TTF_Font* TTFFontPool::get(const string& filename, int ptSize)
 {
-	auto it = find_if(begin(pool), end(pool),
-		[&](const FontInfo& i) {
-			return (i.name == filename) && (i.size == ptSize); });
+	auto it = ranges::find_if(pool, [&](auto& info) {
+		return (info.name == filename) && (info.size == ptSize);
+	});
 	if (it != end(pool)) {
 		++it->count;
 		return it->font;
@@ -126,8 +105,8 @@ TTF_Font* TTFFontPool::get(const string& filename, int ptSize)
 
 	SDLTTF::instance(); // init library
 	FontInfo info;
-	info.file = make_unique<LocalFileReference>(filename);
-	auto* result = TTF_OpenFont(info.file->getFilename().c_str(), ptSize);
+	info.file = LocalFileReference(filename);
+	auto* result = TTF_OpenFont(info.file.getFilename().c_str(), ptSize);
 	if (!result) {
 		throw MSXException(TTF_GetError());
 	}
@@ -141,13 +120,12 @@ TTF_Font* TTFFontPool::get(const string& filename, int ptSize)
 
 void TTFFontPool::release(TTF_Font* font)
 {
-	auto it = find_if_unguarded(pool,
+	auto it = rfind_if_unguarded(pool,
 		[&](const FontInfo& i) { return i.font == font; });
 	--it->count;
 	if (it->count == 0) {
 		TTF_CloseFont(it->font);
-		if (it != (end(pool) - 1)) std::swap(*it, *(end(pool) - 1));
-		pool.pop_back();
+		move_pop_back(pool, it);
 	}
 }
 
@@ -195,7 +173,7 @@ SDLSurfacePtr TTFFont::render(std::string text, byte r, byte g, byte b) const
 	unsigned lineHeight = 0; // initialize to avoid warning
 	for (auto& s : lines) {
 		unsigned w;
-		getSize(s.str(), w, lineHeight);
+		getSize(string(s), w, lineHeight);
 		width = std::max(width, w);
 	}
 	// There might be extra space between two successive lines
@@ -203,7 +181,7 @@ SDLSurfacePtr TTFFont::render(std::string text, byte r, byte g, byte b) const
 	unsigned lineSkip = getHeight();
 	// We assume that height is the same for all lines.
 	// For the last line we don't include spacing between two lines.
-	unsigned height = unsigned((lines.size() - 1) * lineSkip + lineHeight);
+	auto height = unsigned((lines.size() - 1) * lineSkip + lineHeight);
 
 	// Create destination surface (initial surface is fully transparent)
 	SDLSurfacePtr destination(SDL_CreateRGBSurface(SDL_SWSURFACE, width, height,
@@ -222,7 +200,7 @@ SDLSurfacePtr TTFFont::render(std::string text, byte r, byte g, byte b) const
 		}
 		SDLSurfacePtr line(TTF_RenderUTF8_Blended(
 			static_cast<TTF_Font*>(font),
-			lines[i].str().c_str(), color));
+			string(lines[i]).c_str(), color));
 		if (!line) {
 			throw MSXException(TTF_GetError());
 		}
@@ -231,7 +209,7 @@ SDLSurfacePtr TTFFont::render(std::string text, byte r, byte g, byte b) const
 		SDL_Rect rect;
 		rect.x = 0;
 		rect.y = Sint16(i * lineSkip);
-		SDL_SetAlpha(line.get(), 0, 0); // no blending during copy
+		SDL_SetSurfaceBlendMode(line.get(), SDL_BLENDMODE_NONE); // no blending during copy
 		SDL_BlitSurface(line.get(), nullptr, destination.get(), &rect);
 	}
 	return destination;
