@@ -1,118 +1,82 @@
 #include "MSXCPU.hh"
+#include "MSXCPUInterface.hh"
 #include "MSXMotherBoard.hh"
 #include "Debugger.hh"
 #include "Scheduler.hh"
-#include "SimpleDebuggable.hh"
-#include "BooleanSetting.hh"
 #include "IntegerSetting.hh"
-#include "TclCallback.hh"
 #include "CPUCore.hh"
 #include "Z80.hh"
 #include "R800.hh"
-#include "InfoTopic.hh"
 #include "TclObject.hh"
+#include "outer.hh"
+#include "ranges.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
-#include "memory.hh"
 #include <cassert>
+#include <memory>
 
 using std::string;
 using std::vector;
 
 namespace openmsx {
 
-class TimeInfoTopic final : public InfoTopic
-{
-public:
-	TimeInfoTopic(InfoCommand& machineInfoCommand,
-	              MSXCPU& msxcpu);
-	void execute(array_ref<TclObject> tokens,
-	             TclObject& result) const override;
-	string help (const vector<string>& tokens) const override;
-private:
-	MSXCPU& msxcpu;
-};
-
-class CPUFreqInfoTopic final : public InfoTopic
-{
-public:
-	CPUFreqInfoTopic(InfoCommand& machineInfoCommand,
-	                 const string& name, CPUClock& clock);
-	void execute(array_ref<TclObject> tokens,
-	             TclObject& result) const override;
-	string help (const vector<string>& tokens) const override;
-private:
-	CPUClock& clock;
-};
-
-class MSXCPUDebuggable final : public SimpleDebuggable
-{
-public:
-	MSXCPUDebuggable(MSXMotherBoard& motherboard, MSXCPU& cpu);
-	byte read(unsigned address) override;
-	void write(unsigned address, byte value) override;
-private:
-	MSXCPU& cpu;
-};
-
-
 MSXCPU::MSXCPU(MSXMotherBoard& motherboard_)
 	: motherboard(motherboard_)
-	, traceSetting(make_unique<BooleanSetting>(
+	, traceSetting(
 		motherboard.getCommandController(), "cputrace",
-		"CPU tracing on/off", false, Setting::DONT_SAVE))
-	, diHaltCallback(make_unique<TclCallback>(
+		"CPU tracing on/off", false, Setting::DONT_SAVE)
+	, diHaltCallback(
 		motherboard.getCommandController(), "di_halt_callback",
-		"Tcl proc called when the CPU executed a DI/HALT sequence"))
-	, z80(make_unique<CPUCore<Z80TYPE>>(
-		motherboard, "z80", *traceSetting,
-		*diHaltCallback, EmuTime::zero))
+		"Tcl proc called when the CPU executed a DI/HALT sequence")
+	, z80(std::make_unique<CPUCore<Z80TYPE>>(
+		motherboard, "z80", traceSetting,
+		diHaltCallback, EmuTime::zero()))
 	, r800(motherboard.isTurboR()
-		? make_unique<CPUCore<R800TYPE>>(
-			motherboard, "r800", *traceSetting,
-			*diHaltCallback, EmuTime::zero)
+		? std::make_unique<CPUCore<R800TYPE>>(
+			motherboard, "r800", traceSetting,
+			diHaltCallback, EmuTime::zero())
 		: nullptr)
-	, timeInfo(make_unique<TimeInfoTopic>(
-		motherboard.getMachineInfoCommand(), *this))
-	, z80FreqInfo(make_unique<CPUFreqInfoTopic>(
-		motherboard.getMachineInfoCommand(), "z80_freq", *z80))
+	, timeInfo(motherboard.getMachineInfoCommand())
+	, z80FreqInfo(motherboard.getMachineInfoCommand(), "z80_freq", *z80)
 	, r800FreqInfo(r800
-		? make_unique<CPUFreqInfoTopic>(
+		? std::make_unique<CPUFreqInfoTopic>(
 			motherboard.getMachineInfoCommand(), "r800_freq", *r800)
 		: nullptr)
-	, debuggable(make_unique<MSXCPUDebuggable>(motherboard_, *this))
-	, reference(EmuTime::zero)
+	, debuggable(motherboard_)
+	, reference(EmuTime::zero())
 {
 	z80Active = true; // setActiveCPU(CPU_Z80);
 	newZ80Active = z80Active;
 
 	motherboard.getDebugger().setCPU(this);
 	motherboard.getScheduler().setCPU(this);
-	traceSetting->attach(*this);
+	traceSetting.attach(*this);
 
-	z80->freqLocked->attach(*this);
-	z80->freqValue->attach(*this);
+	z80->freqLocked.attach(*this);
+	z80->freqValue.attach(*this);
 	if (r800) {
-		r800->freqLocked->attach(*this);
-		r800->freqValue->attach(*this);
+		r800->freqLocked.attach(*this);
+		r800->freqValue.attach(*this);
 	}
+	invalidateMemCacheSlot();
 }
 
 MSXCPU::~MSXCPU()
 {
-	traceSetting->detach(*this);
-	z80->freqLocked->detach(*this);
-	z80->freqValue->detach(*this);
+	traceSetting.detach(*this);
+	z80->freqLocked.detach(*this);
+	z80->freqValue.detach(*this);
 	if (r800) {
-		r800->freqLocked->detach(*this);
-		r800->freqValue->detach(*this);
+		r800->freqLocked.detach(*this);
+		r800->freqValue.detach(*this);
 	}
 	motherboard.getScheduler().setCPU(nullptr);
 	motherboard.getDebugger() .setCPU(nullptr);
 }
 
-void MSXCPU::setInterface(MSXCPUInterface* interface)
+void MSXCPU::setInterface(MSXCPUInterface* interface_)
 {
+	interface = interface_;
 	          z80 ->setInterface(interface);
 	if (r800) r800->setInterface(interface);
 }
@@ -122,23 +86,16 @@ void MSXCPU::doReset(EmuTime::param time)
 	          z80 ->doReset(time);
 	if (r800) r800->doReset(time);
 
+	invalidateAllSlotsRWCache(0x0000, 0x10000);
+
 	reference = time;
 }
 
 void MSXCPU::setActiveCPU(CPUType cpu)
 {
+	if (cpu == CPU_R800) assert(r800);
+
 	bool tmp = cpu == CPU_Z80;
-	switch (cpu) {
-		case CPU_Z80:
-			PRT_DEBUG("Active CPU: Z80");
-			break;
-		case CPU_R800:
-			PRT_DEBUG("Active CPU: R800");
-			assert(r800);
-			break;
-		default:
-			UNREACHABLE;
-	}
 	if (tmp != z80Active) {
 		exitCPULoopSync();
 		newZ80Active = tmp;
@@ -158,7 +115,13 @@ void MSXCPU::execute(bool fastForward)
 		z80Active = newZ80Active;
 		z80Active ? z80 ->warp(time)
 		          : r800->warp(time);
-		invalidateMemCache(0x0000, 0x10000);
+		// copy Z80/R800 cache lines
+		auto zCache = z80 ->getCacheLines();
+		auto rCache = r800->getCacheLines();
+		auto from = z80Active ? rCache : zCache;
+		auto to   = z80Active ? zCache : rCache;
+		std::copy_n(from.first,  CacheLine::NUM, to.first );
+		std::copy_n(from.second, CacheLine::NUM, to.second);
 	}
 	z80Active ? z80 ->execute(fastForward)
 	          : r800->execute(fastForward);
@@ -187,16 +150,140 @@ void MSXCPU::setNextSyncPoint(EmuTime::param time)
 	          : r800->setNextSyncPoint(time);
 }
 
+void MSXCPU::invalidateMemCacheSlot()
+{
+	ranges::fill(slots, 0);
+
+	// nullptr: means not a valid entry and not yet attempted to fill this entry
+	for (int i = 0; i < 16; ++i) {
+		ranges::fill(slotReadLines[i], nullptr);
+		ranges::fill(slotWriteLines[i], nullptr);
+	}
+}
+
 void MSXCPU::updateVisiblePage(byte page, byte primarySlot, byte secondarySlot)
 {
-	invalidateMemCache(page * 0x4000, 0x4000);
+	byte from = slots[page];
+	byte to = 4 * primarySlot + secondarySlot;
+	slots[page] = to;
+
+	auto [cpuReadLines, cpuWriteLines] = z80Active ? z80->getCacheLines() : r800->getCacheLines();
+
+	unsigned first = page * (0x4000 / CacheLine::SIZE);
+	unsigned num = 0x4000 / CacheLine::SIZE;
+	std::copy_n(&cpuReadLines      [first], num, &slotReadLines [from][first]);
+	std::copy_n(&slotReadLines [to][first], num, &cpuReadLines        [first]);
+	std::copy_n(&cpuWriteLines     [first], num, &slotWriteLines[from][first]);
+	std::copy_n(&slotWriteLines[to][first], num, &cpuWriteLines       [first]);
+
 	if (r800) r800->updateVisiblePage(page, primarySlot, secondarySlot);
 }
 
-void MSXCPU::invalidateMemCache(word start, unsigned size)
+void MSXCPU::invalidateAllSlotsRWCache(word start, unsigned size)
 {
-	z80Active ? z80 ->invalidateMemCache(start, size)
-	          : r800->invalidateMemCache(start, size);
+	if (interface) interface->tick(CacheLineCounters::InvalidateAllSlots);
+	auto [cpuReadLines, cpuWriteLines] = z80Active ? z80->getCacheLines() : r800->getCacheLines();
+
+	unsigned first = start / CacheLine::SIZE;
+	unsigned num = (size + CacheLine::SIZE - 1) / CacheLine::SIZE;
+	std::fill_n(cpuReadLines  + first, num, nullptr); // nullptr: means not a valid entry and not
+	std::fill_n(cpuWriteLines + first, num, nullptr); //   yet attempted to fill this entry
+
+	for (int i = 0; i < 16; ++i) {
+		std::fill_n(slotReadLines[i] + first, num, nullptr);
+		std::fill_n(slotWriteLines[i] + first, num, nullptr);
+	}
+}
+
+template<bool READ, bool WRITE, bool SUB_START>
+void MSXCPU::setRWCache(unsigned start, unsigned size, const byte* rData, byte* wData, int ps, int ss,
+                             const byte* disallowRead, const byte* disallowWrite)
+{
+	if (!SUB_START) {
+		assert(rData == nullptr);
+		assert(wData == nullptr);
+	}
+
+	// aligned on cache lines
+	assert((start & CacheLine::LOW) == 0);
+	assert((size  & CacheLine::LOW) == 0);
+
+	int slot = 4 * ps + ss;
+	unsigned page = start >> 14;
+	assert(((start + size - 1) >> 14) == page); // all in same page
+	if (SUB_START && READ)  rData -= start;
+	if (SUB_START && WRITE) wData -= start;
+
+	// select between 'active' or 'shadow' cache lines
+	const byte** readLines;
+	      byte** writeLines;
+	if (slot == slots[page]) {
+		auto cache = z80Active ? z80->getCacheLines() : r800->getCacheLines();
+		readLines  = cache.first;
+		writeLines = cache.second;
+	} else {
+		readLines  = slotReadLines [slot];
+		writeLines = slotWriteLines[slot];
+	}
+
+	unsigned first = start / CacheLine::SIZE;
+	readLines     += first;
+	writeLines    += first;
+	disallowRead  += first;
+	disallowWrite += first;
+	unsigned num = size / CacheLine::SIZE;
+
+	static const auto NON_CACHEABLE = reinterpret_cast<byte*>(1);
+	for (unsigned i = 0; i < num; ++i) {
+		if (READ)  readLines [i] = disallowRead [i] ? NON_CACHEABLE : rData;
+		if (WRITE) writeLines[i] = disallowWrite[i] ? NON_CACHEABLE : wData;
+	}
+}
+
+static void extendForAlignment(unsigned& start, unsigned& size)
+{
+	static constexpr unsigned MASK = ~(CacheLine::LOW); // not CacheLine::HIGH because 0x0000ff00 != 0xffffff00
+
+	auto end = start + size;
+	start &= MASK;                            // round down to cacheline
+	end = (end + CacheLine::SIZE - 1) & MASK; // round up to cacheline
+	size = end - start;
+}
+
+void MSXCPU::invalidateRWCache(unsigned start, unsigned size, int ps, int ss,
+                               const byte* disallowRead, const byte* disallowWrite)
+{
+	// unaligned [start, start+size) is OK for invalidate, then simply invalidate a little more.
+	extendForAlignment(start, size);
+	setRWCache<true, true, false>(start, size, nullptr, nullptr, ps, ss, disallowRead, disallowWrite);
+}
+void MSXCPU::invalidateRCache(unsigned start, unsigned size, int ps, int ss,
+                              const byte* disallowRead)
+{
+	extendForAlignment(start, size);
+	setRWCache<true, false, false>(start, size, nullptr, nullptr, ps, ss, disallowRead, nullptr);
+}
+void MSXCPU::invalidateWCache(unsigned start, unsigned size, int ps, int ss,
+                              const byte* disallowWrite)
+{
+	extendForAlignment(start, size);
+	setRWCache<false, true, false>(start, size, nullptr, nullptr, ps, ss, nullptr, disallowWrite);
+}
+
+void MSXCPU::fillRWCache(unsigned start, unsigned size, const byte* rData, byte* wData, int ps, int ss,
+                         const byte* disallowRead, const byte* disallowWrite)
+{
+	setRWCache<true, true, true>(start, size, rData, wData, ps, ss, disallowRead, disallowWrite);
+}
+void MSXCPU::fillRCache(unsigned start, unsigned size, const byte* rData, int ps, int ss,
+                        const byte* disallowRead)
+{
+	setRWCache<true, false, true>(start, size, rData, nullptr, ps, ss, disallowRead, nullptr);
+}
+void MSXCPU::fillWCache(unsigned start, unsigned size, byte* wData, int ps, int ss,
+                        const byte* disallowWrite)
+{
+	setRWCache<false, true, true>(start, size, nullptr, wData, ps, ss, nullptr, disallowWrite);
 }
 
 void MSXCPU::raiseIRQ()
@@ -237,17 +324,16 @@ void MSXCPU::wait(EmuTime::param time)
 	          : r800->wait(time);
 }
 
-void MSXCPU::waitCycles(unsigned cycles)
+EmuTime MSXCPU::waitCyclesZ80(EmuTime::param time, unsigned cycles)
 {
-	z80Active ? z80 ->waitCycles(cycles)
-	          : r800->waitCycles(cycles);
+	return z80Active ? z80 ->waitCycles(time, cycles)
+	                 : time;
 }
 
-void MSXCPU::waitCyclesR800(unsigned cycles)
+EmuTime MSXCPU::waitCyclesR800(EmuTime::param time, unsigned cycles)
 {
-	if (isR800Active()) {
-		r800->waitCycles(cycles);
-	}
+	return z80Active ? time
+	                 : r800->waitCycles(time, cycles);
 }
 
 CPURegs& MSXCPU::getRegisters()
@@ -269,7 +355,7 @@ void MSXCPU::update(const Setting& setting)
 // Command
 
 void MSXCPU::disasmCommand(
-	Interpreter& interp, array_ref<TclObject> tokens,
+	Interpreter& interp, span<const TclObject> tokens,
 	TclObject& result) const
 {
 	z80Active ? z80 ->disasmCommand(interp, tokens, result)
@@ -290,21 +376,20 @@ void MSXCPU::setPaused(bool paused)
 
 // class TimeInfoTopic
 
-TimeInfoTopic::TimeInfoTopic(InfoCommand& machineInfoCommand,
-                             MSXCPU& msxcpu_)
+MSXCPU::TimeInfoTopic::TimeInfoTopic(InfoCommand& machineInfoCommand)
 	: InfoTopic(machineInfoCommand, "time")
-	, msxcpu(msxcpu_)
 {
 }
 
-void TimeInfoTopic::execute(array_ref<TclObject> /*tokens*/,
-                            TclObject& result) const
+void MSXCPU::TimeInfoTopic::execute(
+	span<const TclObject> /*tokens*/, TclObject& result) const
 {
-	EmuDuration dur = msxcpu.getCurrentTime() - msxcpu.reference;
-	result.setDouble(dur.toDouble());
+	auto& cpu = OUTER(MSXCPU, timeInfo);
+	EmuDuration dur = cpu.getCurrentTime() - cpu.reference;
+	result = dur.toDouble();
 }
 
-string TimeInfoTopic::help(const vector<string>& /*tokens*/) const
+string MSXCPU::TimeInfoTopic::help(const vector<string>& /*tokens*/) const
 {
 	return "Prints the time in seconds that the MSX is powered on\n";
 }
@@ -312,20 +397,21 @@ string TimeInfoTopic::help(const vector<string>& /*tokens*/) const
 
 // class CPUFreqInfoTopic
 
-CPUFreqInfoTopic::CPUFreqInfoTopic(InfoCommand& machineInfoCommand,
-                                   const string& name, CPUClock& clock_)
-	: InfoTopic(machineInfoCommand, name)
+MSXCPU::CPUFreqInfoTopic::CPUFreqInfoTopic(
+		InfoCommand& machineInfoCommand,
+		const string& name_, CPUClock& clock_)
+	: InfoTopic(machineInfoCommand, name_)
 	, clock(clock_)
 {
 }
 
-void CPUFreqInfoTopic::execute(array_ref<TclObject> /*tokens*/,
-                               TclObject& result) const
+void MSXCPU::CPUFreqInfoTopic::execute(
+	span<const TclObject> /*tokens*/, TclObject& result) const
 {
-	result.setInt(clock.getFreq());
+	result = clock.getFreq();
 }
 
-string CPUFreqInfoTopic::help(const vector<string>& /*tokens*/) const
+string MSXCPU::CPUFreqInfoTopic::help(const vector<string>& /*tokens*/) const
 {
 	return "Returns the actual frequency of this CPU.\n"
 	       "This frequency can vary because:\n"
@@ -335,9 +421,9 @@ string CPUFreqInfoTopic::help(const vector<string>& /*tokens*/) const
 }
 
 
-// class MSXCPUDebuggable
+// class Debuggable
 
-static const char* const CPU_REGS_DESC =
+constexpr const char* const CPU_REGS_DESC =
 	"Registers of the active CPU (Z80 or R800).\n"
 	"Each byte in this debuggable represents one 8 bit register:\n"
 	"  0 ->  A      1 ->  F      2 -> B       3 -> C\n"
@@ -352,14 +438,14 @@ static const char* const CPU_REGS_DESC =
 	"this effectively indicates that the CPU could accept an interrupt at\n"
 	"the start of the current instruction.\n";
 
-MSXCPUDebuggable::MSXCPUDebuggable(MSXMotherBoard& motherboard, MSXCPU& cpu_)
-	: SimpleDebuggable(motherboard, "CPU regs", CPU_REGS_DESC, 28)
-	, cpu(cpu_)
+MSXCPU::Debuggable::Debuggable(MSXMotherBoard& motherboard_)
+	: SimpleDebuggable(motherboard_, "CPU regs", CPU_REGS_DESC, 28)
 {
 }
 
-byte MSXCPUDebuggable::read(unsigned address)
+byte MSXCPU::Debuggable::read(unsigned address)
 {
+	auto& cpu = OUTER(MSXCPU, debuggable);
 	const CPURegs& regs = cpu.getRegisters();
 	switch (address) {
 	case  0: return regs.getA();
@@ -391,13 +477,14 @@ byte MSXCPUDebuggable::read(unsigned address)
 	case 26: return regs.getIM();
 	case 27: return 1 *  regs.getIFF1() +
 	                2 *  regs.getIFF2() +
-	                4 * (regs.getIFF1() && !regs.debugGetAfterEI());
+	                4 * (regs.getIFF1() && !regs.prevWasEI());
 	default: UNREACHABLE; return 0;
 	}
 }
 
-void MSXCPUDebuggable::write(unsigned address, byte value)
+void MSXCPU::Debuggable::write(unsigned address, byte value)
 {
+	auto& cpu = OUTER(MSXCPU, debuggable);
 	CPURegs& regs = cpu.getRegisters();
 	switch (address) {
 	case  0: regs.setA(value); break;
@@ -447,8 +534,8 @@ void MSXCPU::serialize(Archive& ar, unsigned version)
 	if (ar.versionAtLeast(version, 2)) {
 		          ar.serialize("z80",  *z80);
 		if (r800) ar.serialize("r800", *r800);
-		ar.serialize("z80Active", z80Active);
-		ar.serialize("newZ80Active", newZ80Active);
+		ar.serialize("z80Active",    z80Active,
+		             "newZ80Active", newZ80Active);
 	} else {
 		// backwards-compatibility
 		assert(ar.isLoader());
@@ -467,6 +554,11 @@ void MSXCPU::serialize(Archive& ar, unsigned version)
 		}
 	}
 	ar.serialize("resetTime", reference);
+
+	if (ar.isLoader()) {
+		invalidateMemCacheSlot();
+		invalidateAllSlotsRWCache(0x0000, 0x10000);
+	}
 }
 INSTANTIATE_SERIALIZE_METHODS(MSXCPU);
 
