@@ -1,6 +1,7 @@
 #include "MSXMultiIODevice.hh"
+#include "MSXException.hh"
 #include "TclObject.hh"
-#include <algorithm>
+#include "stl.hh"
 #include <cassert>
 
 namespace openmsx {
@@ -17,26 +18,24 @@ MSXMultiIODevice::~MSXMultiIODevice()
 
 void MSXMultiIODevice::addDevice(MSXDevice* device)
 {
-	assert(std::count(devices.begin(), devices.end(), device) == 0);
+	if (contains(devices, device)) {
+		throw MSXException(
+			"Overlapping IO-port ranges for \"",
+			device->getName(), "\".");
+	}
 	devices.push_back(device);
 }
 
 void MSXMultiIODevice::removeDevice(MSXDevice* device)
 {
-	assert(std::count(devices.begin(), devices.end(), device) == 1);
-	devices.erase(std::find(devices.begin(), devices.end(), device));
-}
-
-MSXMultiIODevice::Devices& MSXMultiIODevice::getDevices()
-{
-	return devices;
+	devices.erase(rfind_unguarded(devices, device));
 }
 
 std::string MSXMultiIODevice::getName() const
 {
 	TclObject list;
 	getNameList(list);
-	return list.getString().str();
+	return std::string(list.getString());
 }
 void MSXMultiIODevice::getNameList(TclObject& result) const
 {
@@ -50,28 +49,30 @@ void MSXMultiIODevice::getNameList(TclObject& result) const
 
 byte MSXMultiIODevice::readIO(word port, EmuTime::param time)
 {
-	// conflict: return the result from the first device, call readIO()
-	//           also on all other devices, but discard result
-	assert(!devices.empty());
-	auto it = devices.begin();
-	byte result = (*it)->readIO(port, time);
-	for (++it; it != devices.end(); ++it) {
-		(*it)->readIO(port, time);
+	// conflict: In practice, pull down seems to win over pull up,
+	//           so a logical AND over the read values most accurately
+	//           resembles what real hardware does.
+	byte result = 0xFF;
+	for (auto& dev : devices) {
+		result &= dev->readIO(port, time);
 	}
 	return result;
 }
 
 byte MSXMultiIODevice::peekIO(word port, EmuTime::param time) const
 {
-	// conflict: just peek first device
-	assert(!devices.empty());
-	return devices.front()->peekIO(port, time);
+	// conflict: Handle this in the same way as readIO.
+	byte result = 0xFF;
+	for (auto& dev : devices) {
+		result &= dev->peekIO(port, time);
+	}
+	return result;
 }
 
 void MSXMultiIODevice::writeIO(word port, byte value, EmuTime::param time)
 {
-	for (auto& d : devices) {
-		d->writeIO(port, value, time);
+	for (auto& dev : devices) {
+		dev->writeIO(port, value, time);
 	}
 }
 
