@@ -3,9 +3,10 @@
 #include "MSXCPUInterface.hh"
 #include "TclObject.hh"
 #include "likely.hh"
+#include "ranges.hh"
+#include "stl.hh"
 #include "unreachable.hh"
-#include "xrange.hh"
-#include <algorithm>
+#include "view.hh"
 #include <cassert>
 
 namespace openmsx {
@@ -50,47 +51,33 @@ static bool overlap(unsigned start1, unsigned size1,
 
 bool MSXMultiMemDevice::canAdd(int base, int size)
 {
-	for (auto i : xrange(ranges.size() - 1)) {
-		if (overlap(base, size, ranges[i].base, ranges[i].size)) {
-			return false;
-		}
-	}
-	return true;
+	return ranges::none_of(view::drop_back(ranges, 1), [&](auto& rn) {
+		return overlap(base, size, rn.base, rn.size);
+	});
 }
 
 void MSXMultiMemDevice::add(MSXDevice& device, int base, int size)
 {
 	assert(canAdd(base, size));
-	ranges.insert(ranges.begin(), Range(base, size, device));
+	ranges.insert(begin(ranges), Range(base, size, device));
 }
 
 void MSXMultiMemDevice::remove(MSXDevice& device, int base, int size)
 {
-	auto it = find(ranges.begin(), ranges.end(),
-	               Range(base, size, device));
-	assert(it != ranges.end());
-	ranges.erase(it);
-}
-
-bool MSXMultiMemDevice::empty() const
-{
-	return ranges.size() == 1;
+	ranges.erase(rfind_unguarded(ranges, Range(base, size, device)));
 }
 
 std::vector<MSXDevice*> MSXMultiMemDevice::getDevices() const
 {
-	std::vector<MSXDevice*> result;
-	for (auto i : xrange(ranges.size() - 1)) {
-		result.push_back(ranges[i].device);
-	}
-	return result;
+	return to_vector(view::transform(view::drop_back(ranges, 1),
+	                                 [](auto& rn) { return rn.device; }));
 }
 
 std::string MSXMultiMemDevice::getName() const
 {
 	TclObject list;
 	getNameList(list);
-	return list.getString().str();
+	return std::string(list.getString());
 }
 void MSXMultiMemDevice::getNameList(TclObject& result) const
 {
@@ -109,7 +96,7 @@ const MSXMultiMemDevice::Range& MSXMultiMemDevice::searchRange(unsigned address)
 			return r;
 		}
 	}
-	UNREACHABLE;
+	UNREACHABLE; return ranges.back();
 }
 
 MSXDevice* MSXMultiMemDevice::searchDevice(unsigned address) const
@@ -144,7 +131,7 @@ const byte* MSXMultiMemDevice::getReadCacheLine(word start) const
 		// cacheline. This can't be cached.
 		return nullptr;
 	}
-	return searchDevice(start)->getReadCacheLine(start);
+	return range.device->getReadCacheLine(start);
 }
 
 byte* MSXMultiMemDevice::getWriteCacheLine(word start) const
@@ -154,7 +141,7 @@ byte* MSXMultiMemDevice::getWriteCacheLine(word start) const
 	if (unlikely(((range.base + range.size) & CacheLine::HIGH) == start)) {
 		return nullptr;
 	}
-	return searchDevice(start)->getWriteCacheLine(start);
+	return range.device->getWriteCacheLine(start);
 }
 
 } // namespace openmsx
