@@ -1,7 +1,6 @@
 #include "VDPIODelay.hh"
 #include "MSXCPU.hh"
 #include "MSXCPUInterface.hh"
-#include "DeviceConfig.hh"
 #include "DummyDevice.hh"
 #include "serialize.hh"
 #include <cassert>
@@ -11,7 +10,7 @@ namespace openmsx {
 VDPIODelay::VDPIODelay(const DeviceConfig& config, MSXCPUInterface& cpuInterface)
 	: MSXDevice(config)
 	, cpu(getCPU()) // used frequently, so cache it
-	, lastTime(EmuTime::zero)
+	, lastTime(EmuTime::zero())
 {
 	for (int port = 0x098; port <= 0x9B; ++port) {
 		getInDevicePtr (port) = &cpuInterface.getDummyDevice();
@@ -56,7 +55,6 @@ void VDPIODelay::writeIO(word port, byte value, EmuTime::param time)
 
 void VDPIODelay::delay(EmuTime::param time)
 {
-	cpu.waitCycles(1);
 	if (cpu.isR800Active()) {
 		// Number of cycles based on measurements on real HW.
 		// See doc/turbor-vdp-io-timing.ods for details.
