@@ -1,21 +1,20 @@
 #include "ProbeBreakPoint.hh"
 #include "Probe.hh"
 #include "Debugger.hh"
+#include "MSXMotherBoard.hh"
+#include "Reactor.hh"
 #include "TclObject.hh"
 
 namespace openmsx {
 
-unsigned ProbeBreakPoint::lastId = 0;
-
 ProbeBreakPoint::ProbeBreakPoint(
-		GlobalCliComm& cliComm,
-		Interpreter& interp,
-		TclObject command,
-		TclObject condition,
+		TclObject command_,
+		TclObject condition_,
 		Debugger& debugger_,
 		ProbeBase& probe_,
+		bool once_,
 		unsigned newId /*= -1*/)
-	: BreakPointBase(cliComm, interp, command, condition)
+	: BreakPointBase(command_, condition_, once_)
 	, debugger(debugger_)
 	, probe(probe_)
 	, id((newId == unsigned(-1)) ? ++lastId : newId)
@@ -30,7 +29,13 @@ ProbeBreakPoint::~ProbeBreakPoint()
 
 void ProbeBreakPoint::update(const ProbeBase& /*subject*/)
 {
-	checkAndExecute();
+	auto& reactor = debugger.getMotherBoard().getReactor();
+	auto& cliComm = reactor.getGlobalCliComm();
+	auto& interp  = reactor.getInterpreter();
+	checkAndExecute(cliComm, interp);
+	if (onlyOnce()) {
+		debugger.removeProbeBreakPoint(*this);
+	}
 }
 
 void ProbeBreakPoint::subjectDeleted(const ProbeBase& /*subject*/)
