@@ -5,44 +5,40 @@
 #include "Interpreter.hh"
 #include "InputEventGenerator.hh"
 #include "Thread.hh"
-#include "KeyRange.hh"
+#include "ranges.hh"
 #include "stl.hh"
-#include <algorithm>
+#include "view.hh"
 #include <cassert>
+#include <chrono>
 
-using std::pair;
 using std::string;
 
 namespace openmsx {
 
 EventDistributor::EventDistributor(Reactor& reactor_)
 	: reactor(reactor_)
-	, sem(1)
 {
 }
 
 void EventDistributor::registerEventListener(
 		EventType type, EventListener& listener, Priority priority)
 {
-	ScopedLock lock(sem);
+	std::lock_guard<std::mutex> lock(mutex);
 	auto& priorityMap = listeners[type];
-	for (auto* l : values(priorityMap)) {
-		// a listener may only be registered once for each type
-		assert(l != &listener); (void)l;
-	}
+	// a listener may only be registered once for each type
+	assert(!contains(view::values(priorityMap), &listener));
 	// insert at highest position that keeps listeners sorted on priority
-	auto it = upper_bound(begin(priorityMap), end(priorityMap), priority,
-	                      LessTupleElement<0>());
+	auto it = ranges::upper_bound(priorityMap, priority, LessTupleElement<0>());
 	priorityMap.insert(it, {priority, &listener});
 }
 
 void EventDistributor::unregisterEventListener(
 		EventType type, EventListener& listener)
 {
-	ScopedLock lock(sem);
+	std::lock_guard<std::mutex> lock(mutex);
 	auto& priorityMap = listeners[type];
-	priorityMap.erase(find_if_unguarded(priorityMap,
-		[&](PriorityMap::value_type v) { return v.second == &listener; }));
+	priorityMap.erase(rfind_if_unguarded(priorityMap,
+		[&](auto& v) { return v.second == &listener; }));
 }
 
 void EventDistributor::distributeEvent(const EventPtr& event)
@@ -54,7 +50,7 @@ void EventDistributor::distributeEvent(const EventPtr& event)
 	// TODO: Is it useful to test for 0 listeners or should we just always
 	//       queue the event?
 	assert(event);
-	ScopedLock lock(sem);
+	std::unique_lock<std::mutex> lock(mutex);
 	if (!listeners[event->getType()].empty()) {
 		scheduledEvents.push_back(event);
 		// must release lock, otherwise there's a deadlock:
@@ -62,18 +58,15 @@ void EventDistributor::distributeEvent(const EventPtr& event)
 		//             EventDistributor::unregisterEventListener()
 		//   thread 2: EventDistributor::distributeEvent()
 		//             Reactor::enterMainLoop()
-		cond.signalAll();
-		lock.release();
+		condition.notify_all();
+		lock.unlock();
 		reactor.enterMainLoop();
 	}
 }
 
 bool EventDistributor::isRegistered(EventType type, EventListener* listener) const
 {
-	for (auto* l : values(listeners[type])) {
-		if (l == listener) return true;
-	}
-	return false;
+	return contains(view::values(listeners[type]), listener);
 }
 
 void EventDistributor::deliverEvents()
@@ -84,7 +77,7 @@ void EventDistributor::deliverEvents()
 	reactor.getInterpreter().poll();
 	reactor.getRTScheduler().execute();
 
-	ScopedLock lock(sem);
+	std::unique_lock<std::mutex> lock(mutex);
 	// It's possible that executing an event triggers scheduling of another
 	// event. We also want to execute those secondary events. That's why
 	// we have this while loop here.
@@ -99,29 +92,30 @@ void EventDistributor::deliverEvents()
 		for (auto& event : eventsCopy) {
 			auto type = event->getType();
 			auto priorityMapCopy = listeners[type];
-			sem.up();
-			unsigned blockPriority = unsigned(-1); // allow all
-			for (auto& p : priorityMapCopy) {
+			lock.unlock();
+			auto blockPriority = unsigned(-1); // allow all
+			for (const auto& [priority, listener] : priorityMapCopy) {
 				// It's possible delivery to one of the previous
 				// Listeners unregistered the current Listener.
-				if (!isRegistered(type, p.second)) continue;
+				if (!isRegistered(type, listener)) continue;
 
-				unsigned currentPriority = p.first;
-				if (currentPriority >= blockPriority) break;
+				if (priority >= blockPriority) break;
 
-				if (unsigned block = p.second->signalEvent(event)) {
-					assert(block > currentPriority);
+				if (unsigned block = listener->signalEvent(event)) {
+					assert(block > priority);
 					blockPriority = block;
 				}
 			}
-			sem.down();
+			lock.lock();
 		}
 	}
 }
 
 bool EventDistributor::sleep(unsigned us)
 {
-	return cond.waitTimeout(us);
+	std::chrono::microseconds duration(us);
+	std::unique_lock<std::mutex> lock(cvMutex);
+	return condition.wait_for(lock, duration) == std::cv_status::timeout;
 }
 
 } // namespace openmsx
