@@ -1,16 +1,18 @@
-#include "Command.hh"
 #include "InputEventGenerator.hh"
 #include "EventDistributor.hh"
 #include "InputEvents.hh"
-#include "BooleanSetting.hh"
 #include "IntegerSetting.hh"
 #include "GlobalSettings.hh"
 #include "Keys.hh"
+#include "FileOperations.hh"
 #include "checked_cast.hh"
-#include "memory.hh"
+#include "one_of.hh"
+#include "outer.hh"
 #include "unreachable.hh"
+#include "utf8_unchecked.hh"
 #include "build-info.hh"
 #include <cassert>
+#include <memory>
 
 using std::string;
 using std::vector;
@@ -18,40 +20,21 @@ using std::make_shared;
 
 namespace openmsx {
 
-class EscapeGrabCmd final : public Command
-{
-public:
-	EscapeGrabCmd(CommandController& commandController,
-		      InputEventGenerator& inputEventGenerator);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
-	string help(const vector<string>& tokens) const override;
-private:
-	InputEventGenerator& inputEventGenerator;
-};
-
-
-bool InputEventGenerator::androidButtonA = false;
-bool InputEventGenerator::androidButtonB = false;
-
 InputEventGenerator::InputEventGenerator(CommandController& commandController,
                                          EventDistributor& eventDistributor_,
                                          GlobalSettings& globalSettings_)
 	: eventDistributor(eventDistributor_)
 	, globalSettings(globalSettings_)
-	, grabInput(make_unique<BooleanSetting>(
+	, grabInput(
 		commandController, "grabinput",
 		"This setting controls if openMSX takes over mouse and keyboard input",
-		false, Setting::DONT_SAVE))
-	, escapeGrabCmd(make_unique<EscapeGrabCmd>(commandController, *this))
+		false, Setting::DONT_SAVE)
+	, escapeGrabCmd(commandController)
 	, escapeGrabState(ESCAPE_GRAB_WAIT_CMD)
-	, keyRepeat(false)
 {
-	setGrabInput(grabInput->getBoolean());
-	grabInput->attach(*this);
+	setGrabInput(grabInput.getBoolean());
 	eventDistributor.registerEventListener(OPENMSX_FOCUS_EVENT, *this);
 
-	reinit();
-
 	osdControlButtonsState = unsigned(~0); // 0 is pressed, 1 is released
 
 #ifndef SDL_JOYSTICK_DISABLED
@@ -62,13 +45,6 @@ InputEventGenerator::InputEventGenerator(CommandController& commandController,
 InputEventGenerator::~InputEventGenerator()
 {
 	eventDistributor.unregisterEventListener(OPENMSX_FOCUS_EVENT, *this);
-	grabInput->detach(*this);
-}
-
-void InputEventGenerator::reinit()
-{
-	SDL_EnableUNICODE(1);
-	setKeyRepeat(keyRepeat);
 }
 
 void InputEventGenerator::wait()
@@ -85,47 +61,67 @@ void InputEventGenerator::wait()
 
 void InputEventGenerator::poll()
 {
-	SDL_Event event;
-	while (SDL_PollEvent(&event) == 1) {
-#ifdef DEBUG
-		string t;
-		switch (event.type) {
-			case SDL_ACTIVEEVENT:     t = "SDL_ACTIVEEVENT";     break;
-			case SDL_KEYDOWN:         t = "SDL_KEYDOWN";         break;
-			case SDL_KEYUP:           t = "SDL_KEYUP";           break;
-			case SDL_MOUSEMOTION:     t = "SDL_MOUSEMOTION";     break;
-			case SDL_MOUSEBUTTONDOWN: t = "SDL_MOUSEBUTTONDOWN"; break;
-			case SDL_MOUSEBUTTONUP:   t = "SDL_MOUSEBUTTONUP";   break;
-			case SDL_JOYAXISMOTION:   t = "SDL_JOYAXISMOTION";   break;
-			case SDL_JOYBALLMOTION:   t = "SDL_JOYBALLMOTION";   break;
-			case SDL_JOYHATMOTION:    t = "SDL_JOYHATMOTION";    break;
-			case SDL_JOYBUTTONDOWN:   t = "SDL_JOYBUTTONDOWN";   break;
-			case SDL_JOYBUTTONUP:     t = "SDL_JOYBUTTONUP";     break;
-			case SDL_QUIT:            t = "SDL_QUIT";            break;
-			case SDL_SYSWMEVENT:      t = "SDL_SYSWMEVENT";      break;
-			case SDL_VIDEORESIZE:     t = "SDL_VIDEORESIZE";     break;
-			case SDL_VIDEOEXPOSE:     t = "SDL_VIDEOEXPOSE";     break;
-			case SDL_USEREVENT:       t = "SDL_USEREVENT";       break;
-			default:                  t = "UNKNOWN";             break;
+	// Heuristic to emulate the old SDL1 behavior:
+	//
+	// SDL1 had a unicode field on each KEYDOWN event. In SDL2 that
+	// information is moved to the (new) SDL_TEXTINPUT events.
+	//
+	// Though our MSX keyboard emulation code needs to relate KEYDOWN
+	// events with the associated unicode. We try to mimic this by the
+	// following heuristic:
+	//   When two successive events in a single batch (so, the same
+	//   invocation of poll()) are KEYDOWN followed by TEXTINPUT, then copy
+	//   the unicode (of the first character) of the TEXT event to the
+	//   KEYDOWN event.
+	// Implementing this requires a lookahead of 1 event. So the code below
+	// deals with a 'current' and a 'previous' event, and keeps track of
+	// whether the previous event is still pending (not yet processed).
+	//
+	// In a previous version we also added the constraint that these two
+	// consecutive events must have the same timestamp, but that had mixed
+	// results:
+	// - on Linux it worked fine
+	// - on Windows the timestamps sometimes did not match
+	// - on Mac the timestamps mostly did not match
+	// So we removed this constraint.
+	//
+	// We also split SDL_TEXTINPUT events into (possibly) multiple KEYDOWN
+	// events because a single event type makes it easier to handle higher
+	// priority listeners that can block the event for lower priority
+	// listener (console > hotkey > msx).
+
+	SDL_Event event1, event2;
+	auto* prev = &event1;
+	auto* curr = &event2;
+	bool pending = false;
+
+	while (SDL_PollEvent(curr)) {
+		if (pending) {
+			pending = false;
+			if ((prev->type == SDL_KEYDOWN) && (curr->type == SDL_TEXTINPUT)) {
+				const char* utf8 = curr->text.text;
+				auto unicode = utf8::unchecked::next(utf8);
+				handleKeyDown(prev->key, unicode);
+				if (unicode) { // possibly there are more characters
+					handleText(utf8);
+				}
+				continue;
+			} else {
+				handle(*prev);
+			}
 		}
-		PRT_DEBUG("SDL event received, type: " + t);
-#endif
-		handle(event);
+		if (curr->type == SDL_KEYDOWN) {
+			pending = true;
+			std::swap(curr, prev);
+		} else {
+			handle(*curr);
+		}
+	}
+	if (pending) {
+		handle(*prev);
 	}
 }
 
-void InputEventGenerator::setKeyRepeat(bool enable)
-{
-	keyRepeat = enable;
-	if (keyRepeat) {
-		SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,
-		                    SDL_DEFAULT_REPEAT_INTERVAL);
-	} else {
-		SDL_EnableKeyRepeat(0, 0);
-	}
-}
-
-
 void InputEventGenerator::setNewOsdControlButtonState(
 		unsigned newState, const EventPtr& origEvent)
 {
@@ -148,7 +144,7 @@ void InputEventGenerator::setNewOsdControlButtonState(
 }
 
 void InputEventGenerator::triggerOsdControlEventsFromJoystickAxisMotion(
-	unsigned axis, short value, const EventPtr& origEvent)
+	unsigned axis, int value, const EventPtr& origEvent)
 {
 	unsigned neg_button, pos_button;
 	switch (axis) {
@@ -217,27 +213,72 @@ void InputEventGenerator::triggerOsdControlEventsFromJoystickButtonEvent(
 }
 
 void InputEventGenerator::triggerOsdControlEventsFromKeyEvent(
-	Keys::KeyCode keyCode, bool up, const EventPtr& origEvent)
+	Keys::KeyCode keyCode, bool up, bool repeat, const EventPtr& origEvent)
 {
-	keyCode = static_cast<Keys::KeyCode>(keyCode & Keys::K_MASK);
-	if (keyCode == Keys::K_LEFT) {
-		osdControlChangeButton(up, 1 << OsdControlEvent::LEFT_BUTTON,
-		                       origEvent);
-	} else if (keyCode == Keys::K_RIGHT) {
-		osdControlChangeButton(up, 1 << OsdControlEvent::RIGHT_BUTTON,
-		                       origEvent);
-	} else if (keyCode == Keys::K_UP) {
-		osdControlChangeButton(up, 1 << OsdControlEvent::UP_BUTTON,
-		                       origEvent);
-	} else if (keyCode == Keys::K_DOWN) {
-		osdControlChangeButton(up, 1 << OsdControlEvent::DOWN_BUTTON,
-		                       origEvent);
-	} else if (keyCode == Keys::K_SPACE || keyCode == Keys::K_RETURN) {
-		osdControlChangeButton(up, 1 << OsdControlEvent::A_BUTTON,
-		                       origEvent);
-	} else if (keyCode == Keys::K_ESCAPE) {
-		osdControlChangeButton(up, 1 << OsdControlEvent::B_BUTTON,
-		                       origEvent);
+	unsigned buttonMask = [&] {
+		switch (static_cast<Keys::KeyCode>(keyCode & Keys::K_MASK)) {
+		case Keys::K_LEFT:   return 1 << OsdControlEvent::LEFT_BUTTON;
+		case Keys::K_RIGHT:  return 1 << OsdControlEvent::RIGHT_BUTTON;
+		case Keys::K_UP:     return 1 << OsdControlEvent::UP_BUTTON;
+		case Keys::K_DOWN:   return 1 << OsdControlEvent::DOWN_BUTTON;
+		case Keys::K_SPACE:  return 1 << OsdControlEvent::A_BUTTON;
+		case Keys::K_RETURN: return 1 << OsdControlEvent::A_BUTTON;
+		case Keys::K_ESCAPE: return 1 << OsdControlEvent::B_BUTTON;
+		default: return 0;
+		}
+	}();
+	if (buttonMask) {
+		if (repeat) {
+			osdControlChangeButton(!up, buttonMask, origEvent);
+		}
+		osdControlChangeButton(up, buttonMask, origEvent);
+	}
+}
+
+static Uint16 normalizeModifier(SDL_Keycode sym, Uint16 mod)
+{
+	// Apparently modifier-keys also have the corresponding
+	// modifier attribute set. See here for a discussion:
+	//     https://github.com/openMSX/openMSX/issues/1202
+	// As a solution, on pure modifier keys, we now clear the
+	// modifier attributes.
+	return (sym == one_of(SDLK_LCTRL, SDLK_LSHIFT, SDLK_LALT, SDLK_LGUI,
+	                      SDLK_RCTRL, SDLK_RSHIFT, SDLK_RALT, SDLK_RGUI,
+	                      SDLK_MODE))
+		? 0
+		: mod;
+}
+
+void InputEventGenerator::handleKeyDown(const SDL_KeyboardEvent& key, uint32_t unicode)
+{
+	EventPtr event;
+	/*if (PLATFORM_ANDROID && evt.key.keysym.sym == SDLK_WORLD_93) {
+		event = make_shared<JoystickButtonDownEvent>(0, 0);
+		triggerOsdControlEventsFromJoystickButtonEvent(
+			0, false, event);
+		androidButtonA = true;
+	} else if (PLATFORM_ANDROID && evt.key.keysym.sym == SDLK_WORLD_94) {
+		event = make_shared<JoystickButtonDownEvent>(0, 1);
+		triggerOsdControlEventsFromJoystickButtonEvent(
+			1, false, event);
+		androidButtonB = true;
+	} else*/ {
+		auto mod = normalizeModifier(key.keysym.sym, key.keysym.mod);
+		auto [keyCode, scanCode] = Keys::getCodes(
+			key.keysym.sym, mod, key.keysym.scancode, false);
+		event = make_shared<KeyDownEvent>(keyCode, scanCode, unicode);
+		triggerOsdControlEventsFromKeyEvent(keyCode, false, key.repeat, event);
+	}
+	eventDistributor.distributeEvent(event);
+}
+
+void InputEventGenerator::handleText(const char* utf8)
+{
+	while (true) {
+		auto unicode = utf8::unchecked::next(utf8);
+		if (unicode == 0) return;
+		eventDistributor.distributeEvent(
+			make_shared<KeyDownEvent>(Keys::K_NONE, unicode));
 	}
 }
 
@@ -252,7 +293,8 @@ void InputEventGenerator::handle(const SDL_Event& evt)
 		// will be mapped to keys SDLK_WORLD_93 and 94 and are
 		// interpeted here as joystick buttons (respectively button 0
 		// and 1).
-		if (PLATFORM_ANDROID && evt.key.keysym.sym == SDLK_WORLD_93) {
+		// TODO Android code should be rewritten for SDL2
+		/*if (PLATFORM_ANDROID && evt.key.keysym.sym == SDLK_WORLD_93) {
 			event = make_shared<JoystickButtonUpEvent>(0, 0);
 			triggerOsdControlEventsFromJoystickButtonEvent(
 				0, true, event);
@@ -262,34 +304,17 @@ void InputEventGenerator::handle(const SDL_Event& evt)
 			triggerOsdControlEventsFromJoystickButtonEvent(
 				1, true, event);
 			androidButtonB = false;
-		} else {
-			auto keyCode = Keys::getCode(
-				evt.key.keysym.sym, evt.key.keysym.mod,
-				evt.key.keysym.scancode, true);
-			event = make_shared<KeyUpEvent>(
-				keyCode, evt.key.keysym.unicode);
-			triggerOsdControlEventsFromKeyEvent(keyCode, true, event);
+		} else*/ {
+			auto mod = normalizeModifier(evt.key.keysym.sym, evt.key.keysym.mod);
+			auto [keyCode, scanCode] = Keys::getCodes(
+				evt.key.keysym.sym, mod, evt.key.keysym.scancode, true);
+			event = make_shared<KeyUpEvent>(keyCode, scanCode);
+			bool repeat = false;
+			triggerOsdControlEventsFromKeyEvent(keyCode, true, repeat, event);
 		}
 		break;
 	case SDL_KEYDOWN:
-		if (PLATFORM_ANDROID && evt.key.keysym.sym == SDLK_WORLD_93) {
-			event = make_shared<JoystickButtonDownEvent>(0, 0);
-			triggerOsdControlEventsFromJoystickButtonEvent(
-				0, false, event);
-			androidButtonA = true;
-		} else if (PLATFORM_ANDROID && evt.key.keysym.sym == SDLK_WORLD_94) {
-			event = make_shared<JoystickButtonDownEvent>(0, 1);
-			triggerOsdControlEventsFromJoystickButtonEvent(
-				1, false, event);
-			androidButtonB = true;
-		} else {
-			auto keyCode = Keys::getCode(
-				evt.key.keysym.sym, evt.key.keysym.mod,
-				evt.key.keysym.scancode, false);
-			event = make_shared<KeyDownEvent>(
-				keyCode, evt.key.keysym.unicode);
-			triggerOsdControlEventsFromKeyEvent(keyCode, false, event);
-		}
+		handleKeyDown(evt.key, 0);
 		break;
 
 	case SDL_MOUSEBUTTONUP:
@@ -298,6 +323,17 @@ void InputEventGenerator::handle(const SDL_Event& evt)
 	case SDL_MOUSEBUTTONDOWN:
 		event = make_shared<MouseButtonDownEvent>(evt.button.button);
 		break;
+	case SDL_MOUSEWHEEL: {
+		int x = evt.wheel.x;
+		int y = evt.wheel.y;
+		if (evt.wheel.direction == SDL_MOUSEWHEEL_FLIPPED)
+		{
+			x = -x;
+			y = -y;
+		}
+		event = make_shared<MouseWheelEvent>(x, y);
+		break;
+	}
 	case SDL_MOUSEMOTION:
 		event = make_shared<MouseMotionEvent>(
 			evt.motion.xrel, evt.motion.yrel,
@@ -334,16 +370,33 @@ void InputEventGenerator::handle(const SDL_Event& evt)
 		triggerOsdControlEventsFromJoystickHat(evt.jhat.value, event);
 		break;
 
-	case SDL_ACTIVEEVENT:
-		event = make_shared<FocusEvent>(evt.active.gain != 0);
+	case SDL_TEXTINPUT:
+		handleText(evt.text.text);
 		break;
 
-	case SDL_VIDEORESIZE:
-		event = make_shared<ResizeEvent>(evt.resize.w, evt.resize.h);
+	case SDL_WINDOWEVENT:
+		switch (evt.window.event) {
+		case SDL_WINDOWEVENT_FOCUS_GAINED:
+			event = make_shared<FocusEvent>(true);
+			break;
+		case SDL_WINDOWEVENT_FOCUS_LOST:
+			event = make_shared<FocusEvent>(false);
+			break;
+		case SDL_WINDOWEVENT_RESIZED:
+			event = make_shared<ResizeEvent>(
+				evt.window.data1, evt.window.data2);
+			break;
+		case SDL_WINDOWEVENT_EXPOSED:
+			event = make_shared<SimpleEvent>(OPENMSX_EXPOSE_EVENT);
+			break;
+		default:
+			break;
+		}
 		break;
 
-	case SDL_VIDEOEXPOSE:
-		event = make_shared<SimpleEvent>(OPENMSX_EXPOSE_EVENT);
+	case SDL_DROPFILE:
+		event = make_shared<FileDropEvent>(FileOperations::getConventionalPath(evt.drop.file));
+		SDL_free(evt.drop.file);
 		break;
 
 	case SDL_QUIT:
@@ -354,11 +407,11 @@ void InputEventGenerator::handle(const SDL_Event& evt)
 		break;
 	}
 
-#ifdef DEBUG
+#if 0
 	if (event) {
-		PRT_DEBUG("SDL event converted to: " + event->toString());
+		std::cerr << "SDL event converted to: " << event->toString() << '\n';
 	} else {
-		PRT_DEBUG("SDL event was of unknown type, not converted to an openMSX event");
+		std::cerr << "SDL event was of unknown type, not converted to an openMSX event\n";
 	}
 #endif
 
@@ -366,12 +419,10 @@ void InputEventGenerator::handle(const SDL_Event& evt)
 }
 
 
-void InputEventGenerator::update(const Setting& setting)
+void InputEventGenerator::updateGrab(bool grab)
 {
-	(void)setting;
-	assert(&setting == grabInput.get());
 	escapeGrabState = ESCAPE_GRAB_WAIT_CMD;
-	setGrabInput(grabInput->getBoolean());
+	setGrabInput(grab);
 }
 
 int InputEventGenerator::signalEvent(const std::shared_ptr<const Event>& event)
@@ -382,12 +433,12 @@ int InputEventGenerator::signalEvent(const std::shared_ptr<const Event>& event)
 			// nothing
 			break;
 		case ESCAPE_GRAB_WAIT_LOST:
-			if (focusEvent.getGain() == false) {
+			if (!focusEvent.getGain()) {
 				escapeGrabState = ESCAPE_GRAB_WAIT_GAIN;
 			}
 			break;
 		case ESCAPE_GRAB_WAIT_GAIN:
-			if (focusEvent.getGain() == true) {
+			if (focusEvent.getGain()) {
 				escapeGrabState = ESCAPE_GRAB_WAIT_CMD;
 			}
 			setGrabInput(true);
@@ -400,12 +451,12 @@ int InputEventGenerator::signalEvent(const std::shared_ptr<const Event>& event)
 
 void InputEventGenerator::setGrabInput(bool grab)
 {
-	// Note that this setting is also changed in VisibleSurface constructor
-	// because for Mac we want to enable it in fullscreen.
-	// It's not worth it to get that exactly right here, because here
-	// we don't have easy access to renderer settings and it may only
-	// go wrong if you explicitly change grab input at full screen (on Mac)
-	SDL_WM_GrabInput(grab ? SDL_GRAB_ON : SDL_GRAB_OFF);
+	SDL_SetRelativeMouseMode(grab ? SDL_TRUE : SDL_FALSE);
+
+	// TODO is this still the correct place in SDL2
+	// TODO get the SDL_window
+	//SDL_Window* window = ...;
+	//SDL_SetWindowGrab(window, grab ? SDL_TRUE : SDL_FALSE);
 }
 
 
@@ -436,24 +487,26 @@ bool InputEventGenerator::joystickGetButton(SDL_Joystick* joystick, int button)
 
 
 // class EscapeGrabCmd
-EscapeGrabCmd::EscapeGrabCmd(CommandController& commandController,
-                             InputEventGenerator& inputEventGenerator_)
-	: Command(commandController, "escape_grab")
-	, inputEventGenerator(inputEventGenerator_)
+
+InputEventGenerator::EscapeGrabCmd::EscapeGrabCmd(
+		CommandController& commandController_)
+	: Command(commandController_, "escape_grab")
 {
 }
 
-void EscapeGrabCmd::execute(array_ref<TclObject> /*tokens*/,
-		            TclObject& /*result*/)
+void InputEventGenerator::EscapeGrabCmd::execute(
+	span<const TclObject> /*tokens*/, TclObject& /*result*/)
 {
-	if (inputEventGenerator.grabInput->getBoolean()) {
+	auto& inputEventGenerator = OUTER(InputEventGenerator, escapeGrabCmd);
+	if (inputEventGenerator.grabInput.getBoolean()) {
 		inputEventGenerator.escapeGrabState =
 			InputEventGenerator::ESCAPE_GRAB_WAIT_LOST;
 		inputEventGenerator.setGrabInput(false);
 	}
 }
 
-string EscapeGrabCmd::help(const vector<string>& /*tokens*/) const
+string InputEventGenerator::EscapeGrabCmd::help(
+	const vector<string>& /*tokens*/) const
 {
 	return "Temporarily release input grab.";
 }
