@@ -10,22 +10,21 @@ MSXCliComm::MSXCliComm(MSXMotherBoard& motherBoard_, GlobalCliComm& cliComm_)
 {
 }
 
-void MSXCliComm::log(LogLevel level, string_ref message)
+void MSXCliComm::log(LogLevel level, std::string_view message)
 {
 	cliComm.log(level, message);
 }
 
-void MSXCliComm::update(UpdateType type, string_ref name, string_ref value)
+void MSXCliComm::update(UpdateType type, std::string_view name, std::string_view value)
 {
 	assert(type < NUM_UPDATES);
-	auto it = prevValues[type].find(name);
-	if (it != end(prevValues[type])) {
-		if (it->second == value) {
+	if (auto v = lookup(prevValues[type], name)) {
+		if (*v == value) {
 			return;
 		}
-		it->second = value.str();
+		*v = std::string(value);
 	} else {
-		prevValues[type][name] = value.str();
+		prevValues[type].emplace_noDuplicateCheck(name, value);
 	}
 	cliComm.updateHelper(type, motherBoard.getMachineID(), name, value);
 }
