@@ -1,6 +1,7 @@
 #include "MessageCommand.hh"
 #include "CommandException.hh"
 #include "CliComm.hh"
+#include "TclObject.hh"
 #include "xrange.hh"
 
 namespace openmsx {
@@ -10,7 +11,7 @@ MessageCommand::MessageCommand(CommandController& controller)
 {
 }
 
-static CliComm::LogLevel getLevel(const std::string& level)
+static CliComm::LogLevel getLevel(std::string_view level)
 {
 	auto levels = CliComm::getLevelStrings();
 	for (auto i : xrange(levels.size())) {
@@ -18,24 +19,22 @@ static CliComm::LogLevel getLevel(const std::string& level)
 			return static_cast<CliComm::LogLevel>(i);
 		}
 	}
-	throw CommandException("Unknown level string: " + level);
+	throw CommandException("Unknown level string: ", level);
 }
 
-std::string MessageCommand::execute(const std::vector<std::string>& tokens)
+void MessageCommand::execute(span<const TclObject> tokens, TclObject& /*result*/)
 {
+	checkNumArgs(tokens, Between{2, 3}, "string ?level?");
 	CliComm& cliComm = getCliComm();
 	CliComm::LogLevel level = CliComm::INFO;
 	switch (tokens.size()) {
 	case 3:
-		level = getLevel(tokens[2]);
-		// fall-through
+		level = getLevel(tokens[2].getString());
+		[[fallthrough]];
 	case 2:
-		cliComm.log(level, tokens[1]);
+		cliComm.log(level, tokens[1].getString());
 		break;
-	default:
-		throw SyntaxError();
 	}
-	return "";
 }
 
 std::string MessageCommand::help(const std::vector<std::string>& /*tokens*/) const
