@@ -1,38 +1,35 @@
 #include "TclCallbackMessages.hh"
 #include "GlobalCliComm.hh"
-#include "TclCallback.hh"
-#include "memory.hh"
-
-using std::string;
 
 namespace openmsx {
 
 TclCallbackMessages::TclCallbackMessages(GlobalCliComm& cliComm_,
                                          CommandController& controller)
 	: cliComm(cliComm_)
-	, messageCallback(make_unique<TclCallback>(
+	, messageCallback(
 		controller, "message_callback",
 		"Tcl proc called when a new message is available",
 		false, // don't print callback err on cliComm (would cause infinite loop)
-		false)) // don't save setting
+		false) // don't save setting
 {
-	cliComm.addListener(this);
+	cliComm.addListener(std::unique_ptr<CliListener>(this)); // wrap in unique_ptr
 }
 
 TclCallbackMessages::~TclCallbackMessages()
 {
-	cliComm.removeListener(this);
+	std::unique_ptr<CliListener> ptr = cliComm.removeListener(*this);
+	ptr.release();
 }
 
-void TclCallbackMessages::log(CliComm::LogLevel level, string_ref message)
+void TclCallbackMessages::log(CliComm::LogLevel level, std::string_view message)
 {
 	auto levelStr = CliComm::getLevelStrings();
-	messageCallback->execute(message, levelStr[level]);
+	messageCallback.execute(message, levelStr[level]);
 }
 
 void TclCallbackMessages::update(
-	CliComm::UpdateType /*type*/, string_ref /*machine*/,
-	string_ref /*name*/, string_ref /*value*/)
+	CliComm::UpdateType /*type*/, std::string_view /*machine*/,
+	std::string_view /*name*/, std::string_view /*value*/)
 {
 	// ignore
 }
