@@ -1,5 +1,6 @@
 #include "DSKDiskImage.hh"
 #include "File.hh"
+#include "FilePool.hh"
 
 namespace openmsx {
 
@@ -11,17 +12,13 @@ DSKDiskImage::DSKDiskImage(const Filename& fileName)
 }
 
 DSKDiskImage::DSKDiskImage(const Filename& fileName,
-                           const std::shared_ptr<File>& file_)
+                           std::shared_ptr<File> file_)
 	: SectorBasedDisk(fileName)
-	, file(file_)
+	, file(std::move(file_))
 {
 	setNbSectors(file->getSize() / sizeof(SectorBuffer));
 }
 
-DSKDiskImage::~DSKDiskImage()
-{
-}
-
 void DSKDiskImage::readSectorImpl(size_t sector, SectorBuffer& buf)
 {
 	file->seek(sector * sizeof(buf));
@@ -39,12 +36,12 @@ bool DSKDiskImage::isWriteProtectedImpl() const
 	return file->isReadOnly();
 }
 
-Sha1Sum DSKDiskImage::getSha1Sum()
+Sha1Sum DSKDiskImage::getSha1SumImpl(FilePool& filePool)
 {
 	if (hasPatches()) {
-		return SectorAccessibleDisk::getSha1Sum();
+		return SectorAccessibleDisk::getSha1SumImpl(filePool);
 	}
-	return file->getSha1Sum();
+	return filePool.getSha1Sum(*file);
 }
 
 } // namespace openmsx
