@@ -7,31 +7,34 @@
 #include "FileException.hh"
 #include "ReadDir.hh"
 #include "StringOp.hh"
-#include <algorithm>
+#include "one_of.hh"
+#include "ranges.hh"
+#include "stl.hh"
 #include <cassert>
 #include <cstring>
 #include <vector>
 
-using std::map;
 using std::string;
 using std::vector;
 
 namespace openmsx {
 
-static const unsigned SECTOR_SIZE = sizeof(SectorBuffer);
-static const unsigned SECTORS_PER_DIR = 7;
-static const unsigned NUM_FATS = 2;
-static const unsigned SECTORS_PER_CLUSTER = 2;
-static const unsigned FIRST_FAT_SECTOR = 1;
-static const unsigned DIR_ENTRIES_PER_SECTOR =
+constexpr unsigned SECTOR_SIZE = sizeof(SectorBuffer);
+constexpr unsigned SECTORS_PER_DIR = 7;
+constexpr unsigned NUM_FATS = 2;
+constexpr unsigned NUM_TRACKS = 80;
+constexpr unsigned SECTORS_PER_CLUSTER = 2;
+constexpr unsigned SECTORS_PER_TRACK = 9;
+constexpr unsigned FIRST_FAT_SECTOR = 1;
+constexpr unsigned DIR_ENTRIES_PER_SECTOR =
 	SECTOR_SIZE / sizeof(MSXDirEntry);
 
 // First valid regular cluster number.
-static const unsigned FIRST_CLUSTER = 2;
+constexpr unsigned FIRST_CLUSTER = 2;
 
-static const unsigned FREE_FAT = 0x000;
-static const unsigned BAD_FAT  = 0xFF7;
-static const unsigned EOF_FAT  = 0xFFF; // actually 0xFF8-0xFFF
+constexpr unsigned FREE_FAT = 0x000;
+constexpr unsigned BAD_FAT  = 0xFF7;
+constexpr unsigned EOF_FAT  = 0xFFF; // actually 0xFF8-0xFFF
 
 
 // Transform BAD_FAT (0xFF7) and EOF_FAT-range (0xFF8-0xFFF)
@@ -41,11 +44,11 @@ static unsigned normalizeFAT(unsigned cluster)
 	return (cluster < BAD_FAT) ? cluster : EOF_FAT;
 }
 
-unsigned DirAsDSK::readFATHelper(const SectorBuffer* fat, unsigned cluster) const
+unsigned DirAsDSK::readFATHelper(const SectorBuffer* fatBuf, unsigned cluster) const
 {
 	assert(FIRST_CLUSTER <= cluster);
 	assert(cluster < maxCluster);
-	auto* buf = fat[0].raw;
+	auto* buf = fatBuf[0].raw;
 	auto* p = &buf[(cluster * 3) / 2];
 	unsigned result = (cluster & 1)
 	                ? (p[0] >> 4) + (p[1] << 4)
@@ -53,11 +56,11 @@ unsigned DirAsDSK::readFATHelper(const SectorBuffer* fat, unsigned cluster) cons
 	return normalizeFAT(result);
 }
 
-void DirAsDSK::writeFATHelper(SectorBuffer* fat, unsigned cluster, unsigned val) const
+void DirAsDSK::writeFATHelper(SectorBuffer* fatBuf, unsigned cluster, unsigned val) const
 {
 	assert(FIRST_CLUSTER <= cluster);
 	assert(cluster < maxCluster);
-	auto* buf = fat[0].raw;
+	auto* buf = fatBuf[0].raw;
 	auto* p = &buf[(cluster * 3) / 2];
 	if (cluster & 1) {
 		p[0] = (p[0] & 0x0F) + (val << 4);
@@ -180,7 +183,14 @@ unsigned DirAsDSK::nextMsxDirSector(unsigned sector)
 bool DirAsDSK::checkMSXFileExists(
 	const string& msxFilename, unsigned msxDirSector)
 {
+	vector<bool> visited(nofSectors, false);
 	do {
+		if (visited[msxDirSector]) {
+			// cycle detected, invalid disk, but don't crash on it
+			return false;
+		}
+		visited[msxDirSector] = true;
+
 		for (unsigned idx = 0; idx < DIR_ENTRIES_PER_SECTOR; ++idx) {
 			DirIndex dirIndex(msxDirSector, idx);
 			if (memcmp(msxDir(dirIndex).filename,
@@ -199,12 +209,12 @@ bool DirAsDSK::checkMSXFileExists(
 // is not mapped in the virtual disk.
 DirAsDSK::DirIndex DirAsDSK::findHostFileInDSK(const string& hostName)
 {
-	for (auto& p : mapDirs) {
-		if (p.second.hostName == hostName) {
-			return p.first;
+	for (const auto& [dirIdx, mapDir] : mapDirs) {
+		if (mapDir.hostName == hostName) {
+			return dirIdx;
 		}
 	}
-	return DirIndex(unsigned(-1), unsigned(-1));
+	return {unsigned(-1), unsigned(-1)};
 }
 
 // Check if a host file is already mapped in the virtual disk.
@@ -217,17 +227,16 @@ bool DirAsDSK::checkFileUsedInDSK(const string& hostName)
 static string hostToMsxName(string hostName)
 {
 	// Create an MSX filename 8.3 format. TODO use vfat-like abbreviation
-	transform(hostName.begin(), hostName.end(), hostName.begin(),
-			[](char a) { return (a == ' ') ? '_' : ::toupper(a); });
-
-	string_ref file, ext;
-	StringOp::splitOnLast(hostName, '.', file, ext);
+	transform_in_place(hostName, [](char a) {
+		return (a == ' ') ? '_' : ::toupper(a);
+	});
+	auto [file, ext] = StringOp::splitOnLast(hostName, '.');
 	if (file.empty()) std::swap(file, ext);
 
 	string result(8 + 3, ' ');
-	memcpy(&*result.begin() + 0, file.data(), std::min<size_t>(8, file.size()));
-	memcpy(&*result.begin() + 8, ext .data(), std::min<size_t>(3, ext .size()));
-	replace(result.begin(), result.end(), '.', '_');
+	memcpy(result.data() + 0, file.data(), std::min<size_t>(8, file.size()));
+	memcpy(result.data() + 8, ext .data(), std::min<size_t>(3, ext .size()));
+	ranges::replace(result, '.', '_');
 	return result;
 }
 
@@ -235,12 +244,12 @@ static string msxToHostName(const char* msxName)
 {
 	string result;
 	for (unsigned i = 0; (i < 8) && (msxName[i] != ' '); ++i) {
-		result += tolower(msxName[i]);
+		result += char(tolower(msxName[i]));
 	}
 	if (msxName[8] != ' ') {
 		result += '.';
 		for (unsigned i = 8; (i < (8 + 3)) && (msxName[i] != ' '); ++i) {
-			result += tolower(msxName[i]);
+			result += char(tolower(msxName[i]));
 		}
 	}
 	return result;
@@ -255,9 +264,9 @@ DirAsDSK::DirAsDSK(DiskChanger& diskChanger_, CliComm& cliComm_,
 	, cliComm(cliComm_)
 	, hostDir(hostDir_.getResolved() + '/')
 	, syncMode(syncMode_)
-	, lastAccess(EmuTime::zero)
-	, nofSectors(diskChanger_.isDoubleSidedDrive() ? 1440 : 720)
-	, nofSectorsPerFat(diskChanger_.isDoubleSidedDrive() ? 3 : 2)
+	, lastAccess(EmuTime::zero())
+	, nofSectors((diskChanger_.isDoubleSidedDrive() ? 2 : 1) * SECTORS_PER_TRACK * NUM_TRACKS)
+	, nofSectorsPerFat((((3 * nofSectors) / (2 * SECTORS_PER_CLUSTER)) + SECTOR_SIZE - 1) / SECTOR_SIZE)
 	, firstSector2ndFAT(FIRST_FAT_SECTOR + nofSectorsPerFat)
 	, firstDirSector(FIRST_FAT_SECTOR + NUM_FATS * nofSectorsPerFat)
 	, firstDataSector(firstDirSector + SECTORS_PER_DIR)
@@ -269,19 +278,31 @@ DirAsDSK::DirAsDSK(DiskChanger& diskChanger_, CliComm& cliComm_,
 	}
 
 	// First create structure for the virtual disk.
+	byte numSides = diskChanger_.isDoubleSidedDrive() ? 2 : 1;
 	setNbSectors(nofSectors);
-	setSectorsPerTrack(9);
-	setNbSides(diskChanger_.isDoubleSidedDrive() ? 2 : 1);
+	setSectorsPerTrack(SECTORS_PER_TRACK);
+	setNbSides(numSides);
 
 	// Initially the whole disk is filled with 0xE5 (at least on Philips
 	// NMS8250).
 	memset(sectors.data(), 0xE5, sizeof(SectorBuffer) * nofSectors);
 
-	// Use selected bootsector.
-	auto& bootSector = bootSectorType == BOOTSECTOR_DOS1
-	                 ? BootBlocks::dos1BootBlock
-	                 : BootBlocks::dos2BootBlock;
-	memcpy(&sectors[0], &bootSector, sizeof(bootSector));
+	// Use selected bootsector, fill-in values.
+	byte mediaDescriptor = (numSides == 2) ? 0xF9 : 0xF8;
+	const auto& protoBootSector = bootSectorType == BOOTSECTOR_DOS1
+		? BootBlocks::dos1BootBlock
+		: BootBlocks::dos2BootBlock;
+	memcpy(&sectors[0], &protoBootSector, sizeof(protoBootSector));
+	auto& bootSector = sectors[0].bootSector;
+	bootSector.bpSector     = SECTOR_SIZE;
+	bootSector.spCluster    = SECTORS_PER_CLUSTER;
+	bootSector.nrFats       = NUM_FATS;
+	bootSector.dirEntries   = SECTORS_PER_DIR * (SECTOR_SIZE / sizeof(MSXDirEntry));
+	bootSector.nrSectors    = nofSectors;
+	bootSector.descriptor   = mediaDescriptor;
+	bootSector.sectorsFat   = nofSectorsPerFat;
+	bootSector.sectorsTrack = SECTORS_PER_TRACK;
+	bootSector.nrSides      = numSides;
 
 	// Clear FAT1 + FAT2.
 	memset(fat(), 0, SECTOR_SIZE * nofSectorsPerFat * NUM_FATS);
@@ -289,7 +310,6 @@ DirAsDSK::DirAsDSK(DiskChanger& diskChanger_, CliComm& cliComm_,
 	//  'cluster 0' contains the media descriptor
 	//  'cluster 1' is marked as EOF_FAT
 	//  So cluster 2 is the first usable cluster number
-	byte mediaDescriptor = diskChanger_.isDoubleSidedDrive() ? 0xF9 : 0xF8;
 	fat ()->raw[0] = mediaDescriptor; fat ()->raw[1] = 0xFF; fat ()->raw[2] = 0xFF;
 	fat2()->raw[0] = mediaDescriptor; fat2()->raw[1] = 0xFF; fat2()->raw[2] = 0xFF;
 
@@ -373,15 +393,15 @@ void DirAsDSK::syncWithHost()
 	checkModifiedHostFiles();
 
 	// Last add new host files (this can only consume virtual disk space).
-	addNewHostFiles("", firstDirSector);
+	addNewHostFiles({}, firstDirSector);
 }
 
 void DirAsDSK::checkDeletedHostFiles()
 {
 	// This handles both host files and directories.
 	auto copy = mapDirs;
-	for (auto& p : copy) {
-		if (mapDirs.find(p.first) == mapDirs.end()) {
+	for (const auto& [dirIdx, mapDir] : copy) {
+		if (!mapDirs.contains(dirIdx)) {
 			// While iterating over (the copy of) mapDirs we delete
 			// entries of mapDirs (when we delete files only the
 			// current entry is deleted, when we delete
@@ -392,13 +412,11 @@ void DirAsDSK::checkDeletedHostFiles()
 			// mapDirs. Ignore it.
 			continue;
 		}
-		const DirIndex& dirIndex = p.first;
-		MapDir& mapDir = p.second;
 		string fullHostName = hostDir + mapDir.hostName;
-		bool isMSXDirectory = (msxDir(dirIndex).attrib &
+		bool isMSXDirectory = (msxDir(dirIdx).attrib &
 		                       MSXDirEntry::ATT_DIRECTORY) != 0;
 		FileOperations::Stat fst;
-		if ((!FileOperations::getStat(fullHostName, fst) != 0) ||
+		if ((!FileOperations::getStat(fullHostName, fst)) ||
 		    (FileOperations::isDirectory(fst) != isMSXDirectory)) {
 			// TODO also check access permission
 			// Error stat-ing file, or directory/file type is not
@@ -406,7 +424,7 @@ void DirAsDSK::checkDeletedHostFiles()
 			// has been removed and a host directory with the same
 			// name has been created). In both cases delete the msx
 			// entry (if needed it will be recreated soon).
-			deleteMSXFile(dirIndex);
+			deleteMSXFile(dirIdx);
 		}
 	}
 }
@@ -417,7 +435,7 @@ void DirAsDSK::deleteMSXFile(DirIndex dirIndex)
 	mapDirs.erase(dirIndex);
 
 	char c = msxDir(dirIndex).filename[0];
-	if (c == 0 || c == char(0xE5)) {
+	if (c == one_of(0, char(0xE5))) {
 		// Directory entry not in use, don't need to do anything.
 		return;
 	}
@@ -449,7 +467,14 @@ void DirAsDSK::deleteMSXFile(DirIndex dirIndex)
 
 void DirAsDSK::deleteMSXFilesInDir(unsigned msxDirSector)
 {
+	vector<bool> visited(nofSectors, false);
 	do {
+		if (visited[msxDirSector]) {
+			// cycle detected, invalid disk, but don't crash on it
+			return;
+		}
+		visited[msxDirSector] = true;
+
 		for (unsigned idx = 0; idx < DIR_ENTRIES_PER_SECTOR; ++idx) {
 			deleteMSXFile(DirIndex(msxDirSector, idx));
 		}
@@ -470,18 +495,16 @@ void DirAsDSK::freeFATChain(unsigned cluster)
 void DirAsDSK::checkModifiedHostFiles()
 {
 	auto copy = mapDirs;
-	for (auto& p : copy) {
-		if (mapDirs.find(p.first) == mapDirs.end()) {
+	for (const auto& [dirIdx, mapDir] : copy) {
+		if (!mapDirs.contains(dirIdx)) {
 			// See comment in checkDeletedHostFiles().
 			continue;
 		}
-		const DirIndex& dirIndex = p.first;
-		MapDir& mapDir = p.second;
 		string fullHostName = hostDir + mapDir.hostName;
-		bool isMSXDirectory = (msxDir(dirIndex).attrib &
+		bool isMSXDirectory = (msxDir(dirIdx).attrib &
 		                       MSXDirEntry::ATT_DIRECTORY) != 0;
 		FileOperations::Stat fst;
-		if ((!FileOperations::getStat(fullHostName, fst) == 0) &&
+		if (FileOperations::getStat(fullHostName, fst) &&
 		    (FileOperations::isDirectory(fst) == isMSXDirectory)) {
 			// Detect changes in host file.
 			// Heuristic: we use filesize and modification time to detect
@@ -495,12 +518,12 @@ void DirAsDSK::checkModifiedHostFiles()
 			if (!isMSXDirectory &&
 			    ((mapDir.mtime    != fst.st_mtime) ||
 			     (mapDir.filesize != size_t(fst.st_size)))) {
-				importHostFile(dirIndex, fst);
+				importHostFile(dirIdx, fst);
 			}
 		} else {
 			// Only very rarely happens (because checkDeletedHostFiles()
 			// checked this just recently).
-			deleteMSXFile(dirIndex);
+			deleteMSXFile(dirIdx);
 		}
 	}
 }
@@ -508,7 +531,7 @@ void DirAsDSK::checkModifiedHostFiles()
 void DirAsDSK::importHostFile(DirIndex dirIndex, FileOperations::Stat& fst)
 {
 	assert(!(msxDir(dirIndex).attrib & MSXDirEntry::ATT_DIRECTORY));
-	assert(mapDirs.find(dirIndex) != mapDirs.end());
+	assert(mapDirs.contains(dirIndex));
 
 	// Set _msx_ modification time.
 	setMSXTimeStamp(dirIndex, fst);
@@ -588,14 +611,14 @@ void DirAsDSK::importHostFile(DirIndex dirIndex, FileOperations::Stat& fst)
 			}
 		}
 		if (remainingSize != 0) {
-			cliComm.printWarning("Virtual diskimage full: " +
-			                     mapDir.hostName + " truncated.");
+			cliComm.printWarning("Virtual diskimage full: ",
+			                     mapDir.hostName, " truncated.");
 		}
 	} catch (FileException& e) {
 		// Error opening or reading host file.
-		cliComm.printWarning("Error reading host file: " +
-				mapDir.hostName + ": " + e.getMessage() +
-				" Truncated file on MSX disk.");
+		cliComm.printWarning("Error reading host file: ",
+		                     mapDir.hostName, ": ", e.getMessage(),
+		                     " Truncated file on MSX disk.");
 	}
 
 	// In all cases (no error / image full / host read error) we need to
@@ -639,21 +662,20 @@ void DirAsDSK::setMSXTimeStamp(DirIndex dirIndex, FileOperations::Stat& fst)
 }
 
 // Used to add 'regular' files before 'derived' files. E.g. when editing a file
-// in a host editor, you often get a backup/swap fileis like this:
+// in a host editor, you often get backup/swap files like this:
 //   myfile.txt  myfile.txt~  .myfile.txt.swp
 // Currently the 1st and 2nd are mapped to the same MSX filename. If more
 // host files map to the same MSX file then (currently) one of the two is
 // ignored. Which one is ignored depends on the order in which they are added
 // to the virtual disk. This routine/heuristic tries to add 'regular' files
 // before derived files.
-static int weight(const string& hostName)
+static size_t weight(const string& hostName)
 {
 	// TODO this weight function can most likely be improved
-	int result = 0;
-	string_ref file, ext;
-	StringOp::splitOnLast(hostName, '.', file, ext);
+	size_t result = 0;
+	auto [file, ext] = StringOp::splitOnLast(hostName, '.');
 	// too many '.' characters
-	result += std::count(file.begin(), file.end(), '.') * 100;
+	result += ranges::count(file, '.') * 100;
 	// too long extension
 	result += ext.size() * 10;
 	// too long file
@@ -673,26 +695,27 @@ void DirAsDSK::addNewHostFiles(const string& hostSubDir, unsigned msxDirSector)
 			hostNames.emplace_back(d->d_name);
 		}
 	}
-	sort(hostNames.begin(), hostNames.end(),
+	ranges::sort(hostNames,
 	     [](const string& l, const string& r) { return weight(l) < weight(r); });
 
 	for (auto& hostName : hostNames) {
 		try {
-			string fullHostName = hostDir + hostSubDir + hostName;
+			if (StringOp::startsWith(hostName, '.')) {
+				// skip '.' and '..'
+				// also skip hidden files on unix
+				continue;
+			}
+			string fullHostName = strCat(hostDir, hostSubDir, hostName);
 			FileOperations::Stat fst;
 			if (!FileOperations::getStat(fullHostName, fst)) {
-				throw MSXException("Error accessing " + fullHostName);
+				throw MSXException("Error accessing ", fullHostName);
 			}
 			if (FileOperations::isDirectory(fst)) {
-				if ((hostName == "..") || (hostName == ".")) {
-					continue;
-				}
 				addNewDirectory(hostSubDir, hostName, msxDirSector, fst);
 			} else if (FileOperations::isRegularFile(fst)) {
 				addNewHostFile(hostSubDir, hostName, msxDirSector, fst);
 			} else {
-				throw MSXException("Not a regular file: " +
-				                   fullHostName);
+				throw MSXException("Not a regular file: ", fullHostName);
 			}
 		} catch (MSXException& e) {
 			cliComm.printWarning(e.getMessage());
@@ -760,7 +783,7 @@ void DirAsDSK::addNewDirectory(const string& hostSubDir, const string& hostName,
 	}
 
 	// Recursively process this directory.
-	addNewHostFiles(hostSubDir + hostName + '/', newMsxDirSector);
+	addNewHostFiles(strCat(hostSubDir, hostName, '/'), newMsxDirSector);
 }
 
 void DirAsDSK::addNewHostFile(const string& hostSubDir, const string& hostName,
@@ -774,9 +797,9 @@ void DirAsDSK::addNewHostFile(const string& hostSubDir, const string& hostName,
 	string fullHostName = hostDir + hostPath;
 
 	// TODO check for available free space on disk instead of max free space
-	static const int DISK_SPACE = (nofSectors - firstDataSector) * SECTOR_SIZE;
-	if (fst.st_size > DISK_SPACE) {
-		cliComm.printWarning("File too large: " + fullHostName);
+	int diskSpace = (nofSectors - firstDataSector) * SECTOR_SIZE;
+	if (fst.st_size > diskSpace) {
+		cliComm.printWarning("File too large: ", fullHostName);
 		return;
 	}
 
@@ -797,7 +820,8 @@ DirAsDSK::DirIndex DirAsDSK::fillMSXDirEntry(
 		if (checkMSXFileExists(msxFilename, msxDirSector)) {
 			// TODO: actually should increase vfat abrev if possible!!
 			throw MSXException(
-				"MSX name " + msxToHostName(msxFilename.c_str()) + " already exists");
+				"MSX name ", msxToHostName(msxFilename.c_str()),
+				" already exists");
 		}
 
 		// Fill in hostName / msx filename.
@@ -807,22 +831,28 @@ DirAsDSK::DirIndex DirAsDSK::fillMSXDirEntry(
 		memcpy(msxDir(dirIndex).filename, msxFilename.data(), 8 + 3);
 		return dirIndex;
 	} catch (MSXException& e) {
-		throw MSXException("Couldn't add " + hostPath + ": " +
+		throw MSXException("Couldn't add ", hostPath, ": ",
 		                   e.getMessage());
 	}
 }
 
 DirAsDSK::DirIndex DirAsDSK::getFreeDirEntry(unsigned msxDirSector)
 {
+	vector<bool> visited(nofSectors, false);
 	while (true) {
+		if (visited[msxDirSector]) {
+			// cycle detected, invalid disk, but don't crash on it
+			throw MSXException("cycle in FAT");
+		}
+		visited[msxDirSector] = true;
+
 		for (unsigned idx = 0; idx < DIR_ENTRIES_PER_SECTOR; ++idx) {
 			DirIndex dirIndex(msxDirSector, idx);
 			const char* msxName = msxDir(dirIndex).filename;
-			if ((msxName[0] == char(0x00)) ||
-			    (msxName[0] == char(0xE5))) {
+			if (msxName[0] == one_of(char(0x00), char(0xE5))) {
 				// Found an unused msx entry. There shouldn't
 				// be any hostfile mapped to this entry.
-				assert(mapDirs.find(dirIndex) == mapDirs.end());
+				assert(!mapDirs.contains(dirIndex));
 				return dirIndex;
 			}
 		}
@@ -840,14 +870,14 @@ DirAsDSK::DirIndex DirAsDSK::getFreeDirEntry(unsigned msxDirSector)
 	// Extend sub-directory: allocate and clear a new cluster, add this
 	// cluster in the existing FAT chain.
 	unsigned cluster = sectorToCluster(msxDirSector);
-	unsigned newCluster = getFreeCluster();
+	unsigned newCluster = getFreeCluster(); // throws if disk full
 	unsigned sector = clusterToSector(newCluster);
 	memset(&sectors[sector], 0, SECTORS_PER_CLUSTER * SECTOR_SIZE);
 	writeFAT12(cluster, newCluster);
 	writeFAT12(newCluster, EOF_FAT);
 
 	// First entry in this newly allocated cluster is free. Return it.
-	return DirIndex(sector, 0);
+	return {sector, 0};
 }
 
 void DirAsDSK::writeSectorImpl(size_t sector_, const SectorBuffer& buf)
@@ -886,7 +916,7 @@ void DirAsDSK::writeFATSector(unsigned sector, const SectorBuffer& buf)
 {
 	// Create copy of old FAT (to be able to detect changes).
 	vector<SectorBuffer> oldFAT(nofSectorsPerFat);
-	memcpy(&oldFAT[0], fat(), sizeof(oldFAT));
+	memcpy(&oldFAT[0], fat(), SECTOR_SIZE * nofSectorsPerFat);
 
 	// Update current FAT with new data.
 	memcpy(&sectors[sector], &buf, sizeof(buf));
@@ -917,8 +947,15 @@ void DirAsDSK::exportFileFromFATChange(unsigned cluster, SectorBuffer* oldFAT)
 
 	// Copy this whole chain from FAT1 to FAT2 (so that the loop in
 	// writeFATSector() sees this part is already handled).
+	vector<bool> visited(maxCluster, false);
 	unsigned tmp = startCluster;
 	while ((FIRST_CLUSTER <= tmp) && (tmp < maxCluster)) {
+		if (visited[tmp]) {
+			// detected cycle, don't export file
+			return;
+		}
+		visited[tmp] = true;
+
 		unsigned next = readFAT(tmp);
 		writeFATHelper(oldFAT, tmp, next);
 		tmp = next;
@@ -969,8 +1006,7 @@ template<typename FUNC> bool DirAsDSK::scanMsxDirs(FUNC func, unsigned sector)
 				const MSXDirEntry& entry = msxDir(dirIndex);
 				if (func.onDirEntry(dirIndex, entry)) return true;
 
-				if ((entry.filename[0] == char(0x00)) ||
-				    (entry.filename[0] == char(0xE5)) ||
+				if ((entry.filename[0] == one_of(char(0x00), char(0xE5))) ||
 				    !(entry.attrib & MSXDirEntry::ATT_DIRECTORY)) {
 					// Not a directory.
 					continue;
@@ -984,8 +1020,7 @@ template<typename FUNC> bool DirAsDSK::scanMsxDirs(FUNC func, unsigned sector)
 					continue;
 				}
 				unsigned dir = clusterToSector(cluster);
-				if (find(dirs.begin(), dirs.end(), dir) !=
-				    dirs.end()) {
+				if (contains(dirs, dir)) {
 					// Already found this sector. Except
 					// for the special "." and ".."
 					// entries, loops should not occur in
@@ -1033,7 +1068,7 @@ struct NullScanner {
 // Base class for the IsDirSector and DirEntryForCluster scanner algorithms
 // below. This class remembers the directory entry of the last visited subdir.
 struct DirScanner : NullScanner {
-	DirScanner(DirAsDSK::DirIndex& dirDirIndex_)
+	explicit DirScanner(DirAsDSK::DirIndex& dirDirIndex_)
 		: dirDirIndex(dirDirIndex_)
 	{
 		dirDirIndex = DirAsDSK::DirIndex(0, 0); // represents entry for root dir
@@ -1049,8 +1084,8 @@ struct DirScanner : NullScanner {
 
 // Figure out whether a given sector is part of the msx directory structure.
 struct IsDirSector : DirScanner {
-	IsDirSector(unsigned sector_, DirAsDSK::DirIndex& dirDirIndex)
-		: DirScanner(dirDirIndex)
+	IsDirSector(unsigned sector_, DirAsDSK::DirIndex& dirDirIndex_)
+		: DirScanner(dirDirIndex_)
 		, sector(sector_) {}
 	bool onDirSector(unsigned dirSector) {
 		return sector == dirSector;
@@ -1066,8 +1101,8 @@ bool DirAsDSK::isDirSector(unsigned sector, DirIndex& dirDirIndex)
 struct DirEntryForCluster : DirScanner {
 	DirEntryForCluster(unsigned cluster_,
 	                   DirAsDSK::DirIndex& dirIndex_,
-	                   DirAsDSK::DirIndex& dirDirIndex)
-		: DirScanner(dirDirIndex)
+	                   DirAsDSK::DirIndex& dirDirIndex_)
+		: DirScanner(dirDirIndex_)
 		, cluster(cluster_)
 		, result(dirIndex_) {}
 	bool onDirEntry(DirAsDSK::DirIndex dirIndex, const MSXDirEntry& entry) {
@@ -1092,14 +1127,14 @@ DirAsDSK::DirIndex DirAsDSK::getDirEntryForCluster(unsigned cluster)
 	if (getDirEntryForCluster(cluster, dirIndex, dirDirIndex)) {
 		return dirIndex;
 	} else {
-		return DirIndex(unsigned(-1), unsigned(-1)); // not found
+		return {unsigned(-1), unsigned(-1)}; // not found
 	}
 }
 
 // Remove the mapping between the msx and host for all the files/dirs in the
 // given msx directory (+ subdirectories).
 struct UnmapHostFiles : NullScanner {
-	UnmapHostFiles(DirAsDSK::MapDirs& mapDirs_)
+	explicit UnmapHostFiles(DirAsDSK::MapDirs& mapDirs_)
 		: mapDirs(mapDirs_) {}
 	bool onDirEntry(DirAsDSK::DirIndex dirIndex,
 	                const MSXDirEntry& /*entry*/) {
@@ -1122,28 +1157,27 @@ void DirAsDSK::exportToHost(DirIndex dirIndex, DirIndex dirDirIndex)
 	}
 	const char* msxName = msxDir(dirIndex).filename;
 	string hostName;
-	auto it = mapDirs.find(dirIndex);
-	if (it == mapDirs.end()) {
+	if (auto v = lookup(mapDirs, dirIndex)) {
+		// Hostname is already known.
+		hostName = v->hostName;
+	} else {
 		// Host file/dir does not yet exist, create hostname from
 		// msx name.
-		if ((msxName[0] == char(0x00)) || (msxName[0] == char(0xE5))) {
+		if (msxName[0] == one_of(char(0x00), char(0xE5))) {
 			// Invalid MSX name, don't do anything.
 			return;
 		}
 		string hostSubDir;
 		if (dirDirIndex.sector != 0) {
 			// Not the msx root directory.
-			auto it2 = mapDirs.find(dirDirIndex);
-			assert(it2 != mapDirs.end());
-			hostSubDir = it2->second.hostName;
+			auto v2 = lookup(mapDirs, dirDirIndex);
+			assert(v2);
+			hostSubDir = v2->hostName;
 			assert(!StringOp::endsWith(hostSubDir, '/'));
 			hostSubDir += '/';
 		}
 		hostName = hostSubDir + msxToHostName(msxName);
 		mapDirs[dirIndex].hostName = hostName;
-	} else {
-		// Hostname is already known.
-		hostName = it->second.hostName;
 	}
 	if (msxDir(dirIndex).attrib & MSXDirEntry::ATT_DIRECTORY) {
 		if ((memcmp(msxName, ".          ", 11) == 0) ||
@@ -1172,7 +1206,14 @@ void DirAsDSK::exportToHostDir(DirIndex dirIndex, const string& hostName)
 		FileOperations::mkdirp(fullHostName);
 
 		// Export all the components in this directory.
+		vector<bool> visited(nofSectors, false);
 		do {
+			if (visited[msxDirSector]) {
+				// detected cycle, invalid disk, but don't crash on it
+				return;
+			}
+			visited[msxDirSector] = true;
+
 			if (readFAT(sectorToCluster(msxDirSector)) == FREE_FAT) {
 				// This happens e.g. on a TurboR when a directory
 				// is removed: first the FAT is cleared before
@@ -1185,8 +1226,8 @@ void DirAsDSK::exportToHostDir(DirIndex dirIndex, const string& hostName)
 			msxDirSector = nextMsxDirSector(msxDirSector);
 		} while (msxDirSector != unsigned(-1));
 	} catch (FileException& e) {
-		cliComm.printWarning("Error while syncing host directory: " +
-			hostName + ": " + e.getMessage());
+		cliComm.printWarning("Error while syncing host directory: ",
+		                     hostName, ": ", e.getMessage());
 	}
 }
 
@@ -1203,7 +1244,15 @@ void DirAsDSK::exportToHostFile(DirIndex dirIndex, const string& hostName)
 		string fullHostName = hostDir + hostName;
 		File file(fullHostName, File::TRUNCATE);
 		unsigned offset = 0;
+		vector<bool> visited(maxCluster, false);
+
 		while ((FIRST_CLUSTER <= curCl) && (curCl < maxCluster)) {
+			if (visited[curCl]) {
+				// detected cycle, invalid disk, but don't crash on it
+				return;
+			}
+			visited[curCl] = true;
+
 			unsigned logicalSector = clusterToSector(curCl);
 			for (unsigned i = 0; i < SECTORS_PER_CLUSTER; ++i) {
 				if (offset >= msxSize) break;
@@ -1217,8 +1266,8 @@ void DirAsDSK::exportToHostFile(DirIndex dirIndex, const string& hostName)
 			curCl = readFAT(curCl);
 		}
 	} catch (FileException& e) {
-		cliComm.printWarning("Error while syncing host file: " +
-			hostName + ": " + e.getMessage());
+		cliComm.printWarning("Error while syncing host file: ",
+		                     hostName, ": ", e.getMessage());
 	}
 }
 
@@ -1240,12 +1289,12 @@ void DirAsDSK::writeDIRSector(unsigned sector, DirIndex dirDirIndex,
 void DirAsDSK::writeDIREntry(DirIndex dirIndex, DirIndex dirDirIndex,
                              const MSXDirEntry& newEntry)
 {
-	if (memcmp(msxDir(dirIndex).filename, newEntry.filename, 8 + 3) ||
+	if (memcmp(msxDir(dirIndex).filename, newEntry.filename, 8 + 3) != 0 ||
 	    ((msxDir(dirIndex).attrib & MSXDirEntry::ATT_DIRECTORY) !=
 	     (        newEntry.attrib & MSXDirEntry::ATT_DIRECTORY))) {
 		// Name or file-type in the direntry was changed.
 		auto it = mapDirs.find(dirIndex);
-		if (it != mapDirs.end()) {
+		if (it != end(mapDirs)) {
 			// If there is an associated hostfile, then delete it
 			// (in case of a rename, the file will be recreated
 			// below).
@@ -1289,14 +1338,14 @@ void DirAsDSK::writeDataSector(unsigned sector, const SectorBuffer& buf)
 	// Get corresponding directory entry.
 	DirIndex dirIndex = getDirEntryForCluster(startCluster);
 	// no need to check for 'dirIndex.sector == unsigned(-1)'
-	auto it = mapDirs.find(dirIndex);
-	if (it == mapDirs.end()) {
+	auto v = lookup(mapDirs, dirIndex);
+	if (!v) {
 		// This sector was not mapped to a file, nothing more to do.
 		return;
 	}
 
 	// Actually write data to host file.
-	string fullHostName = hostDir + it->second.hostName;
+	string fullHostName = hostDir + v->hostName;
 	try {
 		File file(fullHostName, "rb+"); // don't uncompress
 		file.seek(offset);
@@ -1306,8 +1355,8 @@ void DirAsDSK::writeDataSector(unsigned sector, const SectorBuffer& buf)
 			file.write(&buf, writeSize);
 		}
 	} catch (FileException& e) {
-		cliComm.printWarning("Couldn't write to file " + fullHostName +
-		                     ": " + e.getMessage());
+		cliComm.printWarning("Couldn't write to file ", fullHostName,
+		                     ": ", e.getMessage());
 	}
 }
 
