@@ -1,28 +1,20 @@
 #include "Disk.hh"
 #include "DiskExceptions.hh"
+#include "one_of.hh"
 
 using std::string;
 
 namespace openmsx {
 
-Disk::Disk(const DiskName& name_)
-	: name(name_), nbSides(0)
+Disk::Disk(DiskName name_)
+	: name(std::move(name_)), nbSides(0)
 {
 }
 
-Disk::~Disk()
-{
-}
-
-const DiskName& Disk::getName() const
-{
-	return name;
-}
-
 void Disk::writeTrack(byte track, byte side, const RawTrack& input)
 {
 	if (isWriteProtected()) {
-		throw WriteProtectedException("");
+		throw WriteProtectedException();
 	}
 	writeTrackImpl(track, side, input);
 	flushCaches();
@@ -68,10 +60,6 @@ void Disk::logToPhys(size_t log, byte& track, byte& side, byte& sector)
 	sector = byte((log % sectorsPerTrack) + 1);
 }
 
-void Disk::setSectorsPerTrack(unsigned num)
-{
-	sectorsPerTrack = num;
-}
 unsigned Disk::getSectorsPerTrack()
 {
 	if (!nbSides) {
@@ -79,10 +67,6 @@ unsigned Disk::getSectorsPerTrack()
 	}
 	return sectorsPerTrack;
 }
-void Disk::setNbSides(unsigned num)
-{
-	nbSides = num;
-}
 
 void Disk::detectGeometryFallback() // if all else fails, use statistics
 {
@@ -130,7 +114,7 @@ void Disk::detectGeometry()
 	try {
 		SectorBuffer buf;
 		readSector(0, buf); // bootsector
-		if ((buf.raw[0] == 0xE9) || (buf.raw[0] == 0xEB)) {
+		if (buf.raw[0] == one_of(0xE9, 0xEB)) {
 			// use values from bootsector
 			sectorsPerTrack = buf.bootSector.sectorsTrack;
 			nbSides         = buf.bootSector.nrSides;
