@@ -8,10 +8,10 @@
 #include "RamDSKDiskImage.hh"
 #include "DirAsDSK.hh"
 #include "DiskPartition.hh"
-#include "EnumSetting.hh"
 #include "MSXException.hh"
 #include "StringOp.hh"
-#include "memory.hh"
+#include <memory>
+#include <stdexcept>
 
 using std::string;
 
@@ -19,61 +19,57 @@ namespace openmsx {
 
 DiskFactory::DiskFactory(Reactor& reactor_)
 	: reactor(reactor_)
-{
-	CommandController& controller = reactor.getCommandController();
-
-	syncDirAsDSKSetting = make_unique<EnumSetting<DirAsDSK::SyncMode>>(
-		controller, "DirAsDSKmode",
+	, syncDirAsDSKSetting(
+		reactor.getCommandController(), "DirAsDSKmode",
 		"type of syncronisation between host directory and dir-as-dsk diskimage",
 		DirAsDSK::SYNC_FULL, EnumSetting<DirAsDSK::SyncMode>::Map{
 			{"read_only", DirAsDSK::SYNC_READONLY},
-			{"full",      DirAsDSK::SYNC_FULL}});
-
-	bootSectorSetting = make_unique<EnumSetting<DirAsDSK::BootSectorType>>(
-		controller, "bootsector", "boot sector type for dir-as-dsk",
+			{"full",      DirAsDSK::SYNC_FULL}})
+	, bootSectorSetting(
+		reactor.getCommandController(), "bootsector",
+		"boot sector type for dir-as-dsk",
 		DirAsDSK::BOOTSECTOR_DOS2, EnumSetting<DirAsDSK::BootSectorType>::Map{
 			{"DOS1", DirAsDSK::BOOTSECTOR_DOS1},
-			{"DOS2", DirAsDSK::BOOTSECTOR_DOS2}});
+			{"DOS2", DirAsDSK::BOOTSECTOR_DOS2}})
+{
 }
 
 std::unique_ptr<Disk> DiskFactory::createDisk(
 	const string& diskImage, DiskChanger& diskChanger)
 {
 	if (diskImage == "ramdsk") {
-		return make_unique<RamDSKDiskImage>();
+		return std::make_unique<RamDSKDiskImage>();
 	}
 
-	Filename filename(diskImage, UserFileContext());
+	Filename filename(diskImage, userFileContext());
 	try {
 		// First try DirAsDSK
-		return make_unique<DirAsDSK>(
+		return std::make_unique<DirAsDSK>(
 			diskChanger,
 			reactor.getCliComm(),
 			filename,
-			syncDirAsDSKSetting->getEnum(),
-			bootSectorSetting->getEnum());
+			syncDirAsDSKSetting.getEnum(),
+			bootSectorSetting.getEnum());
 	} catch (MSXException&) {
 		// DirAsDSK didn't work, no problem
 	}
 	try {
 		auto file = std::make_shared<File>(filename, File::PRE_CACHE);
-		file->setFilePool(reactor.getFilePool());
-
 		try {
 			// first try XSA
-			return make_unique<XSADiskImage>(filename, *file);
+			return std::make_unique<XSADiskImage>(filename, *file);
 		} catch (MSXException&) {
 			// XSA didn't work, still no problem
 		}
 		try {
 			// next try dmk
 			file->seek(0);
-			return make_unique<DMKDiskImage>(filename, file);
+			return std::make_unique<DMKDiskImage>(filename, file);
 		} catch (MSXException& /*e*/) {
 			// DMK didn't work, still no problem
 		}
 		// next try normal DSK
-		return make_unique<DSKDiskImage>(filename, file);
+		return std::make_unique<DSKDiskImage>(filename, std::move(file));
 
 	} catch (MSXException& e) {
 		// File could not be opened or (very rare) something is wrong
@@ -98,9 +94,15 @@ std::unique_ptr<Disk> DiskFactory::createDisk(
 			// likely more descriptive.
 			throw e;
 		}
-		unsigned num = StringOp::stringToUint(
-			diskImage.substr(pos + 1));
-		return make_unique<DiskPartition>(*wholeDisk, num, wholeDisk);
+		unsigned num;
+		try {
+			num = StringOp::fast_stou(std::string_view(diskImage).substr(pos + 1));
+		} catch (std::invalid_argument&) {
+			// not a valid partion number, throw previous exception
+			throw e;
+		}
+		SectorAccessibleDisk& disk = *wholeDisk;
+		return std::make_unique<DiskPartition>(disk, num, std::move(wholeDisk));
 	}
 }
 
