@@ -5,9 +5,14 @@ using std::string;
 
 namespace openmsx {
 
-DiskName::DiskName(const Filename& name_, const string& extra_)
-	: name(name_)
-	, extra(extra_)
+DiskName::DiskName(Filename name_)
+	: name(std::move(name_))
+{
+}
+
+DiskName::DiskName(Filename name_, string extra_)
+	: name(std::move(name_))
+	, extra(std::move(extra_))
 {
 }
 
@@ -31,16 +36,11 @@ bool DiskName::empty() const
 	return name.empty() && extra.empty();
 }
 
-const Filename& DiskName::getFilename() const
-{
-	return name;
-}
-
 template<typename Archive>
 void DiskName::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("filename", name);
-	ar.serialize("extra", extra);
+	ar.serialize("filename", name,
+	             "extra",    extra);
 }
 INSTANTIATE_SERIALIZE_METHODS(DiskName);
 
