@@ -12,37 +12,40 @@
 #include "FileOperations.hh"
 #include "MSXException.hh"
 #include "StringOp.hh"
+#include "strCat.hh"
 #include "File.hh"
+#include "one_of.hh"
+#include "stl.hh"
 #include <cstring>
-#include <algorithm>
 #include <cassert>
+#include <cctype>
 #include <sys/stat.h>
-#include <ctype.h>
 
 using std::string;
+using std::string_view;
 
 namespace openmsx {
 
-static const unsigned BAD_FAT = 0xFF7;
-static const unsigned EOF_FAT = 0xFFF; // actually 0xFF8-0xFFF, signals EOF in FAT12
-static const unsigned SECTOR_SIZE = SectorAccessibleDisk::SECTOR_SIZE;
+constexpr unsigned BAD_FAT = 0xFF7;
+constexpr unsigned EOF_FAT = 0xFFF; // actually 0xFF8-0xFFF, signals EOF in FAT12
+constexpr unsigned SECTOR_SIZE = SectorAccessibleDisk::SECTOR_SIZE;
 
-static const byte T_MSX_REG  = 0x00; // Normal file
-static const byte T_MSX_READ = 0x01; // Read-Only file
-static const byte T_MSX_HID  = 0x02; // Hidden file
-static const byte T_MSX_SYS  = 0x04; // System file
-static const byte T_MSX_VOL  = 0x08; // filename is Volume Label
-static const byte T_MSX_DIR  = 0x10; // entry is a subdir
-static const byte T_MSX_ARC  = 0x20; // Archive bit
+constexpr byte T_MSX_REG  = 0x00; // Normal file
+constexpr byte T_MSX_READ = 0x01; // Read-Only file
+constexpr byte T_MSX_HID  = 0x02; // Hidden file
+constexpr byte T_MSX_SYS  = 0x04; // System file
+constexpr byte T_MSX_VOL  = 0x08; // filename is Volume Label
+constexpr byte T_MSX_DIR  = 0x10; // entry is a subdir
+constexpr byte T_MSX_ARC  = 0x20; // Archive bit
 // This particular combination of flags indicates that this dir entry is used
 // to store a long Unicode file name.
 // For details, read http://home.teleport.com/~brainy/lfn.htm
-static const byte T_MSX_LFN  = 0x0F; // LFN entry (long files names)
+constexpr byte T_MSX_LFN  = 0x0F; // LFN entry (long files names)
 
 /** Transforms a clusternumber towards the first sector of this cluster
   * The calculation uses info read fom the bootsector
   */
-unsigned MSXtar::clusterToSector(unsigned cluster)
+unsigned MSXtar::clusterToSector(unsigned cluster) const
 {
 	return 1 + rootDirLast + sectorsPerCluster * (cluster - 2);
 }
@@ -50,7 +53,7 @@ unsigned MSXtar::clusterToSector(unsigned cluster)
 /** Transforms a sectornumber towards it containing cluster
   * The calculation uses info read fom the bootsector
   */
-unsigned MSXtar::sectorToCluster(unsigned sector)
+unsigned MSXtar::sectorToCluster(unsigned sector) const
 {
 	return 2 + ((sector - (1 + rootDirLast)) / sectorsPerCluster);
 }
@@ -65,28 +68,28 @@ void MSXtar::parseBootSector(const MSXBootSector& boot)
 	sectorsPerCluster = boot.spCluster;
 
 	if (boot.nrSectors == 0) { // TODO: check limits more accurately
-		throw MSXException(StringOp::Builder() <<
-			"Illegal number of sectors: " << boot.nrSectors);
+		throw MSXException(
+			"Illegal number of sectors: ", boot.nrSectors);
 	}
 	if (boot.nrSides == 0) { // TODO: check limits more accurately
-		throw MSXException(StringOp::Builder() <<
-			"Illegal number of sides: " << boot.nrSides);
+		throw MSXException(
+			"Illegal number of sides: ", boot.nrSides);
 	}
 	if (boot.nrFats == 0) { // TODO: check limits more accurately
-		throw MSXException(StringOp::Builder() <<
-			"Illegal number of FATs: " << boot.nrFats);
+		throw MSXException(
+			"Illegal number of FATs: ", boot.nrFats);
 	}
 	if (sectorsPerFat == 0) { // TODO: check limits more accurately
-		throw MSXException(StringOp::Builder() <<
-			"Illegal number sectors per FAT: " << sectorsPerFat);
+		throw MSXException(
+			"Illegal number sectors per FAT: ", sectorsPerFat);
 	}
 	if (nbRootDirSectors == 0) { // TODO: check limits more accurately
-		throw MSXException(StringOp::Builder() <<
-			"Illegal number of root dir sectors: " << nbRootDirSectors);
+		throw MSXException(
+			"Illegal number of root dir sectors: ", nbRootDirSectors);
 	}
 	if (sectorsPerCluster == 0) { // TODO: check limits more accurately
-		throw MSXException(StringOp::Builder() <<
-			"Illegal number of sectors per cluster: " << sectorsPerCluster);
+		throw MSXException(
+			"Illegal number of sectors per cluster: ", sectorsPerCluster);
 	}
 
 	rootDirStart = 1 + boot.nrFats * sectorsPerFat;
@@ -103,8 +106,9 @@ void MSXtar::parseBootSector(const MSXBootSector& boot)
 
 void MSXtar::writeLogicalSector(unsigned sector, const SectorBuffer& buf)
 {
+	assert(!fatBuffer.empty());
 	unsigned fatSector = sector - 1;
-	if ((fatSector < sectorsPerFat) && !fatBuffer.empty()) {
+	if (fatSector < sectorsPerFat) {
 		// we have a cache and this is a sector of the 1st FAT
 		//   --> update cache
 		memcpy(&fatBuffer[fatSector], &buf, sizeof(buf));
@@ -116,8 +120,9 @@ void MSXtar::writeLogicalSector(unsigned sector, const SectorBuffer& buf)
 
 void MSXtar::readLogicalSector(unsigned sector, SectorBuffer& buf)
 {
+	assert(!fatBuffer.empty());
 	unsigned fatSector = sector - 1;
-	if ((fatSector < sectorsPerFat) && !fatBuffer.empty()) {
+	if (fatSector < sectorsPerFat) {
 		// we have a cache and this is a sector of the 1st FAT
 		//   --> read from cache
 		memcpy(&buf, &fatBuffer[fatSector], sizeof(buf));
@@ -137,7 +142,7 @@ MSXtar::MSXtar(SectorAccessibleDisk& sectordisk)
 		disk.readSector(0, buf);
 		parseBootSector(buf.bootSector);
 	} catch (MSXException& e) {
-		throw MSXException("Bad disk image: " + e.getMessage());
+		throw MSXException("Bad disk image: ", e.getMessage());
 	}
 
 	// cache complete FAT
@@ -152,7 +157,7 @@ MSXtar::~MSXtar()
 {
 	if (!fatCacheDirty) return;
 
-	for (unsigned i = 0; i < fatBuffer.size(); ++i) {
+	for (unsigned i = 0; i < sectorsPerFat; ++i) {
 		try {
 			disk.writeSector(i + 1, fatBuffer[i]);
 		} catch (MSXException&) {
@@ -270,8 +275,7 @@ unsigned MSXtar::findUsableIndexInSector(unsigned sector)
 
 	// find a not used (0x00) or delete entry (0xE5)
 	for (unsigned i = 0; i < 16; ++i) {
-		byte tmp = buf.dirEntry[i].filename[0];
-		if ((tmp == 0x00) || (tmp == 0xE5)) {
+		if (buf.dirEntry[i].filename[0] == one_of(0x00, char(0xE5))) {
 			return i;
 		}
 	}
@@ -319,7 +323,7 @@ MSXtar::DirEntry MSXtar::addEntryToDir(unsigned sector)
 static char toMSXChr(char a)
 {
 	a = toupper(a);
-	if (a == ' ' || a == '.') {
+	if (a == one_of(' ', '.')) {
 		a = '_';
 	}
 	return a;
@@ -327,20 +331,18 @@ static char toMSXChr(char a)
 
 // Transform a long hostname in a 8.3 uppercase filename as used in the
 // direntries on an MSX
-static string makeSimpleMSXFileName(string_ref fullFilename)
+static string makeSimpleMSXFileName(string_view fullFilename)
 {
-	string_ref dir, fullFile;
-	StringOp::splitOnLast(fullFilename, '/', dir, fullFile);
+	auto [dir, fullFile] = StringOp::splitOnLast(fullFilename, '/');
 
 	// handle speciale case '.' and '..' first
 	string result(8 + 3, ' ');
-	if ((fullFile == ".") || (fullFile == "..")) {
-		memcpy(&*begin(result), fullFile.data(), fullFile.size());
+	if (fullFile == one_of(".", "..")) {
+		memcpy(result.data(), fullFile.data(), fullFile.size());
 		return result;
 	}
 
-	string_ref file, ext;
-	StringOp::splitOnLast(fullFile, '.', file, ext);
+	auto [file, ext] = StringOp::splitOnLast(fullFile, '.');
 	if (file.empty()) std::swap(file, ext);
 
 	StringOp::trimRight(file, ' ');
@@ -349,12 +351,12 @@ static string makeSimpleMSXFileName(string_ref fullFilename)
 	// put in major case and create '_' if needed
 	string fileS(file.data(), std::min<size_t>(8, file.size()));
 	string extS (ext .data(), std::min<size_t>(3, ext .size()));
-	std::transform(begin(fileS), end(fileS), begin(fileS), toMSXChr);
-	std::transform(begin(extS ), end(extS ), begin(extS ), toMSXChr);
+	transform_in_place(fileS, toMSXChr);
+	transform_in_place(extS,  toMSXChr);
 
 	// add correct number of spaces
-	memcpy(&*begin(result) + 0, fileS.data(), fileS.size());
-	memcpy(&*begin(result) + 8, extS .data(), extS .size());
+	memcpy(result.data() + 0, fileS.data(), fileS.size());
+	memcpy(result.data() + 8, extS .data(), extS .size());
 	return result;
 }
 
@@ -440,8 +442,11 @@ static void getTimeDate(const string& filename, unsigned& time, unsigned& date)
 		time = 0;
 		date = 0;
 	} else {
-		// some info indicates that st.st_mtime could be useless on win32 with vfat.
-		getTimeDate((time_t&)st.st_mtime, time, date);
+		// Some info indicates that st.st_mtime could be useless on win32 with vfat.
+		// On Android 'st_mtime' is 'unsigned long' instead of 'time_t'
+		// (like on linux), so we require a reinterpret_cast. That cast
+		// is fine (but redundant) on linux.
+		getTimeDate(reinterpret_cast<time_t&>(st.st_mtime), time, date);
 	}
 }
 
@@ -464,7 +469,7 @@ void MSXtar::alterFileInDSK(MSXDirEntry& msxDirEntry, const string& hostName)
 	// get host file size
 	struct stat st;
 	if (stat(hostName.c_str(), &st)) {
-		throw MSXException("Error reading host file: " + hostName);
+		throw MSXException("Error reading host file: ", hostName);
 	}
 	unsigned hostSize = st.st_size;
 	unsigned remaining = hostSize;
@@ -478,7 +483,7 @@ void MSXtar::alterFileInDSK(MSXDirEntry& msxDirEntry, const string& hostName)
 	while (remaining) {
 		// allocate new cluster if needed
 		try {
-			if ((curCl == 0) || (curCl == EOF_FAT)) {
+			if (curCl == one_of(0u, EOF_FAT)) {
 				unsigned newCl = findFirstFreeCluster();
 				if (prevCl == 0) {
 					msxDirEntry.startCluster = newCl;
@@ -517,7 +522,7 @@ void MSXtar::alterFileInDSK(MSXDirEntry& msxDirEntry, const string& hostName)
 	}
 
 	// free rest of FAT chain
-	while ((curCl != EOF_FAT) && (curCl != 0)) {
+	while (curCl != one_of(EOF_FAT, 0u)) {
 		unsigned nextCl = readFAT(curCl);
 		writeFAT(curCl, 0);
 		curCl = nextCl;
@@ -527,7 +532,7 @@ void MSXtar::alterFileInDSK(MSXDirEntry& msxDirEntry, const string& hostName)
 	msxDirEntry.size = hostSize - remaining;
 
 	if (remaining) {
-		throw MSXException("Disk full, " + hostName + " truncated.");
+		throw MSXException("Disk full, ", hostName, " truncated.");
 	}
 }
 
@@ -557,10 +562,9 @@ MSXtar::DirEntry MSXtar::findEntryInDir(
 
 // Add file to the MSX disk in the subdir pointed to by 'sector'
 // @throws when file could not be added
-string MSXtar::addFileToDSK(const string& fullname, unsigned rootSector)
+string MSXtar::addFileToDSK(const string& fullHostName, unsigned rootSector)
 {
-	string_ref dir, hostName;
-	StringOp::splitOnLast(fullname, "/\\", dir, hostName);
+	auto [directory, hostName] = StringOp::splitOnLast(fullHostName, "/\\");
 	string msxName = makeSimpleMSXFileName(hostName);
 
 	// first find out if the filename already exists in current dir
@@ -568,7 +572,7 @@ string MSXtar::addFileToDSK(const string& fullname, unsigned rootSector)
 	DirEntry fullMsxDirEntry = findEntryInDir(msxName, rootSector, dummy);
 	if (fullMsxDirEntry.sector != 0) {
 		// TODO implement overwrite option
-		return "Warning: preserving entry " + hostName + '\n';
+		return strCat("Warning: preserving entry ", hostName, '\n');
 	}
 
 	SectorBuffer buf;
@@ -581,30 +585,30 @@ string MSXtar::addFileToDSK(const string& fullname, unsigned rootSector)
 
 	// compute time/date stamps
 	unsigned time, date;
-	getTimeDate(fullname, time, date);
+	getTimeDate(fullHostName, time, date);
 	dirEntry.time = time;
 	dirEntry.date = date;
 
 	try {
-		alterFileInDSK(dirEntry, fullname);
+		alterFileInDSK(dirEntry, fullHostName);
 	} catch (MSXException&) {
 		// still write directory entry
 		writeLogicalSector(entry.sector, buf);
 		throw;
 	}
 	writeLogicalSector(entry.sector, buf);
-	return "";
+	return {};
 }
 
 // Transfer directory and all its subdirectories to the MSX diskimage
 // @throws when an error occurs
-string MSXtar::recurseDirFill(string_ref dirName, unsigned sector)
+string MSXtar::recurseDirFill(string_view dirName, unsigned sector)
 {
 	string messages;
-	ReadDir dir(dirName.str());
-	while (dirent* d = dir.getEntry()) {
+	ReadDir readDir{string(dirName)};
+	while (dirent* d = readDir.getEntry()) {
 		string name(d->d_name);
-		string fullName = dirName + '/' + name;
+		string fullName = strCat(dirName, '/', name);
 
 		FileOperations::Stat st;
 		if (!FileOperations::getStat(fullName, st)) {
@@ -616,8 +620,7 @@ string MSXtar::recurseDirFill(string_ref dirName, unsigned sector)
 			// add new file
 			messages += addFileToDSK(fullName, sector);
 
-		} else if (FileOperations::isDirectory(st) &&
-			   (name != ".") && (name != "..")) {
+		} else if (FileOperations::isDirectory(st) && name != one_of(".", "..")) {
 			string msxFileName = makeSimpleMSXFileName(name);
 			SectorBuffer buf;
 			DirEntry entry = findEntryInDir(msxFileName, sector, buf);
@@ -631,8 +634,9 @@ string MSXtar::recurseDirFill(string_ref dirName, unsigned sector)
 					messages += recurseDirFill(fullName, nextSector);
 				} else {
 					// .. but is NOT a directory
-					messages += "MSX file " + msxFileName +
-					            " is not a directory.\n";
+					strAppend(messages,
+					          "MSX file ", msxFileName,
+					          " is not a directory.\n");
 				}
 			} else {
 				// add new directory
@@ -649,12 +653,12 @@ string MSXtar::condensName(const MSXDirEntry& dirEntry)
 {
 	string result;
 	for (unsigned i = 0; (i < 8) && (dirEntry.name.base[i] != ' '); ++i) {
-		result += tolower(dirEntry.name.base[i]);
+		result += char(tolower(dirEntry.name.base[i]));
 	}
 	if (dirEntry.name.ext[0] != ' ') {
 		result += '.';
 		for (unsigned i = 0; (i < 3) && (dirEntry.name.ext[i] != ' '); ++i) {
-			result += tolower(dirEntry.name.ext[i]);
+			result += char(tolower(dirEntry.name.ext[i]));
 		}
 	}
 	return result;
@@ -682,58 +686,55 @@ void MSXtar::changeTime(const string& resultFile, const MSXDirEntry& dirEntry)
 
 string MSXtar::dir()
 {
-	StringOp::Builder result;
+	string result;
 	for (unsigned sector = chrootSector; sector != 0; sector = getNextSector(sector)) {
 		SectorBuffer buf;
 		readLogicalSector(sector, buf);
-		for (unsigned i = 0; i < 16; ++i) {
-			auto& dirEntry = buf.dirEntry[i];
-			if ((dirEntry.filename[0] == char(0xe5)) ||
-			    (dirEntry.filename[0] == char(0x00)) ||
+		for (auto& dirEntry : buf.dirEntry) {
+			if ((dirEntry.filename[0] == one_of(char(0xe5), char(0x00))) ||
 			    (dirEntry.attrib == T_MSX_LFN)) continue;
 
 			// filename first (in condensed form for human readablitly)
 			string tmp = condensName(dirEntry);
 			tmp.resize(13, ' ');
-			result << tmp;
-			// attributes
-			result << (dirEntry.attrib & T_MSX_DIR  ? 'd' : '-')
-			       << (dirEntry.attrib & T_MSX_READ ? 'r' : '-')
-			       << (dirEntry.attrib & T_MSX_HID  ? 'h' : '-')
-			       << (dirEntry.attrib & T_MSX_VOL  ? 'v' : '-') // TODO check if this is the output of files,l
-			       << (dirEntry.attrib & T_MSX_ARC  ? 'a' : '-') // TODO check if this is the output of files,l
-			       << "  ";
-			// filesize
-			result << dirEntry.size << '\n';
+			strAppend(result, tmp,
+			          // attributes
+			          (dirEntry.attrib & T_MSX_DIR  ? 'd' : '-'),
+			          (dirEntry.attrib & T_MSX_READ ? 'r' : '-'),
+			          (dirEntry.attrib & T_MSX_HID  ? 'h' : '-'),
+			          (dirEntry.attrib & T_MSX_VOL  ? 'v' : '-'), // TODO check if this is the output of files,l
+			          (dirEntry.attrib & T_MSX_ARC  ? 'a' : '-'), // TODO check if this is the output of files,l
+			          "  ",
+			          // filesize
+			          dirEntry.size << '\n');
 		}
 	}
 	return result;
 }
 
 // routines to update the global vars: chrootSector
-void MSXtar::chdir(string_ref newRootDir)
+void MSXtar::chdir(string_view newRootDir)
 {
 	chroot(newRootDir, false);
 }
 
-void MSXtar::mkdir(string_ref newRootDir)
+void MSXtar::mkdir(string_view newRootDir)
 {
 	unsigned tmpMSXchrootSector = chrootSector;
 	chroot(newRootDir, true);
 	chrootSector = tmpMSXchrootSector;
 }
 
-void MSXtar::chroot(string_ref newRootDir, bool createDir)
+void MSXtar::chroot(string_view newRootDir, bool createDir)
 {
-	if (newRootDir.starts_with("/") || newRootDir.starts_with("\\")) {
+	if (StringOp::startsWith(newRootDir, '/') || StringOp::startsWith(newRootDir, '\\')) {
 		// absolute path, reset chrootSector
 		chrootSector = rootDirStart;
 		StringOp::trimLeft(newRootDir, "/\\");
 	}
 
 	while (!newRootDir.empty()) {
-		string_ref firstPart, lastPart;
-		StringOp::splitOnFirst(newRootDir, "/\\", firstPart, lastPart);
+		auto [firstPart, lastPart] = StringOp::splitOnFirst(newRootDir, "/\\");
 		newRootDir = lastPart;
 		StringOp::trimLeft(newRootDir, "/\\");
 
@@ -743,7 +744,7 @@ void MSXtar::chroot(string_ref newRootDir, bool createDir)
 		DirEntry entry = findEntryInDir(simple, chrootSector, buf);
 		if (entry.sector == 0) {
 			if (!createDir) {
-				throw MSXException("Subdirectory " + firstPart +
+				throw MSXException("Subdirectory ", firstPart,
 				                   " not found.");
 			}
 			// creat new subdir
@@ -755,7 +756,7 @@ void MSXtar::chroot(string_ref newRootDir, bool createDir)
 		} else {
 			auto& dirEntry = buf.dirEntry[entry.index];
 			if (!(dirEntry.attrib & T_MSX_DIR)) {
-				throw MSXException(firstPart + " is not a directory.");
+				throw MSXException(firstPart, " is not a directory.");
 			}
 			chrootSector = clusterToSector(getStartCluster(dirEntry));
 		}
@@ -781,7 +782,7 @@ void MSXtar::fileExtract(const string& resultFile, const MSXDirEntry& dirEntry)
 }
 
 // extracts a single item (file or directory) from the msximage to the host OS
-string MSXtar::singleItemExtract(string_ref dirName, string_ref itemName,
+string MSXtar::singleItemExtract(string_view dirName, string_view itemName,
                                  unsigned sector)
 {
 	// first find out if the filename exists in current dir
@@ -789,12 +790,12 @@ string MSXtar::singleItemExtract(string_ref dirName, string_ref itemName,
 	string msxName = makeSimpleMSXFileName(itemName);
 	DirEntry entry = findEntryInDir(msxName, sector, buf);
 	if (entry.sector == 0) {
-		return itemName + " not found!\n";
+		return strCat(itemName, " not found!\n");
 	}
 
 	auto& msxDirEntry = buf.dirEntry[entry.index];
 	// create full name for loacl filesystem
-	string fullName = dirName + '/' + condensName(msxDirEntry);
+	string fullName = strCat(dirName, '/', condensName(msxDirEntry));
 
 	// ...and extract
 	if  (msxDirEntry.attrib & T_MSX_DIR) {
@@ -807,26 +808,24 @@ string MSXtar::singleItemExtract(string_ref dirName, string_ref itemName,
 		// it is a file
 		fileExtract(fullName, msxDirEntry);
 	}
-	return "";
+	return {};
 }
 
 
 // extracts the contents of the directory (at sector) and all its subdirs to the host OS
-void MSXtar::recurseDirExtract(string_ref dirName, unsigned sector)
+void MSXtar::recurseDirExtract(string_view dirName, unsigned sector)
 {
 	for (/* */ ; sector != 0; sector = getNextSector(sector)) {
 		SectorBuffer buf;
 		readLogicalSector(sector, buf);
-		for (unsigned i = 0; i < 16; ++i) {
-			auto& dirEntry = buf.dirEntry[i];
-			if ((dirEntry.filename[0] == char(0xe5)) ||
-			    (dirEntry.filename[0] == char(0x00)) ||
-			    (dirEntry.filename[0] == '.')) continue;
-
+		for (auto& dirEntry : buf.dirEntry) {
+			if (dirEntry.filename[0] == one_of(char(0xe5), char(0x00), '.')) {
+				continue;
+			}
 			string filename = condensName(dirEntry);
 			string fullName = filename;
 			if (!dirName.empty()) {
-				fullName = dirName + '/' + filename;
+				fullName = strCat(dirName, '/', filename);
 			}
 			if (dirEntry.attrib != T_MSX_DIR) { // TODO
 				fileExtract(fullName, dirEntry);
@@ -843,7 +842,7 @@ void MSXtar::recurseDirExtract(string_ref dirName, unsigned sector)
 	}
 }
 
-string MSXtar::addDir(string_ref rootDirName)
+string MSXtar::addDir(string_view rootDirName)
 {
 	return recurseDirFill(rootDirName, chrootSector);
 }
@@ -853,12 +852,12 @@ string MSXtar::addFile(const string& filename)
 	return addFileToDSK(filename, chrootSector);
 }
 
-string MSXtar::getItemFromDir(string_ref rootDirName, string_ref itemName)
+string MSXtar::getItemFromDir(string_view rootDirName, string_view itemName)
 {
 	return singleItemExtract(rootDirName, itemName, chrootSector);
 }
 
-void MSXtar::getDir(string_ref rootDirName)
+void MSXtar::getDir(string_view rootDirName)
 {
 	recurseDirExtract(rootDirName, chrootSector);
 }
