@@ -3,15 +3,16 @@
 #include "SectorAccessibleDisk.hh"
 #include "serialize.hh"
 #include "serialize_stl.hh"
+#include "one_of.hh"
 #include "unreachable.hh"
-#include "memory.hh"
-#include <fstream>
 #include <algorithm>
 #include <cassert>
+#include <cctype>
 #include <cstdio>
-#include <ctype.h>
-#include <time.h>
-#include <stdarg.h>
+#include <cstdarg>
+#include <ctime>
+#include <fstream>
+#include <memory>
 
 using std::string;
 using std::vector;
@@ -20,7 +21,7 @@ using std::ios;
 
 namespace openmsx {
 
-static const unsigned SECTOR_SIZE = sizeof(SectorBuffer);
+constexpr unsigned SECTOR_SIZE = sizeof(SectorBuffer);
 
 static void DBERR(const char* message, ...)
 {
@@ -35,7 +36,7 @@ static void DBERR(const char* message, ...)
 }
 
 
-NowindHost::NowindHost(const NowindInterface::Drives& drives_)
+NowindHost::NowindHost(const Drives& drives_)
 	: drives(drives_)
 	, lastTime(0)
 	, state(STATE_SYNC1)
@@ -45,9 +46,7 @@ NowindHost::NowindHost(const NowindInterface::Drives& drives_)
 {
 }
 
-NowindHost::~NowindHost()
-{
-}
+NowindHost::~NowindHost() = default;
 
 byte NowindHost::peek() const
 {
@@ -120,10 +119,10 @@ void NowindHost::write(byte data, unsigned time)
 	case STATE_IMAGE:
 		assert(recvCount < 40);
 		extraData[recvCount] = data;
-		if ((data == 0) || (data == ':') ||
+		if (data == one_of(0, ':') ||
 		    (++recvCount == 40)) {
-			auto data = reinterpret_cast<char*>(extraData);
-			callImage(string(data, recvCount));
+			char* eData = reinterpret_cast<char*>(extraData);
+			callImage(string(eData, recvCount));
 			state = STATE_SYNC1;
 		}
 		break;
@@ -292,7 +291,7 @@ void NowindHost::DRIVES()
 	send(numberOfDrives);
 
 	romdisk = 255; // no romdisk
-	for (unsigned i = 0; i < drives.size(); ++i) {
+	for (size_t i = 0; i < drives.size(); ++i) {
 		if (drives[i]->isRomdisk()) {
 			romdisk = i;
 			break;
@@ -375,7 +374,7 @@ void NowindHost::doDiskRead1()
 		return;
 	}
 
-	static const unsigned NUMBEROFBLOCKS = 32; // 32 * 64 bytes = 2048 bytes
+	constexpr unsigned NUMBEROFBLOCKS = 32; // 32 * 64 bytes = 2048 bytes
 	transferSize = std::min(bytesLeft, NUMBEROFBLOCKS * 64); // hardcoded in firmware
 
 	unsigned address = getCurrentAddress();
@@ -453,7 +452,7 @@ void NowindHost::transferSectors(unsigned transferAddress, unsigned amount)
 	send(0x07); // used for validation
 }
 
- // sends "02" + "transfer_addr" + "amount" + "data" + "0F 07"
+// sends "02" + "transfer_addr" + "amount" + "data" + "0F 07"
 void NowindHost::transferSectorsBackwards(unsigned transferAddress, unsigned amount)
 {
 	sendHeader();
@@ -491,7 +490,7 @@ void NowindHost::doDiskWrite1()
 	unsigned bytesLeft = unsigned(buffer.size() * SECTOR_SIZE) - transfered;
 	if (bytesLeft == 0) {
 		// All data transferred!
-		unsigned sectorAmount = unsigned(buffer.size());
+		auto sectorAmount = unsigned(buffer.size());
 		unsigned startSector = getStartSector();
 		if (auto* disk = getDisk()) {
 			if (disk->writeSectors(&buffer[0], startSector, sectorAmount)) {
@@ -504,7 +503,7 @@ void NowindHost::doDiskWrite1()
 		return;
 	}
 
-	static const unsigned BLOCKSIZE = 240;
+	constexpr unsigned BLOCKSIZE = 240;
 	transferSize = std::min(bytesLeft, BLOCKSIZE);
 
 	unsigned address = getCurrentAddress();
@@ -570,7 +569,7 @@ string NowindHost::extractName(int begin, int end) const
 	for (int i = begin; i < end; ++i) {
 		char c = extraData[i];
 		if (c == ' ') break;
-		result += toupper(c);
+		result += char(toupper(c));
 	}
 	return result;
 }
@@ -615,13 +614,12 @@ void NowindHost::deviceOpen()
 	string filename = extractName(0, 8);
 	string ext      = extractName(8, 11);
 	if (!ext.empty()) {
-		filename += '.';
-		filename += ext;
+		strAppend(filename, '.', ext);
 	}
 
 	unsigned fcb = getFCB();
 	unsigned dev = getFreeDeviceNum();
-	devices[dev].fs = make_unique<fstream>(); // takes care of deleting old fs
+	devices[dev].fs = std::make_unique<fstream>(); // takes care of deleting old fs
 	devices[dev].fcb = fcb;
 
 	sendHeader();
@@ -656,10 +654,10 @@ void NowindHost::deviceOpen()
 
 	unsigned readLen = 0;
 	bool eof = false;
-	char buffer[256];
+	char buf[256];
 	if (openMode == 1) {
 		// read-only mode, already buffer first 256 bytes
-		readLen = readHelper1(dev, buffer);
+		readLen = readHelper1(dev, buf);
 		assert(readLen <= 256);
 		eof = readLen < 256;
 	}
@@ -679,7 +677,7 @@ void NowindHost::deviceOpen()
 	send(0);
 
 	if (openMode == 1) {
-		readHelper2(readLen, buffer);
+		readHelper2(readLen, buf);
 	}
 }
 
@@ -703,32 +701,32 @@ void NowindHost::deviceRead()
 	int dev = getDeviceNum();
 	if (dev == -1) return;
 
-	char buffer[256];
-	unsigned readLen = readHelper1(dev, buffer);
+	char buf[256];
+	unsigned readLen = readHelper1(dev, buf);
 	bool eof = readLen < 256;
 	send(0xAF);
 	send(0x05);
 	send(0x00); // dummy
 	send16(getFCB() + 9);
 	send16(readLen + (eof ? 1 : 0));
-	readHelper2(readLen, buffer);
+	readHelper2(readLen, buf);
 }
 
-unsigned NowindHost::readHelper1(unsigned dev, char* buffer)
+unsigned NowindHost::readHelper1(unsigned dev, char* buf)
 {
 	assert(dev < MAX_DEVICES);
 	unsigned len = 0;
 	for (/**/; len < 256; ++len) {
-		devices[dev].fs->read(&buffer[len], 1);
+		devices[dev].fs->read(&buf[len], 1);
 		if (devices[dev].fs->eof()) break;
 	}
 	return len;
 }
 
-void NowindHost::readHelper2(unsigned len, const char* buffer)
+void NowindHost::readHelper2(unsigned len, const char* buf)
 {
 	for (unsigned i = 0; i < len; ++i) {
-		send(buffer[i]);
+		send(buf[i]);
 	}
 	if (len < 256) {
 		send(0x1A); // end-of-file
@@ -738,7 +736,7 @@ void NowindHost::readHelper2(unsigned len, const char* buffer)
 
 // strips a string from outer double-quotes and anything outside them
 // ie: 'pre("foo")bar' will result in 'foo'
-static string stripquotes(const string& str)
+static std::string_view stripquotes(std::string_view str)
 {
 	auto first = str.find_first_of('\"');
 	if (first == string::npos) {
@@ -748,7 +746,7 @@ static string stripquotes(const string& str)
 	auto last = str.find_last_of ('\"');
 	if (first == last) {
 		// Error, there's only a single double-quote char.
-		return "";
+		return {};
 	}
 	// Return the part between the quotes.
 	return str.substr(first + 1, last - first - 1);
@@ -767,7 +765,7 @@ void NowindHost::callImage(const string& filename)
 }
 
 
-static enum_string<NowindHost::State> stateInfo[] = {
+static std::initializer_list<enum_string<NowindHost::State>> stateInfo = {
 	{ "SYNC1",     NowindHost::STATE_SYNC1     },
 	{ "SYNC2",     NowindHost::STATE_SYNC2     },
 	{ "COMMAND",   NowindHost::STATE_COMMAND   },
@@ -783,12 +781,12 @@ void NowindHost::serialize(Archive& ar, unsigned /*version*/)
 {
 	// drives is serialized elsewhere
 
-	ar.serialize("hostToMsxFifo", hostToMsxFifo);
-	ar.serialize("lastTime", lastTime);
-	ar.serialize("state", state);
-	ar.serialize("recvCount", recvCount);
-	ar.serialize("cmdData", cmdData);
-	ar.serialize("extraData", extraData);
+	ar.serialize("hostToMsxFifo", hostToMsxFifo,
+	             "lastTime",      lastTime,
+	             "state",         state,
+	             "recvCount",     recvCount,
+	             "cmdData",       cmdData,
+	             "extraData",     extraData);
 
 	// for backwards compatibility, serialize buffer as a vector<byte>
 	size_t bufSize = buffer.size() * sizeof(SectorBuffer);
@@ -797,12 +795,12 @@ void NowindHost::serialize(Archive& ar, unsigned /*version*/)
 	ar.serialize("buffer", tmp);
 	memcpy(bufRaw, tmp.data(), bufSize);
 
-	ar.serialize("transfered", transfered);
-	ar.serialize("retryCount", retryCount);
-	ar.serialize("transferSize", transferSize);
-	ar.serialize("romdisk", romdisk);
-	ar.serialize("allowOtherDiskroms", allowOtherDiskroms);
-	ar.serialize("enablePhantomDrives", enablePhantomDrives);
+	ar.serialize("transfered",          transfered,
+	             "retryCount",          retryCount,
+	             "transferSize",        transferSize,
+	             "romdisk",             romdisk,
+	             "allowOtherDiskroms",  allowOtherDiskroms,
+	             "enablePhantomDrives", enablePhantomDrives);
 
 	// Note: We don't serialize 'devices'. So after a loadstate it will be
 	// as-if the devices are closed again. The reason for not serializing
