@@ -4,27 +4,16 @@
 #include "DiskExceptions.hh"
 #include "sha1.hh"
 #include "xrange.hh"
-#include "memory.hh"
+#include <memory>
 
 namespace openmsx {
 
-#ifndef _MSC_VER
-// This line is required according to the c++ standard, but because of a vc++
-// extension, we get a link error in vc++ when we add this line. See also:
-//   http://blogs.msdn.com/b/xiangfan/archive/2010/03/03/vc-s-evil-extension-implicit-definition-of-static-constant-member.aspx
-const size_t SectorAccessibleDisk::SECTOR_SIZE;
-#endif
-
 SectorAccessibleDisk::SectorAccessibleDisk()
-	: patch(make_unique<EmptyDiskPatch>(*this))
-	, forcedWriteProtect(false)
-	, peekMode(false)
+	: patch(std::make_unique<EmptyDiskPatch>(*this))
 {
 }
 
-SectorAccessibleDisk::~SectorAccessibleDisk()
-{
-}
+SectorAccessibleDisk::~SectorAccessibleDisk() = default;
 
 void SectorAccessibleDisk::readSector(size_t sector, SectorBuffer& buf)
 {
@@ -40,14 +29,14 @@ void SectorAccessibleDisk::readSector(size_t sector, SectorBuffer& buf)
 		// in the end this calls readSectorImpl()
 		patch->copyBlock(sector * sizeof(buf), buf.raw, sizeof(buf));
 	} catch (MSXException& e) {
-		throw DiskIOErrorException("Disk I/O error: " + e.getMessage());
+		throw DiskIOErrorException("Disk I/O error: ", e.getMessage());
 	}
 }
 
 void SectorAccessibleDisk::writeSector(size_t sector, const SectorBuffer& buf)
 {
 	if (isWriteProtected()) {
-		throw WriteProtectedException("");
+		throw WriteProtectedException();
 	}
 	if (!isDummyDisk() && (getNbSectors() <= sector)) {
 		throw NoSuchSectorException("No such sector");
@@ -55,7 +44,7 @@ void SectorAccessibleDisk::writeSector(size_t sector, const SectorBuffer& buf)
 	try {
 		writeSectorImpl(sector, buf);
 	} catch (MSXException& e) {
-		throw DiskIOErrorException("Disk I/O error: " + e.getMessage());
+		throw DiskIOErrorException("Disk I/O error: ", e.getMessage());
 	}
 	flushCaches();
 }
@@ -65,9 +54,9 @@ size_t SectorAccessibleDisk::getNbSectors() const
 	return getNbSectorsImpl();
 }
 
-void SectorAccessibleDisk::applyPatch(const Filename& patchFile)
+void SectorAccessibleDisk::applyPatch(Filename patchFile)
 {
-	patch = make_unique<IPSPatch>(patchFile, std::move(patch));
+	patch = std::make_unique<IPSPatch>(std::move(patchFile), std::move(patch));
 }
 
 std::vector<Filename> SectorAccessibleDisk::getPatches() const
@@ -80,28 +69,33 @@ bool SectorAccessibleDisk::hasPatches() const
 	return !patch->isEmptyPatch();
 }
 
-Sha1Sum SectorAccessibleDisk::getSha1Sum()
+Sha1Sum SectorAccessibleDisk::getSha1Sum(FilePool& filePool)
 {
 	checkCaches();
 	if (sha1cache.empty()) {
-		try {
-			setPeekMode(true);
-			SHA1 sha1;
-			for (auto i : xrange(getNbSectors())) {
-				SectorBuffer buf;
-				readSector(i, buf);
-				sha1.update(buf.raw, sizeof(buf));
-			}
-			sha1cache = sha1.digest();
-			setPeekMode(false);
-		} catch (MSXException&) {
-			setPeekMode(false);
-			throw;
-		}
+		sha1cache = getSha1SumImpl(filePool);
 	}
 	return sha1cache;
 }
 
+Sha1Sum SectorAccessibleDisk::getSha1SumImpl(FilePool& /*filePool*/)
+{
+	try {
+		setPeekMode(true);
+		SHA1 sha1;
+		for (auto i : xrange(getNbSectors())) {
+			SectorBuffer buf;
+			readSector(i, buf);
+			sha1.update(buf.raw, sizeof(buf));
+		}
+		setPeekMode(false);
+		return sha1.digest();
+	} catch (MSXException&) {
+		setPeekMode(false);
+		throw;
+	}
+}
+
 int SectorAccessibleDisk::readSectors (
 	SectorBuffer* buffers, size_t startSector, size_t nbSectors)
 {
