@@ -2,9 +2,9 @@
 #include "DiskExceptions.hh"
 #include "File.hh"
 #include <cstring>
+#include <utility>
 
 using std::string;
-using std::vector;
 
 namespace openmsx {
 
@@ -12,12 +12,12 @@ class XSAExtractor
 {
 public:
 	explicit XSAExtractor(File& file);
-	void getData(MemBuffer<SectorBuffer>& data);
+	std::pair<MemBuffer<SectorBuffer>, unsigned> extractData();
 
 private:
-	static const int MAXSTRLEN = 254;
-	static const int TBLSIZE = 16;
-	static const int MAXHUFCNT = 127;
+	static constexpr int MAXSTRLEN = 254;
+	static constexpr int TBLSIZE = 16;
+	static constexpr int MAXHUFCNT = 127;
 
 	inline byte charIn();
 	void chkHeader();
@@ -37,6 +37,7 @@ private:
 	MemBuffer<SectorBuffer> outBuf;	// the output buffer
 	const byte* inBufPos;	// pos in input buffer
 	const byte* inBufEnd;
+	unsigned sectors;
 
 	int updHufCnt;
 	int cpDist[TBLSIZE + 1];
@@ -47,7 +48,9 @@ private:
 	byte bitFlg;		// flag with the bits
 	byte bitCnt;		// nb bits left
 
-	static const int cpdExt[TBLSIZE];	// Extra bits for distance codes
+	static constexpr int cpdExt[TBLSIZE] = { // Extra bits for distance codes
+		  0,  0,  0,  0,  1,  2,  3,  4, 5,  6,  7,  8,  9, 10, 11, 12
+	};
 };
 
 
@@ -57,8 +60,9 @@ XSADiskImage::XSADiskImage(Filename& filename, File& file)
 	: SectorBasedDisk(filename)
 {
 	XSAExtractor extractor(file);
-	extractor.getData(data);
-	setNbSectors(data.size());
+	unsigned sectors;
+	std::tie(data, sectors) = extractor.extractData();
+	setNbSectors(sectors);
 }
 
 void XSADiskImage::readSectorImpl(size_t sector, SectorBuffer& buf)
@@ -79,15 +83,11 @@ bool XSADiskImage::isWriteProtectedImpl() const
 
 // XSAExtractor
 
-const int XSAExtractor::cpdExt[TBLSIZE] = {
-	  0,  0,  0,  0,  1,  2,  3,  4, 5,  6,  7,  8,  9, 10, 11, 12
-};
-
 XSAExtractor::XSAExtractor(File& file)
 {
-	size_t size;
-	inBufPos = file.mmap(size);
-	inBufEnd = inBufPos + size;
+	auto mmap = file.mmap();
+	inBufPos = mmap.begin();
+	inBufEnd = mmap.end();
 
 	if ((charIn() != 'P') || (charIn() != 'C') ||
 	    (charIn() != 'K') || (charIn() != '\010')) {
@@ -99,10 +99,10 @@ XSAExtractor::XSAExtractor(File& file)
 	unLz77();
 }
 
-void XSAExtractor::getData(MemBuffer<SectorBuffer>& data)
+std::pair<MemBuffer<SectorBuffer>, unsigned> XSAExtractor::extractData()
 {
 	// destroys internal outBuf, but that's ok
-	data.swap(outBuf);
+	return {std::move(outBuf), sectors};
 }
 
 // Get the next character from the input buffer
@@ -122,7 +122,8 @@ void XSAExtractor::chkHeader()
 	for (int i = 0, base = 1; i < 4; ++i, base <<= 8) {
 		outBufLen += base * charIn();
 	}
-	outBuf.resize((outBufLen + 511) / 512);
+	sectors = (outBufLen + 511) / 512;
+	outBuf.resize(sectors);
 
 	// skip compressed length
 	inBufPos += 4;
@@ -136,7 +137,7 @@ void XSAExtractor::unLz77()
 {
 	bitCnt = 0; // no bits read yet
 
-	size_t remaining = outBuf.size() * sizeof(SectorBuffer);
+	size_t remaining = sectors * sizeof(SectorBuffer);
 	byte* out = outBuf.data()->raw;
 	size_t outIdx = 0;
 	while (true) {
