@@ -7,20 +7,20 @@ using std::string;
 
 namespace openmsx {
 
-FileBase::~FileBase()
-{
-	munmap();
-}
-
-const byte* FileBase::mmap(size_t& size)
+span<uint8_t> FileBase::mmap()
 {
+	auto size = getSize();
 	if (mmapBuf.empty()) {
-		size = getSize();
-		MemBuffer<byte> tmpBuf(size);
+		auto pos = getPos();
+		seek(0);
+
+		MemBuffer<uint8_t> tmpBuf(size);
 		read(tmpBuf.data(), size);
 		std::swap(mmapBuf, tmpBuf);
+
+		seek(pos);
 	}
-	return mmapBuf.data();
+	return {mmapBuf.data(), size};
 }
 
 void FileBase::munmap()
@@ -32,14 +32,14 @@ void FileBase::truncate(size_t newSize)
 {
 	auto oldSize = getSize();
 	if (newSize < oldSize) {
-		PRT_DEBUG("Default truncate() can't shrink file!");
+		// default truncate() can't shrink file
 		return;
 	}
 	auto remaining = newSize - oldSize;
 	seek(oldSize);
 
-	static const size_t BUF_SIZE = 4096;
-	byte buf[BUF_SIZE];
+	constexpr size_t BUF_SIZE = 4096;
+	uint8_t buf[BUF_SIZE];
 	memset(buf, 0, sizeof(buf));
 	while (remaining) {
 		auto chunkSize = std::min(BUF_SIZE, remaining);
@@ -48,17 +48,17 @@ void FileBase::truncate(size_t newSize)
 	}
 }
 
-const string FileBase::getLocalReference()
+string FileBase::getLocalReference()
 {
 	// default implementation, file is not backed (uncompressed) on
 	// the local file system
-	return "";
+	return {};
 }
 
-const string FileBase::getOriginalName()
+string FileBase::getOriginalName()
 {
 	// default implementation just returns filename portion of URL
-	return FileOperations::getFilename(getURL()).str();
+	return string(FileOperations::getFilename(getURL()));
 }
 
 } // namespace openmsx
