@@ -4,12 +4,12 @@
 #include "StringOp.hh"
 #include "serialize.hh"
 #include "serialize_stl.hh"
-#include "openmsx.hh"
 #include "stl.hh"
-#include <algorithm>
 #include <cassert>
+#include <utility>
 
 using std::string;
+using std::string_view;
 using std::vector;
 
 namespace openmsx {
@@ -20,10 +20,10 @@ const string USER_DATA    = "{{USER_DATA}}";
 const string SYSTEM_DATA  = "{{SYSTEM_DATA}}";
 
 
-static string subst(string_ref path, string_ref before, string_ref after)
+static string subst(string_view path, string_view before, string_view after)
 {
-	assert(path.starts_with(before));
-	return after + path.substr(before.size());
+	assert(StringOp::startsWith(path, before));
+	return strCat(after, path.substr(before.size()));
 }
 
 static vector<string> getPathsHelper(const vector<string>& input)
@@ -49,9 +49,8 @@ static vector<string> getPathsHelper(const vector<string>& input)
 }
 
 static string resolveHelper(const vector<string>& pathList,
-                            string_ref filename)
+                            string_view filename)
 {
-	PRT_DEBUG("Context: " << filename);
 	string filepath = FileOperations::expandCurrentDirFromDrive(filename);
 	filepath = FileOperations::expandTilde(filepath);
 	if (FileOperations::isAbsolutePath(filepath)) {
@@ -62,16 +61,20 @@ static string resolveHelper(const vector<string>& pathList,
 	for (auto& p : pathList) {
 		string name = FileOperations::join(p, filename);
 		name = FileOperations::expandTilde(name);
-		PRT_DEBUG("Context: try " << name);
 		if (FileOperations::exists(name)) {
 			return name;
 		}
 	}
 	// not found in any path
-	throw FileException(filename + " not found in this context");
+	throw FileException(filename, " not found in this context");
 }
 
-const string FileContext::resolve(string_ref filename) const
+FileContext::FileContext(vector<string>&& paths_, vector<string>&& savePaths_)
+	: paths(std::move(paths_)), savePaths(std::move(savePaths_))
+{
+}
+
+string FileContext::resolve(string_view filename) const
 {
 	vector<string> pathList = getPathsHelper(paths);
 	string result = resolveHelper(pathList, filename);
@@ -79,7 +82,7 @@ const string FileContext::resolve(string_ref filename) const
 	return result;
 }
 
-const string FileContext::resolveCreate(string_ref filename) const
+string FileContext::resolveCreate(string_view filename) const
 {
 	string result;
 	vector<string> pathList = getPathsHelper(savePaths);
@@ -89,9 +92,8 @@ const string FileContext::resolveCreate(string_ref filename) const
 		string path = pathList.front();
 		try {
 			FileOperations::mkdirp(path);
-		} catch (FileException& e) {
-			PRT_DEBUG(e.getMessage());
-			(void)&e; // Prevent warning
+		} catch (FileException&) {
+			// ignore
 		}
 		result = FileOperations::join(path, filename);
 	}
@@ -109,19 +111,27 @@ bool FileContext::isUserContext() const
 	return contains(paths, USER_DIRS);
 }
 
+template<typename Archive>
+void FileContext::serialize(Archive& ar, unsigned /*version*/)
+{
+	ar.serialize("paths",     paths,
+	             "savePaths", savePaths);
+}
+INSTANTIATE_SERIALIZE_METHODS(FileContext);
+
 ///
 
 static string backSubstSymbols(const string& path)
 {
-	string systemData = FileOperations::getSystemDataDir();
+	const string& systemData = FileOperations::getSystemDataDir();
 	if (StringOp::startsWith(path, systemData)) {
 		return subst(path, systemData, SYSTEM_DATA);
 	}
-	string userData = FileOperations::getSystemDataDir();
+	const string& userData = FileOperations::getSystemDataDir();
 	if (StringOp::startsWith(path, userData)) {
 		return subst(path, userData, SYSTEM_DATA);
 	}
-	string userDir = FileOperations::getUserOpenMSXDir();
+	const string& userDir = FileOperations::getUserOpenMSXDir();
 	if (StringOp::startsWith(path, userDir)) {
 		return subst(path, userDir, USER_OPENMSX);
 	}
@@ -129,53 +139,44 @@ static string backSubstSymbols(const string& path)
 	return path;
 }
 
-ConfigFileContext::ConfigFileContext(string_ref path,
-                                     string_ref hwDescr,
-                                     string_ref userName)
+FileContext configFileContext(string_view path, string_view hwDescr, string_view userName)
 {
-	paths = { backSubstSymbols(FileOperations::expandTilde(path)) };
-	savePaths = { FileOperations::join(
-	                  USER_OPENMSX, "persistent", hwDescr, userName) };
+	return { { backSubstSymbols(FileOperations::expandTilde(path)) },
+	         { FileOperations::join(
+	               USER_OPENMSX, "persistent", hwDescr, userName) } };
 }
 
-SystemFileContext::SystemFileContext()
+FileContext systemFileContext()
 {
-	paths = { USER_DATA, SYSTEM_DATA };
-	savePaths = { USER_DATA };
+	return { { USER_DATA, SYSTEM_DATA },
+	         { USER_DATA } };
 }
 
-PreferSystemFileContext::PreferSystemFileContext()
+FileContext preferSystemFileContext()
 {
-	paths = { SYSTEM_DATA, USER_DATA }; // first system dir
+	return { { SYSTEM_DATA, USER_DATA },  // first system dir
+	         {} };
 }
 
-UserFileContext::UserFileContext(string_ref savePath)
+FileContext userFileContext(string_view savePath)
 {
-	paths = { "", USER_DIRS };
+	vector<string> savePaths;
 	if (!savePath.empty()) {
 		savePaths = { FileOperations::join(
 		                 USER_OPENMSX, "persistent", savePath) };
 	}
+	return { { string{}, USER_DIRS }, std::move(savePaths) };
 }
 
-UserDataFileContext::UserDataFileContext(string_ref subDir)
+FileContext userDataFileContext(string_view subDir)
 {
-	paths.emplace_back();
-	paths.emplace_back(USER_OPENMSX + '/' + subDir);
+	return { { string{}, strCat(USER_OPENMSX, '/', subDir) },
+	         {} };
 }
 
-CurrentDirFileContext::CurrentDirFileContext()
+FileContext currentDirFileContext()
 {
-	paths = { "" };
+	return {{string{}}, {string{}}};
 }
 
-
-template<typename Archive>
-void FileContext::serialize(Archive& ar, unsigned /*version*/)
-{
-	ar.serialize("paths", paths);
-	ar.serialize("savePaths", savePaths);
-}
-INSTANTIATE_SERIALIZE_METHODS(FileContext);
-
 } // namespace openmsx
