@@ -47,27 +47,34 @@
 #include "StringOp.hh"
 #include "statp.hh"
 #include "unistdp.hh"
-#include "countof.hh"
+#include "one_of.hh"
+#include "ranges.hh"
+#include "strCat.hh"
 #include "build-info.hh"
-#include "AndroidApiWrapper.hh"
+#include <algorithm>
 #include <sstream>
 #include <cerrno>
 #include <cstdlib>
+#include <stdexcept>
 #include <cassert>
+#include <iterator>
 
 #ifndef _MSC_VER
 #include <dirent.h>
 #endif
 
+#if PLATFORM_ANDROID
+#include "SDL_system.h" // for GetExternalStorage stuff
+#endif
+
 using std::string;
+using std::string_view;
 
 #ifdef _WIN32
 using namespace utf8;
 #endif
 
-namespace openmsx {
-
-namespace FileOperations {
+namespace openmsx::FileOperations {
 
 #ifdef __APPLE__
 
@@ -185,27 +192,27 @@ static std::string findShareDir()
 
 #endif // __APPLE__
 
-string expandTilde(string_ref path)
+string expandTilde(string_view path)
 {
 	if (path.empty() || path[0] != '~') {
-		return path.str();
+		return string(path);
 	}
 	auto pos = path.find_first_of('/');
-	string_ref user = ((path.size() == 1) || (pos == 1))
-	                ? ""
-	                : path.substr(1, (pos == string_ref::npos) ? pos : pos - 1);
+	string_view user = ((path.size() == 1) || (pos == 1))
+	                ? string_view{}
+	                : path.substr(1, (pos == string_view::npos) ? pos : pos - 1);
 	string result = getUserHomeDir(user);
 	if (result.empty()) {
 		// failed to find homedir, return the path unchanged
-		return path.str();
+		return string(path);
 	}
-	if (pos == string_ref::npos) {
+	if (pos == string_view::npos) {
 		return result;
 	}
 	if (result.back() != '/') {
 		result += '/';
 	}
-	string_ref last = path.substr(pos + 1);
+	string_view last = path.substr(pos + 1);
 	result.append(last.data(), last.size());
 	return result;
 }
@@ -224,25 +231,51 @@ void mkdir(const string& path, mode_t mode)
 	int result = ::mkdir(path.c_str(), mode);
 #endif
 	if (result && (errno != EEXIST)) {
-		throw FileException("Error creating dir " + path);
+		throw FileException("Error creating dir ", path);
 	}
 }
 
-void mkdirp(string_ref path_)
+static bool isUNCPath(string_view path)
+{
+#ifdef _WIN32
+	return StringOp::startsWith(path, "//") || StringOp::startsWith(path, "\\\\");
+#else
+	(void)path;
+	return false;
+#endif
+}
+
+void mkdirp(string_view path_)
 {
 	if (path_.empty()) {
 		return;
 	}
-	string path = expandTilde(path_);
 
+	// We may receive platform-specific paths here, so conventionalize early
+	string path = getConventionalPath(expandTilde(path_));
+
+	// If the directory already exists, don't try to recreate it
+	if (isDirectory(path))
+		return;
+
+	// If the path is a UNC path (e.g. \\server\share) then the first two paths in the loop below will be \ and \\server
+	// If the path is an absolute path (e.g. c:\foo\bar) then the first path in the loop will be C:
+	// None of those are valid directory paths, so we skip over them and don't call mkdir.
+	// Relative paths are fine, since each segment in the path is significant.
+	int skip = isUNCPath(path) ? 2 :
+		   isAbsolutePath(path) ? 1 : 0;
 	string::size_type pos = 0;
 	do {
 		pos = path.find_first_of('/', pos + 1);
+		if (skip) {
+			skip--;
+			continue;
+		}
 		mkdir(path.substr(0, pos), 0755);
 	} while (pos != string::npos);
 
 	if (!isDirectory(path)) {
-		throw FileException("Error creating dir " + path);
+		throw FileException("Error creating dir ", path);
 	}
 }
 
@@ -282,8 +315,8 @@ int deleteRecursive(const std::string& path)
 	return SHFileOperationW(&rmdirFileop);
 }
 #elif HAVE_NFTW
-int deleteRecursive_cb(const char* fpath, const struct stat* /*sb*/,
-                       int /*typeflag*/, struct FTW* /*ftwbuf*/)
+static int deleteRecursive_cb(const char* fpath, const struct stat* /*sb*/,
+                              int /*typeflag*/, struct FTW* /*ftwbuf*/)
 {
 	return remove(fpath);
 }
@@ -354,82 +387,82 @@ void openofstream(std::ofstream& stream, const std::string& filename,
 #endif
 }
 
-string_ref getFilename(string_ref path)
+string_view getFilename(string_view path)
 {
 	auto pos = path.rfind('/');
-	if (pos == string_ref::npos) {
+	if (pos == string_view::npos) {
 		return path;
 	} else {
 		return path.substr(pos + 1);
 	}
 }
 
-string_ref getBaseName(string_ref path)
+string_view getDirName(string_view path)
 {
 	auto pos = path.rfind('/');
-	if (pos == string_ref::npos) {
-		return "";
+	if (pos == string_view::npos) {
+		return {};
 	} else {
 		return path.substr(0, pos + 1);
 	}
 }
 
-string_ref getExtension(string_ref path)
+string_view getExtension(string_view path)
 {
-	string_ref filename = getFilename(path);
+	string_view filename = getFilename(path);
 	auto pos = filename.rfind('.');
-	if (pos == string_ref::npos) {
-		return "";
+	if (pos == string_view::npos) {
+		return string_view();
 	} else {
-		return filename.substr(pos + 1);
+		return filename.substr(pos);
 	}
 }
 
-string_ref stripExtension(string_ref path)
+string_view stripExtension(string_view path)
 {
 	auto pos = path.rfind('.');
-	if (pos == string_ref::npos) {
+	if (pos == string_view::npos) {
 		return path;
 	} else {
 		return path.substr(0, pos);
 	}
 }
 
-string join(string_ref part1, string_ref part2)
+string join(string_view part1, string_view part2)
 {
 	if (part1.empty() || isAbsolutePath(part2)) {
-		return part2.str();
+		return string(part2);
 	}
 	if (part1.back() == '/') {
-		return part1 + part2;
+		return strCat(part1, part2);
 	}
-	return part1 + '/' + part2;
+	return strCat(part1, '/', part2);
 }
-string join(string_ref part1, string_ref part2, string_ref part3)
+string join(string_view part1, string_view part2, string_view part3)
 {
 	return join(part1, join(part2, part3));
 }
 
-string join(string_ref part1, string_ref part2,
-            string_ref part3, string_ref part4)
+string join(string_view part1, string_view part2,
+            string_view part3, string_view part4)
 {
 	return join(part1, join(part2, join(part3, part4)));
 }
 
-string getNativePath(string_ref path)
+string getNativePath(string_view path)
 {
-	string result = path.str();
+	string result(path);
 #ifdef _WIN32
-	replace(begin(result), end(result), '/', '\\');
+	ranges::replace(result, '/', '\\');
 #endif
 	return result;
 }
 
-string getConventionalPath(string_ref path)
+string getConventionalPath(string_view path)
 {
-	string result = path.str();
+	string result(path);
 #ifdef _WIN32
-	replace(begin(result), end(result), '\\', '/');
+	ranges::replace(result, '\\', '/');
 #endif
 	return result;
 }
@@ -453,21 +486,22 @@ string getCurrentWorkingDirectory()
 	return buf;
 }
 
-string getAbsolutePath(string_ref path)
+string getAbsolutePath(string_view path)
 {
 	// In rare cases getCurrentWorkingDirectory() can throw,
 	// so only call it when really necessary.
 	if (isAbsolutePath(path)) {
-		return path.str();
+		return string(path);
 	}
 	string currentDir = getCurrentWorkingDirectory();
 	return join(currentDir, path);
 }
 
-bool isAbsolutePath(string_ref path)
+bool isAbsolutePath(string_view path)
 {
+	if (isUNCPath(path)) return true;
 #ifdef _WIN32
-	if ((path.size() >= 3) && (path[1] == ':') && (path[2] == '/')) {
+	if ((path.size() >= 3) && (path[1] == ':') && (path[2] == one_of('/', '\\'))) {
 		char drive = tolower(path[0]);
 		if (('a' <= drive) && (drive <= 'z')) {
 			return true;
@@ -477,15 +511,15 @@ bool isAbsolutePath(string_ref path)
 	return !path.empty() && (path[0] == '/');
 }
 
-string getUserHomeDir(string_ref username)
+string getUserHomeDir(string_view username)
 {
 #ifdef _WIN32
 	(void)(&username); // ignore parameter, avoid warning
 
 	wchar_t bufW[MAXPATHLEN + 1];
 	if (!SHGetSpecialFolderPathW(nullptr, bufW, CSIDL_PERSONAL, TRUE)) {
-		throw FatalError(StringOp::Builder() <<
-			"SHGetSpecialFolderPathW failed: " << GetLastError());
+		throw FatalError(
+			"SHGetSpecialFolderPathW failed: ", GetLastError());
 	}
 
 	return getConventionalPath(utf16to8(bufW));
@@ -498,12 +532,12 @@ string getUserHomeDir(string_ref username)
 			pw = getpwuid(getuid());
 		}
 	} else {
-		pw = getpwnam(username.str().c_str());
+		pw = getpwnam(string(username).c_str());
 	}
 	if (pw) {
 		dir = pw->pw_dir;
 	}
-	return dir ? dir : "";
+	return dir ? dir : string{};
 #endif
 }
 
@@ -512,7 +546,9 @@ const string& getUserOpenMSXDir()
 #ifdef _WIN32
 	static const string OPENMSX_DIR = expandTilde("~/openMSX");
 #elif PLATFORM_ANDROID
-	static const string OPENMSX_DIR = AndroidApiWrapper::getStorageDirectory() + "/openMSX";
+	// TODO: do something to query whether the storage is available
+	// via SDL_AndroidGetExternalStorageState
+	static const string OPENMSX_DIR = strCat(SDL_AndroidGetExternalStoragePath(), "/openMSX");
 #else
 	static const string OPENMSX_DIR = expandTilde("~/.openMSX");
 #endif
@@ -536,10 +572,10 @@ string getSystemDataDir()
 	string newValue;
 #ifdef _WIN32
 	wchar_t bufW[MAXPATHLEN + 1];
-	int res = GetModuleFileNameW(nullptr, bufW, countof(bufW));
+	int res = GetModuleFileNameW(nullptr, bufW, std::size(bufW));
 	if (!res) {
-		throw FatalError(StringOp::Builder() <<
-			"Cannot detect openMSX directory. GetModuleFileNameW failed: " <<
+		throw FatalError(
+			"Cannot detect openMSX directory. GetModuleFileNameW failed: ",
 			GetLastError());
 	}
 
@@ -562,16 +598,16 @@ string getSystemDataDir()
 }
 
 #ifdef _WIN32
-bool driveExists(char driveLetter)
+static bool driveExists(char driveLetter)
 {
 	char buf[] = { driveLetter, ':', 0 };
 	return GetFileAttributesA(buf) != INVALID_FILE_ATTRIBUTES;
 }
 #endif
 
-string expandCurrentDirFromDrive(string_ref path)
+string expandCurrentDirFromDrive(string_view path)
 {
-	string result = path.str();
+	string result(path);
 #ifdef _WIN32
 	if (((path.size() == 2) && (path[1] == ':')) ||
 		((path.size() >= 3) && (path[1] == ':') && (path[2] != '/'))) {
@@ -586,7 +622,7 @@ string expandCurrentDirFromDrive(string_ref path)
 					result += '/';
 				}
 				if (path.size() > 2) {
-					string_ref tmp = path.substr(2);
+					string_view tmp = path.substr(2);
 					result.append(tmp.data(), tmp.size());
 				}
 			}
@@ -596,7 +632,7 @@ string expandCurrentDirFromDrive(string_ref path)
 	return result;
 }
 
-bool getStat(string_ref filename_, Stat& st)
+bool getStat(string_view filename_, Stat& st)
 {
 	string filename = expandTilde(filename_);
 	// workaround for VC++: strip trailing slashes (but keep it if it's the
@@ -604,7 +640,7 @@ bool getStat(string_ref filename_, Stat& st)
 	auto pos = filename.find_last_not_of('/');
 	if (pos == string::npos) {
 		// string was either empty or a (sequence of) '/' character(s)
-		filename = filename.empty() ? "" : "/";
+		filename = filename.empty() ? string{} : "/";
 	} else {
 		filename.resize(pos + 1);
 	}
@@ -619,7 +655,7 @@ bool isRegularFile(const Stat& st)
 {
 	return S_ISREG(st.st_mode);
 }
-bool isRegularFile(string_ref filename)
+bool isRegularFile(string_view filename)
 {
 	Stat st;
 	return getStat(filename, st) && isRegularFile(st);
@@ -630,13 +666,13 @@ bool isDirectory(const Stat& st)
 	return S_ISDIR(st.st_mode);
 }
 
-bool isDirectory(string_ref directory)
+bool isDirectory(string_view directory)
 {
 	Stat st;
 	return getStat(directory, st) && isDirectory(st);
 }
 
-bool exists(string_ref filename)
+bool exists(string_view filename)
 {
 	Stat st; // dummy
 	return getStat(filename, st);
@@ -647,32 +683,33 @@ time_t getModificationDate(const Stat& st)
 	return st.st_mtime;
 }
 
-static int getNextNum(dirent* d, string_ref prefix, string_ref extension,
-                      unsigned nofdigits)
+static unsigned getNextNum(dirent* d, string_view prefix, string_view extension,
+                           unsigned nofdigits)
 {
 	auto extensionLen = extension.size();
 	auto prefixLen = prefix.size();
-	string_ref name(d->d_name);
+	string_view name(d->d_name);
 
 	if ((name.size() != (prefixLen + nofdigits + extensionLen)) ||
 	    (name.substr(0, prefixLen) != prefix) ||
 	    (name.substr(prefixLen + nofdigits, extensionLen) != extension)) {
 		return 0;
 	}
-	string_ref num = name.substr(prefixLen, nofdigits);
-	string_ref::size_type idx;
-	unsigned long n = stoul(num, &idx, 10);
-	return (idx == num.size()) ? n : 0;
+	try {
+		return StringOp::fast_stou(name.substr(prefixLen, nofdigits));
+	} catch (std::invalid_argument&) {
+		return 0;
+	}
 }
 
 string getNextNumberedFileName(
-	string_ref directory, string_ref prefix, string_ref extension)
+	string_view directory, string_view prefix, string_view extension)
 {
 	const unsigned nofdigits = 4;
 
-	int max_num = 0;
+	unsigned max_num = 0;
 
-	string dirName = getUserOpenMSXDir() + '/' + directory;
+	string dirName = strCat(getUserOpenMSXDir(), '/', directory);
 	try {
 		mkdirp(dirName);
 	} catch (FileException&) {
@@ -680,7 +717,7 @@ string getNextNumberedFileName(
 	}
 
 	ReadDir dir(dirName);
-	while (dirent* d = dir.getEntry()) {
+	while (auto* d = dir.getEntry()) {
 		max_num = std::max(max_num, getNextNum(d, prefix, extension, nofdigits));
 	}
 
@@ -693,20 +730,20 @@ string getNextNumberedFileName(
 }
 
 string parseCommandFileArgument(
-	string_ref argument, string_ref directory,
-	string_ref prefix,   string_ref extension)
+	string_view argument, string_view directory,
+	string_view prefix,   string_view extension)
 {
 	if (argument.empty()) {
 		// directory is also created when needed
 		return getNextNumberedFileName(directory, prefix, extension);
 	}
 
-	string filename = argument.str();
-	if (getBaseName(filename).empty()) {
+	string filename(argument);
+	if (getDirName(filename).empty()) {
 		// no dir given, use standard dir (and create it)
-		string dir = getUserOpenMSXDir() + '/' + directory;
+		string dir = strCat(getUserOpenMSXDir(), '/', directory);
 		mkdirp(dir);
-		filename = dir + '/' + filename;
+		filename = strCat(dir, '/', filename);
 	} else {
 		filename = expandTilde(filename);
 	}
@@ -737,8 +774,7 @@ string getTempDir()
 			return utf16to8(bufW);
 		}
 	}
-	throw FatalError(StringOp::Builder() <<
-		"GetTempPathW failed: " << GetLastError());
+	throw FatalError("GetTempPathW failed: ", GetLastError());
 #elif PLATFORM_ANDROID
 	string result = getSystemDataDir() + "/tmp";
 	return result;
@@ -760,14 +796,15 @@ FILE_t openUniqueFile(const std::string& directory, std::string& filename)
 	std::wstring directoryW = utf8to16(directory);
 	wchar_t filenameW[MAX_PATH];
 	if (!GetTempFileNameW(directoryW.c_str(), L"msx", 0, filenameW)) {
-		throw FileException(StringOp::Builder() <<
-			"GetTempFileNameW failed: " << GetLastError());
+		throw FileException("GetTempFileNameW failed: ", GetLastError());
 	}
 	filename = utf16to8(filenameW);
 	return FILE_t(_wfopen(filenameW, L"wb"));
 #else
 	filename = directory + "/XXXXXX";
+	auto oldMask = umask(S_IRWXO | S_IRWXG);
 	int fd = mkstemp(const_cast<char*>(filename.c_str()));
+	umask(oldMask);
 	if (fd == -1) {
 		throw FileException("Coundn't get temp file name");
 	}
@@ -775,6 +812,4 @@ FILE_t openUniqueFile(const std::string& directory, std::string& filename)
 #endif
 }
 
-} // namespace FileOperations
-
-} // namespace openmsx
+} // namespace openmsx::FileOperations
