@@ -3,36 +3,49 @@
 #include "Filename.hh"
 #include "FileOperations.hh"
 #include "FileException.hh"
-#include "StringOp.hh"
 #include "build-info.hh"
 #include <cstdio>
 #include <cassert>
 
-using std::string;
-
 namespace openmsx {
 
-LocalFileReference::LocalFileReference(const Filename& filename)
-{
-	init(filename.getResolved());
-}
-
-LocalFileReference::LocalFileReference(const string& url)
-{
-	init(url);
-}
-
 LocalFileReference::LocalFileReference(File& file)
 {
 	init(file);
 }
 
-void LocalFileReference::init(const string& url)
+LocalFileReference::LocalFileReference(const std::string& filename)
 {
-	File file(url);
+	File file(filename);
 	init(file);
 }
 
+LocalFileReference::LocalFileReference(const Filename& filename)
+	: LocalFileReference(filename.getResolved())
+{
+}
+
+LocalFileReference::LocalFileReference(LocalFileReference&& other)
+	: tmpFile(std::move(other.tmpFile))
+	, tmpDir (std::move(other.tmpDir ))
+{
+	other.tmpDir.clear();
+}
+
+LocalFileReference& LocalFileReference::operator=(LocalFileReference&& other)
+{
+	cleanup();
+	tmpFile = std::move(other.tmpFile);
+	tmpDir  = std::move(other.tmpDir);
+	other.tmpDir.clear();
+	return *this;
+}
+
+LocalFileReference::~LocalFileReference()
+{
+	cleanup();
+}
+
 void LocalFileReference::init(File& file)
 {
 	tmpFile = file.getLocalReference();
@@ -48,28 +61,26 @@ void LocalFileReference::init(File& file)
 	tmpDir = FileOperations::getTempDir() + FileOperations::nativePathSeparator + "openmsx";
 #else
 	// TODO - why not just use getTempDir()?
-	tmpDir = StringOp::Builder() << "/tmp/openmsx." << int(getpid());
+	tmpDir = strCat("/tmp/openmsx.", int(getpid()));
 #endif
 	// it's possible this directory already exists, in that case the
 	// following function does nothing
 	FileOperations::mkdirp(tmpDir);
 
 	// create temp file
-	FILE* fp = FileOperations::openUniqueFile(tmpDir, tmpFile);
+	auto fp = FileOperations::openUniqueFile(tmpDir, tmpFile);
 	if (!fp) {
 		throw FileException("Couldn't create temp file");
 	}
 
 	// write temp file
-	size_t size;
-	const byte* buf = file.mmap(size);
-	if (fwrite(buf, 1, size, fp) != size) {
+	auto mmap = file.mmap();
+	if (fwrite(mmap.data(), 1, mmap.size(), fp.get()) != mmap.size()) {
 		throw FileException("Couldn't write temp file");
 	}
-	fclose(fp);
 }
 
-LocalFileReference::~LocalFileReference()
+void LocalFileReference::cleanup()
 {
 	if (!tmpDir.empty()) {
 		FileOperations::unlink(tmpFile);
@@ -79,7 +90,7 @@ LocalFileReference::~LocalFileReference()
 	}
 }
 
-const string LocalFileReference::getFilename() const
+const std::string& LocalFileReference::getFilename() const
 {
 	assert(!tmpFile.empty());
 	return tmpFile;
