@@ -9,11 +9,9 @@ ZipFileAdapter::ZipFileAdapter(std::unique_ptr<FileBase> file_)
 {
 }
 
-void ZipFileAdapter::decompress(FileBase& file, Decompressed& decompressed)
+void ZipFileAdapter::decompress(FileBase& f, Decompressed& d)
 {
-	size_t size;
-	const byte* data = file.mmap(size);
-	ZlibInflate zlib(data, size);
+	ZlibInflate zlib(f.mmap());
 
 	if (zlib.get32LE() != 0x04034B50) {
 		throw FileException("Invalid ZIP file");
@@ -34,10 +32,10 @@ void ZipFileAdapter::decompress(FileBase& file, Decompressed& decompressed)
 	unsigned origSize = zlib.get32LE(); // uncompressed size
 	unsigned filenameLen = zlib.get16LE(); // filename length
 	unsigned extraFieldLen = zlib.get16LE(); // extra field length
-	decompressed.originalName = zlib.getString(filenameLen); // original filename
+	d.originalName = zlib.getString(filenameLen); // original filename
 	zlib.skip(extraFieldLen); // skip "extra field"
 
-	zlib.inflate(decompressed.buf, origSize);
+	d.size = zlib.inflate(d.buf, origSize);
 }
 
 } // namespace openmsx
