@@ -1,23 +1,22 @@
 #include "ZlibInflate.hh"
 #include "FileException.hh"
 #include "MemBuffer.hh"
-#include "StringOp.hh"
 #include <limits>
 
 namespace openmsx {
 
-ZlibInflate::ZlibInflate(const byte* input, size_t inputLen_)
+ZlibInflate::ZlibInflate(span<uint8_t> input)
 {
-	if (inputLen_ > std::numeric_limits<decltype(s.avail_in)>::max()) {
+	if (input.size() > std::numeric_limits<decltype(s.avail_in)>::max()) {
 		throw FileException(
 			"Error while decompressing: input file too big");
 	}
-	auto inputLen = static_cast<decltype(s.avail_in)>(inputLen_);
+	auto inputLen = static_cast<decltype(s.avail_in)>(input.size());
 
 	s.zalloc = nullptr;
 	s.zfree  = nullptr;
 	s.opaque = nullptr;
-	s.next_in  = const_cast<byte*>(input);
+	s.next_in  = const_cast<uint8_t*>(input.data());
 	s.avail_in = inputLen;
 	wasInit = false;
 }
@@ -36,7 +35,7 @@ void ZlibInflate::skip(size_t num)
 	}
 }
 
-byte ZlibInflate::getByte()
+uint8_t ZlibInflate::getByte()
 {
 	if (s.avail_in <= 0) {
 		throw FileException(
@@ -80,17 +79,18 @@ std::string ZlibInflate::getCString()
 	return result;
 }
 
-void ZlibInflate::inflate(MemBuffer<byte>& output, size_t sizeHint)
+size_t ZlibInflate::inflate(MemBuffer<uint8_t>& output, size_t sizeHint)
 {
 	int initErr = inflateInit2(&s, -MAX_WBITS);
 	if (initErr != Z_OK) {
-		throw FileException(StringOp::Builder()
-			<< "Error initializing inflate struct: " << zError(initErr));
+		throw FileException(
+			"Error initializing inflate struct: ", zError(initErr));
 	}
 	wasInit = true;
 
-	output.resize(sizeHint);
-	s.avail_out = uInt(output.size()); // TODO overflow?
+	size_t outSize = sizeHint;
+	output.resize(outSize);
+	s.avail_out = uInt(outSize); // TODO overflow?
 	while (true) {
 		s.next_out = output.data() + s.total_out;
 		int err = ::inflate(&s, Z_NO_FLUSH);
@@ -98,16 +98,17 @@ void ZlibInflate::inflate(MemBuffer<byte>& output, size_t sizeHint)
 			break;
 		}
 		if (err != Z_OK) {
-			throw FileException(StringOp::Builder()
-				<< "Error decompressing gzip: " << zError(err));
+			throw FileException("Error decompressing gzip: ", zError(err));
 		}
-		auto oldSize = output.size();
-		output.resize(oldSize * 2); // double buffer size
-		s.avail_out = uInt(output.size() - oldSize); // TODO overflow?
+		auto oldSize = outSize;
+		outSize = oldSize * 2; // double buffer size
+		output.resize(outSize);
+		s.avail_out = uInt(outSize - oldSize); // TODO overflow?
 	}
 
 	// set actual size
 	output.resize(s.total_out);
+	return s.total_out;
 }
 
 } // namespace openmsx
