@@ -1,5 +1,3 @@
-// $Id: AbstractIDEDevice.cc 12026 2011-03-15 20:23:24Z m9710797 $
-
 #include "AbstractIDEDevice.hh"
 #include "MSXMotherBoard.hh"
 #include "LedStatus.hh"
@@ -25,10 +23,6 @@ AbstractIDEDevice::AbstractIDEDevice(MSXMotherBoard& motherBoard_)
 	transferCount = 0;
 }
 
-AbstractIDEDevice::~AbstractIDEDevice()
-{
-}
-
 byte AbstractIDEDevice::diagnostic()
 {
 	// The Execute Device Diagnostic command is executed by both devices in
@@ -172,7 +166,7 @@ word AbstractIDEDevice::readData(EmuTime::param /*time*/)
 		// no read in progress
 		return 0x7F7F;
 	}
-	assert(transferIdx < sizeof(buffer));
+	assert((transferIdx + 1) < sizeof(buffer));
 	word result = (buffer[transferIdx + 0] << 0) +
 	              (buffer[transferIdx + 1] << 8);
 	transferIdx += 2;
@@ -206,7 +200,7 @@ void AbstractIDEDevice::writeData(word value, EmuTime::param /*time*/)
 		// no write in progress
 		return;
 	}
-	assert(transferIdx < sizeof(buffer));
+	assert((transferIdx + 1) < sizeof(buffer));
 	buffer[transferIdx + 0] = value & 0xFF;
 	buffer[transferIdx + 1] = value >> 8;
 	transferIdx += 2;
@@ -263,7 +257,7 @@ void AbstractIDEDevice::setInterruptReason(byte value)
 	sectorCountReg = value;
 }
 
-unsigned AbstractIDEDevice::getByteCount()
+unsigned AbstractIDEDevice::getByteCount() const
 {
 	return cylinderLowReg | (cylinderHighReg << 8);
 }
@@ -344,7 +338,7 @@ void AbstractIDEDevice::executeCommand(byte cmd)
 	}
 }
 
-byte* AbstractIDEDevice::startShortReadTransfer(unsigned count)
+AlignedBuffer& AbstractIDEDevice::startShortReadTransfer(unsigned count)
 {
 	assert(count <= sizeof(buffer));
 	assert((count & 1) == 0);
@@ -431,18 +425,19 @@ static void writeIdentifyString(byte* p, unsigned len, std::string s)
 	}
 }
 
-void AbstractIDEDevice::createIdentifyBlock(byte* buffer)
+void AbstractIDEDevice::createIdentifyBlock(AlignedBuffer& buf)
 {
 	// According to the spec, the combination of model and serial should be
 	// unique. But I don't know any MSX software that cares about this.
-	writeIdentifyString(&buffer[10 * 2], 10, "s00000001"); // serial
-	writeIdentifyString(&buffer[23 * 2], 4,
+	writeIdentifyString(&buf[10 * 2], 10, "s00000001"); // serial
+	writeIdentifyString(&buf[23 * 2], 4,
 		// Use openMSX version as firmware revision, because most of our
 		// IDE emulation code is in fact emulating the firmware.
-		Version::RELEASE ? 'v' + Version::VERSION : 'd' + Version::REVISION);
-	writeIdentifyString(&buffer[27 * 2], 20, getDeviceName()); // model
+		Version::RELEASE ? strCat('v', Version::VERSION)
+		                 : strCat('d', Version::REVISION));
+	writeIdentifyString(&buf[27 * 2], 20, getDeviceName()); // model
 
-	fillIdentifyBlock(buffer);
+	fillIdentifyBlock(buf);
 }
 
 
@@ -451,22 +446,22 @@ void AbstractIDEDevice::serialize(Archive& ar, unsigned /*version*/)
 {
 	// no need to serialize IDEDevice base class
 	ar.serialize_blob("buffer", buffer, sizeof(buffer));
-	ar.serialize("transferIdx", transferIdx);
-	ar.serialize("bufferLeft", bufferLeft);
-	ar.serialize("transferCount", transferCount);
-	ar.serialize("errorReg", errorReg);
-	ar.serialize("sectorCountReg", sectorCountReg);
-	ar.serialize("sectorNumReg", sectorNumReg);
-	ar.serialize("cylinderLowReg", cylinderLowReg);
-	ar.serialize("cylinderHighReg", cylinderHighReg);
-	ar.serialize("devHeadReg", devHeadReg);
-	ar.serialize("statusReg", statusReg);
-	ar.serialize("featureReg", featureReg);
+	ar.serialize("transferIdx",     transferIdx,
+	             "bufferLeft",      bufferLeft,
+	             "transferCount",   transferCount,
+	             "errorReg",        errorReg,
+	             "sectorCountReg",  sectorCountReg,
+	             "sectorNumReg",    sectorNumReg,
+	             "cylinderLowReg",  cylinderLowReg,
+	             "cylinderHighReg", cylinderHighReg,
+	             "devHeadReg",      devHeadReg,
+	             "statusReg",       statusReg,
+	             "featureReg",      featureReg);
 	bool transferIdentifyBlock = false; // remove on next version increment
 	                                    // no need to break bw-compat now
-	ar.serialize("transferIdentifyBlock", transferIdentifyBlock);
-	ar.serialize("transferRead", transferRead);
-	ar.serialize("transferWrite", transferWrite);
+	ar.serialize("transferIdentifyBlock", transferIdentifyBlock,
+	             "transferRead",          transferRead,
+	             "transferWrite",         transferWrite);
 }
 INSTANTIATE_SERIALIZE_METHODS(AbstractIDEDevice);
 
