@@ -13,38 +13,37 @@ using std::vector;
 
 // class HDCommand
 
-HDCommand::HDCommand(CommandController& commandController,
-                     StateChangeDistributor& stateChangeDistributor,
-                     Scheduler& scheduler, HD& hd_,
+HDCommand::HDCommand(CommandController& commandController_,
+                     StateChangeDistributor& stateChangeDistributor_,
+                     Scheduler& scheduler_, HD& hd_,
                      BooleanSetting& powerSetting_)
-	: RecordedCommand(commandController, stateChangeDistributor,
-	                  scheduler, hd_.getName())
+	: RecordedCommand(commandController_, stateChangeDistributor_,
+	                  scheduler_, hd_.getName())
 	, hd(hd_)
 	, powerSetting(powerSetting_)
 {
 }
 
-void HDCommand::execute(array_ref<TclObject> tokens, TclObject& result,
+void HDCommand::execute(span<const TclObject> tokens, TclObject& result,
                         EmuTime::param /*time*/)
 {
 	if (tokens.size() == 1) {
-		result.addListElement(hd.getName() + ':');
-		result.addListElement(hd.getImageName().getResolved());
+		result.addListElement(hd.getName() + ':',
+		                      hd.getImageName().getResolved());
 
 		if (hd.isWriteProtected()) {
-			TclObject options;
-			options.addListElement("readonly");
+			TclObject options = makeTclList("readonly");
 			result.addListElement(options);
 		}
 	} else if ((tokens.size() == 2) ||
-	           ((tokens.size() == 3) && tokens[1].getString() == "insert")) {
+	           ((tokens.size() == 3) && tokens[1] == "insert")) {
 		if (powerSetting.getBoolean()) {
 			throw CommandException(
 				"Can only change hard disk image when MSX "
 				"is powered down.");
 		}
 		int fileToken = 1;
-		if (tokens[1].getString() == "insert") {
+		if (tokens[1] == "insert") {
 			if (tokens.size() > 2) {
 				fileToken = 2;
 			} else {
@@ -53,14 +52,14 @@ void HDCommand::execute(array_ref<TclObject> tokens, TclObject& result,
 			}
 		}
 		try {
-			Filename filename(tokens[fileToken].getString().str(),
-			                  UserFileContext());
+			Filename filename(string(tokens[fileToken].getString()),
+			                  userFileContext());
 			hd.switchImage(filename);
 			// Note: the diskX command doesn't do this either,
 			// so this has not been converted to TclObject style here
 			// return filename;
 		} catch (FileException& e) {
-			throw CommandException("Can't change hard disk image: " +
+			throw CommandException("Can't change hard disk image: ",
 			                       e.getMessage());
 		}
 	} else {
@@ -79,10 +78,10 @@ void HDCommand::tabCompletion(vector<string>& tokens) const
 	if (tokens.size() < 3) {
 		extra = { "insert" };
 	}
-	completeFileName(tokens, UserFileContext(), extra);
+	completeFileName(tokens, userFileContext(), extra);
 }
 
-bool HDCommand::needRecord(array_ref<TclObject> tokens) const
+bool HDCommand::needRecord(span<const TclObject> tokens) const
 {
 	return tokens.size() > 1;
 }
