@@ -6,6 +6,7 @@
 #include "DiskManipulator.hh"
 #include "endian.hh"
 #include "serialize.hh"
+#include "strCat.hh"
 #include <cassert>
 
 namespace openmsx {
@@ -15,7 +16,9 @@ IDEHD::IDEHD(const DeviceConfig& config)
 	, AbstractIDEDevice(config.getMotherBoard())
 	, diskManipulator(config.getReactor().getDiskManipulator())
 {
-	diskManipulator.registerDrive(*this, config.getMotherBoard().getMachineID() + "::");
+	transferSectorNumber = 0; // avoid UMR is serialize()
+	diskManipulator.registerDrive(
+		*this, strCat(config.getMotherBoard().getMachineID(), "::"));
 }
 
 IDEHD::~IDEHD()
@@ -34,34 +37,34 @@ const std::string& IDEHD::getDeviceName()
 	return NAME;
 }
 
-void IDEHD::fillIdentifyBlock(AlignedBuffer& buffer)
+void IDEHD::fillIdentifyBlock(AlignedBuffer& buf)
 {
 	auto totalSectors = getNbSectors();
 	uint16_t heads = 16;
 	uint16_t sectors = 32;
-	uint16_t cylinders = uint16_t(totalSectors / (heads * sectors)); // TODO overflow?
-	Endian::writeL16(&buffer[1 * 2], cylinders);
-	Endian::writeL16(&buffer[3 * 2], heads);
-	Endian::writeL16(&buffer[6 * 2], sectors);
+	auto cylinders = uint16_t(totalSectors / (heads * sectors)); // TODO overflow?
+	Endian::writeL16(&buf[1 * 2], cylinders);
+	Endian::writeL16(&buf[3 * 2], heads);
+	Endian::writeL16(&buf[6 * 2], sectors);
 
-	buffer[47 * 2 + 0] = 16; // max sector transfer per interrupt
-	buffer[47 * 2 + 1] = 0x80; // specced value
+	buf[47 * 2 + 0] = 16; // max sector transfer per interrupt
+	buf[47 * 2 + 1] = 0x80; // specced value
 
 	// .... 1...: IORDY supported (hardware signal used by PIO modes >3)
 	// .... ..1.: LBA supported
-	buffer[49 * 2 + 1] = 0x0A;
+	buf[49 * 2 + 1] = 0x0A;
 
 	// TODO check for overflow
-	Endian::writeL32(&buffer[60 * 2], unsigned(totalSectors));
+	Endian::writeL32(&buf[60 * 2], unsigned(totalSectors));
 }
 
-unsigned IDEHD::readBlockStart(AlignedBuffer& buffer, unsigned count)
+unsigned IDEHD::readBlockStart(AlignedBuffer& buf, unsigned count)
 {
 	try {
 		assert(count >= 512);
 		(void)count; // avoid warning
 		readSector(transferSectorNumber,
-		           *aligned_cast<SectorBuffer*>(buffer));
+		           *aligned_cast<SectorBuffer*>(buf));
 		++transferSectorNumber;
 		return 512;
 	} catch (MSXException&) {
@@ -70,14 +73,14 @@ unsigned IDEHD::readBlockStart(AlignedBuffer& buffer, unsigned count)
 	}
 }
 
-void IDEHD::writeBlockComplete(AlignedBuffer& buffer, unsigned count)
+void IDEHD::writeBlockComplete(AlignedBuffer& buf, unsigned count)
 {
 	try {
 		assert((count % 512) == 0);
 		unsigned num = count / 512;
 		for (unsigned i = 0; i < num; ++i) {
 			writeSector(transferSectorNumber++,
-			            *aligned_cast<SectorBuffer*>(buffer + 512 * i));
+			            *aligned_cast<SectorBuffer*>(buf + 512 * i));
 		}
 	} catch (MSXException&) {
 		abortWriteTransfer(UNC);
@@ -86,9 +89,17 @@ void IDEHD::writeBlockComplete(AlignedBuffer& buffer, unsigned count)
 
 void IDEHD::executeCommand(byte cmd)
 {
+	if (0x10 <= cmd && cmd < 0x20) {
+		// Recalibrate
+		setError(0);
+		setByteCount(0);
+		return;
+	}
 	switch (cmd) {
 	case 0x20: // Read Sector
-	case 0x30: { // Write Sector
+	case 0x21: // Read Sector without Retry
+	case 0x30: // Write Sector
+	case 0x31: { // Write Sector without Retry
 		unsigned sectorNumber = getSectorNumber();
 		unsigned numSectors = getNumSectors();
 		if ((sectorNumber + numSectors) > getNbSectors()) {
@@ -98,7 +109,7 @@ void IDEHD::executeCommand(byte cmd)
 			break;
 		}
 		transferSectorNumber = sectorNumber;
-		if (cmd == 0x20) {
+		if (cmd < 0x30) {
 			startLongReadTransfer(numSectors * 512);
 		} else {
 			startWriteTransfer(numSectors * 512);
