@@ -20,80 +20,78 @@
 #include "DeviceConfig.hh"
 #include "XMLElement.hh"
 #include "MSXException.hh"
-#include "StringOp.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include <cassert>
 #include <string>
 #include <cstring>
+#include <memory>
 
 using std::string;
 
 namespace openmsx {
 
-static const byte REG_BDID =  0;   // Bus Device ID        (r/w)
-static const byte REG_SCTL =  1;   // Spc Control          (r/w)
-static const byte REG_SCMD =  2;   // Command              (r/w)
-static const byte REG_OPEN =  3;   //                      (open)
-static const byte REG_INTS =  4;   // Interrupt Sense      (r/w)
-static const byte REG_PSNS =  5;   // Phase Sense          (r)
-static const byte REG_SDGC =  5;   // SPC Diag. Control    (w)
-static const byte REG_SSTS =  6;   // SPC SCSI::STATUS           (r)
-static const byte REG_SERR =  7;   // SPC Error SCSI::STATUS     (r/w?)
-static const byte REG_PCTL =  8;   // Phase Control        (r/w)
-static const byte REG_MBC  =  9;   // Modified Byte Counter(r)
-static const byte REG_DREG = 10;   // Data Register        (r/w)
-static const byte REG_TEMP = 11;   // Temporary Register   (r/w)
-                                   // Another value is maintained respec-
-                                   // tively for writing and for reading
-static const byte REG_TCH  = 12;   // Transfer Counter High(r/w)
-static const byte REG_TCM  = 13;   // Transfer Counter Mid (r/w)
-static const byte REG_TCL  = 14;   // Transfer Counter Low (r/w)
+constexpr byte REG_BDID =  0;   // Bus Device ID        (r/w)
+constexpr byte REG_SCTL =  1;   // Spc Control          (r/w)
+constexpr byte REG_SCMD =  2;   // Command              (r/w)
+constexpr byte REG_OPEN =  3;   //                      (open)
+constexpr byte REG_INTS =  4;   // Interrupt Sense      (r/w)
+constexpr byte REG_PSNS =  5;   // Phase Sense          (r)
+constexpr byte REG_SDGC =  5;   // SPC Diag. Control    (w)
+constexpr byte REG_SSTS =  6;   // SPC SCSI::STATUS           (r)
+constexpr byte REG_SERR =  7;   // SPC Error SCSI::STATUS     (r/w?)
+constexpr byte REG_PCTL =  8;   // Phase Control        (r/w)
+constexpr byte REG_MBC  =  9;   // Modified Byte Counter(r)
+constexpr byte REG_DREG = 10;   // Data Register        (r/w)
+constexpr byte REG_TEMP = 11;   // Temporary Register   (r/w)
+                                // Another value is maintained respec-
+                                // tively for writing and for reading
+constexpr byte REG_TCH  = 12;   // Transfer Counter High(r/w)
+constexpr byte REG_TCM  = 13;   // Transfer Counter Mid (r/w)
+constexpr byte REG_TCL  = 14;   // Transfer Counter Low (r/w)
 
-static const byte REG_TEMPWR = 13; // (TEMP register preservation place for writing)
-static const byte FIX_PCTL   = 14; // (REG_PCTL & 7)
+constexpr byte REG_TEMPWR = 13; // (TEMP register preservation place for writing)
+constexpr byte FIX_PCTL   = 14; // (REG_PCTL & 7)
 
-static const byte PSNS_IO  = 0x01;
-static const byte PSNS_CD  = 0x02;
-static const byte PSNS_MSG = 0x04;
-static const byte PSNS_BSY = 0x08;
-static const byte PSNS_SEL = 0x10;
-static const byte PSNS_ATN = 0x20;
-static const byte PSNS_ACK = 0x40;
-static const byte PSNS_REQ = 0x80;
+constexpr byte PSNS_IO  = 0x01;
+constexpr byte PSNS_CD  = 0x02;
+constexpr byte PSNS_MSG = 0x04;
+constexpr byte PSNS_BSY = 0x08;
+constexpr byte PSNS_SEL = 0x10;
+constexpr byte PSNS_ATN = 0x20;
+constexpr byte PSNS_ACK = 0x40;
+constexpr byte PSNS_REQ = 0x80;
 
-static const byte PSNS_SELECTION = PSNS_SEL;
-static const byte PSNS_COMMAND   = PSNS_CD;
-static const byte PSNS_DATAIN    = PSNS_IO;
-static const byte PSNS_DATAOUT   = 0;
-static const byte PSNS_STATUS    = PSNS_CD  | PSNS_IO;
-static const byte PSNS_MSGIN     = PSNS_MSG | PSNS_CD | PSNS_IO;
-static const byte PSNS_MSGOUT    = PSNS_MSG | PSNS_CD;
+constexpr byte PSNS_SELECTION = PSNS_SEL;
+constexpr byte PSNS_COMMAND   = PSNS_CD;
+constexpr byte PSNS_DATAIN    = PSNS_IO;
+constexpr byte PSNS_DATAOUT   = 0;
+constexpr byte PSNS_STATUS    = PSNS_CD  | PSNS_IO;
+constexpr byte PSNS_MSGIN     = PSNS_MSG | PSNS_CD | PSNS_IO;
+constexpr byte PSNS_MSGOUT    = PSNS_MSG | PSNS_CD;
 
-static const byte INTS_ResetCondition  = 0x01;
-static const byte INTS_SPC_HardError   = 0x02;
-static const byte INTS_TimeOut         = 0x04;
-static const byte INTS_ServiceRequited = 0x08;
-static const byte INTS_CommandComplete = 0x10;
-static const byte INTS_Disconnected    = 0x20;
-static const byte INTS_ReSelected      = 0x40;
-static const byte INTS_Selected        = 0x80;
+constexpr byte INTS_ResetCondition  = 0x01;
+constexpr byte INTS_SPC_HardError   = 0x02;
+constexpr byte INTS_TimeOut         = 0x04;
+constexpr byte INTS_ServiceRequited = 0x08;
+constexpr byte INTS_CommandComplete = 0x10;
+constexpr byte INTS_Disconnected    = 0x20;
+constexpr byte INTS_ReSelected      = 0x40;
+constexpr byte INTS_Selected        = 0x80;
 
-static const byte CMD_BusRelease    = 0x00;
-static const byte CMD_Select        = 0x20;
-static const byte CMD_ResetATN      = 0x40;
-static const byte CMD_SetATN        = 0x60;
-static const byte CMD_Transfer      = 0x80;
-static const byte CMD_TransferPause = 0xA0;
-static const byte CMD_Reset_ACK_REQ = 0xC0;
-static const byte CMD_Set_ACK_REQ   = 0xE0;
-static const byte CMD_MASK          = 0xE0;
+constexpr byte CMD_BusRelease    = 0x00;
+constexpr byte CMD_Select        = 0x20;
+constexpr byte CMD_ResetATN      = 0x40;
+constexpr byte CMD_SetATN        = 0x60;
+constexpr byte CMD_Transfer      = 0x80;
+constexpr byte CMD_TransferPause = 0xA0;
+constexpr byte CMD_Reset_ACK_REQ = 0xC0;
+constexpr byte CMD_Set_ACK_REQ   = 0xE0;
+constexpr byte CMD_MASK          = 0xE0;
 
-static const unsigned MAX_DEV = 8;
+constexpr unsigned MAX_DEV = 8;
 
 MB89352::MB89352(const DeviceConfig& config)
 {
-	PRT_DEBUG("spc create");
 	// TODO: devBusy = false;
 
 	// ALMOST COPY PASTED FROM WD33C93:
@@ -101,29 +99,28 @@ MB89352::MB89352(const DeviceConfig& config)
 	for (auto* t : config.getXML()->getChildren("target")) {
 		unsigned id = t->getAttributeAsInt("id");
 		if (id >= MAX_DEV) {
-			throw MSXException(StringOp::Builder() <<
-				"Invalid SCSI id: " << id <<
-				" (should be 0.." + MAX_DEV - 1 << ')');
+			throw MSXException(
+				"Invalid SCSI id: ", id,
+				" (should be 0..", MAX_DEV - 1, ')');
 		}
 		if (dev[id]) {
-			throw MSXException(StringOp::Builder() <<
-				"Duplicate SCSI id: " << id);
+			throw MSXException("Duplicate SCSI id: ", id);
 		}
 		DeviceConfig conf(config, *t);
 		auto& type = t->getChild("type").getData();
 		if (type == "SCSIHD") {
-			dev[id] = make_unique<SCSIHD>(conf, buffer,
+			dev[id] = std::make_unique<SCSIHD>(conf, buffer,
 			        SCSIDevice::MODE_SCSI2 | SCSIDevice::MODE_MEGASCSI);
 		} else if (type == "SCSILS120") {
-			dev[id] = make_unique<SCSILS120>(conf, buffer,
+			dev[id] = std::make_unique<SCSILS120>(conf, buffer,
 			        SCSIDevice::MODE_SCSI2 | SCSIDevice::MODE_MEGASCSI);
 		} else {
-			throw MSXException("Unknown SCSI device: " + type);
+			throw MSXException("Unknown SCSI device: ", type);
 		}
 	}
 	// fill remaining targets with dummy SCSI devices to prevent crashes
 	for (auto& d : dev) {
-		if (!d) d = make_unique<DummySCSIDevice>();
+		if (!d) d = std::make_unique<DummySCSIDevice>();
 	}
 	reset(false);
 
@@ -135,10 +132,6 @@ MB89352::MB89352(const DeviceConfig& config)
 	targetId = 0;
 }
 
-MB89352::~MB89352()
-{
-}
-
 void MB89352::disconnect()
 {
 	if (phase != SCSI::BUS_FREE) {
@@ -175,7 +168,6 @@ void MB89352::softReset()
 
 void MB89352::reset(bool scsireset)
 {
-	//PRT_DEBUG("MB89352 reset");
 	regs[REG_BDID] = 0x80;     // Initial value
 	regs[REG_SCTL] = 0x80;
 	rst  = false;
@@ -195,7 +187,7 @@ void MB89352::setACKREQ(byte& value)
 {
 	// REQ check
 	if ((regs[REG_PSNS] & (PSNS_REQ | PSNS_BSY)) != (PSNS_REQ | PSNS_BSY)) {
-		PRT_DEBUG("set ACK/REQ: REQ/BSY check error");
+		// set ACK/REQ: REQ/BSY check error
 		if (regs[REG_PSNS] & PSNS_IO) { // SCSI -> SPC
 			value = 0xFF;
 		}
@@ -204,7 +196,7 @@ void MB89352::setACKREQ(byte& value)
 
 	// phase check
 	if (regs[FIX_PCTL] != (regs[REG_PSNS] & 7)) {
-		PRT_DEBUG("set ACK/REQ: phase check error");
+		// set ACK/REQ: phase check error
 		if (regs[REG_PSNS] & PSNS_IO) { // SCSI -> SPC
 			value = 0xFF;
 		}
@@ -254,7 +246,7 @@ void MB89352::setACKREQ(byte& value)
 		break;
 
 	default:
-		PRT_DEBUG("set ACK/REQ code error");
+		// set ACK/REQ code error
 		break;
 	}
 }
@@ -263,13 +255,13 @@ void MB89352::resetACKREQ()
 {
 	// ACK check
 	if ((regs[REG_PSNS] & (PSNS_ACK | PSNS_BSY)) != (PSNS_ACK | PSNS_BSY)) {
-		PRT_DEBUG("reset ACK/REQ: ACK/BSY check error");
+		// reset ACK/REQ: ACK/BSY check error
 		return;
 	}
 
 	// phase check
 	if (regs[FIX_PCTL] != (regs[REG_PSNS] & 7)) {
-		PRT_DEBUG("reset ACK/REQ: phase check error");
+		// reset ACK/REQ: phase check error
 		if (isTransfer) {
 			regs[REG_INTS] |= INTS_ServiceRequited;
 		}
@@ -330,7 +322,7 @@ void MB89352::resetACKREQ()
 				regs[REG_PSNS] = PSNS_BSY;
 				return; // note: return iso break
 			default:
-				PRT_DEBUG("phase error");
+				// phase error
 				break;
 			}
 			// TODO: devBusy = false;
@@ -348,8 +340,7 @@ void MB89352::resetACKREQ()
 			break;
 		}
 		msgin = 0;
-		// throw to SCSI::MSG_OUT...
-
+		[[fallthrough]];
 	case SCSI::MSG_OUT: // Message Out phase
 		if (msgin == -1) {
 			disconnect();
@@ -396,14 +387,14 @@ void MB89352::resetACKREQ()
 			regs[REG_PSNS] = PSNS_REQ | PSNS_BSY | PSNS_MSGIN;
 			break;
 		default:
-			PRT_DEBUG("MsgOut code error");
+			// MsgOut code error
 			break;
 		}
 		return;
 
 	default:
 		//UNREACHABLE;
-		PRT_DEBUG("reset ACK/REQ code error");
+		// reset ACK/REQ code error
 		break;
 	}
 
@@ -419,7 +410,6 @@ byte MB89352::readDREG()
 	if (isTransfer && (tc > 0)) {
 		setACKREQ(regs[REG_DREG]);
 		resetACKREQ();
-		//PRT_DEBUG("DREG read: " << tc << ' ' << std::hex << (int)regs[REG_DREG]);
 
 		--tc;
 		if (tc == 0) {
@@ -436,8 +426,6 @@ byte MB89352::readDREG()
 void MB89352::writeDREG(byte value)
 {
 	if (isTransfer && (tc > 0)) {
-		//PRT_DEBUG("DREG write: " << tc << ' ' << std::hex << (int)value);
-
 		setACKREQ(value);
 		resetACKREQ();
 
@@ -452,7 +440,6 @@ void MB89352::writeDREG(byte value)
 
 void MB89352::writeRegister(byte reg, byte value)
 {
-	//PRT_DEBUG("SPC write register: " << std::hex << (int)reg << ' ' << std::hex << (int)value);
 	switch (reg) {
 	case REG_DREG: // write data Register
 		writeDREG(value);
@@ -466,7 +453,6 @@ void MB89352::writeRegister(byte reg, byte value)
 		// bus reset
 		if (value & 0x10) {
 			if (((regs[REG_SCMD] & 0x10) == 0) & (regs[REG_SCTL] == 0)) {
-				PRT_DEBUG("SPC: bus reset");
 				rst = true;
 				regs[REG_INTS] |= INTS_ResetCondition;
 				for (auto& d : dev) {
@@ -505,7 +491,7 @@ void MB89352::writeRegister(byte reg, byte value)
 			}
 
 			if (regs[REG_PCTL] & 1) {
-				PRT_DEBUG("reselection error " << std::hex << int(regs[REG_TEMPWR]));
+				// reselection error
 				regs[REG_INTS] |= INTS_TimeOut;
 				disconnect();
 				break;
@@ -514,18 +500,20 @@ void MB89352::writeRegister(byte reg, byte value)
 			int x = regs[REG_BDID] & regs[REG_TEMPWR];
 			if (phase == SCSI::BUS_FREE && x && x != regs[REG_TEMPWR]) {
 				x = regs[REG_TEMPWR] & ~regs[REG_BDID];
+				assert(x != 0); // because of the check 2 lines above
 
 				// the targetID is calculated.
 				// It is given priority that the number is large.
-				for (targetId = 0; targetId < MAX_DEV; ++targetId) {
+				targetId = 0;
+				while (true) {
 					x >>= 1;
-					if (x == 0) {
-						break;
-					}
+					if (x == 0) break;
+					++targetId;
+					assert(targetId < MAX_DEV);
 				}
 
 				if (/*!TODO: devBusy &&*/ dev[targetId]->isSelected()) {
-					PRT_DEBUG("selection OK of target " << int(targetId));
+					// target selection OK
 					regs[REG_INTS] |= INTS_CommandComplete;
 					isBusy  = true;
 					msgin   =  0;
@@ -548,7 +536,7 @@ void MB89352::writeRegister(byte reg, byte value)
 			}
 
 			if (err) {
-				PRT_DEBUG("selection error on target " << int(targetId));
+				// target selection error
 				regs[REG_INTS] |= INTS_TimeOut;
 				disconnect();
 			}
@@ -556,34 +544,29 @@ void MB89352::writeRegister(byte reg, byte value)
 		}
 		// hardware transfer
 		case CMD_Transfer:
-			PRT_DEBUG("CMD_Transfer " << tc << " ( " << (tc/512) << ')');
 			if ((regs[FIX_PCTL] == (regs[REG_PSNS] & 7)) &&
 			    (regs[REG_PSNS] & (PSNS_REQ | PSNS_BSY))) {
 				isTransfer = true; // set Xfer in Progress
 			} else {
+				// phase error
 				regs[REG_INTS] |= INTS_ServiceRequited;
-				PRT_DEBUG("phase error");
 			}
 			break;
 
 		case CMD_BusRelease:
-			PRT_DEBUG("CMD_BusRelease");
 			disconnect();
 			break;
 
 		case CMD_SetATN:
-			PRT_DEBUG("CMD_SetATN");
 			atn = PSNS_ATN;
 			break;
 
 		case CMD_ResetATN:
-			PRT_DEBUG("CMD_ResetATN");
 			atn = 0;
 			break;
 
 		case CMD_TransferPause:
 			// nothing is done in the initiator.
-			PRT_DEBUG("CMD_TransferPause");
 			break;
 		}
 		break;  // end of REG_SCMD
@@ -593,7 +576,6 @@ void MB89352::writeRegister(byte reg, byte value)
 		if (rst) {
 			regs[REG_INTS] |= INTS_ResetCondition;
 		}
-		//PRT_DEBUG2("INTS reset: %x %x\n", value, regs[REG_INTS]);
 		break;
 
 	case REG_TEMP:
@@ -602,17 +584,14 @@ void MB89352::writeRegister(byte reg, byte value)
 
 	case REG_TCL:
 		tc = (tc & 0xFFFF00) + (value <<  0);
-		//PRT_DEBUG1("set tcl: %d\n", tc);
 		break;
 
 	case REG_TCM:
 		tc = (tc & 0xFF00FF) + (value <<  8);
-		//PRT_DEBUG1("set tcm: %d\n", tc);
 		break;
 
 	case REG_TCH:
 		tc = (tc & 0x00FFFF) + (value << 16);
-		//PRT_DEBUG1("set tch: %d\n", tc);
 		break;
 
 	case REG_PCTL:
@@ -643,7 +622,7 @@ void MB89352::writeRegister(byte reg, byte value)
 				softReset();
 			}
 		}
-		// throw to default
+		[[fallthrough]];
 	}
 	default:
 		regs[reg] = value;
@@ -684,12 +663,9 @@ byte MB89352::getSSTS() const
 
 byte MB89352::readRegister(byte reg)
 {
-	//PRT_DEBUG("MB89352: Read register " << (int)reg);
-	byte result;
 	switch (reg) {
 	case REG_DREG:
-		result = readDREG();
-		break;
+		return readDREG();
 
 	case REG_PSNS:
 		if (phase == SCSI::EXECUTE) {
@@ -713,18 +689,16 @@ byte MB89352::readRegister(byte reg)
 					regs[REG_PSNS] = PSNS_BSY;
 					break;
 				default:
-					PRT_DEBUG("phase error");
+					// phase error
 					break;
 				}
 			}
 		}
-		result = regs[REG_PSNS] | atn;
-		break;
+		return regs[REG_PSNS] | atn;
+
 	default:
-		result = peekRegister(reg);
+		return peekRegister(reg);
 	}
-	//PRT_DEBUG2("SPC reg read: %x %x\n", reg, result);
-	return result;
 }
 
 byte MB89352::peekDREG() const
@@ -758,7 +732,7 @@ byte MB89352::peekRegister(byte reg) const
 
 
 // TODO duplicated in WD33C93.cc
-static enum_string<SCSI::Phase> phaseInfo[] = {
+static std::initializer_list<enum_string<SCSI::Phase>> phaseInfo = {
 	{ "UNDEFINED",   SCSI::UNDEFINED   },
 	{ "BUS_FREE",    SCSI::BUS_FREE    },
 	{ "ARBITRATION", SCSI::ARBITRATION },
@@ -783,22 +757,22 @@ void MB89352::serialize(Archive& ar, unsigned /*version*/)
 		tag[6] = char('0' + i);
 		ar.serializePolymorphic(tag, *dev[i]);
 	}
-	ar.serialize("bufIdx", bufIdx);
-	ar.serialize("msgin", msgin);
-	ar.serialize("counter", counter);
-	ar.serialize("blockCounter", blockCounter);
-	ar.serialize("tc", tc);
-	ar.serialize("phase", phase);
-	ar.serialize("nextPhase", nextPhase);
-	ar.serialize("myId", myId);
-	ar.serialize("targetId", targetId);
+	ar.serialize("bufIdx",       bufIdx,
+	             "msgin",        msgin,
+	             "counter",      counter,
+	             "blockCounter", blockCounter,
+	             "tc",           tc,
+	             "phase",        phase,
+	             "nextPhase",    nextPhase,
+	             "myId",         myId,
+	             "targetId",     targetId);
 	ar.serialize_blob("registers", regs, sizeof(regs));
-	ar.serialize("rst", rst);
-	ar.serialize("atn", atn);
-	ar.serialize("isEnabled", isEnabled);
-	ar.serialize("isBusy", isBusy);
-	ar.serialize("isTransfer", isTransfer);
-	ar.serialize("cdbIdx", cdbIdx);
+	ar.serialize("rst",        rst,
+	             "atn",        atn,
+	             "isEnabled",  isEnabled,
+	             "isBusy",     isBusy,
+	             "isTransfer", isTransfer,
+	             "cdbIdx",     cdbIdx);
 	ar.serialize_blob("cdb", cdb, sizeof(cdb));
 }
 INSTANTIATE_SERIALIZE_METHODS(MB89352);
