@@ -1,19 +1,16 @@
 #include "SunriseIDE.hh"
-#include "RomBlockDebuggable.hh"
 #include "IDEDeviceFactory.hh"
 #include "IDEDevice.hh"
-#include "Rom.hh"
 #include "Math.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include "outer.hh"
 
 namespace openmsx {
 
 SunriseIDE::SunriseIDE(const DeviceConfig& config)
 	: MSXDevice(config)
-	, rom(make_unique<Rom>(getName() + " ROM", "rom", config))
-	, romBlockDebug(make_unique<RomBlockDebuggable>(
-		*this, &control, 0x4000, 0x4000, 14))
+	, romBlockDebug(*this)
+	, rom(getName() + " ROM", "rom", config)
 {
 	device[0] = IDEDeviceFactory::create(
 		DeviceConfig(config, config.findChild("master")));
@@ -27,9 +24,7 @@ SunriseIDE::SunriseIDE(const DeviceConfig& config)
 	powerUp(getCurrentTime());
 }
 
-SunriseIDE::~SunriseIDE()
-{
-}
+SunriseIDE::~SunriseIDE() = default;
 
 void SunriseIDE::powerUp(EmuTime::param time)
 {
@@ -105,24 +100,30 @@ void SunriseIDE::writeMem(word address, byte value, EmuTime::param time)
 	// all other writes ignored
 }
 
+byte SunriseIDE::getBank() const
+{
+	byte bank = Math::reverseByte(control & 0xF8);
+	if (bank >= (rom.getSize() / 0x4000)) {
+		bank &= ((rom.getSize() / 0x4000) - 1);
+	}
+	return bank;
+}
+
 void SunriseIDE::writeControl(byte value)
 {
 	control = value;
 	if (ideRegsEnabled != (control & 1)) {
 		ideRegsEnabled = control & 1;
-		invalidateMemCache(0x3C00, 0x0300);
-		invalidateMemCache(0x7C00, 0x0300);
-		invalidateMemCache(0xBC00, 0x0300);
-		invalidateMemCache(0xFC00, 0x0300);
+		invalidateDeviceRCache(0x3C00, 0x0300);
+		invalidateDeviceRCache(0x7C00, 0x0300);
+		invalidateDeviceRCache(0xBC00, 0x0300);
+		invalidateDeviceRCache(0xFC00, 0x0300);
 	}
 
-	byte bank = Math::reverseByte(control & 0xF8);
-	if (bank >= (rom->getSize() / 0x4000)) {
-		bank &= ((rom->getSize() / 0x4000) - 1);
-	}
-	if (internalBank != &(*rom)[0x4000 * bank]) {
-		internalBank = &(*rom)[0x4000 * bank];
-		invalidateMemCache(0x4000, 0x4000);
+	byte bank = getBank();
+	if (internalBank != &rom[0x4000 * bank]) {
+		internalBank = &rom[0x4000 * bank];
+		invalidateDeviceRCache(0x4000, 0x4000);
 	}
 }
 
@@ -139,13 +140,11 @@ byte SunriseIDE::readDataHigh(EmuTime::param /*time*/)
 word SunriseIDE::readData(EmuTime::param time)
 {
 	word result = device[selectedDevice]->readData(time);
-	//PRT_DEBUG("IDE read data: 0x" << std::hex << int(result) << std::dec);
 	return result;
 }
 
 byte SunriseIDE::readReg(nibble reg, EmuTime::param time)
 {
-	//PRT_DEBUG("IDE read reg: " << (int)reg);
 	byte result;
 	if (reg == 14) {
 		// alternate status register
@@ -220,11 +219,11 @@ void SunriseIDE::serialize(Archive& ar, unsigned /*version*/)
 	ar.template serializeBase<MSXDevice>(*this);
 	ar.serializePolymorphic("master", *device[0]);
 	ar.serializePolymorphic("slave",  *device[1]);
-	ar.serialize("readLatch", readLatch);
-	ar.serialize("writeLatch", writeLatch);
-	ar.serialize("selectedDevice", selectedDevice);
-	ar.serialize("control", control);
-	ar.serialize("softReset", softReset);
+	ar.serialize("readLatch",      readLatch,
+	             "writeLatch",     writeLatch,
+	             "selectedDevice", selectedDevice,
+	             "control",        control,
+	             "softReset",      softReset);
 
 	if (ar.isLoader()) {
 		// restore internalBank, ideRegsEnabled
@@ -234,4 +233,17 @@ void SunriseIDE::serialize(Archive& ar, unsigned /*version*/)
 INSTANTIATE_SERIALIZE_METHODS(SunriseIDE);
 REGISTER_MSXDEVICE(SunriseIDE, "SunriseIDE");
 
+
+SunriseIDE::Blocks::Blocks(SunriseIDE& device)
+	: RomBlockDebuggableBase(device)
+{
+}
+
+byte SunriseIDE::Blocks::read(unsigned address)
+{
+	if ((address < 0x4000) || (address >= 0x8000)) return 255;
+	auto& ide = OUTER(SunriseIDE, romBlockDebug);
+	return ide.getBank();
+}
+
 } // namespace openmsx
