@@ -19,74 +19,73 @@
 #include "DeviceConfig.hh"
 #include "XMLElement.hh"
 #include "MSXException.hh"
-#include "StringOp.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include <cassert>
 #include <cstring>
+#include <memory>
 
 namespace openmsx {
 
-static const unsigned MAX_DEV = 8;
+constexpr unsigned MAX_DEV = 8;
 
-static const byte REG_OWN_ID      = 0x00;
-static const byte REG_CONTROL     = 0x01;
-static const byte REG_TIMEO       = 0x02;
-static const byte REG_TSECS       = 0x03;
-static const byte REG_THEADS      = 0x04;
-static const byte REG_TCYL_HI     = 0x05;
-static const byte REG_TCYL_LO     = 0x06;
-static const byte REG_ADDR_HI     = 0x07;
-static const byte REG_ADDR_2      = 0x08;
-static const byte REG_ADDR_3      = 0x09;
-static const byte REG_ADDR_LO     = 0x0a;
-static const byte REG_SECNO       = 0x0b;
-static const byte REG_HEADNO      = 0x0c;
-static const byte REG_CYLNO_HI    = 0x0d;
-static const byte REG_CYLNO_LO    = 0x0e;
-static const byte REG_TLUN        = 0x0f;
-static const byte REG_CMD_PHASE   = 0x10;
-static const byte REG_SYN         = 0x11;
-static const byte REG_TCH         = 0x12;
-static const byte REG_TCM         = 0x13;
-static const byte REG_TCL         = 0x14;
-static const byte REG_DST_ID      = 0x15;
-static const byte REG_SRC_ID      = 0x16;
-static const byte REG_SCSI_STATUS = 0x17; // (r)
-static const byte REG_CMD         = 0x18;
-static const byte REG_DATA        = 0x19;
-static const byte REG_QUEUE_TAG   = 0x1a;
-static const byte REG_AUX_STATUS  = 0x1f; // (r)
+constexpr byte REG_OWN_ID      = 0x00;
+constexpr byte REG_CONTROL     = 0x01;
+constexpr byte REG_TIMEO       = 0x02;
+constexpr byte REG_TSECS       = 0x03;
+constexpr byte REG_THEADS      = 0x04;
+constexpr byte REG_TCYL_HI     = 0x05;
+constexpr byte REG_TCYL_LO     = 0x06;
+constexpr byte REG_ADDR_HI     = 0x07;
+constexpr byte REG_ADDR_2      = 0x08;
+constexpr byte REG_ADDR_3      = 0x09;
+constexpr byte REG_ADDR_LO     = 0x0a;
+constexpr byte REG_SECNO       = 0x0b;
+constexpr byte REG_HEADNO      = 0x0c;
+constexpr byte REG_CYLNO_HI    = 0x0d;
+constexpr byte REG_CYLNO_LO    = 0x0e;
+constexpr byte REG_TLUN        = 0x0f;
+constexpr byte REG_CMD_PHASE   = 0x10;
+constexpr byte REG_SYN         = 0x11;
+constexpr byte REG_TCH         = 0x12;
+constexpr byte REG_TCM         = 0x13;
+constexpr byte REG_TCL         = 0x14;
+constexpr byte REG_DST_ID      = 0x15;
+constexpr byte REG_SRC_ID      = 0x16;
+constexpr byte REG_SCSI_STATUS = 0x17; // (r)
+constexpr byte REG_CMD         = 0x18;
+constexpr byte REG_DATA        = 0x19;
+constexpr byte REG_QUEUE_TAG   = 0x1a;
+constexpr byte REG_AUX_STATUS  = 0x1f; // (r)
 
-static const byte REG_CDBSIZE     = 0x00;
-static const byte REG_CDB1        = 0x03;
-static const byte REG_CDB2        = 0x04;
-static const byte REG_CDB3        = 0x05;
-static const byte REG_CDB4        = 0x06;
-static const byte REG_CDB5        = 0x07;
-static const byte REG_CDB6        = 0x08;
-static const byte REG_CDB7        = 0x09;
-static const byte REG_CDB8        = 0x0a;
-static const byte REG_CDB9        = 0x0b;
-static const byte REG_CDB10       = 0x0c;
-static const byte REG_CDB11       = 0x0d;
-static const byte REG_CDB12       = 0x0e;
+constexpr byte REG_CDBSIZE     = 0x00;
+constexpr byte REG_CDB1        = 0x03;
+constexpr byte REG_CDB2        = 0x04;
+constexpr byte REG_CDB3        = 0x05;
+constexpr byte REG_CDB4        = 0x06;
+constexpr byte REG_CDB5        = 0x07;
+constexpr byte REG_CDB6        = 0x08;
+constexpr byte REG_CDB7        = 0x09;
+constexpr byte REG_CDB8        = 0x0a;
+constexpr byte REG_CDB9        = 0x0b;
+constexpr byte REG_CDB10       = 0x0c;
+constexpr byte REG_CDB11       = 0x0d;
+constexpr byte REG_CDB12       = 0x0e;
 
-static const byte OWN_EAF         = 0x08; // ENABLE ADVANCED FEATURES
+constexpr byte OWN_EAF         = 0x08; // ENABLE ADVANCED FEATURES
 
 // SCSI STATUS
-static const byte SS_RESET        = 0x00; // reset
-static const byte SS_RESET_ADV    = 0x01; // reset w/adv. features
-static const byte SS_XFER_END     = 0x16; // select and transfer complete
-static const byte SS_SEL_TIMEOUT  = 0x42; // selection timeout
-static const byte SS_DISCONNECT   = 0x85;
+constexpr byte SS_RESET        = 0x00; // reset
+constexpr byte SS_RESET_ADV    = 0x01; // reset w/adv. features
+constexpr byte SS_XFER_END     = 0x16; // select and transfer complete
+constexpr byte SS_SEL_TIMEOUT  = 0x42; // selection timeout
+constexpr byte SS_DISCONNECT   = 0x85;
 
 // AUX STATUS
-static const byte AS_DBR          = 0x01; // data buffer ready
-static const byte AS_CIP          = 0x10; // command in progress, chip is busy
-static const byte AS_BSY          = 0x20; // Level 2 command in progress
-static const byte AS_LCI          = 0x40; // last command ignored
-static const byte AS_INT          = 0x80;
+constexpr byte AS_DBR          = 0x01; // data buffer ready
+constexpr byte AS_CIP          = 0x10; // command in progress, chip is busy
+constexpr byte AS_BSY          = 0x20; // Level 2 command in progress
+constexpr byte AS_LCI          = 0x40; // last command ignored
+constexpr byte AS_INT          = 0x80;
 
 /* command phase
 0x00    NO_SELECT
@@ -111,31 +110,29 @@ WD33C93::WD33C93(const DeviceConfig& config)
 	for (auto* t : config.getXML()->getChildren("target")) {
 		unsigned id = t->getAttributeAsInt("id");
 		if (id >= MAX_DEV) {
-			throw MSXException(StringOp::Builder() <<
-				"Invalid SCSI id: " << id <<
-				" (should be 0.." << MAX_DEV - 1 << ')');
+			throw MSXException("Invalid SCSI id: ", id,
+			                   " (should be 0..", MAX_DEV - 1, ')');
 		}
 		if (dev[id]) {
-			throw MSXException(StringOp::Builder() <<
-				"Duplicate SCSI id: " << id);
+			throw MSXException("Duplicate SCSI id: ", id);
 		}
 		DeviceConfig conf(config, *t);
 		auto& type = t->getChild("type").getData();
 		if (type == "SCSIHD") {
-			dev[id] = make_unique<SCSIHD>(conf, buffer,
+			dev[id] = std::make_unique<SCSIHD>(conf, buffer,
 			        SCSIDevice::MODE_SCSI1 | SCSIDevice::MODE_UNITATTENTION |
 			        SCSIDevice::MODE_NOVAXIS);
 		} else if (type == "SCSILS120") {
-			dev[id] = make_unique<SCSILS120>(conf, buffer,
+			dev[id] = std::make_unique<SCSILS120>(conf, buffer,
 			        SCSIDevice::MODE_SCSI1 | SCSIDevice::MODE_UNITATTENTION |
 			        SCSIDevice::MODE_NOVAXIS);
 		} else {
-			throw MSXException("Unknown SCSI device: " + type);
+			throw MSXException("Unknown SCSI device: ", type);
 		}
 	}
 	// fill remaining targets with dummy SCSI devices to prevent crashes
 	for (auto& d : dev) {
-		if (!d) d = make_unique<DummySCSIDevice>();
+		if (!d) d = std::make_unique<DummySCSIDevice>();
 	}
 	reset(false);
 
@@ -146,10 +143,6 @@ WD33C93::WD33C93(const DeviceConfig& config)
 	targetId = 0;
 }
 
-WD33C93::~WD33C93()
-{
-}
-
 void WD33C93::disconnect()
 {
 	if (phase != SCSI::BUS_FREE) {
@@ -162,14 +155,12 @@ void WD33C93::disconnect()
 		phase = SCSI::BUS_FREE;
 	}
 	tc = 0;
-
-	PRT_DEBUG("busfree()");
 }
 
 void WD33C93::execCmd(byte value)
 {
 	if (regs[REG_AUX_STATUS] & AS_CIP) {
-		PRT_DEBUG("wd33c93ExecCmd() CIP error");
+		// CIP error
 		return;
 	}
 	//regs[REG_AUX_STATUS] |= AS_CIP;
@@ -178,7 +169,6 @@ void WD33C93::execCmd(byte value)
 	bool atn = false;
 	switch (value) {
 	case 0x00: // Reset controller (software reset)
-		PRT_DEBUG("wd33c93 [CMD] Reset controller");
 		memset(regs + 1, 0, 0x1a);
 		disconnect();
 		latch = 0; // TODO: is this correct? Some doc says: reset to zero by masterreset-signal but not by reset command.
@@ -187,19 +177,16 @@ void WD33C93::execCmd(byte value)
 		break;
 
 	case 0x02: // Assert ATN
-		PRT_DEBUG("wd33c93 [CMD] Assert ATN");
 		break;
 
 	case 0x04: // Disconnect
-		PRT_DEBUG("wd33c93 [CMD] Disconnect");
 		disconnect();
 		break;
 
 	case 0x06: // Select with ATN (Lv2)
 		atn = true;
-		// fall-through
+		[[fallthrough]];
 	case 0x07: // Select Without ATN (Lv2)
-		PRT_DEBUG("wd33c93 [CMD] Select (ATN " << atn << ')');
 		targetId = regs[REG_DST_ID] & 7;
 		regs[REG_SCSI_STATUS] = SS_SEL_TIMEOUT;
 		tc = 0;
@@ -208,9 +195,8 @@ void WD33C93::execCmd(byte value)
 
 	case 0x08: // Select with ATN and transfer (Lv2)
 		atn = true;
-		// fall-through
+		[[fallthrough]];
 	case 0x09: // Select without ATN and Transfer (Lv2)
-		PRT_DEBUG("wd33c93 [CMD] Select and transfer (ATN " << atn << ')');
 		targetId = regs[REG_DST_ID] & 7;
 
 		if (!devBusy && targetId < MAX_DEV && /* targetId != myId  && */
@@ -243,7 +229,7 @@ void WD33C93::execCmd(byte value)
 			}
 			//regs[REG_SRC_ID] |= regs[REG_DST_ID] & 7;
 		} else {
-			PRT_DEBUG("wd33c93 timeout on target " << int(targetId));
+			// timeout
 			tc = 0;
 			regs[REG_SCSI_STATUS] = SS_SEL_TIMEOUT;
 			regs[REG_AUX_STATUS]  = AS_INT;
@@ -252,14 +238,13 @@ void WD33C93::execCmd(byte value)
 
 	case 0x18: // Translate Address (Lv2)
 	default:
-		PRT_DEBUG("wd33c93 [CMD] unsupport command " << int(value));
+		// unsupport command
 		break;
 	}
 }
 
 void WD33C93::writeAdr(byte value)
 {
-	//PRT_DEBUG("WriteAdr value " << std::hex << (int)value);
 	latch = value & 0x1f;
 }
 
@@ -267,12 +252,10 @@ void WD33C93::writeAdr(byte value)
 // except for the address-, aux.status-, data- and command registers.
 void WD33C93::writeCtrl(byte value)
 {
-	//PRT_DEBUG("wd33c93 write #" << std::hex << (int)latch << ", " << (int)value);
 	switch (latch) {
 	case REG_OWN_ID:
 		regs[REG_OWN_ID] = value;
 		myId = value & 7;
-		PRT_DEBUG("wd33c93 myid = " << int(myId));
 		break;
 
 	case REG_TCH:
@@ -288,7 +271,6 @@ void WD33C93::writeCtrl(byte value)
 		break;
 
 	case REG_CMD_PHASE:
-		PRT_DEBUG("wd33c93 CMD_PHASE = " << int(value));
 		regs[REG_CMD_PHASE] = value;
 		break;
 
@@ -349,7 +331,6 @@ byte WD33C93::readAuxStatus()
 			regs[REG_AUX_STATUS] |= AS_DBR;
 		}
 	}
-	//PRT_DEBUG("readAuxStatus returning " << std::hex << (int)rv);
 	return rv;
 }
 
@@ -357,13 +338,10 @@ byte WD33C93::readAuxStatus()
 // except for the address-, aux.status-, data- and command registers.
 byte WD33C93::readCtrl()
 {
-	//PRT_DEBUG("ReadCtrl");
 	byte rv;
-
 	switch (latch) {
 	case REG_SCSI_STATUS:
 		rv = regs[REG_SCSI_STATUS];
-		//PRT_DEBUG1("wd33c93 SCSI_STATUS = %X\n", rv);
 		if (rv != SS_XFER_END) {
 			regs[REG_AUX_STATUS] &= ~AS_INT;
 		} else {
@@ -417,7 +395,6 @@ byte WD33C93::readCtrl()
 		rv = regs[latch];
 		break;
 	}
-	//PRT_DEBUG2("wd33c93 read #%X, %X\n", latch, rv);
 
 	latch = (latch + 1) & 0x1f;
 	return rv;
@@ -444,8 +421,6 @@ byte WD33C93::peekCtrl() const
 
 void WD33C93::reset(bool scsireset)
 {
-	PRT_DEBUG("wd33c93 reset");
-
 	// initialized register
 	memset(regs, 0, 0x1b);
 	memset(regs + 0x1b, 0xff, 4);
@@ -463,7 +438,7 @@ void WD33C93::reset(bool scsireset)
 }
 
 
-static enum_string<SCSI::Phase> phaseInfo[] = {
+static std::initializer_list<enum_string<SCSI::Phase>> phaseInfo = {
 	{ "UNDEFINED",   SCSI::UNDEFINED   },
 	{ "BUS_FREE",    SCSI::BUS_FREE    },
 	{ "ARBITRATION", SCSI::ARBITRATION },
@@ -488,16 +463,16 @@ void WD33C93::serialize(Archive& ar, unsigned /*version*/)
 		tag[6] = char('0' + i);
 		ar.serializePolymorphic(tag, *dev[i]);
 	}
-	ar.serialize("bufIdx", bufIdx);
-	ar.serialize("counter", counter);
-	ar.serialize("blockCounter", blockCounter);
-	ar.serialize("tc", tc);
-	ar.serialize("phase", phase);
-	ar.serialize("myId", myId);
-	ar.serialize("targetId", targetId);
+	ar.serialize("bufIdx",       bufIdx,
+	             "counter",      counter,
+	             "blockCounter", blockCounter,
+	             "tc",           tc,
+	             "phase",        phase,
+	             "myId",         myId,
+	             "targetId",     targetId);
 	ar.serialize_blob("registers", regs, sizeof(regs));
-	ar.serialize("latch", latch);
-	ar.serialize("devBusy", devBusy);
+	ar.serialize("latch",   latch,
+	             "devBusy", devBusy);
 }
 INSTANTIATE_SERIALIZE_METHODS(WD33C93);
 
