@@ -3,31 +3,29 @@
 #include "MSXEventDistributor.hh"
 #include "ReverseManager.hh"
 #include "InputEvents.hh"
-#include "FloatSetting.hh"
 #include "Timer.hh"
 #include "MSXException.hh"
 #include "checked_cast.hh"
-#include "memory.hh"
+#include "one_of.hh"
+#include "ranges.hh"
 #include "stl.hh"
 #include <cassert>
 
-using std::make_shared;
-
 namespace openmsx {
 
-EventDelay::EventDelay(Scheduler& scheduler,
+EventDelay::EventDelay(Scheduler& scheduler_,
                        CommandController& commandController,
                        EventDistributor& eventDistributor_,
                        MSXEventDistributor& msxEventDistributor_,
                        ReverseManager& reverseManager)
-	: Schedulable(scheduler)
+	: Schedulable(scheduler_)
 	, eventDistributor(eventDistributor_)
 	, msxEventDistributor(msxEventDistributor_)
-	, prevEmu(EmuTime::zero)
+	, prevEmu(EmuTime::zero())
 	, prevReal(Timer::getTime())
-	, delaySetting(make_unique<FloatSetting>(
+	, delaySetting(
 		commandController, "inputdelay",
-		"delay input to avoid key-skips", 0.0, 0.0, 10.0))
+		"delay input to avoid key-skips", 0.0, 0.0, 10.0)
 {
 	eventDistributor.registerEventListener(
 		OPENMSX_KEY_DOWN_EVENT, *this, EventDistributor::MSX);
@@ -80,7 +78,7 @@ EventDelay::~EventDelay()
 int EventDelay::signalEvent(const EventPtr& event)
 {
 	toBeScheduledEvents.push_back(event);
-	if (delaySetting->getDouble() == 0.0) {
+	if (delaySetting.getDouble() == 0.0) {
 		sync(getCurrentTime());
 	}
 	return 0;
@@ -95,7 +93,7 @@ void EventDelay::sync(EmuTime::param curEmu)
 	prevEmu = curEmu;
 
 	double factor = emuDuration.toDouble() / realDuration;
-	EmuDuration extraDelay(delaySetting->getDouble());
+	EmuDuration extraDelay(delaySetting.getDouble());
 
 #if PLATFORM_ANDROID
 	// The virtual keyboard on Android sends a key press and the
@@ -122,12 +120,11 @@ void EventDelay::sync(EmuTime::param curEmu)
 	EmuTime time = curEmu + extraDelay;
 	for (auto& e : toBeScheduledEvents) {
 #if PLATFORM_ANDROID
-		if (e->getType() == OPENMSX_KEY_DOWN_EVENT ||
-		    e->getType() == OPENMSX_KEY_UP_EVENT) {
+		if (e->getType() == one_of(OPENMSX_KEY_DOWN_EVENT, OPENMSX_KEY_UP_EVENT)) {
 			auto keyEvent = checked_cast<const KeyEvent*>(e.get());
 			int maskedKeyCode = int(keyEvent->getKeyCode()) & int(Keys::K_MASK);
-			auto it = find_if(begin(nonMatchedKeyPresses), end(nonMatchedKeyPresses),
-				          EqualTupleValue<0>(maskedKeyCode));
+			auto it = ranges::find_if(nonMatchedKeyPresses,
+				                  EqualTupleValue<0>(maskedKeyCode));
 			if (e->getType() == OPENMSX_KEY_DOWN_EVENT) {
 				if (it == end(nonMatchedKeyPresses)) {
 					nonMatchedKeyPresses.emplace_back(maskedKeyCode, e);
@@ -146,13 +143,11 @@ void EventDelay::sync(EmuTime::param curEmu)
 						// Reschedule it for the next sync, with the realTime updated to now, so that it seems like the
 						// key was released now and not when android released it.
 						// Otherwise, the offset calculation for the emutime further down below will go wrong on the next sync
-						EventPtr newKeyupEvent = make_shared<KeyUpEvent>(keyEvent->getKeyCode(), keyEvent->getUnicode());
+						EventPtr newKeyupEvent = std::make_shared<KeyUpEvent>(keyEvent->getKeyCode());
 						toBeRescheduledEvents.push_back(newKeyupEvent);
 						continue; // continue with next to be scheduled event
 					}
-					auto backIt = end(nonMatchedKeyPresses) - 1;
-					if (it != backIt) std::swap(*it, *backIt);
-					nonMatchedKeyPresses.pop_back();
+					move_pop_back(nonMatchedKeyPresses, it);
 				}
 			}
 		}
@@ -172,13 +167,11 @@ void EventDelay::sync(EmuTime::param curEmu)
 	toBeScheduledEvents.clear();
 
 #if PLATFORM_ANDROID
-	toBeScheduledEvents.insert(end(toBeScheduledEvents),
-		make_move_iterator(begin(toBeRescheduledEvents)),
-		make_move_iterator(end  (toBeRescheduledEvents)));
+	append(toBeScheduledEvents, std::move(toBeRescheduledEvents));
 #endif
 }
 
-void EventDelay::executeUntil(EmuTime::param time, int /*userData*/)
+void EventDelay::executeUntil(EmuTime::param time)
 {
 	try {
 		auto event = std::move(scheduledEvents.front());
