@@ -1,34 +1,32 @@
 #include "JoyTap.hh"
 #include "JoystickPort.hh"
-#include "StringOp.hh"
+#include "PluggingController.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include "strCat.hh"
+#include <memory>
 
 namespace openmsx {
 
 using std::string;
 
-JoyTap::JoyTap(PluggingController& pluggingController_,
-               const string& name_)
+JoyTap::JoyTap(PluggingController& pluggingController_, string name_)
 	: pluggingController(pluggingController_)
-	, name(name_)
+	, name(std::move(name_))
 {
 }
 
-JoyTap::~JoyTap()
-{
-}
+JoyTap::~JoyTap() = default;
 
-void JoyTap::createPorts(PluggingController& pluggingController, const string& baseDescription) {
+void JoyTap::createPorts(const string& baseDescription) {
 	for (int i = 0; i < 4; ++i) {
-		slaves[i] = make_unique<JoystickPort>(
+		slaves[i] = std::make_unique<JoystickPort>(
 			pluggingController,
-			StringOp::Builder() << name << "_port_" << char('1' + i),
-			StringOp::Builder() << baseDescription << char('1' + i));
+			strCat(name, "_port_", char('1' + i)),
+			strCat(baseDescription, char('1' + i)));
 	}
 }
 
-string_ref JoyTap::getDescription() const
+std::string_view JoyTap::getDescription() const
 {
 	return "MSX Joy Tap device";
 }
@@ -40,36 +38,43 @@ const std::string& JoyTap::getName() const
 
 void JoyTap::plugHelper(Connector& /*connector*/, EmuTime::param /*time*/)
 {
-	createPorts(pluggingController, "Joy Tap port ");
+	createPorts("Joy Tap port ");
 }
 
 void JoyTap::unplugHelper(EmuTime::param time)
 {
-	for (int i = 0; i < 4; ++i) {
-		slaves[i]->unplug(time);
-		slaves[i].reset();
+	for (auto& s : slaves) {
+		s->unplug(time);
+		s.reset();
 	}
 }
 
 byte JoyTap::read(EmuTime::param time)
 {
 	byte value = 255;
-	for (int i = 0; i < 4; ++i) {
-		value &= slaves[i]->read(time);
+	for (auto& s : slaves) {
+		value &= s->read(time);
 	}
 	return value;
 }
 
 void JoyTap::write(byte value, EmuTime::param time)
 {
-	for (int i = 0; i < 4; ++i) {
-		slaves[i]->write(value, time);
+	for (auto& s : slaves) {
+		s->write(value, time);
 	}
 }
 
 template<typename Archive>
 void JoyTap::serialize(Archive& ar, unsigned /*version*/)
 {
+	// saving only happens when plugged in
+	if (!ar.isLoader()) assert(isPluggedIn());
+	// restore plugged state when loading
+	if (ar.isLoader()) {
+		plugHelper(*getConnector(), pluggingController.getCurrentTime());
+	}
+
 	char tag[6] = { 'p', 'o', 'r', 't', 'X', 0 };
 	for (int i = 0; i < 4; ++i) {
 		tag[4] = char('0' + i);
