@@ -8,15 +8,15 @@
 #include "StateChange.hh"
 #include "TclObject.hh"
 #include "GlobalSettings.hh"
-#include "StringSetting.hh"
 #include "IntegerSetting.hh"
 #include "CommandController.hh"
 #include "CommandException.hh"
 #include "serialize.hh"
 #include "serialize_meta.hh"
-#include "memory.hh"
+#include "StringOp.hh"
+#include "one_of.hh"
 #include "xrange.hh"
-#include "build-info.hh"
+#include <memory>
 
 using std::string;
 using std::shared_ptr;
@@ -47,7 +47,7 @@ void Joystick::registerAll(MSXEventDistributor& eventDistributor,
 			// practice.
 			if (InputEventGenerator::joystickNumButtons(joystick) != 0) {
 				controller.registerPluggable(
-					make_unique<Joystick>(
+					std::make_unique<Joystick>(
 						eventDistributor,
 						stateChangeDistributor,
 						commandController,
@@ -63,9 +63,9 @@ void Joystick::registerAll(MSXEventDistributor& eventDistributor,
 class JoyState final : public StateChange
 {
 public:
-	JoyState() {} // for serialize
-	JoyState(EmuTime::param time, unsigned joyNum_, byte press_, byte release_)
-		: StateChange(time)
+	JoyState() = default; // for serialize
+	JoyState(EmuTime::param time_, unsigned joyNum_, byte press_, byte release_)
+		: StateChange(time_)
 		, joyNum(joyNum_), press(press_), release(release_)
 	{
 		assert((press != 0) || (release != 0));
@@ -78,9 +78,9 @@ public:
 	template<typename Archive> void serialize(Archive& ar, unsigned /*version*/)
 	{
 		ar.template serializeBase<StateChange>(*this);
-		ar.serialize("joyNum", joyNum);
-		ar.serialize("press", press);
-		ar.serialize("release", release);
+		ar.serialize("joyNum",  joyNum,
+		             "press",   press,
+		             "release", release);
 	}
 private:
 	unsigned joyNum;
@@ -89,31 +89,30 @@ private:
 REGISTER_POLYMORPHIC_CLASS(StateChange, JoyState, "JoyState");
 
 
-void checkJoystickConfig(Interpreter& interp, TclObject& newValue)
+#ifndef SDL_JOYSTICK_DISABLED
+static void checkJoystickConfig(Interpreter& interp, TclObject& newValue)
 {
 	unsigned n = newValue.getListLength(interp);
 	if (n & 1) {
 		throw CommandException("Need an even number of elements");
 	}
 	for (unsigned i = 0; i < n; i += 2) {
-		string_ref key  = newValue.getListIndex(interp, i + 0).getString();
+		std::string_view key  = newValue.getListIndex(interp, i + 0).getString();
 		TclObject value = newValue.getListIndex(interp, i + 1);
-		if ((key != "A"   ) && (key != "B"    ) &&
-		    (key != "LEFT") && (key != "RIGHT") &&
-		    (key != "UP"  ) && (key != "DOWN" )) {
+		if (key != one_of("A", "B", "LEFT", "RIGHT", "UP", "DOWN")) {
 			throw CommandException(
 				"Invalid MSX joystick action: must be one of "
 				"'A', 'B', 'LEFT', 'RIGHT', 'UP', 'DOWN'.");
 		}
 		for (auto j : xrange(value.getListLength(interp))) {
-			string_ref host = value.getListIndex(interp, j).getString();
-			if (!host.starts_with("button") &&
-			    !host.starts_with("+axis") &&
-			    !host.starts_with("-axis") &&
-			    !host.starts_with("L_hat") &&
-			    !host.starts_with("R_hat") &&
-			    !host.starts_with("U_hat") &&
-			    !host.starts_with("D_hat")) {
+			std::string_view host = value.getListIndex(interp, j).getString();
+			if (!StringOp::startsWith(host, "button") &&
+			    !StringOp::startsWith(host, "+axis") &&
+			    !StringOp::startsWith(host, "-axis") &&
+			    !StringOp::startsWith(host, "L_hat") &&
+			    !StringOp::startsWith(host, "R_hat") &&
+			    !StringOp::startsWith(host, "U_hat") &&
+			    !StringOp::startsWith(host, "D_hat")) {
 				throw CommandException(
 					"Invalid host joystick action: must be "
 					"one of 'button<N>', '+axis<N>', '-axis<N>', "
@@ -123,8 +122,32 @@ void checkJoystickConfig(Interpreter& interp, TclObject& newValue)
 	}
 }
 
+static string getJoystickName(unsigned joyNum)
+{
+	return strCat("joystick", char('1' + joyNum));
+}
+
+static TclObject getConfigValue(SDL_Joystick* joystick)
+{
+	TclObject listA, listB;
+	for (auto i : xrange(InputEventGenerator::joystickNumButtons(joystick))) {
+		string button = strCat("button", i);
+		if (i & 1) {
+			listB.addListElement(button);
+		} else {
+			listA.addListElement(button);
+		}
+	}
+	TclObject value;
+	value.addDictKeyValues("LEFT",  "-axis0",
+	                       "RIGHT", "+axis0",
+	                       "UP",    "-axis1",
+	                       "DOWN",  "+axis1",
+	                       "A",     listA,
+	                       "B",     listB);
+	return value;
+}
 
-#ifndef SDL_JOYSTICK_DISABLED
 // Note: It's OK to open/close the same SDL_Joystick multiple times (we open it
 // once per MSX machine). The SDL documentation doesn't state this, but I
 // checked the implementation and a SDL_Joystick uses a 'reference count' on
@@ -137,35 +160,15 @@ Joystick::Joystick(MSXEventDistributor& eventDistributor_,
 	: eventDistributor(eventDistributor_)
 	, stateChangeDistributor(stateChangeDistributor_)
 	, joystick(joystick_)
-	, joyNum(SDL_JoystickIndex(joystick_))
+	, joyNum(SDL_JoystickInstanceID(joystick_))
 	, deadSetting(globalSettings.getJoyDeadzoneSetting(joyNum))
-	, name("joystickX") // 'X' is filled in below
-	, desc(string(SDL_JoystickName(joyNum)))
+	, name(getJoystickName(joyNum))
+	, desc(string(SDL_JoystickName(joystick_)))
+	, configSetting(commandController, name + "_config",
+		"joystick configuration", getConfigValue(joystick).getString())
 {
-	const_cast<string&>(name)[8] = char('1' + joyNum);
-
-	// create config setting
-	TclObject value;
-	value.addListElement("LEFT" ); value.addListElement("-axis0");
-	value.addListElement("RIGHT"); value.addListElement("+axis0");
-	value.addListElement("UP"   ); value.addListElement("-axis1");
-	value.addListElement("DOWN" ); value.addListElement("+axis1");
-	TclObject listA, listB;
-	for (auto i : xrange(InputEventGenerator::joystickNumButtons(joystick))) {
-		string button = "button" + StringOp::toString(i);
-		if (i & 1) {
-			listB.addListElement(button);
-		} else {
-			listA.addListElement(button);
-		}
-	}
-	value.addListElement("A"); value.addListElement(listA);
-	value.addListElement("B"); value.addListElement(listB);
-	configSetting = make_unique<StringSetting>(
-		commandController, name + "_config", "joystick configuration",
-		value.getString());
 	auto& interp = commandController.getInterpreter();
-	configSetting->setChecker([&interp](TclObject& newValue) {
+	configSetting.setChecker([&interp](TclObject& newValue) {
 		checkJoystickConfig(interp, newValue); });
 
 	pin8 = false; // avoid UMR
@@ -187,7 +190,7 @@ const string& Joystick::getName() const
 	return name;
 }
 
-string_ref Joystick::getDescription() const
+std::string_view Joystick::getDescription() const
 {
 	return desc;
 }
@@ -231,8 +234,8 @@ byte Joystick::calcState()
 	              JOY_BUTTONA | JOY_BUTTONB;
 	if (joystick) {
 		int threshold = (deadSetting.getInt() * 32768) / 100;
-		auto& interp = configSetting->getInterpreter();
-		auto& dict   = configSetting->getValue();
+		auto& interp = configSetting.getInterpreter();
+		auto& dict   = configSetting.getValue();
 		if (getState(interp, dict, "A"    , threshold)) result &= ~JOY_BUTTONA;
 		if (getState(interp, dict, "B"    , threshold)) result &= ~JOY_BUTTONB;
 		if (getState(interp, dict, "UP"   , threshold)) result &= ~JOY_UP;
@@ -244,58 +247,59 @@ byte Joystick::calcState()
 }
 
 bool Joystick::getState(Interpreter& interp, const TclObject& dict,
-                        string_ref key, int threshold)
+                        std::string_view key, int threshold)
 {
 	try {
-		const auto& list = dict.getDictValue(interp, TclObject(key));
+		const auto& list = dict.getDictValue(interp, key);
 		for (auto i : xrange(list.getListLength(interp))) {
 			const auto& elem = list.getListIndex(interp, i).getString();
-			if (elem.starts_with("button")) {
-				int n = stoi(elem.substr(6));
+			if (StringOp::startsWith(elem, "button")) {
+				unsigned n = StringOp::fast_stou(elem.substr(6));
 				if (InputEventGenerator::joystickGetButton(joystick, n)) {
 					return true;
 				}
-			} else if (elem.starts_with("+axis")) {
-				int n = stoi(elem.substr(5));
+			} else if (StringOp::startsWith(elem, "+axis")) {
+				unsigned n = StringOp::fast_stou(elem.substr(5));
 				if (SDL_JoystickGetAxis(joystick, n) > threshold) {
 					return true;
 				}
-			} else if (elem.starts_with("-axis")) {
-				int n = stoi(elem.substr(5));
+			} else if (StringOp::startsWith(elem, "-axis")) {
+				unsigned n = StringOp::fast_stou(elem.substr(5));
 				if (SDL_JoystickGetAxis(joystick, n) < -threshold) {
 					return true;
 				}
-			} else if (elem. starts_with("L_hat")) {
-				int n = stoi(elem.substr(5));
+			} else if (StringOp::startsWith(elem, "L_hat")) {
+				unsigned n = StringOp::fast_stou(elem.substr(5));
 				if (SDL_JoystickGetHat(joystick, n) & SDL_HAT_LEFT) {
 					return true;
 				}
-			} else if (elem.starts_with("R_hat")) {
-				int n = stoi(elem.substr(5));
+			} else if (StringOp::startsWith(elem, "R_hat")) {
+				unsigned n = StringOp::fast_stou(elem.substr(5));
 				if (SDL_JoystickGetHat(joystick, n) & SDL_HAT_RIGHT) {
 					return true;
 				}
-			} else if (elem.starts_with("U_hat")) {
-				int n = stoi(elem.substr(5));
+			} else if (StringOp::startsWith(elem, "U_hat")) {
+				unsigned n = StringOp::fast_stou(elem.substr(5));
 				if (SDL_JoystickGetHat(joystick, n) & SDL_HAT_UP) {
 					return true;
 				}
-			} else if (elem.starts_with("D_hat")) {
-				int n = stoi(elem.substr(5));
+			} else if (StringOp::startsWith(elem, "D_hat")) {
+				unsigned n = StringOp::fast_stou(elem.substr(5));
 				if (SDL_JoystickGetHat(joystick, n) & SDL_HAT_DOWN) {
 					return true;
 				}
 			}
 		}
-	} catch (MSXException&) {
-		// ignore
+	} catch (...) {
+		// Error, in getListLength()/getListIndex() or in fast_stou().
+		// In either case we can't do anything about it here, so ignore.
 	}
 	return false;
 }
 
 // MSXEventListener
-void Joystick::signalEvent(const shared_ptr<const Event>& event,
-                           EmuTime::param time)
+void Joystick::signalMSXEvent(const shared_ptr<const Event>& event,
+                              EmuTime::param time)
 {
 	auto joyEvent = dynamic_cast<const JoystickEvent*>(event.get());
 	if (!joyEvent) return;
