@@ -1,14 +1,10 @@
 #include "MSXEventDistributor.hh"
 #include "MSXEventListener.hh"
-#include <algorithm>
+#include "stl.hh"
 #include <cassert>
 
 namespace openmsx {
 
-MSXEventDistributor::MSXEventDistributor()
-{
-}
-
 MSXEventDistributor::~MSXEventDistributor()
 {
 	assert(listeners.empty());
@@ -16,8 +12,7 @@ MSXEventDistributor::~MSXEventDistributor()
 
 bool MSXEventDistributor::isRegistered(MSXEventListener* listener) const
 {
-	return find(listeners.begin(), listeners.end(), listener) !=
-	       listeners.end();
+	return contains(listeners, listener);
 }
 
 void MSXEventDistributor::registerEventListener(MSXEventListener& listener)
@@ -28,15 +23,14 @@ void MSXEventDistributor::registerEventListener(MSXEventListener& listener)
 
 void MSXEventDistributor::unregisterEventListener(MSXEventListener& listener)
 {
-	assert(isRegistered(&listener));
-	listeners.erase(find(listeners.begin(), listeners.end(), &listener));
+	move_pop_back(listeners, rfind_unguarded(listeners, &listener));
 }
 
 void MSXEventDistributor::distributeEvent(const EventPtr& event, EmuTime::param time)
 {
-	// Iterate over a copy because signalEvent() may indirect call back into
+	// Iterate over a copy because signalMSXEvent() may indirect call back into
 	// registerEventListener().
-	//   e.g. signalEvent() -> .. -> PlugCmd::execute() -> .. ->
+	//   e.g. signalMSXEvent() -> .. -> PlugCmd::execute() -> .. ->
 	//        Connector::plug() -> .. -> Joystick::plugHelper() ->
 	//        registerEventListener()
 	auto copy = listeners;
@@ -44,7 +38,7 @@ void MSXEventDistributor::distributeEvent(const EventPtr& event, EmuTime::param
 		if (isRegistered(l)) {
 			// it's possible the listener unregistered itself
 			// (but is still present in the copy)
-			l->signalEvent(event, time);
+			l->signalMSXEvent(event, time);
 		}
 	}
 }
