@@ -1,5 +1,3 @@
-// $Id: MagicKey.cc 12631 2012-06-14 20:18:24Z m9710797 $
-
 #include "MagicKey.hh"
 #include "serialize.hh"
 #include "serialize_meta.hh"
@@ -13,7 +11,7 @@ const std::string& MagicKey::getName() const
 	return NAME;
 }
 
-string_ref MagicKey::getDescription() const
+std::string_view MagicKey::getDescription() const
 {
 	return "Dongle used by some Japanese games to enable cheat mode";
 }
