@@ -7,31 +7,31 @@
 #include "checked_cast.hh"
 #include "serialize.hh"
 #include "serialize_meta.hh"
+#include "Math.hh"
 #include "unreachable.hh"
+#include <SDL.h>
 
 using std::string;
-using std::min;
-using std::max;
 using std::shared_ptr;
 
 namespace openmsx {
 
-static const int TRESHOLD = 2;
-static const int SCALE = 2;
-static const int PHASE_XHIGH = 0;
-static const int PHASE_XLOW  = 1;
-static const int PHASE_YHIGH = 2;
-static const int PHASE_YLOW  = 3;
-static const int STROBE = 0x04;
+constexpr int TRESHOLD = 2;
+constexpr int SCALE = 2;
+constexpr int PHASE_XHIGH = 0;
+constexpr int PHASE_XLOW  = 1;
+constexpr int PHASE_YHIGH = 2;
+constexpr int PHASE_YLOW  = 3;
+constexpr int STROBE = 0x04;
 
 
 class MouseState final : public StateChange
 {
 public:
-	MouseState() {} // for serialize
-	MouseState(EmuTime::param time, int deltaX_, int deltaY_,
+	MouseState() = default; // for serialize
+	MouseState(EmuTime::param time_, int deltaX_, int deltaY_,
 	           byte press_, byte release_)
-		: StateChange(time)
+		: StateChange(time_)
 		, deltaX(deltaX_), deltaY(deltaY_)
 		, press(press_), release(release_) {}
 	int  getDeltaX()  const { return deltaX; }
@@ -41,10 +41,10 @@ public:
 	template<typename Archive> void serialize(Archive& ar, unsigned /*version*/)
 	{
 		ar.template serializeBase<StateChange>(*this);
-		ar.serialize("deltaX", deltaX);
-		ar.serialize("deltaY", deltaY);
-		ar.serialize("press", press);
-		ar.serialize("release", release);
+		ar.serialize("deltaX",  deltaX,
+		             "deltaY",  deltaY,
+		             "press",   press,
+		             "release", release);
 	}
 private:
 	int deltaX, deltaY;
@@ -57,7 +57,7 @@ Mouse::Mouse(MSXEventDistributor& eventDistributor_,
              StateChangeDistributor& stateChangeDistributor_)
 	: eventDistributor(eventDistributor_)
 	, stateChangeDistributor(stateChangeDistributor_)
-	, lastTime(EmuTime::zero)
+	, lastTime(EmuTime::zero())
 {
 	status = JOY_BUTTONA | JOY_BUTTONB;
 	phase = PHASE_YLOW;
@@ -81,20 +81,20 @@ const string& Mouse::getName() const
 	return name;
 }
 
-string_ref Mouse::getDescription() const
+std::string_view Mouse::getDescription() const
 {
 	return "MSX mouse";
 }
 
 void Mouse::plugHelper(Connector& /*connector*/, EmuTime::param time)
 {
-	if (status & JOY_BUTTONA) {
+	if (SDL_GetMouseState(nullptr, nullptr) & SDL_BUTTON(SDL_BUTTON_LEFT)) {
+		// left mouse button pressed, joystick emulation mode
+		mouseMode = false;
+	} else {
 		// not pressed, mouse mode
 		mouseMode = true;
 		lastTime = time;
-	} else {
-		// left mouse button pressed, joystick emulation mode
-		mouseMode = false;
 	}
 	plugHelper2();
 }
@@ -218,8 +218,21 @@ void Mouse::write(byte value, EmuTime::param time)
 		case PHASE_YLOW:
 			if ((value & STROBE) != 0) {
 				phase = PHASE_XHIGH;
+#if 0
+				// Real MSX mice don't have overflow protection,
+				// verified on a Philips SBC3810 MSX mouse.
 				xrel = curxrel; yrel = curyrel;
 				curxrel = 0; curyrel = 0;
+#else
+				// Nevertheless we do emulate it here. See
+				// sdsnatcher's post of 30 aug 2018 for a
+				// motivation for this difference:
+				//   https://github.com/openMSX/openMSX/issues/892
+				xrel = Math::clip<-127, 127>(curxrel);
+				yrel = Math::clip<-127, 127>(curyrel);
+				curxrel -= xrel;
+				curyrel -= yrel;
+#endif
 			}
 			break;
 		}
@@ -230,7 +243,7 @@ void Mouse::write(byte value, EmuTime::param time)
 
 
 // MSXEventListener
-void Mouse::signalEvent(const shared_ptr<const Event>& event, EmuTime::param time)
+void Mouse::signalMSXEvent(const shared_ptr<const Event>& event, EmuTime::param time)
 {
 	switch (event->getType()) {
 	case OPENMSX_MOUSE_MOTION_EVENT: {
@@ -340,21 +353,21 @@ void Mouse::serialize(Archive& ar, unsigned version)
 
 	if (ar.versionBelow(version, 4)) {
 		assert(ar.isLoader());
-		Clock<1000> tmp(EmuTime::zero);
+		Clock<1000> tmp(EmuTime::zero());
 		ar.serialize("lastTime", tmp);
 		lastTime = tmp.getTime();
 	} else {
 		ar.serialize("lastTime", lastTime);
 	}
-	ar.serialize("faze", phase); // TODO fix spelling if there's ever a need
-	                             // to bump the serialization verion
-	ar.serialize("xrel", xrel);
-	ar.serialize("yrel", yrel);
-	ar.serialize("mouseMode", mouseMode);
+	ar.serialize("faze",      phase, // TODO fix spelling if there's ever a need
+	                                 // to bump the serialization verion
+	             "xrel",      xrel,
+	             "yrel",      yrel,
+	             "mouseMode", mouseMode);
 	if (ar.versionAtLeast(version, 2)) {
-		ar.serialize("curxrel", curxrel);
-		ar.serialize("curyrel", curyrel);
-		ar.serialize("status",  status);
+		ar.serialize("curxrel", curxrel,
+		             "curyrel", curyrel,
+		             "status",  status);
 	}
 	if (ar.versionBelow(version, 3)) {
 		xrel    /= SCALE;
