@@ -1,21 +1,19 @@
 #include "NinjaTap.hh"
 #include "JoystickPort.hh"
+#include "ranges.hh"
 #include "serialize.hh"
 
 namespace openmsx {
 
-NinjaTap::NinjaTap(PluggingController& pluggingController,
-                   const std::string& name)
-	: JoyTap(pluggingController, name)
+NinjaTap::NinjaTap(PluggingController& pluggingController_, std::string name_)
+	: JoyTap(pluggingController_, std::move(name_))
 {
 	status = 0x3F; // TODO check initial value
 	previous = 0;
-	for (int i = 0; i < 4; ++i) {
-		buf[i] = 0xFF;
-	}
+	ranges::fill(buf, 0xFF);
 }
 
-string_ref NinjaTap::getDescription() const
+std::string_view NinjaTap::getDescription() const
 {
 	return "MSX Ninja Tap device";
 }
@@ -23,7 +21,7 @@ string_ref NinjaTap::getDescription() const
 
 void NinjaTap::plugHelper(Connector& /*connector*/, EmuTime::param /*time*/)
 {
-	createPorts(pluggingController, "Ninja Tap port ");
+	createPorts("Ninja Tap port ");
 }
 
 byte NinjaTap::read(EmuTime::param /*time*/)
@@ -75,9 +73,9 @@ template<typename Archive>
 void NinjaTap::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.template serializeBase<JoyTap>(*this);
-	ar.serialize("status", status);
-	ar.serialize("previous", previous);
-	ar.serialize("buf", buf);
+	ar.serialize("status",   status,
+	             "previous", previous,
+	             "buf",      buf);
 }
 INSTANTIATE_SERIALIZE_METHODS(NinjaTap);
 REGISTER_POLYMORPHIC_INITIALIZER(Pluggable, NinjaTap, "NinjaTap");
