@@ -6,23 +6,22 @@
 #include "ScopedAssign.hh"
 #include "serialize.hh"
 #include "serialize_stl.hh"
-#include "unreachable.hh"
-#include "memory.hh"
+#include "stl.hh"
+#include "view.hh"
 
 using std::vector;
 using std::string;
 
 namespace openmsx {
 
-RecordedCommand::RecordedCommand(CommandController& commandController,
+RecordedCommand::RecordedCommand(CommandController& commandController_,
                                  StateChangeDistributor& stateChangeDistributor_,
                                  Scheduler& scheduler_,
-                                 string_ref name)
-	: Command(commandController, name)
+                                 std::string_view name_)
+	: Command(commandController_, name_)
 	, stateChangeDistributor(stateChangeDistributor_)
 	, scheduler(scheduler_)
-	, dummyResultObject(make_unique<TclObject>())
-	, currentResultObject(dummyResultObject.get())
+	, currentResultObject(&dummyResultObject)
 {
 	stateChangeDistributor.registerListener(*this);
 }
@@ -32,11 +31,11 @@ RecordedCommand::~RecordedCommand()
 	stateChangeDistributor.unregisterListener(*this);
 }
 
-void RecordedCommand::execute(array_ref<TclObject> tokens, TclObject& result)
+void RecordedCommand::execute(span<const TclObject> tokens, TclObject& result)
 {
 	auto time = scheduler.getCurrentTime();
 	if (needRecord(tokens)) {
-		ScopedAssign<TclObject*> sa(currentResultObject, &result);
+		ScopedAssign sa(currentResultObject, &result);
 		stateChangeDistributor.distributeNew(
 			std::make_shared<MSXCommandEvent>(tokens, time));
 	} else {
@@ -44,15 +43,15 @@ void RecordedCommand::execute(array_ref<TclObject> tokens, TclObject& result)
 	}
 }
 
-bool RecordedCommand::needRecord(array_ref<TclObject> /*tokens*/) const
+bool RecordedCommand::needRecord(span<const TclObject> /*tokens*/) const
 {
 	return true;
 }
 
-static string_ref getBaseName(string_ref str)
+static std::string_view getBaseName(std::string_view str)
 {
 	auto pos = str.rfind("::");
-	return (pos == string_ref::npos) ? str : str.substr(pos + 2);
+	return (pos == std::string_view::npos) ? str : str.substr(pos + 2);
 }
 
 void RecordedCommand::signalStateChange(const std::shared_ptr<StateChange>& event)
@@ -88,17 +87,15 @@ void RecordedCommand::stopReplay(EmuTime::param /*time*/)
 
 // class MSXCommandEvent
 
-MSXCommandEvent::MSXCommandEvent(array_ref<string> tokens_, EmuTime::param time)
-	: StateChange(time)
+MSXCommandEvent::MSXCommandEvent(span<string> tokens_, EmuTime::param time_)
+	: StateChange(time_)
 {
-	for (auto& t : tokens_) {
-		tokens.emplace_back(t);
-	}
+	tokens = to_vector<TclObject>(tokens_);
 }
 
-MSXCommandEvent::MSXCommandEvent(array_ref<TclObject> tokens_, EmuTime::param time)
-	: StateChange(time)
-	, tokens(tokens_.begin(), tokens_.end())
+MSXCommandEvent::MSXCommandEvent(span<const TclObject> tokens_, EmuTime::param time_)
+	: StateChange(time_)
+	, tokens(to_vector(tokens_))
 {
 }
 
@@ -110,16 +107,14 @@ void MSXCommandEvent::serialize(Archive& ar, unsigned /*version*/)
 	// serialize vector<TclObject> as vector<string>
 	vector<string> str;
 	if (!ar.isLoader()) {
-		for (auto& t : tokens) {
-			str.push_back(t.getString().str());
-		}
+		str = to_vector(view::transform(
+			tokens, [](auto& t) { return string(t.getString()); }));
 	}
 	ar.serialize("tokens", str);
 	if (ar.isLoader()) {
 		assert(tokens.empty());
-		for (auto& s : str) {
-			tokens.push_back(TclObject(s));
-		}
+		tokens = to_vector(view::transform(
+			str, [](auto& s) { return TclObject(s); }));
 	}
 }
 REGISTER_POLYMORPHIC_CLASS(StateChange, MSXCommandEvent, "MSXCommandEvent");
