@@ -1,5 +1,3 @@
-// $Id: SETetrisDongle.cc 12631 2012-06-14 20:18:24Z m9710797 $
-
 #include "SETetrisDongle.hh"
 #include "serialize.hh"
 #include "serialize_meta.hh"
@@ -19,7 +17,7 @@ const std::string& SETetrisDongle::getName() const
 	return name;
 }
 
-string_ref SETetrisDongle::getDescription() const
+std::string_view SETetrisDongle::getDescription() const
 {
 	return "Tetris II Special Edition dongle";
 }
