@@ -1,17 +1,11 @@
 #include "StateChangeDistributor.hh"
 #include "StateChangeListener.hh"
 #include "StateChange.hh"
-#include <algorithm>
+#include "stl.hh"
 #include <cassert>
 
 namespace openmsx {
 
-StateChangeDistributor::StateChangeDistributor()
-	: recorder(nullptr)
-	, viewOnlyMode(false)
-{
-}
-
 StateChangeDistributor::~StateChangeDistributor()
 {
 	assert(listeners.empty());
@@ -19,8 +13,7 @@ StateChangeDistributor::~StateChangeDistributor()
 
 bool StateChangeDistributor::isRegistered(StateChangeListener* listener) const
 {
-	return find(listeners.begin(), listeners.end(), listener) !=
-	       listeners.end();
+	return contains(listeners, listener);
 }
 
 void StateChangeDistributor::registerListener(StateChangeListener& listener)
@@ -31,8 +24,7 @@ void StateChangeDistributor::registerListener(StateChangeListener& listener)
 
 void StateChangeDistributor::unregisterListener(StateChangeListener& listener)
 {
-	assert(isRegistered(&listener));
-	listeners.erase(find(listeners.begin(), listeners.end(), &listener));
+	move_pop_back(listeners, rfind_unguarded(listeners, &listener));
 }
 
 void StateChangeDistributor::registerRecorder(StateChangeRecorder& recorder_)
@@ -92,16 +84,6 @@ void StateChangeDistributor::stopReplay(EmuTime::param time)
 	}
 }
 
-void StateChangeDistributor::setViewOnlyMode(bool value)
-{
-	viewOnlyMode = value;
-}
-
-bool StateChangeDistributor::isViewOnlyMode() const
-{
-	return viewOnlyMode;
-}
-
 bool StateChangeDistributor::isReplaying() const
 {
 	if (recorder) {
