@@ -3,6 +3,7 @@
 #include "StateChangeDistributor.hh"
 #include "InputEvents.hh"
 #include "StateChange.hh"
+#include "Math.hh"
 #include "checked_cast.hh"
 #include "serialize.hh"
 #include "serialize_meta.hh"
@@ -25,10 +26,10 @@ namespace openmsx {
 class TrackballState final : public StateChange
 {
 public:
-	TrackballState() {} // for serialize
-	TrackballState(EmuTime::param time, int deltaX_, int deltaY_,
-	                                    byte press_, byte release_)
-		: StateChange(time)
+	TrackballState() = default; // for serialize
+	TrackballState(EmuTime::param time_, int deltaX_, int deltaY_,
+	                                     byte press_, byte release_)
+		: StateChange(time_)
 		, deltaX(deltaX_), deltaY(deltaY_)
 		, press(press_), release(release_) {}
 	int  getDeltaX()  const { return deltaX; }
@@ -39,10 +40,10 @@ public:
 	template<typename Archive> void serialize(Archive& ar, unsigned /*version*/)
 	{
 		ar.template serializeBase<StateChange>(*this);
-		ar.serialize("deltaX", deltaX);
-		ar.serialize("deltaY", deltaY);
-		ar.serialize("press", press);
-		ar.serialize("release", release);
+		ar.serialize("deltaX",  deltaX,
+		             "deltaY",  deltaY,
+		             "press",   press,
+		             "release", release);
 	}
 private:
 	int deltaX, deltaY;
@@ -55,9 +56,12 @@ Trackball::Trackball(MSXEventDistributor& eventDistributor_,
                      StateChangeDistributor& stateChangeDistributor_)
 	: eventDistributor(eventDistributor_)
 	, stateChangeDistributor(stateChangeDistributor_)
-	, deltaX(0), deltaY(0)
+	, lastSync(EmuTime::zero())
+	, targetDeltaX(0), targetDeltaY(0)
+	, currentDeltaX(0), currentDeltaY(0)
 	, lastValue(0)
 	, status(JOY_BUTTONA | JOY_BUTTONB)
+	, smooth(true)
 {
 }
 
@@ -76,16 +80,18 @@ const string& Trackball::getName() const
 	return name;
 }
 
-string_ref Trackball::getDescription() const
+std::string_view Trackball::getDescription() const
 {
 	return "MSX Trackball";
 }
 
-void Trackball::plugHelper(Connector& /*connector*/, EmuTime::param /*time*/)
+void Trackball::plugHelper(Connector& /*connector*/, EmuTime::param time)
 {
 	eventDistributor.registerEventListener(*this);
 	stateChangeDistributor.registerListener(*this);
-	deltaX = deltaY = 0;
+	lastSync = time;
+	targetDeltaX = targetDeltaY = 0;
+	currentDeltaX = currentDeltaY = 0;
 }
 
 void Trackball::unplugHelper(EmuTime::param /*time*/)
@@ -95,7 +101,7 @@ void Trackball::unplugHelper(EmuTime::param /*time*/)
 }
 
 // JoystickDevice
-byte Trackball::read(EmuTime::param /*time*/)
+byte Trackball::read(EmuTime::param time)
 {
 	// From the Sony GB-7 Service manual:
 	//  http://cdn.preterhuman.net/texts/computing/msx/sonygb7sm.pdf
@@ -116,32 +122,95 @@ byte Trackball::read(EmuTime::param /*time*/)
 	//   pin 8. This seems to suggest the actual (X or Y) value is always
 	//   present on reads and toggling pin 8 resets this value and switches
 	//   to the other axis.
-	auto delta = (lastValue & 4) ? deltaY : deltaX;
+	syncCurrentWithTarget(time);
+	auto delta = (lastValue & 4) ? currentDeltaY : currentDeltaX;
 	return (status & ~0x0F) | ((delta + 8) & 0x0F);
 }
 
-void Trackball::write(byte value, EmuTime::param /*time*/)
+void Trackball::write(byte value, EmuTime::param time)
 {
+	syncCurrentWithTarget(time);
 	byte diff = lastValue ^ value;
 	lastValue = value;
 	if (diff & 0x4) {
 		// pin 8 flipped
 		if (value & 4) {
-			deltaX = 0;
+			targetDeltaX = Math::clip<-8, 7>(targetDeltaX - currentDeltaX);
+			currentDeltaX = 0;
 		} else {
-			deltaY = 0;
+			targetDeltaY = Math::clip<-8, 7>(targetDeltaY - currentDeltaY);
+			currentDeltaY = 0;
 		}
 	}
 }
 
+void Trackball::syncCurrentWithTarget(EmuTime::param time)
+{
+	// In the past we only had 'targetDeltaXY' (was named 'deltaXY' then).
+	// 'currentDeltaXY' was introduced to (slightly) smooth-out the
+	// discrete host mouse movement events over time. This method performs
+	// that smoothing calculation.
+	//
+	// In emulation, the trackball movement is driven by host mouse
+	// movement events. These events are discrete. So for example if we
+	// receive a host mouse event with offset (3,-4) we immediately adjust
+	// 'targetDeltaXY' by that offset. However in reality mouse movement
+	// doesn't make such discrete jumps. If you look at small enough time
+	// intervals you'll see that the offset smoothly varies from (0,0) to
+	// (3,-4).
+	//
+	// Most often this discretization step doesn't matter. However the BIOS
+	// routine GTPAD to read mouse/trackball (more specifically PAD(12) and
+	// PAD(16) has an heuristic to distinguish the mouse from the trackball
+	// protocol. I won't go into detail, but in short it's reading the
+	// mouse/trackball two times shortly after each other and checks
+	// whether the offset of the 2nd read is centered around 0 (for mouse)
+	// or 8 (for trackball). There's only about 1 millisecond between the
+	// two reads, so in reality the mouse/trackball won't have moved much
+	// during that time. However in emulation because of the discretization
+	// we can get unlucky and do see a large offset for the 2nd read. This
+	// confuses the BIOS (it thinks it's talking to a mouse instead of
+	// trackball) and it results in erratic trackball movement.
+	//
+	// Thus to work around this problem (=make the heuristic in the BIOS
+	// work) we smear-out host mouse events over (emulated) time. Instead
+	// of immediately following the host movement, we limit changes in the
+	// emulated offsets to a rate of 1 step per millisecond. This
+	// introduces some delay, but usually (for not too fast movements) it's
+	// not noticeable.
+
+	if (!smooth) {
+		// for backwards-compatible replay files
+		currentDeltaX = targetDeltaX;
+		currentDeltaY = targetDeltaY;
+		return;
+	}
+
+	static constexpr auto INTERVAL = EmuDuration::msec(1);
+
+	int maxSteps = (time - lastSync) / INTERVAL;
+	lastSync += INTERVAL * maxSteps;
+
+	if (targetDeltaX >= currentDeltaX) {
+		currentDeltaX = std::min<int>(currentDeltaX + maxSteps, targetDeltaX);
+	} else {
+		currentDeltaX = std::max<int>(currentDeltaX - maxSteps, targetDeltaX);
+	}
+	if (targetDeltaY >= currentDeltaY) {
+		currentDeltaY = std::min<int>(currentDeltaY + maxSteps, targetDeltaY);
+	} else {
+		currentDeltaY = std::max<int>(currentDeltaY - maxSteps, targetDeltaY);
+	}
+}
+
 // MSXEventListener
-void Trackball::signalEvent(const shared_ptr<const Event>& event,
-                            EmuTime::param time)
+void Trackball::signalMSXEvent(const shared_ptr<const Event>& event,
+                               EmuTime::param time)
 {
 	switch (event->getType()) {
 	case OPENMSX_MOUSE_MOTION_EVENT: {
 		auto& mev = checked_cast<const MouseMotionEvent&>(*event);
-		static const int SCALE = 2;
+		constexpr int SCALE = 2;
 		int dx = mev.getX() / SCALE;
 		int dy = mev.getY() / SCALE;
 		if ((dx != 0) || (dy != 0)) {
@@ -198,30 +267,43 @@ void Trackball::signalStateChange(const shared_ptr<StateChange>& event)
 	auto ts = dynamic_cast<TrackballState*>(event.get());
 	if (!ts) return;
 
-	deltaX = std::min(7, std::max(-8, deltaX + ts->getDeltaX()));
-	deltaY = std::min(7, std::max(-8, deltaY + ts->getDeltaY()));
+	targetDeltaX = Math::clip<-8, 7>(targetDeltaX + ts->getDeltaX());
+	targetDeltaY = Math::clip<-8, 7>(targetDeltaY + ts->getDeltaY());
 	status = (status & ~ts->getPress()) | ts->getRelease();
 }
 
 void Trackball::stopReplay(EmuTime::param time)
 {
+	syncCurrentWithTarget(time);
 	// TODO Get actual mouse button(s) state. Is it worth the trouble?
 	byte release = (JOY_BUTTONA | JOY_BUTTONB) & ~status;
-	if ((deltaX != 0) || (deltaY != 0) || (release != 0)) {
+	if ((currentDeltaX != 0) || (currentDeltaY != 0) || (release != 0)) {
 		stateChangeDistributor.distributeNew(
 			std::make_shared<TrackballState>(
-				time, -deltaX, -deltaY, 0, release));
+				time, -currentDeltaX, -currentDeltaY, 0, release));
 	}
 }
 
-
+// version 1: initial version
+// version 2: replaced deltaXY with targetDeltaXY and currentDeltaXY
 template<typename Archive>
-void Trackball::serialize(Archive& ar, unsigned /*version*/)
+void Trackball::serialize(Archive& ar, unsigned version)
 {
-	ar.serialize("deltaX", deltaX);
-	ar.serialize("deltaY", deltaY);
-	ar.serialize("lastValue", lastValue);
-	ar.serialize("status", status);
+	if (ar.versionAtLeast(version, 2)) {
+		ar.serialize("lastSync",      lastSync,
+		             "targetDeltaX",  targetDeltaX,
+		             "targetDeltaY",  targetDeltaY,
+		             "currentDeltaX", currentDeltaX,
+		             "currentDeltaY", currentDeltaY);
+	} else {
+		ar.serialize("deltaX", targetDeltaX,
+		             "deltaY", targetDeltaY);
+		currentDeltaX = targetDeltaX;
+		currentDeltaY = targetDeltaY;
+		smooth = false;
+	}
+	ar.serialize("lastValue", lastValue,
+	             "status",    status);
 
 	if (ar.isLoader() && isPluggedIn()) {
 		plugHelper(*getConnector(), EmuTime::dummy());
