@@ -1,6 +1,4 @@
 #include "LaserdiscPlayer.hh"
-#include "BooleanSetting.hh"
-#include "RecordedCommand.hh"
 #include "CommandException.hh"
 #include "CommandController.hh"
 #include "EventDistributor.hh"
@@ -17,70 +15,58 @@
 #include "PioneerLDControl.hh"
 #include "OggReader.hh"
 #include "LDRenderer.hh"
-#include "ThrottleManager.hh"
+#include "RendererFactory.hh"
 #include "Math.hh"
 #include "likely.hh"
-#include "memory.hh"
+#include "one_of.hh"
 #include <cstdint>
 #include <cstdlib>
+#include <iostream>
+#include <memory>
 
-using std::unique_ptr;
 using std::string;
 using std::vector;
 
 namespace openmsx {
 
-// LaserdiscCommand
+// Command
 
-class LaserdiscCommand final : public RecordedCommand
-{
-public:
-	LaserdiscCommand(CommandController& commandController,
-			 StateChangeDistributor& stateChangeDistributor,
-			 Scheduler& scheduler,
-			 LaserdiscPlayer& laserdiscPlayer);
-	void execute(array_ref<TclObject> tokens, TclObject& result,
-	             EmuTime::param time) override;
-	string help(const vector<string>& tokens) const override;
-	void tabCompletion(vector<string>& tokens) const override;
-private:
-	LaserdiscPlayer& laserdiscPlayer;
-};
-
-LaserdiscCommand::LaserdiscCommand(
+LaserdiscPlayer::Command::Command(
 		CommandController& commandController_,
-		StateChangeDistributor& stateChangeDistributor,
-		Scheduler& scheduler, LaserdiscPlayer& laserdiscPlayer_)
-	: RecordedCommand(commandController_, stateChangeDistributor,
-			  scheduler, "laserdiscplayer")
-	, laserdiscPlayer(laserdiscPlayer_)
+		StateChangeDistributor& stateChangeDistributor_,
+		Scheduler& scheduler_)
+	: RecordedCommand(commandController_, stateChangeDistributor_,
+			  scheduler_, "laserdiscplayer")
 {
 }
 
-void LaserdiscCommand::execute(array_ref<TclObject> tokens, TclObject& result,
-                               EmuTime::param time)
+void LaserdiscPlayer::Command::execute(
+	span<const TclObject> tokens, TclObject& result, EmuTime::param time)
 {
+	auto& laserdiscPlayer = OUTER(LaserdiscPlayer, laserdiscCommand);
 	if (tokens.size() == 1) {
 		// Returning Tcl lists here, similar to the disk commands in
 		// DiskChanger
-		result.addListElement(getName() + ':');
-		result.addListElement(laserdiscPlayer.getImageName().getResolved());
-	} else if (tokens.size() == 2 && tokens[1].getString() == "eject") {
-		result.setString("Ejecting laserdisc.");
+		result.addListElement(getName() + ':',
+		                      laserdiscPlayer.getImageName().getResolved());
+	} else if (tokens[1] == "eject") {
+		checkNumArgs(tokens, 2, Prefix{2}, nullptr);
+		result = "Ejecting laserdisc.";
 		laserdiscPlayer.eject(time);
-	} else if (tokens.size() == 3 && tokens[1].getString() == "insert") {
+	} else if (tokens[1] == "insert") {
+		checkNumArgs(tokens, 3, "filename");
 		try {
-			result.setString("Changing laserdisc.");
-			laserdiscPlayer.setImageName(tokens[2].getString().str(), time);
+			result = "Changing laserdisc.";
+			laserdiscPlayer.setImageName(string(tokens[2].getString()), time);
 		} catch (MSXException& e) {
-			throw CommandException(e.getMessage());
+			throw CommandException(std::move(e).getMessage());
 		}
 	} else {
 		throw SyntaxError();
 	}
 }
 
-string LaserdiscCommand::help(const vector<string>& tokens) const
+string LaserdiscPlayer::Command::help(const vector<string>& tokens) const
 {
 	if (tokens.size() >= 2) {
 		if (tokens[1] == "insert") {
@@ -96,18 +82,20 @@ string LaserdiscCommand::help(const vector<string>& tokens) const
 	       ": eject the laserdisc\n";
 }
 
-void LaserdiscCommand::tabCompletion(vector<string>& tokens) const
+void LaserdiscPlayer::Command::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		static const char* const extra[] = { "eject", "insert" };
+		static constexpr const char* const extra[] = { "eject", "insert" };
 		completeString(tokens, extra);
 	} else if (tokens.size() == 3 && tokens[1] == "insert") {
-		completeFileName(tokens, UserFileContext());
+		completeFileName(tokens, userFileContext());
 	}
 }
 
 // LaserdiscPlayer
 
+constexpr unsigned DUMMY_INPUT_RATE = 44100; // actual rate depends on .ogg file
+
 static XMLElement createXML()
 {
 	XMLElement xml("laserdiscplayer");
@@ -118,31 +106,31 @@ static XMLElement createXML()
 LaserdiscPlayer::LaserdiscPlayer(
 		const HardwareConfig& hwConf, PioneerLDControl& ldcontrol_)
 	: ResampledSoundDevice(hwConf.getMotherBoard(), "laserdiscplayer",
-	                       "Laserdisc Player", 1, true)
-	, Schedulable(hwConf.getMotherBoard().getScheduler())
+	                       "Laserdisc Player", 1, DUMMY_INPUT_RATE, true)
+	, syncAck (hwConf.getMotherBoard().getScheduler())
+	, syncOdd (hwConf.getMotherBoard().getScheduler())
+	, syncEven(hwConf.getMotherBoard().getScheduler())
 	, motherBoard(hwConf.getMotherBoard())
 	, ldcontrol(ldcontrol_)
-	, laserdiscCommand(make_unique<LaserdiscCommand>(
-		   motherBoard.getCommandController(),
-		   motherBoard.getStateChangeDistributor(),
-		   motherBoard.getScheduler(),
-		   *this))
-	, sampleClock(EmuTime::zero)
-	, start(EmuTime::zero)
+	, laserdiscCommand(motherBoard.getCommandController(),
+		           motherBoard.getStateChangeDistributor(),
+		           motherBoard.getScheduler())
+	, sampleClock(EmuTime::zero())
+	, start(EmuTime::zero())
 	, muteLeft(false)
 	, muteRight(false)
 	, remoteState(REMOTE_IDLE)
-	, remoteLastEdge(EmuTime::zero)
+	, remoteLastEdge(EmuTime::zero())
 	, remoteLastBit(false)
 	, remoteProtocol(IR_NONE)
 	, ack(false)
 	, seeking(false)
 	, playerState(PLAYER_STOPPED)
-	, autoRunSetting(make_unique<BooleanSetting>(
+	, autoRunSetting(
 		motherBoard.getCommandController(), "autorunlaserdisc",
-		"automatically try to run Laserdisc", true))
-	, loadingIndicator(make_unique<LoadingIndicator>(
-		motherBoard.getReactor().getGlobalSettings().getThrottleManager()))
+		"automatically try to run Laserdisc", true)
+	, loadingIndicator(
+		motherBoard.getReactor().getGlobalSettings().getThrottleManager())
 	, sampleReads(0)
 {
 	motherBoard.getCassettePort().setLaserdiscPlayer(this);
@@ -152,9 +140,7 @@ LaserdiscPlayer::LaserdiscPlayer(
 
 	createRenderer();
 	reactor.getEventDistributor().registerEventListener(OPENMSX_BOOT_EVENT, *this);
-	scheduleDisplayStart(Schedulable::getCurrentTime());
-
-	setInputRate(44100); // Initialize with dummy value
+	scheduleDisplayStart(getCurrentTime());
 
 	static XMLElement xml = createXML();
 	registerSound(DeviceConfig(hwConf, xml));
@@ -173,8 +159,8 @@ void LaserdiscPlayer::scheduleDisplayStart(EmuTime::param time)
 	Clock<60000, 1001> frameClock(time);
 	// The video is 29.97Hz, however we need to do vblank processing
 	// at the full 59.94Hz
-	setSyncPoint(frameClock + 1, ODD_FRAME);
-	setSyncPoint(frameClock + 2, EVEN_FRAME);
+	syncOdd .setSyncPoint(frameClock + 1);
+	syncEven.setSyncPoint(frameClock + 2);
 }
 
 // The protocol used to communicate over the cable for commands to the
@@ -240,10 +226,10 @@ void LaserdiscPlayer::extControl(bool bit, EmuTime::param time)
 		// is, we process the button here. Note that real hardware
 		// needs the trailing pulse to be at least 200µs
 		if (++remoteBitNr == 32) {
-			byte custom	 = ( remoteBits >>  0) & 0xff;
+			byte custom      = ( remoteBits >>  0) & 0xff;
 			byte customCompl = (~remoteBits >>  8) & 0xff;
-			byte code	 = ( remoteBits >> 16) & 0xff;
-			byte codeCompl	 = (~remoteBits >> 24) & 0xff;
+			byte code        = ( remoteBits >> 16) & 0xff;
+			byte codeCompl   = (~remoteBits >> 24) & 0xff;
 			if (custom == customCompl &&
 			    custom == 0xa8 &&
 			    code == codeCompl) {
@@ -260,20 +246,16 @@ void LaserdiscPlayer::extControl(bool bit, EmuTime::param time)
 
 void LaserdiscPlayer::submitRemote(RemoteProtocol protocol, unsigned code)
 {
-	PRT_DEBUG("Laserdisc::submitRemote(" << std::hex << protocol << ", "
-			<< code << ')');
-
 	// The END command for seeking/waiting acknowledges repeats,
 	// Esh's Aurunmilla needs play as well.
 	if (protocol != remoteProtocol || code != remoteCode ||
-	    (protocol == IR_NEC && (code == 0x42 || code == 0x17))) {
+	    (protocol == IR_NEC && (code == one_of(0x42u, 0x17u)))) {
 		remoteProtocol = protocol;
 		remoteCode = code;
 		remoteVblanksBack = 0;
 		remoteExecuteDelayed = true;
 	} else {
-		PRT_DEBUG("Laserdisc::remote ignored after " << std::dec
-			  << remoteVblanksBack << " vblanks");
+		// remote ignored
 		remoteVblanksBack = 0;
 		remoteExecuteDelayed = false;
 	}
@@ -286,9 +268,9 @@ const RawFrame* LaserdiscPlayer::getRawFrame() const
 
 void LaserdiscPlayer::setAck(EmuTime::param time, int wait)
 {
-	PRT_DEBUG("Laserdisc::Lowering ACK for " << std::dec << wait << "ms");
-	removeSyncPoint(ACK);
-	setSyncPoint(time + EmuDuration::msec(wait), ACK);
+	// activate ACK for 'wait' milliseconds
+	syncAck.removeSyncPoint();
+	syncAck.setSyncPoint(time + EmuDuration::msec(wait));
 	ack = true;
 }
 
@@ -332,9 +314,9 @@ void LaserdiscPlayer::remoteButtonNEC(unsigned code, EmuTime::param time)
 	}
 
 	if (!f.empty()) {
-		PRT_DEBUG("LaserdiscPlayer::remote " << f);
+		std::cerr << "LaserdiscPlayer::remote " << f << '\n';
 	} else {
-		PRT_DEBUG("LaserdiscPlayer::remote unknown " << std::hex << code);
+		std::cerr << "LaserdiscPlayer::remote unknown " << std::hex << code << '\n';
 	}
 #endif
 	// When not playing the following buttons work
@@ -343,7 +325,7 @@ void LaserdiscPlayer::remoteButtonNEC(unsigned code, EmuTime::param time)
 	// 0x49, 0x4a, 0x4b (ack sent)
 	// if 0x49 is a repeat then no ACK is sent
 	// if 0x49 is followed by 0x4a then ACK is sent
-	if (code == 0x49 || code == 0x4a || code == 0x4b) {
+	if (code == one_of(0x49u, 0x4au, 0x4bu)) {
 		updateStream(time);
 
 		switch (code) {
@@ -391,7 +373,7 @@ void LaserdiscPlayer::remoteButtonNEC(unsigned code, EmuTime::param time)
 		case 0x40:
 			seekState = SEEK_CHAPTER;
 			seekNum = 0;
-			nonseekack = video->chapter(0) != 0;
+			nonseekack = video->getChapter(0) != 0;
 			break;
 		case 0x00:
 		case 0x01:
@@ -421,12 +403,10 @@ void LaserdiscPlayer::remoteButtonNEC(unsigned code, EmuTime::param time)
 				seekState = SEEK_NONE;
 				waitFrame = seekNum % 100000;
 				if (waitFrame >= 101 && waitFrame < 200) {
-					auto frame = video->chapter(
+					auto frame = video->getChapter(
 						int(waitFrame - 100));
 					if (frame) waitFrame = frame;
 				}
-				PRT_DEBUG("Wait frame set to " << std::dec <<
-								waitFrame);
 				break;
 			default:
 				seekState = SEEK_NONE;
@@ -495,7 +475,7 @@ void LaserdiscPlayer::remoteButtonNEC(unsigned code, EmuTime::param time)
 		default:
 			motherBoard.getMSXCliComm().printWarning(
 				"The Laserdisc player received an unknown "
-				"command 0x" + StringOp::toHexString(code, 2));
+				"command 0x", hex_string<2>(code));
 			nonseekack = false;
 			break;
 		}
@@ -508,49 +488,41 @@ void LaserdiscPlayer::remoteButtonNEC(unsigned code, EmuTime::param time)
 	}
 }
 
-void LaserdiscPlayer::executeUntil(EmuTime::param time, int userdata)
+void LaserdiscPlayer::execSyncAck(EmuTime::param time)
 {
 	updateStream(time);
 
-	switch (userdata) {
-	case ACK:
-		if (seeking && playerState == PLAYER_PLAYING) {
-			sampleClock.advance(time);
-		}
+	if (seeking && playerState == PLAYER_PLAYING) {
+		sampleClock.advance(time);
+	}
 
-		if (seeking) {
-			PRT_DEBUG("Laserdisc: seek complete");
-		}
+	ack = false;
+	seeking = false;
+}
 
-		ack = false;
-		seeking = false;
-		PRT_DEBUG("Laserdisc: ACK cleared");
-		break;
-	case ODD_FRAME:
-		if (!video || video->getFrameRate() != 60)
-			break;
+void LaserdiscPlayer::execSyncFrame(EmuTime::param time, bool odd)
+{
+	updateStream(time);
 
-	case EVEN_FRAME:
+	if (!odd || (video && video->getFrameRate() == 60)) {
 		if ((playerState != PLAYER_STOPPED) &&
 		    (currentFrame > video->getFrames())) {
 			playerState = PLAYER_STOPPED;
 		}
 
-		if (RawFrame* rawFrame = renderer->getRawFrame()) {
+		if (auto* rawFrame = renderer->getRawFrame()) {
 			renderer->frameStart(time);
 
 			if (isVideoOutputAvailable(time)) {
 				auto frame = currentFrame;
 				if (video->getFrameRate() == 60) {
 					frame *= 2;
-					if (userdata == ODD_FRAME) {
-						frame--;
-					}
+					if (odd) frame--;
 				}
 
 				video->getFrameNo(*rawFrame, frame);
 
-				if (userdata == EVEN_FRAME) {
+				if (!odd) {
 					nextFrame(time);
 				}
 			} else {
@@ -560,28 +532,26 @@ void LaserdiscPlayer::executeUntil(EmuTime::param time, int userdata)
 		}
 
 		// Update throttling
-		loadingIndicator->update(seeking || sampleReads > 500);
+		loadingIndicator.update(seeking || sampleReads > 500);
 		sampleReads = 0;
 
-		if (userdata == EVEN_FRAME) {
+		if (!odd) {
 			scheduleDisplayStart(time);
 		}
 	}
 
-	if (userdata == EVEN_FRAME || userdata == ODD_FRAME) {
-		// Processing of the remote control happens at each frame
-		// (even and odd, so at 59.94Hz)
-		if (remoteProtocol == IR_NEC) {
-			if (remoteExecuteDelayed) {
-				remoteButtonNEC(remoteCode, time);
-			}
+	// Processing of the remote control happens at each frame
+	// (even and odd, so at 59.94Hz)
+	if (remoteProtocol == IR_NEC) {
+		if (remoteExecuteDelayed) {
+			remoteButtonNEC(remoteCode, time);
+		}
 
-			if (++remoteVblanksBack > 6) {
-				remoteProtocol = IR_NONE;
-			}
+		if (++remoteVblanksBack > 6) {
+			remoteProtocol = IR_NONE;
 		}
-		remoteExecuteDelayed = false;
 	}
+	remoteExecuteDelayed = false;
 }
 
 void LaserdiscPlayer::setFrameStep()
@@ -616,9 +586,6 @@ void LaserdiscPlayer::setFrameStep()
 void LaserdiscPlayer::nextFrame(EmuTime::param time)
 {
 	if (waitFrame && waitFrame == currentFrame) {
-		PRT_DEBUG("LaserdiscPlayer: wait frame " << std::dec <<
-						waitFrame << " reached");
-
 		// Leave ACK raised until the next command
 		ack = true;
 		waitFrame = 0;
@@ -652,11 +619,9 @@ void LaserdiscPlayer::nextFrame(EmuTime::param time)
 	}
 
 	// freeze if stop frame
-	if ((playerState == PLAYER_PLAYING || playerState == PLAYER_MULTISPEED)
+	if ((playerState == one_of(PLAYER_PLAYING, PLAYER_MULTISPEED))
 	     && video->stopFrame(currentFrame)) {
-		PRT_DEBUG("LaserdiscPlayer: stopFrame " << std::dec <<
-						currentFrame << " reached");
-
+		// stop frame reached
 		playingFromSample = getCurrentSample(time);
 		playerState = PLAYER_STILL;
 	}
@@ -665,8 +630,8 @@ void LaserdiscPlayer::nextFrame(EmuTime::param time)
 void LaserdiscPlayer::setImageName(string newImage, EmuTime::param time)
 {
 	stop(time);
-	oggImage = Filename(std::move(newImage), UserFileContext());
-	video = make_unique<OggReader>(oggImage, motherBoard.getMSXCliComm());
+	oggImage = Filename(std::move(newImage), userFileContext());
+	video = std::make_unique<OggReader>(oggImage, motherBoard.getMSXCliComm());
 
 	unsigned inputRate = video->getSampleRate();
 	sampleClock.setFreq(inputRate);
@@ -686,27 +651,24 @@ int LaserdiscPlayer::signalEvent(const std::shared_ptr<const Event>& event)
 
 void LaserdiscPlayer::autoRun()
 {
-	if (!autoRunSetting->getBoolean()) {
-		return;
-	}
+	if (!autoRunSetting.getBoolean()) return;
 
 	string var = "::auto_run_ld_counter";
-	string command =
-		"if ![info exists " + var + "] { set " + var + " 0 }\n"
-		"incr " + var + "\n"
-		"after time 2 \"if $" + var + "==\\$" + var + " { "
-		"type 1CALLLD\\\\r }\"";
-
+	string command = strCat(
+		"if ![info exists ", var, "] { set ", var, " 0 }\n"
+		"incr ", var, "\n"
+		"after time 2 \"if $", var, "==\\$", var, " { "
+		"type_via_keyboard 1CALLLD\\\\r }\"");
 	try {
 		motherBoard.getCommandController().executeCommand(command);
 	} catch (CommandException& e) {
 		motherBoard.getMSXCliComm().printWarning(
-			"Error executing loading instruction for AutoRun: " +
-			e.getMessage() + "\n Please report a bug.");
+			"Error executing loading instruction for AutoRun: ",
+			e.getMessage(), "\n Please report a bug.");
 	}
 }
 
-void LaserdiscPlayer::generateChannels(int** buffers, unsigned num)
+void LaserdiscPlayer::generateChannels(float** buffers, unsigned num)
 {
 	// Single channel device: replace content of buffers[0] (not add to it).
 	if (playerState != PLAYER_PLAYING || seeking || (muteLeft && muteRight)) {
@@ -727,8 +689,8 @@ void LaserdiscPlayer::generateChannels(int** buffers, unsigned num)
 		}
 
 		for (/**/; pos < len; ++pos) {
-			buffers[0][pos * 2 + 0] = 0;
-			buffers[0][pos * 2 + 1] = 0;
+			buffers[0][pos * 2 + 0] = 0.0f;
+			buffers[0][pos * 2 + 1] = 0.0f;
 		}
 
 		currentSample = playingFromSample;
@@ -739,9 +701,8 @@ void LaserdiscPlayer::generateChannels(int** buffers, unsigned num)
 	unsigned drift = video->getSampleRate() / 30;
 
 	if (currentSample > (lastPlayedSample + drift) ||
-			(currentSample + drift) < lastPlayedSample) {
-		PRT_DEBUG("Laserdisc audio drift: " << std::dec <<
-				lastPlayedSample << ' ' << currentSample);
+	    (currentSample + drift) < lastPlayedSample) {
+		// audio drift
 		lastPlayedSample = currentSample;
 	}
 
@@ -755,9 +716,11 @@ void LaserdiscPlayer::generateChannels(int** buffers, unsigned num)
 			if (pos == 0) {
 				buffers[0] = nullptr;
 				break;
-			} else for (/**/; pos < num; ++pos) {
-				buffers[0][pos * 2 + 0] = 0;
-				buffers[0][pos * 2 + 1] = 0;
+			} else {
+				for (/**/; pos < num; ++pos) {
+					buffers[0][pos * 2 + 0] = 0.0f;
+					buffers[0][pos * 2 + 1] = 0.0f;
+				}
 			}
 		} else {
 			auto offset = unsigned(lastPlayedSample - audio->position);
@@ -765,10 +728,10 @@ void LaserdiscPlayer::generateChannels(int** buffers, unsigned num)
 
 			// maybe muting should be moved out of the loop?
 			for (unsigned i = 0; i < len; ++i, ++pos) {
-				buffers[0][pos * 2 + 0] = muteLeft ? 0 :
-				   int(audio->pcm[left][offset + i] * 65536.f);
-				buffers[0][pos * 2 + 1] = muteRight ? 0 :
-				   int(audio->pcm[right][offset + i] * 65536.f);
+				buffers[0][pos * 2 + 0] = muteLeft ? 0.0f :
+				   audio->pcm[left][offset + i];
+				buffers[0][pos * 2 + 1] = muteRight ? 0.0f :
+				   audio->pcm[right][offset + i];
 			}
 
 			lastPlayedSample += len;
@@ -776,7 +739,12 @@ void LaserdiscPlayer::generateChannels(int** buffers, unsigned num)
 	}
 }
 
-bool LaserdiscPlayer::updateBuffer(unsigned length, int* buffer,
+float LaserdiscPlayer::getAmplificationFactorImpl() const
+{
+	return 2.0f;
+}
+
+bool LaserdiscPlayer::updateBuffer(unsigned length, float* buffer,
                                    EmuTime::param time)
 {
 	bool result = ResampledSoundDevice::updateBuffer(length, buffer, time);
@@ -787,22 +755,17 @@ bool LaserdiscPlayer::updateBuffer(unsigned length, int* buffer,
 void LaserdiscPlayer::setMuting(bool left, bool right, EmuTime::param time)
 {
 	updateStream(time);
-	PRT_DEBUG("Laserdisc::setMuting L:" << (left  ? "on" : "off")
-				   << " R:" << (right ? "on" : "off"));
 	muteLeft = left;
 	muteRight = right;
 }
 
 void LaserdiscPlayer::play(EmuTime::param time)
 {
-	PRT_DEBUG("Laserdisc::Play");
-
 	if (video) {
 		updateStream(time);
 
 		if (seeking) {
-			// Do not ACK
-			PRT_DEBUG("play while seeking");
+			// Do not ACK, play while seeking
 		} else if (playerState == PLAYER_STOPPED) {
 			// Disk needs to spin up, which takes 9.6s on
 			// my Pioneer LD-92000. Also always seek to
@@ -853,8 +816,6 @@ size_t LaserdiscPlayer::getCurrentSample(EmuTime::param time)
 void LaserdiscPlayer::pause(EmuTime::param time)
 {
 	if (playerState != PLAYER_STOPPED) {
-		PRT_DEBUG("Laserdisc::Pause");
-
 		updateStream(time);
 
 		if (playerState == PLAYER_PLAYING) {
@@ -873,10 +834,7 @@ void LaserdiscPlayer::pause(EmuTime::param time)
 void LaserdiscPlayer::stop(EmuTime::param time)
 {
 	if (playerState != PLAYER_STOPPED) {
-		PRT_DEBUG("Laserdisc::Stop");
-
 		updateStream(time);
-
 		playerState = PLAYER_STOPPED;
 	}
 }
@@ -923,75 +881,58 @@ void LaserdiscPlayer::stepFrame(bool forwards)
 
 void LaserdiscPlayer::seekFrame(size_t toframe, EmuTime::param time)
 {
-	if (playerState != PLAYER_STOPPED) {
-		PRT_DEBUG("Laserdisc::SeekFrame " << std::dec << toframe);
+	if ((playerState != PLAYER_STOPPED) && video) {
+		updateStream(time);
 
-		if (seeking) {
-			PRT_DEBUG("FIXME: seek command while still seeking");
+		if (toframe <= 0) toframe = 1;
+		if (toframe > video->getFrames()) toframe = video->getFrames();
+
+		// Seek time needs to be emulated correctly since
+		// e.g. Astron Belt does not wait for the seek
+		// to complete, it simply assumes a certain
+		// delay.
+		//
+		// This calculation is based on measurements on
+		// a Pioneer LD-92000.
+		auto dist = std::abs(int64_t(toframe) - int64_t(currentFrame));
+		int seektime; // time in ms
+
+		if (dist < 1000) {
+			seektime = dist + 300;
+		} else {
+			seektime = 1800 + dist / 12;
 		}
 
-		if (video) {
-			updateStream(time);
+		int64_t samplePos = (toframe - 1ll) * 1001ll *
+				    video->getSampleRate() / 30000ll;
 
-			if (toframe <= 0)  {
-				toframe = 1;
-			}
-
-			if (toframe > video->getFrames()) {
-				toframe = video->getFrames();
-			}
-
-			// Seek time needs to be emulated correctly since
-			// e.g. Astron Belt does not wait for the seek
-			// to complete, it simply assumes a certain
-			// delay.
-			//
-			// This calculation is based on measurements on
-			// a Pioneer LD-92000.
-			auto dist = std::abs(int64_t(toframe) - int64_t(currentFrame));
-			int seektime; // time in ms
-
-			if (dist < 1000) {
-				seektime = dist + 300;
-			} else {
-				seektime = 1800 + dist / 12;
-			}
-
-			int64_t samplePos = (toframe - 1ll) * 1001ll *
-			                    video->getSampleRate() / 30000ll;
-
-			if (video->getFrameRate() == 60)
-				video->seek(toframe * 2, samplePos);
-			else
-				video->seek(toframe, samplePos);
-
-			playerState = PLAYER_STILL;
-			playingFromSample = samplePos;
-			currentFrame = toframe;
-
-			// Seeking clears the frame to wait for
-			waitFrame = 0;
-
-			seeking = true;
-			setAck(time, seektime);
+		if (video->getFrameRate() == 60) {
+			video->seek(toframe * 2, samplePos);
+		} else {
+			video->seek(toframe, samplePos);
 		}
+		playerState = PLAYER_STILL;
+		playingFromSample = samplePos;
+		currentFrame = toframe;
+
+		// Seeking clears the frame to wait for
+		waitFrame = 0;
+
+		seeking = true;
+		setAck(time, seektime);
 	}
 }
 
 void LaserdiscPlayer::seekChapter(int chapter, EmuTime::param time)
 {
-	if (playerState != PLAYER_STOPPED) {
-		if (video) {
-			auto frameno = video->chapter(chapter);
-			if (!frameno) {
-				return;
-			}
-			seekFrame(frameno, time);
-		}
+	if ((playerState != PLAYER_STOPPED) && video) {
+		auto frameno = video->getChapter(chapter);
+		if (!frameno) return;
+		seekFrame(frameno, time);
 	}
 }
 
-short LaserdiscPlayer::readSample(EmuTime::param time)
+int16_t LaserdiscPlayer::readSample(EmuTime::param time)
 {
 	// Here we should return the value of the sample on the
 	// right audio channel, ignoring muting (this is done in the MSX)
@@ -1045,7 +986,7 @@ void LaserdiscPlayer::createRenderer()
 	renderer = RendererFactory::createLDRenderer(*this, display);
 }
 
-static enum_string<LaserdiscPlayer::RemoteState> RemoteStateInfo[] = {
+static std::initializer_list<enum_string<LaserdiscPlayer::RemoteState>> RemoteStateInfo = {
 	{ "IDLE",		LaserdiscPlayer::REMOTE_IDLE		},
 	{ "HEADER_PULSE",	LaserdiscPlayer::REMOTE_HEADER_PULSE	},
 	{ "NEC_HEADER_SPACE",	LaserdiscPlayer::NEC_HEADER_SPACE	},
@@ -1054,7 +995,7 @@ static enum_string<LaserdiscPlayer::RemoteState> RemoteStateInfo[] = {
 };
 SERIALIZE_ENUM(LaserdiscPlayer::RemoteState, RemoteStateInfo);
 
-static enum_string<LaserdiscPlayer::PlayerState> PlayerStateInfo[] = {
+static std::initializer_list<enum_string<LaserdiscPlayer::PlayerState>> PlayerStateInfo = {
 	{ "STOPPED",		LaserdiscPlayer::PLAYER_STOPPED		},
 	{ "PLAYING",		LaserdiscPlayer::PLAYER_PLAYING		},
 	{ "MULTISPEED",		LaserdiscPlayer::PLAYER_MULTISPEED	},
@@ -1063,7 +1004,7 @@ static enum_string<LaserdiscPlayer::PlayerState> PlayerStateInfo[] = {
 };
 SERIALIZE_ENUM(LaserdiscPlayer::PlayerState, PlayerStateInfo);
 
-static enum_string<LaserdiscPlayer::SeekState> SeekStateInfo[] = {
+static std::initializer_list<enum_string<LaserdiscPlayer::SeekState>> SeekStateInfo = {
 	{ "NONE",		LaserdiscPlayer::SEEK_NONE		},
 	{ "CHAPTER",		LaserdiscPlayer::SEEK_CHAPTER		},
 	{ "FRAME",		LaserdiscPlayer::SEEK_FRAME		},
@@ -1071,14 +1012,14 @@ static enum_string<LaserdiscPlayer::SeekState> SeekStateInfo[] = {
 };
 SERIALIZE_ENUM(LaserdiscPlayer::SeekState, SeekStateInfo);
 
-static enum_string<LaserdiscPlayer::StereoMode> StereoModeInfo[] = {
+static std::initializer_list<enum_string<LaserdiscPlayer::StereoMode>> StereoModeInfo = {
 	{ "LEFT",		LaserdiscPlayer::LEFT			},
 	{ "RIGHT",		LaserdiscPlayer::RIGHT			},
 	{ "STEREO",		LaserdiscPlayer::STEREO			}
 };
 SERIALIZE_ENUM(LaserdiscPlayer::StereoMode, StereoModeInfo);
 
-static enum_string<LaserdiscPlayer::RemoteProtocol> RemoteProtocolInfo[] = {
+static std::initializer_list<enum_string<LaserdiscPlayer::RemoteProtocol>> RemoteProtocolInfo = {
 	{ "NONE",		LaserdiscPlayer::IR_NONE		},
 	{ "NEC",		LaserdiscPlayer::IR_NEC			},
 };
@@ -1087,30 +1028,31 @@ SERIALIZE_ENUM(LaserdiscPlayer::RemoteProtocol, RemoteProtocolInfo);
 // version 1: initial version
 // version 2: added 'stillOnWaitFrame'
 // version 3: reversed bit order of 'remoteBits' and 'remoteCode'
+// version 4: removed 'userData' from Schedulable
 template<typename Archive>
 void LaserdiscPlayer::serialize(Archive& ar, unsigned version)
 {
 	// Serialize remote control
 	ar.serialize("RemoteState", remoteState);
 	if (remoteState != REMOTE_IDLE) {
-		ar.serialize("RemoteBitNr", remoteBitNr);
-		ar.serialize("RemoteBits", remoteBits);
+		ar.serialize("RemoteBitNr", remoteBitNr,
+		             "RemoteBits",  remoteBits);
 		if (ar.versionBelow(version, 3)) {
 			assert(ar.isLoader());
 			remoteBits = Math::reverseNBits(remoteBits, remoteBitNr);
 		}
 	}
-	ar.serialize("RemoteLastBit", remoteLastBit);
-	ar.serialize("RemoteLastEdge", remoteLastEdge);
-	ar.serialize("RemoteProtocol", remoteProtocol);
+	ar.serialize("RemoteLastBit",  remoteLastBit,
+	             "RemoteLastEdge", remoteLastEdge,
+	             "RemoteProtocol", remoteProtocol);
 	if (remoteProtocol != IR_NONE) {
 		ar.serialize("RemoteCode", remoteCode);
 		if (ar.versionBelow(version, 3)) {
 			assert(ar.isLoader());
 			remoteCode = Math::reverseByte(remoteCode);
 		}
-		ar.serialize("RemoteExecuteDelayed", remoteExecuteDelayed);
-		ar.serialize("RemoteVblanksBack", remoteVblanksBack);
+		ar.serialize("RemoteExecuteDelayed", remoteExecuteDelayed,
+		             "RemoteVblanksBack",    remoteVblanksBack);
 	}
 
 	// Serialize filename
@@ -1141,8 +1083,8 @@ void LaserdiscPlayer::serialize(Archive& ar, unsigned version)
 			ar.serialize("StillOnWaitFrame", stillOnWaitFrame);
 		}
 
-		ar.serialize("ACK", ack);
-		ar.serialize("PlayingSpeed", playingSpeed);
+		ar.serialize("ACK",          ack,
+		             "PlayingSpeed", playingSpeed);
 
 		// Frame position
 		ar.serialize("CurrentFrame", currentFrame);
@@ -1151,9 +1093,9 @@ void LaserdiscPlayer::serialize(Archive& ar, unsigned version)
 		}
 
 		// Audio position
-		ar.serialize("StereoMode", stereoMode);
-		ar.serialize("FromSample", playingFromSample);
-		ar.serialize("SampleClock", sampleClock);
+		ar.serialize("StereoMode",  stereoMode,
+		             "FromSample",  playingFromSample,
+		             "SampleClock", sampleClock);
 
 		if (ar.isLoader()) {
 			// If the samplerate differs, adjust accordingly
@@ -1176,7 +1118,13 @@ void LaserdiscPlayer::serialize(Archive& ar, unsigned version)
 		}
 	}
 
-	ar.template serializeBase<Schedulable>(*this);
+	if (ar.versionAtLeast(version, 4)) {
+		ar.serialize("syncEven", syncEven,
+		             "syncOdd",  syncOdd,
+		             "syncAck",  syncAck);
+	} else {
+		Schedulable::restoreOld(ar, {&syncEven, &syncOdd, &syncAck});
+	}
 
 	if (ar.isLoader()) {
 		isVideoOutputAvailable(getCurrentTime());
