@@ -1,17 +1,18 @@
 #include "OggReader.hh"
-#include "File.hh"
 #include "MSXException.hh"
 #include "yuv2rgb.hh"
 #include "likely.hh"
 #include "CliComm.hh"
-#include "StringOp.hh"
 #include "MemoryOps.hh"
-#include "memory.hh"
+#include "one_of.hh"
+#include "ranges.hh"
 #include "stl.hh"
 #include "stringsp.hh" // for strncasecmp
-#include <algorithm>
+#include "view.hh"
 #include <cstring> // for memcpy, memcmp
 #include <cstdlib> // for atoi
+#include <cctype> // for isspace
+#include <memory>
 
 // TODO
 // - Improve error handling
@@ -45,7 +46,7 @@ Frame::~Frame()
 
 OggReader::OggReader(const Filename& filename, CliComm& cli_)
 	: cli(cli_)
-	, file(make_unique<File>(filename))
+	, file(filename)
 {
 	audioSerial = -1;
 	videoSerial = -1;
@@ -71,7 +72,7 @@ OggReader::OggReader(const Filename& filename, CliComm& cli_)
 
 	state = PLAYING;
 	fileOffset = 0;
-	fileSize = file->getSize();
+	fileSize = file.getSize();
 
 	ogg_page page;
 
@@ -238,9 +239,9 @@ OggReader::~OggReader()
 void OggReader::vorbisFoundPosition()
 {
 	auto last = vorbisPos;
-	for (auto it = audioList.rbegin(); it != audioList.rend(); ++it) {
-		last -= (*it)->length;
-		(*it)->position = last;
+	for (auto& audioFrag : view::reverse(audioList)) {
+		last -= audioFrag->length;
+		audioFrag->position = last;
 	}
 
 	// last is now the first vorbis audio decoded
@@ -362,7 +363,7 @@ void OggReader::readVorbis(ogg_packet* packet)
 	while (pos < decoded)  {
 		// Find memory to copy PCM into
 		if (recycleAudioList.empty()) {
-			auto audio = make_unique<AudioFragment>();
+			auto audio = std::make_unique<AudioFragment>();
 			audio->length = 0;
 			recycleAudioList.push_back(std::move(audio));
 		}
@@ -408,12 +409,9 @@ void OggReader::readVorbis(ogg_packet* packet)
 			vorbisFoundPosition();
 		} else {
 			if (vorbisPos != size_t(packet->granulepos)) {
-				cli.printWarning("vorbis audio out of sync, "
-					"expected " +
-					StringOp::toString(vorbisPos) +
-					", got " +
-					StringOp::toString(packet->granulepos));
-
+				cli.printWarning(
+					"vorbis audio out of sync, expected ",
+					vorbisPos, ", got ", packet->granulepos);
 				vorbisPos = packet->granulepos;
 			}
 		}
@@ -423,13 +421,13 @@ void OggReader::readVorbis(ogg_packet* packet)
 	vorbis_synthesis_read(&vd, decoded);
 }
 
-size_t OggReader::frameNo(ogg_packet* packet)
+size_t OggReader::frameNo(ogg_packet* packet) const
 {
 	if (packet->granulepos == -1) {
 		return size_t(-1);
 	}
 
-	size_t intra = packet->granulepos & ((1 << granuleShift) - 1);
+	size_t intra = packet->granulepos & ((size_t(1) << granuleShift) - 1);
 	size_t key = packet->granulepos >> granuleShift;
 	return key + intra;
 }
@@ -451,6 +449,9 @@ void OggReader::readMetadata(th_comment& tc)
 	// Maybe there is a better way of doing this parsing in C++
 	char* p = metadata;
 	while (p) {
+		while (isspace(*p)) {
+			p++;
+		}
 		if (strncasecmp(p, "chapter: ", 9) == 0) {
 			int chapter = atoi(p + 9);
 			p = strchr(p, ',');
@@ -469,8 +470,8 @@ void OggReader::readMetadata(th_comment& tc)
 		p = strchr(p, '\n');
 		if (p) ++p;
 	}
-	sort(begin(stopFrames), end(stopFrames));
-	sort(begin(chapters  ), end(chapters  ), LessTupleElement<0>());
+	ranges::sort(stopFrames);
+	ranges::sort(chapters, LessTupleElement<0>());
 }
 
 void OggReader::readTheora(ogg_packet* packet)
@@ -547,8 +548,7 @@ void OggReader::readTheora(ogg_packet* packet)
 	case 0:
 		break;
 	default:
-		cli.printWarning("Theora error: unknown error " +
-					StringOp::toString(rc));
+		cli.printWarning("Theora error: unknown error ", rc);
 		break;
 	}
 
@@ -570,7 +570,7 @@ void OggReader::readTheora(ogg_packet* packet)
 
 	std::unique_ptr<Frame> frame;
 	if (recycleFrameList.empty()) {
-		frame = make_unique<Frame>(yuv);
+		frame = std::make_unique<Frame>(yuv);
 	} else {
 		frame = std::move(recycleFrameList.back());
 		recycleFrameList.pop_back();
@@ -587,8 +587,7 @@ void OggReader::readTheora(ogg_packet* packet)
 	// postion
 	Frame* last = frameList.empty() ? nullptr : frameList.back().get();
 	if (last && (last->no != size_t(-1))) {
-		if ((frameno != size_t(-1)) &&
-		    (frameno != last->no + last->length)) {
+		if (frameno != one_of(size_t(-1), last->no + last->length)) {
 			cli.printWarning("Theora frame sequence wrong");
 		} else {
 			frameno = last->no + last->length;
@@ -603,10 +602,9 @@ void OggReader::readTheora(ogg_packet* packet)
 	// numbers correctly
 	if (!frameList.empty() && (frameno != size_t(-1)) &&
 	    (frameList[0]->no == size_t(-1))) {
-		for (auto it = frameList.rbegin();
-		     it != frameList.rend(); ++it) {
-			frameno -= (*it)->length;
-			(*it)->no = frameno;
+		for (auto& frm : view::reverse(frameList)) {
+			frameno -= frm->length;
+			frm->no = frameno;
 		}
 	}
 
@@ -637,9 +635,9 @@ void OggReader::getFrameNo(RawFrame& rawFrame, size_t frameno)
 		if (!frameList.empty() && frameList[0]->no > frameno) {
 			// we're missing frames!
 			frame = frameList[0].get();
-			cli.printWarning("Cannot find frame " +
-				StringOp::toString(frameno) + " using " +
-				StringOp::toString(frame->no) + " instead");
+			cli.printWarning(
+					"Cannot find frame ", frameno, " using ",
+			        frame->no, " instead");
 			break;
 		}
 
@@ -658,11 +656,10 @@ void OggReader::getFrameNo(RawFrame& rawFrame, size_t frameno)
 		}
 
 		// Sanity check, should not happen
-		if (frameList.size() > (2u << granuleShift)) {
+		if (frameList.size() > (size_t(2) << granuleShift)) {
 			// We've got more than twice as many frames
 			// as the maximum distance between key frames.
-			cli.printWarning("Cannot find frame " +
-				StringOp::toString(frameno));
+			cli.printWarning("Cannot find frame ", frameno);
 			return;
 		}
 
@@ -762,8 +759,8 @@ bool OggReader::nextPacket()
 				cli.printWarning("Failed to submit theora page");
 			}
 		} else if (serial != skeletonSerial) {
-			cli.printWarning("Unexpected stream with serial " +
-				StringOp::toString(serial) + " in ogg file");
+			cli.printWarning("Unexpected stream with serial ",
+			                 serial, " in ogg file");
 		}
 	}
 }
@@ -771,7 +768,7 @@ bool OggReader::nextPacket()
 
 bool OggReader::nextPage(ogg_page* page)
 {
-	static const size_t CHUNK = 4096;
+	constexpr size_t CHUNK = 4096;
 
 	int ret;
 	while ((ret = ogg_sync_pageseek(&sync, page)) <= 0) {
@@ -789,7 +786,7 @@ bool OggReader::nextPage(ogg_page* page)
 		}
 
 		char* buffer = ogg_sync_buffer(&sync, long(chunk));
-		file->read(buffer, chunk);
+		file.read(buffer, chunk);
 		fileOffset += chunk;
 
 		if (ogg_sync_wrote(&sync, long(chunk)) == -1) {
@@ -806,7 +803,7 @@ size_t OggReader::bisection(
 {
 	// Defined to be a power-of-two such that the arthmetic can be done faster.
 	// Note that the sample-number is in the range of: 1..(44100*60*60)
-	static const uint64_t SHIFT = 0x20000000ull;
+	constexpr uint64_t SHIFT = 0x20000000ull;
 
 	uint64_t offsetA = 0, offsetB = maxOffset;
 	uint64_t sampleA = 0, sampleB = maxSamples;
@@ -826,7 +823,7 @@ size_t OggReader::bisection(
 		uint64_t sampleOffset = ratio * (offsetB - offsetA) / SHIFT + offsetA;
 		auto offset = std::min(sampleOffset, frameOffset);
 
-		file->seek(offset);
+		file.seek(offset);
 		fileOffset = offset;
 		ogg_sync_reset(&sync);
 		currentFrame = size_t(-1);
@@ -858,14 +855,14 @@ size_t OggReader::bisection(
 
 size_t OggReader::findOffset(size_t frame, size_t sample)
 {
-	static const size_t STEP = 32 * 1024;
+	constexpr size_t STEP = 32 * 1024;
 
 	// first calculate total length in bytes, samples and frames
 
 	// The file might have changed since we last requested its size,
 	// we assume that only data will be added to it and the ogg streams
 	// are exactly as before
-	fileSize = file->getSize();
+	fileSize = file.getSize();
 	auto offset = fileSize - 1;
 
 	while (offset > 0) {
@@ -875,7 +872,7 @@ size_t OggReader::findOffset(size_t frame, size_t sample)
 			offset = 0;
 		}
 
-		file->seek(offset);
+		file.seek(offset);
 		fileOffset = offset;
 		ogg_sync_reset(&sync);
 		currentFrame = size_t(-1);
@@ -915,7 +912,7 @@ size_t OggReader::findOffset(size_t frame, size_t sample)
 	offset = bisection(frame, sample, maxOffset, maxSamples, maxFrames);
 
 	// Find key frame
-	file->seek(offset);
+	file.seek(offset);
 	fileOffset = offset;
 	ogg_sync_reset(&sync);
 	currentFrame = frame;
@@ -929,7 +926,7 @@ size_t OggReader::findOffset(size_t frame, size_t sample)
 
 	state = PLAYING;
 
-	if ((keyFrame == size_t(-1)) || (frame == keyFrame)) {
+	if (keyFrame == one_of(size_t(-1), frame)) {
 		return offset;
 	}
 
@@ -940,8 +937,8 @@ bool OggReader::seek(size_t frame, size_t samples)
 {
 	// Remove all queued frames
 	recycleFrameList.insert(end(recycleFrameList),
-		make_move_iterator(begin(frameList)),
-		make_move_iterator(end  (frameList)));
+		std::move_iterator(begin(frameList)),
+		std::move_iterator(end  (frameList)));
 	frameList.clear();
 
 	// Remove all queued audio
@@ -954,7 +951,7 @@ bool OggReader::seek(size_t frame, size_t samples)
 	audioList.clear();
 
 	fileOffset = findOffset(frame, samples);
-	file->seek(fileOffset);
+	file.seek(fileOffset);
 
 	ogg_sync_reset(&sync);
 
@@ -969,13 +966,12 @@ bool OggReader::seek(size_t frame, size_t samples)
 
 bool OggReader::stopFrame(size_t frame) const
 {
-	return std::binary_search(begin(stopFrames), end(stopFrames), frame);
+	return ranges::binary_search(stopFrames, frame);
 }
 
-size_t OggReader::chapter(int chapterNo) const
+size_t OggReader::getChapter(int chapterNo) const
 {
-	auto it = std::lower_bound(begin(chapters), end(chapters),
-	                           chapterNo, LessTupleElement<0>());
+	auto it = ranges::lower_bound(chapters, chapterNo, LessTupleElement<0>());
 	return ((it != end(chapters)) && (it->first == chapterNo))
 		? it->second : 0;
 }
