@@ -1,12 +1,11 @@
 #include "PioneerLDControl.hh"
-#include "Rom.hh"
 #include "CacheLine.hh"
 #include "serialize.hh"
 #include "LaserdiscPlayer.hh"
 #include "MSXPPI.hh"
 #include "MSXException.hh"
 #include "VDP.hh"
-#include "memory.hh"
+#include <memory>
 
 namespace openmsx {
 
@@ -34,13 +33,13 @@ namespace openmsx {
  */
 PioneerLDControl::PioneerLDControl(const DeviceConfig& config)
 	: MSXDevice(config)
-	, rom(make_unique<Rom>(getName() + " ROM", "rom", config))
-	, clock(EmuTime::zero)
+	, rom(getName() + " ROM", "rom", config)
+	, clock(EmuTime::zero())
 	, irq(getMotherBoard(), "PioneerLDControl.IRQdisplayoff")
 	, videoEnabled(false)
 {
 	if (config.getChildDataAsBool("laserdisc", true)) {
-		laserdisc = make_unique<LaserdiscPlayer>(
+		laserdisc = std::make_unique<LaserdiscPlayer>(
 			getHardwareConfig(), *this);
 	}
 	reset(getCurrentTime());
@@ -50,25 +49,21 @@ void PioneerLDControl::init()
 {
 	MSXDevice::init();
 
-	const auto& references = getReferences();
-	ppi = references.size() >= 1 ?
-		dynamic_cast<MSXPPI*>(references[0]) : nullptr;
+	const auto& refs = getReferences();
+	ppi = refs.size() >= 1 ? dynamic_cast<MSXPPI*>(refs[0]) : nullptr;
 	if (!ppi) {
 		throw MSXException("Invalid PioneerLDControl configuration: "
 		                   "need reference to PPI device.");
 	}
 
-	vdp = references.size() == 2 ?
-		dynamic_cast<VDP*>(references[1]) : nullptr;
+	vdp = refs.size() == 2 ? dynamic_cast<VDP*>(refs[1]) : nullptr;
 	if (!vdp) {
 		throw MSXException("Invalid PioneerLDControl configuration: "
 		                   "need reference to VDP device.");
 	}
 }
 
-PioneerLDControl::~PioneerLDControl()
-{
-}
+PioneerLDControl::~PioneerLDControl() = default;
 
 void PioneerLDControl::reset(EmuTime::param time)
 {
@@ -85,9 +80,7 @@ byte PioneerLDControl::readMem(word address, EmuTime::param time)
 	byte val = PioneerLDControl::peekMem(address, time);
 	if (address == 0x7fff) {
 		extint = false;
-		if (irq.getState()) {
-			irq.reset();
-		}
+		irq.reset();
 	}
 	return val;
 }
@@ -111,17 +104,17 @@ byte PioneerLDControl::peekMem(word address, EmuTime::param time) const
 			val &= 0x7f;
 		}
 	} else if (0x4000 <= address && address < 0x6000) {
-		val = (*rom)[address & 0x1fff];
+		val = rom[address & 0x1fff];
 	}
 	return val;
 }
 
-const byte* PioneerLDControl::getReadCacheLine(word start) const
+const byte* PioneerLDControl::getReadCacheLine(word address) const
 {
-	if ((start & CacheLine::HIGH) == (0x7FFE & CacheLine::HIGH)) {
+	if ((address & CacheLine::HIGH) == (0x7FFE & CacheLine::HIGH)) {
 		return nullptr;
-	} else if (0x4000 <= start && start < 0x6000) {
-		return &(*rom)[start & 0x1fff];
+	} else if (0x4000 <= address && address < 0x6000) {
+		return &rom[address & 0x1fff];
 	} else {
 		return unmappedRead;
 	}
@@ -132,13 +125,7 @@ void PioneerLDControl::writeMem(word address, byte value, EmuTime::param time)
 	if (address == 0x7fff) {
 		// superimpose
 		superimposing = !(value & 1);
-		if (superimposing) {
-			if (extint && !irq.getState()) {
-				irq.set();
-			}
-		} else if (irq.getState()) {
-			irq.reset();
-		}
+		irq.set(superimposing && extint);
 
 		updateVideoSource();
 
@@ -154,9 +141,9 @@ void PioneerLDControl::writeMem(word address, byte value, EmuTime::param time)
 	}
 }
 
-byte* PioneerLDControl::getWriteCacheLine(word start) const
+byte* PioneerLDControl::getWriteCacheLine(word address) const
 {
-	if ((start & CacheLine::HIGH) == (0x7FFE & CacheLine::HIGH)) {
+	if ((address & CacheLine::HIGH) == (0x7FFE & CacheLine::HIGH)) {
 		return nullptr;
 	} else {
 		return unmappedWrite;
@@ -168,9 +155,7 @@ void PioneerLDControl::videoIn(bool enabled)
 	if (videoEnabled && !enabled) {
 		// raise an interrupt when external video goes off
 		extint = true;
-		if (superimposing) {
-			irq.set();
-		}
+		if (superimposing) irq.set();
 	}
 	videoEnabled = enabled;
 	updateVideoSource();
@@ -187,17 +172,17 @@ void PioneerLDControl::updateVideoSource()
 template<typename Archive>
 void PioneerLDControl::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("clock", clock);
-	ar.serialize("mutel", mutel);
-	ar.serialize("muter", muter);
+	ar.serialize("clock", clock,
+	             "mutel", mutel,
+	             "muter", muter);
 	// videoEnabled is restored from LaserdiscPlayer. Set to false
 	// for now so that the irq does not get changed during load
 	if (ar.isLoader()) {
 		videoEnabled = false;
 	}
-	ar.serialize("superimposing", superimposing);
-	ar.serialize("extint", extint);
-	ar.serialize("irq", irq);
+	ar.serialize("superimposing", superimposing,
+	             "extint",        extint,
+	             "irq",           irq);
 	if (laserdisc) ar.serialize("laserdisc", *laserdisc);
 
 	if (ar.isLoader()) {
