@@ -13,9 +13,9 @@
 #include "RenderSettings.hh"
 #include "EnumSetting.hh"
 #include "MSXException.hh"
-#include "StringOp.hh"
 #include "Thread.hh"
 #include "build-info.hh"
+#include "random.hh"
 #include <iostream>
 #include <exception>
 #include <ctime>
@@ -37,13 +37,23 @@
 #define LOG_TO_FILE 0
 #endif
 
-using std::cerr;
-using std::cout;
-using std::endl;
-using std::string;
-
 namespace openmsx {
 
+#ifdef _WIN32
+// wrapper for Windows, as the MS runtime doesn't provide setenv!?
+int setenv(const char *name, const char *value, int overwrite);
+int setenv(const char *name, const char *value, int overwrite)
+{
+	if (!overwrite) {
+		char* oldvalue = getenv(name);
+		if (oldvalue) {
+			return 0;
+		}
+	}
+	return _putenv_s(name, value);
+}
+#endif
+
 static void initializeSDL()
 {
 	int flags = 0;
@@ -54,24 +64,15 @@ static void initializeSDL()
 	flags |= SDL_INIT_NOPARACHUTE;
 #endif
 	if (SDL_Init(flags) < 0) {
-		throw FatalError(StringOp::Builder() <<
-			"Couldn't init SDL: " << SDL_GetError());
+		throw FatalError("Couldn't init SDL: ", SDL_GetError());
 	}
 
-// In SDL 1.2.9 and before SDL_putenv has different semantics and is not
-// guaranteed to exist on all platforms.
-#if SDL_VERSION_ATLEAST(1, 2, 10)
-	// On Mac OS X, send key combos like Cmd+H and Cmd+M to Cocoa, so it can
-	// perform the corresponding actions.
-#if defined(__APPLE__)
-	SDL_putenv(const_cast<char*>("SDL_ENABLEAPPEVENTS=1"));
-#endif
-#endif
-}
-
-static void unexpectedExceptionHandler()
-{
-	cerr << "Unexpected exception." << endl;
+	// for now: instruct FreeType to use the v35 TTF engine
+	// this is the rendering we had before FreeType implemented and
+	// switched over to the v40 engine. To keep this the same for now, we
+	// select the old engine explicitly, until we decide how to continue
+	// (e.g. just use v40 or use a font that renders better on v40
+	setenv("FREETYPE_PROPERTIES", "truetype:interpreter-version=35", 0);
 }
 
 static int main(int argc, char **argv)
@@ -82,50 +83,24 @@ static int main(int argc, char **argv)
 
 	if (!freopen(STDOUT_LOG_FILE_NAME, "a", stdout)) {
 		ad_printf("Couldn't redirect stdout to logfile, aborting\n");
-		cerr << "Couldn't redirect stdout to "
-		        STDOUT_LOG_FILE_NAME << endl;
-		exit(1);
+		std::cerr << "Couldn't redirect stdout to "
+		             STDOUT_LOG_FILE_NAME "\n";
+		return 1;
 	}
 	if (!freopen(STDERR_LOG_FILE_NAME, "a", stderr)) {
 		ad_printf("Couldn't redirect stderr to logfile, aborting\n");
-		cout << "Couldn't redirect stderr to "
-		        STDERR_LOG_FILE_NAME << endl;
-		exit(1);
+		std::cout << "Couldn't redirect stderr to "
+		             STDERR_LOG_FILE_NAME "\n";
+		return 1;
 	}
 
-	string msg = Date::toString(time(nullptr)) + ": starting openMSX";
-	cout << msg << endl;
-	cerr << msg << endl;
+	std::string msg = Date::toString(time(nullptr)) + ": starting openMSX";
+	std::cout << msg << '\n';
+	std::cerr << msg << '\n';
 #endif
-	std::set_unexpected(unexpectedExceptionHandler);
 
-	int err = 0;
 	try {
-		// Constructing Reactor already causes parts of SDL to be used
-		// and initialized. If we want to set environment variables
-		// before this, we have to do it here...
-		//
-		// This is to make sure we get no annoying behaviour from SDL
-		// with regards to CAPS lock. This only works in SDL 1.2.14 or
-		// later, but it can't hurt to always set it (if we can rely on
-		// SDL_putenv, so on 1.2.10+).
-		//
-		// On Mac OS X, Cocoa does not report CAPS lock release events.
-		// The input driver inside SDL works around that by sending a
-		// pressed;released combo when CAPS status changes. However,
-		// because there is no time inbetween this does not give the
-		// MSX BIOS a chance to see the CAPS key in a pressed state.
-
-#if SDL_VERSION_ATLEAST(1, 2, 10)
-		SDL_putenv(const_cast<char*>("SDL_DISABLE_LOCK_KEYS="
-#if defined(__APPLE__)
-			"0"
-#else
-			"1"
-#endif
-			));
-#endif
-
+		randomize(); // seed global random generator
 		initializeSDL();
 
 		Thread::setMainThread();
@@ -140,8 +115,8 @@ static int main(int argc, char **argv)
 
 		if (parseStatus != CommandLineParser::EXIT) {
 			if (!parser.isHiddenStartup()) {
-				auto& render = reactor.getDisplay().getRenderSettings(). getRenderer();
-				render.setString(render.getRestoreValue());
+				auto& render = reactor.getDisplay().getRenderSettings().getRendererSetting();
+				render.setValue(render.getRestoreValue());
 				// Switching renderer requires events, handle
 				// these events before continuing with the rest
 				// of initialization. This fixes a bug where
@@ -158,24 +133,24 @@ static int main(int argc, char **argv)
 			}
 		}
 	} catch (FatalError& e) {
-		cerr << "Fatal error: " << e.getMessage() << endl;
-		err = 1;
+		std::cerr << "Fatal error: " << e.getMessage() << '\n';
+		exitCode = 1;
 	} catch (MSXException& e) {
-		cerr << "Uncaught exception: " << e.getMessage() << endl;
-		err = 1;
+		std::cerr << "Uncaught exception: " << e.getMessage() << '\n';
+		exitCode = 1;
 	} catch (std::exception& e) {
-		cerr << "Uncaught std::exception: " << e.what() << endl;
-		err = 1;
+		std::cerr << "Uncaught std::exception: " << e.what() << '\n';
+		exitCode = 1;
 	} catch (...) {
-		cerr << "Uncaught exception of unexpected type." << endl;
-		err = 1;
+		std::cerr << "Uncaught exception of unexpected type." << '\n';
+		exitCode = 1;
 	}
 	// Clean up.
 	if (SDL_WasInit(SDL_INIT_EVERYTHING)) {
 		SDL_Quit();
 	}
 
-	return err;
+	return exitCode;
 }
 
 } // namespace openmsx
@@ -183,5 +158,9 @@ static int main(int argc, char **argv)
 // Enter the openMSX namespace.
 int main(int argc, char **argv)
 {
-	exit(openmsx::main(argc, argv)); // need exit() iso return on win32/SDL
+	// TODO with SDL1 we had the comment:
+	//   need exit() iso return on win32/SDL
+	// Is that still the case with SDL2? Because for Android we need to
+	// return from main instead of exit().
+	return openmsx::main(argc, argv);
 }
