@@ -23,33 +23,27 @@
  */
 
 #include "MSXMegaRam.hh"
-#include "Ram.hh"
 #include "Rom.hh"
-#include "RomBlockDebuggable.hh"
 #include "Math.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include <memory>
 
 namespace openmsx {
 
 MSXMegaRam::MSXMegaRam(const DeviceConfig& config)
 	: MSXDevice(config)
 	, numBlocks(config.getChildDataAsInt("size") / 8) // 8kB blocks
-	, ram(make_unique<Ram>(
-		config, getName() + " RAM", "Mega-RAM", numBlocks * 0x2000))
+	, ram(config, getName() + " RAM", "Mega-RAM", numBlocks * 0x2000)
 	, rom(config.findChild("rom")
-	      ? make_unique<Rom>(getName() + " ROM", "Mega-RAM DiskROM", config)
+	      ? std::make_unique<Rom>(getName() + " ROM", "Mega-RAM DiskROM", config)
 	      : nullptr)
-	, romBlockDebug(make_unique<RomBlockDebuggable>(
-		*this, bank, 0x0000, 0x10000, 13, 0, 3))
-	, maskBlocks(Math::powerOfTwo(numBlocks) - 1)
+	, romBlockDebug(*this, bank, 0x0000, 0x10000, 13, 0, 3)
+	, maskBlocks(Math::ceil2(numBlocks) - 1)
 {
 	powerUp(EmuTime::dummy());
 }
 
-MSXMegaRam::~MSXMegaRam()
-{
-}
+MSXMegaRam::~MSXMegaRam() = default;
 
 void MSXMegaRam::powerUp(EmuTime::param time)
 {
@@ -57,7 +51,7 @@ void MSXMegaRam::powerUp(EmuTime::param time)
 		setBank(i, 0);
 	}
 	writeMode = false;
-	ram->clear();
+	ram.clear();
 	reset(time);
 }
 
@@ -82,7 +76,7 @@ const byte* MSXMegaRam::getReadCacheLine(word address) const
 	}
 	unsigned block = bank[(address & 0x7FFF) / 0x2000];
 	return (block < numBlocks)
-	     ? &(*ram)[(block * 0x2000) + (address & 0x1FFF)]
+	     ? &ram[(block * 0x2000) + (address & 0x1FFF)]
 	     : unmappedRead;
 }
 
@@ -102,7 +96,7 @@ byte* MSXMegaRam::getWriteCacheLine(word address) const
 	if (writeMode) {
 		unsigned block = bank[(address & 0x7FFF) / 0x2000];
 		return (block < numBlocks)
-		     ? &(*ram)[(block * 0x2000) + (address & 0x1FFF)]
+		     ? const_cast<byte*>(&ram[(block * 0x2000) + (address & 0x1FFF)])
 		     : unmappedWrite;
 	} else {
 		return nullptr;
@@ -121,7 +115,7 @@ byte MSXMegaRam::readIO(word port, EmuTime::param /*time*/)
 			if (rom) romMode = true;
 			break;
 	}
-	invalidateMemCache(0x0000, 0x10000);
+	invalidateDeviceRWCache();
 	return 0xFF; // return value doesn't matter
 }
 
@@ -142,25 +136,25 @@ void MSXMegaRam::writeIO(word port, byte /*value*/, EmuTime::param /*time*/)
 			if (rom) romMode = true;
 			break;
 	}
-	invalidateMemCache(0x0000, 0x10000);
+	invalidateDeviceRWCache();
 }
 
 void MSXMegaRam::setBank(byte page, byte block)
 {
 	bank[page] = block & maskBlocks;
 	word adr = page * 0x2000;
-	invalidateMemCache(adr + 0x0000, 0x2000);
-	invalidateMemCache(adr + 0x8000, 0x2000);
+	invalidateDeviceRWCache(adr + 0x0000, 0x2000);
+	invalidateDeviceRWCache(adr + 0x8000, 0x2000);
 }
 
 template<typename Archive>
 void MSXMegaRam::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.template serializeBase<MSXDevice>(*this);
-	ar.serialize("ram", *ram);
-	ar.serialize("bank", bank);
-	ar.serialize("writeMode", writeMode);
-	ar.serialize("romMode", romMode);
+	ar.serialize("ram",       ram,
+	             "bank",      bank,
+	             "writeMode", writeMode,
+	             "romMode",   romMode);
 }
 INSTANTIATE_SERIALIZE_METHODS(MSXMegaRam);
 REGISTER_MSXDEVICE(MSXMegaRam, "MegaRAM");
