@@ -1,5 +1,3 @@
-// $Id: MSXMirrorDevice.cc 12528 2012-05-17 17:36:10Z m9710797 $
-
 #include "MSXMirrorDevice.hh"
 #include "MSXCPUInterface.hh"
 #include "MSXException.hh"
@@ -41,12 +39,18 @@ void MSXMirrorDevice::writeMem(word address, byte value, EmuTime::param time)
 
 const byte* MSXMirrorDevice::getReadCacheLine(word /*start*/) const
 {
-	return NULL;
+	return nullptr;
 }
 
 byte* MSXMirrorDevice::getWriteCacheLine(word /*start*/) const
 {
-	return NULL;
+	return nullptr;
+}
+
+bool MSXMirrorDevice::allowUnaligned() const
+{
+	// OK, because this device doesn't call any 'fillDeviceXXXCache()'functions.
+	return true;
 }
 
 template<typename Archive>
