@@ -3,6 +3,7 @@
 #include "HardwareConfig.hh"
 #include "MSXMotherBoard.hh"
 #include "MSXException.hh"
+#include "one_of.hh"
 #include <cassert>
 
 using std::string;
@@ -19,10 +20,10 @@ MSXRomCLI::MSXRomCLI(CommandLineParser& cmdLineParser_)
 	cmdLineParser.registerOption("-cartb", *this);
 	cmdLineParser.registerOption("-cartc", *this);
 	cmdLineParser.registerOption("-cartd", *this);
-	cmdLineParser.registerFileType("ri,rom", *this);
+	cmdLineParser.registerFileType({"ri", "rom", "mx1", "mx2"}, *this);
 }
 
-void MSXRomCLI::parseOption(const string& option, array_ref<string>& cmdLine)
+void MSXRomCLI::parseOption(const string& option, span<string>& cmdLine)
 {
 	string arg = getArgument(option, cmdLine);
 	string slotname;
@@ -34,33 +35,38 @@ void MSXRomCLI::parseOption(const string& option, array_ref<string>& cmdLine)
 	parse(arg, slotname, cmdLine);
 }
 
-string_ref MSXRomCLI::optionHelp() const
+std::string_view MSXRomCLI::optionHelp() const
 {
 	return "Insert the ROM file (cartridge) specified in argument";
 }
 
-void MSXRomCLI::parseFileType(const string& arg, array_ref<string>& cmdLine)
+void MSXRomCLI::parseFileType(const string& arg, span<string>& cmdLine)
 {
 	parse(arg, "any", cmdLine);
 }
 
-string_ref MSXRomCLI::fileTypeHelp() const
+std::string_view MSXRomCLI::fileTypeHelp() const
 {
 	static const string text("ROM image of a cartridge");
 	return text;
 }
 
+std::string_view MSXRomCLI::fileTypeCategoryName() const
+{
+	return "rom";
+}
+
 void MSXRomCLI::parse(const string& arg, const string& slotname,
-                      array_ref<string>& cmdLine)
+                      span<string>& cmdLine)
 {
 	// parse extra options  -ips  and  -romtype
-	std::vector<string> options;
+	std::vector<TclObject> options;
 	while (true) {
 		string option = peekArgument(cmdLine);
-		if ((option == "-ips") || (option == "-romtype")) {
-			options.push_back(option);
-			cmdLine.pop_front();
-			options.push_back(getArgument(option, cmdLine));
+		if (option == one_of("-ips", "-romtype")) {
+			options.emplace_back(option);
+			cmdLine = cmdLine.subspan(1);
+			options.emplace_back(getArgument(option, cmdLine));
 		} else {
 			break;
 		}
@@ -72,25 +78,25 @@ void MSXRomCLI::parse(const string& arg, const string& slotname,
 }
 
 void MSXRomCLI::IpsOption::parseOption(const string& /*option*/,
-                                       array_ref<string>& /*cmdLine*/)
+                                       span<string>& /*cmdLine*/)
 {
 	throw FatalError(
 		"-ips options should immediately follow a ROM or disk image.");
 }
 
-string_ref MSXRomCLI::IpsOption::optionHelp() const
+std::string_view MSXRomCLI::IpsOption::optionHelp() const
 {
 	return "Apply the given IPS patch to the ROM or disk image specified "
 	       "in the preceding option";
 }
 
 void MSXRomCLI::RomTypeOption::parseOption(const string& /*option*/,
-                                           array_ref<string>& /*cmdLine*/)
+                                           span<string>& /*cmdLine*/)
 {
 	throw FatalError("-romtype options should immediately follow a ROM.");
 }
 
-string_ref MSXRomCLI::RomTypeOption::optionHelp() const
+std::string_view MSXRomCLI::RomTypeOption::optionHelp() const
 {
 	return "Specify the rom type for the ROM image specified in the "
 	       "preceding option";
