@@ -1,13 +1,8 @@
 #include "MegaFlashRomSCCPlus.hh"
-#include "Rom.hh"
-#include "SCC.hh"
-#include "AY8910.hh"
 #include "DummyAY8910Periphery.hh"
-#include "AmdFlash.hh"
 #include "MSXCPUInterface.hh"
 #include "CacheLine.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include <cassert>
 #include <vector>
 
@@ -58,6 +53,7 @@ Main features:
  Port #A1 -> #11
  Port #A2 -> #12
 
+ The PSG is read only.
 
 --------------------------------------------------------------------------------
 [REGISTERS]
@@ -181,34 +177,28 @@ static std::vector<AmdFlash::SectorInfo> getSectorInfo() {
 };
 
 MegaFlashRomSCCPlus::MegaFlashRomSCCPlus(
-		const DeviceConfig& config, std::unique_ptr<Rom> rom_)
+		const DeviceConfig& config, Rom&& rom_)
 	: MSXRom(config, std::move(rom_))
-	, scc(make_unique<SCC>(
-		"MFR SCC+ SCC-I", config, getCurrentTime(),
-		SCC::SCC_Compatible))
-	, psg(make_unique<AY8910>(
-		"MFR SCC+ PSG", DummyAY8910Periphery::instance(), config,
-		getCurrentTime()))
-	, flash(make_unique<AmdFlash>(
-		*rom, getSectorInfo(), 0x205B, false, config))
+	, scc("MFR SCC+ SCC-I", config, getCurrentTime(), SCC::SCC_Compatible)
+	, psg("MFR SCC+ PSG", DummyAY8910Periphery::instance(), config,
+	      getCurrentTime())
+	, flash(rom, getSectorInfo(), 0x205B, false, config)
 {
 	powerUp(getCurrentTime());
 
 	getCPUInterface().register_IO_Out(0x10, this);
 	getCPUInterface().register_IO_Out(0x11, this);
-	getCPUInterface().register_IO_In (0x12, this);
 }
 
 MegaFlashRomSCCPlus::~MegaFlashRomSCCPlus()
 {
 	getCPUInterface().unregister_IO_Out(0x10, this);
 	getCPUInterface().unregister_IO_Out(0x11, this);
-	getCPUInterface().unregister_IO_In (0x12, this);
 }
 
 void MegaFlashRomSCCPlus::powerUp(EmuTime::param time)
 {
-	scc->powerUp(time);
+	scc.powerUp(time);
 	reset(time);
 }
 
@@ -227,14 +217,14 @@ void MegaFlashRomSCCPlus::reset(EmuTime::param time)
 	for (int i = 0; i < 4; ++i) {
 		sccBanks[i] = i;
 	}
-	scc->reset(time);
+	scc.reset(time);
 
 	psgLatch = 0;
-	psg->reset(time);
+	psg.reset(time);
 
-	flash->reset();
+	flash.reset();
 
-	invalidateMemCache(0x0000, 0x10000); // flush all to be sure
+	invalidateDeviceRCache(); // flush all to be sure
 }
 
 MegaFlashRomSCCPlus::SCCEnable MegaFlashRomSCCPlus::getSCCEnable() const
@@ -287,7 +277,7 @@ byte MegaFlashRomSCCPlus::peekMem(word addr, EmuTime::param time) const
 		SCCEnable enable = getSCCEnable();
 		if (((enable == EN_SCC)     && (0x9800 <= addr) && (addr < 0xA000)) ||
 		    ((enable == EN_SCCPLUS) && (0xB800 <= addr) && (addr < 0xC000))) {
-			return scc->peekMem(addr & 0xFF, time);
+			return scc.peekMem(addr & 0xFF, time);
 		}
 	}
 
@@ -296,7 +286,7 @@ byte MegaFlashRomSCCPlus::peekMem(word addr, EmuTime::param time) const
 		// read (flash)rom content
 		unsigned flashAddr = getFlashAddr(addr);
 		assert(flashAddr != unsigned(-1));
-		return flash->peek(flashAddr);
+		return flash.peek(flashAddr);
 	} else {
 		// unmapped read
 		return 0xFF;
@@ -314,7 +304,7 @@ byte MegaFlashRomSCCPlus::readMem(word addr, EmuTime::param time)
 		SCCEnable enable = getSCCEnable();
 		if (((enable == EN_SCC)     && (0x9800 <= addr) && (addr < 0xA000)) ||
 		    ((enable == EN_SCCPLUS) && (0xB800 <= addr) && (addr < 0xC000))) {
-			return scc->readMem(addr & 0xFF, time);
+			return scc.readMem(addr & 0xFF, time);
 		}
 	}
 
@@ -323,7 +313,7 @@ byte MegaFlashRomSCCPlus::readMem(word addr, EmuTime::param time)
 		// read (flash)rom content
 		unsigned flashAddr = getFlashAddr(addr);
 		assert(flashAddr != unsigned(-1));
-		return flash->read(flashAddr);
+		return flash.read(flashAddr);
 	} else {
 		// unmapped read
 		return 0xFF;
@@ -351,7 +341,7 @@ const byte* MegaFlashRomSCCPlus::getReadCacheLine(word addr) const
 		// read (flash)rom content
 		unsigned flashAddr = getFlashAddr(addr);
 		assert(flashAddr != unsigned(-1));
-		return flash->getReadCacheLine(flashAddr);
+		return flash.getReadCacheLine(flashAddr);
 	} else {
 		// unmapped read
 		return unmappedRead;
@@ -373,7 +363,7 @@ void MegaFlashRomSCCPlus::writeMem(word addr, byte value, EmuTime::param time)
 		subslotReg = value;
 		for (int i = 0; i < 4; ++i) {
 			if (diff & (3 << (2 * i))) {
-				invalidateMemCache(0x4000 * i, 0x4000);
+				invalidateDeviceRCache(0x4000 * i, 0x4000);
 			}
 		}
 	}
@@ -381,17 +371,17 @@ void MegaFlashRomSCCPlus::writeMem(word addr, byte value, EmuTime::param time)
 	if (((configReg & 0x04) == 0x00) && ((addr & 0xFFFE) == 0x7FFE)) {
 		// write config register
 		configReg = value;
-		invalidateMemCache(0x0000, 0x10000); // flush all to be sure
+		invalidateDeviceRCache(); // flush all to be sure
 	}
 
 	if ((configReg & 0xE0) == 0x00) {
 		// Konami-SCC
 		if ((addr & 0xFFFE) == 0xBFFE) {
 			sccMode = value;
-			scc->setChipMode((value & 0x20) ? SCC::SCC_plusmode
-			                                : SCC::SCC_Compatible);
-			invalidateMemCache(0x9800, 0x800);
-			invalidateMemCache(0xB800, 0x800);
+			scc.setChipMode((value & 0x20) ? SCC::SCC_plusmode
+			                               : SCC::SCC_Compatible);
+			invalidateDeviceRCache(0x9800, 0x800);
+			invalidateDeviceRCache(0xB800, 0x800);
 		}
 		SCCEnable enable = getSCCEnable();
 		bool isRamSegment2 = ((sccMode & 0x24) == 0x24) ||
@@ -401,9 +391,9 @@ void MegaFlashRomSCCPlus::writeMem(word addr, byte value, EmuTime::param time)
 		     (0x9800 <= addr) && (addr < 0xA000)) ||
 		    ((enable == EN_SCCPLUS) && !isRamSegment3 &&
 		     (0xB800 <= addr) && (addr < 0xC000))) {
-			scc->writeMem(addr & 0xFF, value, time);
+			scc.writeMem(addr & 0xFF, value, time);
+			return; // Pazos: when SCC registers are selected flashROM is not seen, so it does not accept commands.
 		}
-		return; // Pazos: when SCC registers are selected flashROM is not seen, so it does not accept commands.
 	}
 
 	unsigned subslot = getSubslot(addr);
@@ -420,13 +410,13 @@ void MegaFlashRomSCCPlus::writeMem(word addr, byte value, EmuTime::param time)
 				sccBanks[page8kB] = value;
 				if ((value & 0x80) && (page8kB == 0)) {
 					offsetReg = value & 0x7F;
-					invalidateMemCache(0x4000, 0x8000);
+					invalidateDeviceRCache(0x4000, 0x8000);
 				} else {
 					// Masking of the mapper bits is done on
 					// write (and only in Konami(-scc) mode)
 					byte mask = (configReg & 0x01) ? 0x3F : 0x7F;
 					bankRegs[subslot][page8kB] = value & mask;
-					invalidateMemCache(0x4000 + 0x2000 * page8kB, 0x2000);
+					invalidateDeviceRCache(0x4000 + 0x2000 * page8kB, 0x2000);
 				}
 			}
 			break;
@@ -444,14 +434,14 @@ void MegaFlashRomSCCPlus::writeMem(word addr, byte value, EmuTime::param time)
 			if ((addr < 0x5000) || ((0x5800 <= addr) && (addr < 0x6000))) break; // only SCC range works
 			byte mask = (configReg & 0x01) ? 0x1F : 0x7F;
 			bankRegs[subslot][page8kB] = value & mask;
-			invalidateMemCache(0x4000 + 0x2000 * page8kB, 0x2000);
+			invalidateDeviceRCache(0x4000 + 0x2000 * page8kB, 0x2000);
 			break;
 		}
 		case 0x40:
 		case 0x60:
 			// 64kB
 			bankRegs[subslot][page8kB] = value;
-			invalidateMemCache(0x0000 + 0x4000 * page8kB, 0x4000);
+			invalidateDeviceRCache(0x0000 + 0x4000 * page8kB, 0x4000);
 			break;
 		case 0x80:
 		case 0xA0:
@@ -459,7 +449,7 @@ void MegaFlashRomSCCPlus::writeMem(word addr, byte value, EmuTime::param time)
 			if ((0x6000 <= addr) && (addr < 0x8000)) {
 				byte bank = (addr >> 11) & 0x03;
 				bankRegs[subslot][bank] = value;
-				invalidateMemCache(0x4000 + 0x2000 * page8kB, 0x2000);
+				invalidateDeviceRCache(0x4000 + 0x2000 * bank, 0x2000);
 			}
 			break;
 		case 0xC0:
@@ -474,12 +464,12 @@ void MegaFlashRomSCCPlus::writeMem(word addr, byte value, EmuTime::param time)
 			if ((0x6000 <= addr) && (addr < 0x6800)) {
 				bankRegs[subslot][0] = 2 * value + 0;
 				bankRegs[subslot][1] = 2 * value + 1;
-				invalidateMemCache(0x4000, 0x4000);
+				invalidateDeviceRCache(0x4000, 0x4000);
 			}
 			if ((0x7000 <= addr) && (addr < 0x7800)) {
 				bankRegs[subslot][2] = 2 * value + 0;
 				bankRegs[subslot][3] = 2 * value + 1;
-				invalidateMemCache(0x8000, 0x4000);
+				invalidateDeviceRCache(0x8000, 0x4000);
 			}
 			break;
 		}
@@ -489,7 +479,7 @@ void MegaFlashRomSCCPlus::writeMem(word addr, byte value, EmuTime::param time)
 	if (((configReg & 0xC0) == 0x40) ||
 	    ((0x4000 <= addr) && (addr < 0xC000))) {
 		assert(flashAddr != unsigned(-1));
-		return flash->write(flashAddr, value);
+		return flash.write(flashAddr, value);
 	}
 }
 
@@ -499,25 +489,13 @@ byte* MegaFlashRomSCCPlus::getWriteCacheLine(word /*addr*/) const
 }
 
 
-byte MegaFlashRomSCCPlus::readIO(word port, EmuTime::param time)
-{
-	assert((port & 0xFF) == 0x12); (void)port;
-	return psg->readRegister(psgLatch, time);
-}
-
-byte MegaFlashRomSCCPlus::peekIO(word port, EmuTime::param time) const
-{
-	assert((port & 0xFF) == 0x12); (void)port;
-	return psg->peekRegister(psgLatch, time);
-}
-
 void MegaFlashRomSCCPlus::writeIO(word port, byte value, EmuTime::param time)
 {
 	if ((port & 0xFF) == 0x10) {
 		psgLatch = value & 0x0F;
 	} else {
 		assert((port & 0xFF) == 0x11);
-		psg->writeRegister(psgLatch, value, time);
+		psg.writeRegister(psgLatch, value, time);
 	}
 }
 
@@ -528,17 +506,17 @@ void MegaFlashRomSCCPlus::serialize(Archive& ar, unsigned /*version*/)
 	// skip MSXRom base class
 	ar.template serializeBase<MSXDevice>(*this);
 
-	ar.serialize("scc", *scc);
-	ar.serialize("psg", *psg);
-	ar.serialize("flash", *flash);
+	ar.serialize("scc",        scc,
+	             "psg",        psg,
+	             "flash",      flash,
 
-	ar.serialize("configReg", configReg);
-	ar.serialize("offsetReg", offsetReg);
-	ar.serialize("subslotReg", subslotReg);
-	ar.serialize("bankRegs", bankRegs);
-	ar.serialize("psgLatch", psgLatch);
-	ar.serialize("sccMode", sccMode);
-	ar.serialize("sccBanks", sccBanks);
+	             "configReg",  configReg,
+	             "offsetReg",  offsetReg,
+	             "subslotReg", subslotReg,
+	             "bankRegs",   bankRegs,
+	             "psgLatch",   psgLatch,
+	             "sccMode",    sccMode,
+	             "sccBanks",   sccBanks);
 }
 INSTANTIATE_SERIALIZE_METHODS(MegaFlashRomSCCPlus);
 REGISTER_MSXDEVICE(MegaFlashRomSCCPlus, "MegaFlashRomSCCPlus");
