@@ -1,17 +1,11 @@
 #include "MegaFlashRomSCCPlusSD.hh"
-#include "Rom.hh"
-#include "SCC.hh"
-#include "AY8910.hh"
 #include "DummyAY8910Periphery.hh"
-#include "AmdFlash.hh"
 #include "MSXCPUInterface.hh"
 #include "CacheLine.hh"
-#include "Ram.hh"
 #include "CheckedRam.hh"
 #include "SdCard.hh"
 #include "serialize.hh"
-#include "memory.hh"
-#include <cassert>
+#include <memory>
 #include <vector>
 
 /******************************************************************************
@@ -57,6 +51,7 @@ MegaFlashROM SCC+ SD Technical Details
  Port #A1 -> #11
  Port #A2 -> #12
 
+ The PSG is read only.
 
 --------------------------------------------------------------------------------
 [Cartridge layout]
@@ -224,6 +219,10 @@ by VPP/WD pin)
   mapper ports as read only, as stated on MSX Datapack, all read operations on
   these ports will not return any value.
 
+UPDATE:
+  Initial version indeed did not support reading the memory mapper ports. But
+  upon user request this feature was added later.
+
 --------------------------------------------------------------------------------
 [Subslot 3: MegaSD]
 
@@ -254,7 +253,8 @@ by VPP/WD pin)
 
 ******************************************************************************/
 
-static const unsigned MEMORY_MAPPER_SIZE = 512;
+constexpr unsigned MEMORY_MAPPER_SIZE = 512;
+constexpr unsigned MEMORY_MAPPER_MASK = (MEMORY_MAPPER_SIZE / 16) - 1;
 
 namespace openmsx {
 
@@ -268,37 +268,25 @@ static std::vector<AmdFlash::SectorInfo> getSectorInfo()
 	return sectorInfo;
 }
 
-MegaFlashRomSCCPlusSD::MegaFlashRomSCCPlusSD(
-		const DeviceConfig& config, std::unique_ptr<Rom> rom_)
-	: MSXRom(config, std::move(rom_))
-	, flash(make_unique<AmdFlash>(
-		*rom, getSectorInfo(), 0x207E, true, config))
-	, scc(make_unique<SCC>(
-		"MFR SCC+ SD SCC-I", config, getCurrentTime(),
-		SCC::SCC_Compatible))
-	, psg(make_unique<AY8910>(
-		"MFR SCC+ SD PSG", DummyAY8910Periphery::instance(), config,
-		getCurrentTime()))
+MegaFlashRomSCCPlusSD::MegaFlashRomSCCPlusSD(const DeviceConfig& config)
+	: MSXDevice(config)
+	, flash("MFR SCC+ SD flash", getSectorInfo(), 0x207E, true, config)
+	, scc("MFR SCC+ SD SCC-I", config, getCurrentTime(), SCC::SCC_Compatible)
+	, psg("MFR SCC+ SD PSG", DummyAY8910Periphery::instance(), config,
+	      getCurrentTime())
 	, configReg(3) // avoid UMR
 	, checkedRam(config.getChildDataAsBool("hasmemorymapper", true) ?
-		make_unique<CheckedRam>(config, getName() + " memory mapper", "memory mapper", MEMORY_MAPPER_SIZE * 1024)
+		std::make_unique<CheckedRam>(config, getName() + " memory mapper", "memory mapper", MEMORY_MAPPER_SIZE * 1024)
 		: nullptr)
+	, mapperIO(checkedRam ? std::make_unique<MapperIO>(*this) : nullptr) // handles ports 0xfc-0xff
 {
 	powerUp(getCurrentTime());
 
 	getCPUInterface().register_IO_Out(0x10, this);
 	getCPUInterface().register_IO_Out(0x11, this);
-	getCPUInterface().register_IO_In (0x12, this);
 
-	if (checkedRam) {
-		getCPUInterface().register_IO_Out(0xFF, this);
-		getCPUInterface().register_IO_Out(0xFE, this);
-		getCPUInterface().register_IO_Out(0xFD, this);
-		getCPUInterface().register_IO_Out(0xFC, this);
-	}
-
-	sdCard[0] = make_unique<SdCard>(DeviceConfig(config, config.findChild("sdcard1")));
-	sdCard[1] = make_unique<SdCard>(DeviceConfig(config, config.findChild("sdcard2")));
+	sdCard[0] = std::make_unique<SdCard>(DeviceConfig(config, config.findChild("sdcard1")));
+	sdCard[1] = std::make_unique<SdCard>(DeviceConfig(config, config.findChild("sdcard2")));
 }
 
 MegaFlashRomSCCPlusSD::~MegaFlashRomSCCPlusSD()
@@ -308,19 +296,11 @@ MegaFlashRomSCCPlusSD::~MegaFlashRomSCCPlusSD()
 
 	getCPUInterface().unregister_IO_Out(0x10, this);
 	getCPUInterface().unregister_IO_Out(0x11, this);
-	getCPUInterface().unregister_IO_In (0x12, this);
-
-	if (checkedRam) {
-		getCPUInterface().unregister_IO_Out(0xFF, this);
-		getCPUInterface().unregister_IO_Out(0xFE, this);
-		getCPUInterface().unregister_IO_Out(0xFD, this);
-		getCPUInterface().unregister_IO_Out(0xFC, this);
-	}
 }
 
 void MegaFlashRomSCCPlusSD::powerUp(EmuTime::param time)
 {
-	scc->powerUp(time);
+	scc.powerUp(time);
 	reset(time);
 }
 
@@ -338,12 +318,12 @@ void MegaFlashRomSCCPlusSD::reset(EmuTime::param time)
 	for (int i = 0; i < 4; ++i) {
 		sccBanks[i] = i;
 	}
-	scc->reset(time);
+	scc.reset(time);
 
 	psgLatch = 0;
-	psg->reset(time);
+	psg.reset(time);
 
-	flash->reset();
+	flash.reset();
 
 	// memory mapper
 	for (auto i = 0; i < 4; ++i) {
@@ -356,7 +336,7 @@ void MegaFlashRomSCCPlusSD::reset(EmuTime::param time)
 
 	selectedCard = 0;
 
-	invalidateMemCache(0x0000, 0x10000); // flush all to be sure
+	invalidateDeviceRWCache(); // flush all to be sure
 }
 
 byte MegaFlashRomSCCPlusSD::getSubSlot(unsigned addr) const
@@ -368,7 +348,7 @@ byte MegaFlashRomSCCPlusSD::getSubSlot(unsigned addr) const
 void MegaFlashRomSCCPlusSD::writeToFlash(unsigned addr, byte value)
 {
 	if (isFlashRomWriteEnabled()) {
-		flash->write(addr, value);
+		flash.write(addr, value);
 	} else {
 		// flash is write protected, this is implemented by not passing
 		// writes to flash at all.
@@ -435,7 +415,7 @@ void MegaFlashRomSCCPlusSD::writeMem(word addr, byte value, EmuTime::param time)
 		subslotReg = value;
 		for (int i = 0; i < 4; ++i) {
 			if (diff & (3 << (2 * i))) {
-				invalidateMemCache(0x4000 * i, 0x4000);
+				invalidateDeviceRWCache(0x4000 * i, 0x4000);
 			}
 		}
 	}
@@ -476,19 +456,19 @@ byte MegaFlashRomSCCPlusSD::readMemSubSlot0(word addr)
 {
 	// read from the first 16kB of flash
 	// Pazos: ROM and flash can be accessed in all pages (0,1,2,3) (#0000-#FFFF) 
-	return flash->read(addr & 0x3FFF);
+	return flash.read(addr & 0x3FFF);
 }
 
 byte MegaFlashRomSCCPlusSD::peekMemSubSlot0(word addr) const
 {
 	// read from the first 16kB of flash
 	// Pazos: ROM and flash can be accessed in all pages (0,1,2,3) (#0000-#FFFF) 
-	return flash->peek(addr & 0x3FFF);
+	return flash.peek(addr & 0x3FFF);
 }
 
 const byte* MegaFlashRomSCCPlusSD::getReadCacheLineSubSlot0(word addr) const
 {
-	return flash->getReadCacheLine(addr & 0x3FFF);
+	return flash.getReadCacheLine(addr & 0x3FFF);
 }
 
 void MegaFlashRomSCCPlusSD::writeMemSubSlot0(word addr, byte value)
@@ -510,16 +490,14 @@ void MegaFlashRomSCCPlusSD::updateConfigReg(byte value)
 		if (value & 0x08) {
 			getCPUInterface().register_IO_Out(0xA0, this);
 			getCPUInterface().register_IO_Out(0xA1, this);
-			getCPUInterface().register_IO_In (0xA2, this);
 		} else {
 			getCPUInterface().unregister_IO_Out(0xA0, this);
 			getCPUInterface().unregister_IO_Out(0xA1, this);
-			getCPUInterface().unregister_IO_In (0xA2, this);
 		}
 	}
 	configReg = value;
-	flash->setVppWpPinLow(isFlashRomBlockProtectEnabled());
-	invalidateMemCache(0x0000, 0x10000); // flush all to be sure
+	flash.setVppWpPinLow(isFlashRomBlockProtectEnabled());
+	invalidateDeviceRWCache(); // flush all to be sure
 }
 
 MegaFlashRomSCCPlusSD::SCCEnable MegaFlashRomSCCPlusSD::getSCCEnable() const
@@ -559,14 +537,14 @@ byte MegaFlashRomSCCPlusSD::readMemSubSlot1(word addr, EmuTime::param time)
 		SCCEnable enable = getSCCEnable();
 		if (((enable == EN_SCC)     && (0x9800 <= addr) && (addr < 0xA000)) ||
 		    ((enable == EN_SCCPLUS) && (0xB800 <= addr) && (addr < 0xC000))) {
-			byte val = scc->readMem(addr & 0xFF, time);
+			byte val = scc.readMem(addr & 0xFF, time);
 			return val;
 		}
 	}
 
 	unsigned flashAddr = getFlashAddrSubSlot1(addr);
 	return (flashAddr != unsigned(-1))
-		? flash->read(flashAddr)
+		? flash.read(flashAddr)
 		: 0xFF; // unmapped read
 }
 
@@ -576,30 +554,30 @@ byte MegaFlashRomSCCPlusSD::peekMemSubSlot1(word addr, EmuTime::param time) cons
 		SCCEnable enable = getSCCEnable();
 		if (((enable == EN_SCC)     && (0x9800 <= addr) && (addr < 0xA000)) ||
 		    ((enable == EN_SCCPLUS) && (0xB800 <= addr) && (addr < 0xC000))) {
-			byte val = scc->peekMem(addr & 0xFF, time);
+			byte val = scc.peekMem(addr & 0xFF, time);
 			return val;
 		}
 	}
 
 	unsigned flashAddr = getFlashAddrSubSlot1(addr);
 	return (flashAddr != unsigned(-1))
-		? flash->peek(flashAddr)
+		? flash.peek(flashAddr)
 		: 0xFF; // unmapped read
 }
 
 const byte* MegaFlashRomSCCPlusSD::getReadCacheLineSubSlot1(word addr) const
 {
-        if (isKonamiSCCmapperConfigured()) {
-                SCCEnable enable = getSCCEnable();
-                if (((enable == EN_SCC)     && (0x9800 <= addr) && (addr < 0xA000)) ||
-                    ((enable == EN_SCCPLUS) && (0xB800 <= addr) && (addr < 0xC000))) {
-                        return nullptr;
-                }
-        }
+	if (isKonamiSCCmapperConfigured()) {
+		SCCEnable enable = getSCCEnable();
+		if (((enable == EN_SCC)     && (0x9800 <= addr) && (addr < 0xA000)) ||
+		    ((enable == EN_SCCPLUS) && (0xB800 <= addr) && (addr < 0xC000))) {
+			return nullptr;
+		}
+	}
 
 	unsigned flashAddr = getFlashAddrSubSlot1(addr);
 	return (flashAddr != unsigned(-1))
-		? flash->getReadCacheLine(flashAddr)
+		? flash.getReadCacheLine(flashAddr)
 		: unmappedRead;
 }
 
@@ -623,29 +601,29 @@ void MegaFlashRomSCCPlusSD::writeMemSubSlot1(word addr, byte value, EmuTime::par
 	if (!isMapperRegisterDisabled() && (addr == 0x7FFF)) {
 		// write mapper register
 		mapperReg = value;
-		invalidateMemCache(0x0000, 0x10000); // flush all to be sure
+		invalidateDeviceRWCache(); // flush all to be sure
 	}
 
 	if (!areBankRegsAndOffsetRegsDisabled() && (addr == 0x7FFD)) {
 		// write offset register low part
 		offsetReg = (offsetReg & 0x300) | value;
-		invalidateMemCache(0x0000, 0x10000);
+		invalidateDeviceRWCache();
 	}
 
 	if (!areBankRegsAndOffsetRegsDisabled() && (addr == 0x7FFE)) {
 		// write offset register high part (bit 8 and 9)
 		offsetReg = (offsetReg & 0xFF) + ((value & 0x3) << 8);
-		invalidateMemCache(0x0000, 0x10000);
+		invalidateDeviceRWCache();
 	}
 
 	if (isKonamiSCCmapperConfigured()) {
 		// Konami-SCC
 		if ((addr & 0xFFFE) == 0xBFFE) {
 			sccMode = value;
-			scc->setChipMode((value & 0x20) ? SCC::SCC_plusmode
-			                                : SCC::SCC_Compatible);
-			invalidateMemCache(0x9800, 0x800);
-			invalidateMemCache(0xB800, 0x800);
+			scc.setChipMode((value & 0x20) ? SCC::SCC_plusmode
+			                               : SCC::SCC_Compatible);
+			invalidateDeviceRWCache(0x9800, 0x800);
+			invalidateDeviceRWCache(0xB800, 0x800);
 		}
 		SCCEnable enable = getSCCEnable();
 		bool isRamSegment2 = ((sccMode & 0x24) == 0x24) ||
@@ -655,7 +633,7 @@ void MegaFlashRomSCCPlusSD::writeMemSubSlot1(word addr, byte value, EmuTime::par
 		     (0x9800 <= addr) && (addr < 0xA000)) ||
 		    ((enable == EN_SCCPLUS) && !isRamSegment3 &&
 		     (0xB800 <= addr) && (addr < 0xC000))) {
-			scc->writeMem(addr & 0xFF, value, time);
+			scc.writeMem(addr & 0xFF, value, time);
 			return; // Pazos: when SCC registers are selected flashROM is not seen, so it does not accept commands. 
 		}
 	}
@@ -673,9 +651,9 @@ void MegaFlashRomSCCPlusSD::writeMemSubSlot1(word addr, byte value, EmuTime::par
 				sccBanks[page8kB] = value;
 				// Masking of the mapper bits is done on
 				// write (and only in Konami(-scc) mode)
-				byte mask = areKonamiMapperLimitsEnabled() ? 0x3F : 0x7F;
+				byte mask = areKonamiMapperLimitsEnabled() ? 0x3F : 0xFF;
 				bankRegsSubSlot1[page8kB] = value & mask;
-				invalidateMemCache(0x4000 + 0x2000 * page8kB, 0x2000);
+				invalidateDeviceRWCache(0x4000 + 0x2000 * page8kB, 0x2000);
 			}
 			break;
 		case 0x20: {
@@ -690,16 +668,16 @@ void MegaFlashRomSCCPlusSD::writeMemSubSlot1(word addr, byte value, EmuTime::par
 			// Making of the mapper bits is done on
 			// write (and only in Konami(-scc) mode)
 			if ((addr < 0x5000) || ((0x5800 <= addr) && (addr < 0x6000))) break; // only SCC range works
-			byte mask = areKonamiMapperLimitsEnabled() ? 0x1F : 0x7F;
+			byte mask = areKonamiMapperLimitsEnabled() ? 0x1F : 0xFF;
 			bankRegsSubSlot1[page8kB] = value & mask;
-			invalidateMemCache(0x4000 + 0x2000 * page8kB, 0x2000);
+			invalidateDeviceRWCache(0x4000 + 0x2000 * page8kB, 0x2000);
 			break;
 		}
 		case 0x40:
 		case 0x60:
 			// 64kB
 			bankRegsSubSlot1[page8kB] = value;
-			invalidateMemCache(0x0000 + 0x4000 * page8kB, 0x4000);
+			invalidateDeviceRWCache(0x0000 + 0x4000 * page8kB, 0x4000);
 			break;
 		case 0x80:
 		case 0xA0:
@@ -707,7 +685,7 @@ void MegaFlashRomSCCPlusSD::writeMemSubSlot1(word addr, byte value, EmuTime::par
 			if ((0x6000 <= addr) && (addr < 0x8000)) {
 				byte bank = (addr >> 11) & 0x03;
 				bankRegsSubSlot1[bank] = value;
-				invalidateMemCache(0x4000 + 0x2000 * page8kB, 0x2000);
+				invalidateDeviceRWCache(0x4000 + 0x2000 * bank, 0x2000);
 			}
 			break;
 		case 0xC0:
@@ -722,12 +700,12 @@ void MegaFlashRomSCCPlusSD::writeMemSubSlot1(word addr, byte value, EmuTime::par
 			if ((0x6000 <= addr) && (addr < 0x6800)) {
 				bankRegsSubSlot1[0] = 2 * value + 0;
 				bankRegsSubSlot1[1] = 2 * value + 1;
-				invalidateMemCache(0x4000, 0x4000);
+				invalidateDeviceRWCache(0x4000, 0x4000);
 			}
 			if ((0x7000 <= addr) && (addr < 0x7800)) {
 				bankRegsSubSlot1[2] = 2 * value + 0;
 				bankRegsSubSlot1[3] = 2 * value + 1;
-				invalidateMemCache(0x8000, 0x4000);
+				invalidateDeviceRWCache(0x8000, 0x4000);
 			}
 			break;
 		}
@@ -747,9 +725,8 @@ byte* MegaFlashRomSCCPlusSD::getWriteCacheLineSubSlot1(word /*addr*/) const
 
 unsigned MegaFlashRomSCCPlusSD::calcMemMapperAddress(word address) const
 {
-	static const unsigned MASK = (MEMORY_MAPPER_SIZE / 16) - 1;
 	unsigned bank = memMapperRegs[address >> 14];
-	return ((bank & MASK) << 14) | (address & 0x3FFF);
+	return ((bank & MEMORY_MAPPER_MASK) << 14) | (address & 0x3FFF);
 }
 
 byte MegaFlashRomSCCPlusSD::readMemSubSlot2(word addr)
@@ -779,6 +756,27 @@ byte* MegaFlashRomSCCPlusSD::getWriteCacheLineSubSlot2(word addr) const
 	return checkedRam->getWriteCacheLine(calcMemMapperAddress(addr));
 }
 
+byte MegaFlashRomSCCPlusSD::MapperIO::readIO(word port, EmuTime::param time)
+{
+	return peekIO(port, time);
+}
+
+byte MegaFlashRomSCCPlusSD::MapperIO::peekIO(word port, EmuTime::param /*time*/) const
+{
+	return getSelectedSegment(port & 3) | ~MEMORY_MAPPER_MASK;
+}
+
+void MegaFlashRomSCCPlusSD::MapperIO::writeIO(word port, byte value, EmuTime::param /*time*/)
+{
+	mega.memMapperRegs[port & 3] = value & MEMORY_MAPPER_MASK;
+	mega.invalidateDeviceRWCache(0x4000 * (port & 0x03), 0x4000);
+}
+
+byte MegaFlashRomSCCPlusSD::MapperIO::getSelectedSegment(byte page) const
+{
+	return mega.memMapperRegs[page];
+}
+
 /////////////////////// sub slot 3 ////////////////////////////////////////////
 
 unsigned MegaFlashRomSCCPlusSD::getFlashAddrSubSlot3(unsigned addr) const
@@ -797,7 +795,7 @@ byte MegaFlashRomSCCPlusSD::readMemSubSlot3(word addr, EmuTime::param /*time*/)
 	if ((0x4000 <= addr) && (addr < 0xC000)) {
 		// read (flash)rom content
 		unsigned flashAddr = getFlashAddrSubSlot3(addr);
-		return flash->read(flashAddr);
+		return flash.read(flashAddr);
 	} else {
 		// unmapped read
 		return 0xFF;
@@ -809,14 +807,11 @@ byte MegaFlashRomSCCPlusSD::peekMemSubSlot3(word addr, EmuTime::param /*time*/)
 	if ((0x4000 <= addr) && (addr < 0xC000)) {
 		// read (flash)rom content
 		unsigned flashAddr = getFlashAddrSubSlot3(addr);
-		return flash->peek(flashAddr);
+		return flash.peek(flashAddr);
 	} else {
 		// unmapped read
 		return 0xFF;
 	}
-
-	// no peek possible for SD card
-	return 0xFF;
 }
 
 const byte* MegaFlashRomSCCPlusSD::getReadCacheLineSubSlot3(word addr) const
@@ -828,11 +823,10 @@ const byte* MegaFlashRomSCCPlusSD::getReadCacheLineSubSlot3(word addr) const
 	if ((0x4000 <= addr) && (addr < 0xC000)) {
 		// (flash)rom content
 		unsigned flashAddr = getFlashAddrSubSlot3(addr);
-		return flash->getReadCacheLine(flashAddr);
+		return flash.getReadCacheLine(flashAddr);
 	} else {
 		return unmappedRead;
 	}
-	return nullptr;
 }
 
 void MegaFlashRomSCCPlusSD::writeMemSubSlot3(word addr, byte value, EmuTime::param /*time*/)
@@ -857,7 +851,7 @@ void MegaFlashRomSCCPlusSD::writeMemSubSlot3(word addr, byte value, EmuTime::par
 	if ((0x6000 <= addr) && (addr < 0x8000)) {
 		byte page8kB = (addr >> 11) & 0x03;
 		bankRegsSubSlot3[page8kB] = value;
-		invalidateMemCache(0x4000 + 0x2000 * page8kB, 0x2000);
+		invalidateDeviceRWCache(0x4000 + 0x2000 * page8kB, 0x2000);
 	}
 }
 
@@ -868,39 +862,23 @@ byte* MegaFlashRomSCCPlusSD::getWriteCacheLineSubSlot3(word /*addr*/) const
 
 /////////////////////// I/O ////////////////////////////////////////////
 
-byte MegaFlashRomSCCPlusSD::readIO(word port, EmuTime::param time)
-{
-	// Note: it's not possible to read from the Memory Mapper ports
-	assert((port & 0xFF) == 0x12 || (isPSGalsoMappedToNormalPorts() && ((port & 0xFF) == 0xA2))); (void)port;
-	return psg->readRegister(psgLatch, time);
-}
-
-byte MegaFlashRomSCCPlusSD::peekIO(word port, EmuTime::param time) const
-{
-	assert((port & 0xFF) == 0x12 || (isPSGalsoMappedToNormalPorts() && ((port & 0xFF) == 0xA2))); (void)port;
-	return psg->peekRegister(psgLatch, time);
-}
-
 void MegaFlashRomSCCPlusSD::writeIO(word port, byte value, EmuTime::param time)
 {
 	switch (port & 0xFF) {
 		case 0xA0:
 			if (!isPSGalsoMappedToNormalPorts()) return;
+			[[fallthrough]];
 		case 0x10:
 			psgLatch = value & 0x0F;
 			break;
+
 		case 0xA1:
 			if (!isPSGalsoMappedToNormalPorts()) return;
+			[[fallthrough]];
 		case 0x11:
-			psg->writeRegister(psgLatch, value, time);
-			break;
-		case 0xFC:
-		case 0xFD:
-		case 0xFE:
-		case 0xFF:
-			memMapperRegs[port & 3] = value;
-			invalidateMemCache(0x4000 * (port & 3), 0x4000);
+			psg.writeRegister(psgLatch, value, time);
 			break;
+
 		default:
 			UNREACHABLE;
 	}
@@ -913,22 +891,22 @@ void MegaFlashRomSCCPlusSD::serialize(Archive& ar, unsigned /*version*/)
 	ar.template serializeBase<MSXDevice>(*this);
 
 	// overall
-	ar.serialize("flash", *flash);
-	ar.serialize("subslotReg", subslotReg);
+	ar.serialize("flash",      flash,
+	             "subslotReg", subslotReg);
 
 	// subslot 0 stuff
 	// (nothing)
 
 	// subslot 1 stuff
-	ar.serialize("scc", *scc);
-	ar.serialize("sccMode", sccMode);
-	ar.serialize("sccBanks", sccBanks);
-	ar.serialize("psg", *psg);
-	ar.serialize("psgLatch", psgLatch);
-	ar.serialize("configReg", configReg);
-	ar.serialize("mapperReg", mapperReg);
-	ar.serialize("offsetReg", offsetReg);
-	ar.serialize("bankRegsSubSlot1", bankRegsSubSlot1);
+	ar.serialize("scc",       scc,
+	             "sccMode",   sccMode,
+	             "sccBanks",  sccBanks,
+	             "psg",       psg,
+	             "psgLatch",  psgLatch,
+	             "configReg", configReg,
+	             "mapperReg", mapperReg,
+	             "offsetReg", offsetReg,
+	             "bankRegsSubSlot1", bankRegsSubSlot1);
 	if (ar.isLoader()) {
 		// Re-register PSG ports (if needed)
 		byte tmp = configReg;
@@ -942,10 +920,10 @@ void MegaFlashRomSCCPlusSD::serialize(Archive& ar, unsigned /*version*/)
 	ar.serialize("memMapperRegs", memMapperRegs);
 
 	// subslot 3 stuff
-	ar.serialize("bankRegsSubSlot3", bankRegsSubSlot3);
-	ar.serialize("selectedCard", selectedCard);
-	ar.serialize("sdCard0", *sdCard[0]);
-	ar.serialize("sdCard1", *sdCard[1]);
+	ar.serialize("bankRegsSubSlot3", bankRegsSubSlot3,
+	             "selectedCard",     selectedCard,
+	             "sdCard0",          *sdCard[0],
+	             "sdCard1",          *sdCard[1]);
 }
 INSTANTIATE_SERIALIZE_METHODS(MegaFlashRomSCCPlusSD);
 REGISTER_MSXDEVICE(MegaFlashRomSCCPlusSD, "MegaFlashRomSCCPlusSD");
