@@ -5,11 +5,12 @@
 #include "Base64.hh"
 #include "HexDump.hh"
 #include "MSXException.hh"
+#include "one_of.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include <zlib.h>
 #include <algorithm>
 #include <cstring>
-#include <zlib.h>
+#include <memory>
 
 using std::string;
 
@@ -28,25 +29,25 @@ private:
 
 
 Ram::Ram(const DeviceConfig& config, const string& name,
-         const string& description, unsigned size)
+         const string& description, unsigned size_)
 	: xml(*config.getXML())
-	, ram(size)
-	, debuggable(make_unique<RamDebuggable>(
+	, ram(size_)
+	, size(size_)
+	, debuggable(std::make_unique<RamDebuggable>(
 		config.getMotherBoard(), name, description, *this))
 {
 	clear();
 }
 
-Ram::Ram(const DeviceConfig& config, unsigned size)
-	: xml(*config.getXML())
-	, ram(size)
+Ram::Ram(const XMLElement& xml_, unsigned size_)
+	: xml(xml_)
+	, ram(size_)
+	, size(size_)
 {
 	clear();
 }
 
-Ram::~Ram()
-{
-}
+Ram::~Ram() = default;
 
 void Ram::clear(byte c)
 {
@@ -55,26 +56,30 @@ void Ram::clear(byte c)
 		const string& encoding = init->getAttribute("encoding");
 		size_t done = 0;
 		if (encoding == "gz-base64") {
-			string tmp = Base64::decode(init->getData());
+			auto [buf, bufSize] = Base64::decode(init->getData());
 			uLongf dstLen = getSize();
 			if (uncompress(reinterpret_cast<Bytef*>(ram.data()), &dstLen,
-			               reinterpret_cast<const Bytef*>(tmp.data()), uLong(tmp.size()))
+			               reinterpret_cast<const Bytef*>(buf.data()), uLong(bufSize))
 			     != Z_OK) {
 				throw MSXException("Error while decompressing initialContent.");
 			}
 			done = dstLen;
-		} else if ((encoding == "hex") || (encoding == "base64")) {
-			string out = (encoding == "hex")
-			           ? HexDump::decode(init->getData())
-				   : Base64 ::decode(init->getData());
-			done = std::min(ram.size(), out.size());
-			memcpy(ram.data(), out.data(), done);
+		} else if (encoding == one_of("hex", "base64")) {
+			auto [buf, bufSize] = (encoding == "hex")
+			       ? HexDump::decode(init->getData())
+			       : Base64 ::decode(init->getData());
+			if (bufSize == 0) {
+				throw MSXException("Zero-length initial pattern");
+			}
+			done = std::min(size_t(size), bufSize);
+			memcpy(ram.data(), buf.data(), done);
 		} else {
-			throw MSXException("Unsupported encoding \"" + encoding + "\" for initialContent");
+			throw MSXException("Unsupported encoding \"", encoding,
+			                   "\" for initialContent");
 		}
 
 		// repeat pattern over whole ram
-		auto left = ram.size() - done;
+		auto left = size - done;
 		while (left) {
 			auto tmp = std::min(done, left);
 			memcpy(&ram[done], &ram[0], tmp);
@@ -83,7 +88,7 @@ void Ram::clear(byte c)
 		}
 	} else {
 		// no init pattern specified
-		memset(ram.data(), c, ram.size());
+		memset(ram.data(), c, size);
 	}
 
 }
@@ -93,10 +98,10 @@ const string& Ram::getName() const
 	return debuggable->getName();
 }
 
-RamDebuggable::RamDebuggable(MSXMotherBoard& motherBoard,
-                             const string& name,
-                             const string& description, Ram& ram_)
-	: SimpleDebuggable(motherBoard, name, description, ram_.getSize())
+RamDebuggable::RamDebuggable(MSXMotherBoard& motherBoard_,
+                             const string& name_,
+                             const string& description_, Ram& ram_)
+	: SimpleDebuggable(motherBoard_, name_, description_, ram_.getSize())
 	, ram(ram_)
 {
 }
@@ -115,7 +120,7 @@ void RamDebuggable::write(unsigned address, byte value)
 template<typename Archive>
 void Ram::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize_blob("ram", ram.data(), ram.size());
+	ar.serialize_blob("ram", ram.data(), size);
 }
 INSTANTIATE_SERIALIZE_METHODS(Ram);
 
