@@ -3,7 +3,6 @@
 #include "XMLElement.hh"
 #include "RomInfo.hh"
 #include "RomDatabase.hh"
-#include "File.hh"
 #include "FileContext.hh"
 #include "Filename.hh"
 #include "FileException.hh"
@@ -18,10 +17,11 @@
 #include "EmptyPatch.hh"
 #include "IPSPatch.hh"
 #include "StringOp.hh"
+#include "ranges.hh"
 #include "sha1.hh"
-#include "memory.hh"
-#include <limits>
 #include <cstring>
+#include <limits>
+#include <memory>
 
 using std::string;
 using std::unique_ptr;
@@ -37,20 +37,21 @@ public:
 	const std::string& getDescription() const override;
 	byte read(unsigned address) override;
 	void write(unsigned address, byte value) override;
+	void moved(Rom& r);
 private:
 	Debugger& debugger;
-	Rom& rom;
+	Rom* rom;
 };
 
 
-Rom::Rom(const string& name_, const string& description_,
-         const DeviceConfig& config, const string& id /*= ""*/)
-	: name(name_), description(description_)
+Rom::Rom(string name_, string description_,
+         const DeviceConfig& config, const string& id /*= {}*/)
+	: name(std::move(name_)), description(std::move(description_))
 {
 	// Try all <rom> tags with matching "id" attribute.
 	string errors;
 	for (auto& c : config.getXML()->getChildren("rom")) {
-		if (c->getAttribute("id", "") == id) {
+		if (c->getAttribute("id", {}) == id) {
 			try {
 				init(config.getMotherBoard(), *c, config.getFileContext());
 				return;
@@ -65,12 +66,11 @@ Rom::Rom(const string& name_, const string& description_,
 	}
 	if (errors.empty()) {
 		// No matching <rom> tag.
-		StringOp::Builder err;
-		err << "Missing <rom> tag";
+		string err = "Missing <rom> tag";
 		if (!id.empty()) {
-			err << " with id=\"" << id << '"';
+			strAppend(err, " with id=\"", id, '"');
 		}
-		throw ConfigException(err);
+		throw ConfigException(std::move(err));
 	} else {
 		// We got at least one matching <rom>, but it failed to load.
 		// Report error messages of all failed attempts.
@@ -116,8 +116,7 @@ void Rom::init(MSXMotherBoard& motherBoard, const XMLElement& config,
 		// first try already resolved filename ..
 		if (resolvedFilenameElem) {
 			try {
-				file = make_unique<File>(
-					resolvedFilenameElem->getData());
+				file = File(resolvedFilenameElem->getData());
 			} catch (FileException&) {
 				// ignore
 			}
@@ -125,20 +124,20 @@ void Rom::init(MSXMotherBoard& motherBoard, const XMLElement& config,
 		// .. then try the actual sha1sum ..
 		auto fileType = context.isUserContext()
 			? FilePool::ROM : FilePool::SYSTEM_ROM;
-		if (!file && resolvedSha1Elem) {
+		if (!file.is_open() && resolvedSha1Elem) {
 			Sha1Sum sha1(resolvedSha1Elem->getData());
 			file = filepool.getFile(fileType, sha1);
-			if (file) {
+			if (file.is_open()) {
 				// avoid recalculating same sha1 later
 				originalSha1 = sha1;
 			}
 		}
 		// .. and then try filename as originally given by user ..
-		if (!file) {
+		if (!file.is_open()) {
 			for (auto& f : filenames) {
 				try {
 					Filename filename(f->getData(), context);
-					file = make_unique<File>(filename);
+					file = File(filename);
 				} catch (FileException&) {
 					// ignore
 				}
@@ -146,11 +145,11 @@ void Rom::init(MSXMotherBoard& motherBoard, const XMLElement& config,
 		}
 		// .. then try all alternative sha1sums ..
 		// (this might retry the actual sha1sum)
-		if (!file) {
+		if (!file.is_open()) {
 			for (auto& s : sums) {
 				Sha1Sum sha1(s->getData());
 				file = filepool.getFile(fileType, sha1);
-				if (file) {
+				if (file.is_open()) {
 					// avoid recalculating same sha1 later
 					originalSha1 = sha1;
 					break;
@@ -158,20 +157,18 @@ void Rom::init(MSXMotherBoard& motherBoard, const XMLElement& config,
 			}
 		}
 		// .. still no file, then error
-		if (!file) {
-			StringOp::Builder error;
-			error << "Couldn't find ROM file for \""
-			      << name << '"';
+		if (!file.is_open()) {
+			string error = strCat("Couldn't find ROM file for \"", name, '"');
 			if (!filenames.empty()) {
-				error << ' ' << filenames.front()->getData();
+				strAppend(error, ' ', filenames.front()->getData());
 			}
 			if (resolvedSha1Elem) {
-				error << " (sha1: " << resolvedSha1Elem->getData() << ')';
+				strAppend(error, " (sha1: ", resolvedSha1Elem->getData(), ')');
 			} else if (!sums.empty()) {
-                               error << " (sha1: " << sums.front()->getData() << ')';
+				strAppend(error, " (sha1: ", sums.front()->getData(), ')');
 			}
-			error << '.';
-			throw MSXException(error);
+			strAppend(error, '.');
+			throw MSXException(std::move(error));
 		}
 
 		// actually read file content
@@ -183,35 +180,31 @@ void Rom::init(MSXMotherBoard& motherBoard, const XMLElement& config,
 				"supported.");
 		}
 		try {
-			size_t size2;
-			rom = file->mmap(size2);
-			if (size2 > std::numeric_limits<decltype(size)>::max()) {
-				throw MSXException("Rom file too big: " +
-				                   file->getURL());
+			auto mmap = file.mmap();
+			if (mmap.size() > std::numeric_limits<decltype(size)>::max()) {
+				throw MSXException("Rom file too big: ", file.getURL());
 			}
-			size = unsigned(size2);
+			rom = mmap.data();
+			size = unsigned(mmap.size());
 		} catch (FileException&) {
-			throw MSXException("Error reading ROM image: " +
-					   file->getURL());
+			throw MSXException("Error reading ROM image: ", file.getURL());
 		}
 
 		// For file-based roms, calc sha1 via File::getSha1Sum(). It can
 		// possibly use the FilePool cache to avoid the calculation.
 		if (originalSha1.empty()) {
-			file->setFilePool(filepool);
-			originalSha1 = file->getSha1Sum();
+			originalSha1 = filepool.getSha1Sum(file);
 		}
 
 		// verify SHA1
 		if (!checkSHA1(config)) {
 			motherBoard.getMSXCliComm().printWarning(
-				StringOp::Builder() <<
-				"SHA1 sum for '" << name <<
-				"' does not match with sum of '" <<
-				file->getURL() << "'.");
+				"SHA1 sum for '", name,
+				"' does not match with sum of '",
+				file.getURL(), "'.");
 		}
 
-		// We loaded an extrenal file, so check.
+		// We loaded an external file, so check.
 		checkResolvedSha1 = true;
 
 	} else {
@@ -234,12 +227,12 @@ void Rom::init(MSXMotherBoard& motherBoard, const XMLElement& config,
 			getOriginalSHA1();
 
 			unique_ptr<PatchInterface> patch =
-				make_unique<EmptyPatch>(rom, size);
+				std::make_unique<EmptyPatch>(rom, size);
 
 			for (auto& p : patchesElem->getChildren("ips")) {
-				Filename filename(p->getData(), context);
-				patch = make_unique<IPSPatch>(
-					filename, std::move(patch));
+				patch = std::make_unique<IPSPatch>(
+					Filename(p->getData(), context),
+					std::move(patch));
 			}
 			auto patchSize = unsigned(patch->getSize());
 			if (patchSize <= size) {
@@ -252,7 +245,7 @@ void Rom::init(MSXMotherBoard& motherBoard, const XMLElement& config,
 			}
 
 			// calculated because it's different from original
-			patchedSha1 = SHA1::calc(rom, size);
+			actualSha1 = SHA1::calc(rom, size);
 
 			// Content altered by external patch file -> check.
 			checkResolvedSha1 = true;
@@ -260,41 +253,44 @@ void Rom::init(MSXMotherBoard& motherBoard, const XMLElement& config,
 	}
 
 	// TODO fix this, this is a hack that depends heavily on
-	//      HardwareConig::createRomConfig
+	//      HardwareConfig::createRomConfig
 	if (StringOp::startsWith(name, "MSXRom")) {
-		const auto* romInfo = motherBoard.getReactor().
-			getSoftwareDatabase().fetchRomInfo(getOriginalSHA1());
-		if (romInfo && !romInfo->getTitle().empty()) {
-			name = romInfo->getTitle().str();
+		auto& db = motherBoard.getReactor().getSoftwareDatabase();
+		std::string_view title;
+		if (const auto* romInfo = db.fetchRomInfo(getOriginalSHA1())) {
+			title = romInfo->getTitle(db.getBufferStart());
+		}
+		if (!title.empty()) {
+			name = title;
 		} else {
 			// unknown ROM, use file name
-			name = file->getOriginalName();
+			name = file.getOriginalName();
 		}
 	}
 
+	// Make name unique wrt all registered debuggables.
+	auto& debugger = motherBoard.getDebugger();
 	if (size) {
-		auto& debugger = motherBoard.getDebugger();
 		if (debugger.findDebuggable(name)) {
 			unsigned n = 0;
 			string tmp;
 			do {
-				tmp = StringOp::Builder() << name << " (" << ++n << ')';
+				tmp = strCat(name, " (", ++n, ')');
 			} while (debugger.findDebuggable(tmp));
-			name = tmp;
+			name = std::move(tmp);
 		}
-		romDebuggable = make_unique<RomDebuggable>(debugger, *this);
 	}
 
 	if (checkResolvedSha1) {
 		auto& mutableConfig = const_cast<XMLElement&>(config);
-		string patchedSha1Str = getPatchedSHA1().toString();
+		string patchedSha1Str = getSHA1().toString();
 		const auto& actualSha1Elem = mutableConfig.getCreateChild(
 			"resolvedSha1", patchedSha1Str);
 		if (actualSha1Elem.getData() != patchedSha1Str) {
-			string tmp = file ? file->getURL() : name;
+			string tmp = file.is_open() ? file.getURL() : name;
 			// can only happen in case of loadstate
 			motherBoard.getMSXCliComm().printWarning(
-				"The content of the rom " + tmp + " has "
+				"The content of the rom ", tmp, " has "
 				"changed since the time this savestate was "
 				"created. This might result in emulation "
 				"problems.");
@@ -307,80 +303,111 @@ void Rom::init(MSXMotherBoard& motherBoard, const XMLElement& config,
 		unsigned windowBase = windowElem->getAttributeAsInt("base", 0);
 		unsigned windowSize = windowElem->getAttributeAsInt("size", size);
 		if ((windowBase + windowSize) > size) {
-			throw MSXException(StringOp::Builder() <<
-				"The specified window [" << windowBase << ',' <<
-				windowBase + windowSize << ") falls outside "
-				"the rom (with size " << size << ").");
+			throw MSXException(
+				"The specified window [", windowBase, ',',
+				windowBase + windowSize, ") falls outside "
+				"the rom (with size ", size, ").");
 		}
 		rom = &rom[windowBase];
 		size = windowSize;
-		patchedSha1 = SHA1::calc(rom, size);
+	}
+
+	// Only create the debuggable once all checks succeeded.
+	if (size) {
+		romDebuggable = std::make_unique<RomDebuggable>(debugger, *this);
 	}
 }
 
-bool Rom::checkSHA1(const XMLElement& config)
+bool Rom::checkSHA1(const XMLElement& config) const
 {
 	auto sums = config.getChildren("sha1");
 	if (sums.empty()) {
 		return true;
 	}
 	auto& sha1sum = getOriginalSHA1();
-	for (auto& s : sums) {
-		if (Sha1Sum(s->getData()) == sha1sum) {
-			return true;
-		}
-	}
-	return false;
+	return ranges::any_of(sums, [&](auto& s) {
+		return Sha1Sum(s->getData()) == sha1sum;
+	});
 }
 
-Rom::~Rom()
+Rom::Rom(Rom&& r) noexcept
+	: rom          (std::move(r.rom))
+	, extendedRom  (std::move(r.extendedRom))
+	, file         (std::move(r.file))
+	, originalSha1 (std::move(r.originalSha1))
+	, actualSha1   (std::move(r.actualSha1))
+	, name         (std::move(r.name))
+	, description  (std::move(r.description))
+	, size         (std::move(r.size))
+	, romDebuggable(std::move(r.romDebuggable))
 {
+	if (romDebuggable) romDebuggable->moved(*this);
 }
 
+Rom::~Rom() = default;
+
 string Rom::getFilename() const
 {
-	return file ? file->getURL() : "";
+	return file.is_open() ? file.getURL() : string{};
 }
 
 const Sha1Sum& Rom::getOriginalSHA1() const
 {
 	if (originalSha1.empty()) {
-		assert(patchedSha1.empty());
 		originalSha1 = SHA1::calc(rom, size);
 	}
 	return originalSha1;
 }
-const Sha1Sum& Rom::getPatchedSHA1() const
+
+const Sha1Sum& Rom::getSHA1() const
 {
-	return patchedSha1.empty() ? getOriginalSHA1() : patchedSha1;
+	if (actualSha1.empty())
+	{
+		actualSha1 = getOriginalSHA1();
+	}
+	return actualSha1;
 }
 
+void Rom::addPadding(unsigned newSize, byte filler)
+{
+	assert(newSize >= size);
+	if (newSize == size) return;
+
+	MemBuffer<byte> tmp(newSize);
+	auto* newData = tmp.data();
+	memcpy(newData, rom, size);
+	memset(newData + size, filler, newSize - size);
+
+	extendedRom = std::move(tmp);
+	rom = newData;
+	size = newSize;
+}
 
 RomDebuggable::RomDebuggable(Debugger& debugger_, Rom& rom_)
-	: debugger(debugger_), rom(rom_)
+	: debugger(debugger_), rom(&rom_)
 {
-	debugger.registerDebuggable(rom.getName(), *this);
+	debugger.registerDebuggable(rom->getName(), *this);
 }
 
 RomDebuggable::~RomDebuggable()
 {
-	debugger.unregisterDebuggable(rom.getName(), *this);
+	debugger.unregisterDebuggable(rom->getName(), *this);
 }
 
 unsigned RomDebuggable::getSize() const
 {
-	return rom.getSize();
+	return rom->getSize();
 }
 
 const string& RomDebuggable::getDescription() const
 {
-	return rom.getDescription();
+	return rom->getDescription();
 }
 
 byte RomDebuggable::read(unsigned address)
 {
 	assert(address < getSize());
-	return rom[address];
+	return (*rom)[address];
 }
 
 void RomDebuggable::write(unsigned /*address*/, byte /*value*/)
@@ -388,4 +415,9 @@ void RomDebuggable::write(unsigned /*address*/, byte /*value*/)
 	// ignore
 }
 
+void RomDebuggable::moved(Rom& r)
+{
+	rom = &r;
+}
+
 } // namespace openmsx
