@@ -1,49 +1,39 @@
 #include "RomDatabase.hh"
-#include "InfoTopic.hh"
-#include "CommandException.hh"
-#include "TclObject.hh"
+#include "RomInfo.hh"
 #include "FileContext.hh"
 #include "File.hh"
 #include "FileOperations.hh"
-#include "GlobalCommandController.hh"
 #include "CliComm.hh"
+#include "MSXException.hh"
 #include "StringOp.hh"
-#include "StringMap.hh"
+#include "String32.hh"
+#include "hash_map.hh"
+#include "ranges.hh"
 #include "rapidsax.hh"
 #include "unreachable.hh"
-#include "memory.hh"
 #include "stl.hh"
+#include "view.hh"
+#include "xxhash.hh"
+#include <cassert>
+#include <stdexcept>
 
 using std::string;
+using std::string_view;
 using std::vector;
 
 namespace openmsx {
 
-class SoftwareInfoTopic final : public InfoTopic
-{
-public:
-        SoftwareInfoTopic(InfoCommand& openMSXInfoCommand, RomDatabase& romDatabase);
-
-        void execute(array_ref<TclObject> tokens,
-                     TclObject& result) const override;
-        string help(const vector<string>& tokens) const override;
-        void tabCompletion(vector<string>& tokens) const override;
-
-private:
-	const RomDatabase& romDatabase;
-};
-
-
-typedef StringMap<unsigned> UnknownTypes;
+using UnknownTypes = hash_map<string, unsigned, XXHasher>;
 
 class DBParser : public rapidsax::NullHandler
 {
 public:
 	DBParser(RomDatabase::RomDB& db_, UnknownTypes& unknownTypes_,
-	         CliComm& cliComm_)
+	         CliComm& cliComm_, char* bufStart_)
 		: db(db_)
 		, unknownTypes(unknownTypes_)
 		, cliComm(cliComm_)
+		, bufStart(bufStart_)
 		, state(BEGIN)
 		, unknownLevel(0)
 		, initialSize(db.size())
@@ -51,15 +41,16 @@ public:
 	}
 
 	// rapidsax handler interface
-	void start(string_ref name);
-	void attribute(string_ref name, string_ref value);
-	void text(string_ref text);
+	void start(string_view tag);
+	void attribute(string_view name, string_view value);
+	void text(string_view txt);
 	void stop();
-	void doctype(string_ref text);
+	void doctype(string_view txt);
 
-	string_ref getSystemID() const { return systemID; }
+	string_view getSystemID() const { return systemID; }
 
 private:
+	String32 cIndex(string_view str);
 	void addEntries();
 	void addAllEntries();
 
@@ -85,9 +76,9 @@ private:
 	};
 
 	struct Dump {
-		string_ref remark;
+		String32 remark;
 		Sha1Sum hash;
-		string_ref origData;
+		String32 origData;
 		RomType type;
 		bool origValue;
 	};
@@ -95,17 +86,18 @@ private:
 	RomDatabase::RomDB& db;
 	UnknownTypes& unknownTypes;
 	CliComm& cliComm;
+	char* bufStart;
 
-	string_ref systemID;
-	string_ref type;
-	string_ref startVal;
+	string_view systemID;
+	string_view type;
+	string_view startVal;
 
 	vector<Dump> dumps;
-	string_ref system;
-	string_ref title;
-	string_ref company;
-	string_ref year;
-	string_ref country;
+	string_view system;
+	String32 title;
+	String32 company;
+	String32 year;
+	String32 country;
 	int genMSXid;
 
 	State state;
@@ -113,13 +105,14 @@ private:
 	size_t initialSize;
 };
 
-void DBParser::start(string_ref tag)
+void DBParser::start(string_view tag)
 {
 	if (unknownLevel) {
 		++unknownLevel;
 		return;
 	}
 
+	assert(!tag.empty()); // rapidsax will reject empty tags
 	switch (state) {
 	case BEGIN:
 		if (tag == "softwaredb") {
@@ -128,12 +121,12 @@ void DBParser::start(string_ref tag)
 		}
 		throw MSXException("Expected <softwaredb> as root tag.");
 	case SOFTWAREDB:
-		if (tag == "software") {
-			system.clear();
-			title.clear();
-			company.clear();
-			year.clear();
-			country.clear();
+		if (small_compare<'s','o','f','t','w','a','r','e'>(tag)) {
+			system = string_view();
+			toString32(bufStart, bufStart, title);
+			toString32(bufStart, bufStart, company);
+			toString32(bufStart, bufStart, year);
+			toString32(bufStart, bufStart, country);
 			genMSXid = 0;
 			dumps.clear();
 			state = SOFTWARE;
@@ -141,47 +134,48 @@ void DBParser::start(string_ref tag)
 		}
 		break;
 	case SOFTWARE: {
-		char c = tag.front();
-		tag.pop_front();
-		switch (c) {
+		switch (tag.front()) {
 		case 's':
-			if (tag == "ystem") {
+			if (small_compare<'s','y','s','t','e','m'>(tag)) {
 				state = SYSTEM;
 				return;
 			}
 			break;
 		case 't':
-			if (tag == "itle") {
+			tag.remove_prefix(1);
+			if (small_compare<'i','t','l','e'>(tag)) {
 				state = TITLE;
 				return;
 			}
 			break;
 		case 'c':
-			if (tag == "ompany") {
+			if (small_compare<'c','o','m','p','a','n','y'>(tag)) {
 				state = COMPANY;
 				return;
-			} else if (tag == "ountry") {
+			} else if (small_compare<'c','o','u','n','t','r','y'>(tag)) {
 				state = COUNTRY;
 				return;
 			}
 			break;
 		case 'y':
-			if (tag == "ear") {
+			if (small_compare<'y','e','a','r'>(tag)) {
 				state = YEAR;
 				return;
 			}
 			break;
 		case 'g':
-			if (tag == "enmsxid") {
+			if (small_compare<'g','e','n','m','s','x','i','d'>(tag)) {
 				state = GENMSXID;
 				return;
 			}
 			break;
 		case 'd':
-			if (tag == "ump") {
+			if (small_compare<'d','u','m','p'>(tag)) {
 				dumps.resize(dumps.size() + 1);
 				dumps.back().type = ROM_UNKNOWN;
 				dumps.back().origValue = false;
+				toString32(bufStart, bufStart, dumps.back().remark);
+				toString32(bufStart, bufStart, dumps.back().origData);
 				state = DUMP;
 				return;
 			}
@@ -190,28 +184,27 @@ void DBParser::start(string_ref tag)
 		break;
 	}
 	case DUMP: {
-		char c = tag.front();
-		tag.pop_front();
-		switch (c) {
+		switch (tag.front()) {
 		case 'o':
-			if (tag == "riginal") {
+			if (small_compare<'o','r','i','g','i','n','a','l'>(tag)) {
 				dumps.back().origValue = false;
 				state = ORIGINAL;
 				return;
 			}
 			break;
 		case 'm':
-			if (tag == "egarom") {
-				type.clear();
-				startVal.clear();
+			if (small_compare<'m','e','g','a','r','o','m'>(tag)) {
+				type = string_view();
+				startVal = string_view();
 				state = ROM;
 				return;
 			}
 			break;
 		case 'r':
-			if (tag == "om") {
+			tag.remove_prefix(1);
+			if (small_compare<'o','m'>(tag)) {
 				type = "Mirrored";
-				startVal.clear();
+				startVal = string_view();
 				state = ROM;
 				return;
 			}
@@ -220,29 +213,28 @@ void DBParser::start(string_ref tag)
 		break;
 	}
 	case ROM: {
-		char c = tag.front();
-		tag.pop_front();
-		switch (c) {
+		switch (tag.front()) {
 		case 't':
-			if (tag == "ype") {
+			if (small_compare<'t','y','p','e'>(tag)) {
 				state = TYPE;
 				return;
 			}
 			break;
 		case 's':
-			if (tag == "tart") {
+			tag.remove_prefix(1);
+			if (small_compare<'t','a','r','t'>(tag)) {
 				state = START;
 				return;
 			}
 			break;
 		case 'r':
-			if (tag == "emark") {
+			if (small_compare<'r','e','m','a','r','k'>(tag)) {
 				state = DUMP_REMARK;
 				return;
 			}
 			break;
 		case 'h':
-			if (tag == "ash") {
+			if (small_compare<'h','a','s','h'>(tag)) {
 				state = HASH;
 				return;
 			}
@@ -251,7 +243,7 @@ void DBParser::start(string_ref tag)
 		break;
 	}
 	case DUMP_REMARK:
-		if (tag == "text") {
+		if (small_compare<'t','e','x','t'>(tag)) {
 			state = DUMP_TEXT;
 			return;
 		}
@@ -270,7 +262,7 @@ void DBParser::start(string_ref tag)
 		break;
 
 	case END:
-		throw MSXException("Unexpected opening tag: " + tag);
+		throw MSXException("Unexpected opening tag: ", tag);
 
 	default:
 		UNREACHABLE;
@@ -279,13 +271,13 @@ void DBParser::start(string_ref tag)
 	++unknownLevel;
 }
 
-void DBParser::attribute(string_ref name, string_ref value)
+void DBParser::attribute(string_view name, string_view value)
 {
 	if (unknownLevel) return;
 
 	switch (state) {
 	case ORIGINAL:
-		if (name == "value") {
+		if (small_compare<'v','a','l','u','e'>(name)) {
 			dumps.back().origValue = StringOp::stringToBool(value);
 		}
 		break;
@@ -312,49 +304,51 @@ void DBParser::attribute(string_ref name, string_ref value)
 	}
 }
 
-void DBParser::text(string_ref text)
+void DBParser::text(string_view txt)
 {
 	if (unknownLevel) return;
 
 	switch (state) {
 	case SYSTEM:
-		system = text;
+		system = txt;
 		break;
 	case TITLE:
-		title = text;
+		title = cIndex(txt);
 		break;
 	case COMPANY:
-		company = text;
+		company = cIndex(txt);
 		break;
 	case YEAR:
-		year = text;
+		year = cIndex(txt);
 		break;
 	case COUNTRY:
-		country = text;
+		country = cIndex(txt);
 		break;
 	case GENMSXID:
-		genMSXid = stoi(text);
-		// TODO error checks?
-		//	cliComm.printWarning(StringOp::Builder() <<
-		//		"Ignoring bad Generation MSX id (genmsxid) "
-		//		"in entry with title '" << title <<
-		//		": " << data);
+		try {
+			genMSXid = StringOp::fast_stou(txt);
+		} catch (std::invalid_argument&) {
+			cliComm.printWarning(
+				"Ignoring bad Generation MSX id (genmsxid) "
+				"in entry with title '", title,
+				": ", txt);
+		}
 		break;
 	case ORIGINAL:
-		dumps.back().origData = text;
+		dumps.back().origData = cIndex(txt);
 		break;
 	case TYPE:
-		type = text;
+		type = txt;
 		break;
 	case START:
-		startVal = text;
+		startVal = txt;
 		break;
 	case HASH:
-		dumps.back().hash = Sha1Sum(text);
+		dumps.back().hash = Sha1Sum(txt);
 		break;
 	case DUMP_REMARK:
 	case DUMP_TEXT:
-		dumps.back().remark = text;
+		dumps.back().remark = cIndex(txt);
 		break;
 	case BEGIN:
 	case SOFTWAREDB:
@@ -368,20 +362,24 @@ void DBParser::text(string_ref text)
 	}
 }
 
+String32 DBParser::cIndex(string_view str)
+{
+	auto* begin = const_cast<char*>(str.data());
+	auto* end = begin + str.size();
+	*end = 0;
+	String32 result;
+	toString32(bufStart, begin, result);
+	return result;
+}
+
 // called on </software>
 void DBParser::addEntries()
 {
-	if (!system.empty() && (system != "MSX")) {
-		// skip non-MSX entries
-		return;
-	}
-
-	for (auto& d : dumps) {
-		db.emplace_back(d.hash, RomInfo(
-			title, year, company, country,
-			d.origValue, d.origData, d.remark, d.type,
-			genMSXid));
-	}
+	append(db, view::transform(dumps, [&](auto& d) {
+		return std::pair(d.hash,
+		                 RomInfo(title, year, company, country, d.origValue,
+		                         d.origData, d.remark, d.type, genMSXid));
+	}));
 }
 
 // called on </softwaredb>
@@ -397,7 +395,7 @@ void DBParser::addAllEntries()
 	if (mid == last) return; // no new entries
 
 	// Sort new entries, old entries are already sorted.
-	sort(mid, last, LessTupleElement<0>());
+	std::sort(mid, last, LessTupleElement<0>());
 
 	// Filter duplicates from new entries. This is similar to the
 	// unique() algorithm, except that it also warns about duplicates.
@@ -412,7 +410,7 @@ void DBParser::addAllEntries()
 	while (it2 != last) {
 		if (it1->first == it2->first) {
 			cliComm.printWarning(
-				"duplicate softwaredb entry SHA1: " +
+				"duplicate softwaredb entry SHA1: ",
 				it2->first.toString());
 		} else {
 			++it1;
@@ -456,10 +454,10 @@ void DBParser::addAllEntries()
 	swap(result, db);
 }
 
-static const char* parseStart(string_ref s)
+static const char* parseStart(string_view s)
 {
 	// we expect "0x0000", "0x4000", "0x8000", "0xc000" or ""
-	return ((s.size() == 6) && s.starts_with("0x")) ? (s.data() + 2) : nullptr;
+	return ((s.size() == 6) && StringOp::startsWith(s, "0x")) ? (s.data() + 2) : nullptr;
 }
 
 void DBParser::stop()
@@ -497,24 +495,24 @@ void DBParser::stop()
 		state = DUMP;
 		break;
 	case ROM: {
-		string_ref t = type;
+		string_view t = type;
 		char buf[12];
-		if (t == "Mirrored") {
-			if (const char* start = parseStart(startVal)) {
+		if (small_compare<'M','i','r','r','o','r','e','d'>(t)) {
+			if (const char* s = parseStart(startVal)) {
 				memcpy(buf, t.data(), 8);
-				memcpy(buf + 8, start, 4);
-				t = string_ref(buf, 12);
+				memcpy(buf + 8, s, 4);
+				t = string_view(buf, 12);
 			}
-		} else if (t == "Normal") {
-			if (const char* start = parseStart(startVal)) {
+		} else if (small_compare<'N','o','r','m','a','l'>(t)) {
+			if (const char* s = parseStart(startVal)) {
 				memcpy(buf, t.data(), 6);
-				memcpy(buf + 6, start, 4);
-				t = string_ref(buf, 10);
+				memcpy(buf + 6, s, 4);
+				t = string_view(buf, 10);
 			}
 		}
 		RomType romType = RomInfo::nameToRomType(t);
 		if (romType == ROM_UNKNOWN) {
-			unknownTypes[t]++;
+			unknownTypes[string(t)]++;
 		}
 		dumps.back().type = romType;
 		state = DUMP;
@@ -538,28 +536,21 @@ void DBParser::stop()
 	}
 }
 
-void DBParser::doctype(string_ref text)
+void DBParser::doctype(string_view txt)
 {
-	auto pos1 = text.find(" SYSTEM \"");
-	if (pos1 == string_ref::npos) return;
-	auto t = text.substr(pos1 + 9);
+	auto pos1 = txt.find(" SYSTEM \"");
+	if (pos1 == string_view::npos) return;
+	auto t = txt.substr(pos1 + 9);
 	auto pos2 = t.find('"');
-	if (pos2 == string_ref::npos) return;
+	if (pos2 == string_view::npos) return;
 	systemID = t.substr(0, pos2);
 }
 
-static void parseDB(CliComm& cliComm, const string& filename,
-                    MemBuffer<char>& buf, RomDatabase::RomDB& db,
-                    UnknownTypes& unknownTypes)
+static void parseDB(CliComm& cliComm, char* buf, char* bufStart,
+                    RomDatabase::RomDB& db, UnknownTypes& unknownTypes)
 {
-	File file(filename);
-	auto size = file.getSize();
-	buf.resize(size + 1);
-	file.read(buf.data(), size);
-	buf[size] = 0;
-
-	DBParser handler(db, unknownTypes, cliComm);
-	rapidsax::parse<rapidsax::trimWhitespace>(handler, buf.data());
+	DBParser handler(db, unknownTypes, cliComm, bufStart);
+	rapidsax::parse<rapidsax::trimWhitespace>(handler, buf);
 
 	if (handler.getSystemID() != "softwaredb1.dtd") {
 		throw rapidsax::ParseError(
@@ -569,22 +560,18 @@ static void parseDB(CliComm& cliComm, const string& filename,
 	}
 }
 
-RomDatabase::RomDatabase(GlobalCommandController& commandController, CliComm& cliComm)
-	: softwareInfoTopic(make_unique<SoftwareInfoTopic>(
-		commandController.getOpenMSXInfoCommand(), *this))
+RomDatabase::RomDatabase(CliComm& cliComm)
 {
 	db.reserve(3500);
 	UnknownTypes unknownTypes;
 	// first user- then system-directory
-	vector<string> paths = SystemFileContext().getPaths();
+	vector<string> paths = systemFileContext().getPaths();
+	vector<File> files;
+	size_t bufferSize = 0;
 	for (auto& p : paths) {
-		string filename = FileOperations::join(p, "softwaredb.xml");
 		try {
-			buffers.emplace_back();
-			parseDB(cliComm, filename, buffers.back(), db, unknownTypes);
-		} catch (rapidsax::ParseError& e) {
-			cliComm.printWarning(StringOp::Builder() <<
-				"Rom database parsing failed: " << e.what());
+			auto& f = files.emplace_back(FileOperations::join(p, "softwaredb.xml"));
+			bufferSize += f.getSize() + rapidsax::EXTRA_BUFFER_SPACE;
 		} catch (MSXException& /*e*/) {
 			// Ignore. It's not unusual the DB in the user
 			// directory is not found. In case there's an error
@@ -592,87 +579,44 @@ RomDatabase::RomDatabase(GlobalCommandController& commandController, CliComm& cl
 			// warning, but that's done below.
 		}
 	}
+	buffer.resize(bufferSize);
+	size_t bufferOffset = 0;
+	for (auto& file : files) {
+		try {
+			auto size = file.getSize();
+			auto* buf = &buffer[bufferOffset];
+			bufferOffset += size + rapidsax::EXTRA_BUFFER_SPACE;
+			file.read(buf, size);
+			buf[size] = 0;
+
+			parseDB(cliComm, buf, buffer.data(), db, unknownTypes);
+		} catch (rapidsax::ParseError& e) {
+			cliComm.printWarning(
+				"Rom database parsing failed: ", e.what());
+		} catch (MSXException& /*e*/) {
+			// Ignore, see above
+		}
+	}
+	if (bufferSize) buffer[0] = 0;
 	if (db.empty()) {
 		cliComm.printWarning(
 			"Couldn't load software database.\n"
 			"This may cause incorrect ROM mapper types to be used.");
 	}
 	if (!unknownTypes.empty()) {
-		StringOp::Builder output;
-		output << "Unknown mapper types in software database: ";
-		for (auto& p : unknownTypes) {
-			output << p.first() << " (" << p.second << "x); ";
+		string output = "Unknown mapper types in software database: ";
+		for (const auto& [type, count] : unknownTypes) {
+			strAppend(output, type, " (", count, "x); ");
 		}
 		cliComm.printWarning(output);
 	}
 }
 
-RomDatabase::~RomDatabase()
-{
-}
-
 const RomInfo* RomDatabase::fetchRomInfo(const Sha1Sum& sha1sum) const
 {
-	auto it = lower_bound(begin(db), end(db), sha1sum,
-	                      LessTupleElement<0>());
+	auto it = ranges::lower_bound(db, sha1sum, LessTupleElement<0>());
 	return ((it != end(db)) && (it->first == sha1sum))
 		? &it->second : nullptr;
 }
 
-
-// SoftwareInfoTopic
-
-SoftwareInfoTopic::SoftwareInfoTopic(InfoCommand& openMSXInfoCommand,
-                                     RomDatabase& romDatabase_)
-	: InfoTopic(openMSXInfoCommand, "software")
-	, romDatabase(romDatabase_)
-{
-}
-
-void SoftwareInfoTopic::execute(array_ref<TclObject> tokens,
-                                TclObject& result) const
-{
-	if (tokens.size() != 3) {
-		throw CommandException("Wrong number of parameters");
-	}
-
-	Sha1Sum sha1sum = Sha1Sum(tokens[2].getString());
-	const RomInfo* romInfo = romDatabase.fetchRomInfo(sha1sum);
-	if (!romInfo) {
-		// no match found
-		throw CommandException(
-			"Software with sha1sum " + sha1sum.toString() + " not found");
-	}
-
-	result.addListElement("title");
-	result.addListElement(romInfo->getTitle());
-	result.addListElement("year");
-	result.addListElement(romInfo->getYear());
-	result.addListElement("company");
-	result.addListElement(romInfo->getCompany());
-	result.addListElement("country");
-	result.addListElement(romInfo->getCountry());
-	result.addListElement("orig_type");
-	result.addListElement(romInfo->getOrigType());
-	result.addListElement("remark");
-	result.addListElement(romInfo->getRemark());
-	result.addListElement("original");
-	result.addListElement(romInfo->getOriginal());
-	result.addListElement("mapper_type_name");
-	result.addListElement(RomInfo::romTypeToName(romInfo->getRomType()));
-	result.addListElement("genmsxid");
-	result.addListElement(romInfo->getGenMSXid());
-}
-
-string SoftwareInfoTopic::help(const vector<string>& /*tokens*/) const
-{
-	return "Returns information about the software "
-	       "given its sha1sum, in a paired list.";
-}
-
-void SoftwareInfoTopic::tabCompletion(vector<string>& /*tokens*/) const
-{
-	// no useful completion possible
-}
-
 } // namespace openmsx
