@@ -34,65 +34,39 @@
 
 #include "RomFSA1FM.hh"
 #include "CacheLine.hh"
-#include "Rom.hh"
 #include "SRAM.hh"
 #include "MSXMotherBoard.hh"
-#include "FirmwareSwitch.hh"
 #include "MSXException.hh"
+#include "one_of.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include <memory>
 
 namespace openmsx {
 
 // 8kb shared sram //
 
-RomFSA1FMSram::RomFSA1FMSram(const DeviceConfig& config)
-	: motherBoard(config.getMotherBoard())
+static std::shared_ptr<SRAM> getSram(const DeviceConfig& config)
 {
-	auto& info = motherBoard.getSharedStuff("FSA1FM-sram");
-	if (info.counter == 0) {
-		assert(!info.stuff);
-		info.stuff = new SRAM(config.getAttribute("id") + " SRAM",
-		                      0x2000, config);
-	}
-	++info.counter;
-	fsSram = reinterpret_cast<SRAM*>(info.stuff);
-}
-
-RomFSA1FMSram::~RomFSA1FMSram()
-{
-	auto& info = motherBoard.getSharedStuff("FSA1FM-sram");
-	auto* sram = reinterpret_cast<SRAM*>(info.stuff);
-	assert(info.counter);
-	assert(sram);
-	assert(sram == fsSram);
-	--info.counter;
-	if (info.counter == 0) {
-		delete sram;
-		info.stuff = nullptr;
-	}
+	return config.getMotherBoard().getSharedStuff<SRAM>(
+		"FSA1FM-sram",
+		config.getAttribute("id") + " SRAM", 0x2000, config);
 }
 
 
 // Mapper for slot 3-1 //
 
-RomFSA1FM1::RomFSA1FM1(const DeviceConfig& config, std::unique_ptr<Rom> rom_)
+RomFSA1FM1::RomFSA1FM1(const DeviceConfig& config, Rom&& rom_)
 	: MSXRom(config, std::move(rom_))
-	, RomFSA1FMSram(config)
-	, firmwareSwitch(make_unique<FirmwareSwitch>(config))
+	, fsSram(getSram(config))
+	, firmwareSwitch(config)
 {
-	if ((rom->getSize() != 0x100000) &&
-	    (rom->getSize() != 0x200000)) {
+	if (rom.getSize() != one_of(0x100000u, 0x200000u)) {
 		throw MSXException(
 			"Rom for FSA1FM mapper must be 1MB in size "
 			"(some dumps are 2MB, those can be used as well).");
 	}
 }
 
-RomFSA1FM1::~RomFSA1FM1()
-{
-}
-
 void RomFSA1FM1::reset(EmuTime::param /*time*/)
 {
 	// initial rom bank is undefined
@@ -102,14 +76,14 @@ byte RomFSA1FM1::peekMem(word address, EmuTime::param /*time*/) const
 {
 	if ((0x4000 <= address) && (address < 0x6000)) {
 		// read rom
-		return (*rom)[(0x2000 * ((*fsSram)[0x1FC4] & 0x0F)) +
+		return rom[(0x2000 * ((*fsSram)[0x1FC4] & 0x0F)) +
 		              (address & 0x1FFF)];
 	} else if ((0x7FC0 <= address) && (address < 0x7FD0)) {
 		switch (address & 0x0F) {
 		case 4:
 			return (*fsSram)[address & 0x1FFF];
 		case 6:
-			return firmwareSwitch->getStatus() ? 0xFB : 0xFF;
+			return firmwareSwitch.getStatus() ? 0xFB : 0xFF;
 		default:
 			return 0xFF;
 		}
@@ -134,7 +108,7 @@ const byte* RomFSA1FM1::getReadCacheLine(word address) const
 		return nullptr;
 	} else if ((0x4000 <= address) && (address < 0x6000)) {
 		// read rom
-		return &(*rom)[(0x2000 * ((*fsSram)[0x1FC4] & 0x0F)) +
+		return &rom[(0x2000 * ((*fsSram)[0x1FC4] & 0x0F)) +
 		               (address & 0x1FFF)];
 	} else if ((0x6000 <= address) && (address < 0x8000)) {
 		// read sram
@@ -151,7 +125,7 @@ void RomFSA1FM1::writeMem(word address, byte value, EmuTime::param /*time*/)
 	if ((0x6000 <= address) && (address < 0x8000)) {
 		if (address == 0x7FC4) {
 			// switch rom bank
-			invalidateMemCache(0x4000, 0x2000);
+			invalidateDeviceRCache(0x4000, 0x2000);
 		}
 		fsSram->write(address & 0x1FFF, value);
 	}
@@ -183,25 +157,23 @@ REGISTER_MSXDEVICE(RomFSA1FM1, "RomFSA1FM1");
 
 // Mapper for slot 3-3 //
 
-RomFSA1FM2::RomFSA1FM2(const DeviceConfig& config, std::unique_ptr<Rom> rom)
-	: Rom8kBBlocks(config, std::move(rom))
-	, RomFSA1FMSram(config)
+RomFSA1FM2::RomFSA1FM2(const DeviceConfig& config, Rom&& rom_)
+	: Rom8kBBlocks(config, std::move(rom_))
+	, fsSram(getSram(config))
 {
 	reset(EmuTime::dummy());
 }
 
-RomFSA1FM2::~RomFSA1FM2()
-{
-}
-
 void RomFSA1FM2::reset(EmuTime::param /*time*/)
 {
 	control = 0;
 	for (int region = 0; region < 6; ++region) {
 		changeBank(region, 0xA8);
 	}
-	changeBank(6, 0);
+	changeBank(6, 0); // for mapper-state read-back
 	changeBank(7, 0);
+	setUnmapped(6);
+	setUnmapped(7);
 }
 
 byte RomFSA1FM2::peekMem(word address, EmuTime::param time) const
@@ -303,11 +275,14 @@ void RomFSA1FM2::changeBank(byte region, byte bank)
 			isRam[region]   = false;
 			isEmpty[region] = true;
 		}
-		invalidateMemCache(0x2000 * region, 0x2000);
+		invalidateDeviceRWCache(0x2000 * region, 0x2000);
 	} else {
 		isRam[region]   = false;
 		isEmpty[region] = false;
 		setRom(region, bank & 0x7F);
+		if (region == 3) {
+			invalidateDeviceRCache(0x7FF0 & CacheLine::HIGH, CacheLine::SIZE);
+		}
 	}
 }
 
@@ -317,9 +292,9 @@ void RomFSA1FM2::serialize(Archive& ar, unsigned /*version*/)
 	ar.template serializeBase<Rom8kBBlocks>(*this);
 	// note: SRAM can be serialized in this class (as opposed to
 	//       Rom8kBBlocks), because we don't use setBank to map it
-	ar.serialize("SRAM", *fsSram);
-	ar.serialize("bankSelect", bankSelect);
-	ar.serialize("control", control);
+	ar.serialize("SRAM",       *fsSram,
+	             "bankSelect", bankSelect,
+	             "control",    control);
 	if (ar.isLoader()) {
 		// recalculate 'isRam' and 'isEmpty' from bankSelect
 		for (int region = 0; region < 8; ++region) {
