@@ -37,24 +37,19 @@
 // read the same 4Kb of SRAM if SRAM is selected.
 
 #include "RomGameMaster2.hh"
-#include "Rom.hh"
 #include "SRAM.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include <memory>
 
 namespace openmsx {
 
-RomGameMaster2::RomGameMaster2(const DeviceConfig& config, std::unique_ptr<Rom> rom)
-	: Rom4kBBlocks(config, std::move(rom), 1)
+RomGameMaster2::RomGameMaster2(const DeviceConfig& config, Rom&& rom_)
+	: Rom4kBBlocks(config, std::move(rom_), 1)
 {
-	sram = make_unique<SRAM>(getName() + " SRAM", 0x2000, config);
+	sram = std::make_unique<SRAM>(getName() + " SRAM", 0x2000, config);
 	reset(EmuTime::dummy());
 }
 
-RomGameMaster2::~RomGameMaster2()
-{
-}
-
 void RomGameMaster2::reset(EmuTime::param /*time*/)
 {
 	for (int i = 0; i < 4; i++) {
@@ -77,6 +72,7 @@ void RomGameMaster2::writeMem(word address, byte value, EmuTime::param /*time*/)
 			byte region = address >> 12; // 0x6, 0x8 or 0xA
 			if (region == 0x0A) {
 				sramEnabled = (value & 0x10) != 0;
+				invalidateDeviceWCache(0xB000, 0x1000); // 'R' is handled below
 			}
 			if (value & 0x10) {
 				// switch SRAM
@@ -117,8 +113,8 @@ template<typename Archive>
 void RomGameMaster2::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.template serializeBase<Rom4kBBlocks>(*this);
-	ar.serialize("sramOffset", sramOffset);
-	ar.serialize("sramEnabled", sramEnabled);
+	ar.serialize("sramOffset",  sramOffset,
+	             "sramEnabled", sramEnabled);
 }
 INSTANTIATE_SERIALIZE_METHODS(RomGameMaster2);
 REGISTER_MSXDEVICE(RomGameMaster2, "RomGameMaster2");
