@@ -25,29 +25,25 @@
 
 #include "RomHalnote.hh"
 #include "CacheLine.hh"
-#include "Rom.hh"
 #include "SRAM.hh"
 #include "MSXException.hh"
+#include "one_of.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include <memory>
 
 namespace openmsx {
 
-RomHalnote::RomHalnote(const DeviceConfig& config, std::unique_ptr<Rom> rom_)
+RomHalnote::RomHalnote(const DeviceConfig& config, Rom&& rom_)
 	: Rom8kBBlocks(config, std::move(rom_))
 {
-	if (rom->getSize() != 0x100000) {
+	if (rom.getSize() != 0x100000) {
 		throw MSXException(
 			"Rom for HALNOTE mapper must be exactly 1MB in size.");
 	}
-	sram = make_unique<SRAM>(getName() + " SRAM", 0x4000, config);
+	sram = std::make_unique<SRAM>(getName() + " SRAM", 0x4000, config);
 	reset(EmuTime::dummy());
 }
 
-RomHalnote::~RomHalnote()
-{
-}
-
 void RomHalnote::reset(EmuTime::param /*time*/)
 {
 	subBanks[0] = subBanks[1] = 0;
@@ -68,7 +64,7 @@ const byte* RomHalnote::getReadCacheLine(word address) const
 	if (subMapperEnabled && (0x7000 <= address) && (address < 0x8000)) {
 		// sub-mapper
 		int subBank = address < 0x7800 ? 0 : 1;
-		return &(*rom)[0x80000 + subBanks[subBank] * 0x800 + (address & 0x7FF)];
+		return &rom[0x80000 + subBanks[subBank] * 0x800 + (address & 0x7FF)];
 	} else {
 		// default mapper implementation
 		return Rom8kBBlocks::getReadCacheLine(address);
@@ -88,13 +84,13 @@ void RomHalnote::writeMem(word address, byte value, EmuTime::param /*time*/)
 			sram->write(address, value);
 		}
 	} else if (address < 0xC000) {
-		if ((address == 0x77FF) || (address == 0x7FFF)) {
+		if (address == one_of(0x77FF, 0x7FFF)) {
 			// sub-mapper bank switch region
 			int subBank = address < 0x7800 ? 0 : 1;
 			if (subBanks[subBank] != value) {
 				subBanks[subBank] = value;
 				if (subMapperEnabled) {
-					invalidateMemCache(
+					invalidateDeviceRCache(
 						0x7000 + subBank * 0x800, 0x800);
 				}
 			}
@@ -114,13 +110,14 @@ void RomHalnote::writeMem(word address, byte value, EmuTime::param /*time*/)
 						setUnmapped(0);
 						setUnmapped(1);
 					}
+					// 'R' is already handled
+					invalidateDeviceWCache(0x0000, 0x4000);
 				}
 			} else if (bank == 3) {
 				// sub-mapper enable/disable
-				bool newSubMapperEnabled = (value & 0x80) != 0;
-				if (newSubMapperEnabled != subMapperEnabled) {
-					subMapperEnabled = newSubMapperEnabled;
-					invalidateMemCache(0x7000, 0x1000);
+				subMapperEnabled = (value & 0x80) != 0;
+				if (subMapperEnabled) {
+					invalidateDeviceRCache(0x7000, 0x1000);
 				}
 			}
 		}
@@ -135,8 +132,8 @@ byte* RomHalnote::getWriteCacheLine(word address) const
 			return nullptr;
 		}
 	} else if (address < 0xC000) {
-		if (((address & CacheLine::HIGH) == (0x77FF & CacheLine::HIGH)) ||
-		    ((address & CacheLine::HIGH) == (0x7FFF & CacheLine::HIGH))) {
+		if ((address & CacheLine::HIGH) == one_of(0x77FF & CacheLine::HIGH,
+		                                          0x7FFF & CacheLine::HIGH)) {
 			// sub-mapper bank switch region
 			return nullptr;
 		} else if ((address & 0x1FFF & CacheLine::HIGH) ==
@@ -152,9 +149,9 @@ template<typename Archive>
 void RomHalnote::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.template serializeBase<Rom8kBBlocks>(*this);
-	ar.serialize("subBanks", subBanks);
-	ar.serialize("sramEnabled", sramEnabled);
-	ar.serialize("subMapperEnabled", subMapperEnabled);
+	ar.serialize("subBanks",         subBanks,
+	             "sramEnabled",      sramEnabled,
+	             "subMapperEnabled", subMapperEnabled);
 
 }
 INSTANTIATE_SERIALIZE_METHODS(RomHalnote);
