@@ -12,38 +12,35 @@
 //  bank 4: 0xB000 - 0xB7ff (0xB000 used)
 
 #include "RomKonamiSCC.hh"
-#include "SCC.hh"
 #include "CacheLine.hh"
-#include "Rom.hh"
 #include "MSXMotherBoard.hh"
 #include "CliComm.hh"
+#include "sha1.hh"
 #include "serialize.hh"
-#include "memory.hh"
 
 namespace openmsx {
 
-RomKonamiSCC::RomKonamiSCC(const DeviceConfig& config, std::unique_ptr<Rom> rom_)
+// minimal attempt to avoid seeing this warning too often
+static Sha1Sum alreadyWarnedForSha1Sum;
+
+RomKonamiSCC::RomKonamiSCC(const DeviceConfig& config, Rom&& rom_)
 	: Rom8kBBlocks(config, std::move(rom_))
-	, scc(make_unique<SCC>("SCC", config, getCurrentTime()))
+	, scc("SCC", config, getCurrentTime())
 {
-
 	// warn if a ROM is used that would not work on a real KonamiSCC mapper
-	if (rom->getSize() > 512 * 1024) {
-		getMotherBoard().getMSXCliComm().printWarning("The size of "
-				"this ROM image is larger than 512kB, which is "
-				"not supported on real Konami SCC mapper "
-				"chips!"); }
-
+	if ((rom.getSize() > 512 * 1024) && alreadyWarnedForSha1Sum != rom.getOriginalSHA1()) {
+		getMotherBoard().getMSXCliComm().printWarning(
+			"The size of this ROM image is larger than 512kB, "
+			"which is not supported on real Konami SCC mapper "
+			"chips!");
+		alreadyWarnedForSha1Sum = rom.getOriginalSHA1();
+	}
 	powerUp(getCurrentTime());
 }
 
-RomKonamiSCC::~RomKonamiSCC()
-{
-}
-
 void RomKonamiSCC::powerUp(EmuTime::param time)
 {
-	scc->powerUp(time);
+	scc.powerUp(time);
 	reset(time);
 }
 
@@ -58,13 +55,13 @@ void RomKonamiSCC::reset(EmuTime::param time)
 	setUnmapped(7);
 
 	sccEnabled = false;
-	scc->reset(time);
+	scc.reset(time);
 }
 
 byte RomKonamiSCC::peekMem(word address, EmuTime::param time) const
 {
 	if (sccEnabled && (0x9800 <= address) && (address < 0xA000)) {
-		return scc->peekMem(address & 0xFF, time);
+		return scc.peekMem(address & 0xFF, time);
 	} else {
 		return Rom8kBBlocks::peekMem(address, time);
 	}
@@ -73,7 +70,7 @@ byte RomKonamiSCC::peekMem(word address, EmuTime::param time) const
 byte RomKonamiSCC::readMem(word address, EmuTime::param time)
 {
 	if (sccEnabled && (0x9800 <= address) && (address < 0xA000)) {
-		return scc->readMem(address & 0xFF, time);
+		return scc.readMem(address & 0xFF, time);
 	} else {
 		return Rom8kBBlocks::readMem(address, time);
 	}
@@ -96,17 +93,24 @@ void RomKonamiSCC::writeMem(word address, byte value, EmuTime::param time)
 	}
 	if (sccEnabled && (0x9800 <= address) && (address < 0xA000)) {
 		// write to SCC
-		scc->writeMem(address & 0xFF, value, time);
+		scc.writeMem(address & 0xFF, value, time);
 		return;
 	}
 	if ((address & 0xF800) == 0x9000) {
 		// SCC enable/disable
-		sccEnabled = ((value & 0x3F) == 0x3F);
-		invalidateMemCache(0x9800, 0x0800);
+		bool newSccEnabled = ((value & 0x3F) == 0x3F);
+		if (newSccEnabled != sccEnabled) {
+			sccEnabled = newSccEnabled;
+			invalidateDeviceRWCache(0x9800, 0x0800);
+		}
 	}
 	if ((address & 0x1800) == 0x1000) {
 		// page selection
-		setRom(address >> 13, value);
+		byte region = address >> 13;
+		setRom(region, value);
+		if ((region == 4) && sccEnabled) {
+			invalidateDeviceRCache(0x9800, 0x0800);
+		}
 	}
 }
 
@@ -132,8 +136,8 @@ template<typename Archive>
 void RomKonamiSCC::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.template serializeBase<Rom8kBBlocks>(*this);
-	ar.serialize("scc", *scc);
-	ar.serialize("sccEnabled", sccEnabled);
+	ar.serialize("scc",        scc,
+	             "sccEnabled", sccEnabled);
 }
 INSTANTIATE_SERIALIZE_METHODS(RomKonamiSCC);
 REGISTER_MSXDEVICE(RomKonamiSCC, "RomKonamiSCC");
