@@ -4,22 +4,22 @@
 #include "DeviceConfig.hh"
 #include "SRAM.hh"
 #include "CacheLine.hh"
-#include "Rom.hh"
+#include "one_of.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include <memory>
 
 namespace openmsx {
 
 const int SRAM_BASE = 0x80;
 const int RAM_BASE  = 0x180;
 
-RomPanasonic::RomPanasonic(const DeviceConfig& config, std::unique_ptr<Rom> rom_)
+RomPanasonic::RomPanasonic(const DeviceConfig& config, Rom&& rom_)
 	: Rom8kBBlocks(config, std::move(rom_))
 	, panasonicMem(getMotherBoard().getPanasonicMemory())
 {
 	unsigned sramSize = config.getChildDataAsInt("sramsize", 0);
 	if (sramSize) {
-		sram = make_unique<SRAM>(
+		sram = std::make_unique<SRAM>(
 			getName() + " SRAM", sramSize * 1024, config);
 	}
 
@@ -40,10 +40,6 @@ RomPanasonic::RomPanasonic(const DeviceConfig& config, std::unique_ptr<Rom> rom_
 	reset(EmuTime::dummy());
 }
 
-RomPanasonic::~RomPanasonic()
-{
-}
-
 void RomPanasonic::reset(EmuTime::param /*time*/)
 {
 	control = 0;
@@ -51,6 +47,7 @@ void RomPanasonic::reset(EmuTime::param /*time*/)
 		bankSelect[region] = 0;
 		setRom(region, 0);
 	}
+	invalidateDeviceRCache(0x7FF0 & CacheLine::HIGH, CacheLine::SIZE);
 }
 
 byte RomPanasonic::peekMem(word address, EmuTime::param time) const
@@ -97,7 +94,7 @@ void RomPanasonic::writeMem(word address, byte value, EmuTime::param /*time*/)
 	if ((0x6000 <= address) && (address < 0x7FF0)) {
 		// set mapper state (lower 8 bits)
 		int region = (address & 0x1C00) >> 10;
-		if ((region == 5) || (region == 6)) region ^= 3;
+		if (region == one_of(5, 6)) region ^= 3;
 		int selectedBank = bankSelect[region];
 		int newBank = (selectedBank & ~0xFF) | value;
 		changeBank(region, newBank);
@@ -114,7 +111,10 @@ void RomPanasonic::writeMem(word address, byte value, EmuTime::param /*time*/)
 	} else if (address == 0x7FF9) {
 		// write control byte
 		control = value;
-	} else if ((0x8000 <= address) && (address < 0xC000)) {
+	} else {
+		// Tested on real FS-A1GT:
+		//  SRAM/RAM can be written to from all regions, including e.g.
+		//  address 0x7ff0. (I didn't actually test 0xc000-0xffff).
 		int region = address >> 13;
 		int selectedBank = bankSelect[region];
 		if (sram && (SRAM_BASE <= selectedBank) &&
@@ -124,19 +124,17 @@ void RomPanasonic::writeMem(word address, byte value, EmuTime::param /*time*/)
 			sram->write((block * 0x2000) | (address & 0x1FFF), value);
 		} else if (RAM_BASE <= selectedBank) {
 			// RAM
-			const_cast<byte*>(bank[region])[address & 0x1FFF] = value;
+			const_cast<byte*>(bankPtr[region])[address & 0x1FFF] = value;
 		}
 	}
 }
 
 byte* RomPanasonic::getWriteCacheLine(word address) const
 {
-	//return nullptr;
-	if ((0x6000 <= address) && (address < 0x7FF0)) {
+	if ((0x6000 <= address) && (address < 0x8000)) {
+		// mapper select (low/high), control
 		return nullptr;
-	} else if (address == (0x7FF8 & CacheLine::HIGH)) {
-		return nullptr;
-	} else if ((0x8000 <= address) && (address < 0xC000)) {
+	} else {
 		int region = address >> 13;
 		int selectedBank = bankSelect[region];
 		if (sram && (SRAM_BASE <= selectedBank) &&
@@ -145,12 +143,10 @@ byte* RomPanasonic::getWriteCacheLine(word address) const
 			return nullptr;
 		} else if (RAM_BASE <= selectedBank) {
 			// RAM
-			return const_cast<byte*>(&bank[region][address & 0x1FFF]);
+			return const_cast<byte*>(&bankPtr[region][address & 0x1FFF]);
 		} else {
 			return unmappedWrite;
 		}
-	} else {
-		return unmappedWrite;
 	}
 }
 
@@ -178,14 +174,19 @@ void RomPanasonic::changeBank(byte region, int bank)
 		// ROM
 		setRom(region, bank);
 	}
+	invalidateDeviceWCache(0x2000 * region, 0x2000); // 'R' is already handled
+	if (region == 3) {
+		// don't pre-fill [0x7ff0, 0x7fff]
+		invalidateDeviceRCache(0x7FF0 & CacheLine::HIGH, CacheLine::SIZE);
+	}
 }
 
 template<typename Archive>
 void RomPanasonic::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.template serializeBase<Rom8kBBlocks>(*this);
-	ar.serialize("bankSelect", bankSelect);
-	ar.serialize("control", control);
+	ar.serialize("bankSelect", bankSelect,
+	             "control",    control);
 }
 INSTANTIATE_SERIALIZE_METHODS(RomPanasonic);
 REGISTER_MSXDEVICE(RomPanasonic, "RomPanasonic");
