@@ -12,14 +12,13 @@
 
 #include "RomZemina90in1.hh"
 #include "MSXCPUInterface.hh"
-#include "Rom.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
 
 namespace openmsx {
 
-RomZemina90in1::RomZemina90in1(const DeviceConfig& config, std::unique_ptr<Rom> rom)
-	: Rom8kBBlocks(config, std::move(rom))
+RomZemina90in1::RomZemina90in1(const DeviceConfig& config, Rom&& rom_)
+	: Rom8kBBlocks(config, std::move(rom_))
 {
 	reset(EmuTime::dummy());
 	getCPUInterface().register_IO_Out(0x77, this);
@@ -30,13 +29,13 @@ RomZemina90in1::~RomZemina90in1()
 	getCPUInterface().unregister_IO_Out(0x77, this);
 }
 
-void RomZemina90in1::reset(EmuTime::param dummy)
+void RomZemina90in1::reset(EmuTime::param time)
 {
 	setUnmapped(0);
 	setUnmapped(1);
 	setUnmapped(6);
 	setUnmapped(7);
-	writeIO(0x77, 0, dummy);
+	writeIO(0x77, 0, time);
 }
 
 void RomZemina90in1::writeIO(word /*port*/, byte value, EmuTime::param /*time*/)
