@@ -2,6 +2,7 @@
 
 #include "SspiNegotiateServer.hh"
 #include "MSXException.hh"
+#include "one_of.hh"
 #include "openmsx.hh"
 
 namespace openmsx {
@@ -51,15 +52,9 @@ bool SspiNegotiateServer::Authenticate()
 	std::vector<char> buffer;
 	PCtxtHandle phContext = nullptr;
 	while (true) {
-
 		// Receive another buffer from the client
-		PRT_DEBUG("Receiving client chunk");
 		bool ret = RecvChunk(stream, buffer, cbMaxTokenSize);
-		if (!ret) {
-			PRT_DEBUG("RecvChunk failed");
-			return false;
-		}
-		PRT_DEBUG("Received " << buffer.size() << " bytes");
+		if (!ret) return false;
 
 		secClientBuffer.cbBuffer = static_cast<unsigned long>(buffer.size());
 		secClientBuffer.pvBuffer = &buffer[0];
@@ -78,20 +73,15 @@ bool SspiNegotiateServer::Authenticate()
 			&tsContextExpiry);
 
 		DebugPrintSecurityStatus("AcceptSecurityContext", ss);
-		if (ss != SEC_E_OK && ss != SEC_I_CONTINUE_NEEDED) {
+		if (ss != one_of(SEC_E_OK, SEC_I_CONTINUE_NEEDED)) {
 			return false;
 		}
 
 		// If we have something for the client, send it
-		if (secServerBuffer.cbBuffer)
-		{
-			PRT_DEBUG("Sending " << secServerBuffer.cbBuffer << " bytes to client");
-			bool ret = SendChunk(stream, secServerBuffer.pvBuffer, secServerBuffer.cbBuffer);
+		if (secServerBuffer.cbBuffer) {
+			ret = SendChunk(stream, secServerBuffer.pvBuffer, secServerBuffer.cbBuffer);
 			ClearContextBuffers(&secServerBufferDesc);
-			if (!ret) {
-				PRT_DEBUG("SendChunk failed");
-				return false;
-			}
+			if (!ret) return false;
 		}
 
 		// SEC_E_OK means that we're done
@@ -116,8 +106,8 @@ bool SspiNegotiateServer::Authorize()
 	if (!secur32) {
 		return false;
 	}
-	QUERY_SECURITY_CONTEXT_TOKEN_FN QuerySecurityContextToken =
-		(QUERY_SECURITY_CONTEXT_TOKEN_FN)GetProcAddress(secur32, "QuerySecurityContextToken");
+	auto QuerySecurityContextToken = reinterpret_cast<QUERY_SECURITY_CONTEXT_TOKEN_FN>(
+		GetProcAddress(secur32, "QuerySecurityContextToken"));
 	if (!QuerySecurityContextToken) {
 		return false;
 	}
@@ -138,14 +128,13 @@ bool SspiNegotiateServer::Authorize()
 		psd,
 		hClientToken,
 		ACCESS_ALL,
-		(PGENERIC_MAPPING)&mapping,
+		const_cast<PGENERIC_MAPPING>(&mapping),
 		&privilegeSet,
 		&dwPrivSetSize,
 		&dwGranted,
 		&fAccess);
 
 	DebugPrintSecurityBool("AccessCheck", ret);
-	PRT_DEBUG("Access " << (ret && fAccess ? "granted" : "denied"));
 	DebugPrintSecurityPrincipalName(&hContext);
 
 	CloseHandle(hClientToken);
