@@ -4,14 +4,13 @@
 #include "MSXException.hh"
 #include <sddl.h>
 #include <cassert>
-#include "openmsx.hh"
+#include <iostream>
 
 //
 // NOTE: This file MUST be kept in sync between the openmsx and openmsx-debugger projects
 //
 
-namespace openmsx {
-namespace sspiutils {
+namespace openmsx::sspiutils {
 
 SspiPackageBase::SspiPackageBase(StreamWrapper& userStream, const SEC_WCHAR* securityPackage)
 	: stream(userStream)
@@ -44,8 +43,7 @@ void InitTokenContextBuffer(PSecBufferDesc pSecBufferDesc, PSecBuffer pSecBuffer
 
 void ClearContextBuffers(PSecBufferDesc pSecBufferDesc)
 {
-	for (ULONG i = 0; i < pSecBufferDesc->cBuffers; i ++)
-	{
+	for (ULONG i = 0; i < pSecBufferDesc->cBuffers; i ++) {
 		FreeContextBuffer(pSecBufferDesc->pBuffers[i].pvBuffer);
 		pSecBufferDesc->pBuffers[i].cbBuffer = 0;
 		pSecBufferDesc->pBuffers[i].pvBuffer = nullptr;
@@ -56,29 +54,28 @@ void DebugPrintSecurityStatus(const char* context, SECURITY_STATUS ss)
 {
 	(void)&context;
 	(void)&ss;
-#ifdef DEBUG
-	switch (ss)
-	{
+#if 0
+	switch (ss) {
 	case SEC_E_OK:
-		PRT_DEBUG(context << ": SEC_E_OK");
+		std::cerr << context << ": SEC_E_OK\n";
 		break;
 	case SEC_I_CONTINUE_NEEDED:
-		PRT_DEBUG(context << ": SEC_I_CONTINUE_NEEDED");
+		std::cerr << context << ": SEC_I_CONTINUE_NEEDED\n";
 		break;
 	case SEC_E_INVALID_TOKEN:
-		PRT_DEBUG(context << ": SEC_E_INVALID_TOKEN");
+		std::cerr << context << ": SEC_E_INVALID_TOKEN\n";
 		break;
 	case SEC_E_BUFFER_TOO_SMALL:
-		PRT_DEBUG(context << ": SEC_E_BUFFER_TOO_SMALL");
+		std::cerr << context << ": SEC_E_BUFFER_TOO_SMALL\n";
 		break;
 	case SEC_E_INVALID_HANDLE:
-		PRT_DEBUG(context << ": SEC_E_INVALID_HANDLE");
+		std::cerr << context << ": SEC_E_INVALID_HANDLE\n";
 		break;
 	case SEC_E_WRONG_PRINCIPAL:
-		PRT_DEBUG(context << ": SEC_E_WRONG_PRINCIPAL");
+		std::cerr << context << ": SEC_E_WRONG_PRINCIPAL\n";
 		break;
 	default:
-		PRT_DEBUG(context << ": " << ss);
+		std::cerr << context << ": " << ss << '\n';
 		break;
 	}
 #endif
@@ -88,11 +85,11 @@ void DebugPrintSecurityBool(const char* context, BOOL ret)
 {
 	(void)&context;
 	(void)&ret;
-#ifdef DEBUG
+#if 0
 	if (ret) {
-		PRT_DEBUG(context << ": true");
+		std::cerr << context << ": true\n";
 	} else {
-		PRT_DEBUG(context << ": false - " << GetLastError());
+		std::cerr << context << ": false - " << GetLastError() << '\n';
 	}
 #endif
 }
@@ -100,11 +97,11 @@ void DebugPrintSecurityBool(const char* context, BOOL ret)
 void DebugPrintSecurityPackageName(PCtxtHandle phContext)
 {
 	(void)&phContext;
-#ifdef DEBUG
+#if 0
 	SecPkgContext_PackageInfoA package;
 	SECURITY_STATUS ss = QueryContextAttributesA(phContext, SECPKG_ATTR_PACKAGE_INFO, &package);
 	if (ss == SEC_E_OK) {
-		PRT_DEBUG("Using " << package.PackageInfo->Name << " package");
+		std::cerr << "Using " << package.PackageInfo->Name << " package\n";
 	}
 #endif
 }
@@ -112,11 +109,11 @@ void DebugPrintSecurityPackageName(PCtxtHandle phContext)
 void DebugPrintSecurityPrincipalName(PCtxtHandle phContext)
 {
 	(void)&phContext;
-#ifdef DEBUG
+#if 0
 	SecPkgContext_NamesA name;
 	SECURITY_STATUS ss = QueryContextAttributesA(phContext, SECPKG_ATTR_NAMES, &name);
 	if (ss == SEC_E_OK) {
-		PRT_DEBUG("Client principal " << name.sUserName);
+		std::cerr << "Client principal " << name.sUserName << '\n';
 	}
 #endif
 }
@@ -124,7 +121,7 @@ void DebugPrintSecurityPrincipalName(PCtxtHandle phContext)
 void DebugPrintSecurityDescriptor(PSECURITY_DESCRIPTOR psd)
 {
 	(void)&psd;
-#ifdef DEBUG
+#if 0
 	char* sddl;
 	BOOL ret = ConvertSecurityDescriptorToStringSecurityDescriptorA(
 		psd,
@@ -134,7 +131,7 @@ void DebugPrintSecurityDescriptor(PSECURITY_DESCRIPTOR psd)
 		&sddl,
 		nullptr);
 	if (ret) {
-		PRT_DEBUG("SecurityDescriptor: " << sddl);
+		std::cerr << "SecurityDescriptor: " << sddl << '\n';
 		LocalFree(sddl);
 	}
 #endif
@@ -142,7 +139,7 @@ void DebugPrintSecurityDescriptor(PSECURITY_DESCRIPTOR psd)
 
 // If successful, caller must free the results with LocalFree()
 // If unsuccessful, returns null
-PTOKEN_USER GetProcessToken()
+static PTOKEN_USER GetProcessToken()
 {
 	PTOKEN_USER pToken = nullptr;
 
@@ -150,12 +147,11 @@ PTOKEN_USER GetProcessToken()
 	BOOL ret = OpenProcessToken(GetCurrentProcess(), TOKEN_READ, &hProcessToken);
 	DebugPrintSecurityBool("OpenProcessToken", ret);
 	if (ret) {
-
 		DWORD cbToken;
 		ret = GetTokenInformation(hProcessToken, TokenUser, nullptr, 0, &cbToken);
 		assert(!ret && GetLastError() == ERROR_INSUFFICIENT_BUFFER && cbToken);
 
-		pToken = (TOKEN_USER*)LocalAlloc(LMEM_ZEROINIT, cbToken);
+		pToken = static_cast<TOKEN_USER*>(LocalAlloc(LMEM_ZEROINIT, cbToken));
 		if (pToken) {
 			ret = GetTokenInformation(hProcessToken, TokenUser, pToken, cbToken, &cbToken);
 			DebugPrintSecurityBool("GetTokenInformation", ret);
@@ -164,10 +160,8 @@ PTOKEN_USER GetProcessToken()
 				pToken = nullptr;
 			}
 		}
-
 		CloseHandle(hProcessToken);
 	}
-
 	return pToken;
 }
 
@@ -184,28 +178,25 @@ PSECURITY_DESCRIPTOR CreateCurrentUserSecurityDescriptor()
 
 		// Allocate the SD and the ACL in one allocation, so we only have one buffer to manage
 		// The SD structure ends with a pointer, so the start of the ACL will be well aligned
-		BYTE* buffer = (BYTE*)LocalAlloc(LMEM_ZEROINIT, SECURITY_DESCRIPTOR_MIN_LENGTH + cbACL);
+		BYTE* buffer = static_cast<BYTE*>(LocalAlloc(LMEM_ZEROINIT, SECURITY_DESCRIPTOR_MIN_LENGTH + cbACL));
 		if (buffer) {
-			psd = (PSECURITY_DESCRIPTOR)buffer;
-			PACL pacl = (PACL)(buffer + SECURITY_DESCRIPTOR_MIN_LENGTH);
+			psd = static_cast<PSECURITY_DESCRIPTOR>(buffer);
+			PACL pacl = reinterpret_cast<PACL>(buffer + SECURITY_DESCRIPTOR_MIN_LENGTH);
 			PACCESS_ALLOWED_ACE pUserAce;
 			if (InitializeSecurityDescriptor(psd, SECURITY_DESCRIPTOR_REVISION) &&
-				InitializeAcl(pacl, cbACL, ACL_REVISION) &&
-				AddAccessAllowedAce(pacl, ACL_REVISION, ACCESS_ALL, pUserSid) &&
-				SetSecurityDescriptorDacl(psd, TRUE, pacl, FALSE) &&
-				// Need to set the Group and Owner on the SD in order to use it with AccessCheck()
-				GetAce(pacl, 0, (void**)&pUserAce) &&
-				SetSecurityDescriptorGroup(psd, &pUserAce->SidStart, FALSE) &&
-				SetSecurityDescriptorOwner(psd, &pUserAce->SidStart, FALSE))
-			{
+			    InitializeAcl(pacl, cbACL, ACL_REVISION) &&
+			    AddAccessAllowedAce(pacl, ACL_REVISION, ACCESS_ALL, pUserSid) &&
+			    SetSecurityDescriptorDacl(psd, TRUE, pacl, FALSE) &&
+			    // Need to set the Group and Owner on the SD in order to use it with AccessCheck()
+			    GetAce(pacl, 0, reinterpret_cast<void**>(&pUserAce)) &&
+			    SetSecurityDescriptorGroup(psd, &pUserAce->SidStart, FALSE) &&
+			    SetSecurityDescriptorOwner(psd, &pUserAce->SidStart, FALSE)) {
 				buffer = nullptr;
 			} else {
 				psd = nullptr;
 			}
-
 			LocalFree(buffer);
 		}
-
 		LocalFree(pToken);
 	}
 
@@ -213,7 +204,6 @@ PSECURITY_DESCRIPTOR CreateCurrentUserSecurityDescriptor()
 		assert(IsValidSecurityDescriptor(psd));
 		DebugPrintSecurityDescriptor(psd);
 	}
-
 	return psd;
 }
 
@@ -222,24 +212,19 @@ unsigned long GetPackageMaxTokenSize(const SEC_WCHAR* package)
 	PSecPkgInfoW pkgInfo;
 	SECURITY_STATUS ss = QuerySecurityPackageInfoW(const_cast<SEC_WCHAR*>(package), &pkgInfo);
 	DebugPrintSecurityStatus("QuerySecurityPackageInfoW", ss);
-	if (ss != SEC_E_OK) {
-		return 0;
-	}
+	if (ss != SEC_E_OK) return 0;
 
 	unsigned long cbMaxToken = pkgInfo->cbMaxToken;
 	FreeContextBuffer(pkgInfo);
 	return cbMaxToken;
 }
 
-bool Send(StreamWrapper& stream, void* buffer, uint32_t cb)
+static bool Send(StreamWrapper& stream, void* buffer, uint32_t cb)
 {
 	uint32_t sent = 0;
-	while (sent < cb)
-	{
-		uint32_t ret = stream.Write((char*)buffer + sent, cb - sent);
-		if (ret == STREAM_ERROR) {
-			return false;
-		}
+	while (sent < cb) {
+		uint32_t ret = stream.Write(static_cast<char*>(buffer) + sent, cb - sent);
+		if (ret == STREAM_ERROR) return false;
 		sent += ret;
 	}
 	return true;
@@ -254,21 +239,18 @@ bool SendChunk(StreamWrapper& stream, void* buffer, uint32_t cb)
 	return Send(stream, buffer, cb);
 }
 
-bool Recv(StreamWrapper& stream, void* buffer, uint32_t cb)
+static bool Recv(StreamWrapper& stream, void* buffer, uint32_t cb)
 {
 	uint32_t recvd = 0;
 	while (recvd < cb) {
-		uint32_t ret = stream.Read((char*)buffer + recvd, cb - recvd);
-		if (ret == STREAM_ERROR) {
-			return false;
-		}
+		uint32_t ret = stream.Read(static_cast<char*>(buffer) + recvd, cb - recvd);
+		if (ret == STREAM_ERROR) return false;
 		recvd += ret;
 	}
-
 	return true;
 }
 
-bool RecvChunkSize(StreamWrapper& stream, uint32_t* pcb)
+static bool RecvChunkSize(StreamWrapper& stream, uint32_t* pcb)
 {
 	uint32_t cb;
 	bool ret = Recv(stream, &cb, sizeof(cb));
@@ -291,7 +273,6 @@ bool RecvChunk(StreamWrapper& stream, std::vector<char>& buffer, uint32_t cbMaxS
 	return true;
 }
 
-} // namespace sspiutils
-} // namespace openmsx
+} // namespace openmsx::sspiutils
 
 #endif
