@@ -1,69 +1,21 @@
 #include "MSXMidi.hh"
 #include "MidiInDevice.hh"
-#include "I8254.hh"
-#include "I8251.hh"
-#include "MidiOutConnector.hh"
 #include "MSXCPUInterface.hh"
 #include "MSXException.hh"
+#include "outer.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
-#include "memory.hh"
 #include <cassert>
 
 namespace openmsx {
 
 // Documented in MSX-Datapack Vol. 3, section 4 (MSX-MIDI), from page 634
-static const byte LIMITED_RANGE_VALUE = 0x01; // b0 = "E8" => determines port range
-static const byte DISABLED_VALUE      = 0x80; // b7 = EN
-
-class MSXMidiCounter0 final : public ClockPinListener
-{
-public:
-	explicit MSXMidiCounter0(MSXMidi& midi);
-	~MSXMidiCounter0();
-	void signal(ClockPin& pin, EmuTime::param time) override;
-	void signalPosEdge(ClockPin& pin, EmuTime::param time) override;
-private:
-	MSXMidi& midi;
-};
-
-class MSXMidiCounter2 final : public ClockPinListener
-{
-public:
-	explicit MSXMidiCounter2(MSXMidi& midi);
-	~MSXMidiCounter2();
-	void signal(ClockPin& pin, EmuTime::param time) override;
-	void signalPosEdge(ClockPin& pin, EmuTime::param time) override;
-private:
-	MSXMidi& midi;
-};
-
-class MSXMidiI8251Interf final : public I8251Interface
-{
-public:
-	explicit MSXMidiI8251Interf(MSXMidi& midi);
-	~MSXMidiI8251Interf();
-	void setRxRDY(bool status, EmuTime::param time) override;
-	void setDTR(bool status, EmuTime::param time) override;
-	void setRTS(bool status, EmuTime::param time) override;
-	bool getDSR(EmuTime::param time) override;
-	bool getCTS(EmuTime::param time) override;
-	void setDataBits(DataBits bits) override;
-	void setStopBits(StopBits bits) override;
-	void setParityBit(bool enable, ParityBit parity) override;
-	void recvByte(byte value, EmuTime::param time) override;
-	void signal(EmuTime::param time) override;
-private:
-	MSXMidi& midi;
-};
-
+constexpr byte LIMITED_RANGE_VALUE = 0x01; // b0 = "E8" => determines port range
+constexpr byte DISABLED_VALUE      = 0x80; // b7 = EN
 
 MSXMidi::MSXMidi(const DeviceConfig& config)
 	: MSXDevice(config)
 	, MidiInConnector(MSXDevice::getPluggingController(), "msx-midi-in")
-	, cntr0(make_unique<MSXMidiCounter0>(*this))
-	, cntr2(make_unique<MSXMidiCounter2>(*this))
-	, interf(make_unique<MSXMidiI8251Interf>(*this))
 	, timerIRQ(getMotherBoard(), MSXDevice::getName() + ".IRQtimer")
 	, rxrdyIRQ(getMotherBoard(), MSXDevice::getName() + ".IRQrxrdy")
 	, timerIRQlatch(false), timerIRQenabled(false)
@@ -71,20 +23,17 @@ MSXMidi::MSXMidi(const DeviceConfig& config)
 	, isExternalMSXMIDI(config.findChild("external") != nullptr)
 	, isEnabled(!isExternalMSXMIDI)
 	, isLimitedTo8251(true)
-	, outConnector(make_unique<MidiOutConnector>(
-		MSXDevice::getPluggingController(), "msx-midi-out"))
-	, i8251(make_unique<I8251>(getScheduler(), *interf, getCurrentTime()))
-	, i8254(make_unique<I8254>(
-		getScheduler(), cntr0.get(), nullptr, cntr2.get(),
-		getCurrentTime()))
+	, outConnector(MSXDevice::getPluggingController(), "msx-midi-out")
+	, i8251(getScheduler(), interf, getCurrentTime())
+	, i8254(getScheduler(), &cntr0, nullptr, &cntr2, getCurrentTime())
 {
 	EmuDuration total(1.0 / 4e6); // 4MHz
 	EmuDuration hi   (1.0 / 8e6); // 8MHz half clock period
 	EmuTime::param time = getCurrentTime();
-	i8254->getClockPin(0).setPeriodicState(total, hi, time);
-	i8254->getClockPin(1).setState(false, time);
-	i8254->getClockPin(2).setPeriodicState(total, hi, time);
-	i8254->getOutputPin(2).generateEdgeSignals(true, time);
+	i8254.getClockPin(0).setPeriodicState(total, hi, time);
+	i8254.getClockPin(1).setState(false, time);
+	i8254.getClockPin(2).setPeriodicState(total, hi, time);
+	i8254.getOutputPin(2).generateEdgeSignals(true, time);
 	reset(time);
 
 	if (isExternalMSXMIDI) {
@@ -111,7 +60,7 @@ MSXMidi::~MSXMidi()
 	}
 }
 
-void MSXMidi::reset(EmuTime::param /*time*/)
+void MSXMidi::reset(EmuTime::param time)
 {
 	timerIRQlatch = false;
 	timerIRQenabled = false;
@@ -123,6 +72,7 @@ void MSXMidi::reset(EmuTime::param /*time*/)
 	if (isExternalMSXMIDI) {
 		registerIOports(DISABLED_VALUE | LIMITED_RANGE_VALUE); // also resets state
 	}
+	i8251.reset(time);
 }
 
 byte MSXMidi::readIO(word port, EmuTime::param time)
@@ -135,7 +85,7 @@ byte MSXMidi::readIO(word port, EmuTime::param time)
 	switch (port & 7) {
 		case 0: // UART data register
 		case 1: // UART status register
-			return i8251->readIO(port & 1, time);
+			return i8251.readIO(port & 1, time);
 		case 2: // timer interrupt flag off
 		case 3: // no function
 			return 0xFF;
@@ -143,7 +93,7 @@ byte MSXMidi::readIO(word port, EmuTime::param time)
 		case 5: // counter 1 data port
 		case 6: // counter 2 data port
 		case 7: // timer command register
-			return i8254->readIO(port & 3, time);
+			return i8254.readIO(port & 3, time);
 		default:
 			UNREACHABLE; return 0;
 	}
@@ -159,7 +109,7 @@ byte MSXMidi::peekIO(word port, EmuTime::param time) const
 	switch (port & 7) {
 		case 0: // UART data register
 		case 1: // UART status register
-			return i8251->peekIO(port & 1, time);
+			return i8251.peekIO(port & 1, time);
 		case 2: // timer interrupt flag off
 		case 3: // no function
 			return 0xFF;
@@ -167,7 +117,7 @@ byte MSXMidi::peekIO(word port, EmuTime::param time) const
 		case 5: // counter 1 data port
 		case 6: // counter 2 data port
 		case 7: // timer command register
-			return i8254->peekIO(port & 3, time);
+			return i8254.peekIO(port & 3, time);
 		default:
 			UNREACHABLE; return 0;
 	}
@@ -186,7 +136,7 @@ void MSXMidi::writeIO(word port, byte value, EmuTime::param time)
 	switch (port & 7) {
 		case 0: // UART data register
 		case 1: // UART command register
-			i8251->writeIO(port & 1, value, time);
+			i8251.writeIO(port & 1, value, time);
 			break;
 		case 2: // timer interrupt flag off
 			setTimerIRQ(false, time);
@@ -197,7 +147,7 @@ void MSXMidi::writeIO(word port, byte value, EmuTime::param time)
 		case 5: // counter 1 data port
 		case 6: // counter 2 data port
 		case 7: // timer command register
-			i8254->writeIO(port & 3, value, time);
+			i8254.writeIO(port & 3, value, time);
 			break;
 	}
 }
@@ -266,11 +216,7 @@ void MSXMidi::setTimerIRQ(bool status, EmuTime::param time)
 	if (timerIRQlatch != status) {
 		timerIRQlatch = status;
 		if (timerIRQenabled) {
-			if (timerIRQlatch) {
-				timerIRQ.set();
-			} else {
-				timerIRQ.reset();
-			}
+			timerIRQ.set(timerIRQlatch);
 		}
 		updateEdgeEvents(time);
 	}
@@ -281,11 +227,7 @@ void MSXMidi::enableTimerIRQ(bool enabled, EmuTime::param time)
 	if (timerIRQenabled != enabled) {
 		timerIRQenabled = enabled;
 		if (timerIRQlatch) {
-			if (timerIRQenabled) {
-				timerIRQ.set();
-			} else {
-				timerIRQ.reset();
-			}
+			timerIRQ.set(timerIRQenabled);
 		}
 		updateEdgeEvents(time);
 	}
@@ -294,7 +236,7 @@ void MSXMidi::enableTimerIRQ(bool enabled, EmuTime::param time)
 void MSXMidi::updateEdgeEvents(EmuTime::param time)
 {
 	bool wantEdges = timerIRQenabled && !timerIRQlatch;
-	i8254->getOutputPin(2).generateEdgeSignals(wantEdges, time);
+	i8254.getOutputPin(2).generateEdgeSignals(wantEdges, time);
 }
 
 void MSXMidi::setRxRDYIRQ(bool status)
@@ -302,11 +244,7 @@ void MSXMidi::setRxRDYIRQ(bool status)
 	if (rxrdyIRQlatch != status) {
 		rxrdyIRQlatch = status;
 		if (rxrdyIRQenabled) {
-			if (rxrdyIRQlatch) {
-				rxrdyIRQ.set();
-			} else {
-				rxrdyIRQ.reset();
-			}
+			rxrdyIRQ.set(rxrdyIRQlatch);
 		}
 	}
 }
@@ -324,80 +262,72 @@ void MSXMidi::enableRxRDYIRQ(bool enabled)
 
 // I8251Interface  (pass calls from I8251 to outConnector)
 
-MSXMidiI8251Interf::MSXMidiI8251Interf(MSXMidi& midi_)
-	: midi(midi_)
-{
-}
-
-MSXMidiI8251Interf::~MSXMidiI8251Interf()
-{
-}
-
-void MSXMidiI8251Interf::setRxRDY(bool status, EmuTime::param /*time*/)
+void MSXMidi::I8251Interf::setRxRDY(bool status, EmuTime::param /*time*/)
 {
+	auto& midi = OUTER(MSXMidi, interf);
 	midi.setRxRDYIRQ(status);
 }
 
-void MSXMidiI8251Interf::setDTR(bool status, EmuTime::param time)
+void MSXMidi::I8251Interf::setDTR(bool status, EmuTime::param time)
 {
+	auto& midi = OUTER(MSXMidi, interf);
 	midi.enableTimerIRQ(status, time);
 }
 
-void MSXMidiI8251Interf::setRTS(bool status, EmuTime::param /*time*/)
+void MSXMidi::I8251Interf::setRTS(bool status, EmuTime::param /*time*/)
 {
+	auto& midi = OUTER(MSXMidi, interf);
 	midi.enableRxRDYIRQ(status);
 }
 
-bool MSXMidiI8251Interf::getDSR(EmuTime::param /*time*/)
+bool MSXMidi::I8251Interf::getDSR(EmuTime::param /*time*/)
 {
+	auto& midi = OUTER(MSXMidi, interf);
 	return midi.timerIRQ.getState();
 }
 
-bool MSXMidiI8251Interf::getCTS(EmuTime::param /*time*/)
+bool MSXMidi::I8251Interf::getCTS(EmuTime::param /*time*/)
 {
 	return true;
 }
 
-void MSXMidiI8251Interf::setDataBits(DataBits bits)
+void MSXMidi::I8251Interf::setDataBits(DataBits bits)
 {
-	midi.outConnector->setDataBits(bits);
+	auto& midi = OUTER(MSXMidi, interf);
+	midi.outConnector.setDataBits(bits);
 }
 
-void MSXMidiI8251Interf::setStopBits(StopBits bits)
+void MSXMidi::I8251Interf::setStopBits(StopBits bits)
 {
-	midi.outConnector->setStopBits(bits);
+	auto& midi = OUTER(MSXMidi, interf);
+	midi.outConnector.setStopBits(bits);
 }
 
-void MSXMidiI8251Interf::setParityBit(bool enable, ParityBit parity)
+void MSXMidi::I8251Interf::setParityBit(bool enable, ParityBit parity)
 {
-	midi.outConnector->setParityBit(enable, parity);
+	auto& midi = OUTER(MSXMidi, interf);
+	midi.outConnector.setParityBit(enable, parity);
 }
 
-void MSXMidiI8251Interf::recvByte(byte value, EmuTime::param time)
+void MSXMidi::I8251Interf::recvByte(byte value, EmuTime::param time)
 {
-	midi.outConnector->recvByte(value, time);
+	auto& midi = OUTER(MSXMidi, interf);
+	midi.outConnector.recvByte(value, time);
 }
 
-void MSXMidiI8251Interf::signal(EmuTime::param time)
+void MSXMidi::I8251Interf::signal(EmuTime::param time)
 {
+	auto& midi = OUTER(MSXMidi, interf);
 	midi.getPluggedMidiInDev().signal(time);
 }
 
 
 // Counter 0 output
 
-MSXMidiCounter0::MSXMidiCounter0(MSXMidi& midi_)
-	: midi(midi_)
+void MSXMidi::Counter0::signal(ClockPin& pin, EmuTime::param time)
 {
-}
-
-MSXMidiCounter0::~MSXMidiCounter0()
-{
-}
-
-void MSXMidiCounter0::signal(ClockPin& pin, EmuTime::param time)
-{
-	ClockPin& clk = midi.i8251->getClockPin();
+	auto& midi = OUTER(MSXMidi, cntr0);
+	ClockPin& clk = midi.i8251.getClockPin();
 	if (pin.isPeriodic()) {
 		clk.setPeriodicState(pin.getTotalDuration(),
 		                     pin.getHighDuration(), time);
@@ -406,7 +336,7 @@ void MSXMidiCounter0::signal(ClockPin& pin, EmuTime::param time)
 	}
 }
 
-void MSXMidiCounter0::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
+void MSXMidi::Counter0::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
 {
 	UNREACHABLE;
 }
@@ -414,18 +344,10 @@ void MSXMidiCounter0::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
 
 // Counter 2 output
 
-MSXMidiCounter2::MSXMidiCounter2(MSXMidi& midi_)
-	: midi(midi_)
+void MSXMidi::Counter2::signal(ClockPin& pin, EmuTime::param time)
 {
-}
-
-MSXMidiCounter2::~MSXMidiCounter2()
-{
-}
-
-void MSXMidiCounter2::signal(ClockPin& pin, EmuTime::param time)
-{
-	ClockPin& clk = midi.i8254->getClockPin(1);
+	auto& midi = OUTER(MSXMidi, cntr2);
+	ClockPin& clk = midi.i8254.getClockPin(1);
 	if (pin.isPeriodic()) {
 		clk.setPeriodicState(pin.getTotalDuration(),
 		                     pin.getHighDuration(), time);
@@ -434,8 +356,9 @@ void MSXMidiCounter2::signal(ClockPin& pin, EmuTime::param time)
 	}
 }
 
-void MSXMidiCounter2::signalPosEdge(ClockPin& /*pin*/, EmuTime::param time)
+void MSXMidi::Counter2::signalPosEdge(ClockPin& /*pin*/, EmuTime::param time)
 {
+	auto& midi = OUTER(MSXMidi, cntr2);
 	midi.setTimerIRQ(true, time);
 }
 
@@ -444,32 +367,32 @@ void MSXMidiCounter2::signalPosEdge(ClockPin& /*pin*/, EmuTime::param time)
 
 bool MSXMidi::ready()
 {
-	return i8251->isRecvReady();
+	return i8251.isRecvReady();
 }
 
 bool MSXMidi::acceptsData()
 {
-	return i8251->isRecvEnabled();
+	return i8251.isRecvEnabled();
 }
 
 void MSXMidi::setDataBits(DataBits bits)
 {
-	i8251->setDataBits(bits);
+	i8251.setDataBits(bits);
 }
 
 void MSXMidi::setStopBits(StopBits bits)
 {
-	i8251->setStopBits(bits);
+	i8251.setStopBits(bits);
 }
 
 void MSXMidi::setParityBit(bool enable, ParityBit parity)
 {
-	i8251->setParityBit(enable, parity);
+	i8251.setParityBit(enable, parity);
 }
 
 void MSXMidi::recvByte(byte value, EmuTime::param time)
 {
-	i8251->recvByte(value, time);
+	i8251.recvByte(value, time);
 }
 
 
@@ -479,21 +402,20 @@ void MSXMidi::serialize(Archive& ar, unsigned version)
 	ar.template serializeBase<MSXDevice>(*this);
 
 	ar.template serializeBase<MidiInConnector>(*this);
-	ar.serialize("outConnector", *outConnector);
-
-	ar.serialize("timerIRQ", timerIRQ);
-	ar.serialize("rxrdyIRQ", rxrdyIRQ);
-	ar.serialize("timerIRQlatch", timerIRQlatch);
-	ar.serialize("timerIRQenabled", timerIRQenabled);
-	ar.serialize("rxrdyIRQlatch", rxrdyIRQlatch);
-	ar.serialize("rxrdyIRQenabled", rxrdyIRQenabled);
-	ar.serialize("I8251", *i8251);
-	ar.serialize("I8254", *i8254);
+	ar.serialize("outConnector",    outConnector,
+	             "timerIRQ",        timerIRQ,
+	             "rxrdyIRQ",        rxrdyIRQ,
+	             "timerIRQlatch",   timerIRQlatch,
+	             "timerIRQenabled", timerIRQenabled,
+	             "rxrdyIRQlatch",   rxrdyIRQlatch,
+	             "rxrdyIRQenabled", rxrdyIRQenabled,
+	             "I8251",           i8251,
+	             "I8254",           i8254);
 	if (ar.versionAtLeast(version, 2)) {
 		bool newIsEnabled = isEnabled; // copy for saver
 		bool newIsLimitedTo8251 = isLimitedTo8251; // copy for saver
-		ar.serialize("isEnabled", newIsEnabled);
-		ar.serialize("isLimitedTo8251", newIsLimitedTo8251);
+		ar.serialize("isEnabled",       newIsEnabled,
+		             "isLimitedTo8251", newIsLimitedTo8251);
 		if (ar.isLoader() && isExternalMSXMIDI) {
 			registerIOports((newIsEnabled ? 0x00 : DISABLED_VALUE) | (newIsLimitedTo8251 ? LIMITED_RANGE_VALUE : 0x00));
 		}
