@@ -1,80 +1,33 @@
 #include "MSXRS232.hh"
 #include "RS232Device.hh"
 #include "CacheLine.hh"
-#include "I8254.hh"
-#include "I8251.hh"
 #include "Ram.hh"
 #include "Rom.hh"
 #include "BooleanSetting.hh"
+#include "MSXException.hh"
 #include "serialize.hh"
+#include "one_of.hh"
+#include "outer.hh"
 #include "unreachable.hh"
-#include "memory.hh"
 #include <cassert>
+#include <memory>
 
 namespace openmsx {
 
 const unsigned RAM_OFFSET = 0x2000;
 const unsigned RAM_SIZE = 0x800;
 
-
-class Counter0 final : public ClockPinListener
-{
-public:
-	explicit Counter0(MSXRS232& rs232);
-	~Counter0();
-	void signal(ClockPin& pin, EmuTime::param time) override;
-	void signalPosEdge(ClockPin& pin, EmuTime::param time) override;
-private:
-	MSXRS232& rs232;
-};
-
-class Counter1 final : public ClockPinListener
-{
-public:
-	explicit Counter1(MSXRS232& rs232);
-	~Counter1();
-	void signal(ClockPin& pin, EmuTime::param time) override;
-	void signalPosEdge(ClockPin& pin, EmuTime::param time) override;
-private:
-	MSXRS232& rs232;
-};
-
-class I8251Interf final : public I8251Interface
-{
-public:
-	explicit I8251Interf(MSXRS232& rs232);
-	~I8251Interf();
-	void setRxRDY(bool status, EmuTime::param time) override;
-	void setDTR(bool status, EmuTime::param time) override;
-	void setRTS(bool status, EmuTime::param time) override;
-	bool getDSR(EmuTime::param time) override;
-	bool getCTS(EmuTime::param time) override;
-	void setDataBits(DataBits bits) override;
-	void setStopBits(StopBits bits) override;
-	void setParityBit(bool enable, ParityBit parity) override;
-	void recvByte(byte value, EmuTime::param time) override;
-	void signal(EmuTime::param time) override;
-private:
-	MSXRS232& rs232;
-};
-
-
 MSXRS232::MSXRS232(const DeviceConfig& config)
 	: MSXDevice(config)
 	, RS232Connector(MSXDevice::getPluggingController(), "msx-rs232")
-	, cntr0(make_unique<Counter0>(*this))
-	, cntr1(make_unique<Counter1>(*this))
-	, i8254(make_unique<I8254>(
-		getScheduler(), cntr0.get(), cntr1.get(), nullptr,
-		getCurrentTime()))
-	, interf(make_unique<I8251Interf>(*this))
-	, i8251(make_unique<I8251>(getScheduler(), *interf, getCurrentTime()))
+	, i8254(getScheduler(), &cntr0, &cntr1, nullptr, getCurrentTime())
+	, i8251(getScheduler(), interf, getCurrentTime())
 	, rom(config.findChild("rom")
-		? make_unique<Rom>(
+		? std::make_unique<Rom>(
 			MSXDevice::getName() + " ROM", "rom", config)
 		: nullptr) // when the ROM is already mapped, you don't want to specify it again here
 	, ram(config.getChildDataAsBool("ram", false)
-		? make_unique<Ram>(
+		? std::make_unique<Ram>(
 			config, MSXDevice::getName() + " RAM",
 	                "RS232 RAM", RAM_SIZE)
 		: nullptr)
@@ -84,23 +37,25 @@ MSXRS232::MSXRS232(const DeviceConfig& config)
 	, hasMemoryBasedIo(config.getChildDataAsBool("memorybasedio", false))
 	, ioAccessEnabled(!hasMemoryBasedIo)
 	, switchSetting(config.getChildDataAsBool("toshiba_rs232c_switch",
-		false) ? make_unique<BooleanSetting>(getCommandController(),
+		false) ? std::make_unique<BooleanSetting>(getCommandController(),
 		"toshiba_rs232c_switch", "status of the RS-232C enable switch",
 		true) : nullptr)
 {
+	if (rom && (rom->getSize() != one_of(0x2000u, 0x4000u))) {
+		throw MSXException("RS232C only supports 8kB or 16kB ROMs.");
+	}
+
 	EmuDuration total(1.0 / 1.8432e6); // 1.8432MHz
 	EmuDuration hi   (1.0 / 3.6864e6); //   half clock period
 	EmuTime::param time = getCurrentTime();
-	i8254->getClockPin(0).setPeriodicState(total, hi, time);
-	i8254->getClockPin(1).setPeriodicState(total, hi, time);
-	i8254->getClockPin(2).setPeriodicState(total, hi, time);
+	i8254.getClockPin(0).setPeriodicState(total, hi, time);
+	i8254.getClockPin(1).setPeriodicState(total, hi, time);
+	i8254.getClockPin(2).setPeriodicState(total, hi, time);
 
 	powerUp(time);
 }
 
-MSXRS232::~MSXRS232()
-{
-}
+MSXRS232::~MSXRS232() = default;
 
 void MSXRS232::powerUp(EmuTime::param time)
 {
@@ -128,7 +83,7 @@ byte MSXRS232::readMem(word address, EmuTime::param time)
 	if (ram && ((RAM_OFFSET <= addr) && (addr < (RAM_OFFSET + RAM_SIZE)))) {
 		return (*ram)[addr - RAM_OFFSET];
 	} else if (rom && (0x4000 <= address) && (address < 0x8000)) {
-		return (*rom)[addr];
+		return (*rom)[addr & (rom->getSize() - 1)];
 	} else {
 		return 0xFF;
 	}
@@ -136,14 +91,14 @@ byte MSXRS232::readMem(word address, EmuTime::param time)
 
 const byte* MSXRS232::getReadCacheLine(word start) const
 {
-        if (hasMemoryBasedIo && (start == (0xBFF8 & CacheLine::HIGH))) {
-                return nullptr;
-        }
+	if (hasMemoryBasedIo && (start == (0xBFF8 & CacheLine::HIGH))) {
+		return nullptr;
+	}
 	word addr = start & 0x3FFF;
 	if (ram && ((RAM_OFFSET <= addr) && (addr < (RAM_OFFSET + RAM_SIZE)))) {
 		return &(*ram)[addr - RAM_OFFSET];
 	} else if (rom && (0x4000 <= start) && (start < 0x8000)) {
-		return &(*rom)[addr];
+		return &(*rom)[addr & (rom->getSize() - 1)];
 	} else {
 		return unmappedRead;
 	}
@@ -171,9 +126,9 @@ void MSXRS232::writeMem(word address, byte value, EmuTime::param time)
 
 byte* MSXRS232::getWriteCacheLine(word start) const
 {
-        if (hasMemoryBasedIo && (start == (0xBFF8 & CacheLine::HIGH))) {
-                return nullptr;
-        }
+	if (hasMemoryBasedIo && (start == (0xBFF8 & CacheLine::HIGH))) {
+		return nullptr;
+	}
 	word addr = start & 0x3FFF;
 	if (ram && ((RAM_OFFSET <= addr) && (addr < (RAM_OFFSET + RAM_SIZE)))) {
 		return &(*ram)[addr - RAM_OFFSET];
@@ -182,6 +137,12 @@ byte* MSXRS232::getWriteCacheLine(word start) const
 	}
 }
 
+bool MSXRS232::allowUnaligned() const
+{
+	// OK, because this device doesn't call any 'fillDeviceXXXCache()'functions.
+	return true;
+}
+
 byte MSXRS232::readIO(word port, EmuTime::param time)
 {
 	if (ioAccessEnabled) {
@@ -196,7 +157,7 @@ byte MSXRS232::readIOImpl(word port, EmuTime::param time)
 	switch (port) {
 		case 0: // UART data register
 		case 1: // UART status register
-			result = i8251->readIO(port, time);
+			result = i8251.readIO(port, time);
 			break;
 		case 2: // Status sense port
 			result = readStatus(time);
@@ -208,7 +169,7 @@ byte MSXRS232::readIOImpl(word port, EmuTime::param time)
 		case 5: // counter 1 data port
 		case 6: // counter 2 data port
 		case 7: // timer command register
-			result = i8254->readIO(port - 4, time);
+			result = i8254.readIO(port - 4, time);
 			break;
 		default:
 			UNREACHABLE; return 0;
@@ -224,7 +185,7 @@ byte MSXRS232::peekIO(word port, EmuTime::param time) const
 	switch (port) {
 		case 0: // UART data register
 		case 1: // UART status register
-			result = i8251->peekIO(port, time);
+			result = i8251.peekIO(port, time);
 			break;
 		case 2: // Status sense port
 			result = 0; // TODO not implemented
@@ -236,7 +197,7 @@ byte MSXRS232::peekIO(word port, EmuTime::param time) const
 		case 5: // counter 1 data port
 		case 6: // counter 2 data port
 		case 7: // timer command register
-			result = i8254->peekIO(port - 4, time);
+			result = i8254.peekIO(port - 4, time);
 			break;
 		default:
 			UNREACHABLE; return 0;
@@ -254,7 +215,7 @@ void MSXRS232::writeIOImpl(word port, byte value, EmuTime::param time)
 	switch (port) {
 		case 0: // UART data register
 		case 1: // UART command register
-			i8251->writeIO(port, value, time);
+			i8251.writeIO(port, value, time);
 			break;
 		case 2: // interrupt mask register
 			setIRQMask(value);
@@ -265,7 +226,7 @@ void MSXRS232::writeIOImpl(word port, byte value, EmuTime::param time)
 		case 5: // counter 1 data port
 		case 6: // counter 2 data port
 		case 7: // timer command register
-			i8254->writeIO(port - 4, value, time);
+			i8254.writeIO(port - 4, value, time);
 			break;
 	}
 }
@@ -296,10 +257,10 @@ byte MSXRS232::readStatus(EmuTime::param time)
 		result |= 0x08;
 	}
 
-	if (!interf->getCTS(time)) {
+	if (!interf.getCTS(time)) {
 		result |= 0x80;
 	}
-	if (i8254->getOutputPin(2).getState(time)) {
+	if (i8254.getOutputPin(2).getState(time)) {
 		result |= 0x40;
 	}
 	return result;
@@ -337,80 +298,73 @@ void MSXRS232::enableRxRDYIRQ(bool enabled)
 
 // I8251Interface  (pass calls from I8251 to outConnector)
 
-I8251Interf::I8251Interf(MSXRS232& rs232_)
-	: rs232(rs232_)
-{
-}
-
-I8251Interf::~I8251Interf()
-{
-}
-
-void I8251Interf::setRxRDY(bool status, EmuTime::param /*time*/)
+void MSXRS232::I8251Interf::setRxRDY(bool status, EmuTime::param /*time*/)
 {
+	auto& rs232 = OUTER(MSXRS232, interf);
 	rs232.setRxRDYIRQ(status);
 }
 
-void I8251Interf::setDTR(bool status, EmuTime::param time)
+void MSXRS232::I8251Interf::setDTR(bool status, EmuTime::param time)
 {
+	auto& rs232 = OUTER(MSXRS232, interf);
 	rs232.getPluggedRS232Dev().setDTR(status, time);
 }
 
-void I8251Interf::setRTS(bool status, EmuTime::param time)
+void MSXRS232::I8251Interf::setRTS(bool status, EmuTime::param time)
 {
+	auto& rs232 = OUTER(MSXRS232, interf);
 	rs232.getPluggedRS232Dev().setRTS(status, time);
 }
 
-bool I8251Interf::getDSR(EmuTime::param time)
+bool MSXRS232::I8251Interf::getDSR(EmuTime::param time)
 {
+	auto& rs232 = OUTER(MSXRS232, interf);
 	return rs232.getPluggedRS232Dev().getDSR(time);
 }
 
-bool I8251Interf::getCTS(EmuTime::param time)
+bool MSXRS232::I8251Interf::getCTS(EmuTime::param time)
 {
+	auto& rs232 = OUTER(MSXRS232, interf);
 	return rs232.getPluggedRS232Dev().getCTS(time);
 }
 
-void I8251Interf::setDataBits(DataBits bits)
+void MSXRS232::I8251Interf::setDataBits(DataBits bits)
 {
+	auto& rs232 = OUTER(MSXRS232, interf);
 	rs232.getPluggedRS232Dev().setDataBits(bits);
 }
 
-void I8251Interf::setStopBits(StopBits bits)
+void MSXRS232::I8251Interf::setStopBits(StopBits bits)
 {
+	auto& rs232 = OUTER(MSXRS232, interf);
 	rs232.getPluggedRS232Dev().setStopBits(bits);
 }
 
-void I8251Interf::setParityBit(bool enable, ParityBit parity)
+void MSXRS232::I8251Interf::setParityBit(bool enable, ParityBit parity)
 {
+	auto& rs232 = OUTER(MSXRS232, interf);
 	rs232.getPluggedRS232Dev().setParityBit(enable, parity);
 }
 
-void I8251Interf::recvByte(byte value, EmuTime::param time)
+void MSXRS232::I8251Interf::recvByte(byte value, EmuTime::param time)
 {
+	auto& rs232 = OUTER(MSXRS232, interf);
 	rs232.getPluggedRS232Dev().recvByte(value, time);
 }
 
-void I8251Interf::signal(EmuTime::param time)
+void MSXRS232::I8251Interf::signal(EmuTime::param time)
 {
+	auto& rs232 = OUTER(MSXRS232, interf);
 	rs232.getPluggedRS232Dev().signal(time); // for input
 }
 
 
 // Counter 0 output
 
-Counter0::Counter0(MSXRS232& rs232_)
-	: rs232(rs232_)
+void MSXRS232::Counter0::signal(ClockPin& pin, EmuTime::param time)
 {
-}
-
-Counter0::~Counter0()
-{
-}
-
-void Counter0::signal(ClockPin& pin, EmuTime::param time)
-{
-	ClockPin& clk = rs232.i8251->getClockPin();
+	auto& rs232 = OUTER(MSXRS232, cntr0);
+	ClockPin& clk = rs232.i8251.getClockPin();
 	if (pin.isPeriodic()) {
 		clk.setPeriodicState(pin.getTotalDuration(),
 		                     pin.getHighDuration(), time);
@@ -419,7 +373,7 @@ void Counter0::signal(ClockPin& pin, EmuTime::param time)
 	}
 }
 
-void Counter0::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
+void MSXRS232::Counter0::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
 {
 	UNREACHABLE;
 }
@@ -427,18 +381,10 @@ void Counter0::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
 
 // Counter 1 output // TODO split rx tx
 
-Counter1::Counter1(MSXRS232& rs232_)
-	: rs232(rs232_)
+void MSXRS232::Counter1::signal(ClockPin& pin, EmuTime::param time)
 {
-}
-
-Counter1::~Counter1()
-{
-}
-
-void Counter1::signal(ClockPin& pin, EmuTime::param time)
-{
-	ClockPin& clk = rs232.i8251->getClockPin();
+	auto& rs232 = OUTER(MSXRS232, cntr1);
+	ClockPin& clk = rs232.i8251.getClockPin();
 	if (pin.isPeriodic()) {
 		clk.setPeriodicState(pin.getTotalDuration(),
 		                     pin.getHighDuration(), time);
@@ -447,7 +393,7 @@ void Counter1::signal(ClockPin& pin, EmuTime::param time)
 	}
 }
 
-void Counter1::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
+void MSXRS232::Counter1::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
 {
 	UNREACHABLE;
 }
@@ -457,32 +403,32 @@ void Counter1::signalPosEdge(ClockPin& /*pin*/, EmuTime::param /*time*/)
 
 bool MSXRS232::ready()
 {
-	return i8251->isRecvReady();
+	return i8251.isRecvReady();
 }
 
 bool MSXRS232::acceptsData()
 {
-	return i8251->isRecvEnabled();
+	return i8251.isRecvEnabled();
 }
 
 void MSXRS232::setDataBits(DataBits bits)
 {
-	i8251->setDataBits(bits);
+	i8251.setDataBits(bits);
 }
 
 void MSXRS232::setStopBits(StopBits bits)
 {
-	i8251->setStopBits(bits);
+	i8251.setStopBits(bits);
 }
 
 void MSXRS232::setParityBit(bool enable, ParityBit parity)
 {
-	i8251->setParityBit(enable, parity);
+	i8251.setParityBit(enable, parity);
 }
 
 void MSXRS232::recvByte(byte value, EmuTime::param time)
 {
-	i8251->recvByte(value, time);
+	i8251.recvByte(value, time);
 }
 
 // version 1: initial version
@@ -494,12 +440,12 @@ void MSXRS232::serialize(Archive& ar, unsigned version)
 	ar.template serializeBase<MSXDevice>(*this);
 	ar.template serializeBase<RS232Connector>(*this);
 
-	ar.serialize("I8254", *i8254);
-	ar.serialize("I8251", *i8251);
+	ar.serialize("I8254", i8254,
+	             "I8251", i8251);
 	if (ram) ar.serialize("ram", *ram);
-	ar.serialize("rxrdyIRQ", rxrdyIRQ);
-	ar.serialize("rxrdyIRQlatch", rxrdyIRQlatch);
-	ar.serialize("rxrdyIRQenabled", rxrdyIRQenabled);
+	ar.serialize("rxrdyIRQ",        rxrdyIRQ,
+	             "rxrdyIRQlatch",   rxrdyIRQlatch,
+	             "rxrdyIRQenabled", rxrdyIRQenabled);
 	if (ar.versionAtLeast(version, 2)) {
 		ar.serialize("ioAccessEnabled", ioAccessEnabled);
 	} else {
