@@ -3,11 +3,8 @@
 #include "PlugException.hh"
 #include "EventDistributor.hh"
 #include "Scheduler.hh"
-#include "FilenameSetting.hh"
 #include "FileOperations.hh"
-#include "StringOp.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include <cstdio>
 #include <cerrno>
 #include <cstring>
@@ -20,11 +17,10 @@ MidiInReader::MidiInReader(EventDistributor& eventDistributor_,
                            Scheduler& scheduler_,
                            CommandController& commandController)
 	: eventDistributor(eventDistributor_), scheduler(scheduler_)
-	, thread(this), lock(1)
-	, readFilenameSetting(make_unique<FilenameSetting>(
+	, readFilenameSetting(
 		commandController, "midi-in-readfilename",
 		"filename of the file where the MIDI input is read from",
-		"/dev/midi"))
+		"/dev/midi")
 {
 	eventDistributor.registerEventListener(OPENMSX_MIDI_IN_READER_EVENT, *this);
 }
@@ -37,10 +33,9 @@ MidiInReader::~MidiInReader()
 // Pluggable
 void MidiInReader::plugHelper(Connector& connector_, EmuTime::param /*time*/)
 {
-	file = FileOperations::openFile(readFilenameSetting->getString(), "rb");
+	file = FileOperations::openFile(string(readFilenameSetting.getString()), "rb");
 	if (!file) {
-		throw PlugException(StringOp::Builder()
-			<< "Failed to open input: " << strerror(errno));
+		throw PlugException("Failed to open input: ", strerror(errno));
 	}
 
 	auto& midiConnector = static_cast<MidiInConnector&>(connector_);
@@ -50,13 +45,13 @@ void MidiInReader::plugHelper(Connector& connector_, EmuTime::param /*time*/)
 
 	setConnector(&connector_); // base class will do this in a moment,
 	                           // but thread already needs it
-	thread.start();
+	thread = std::thread([this]() { run(); });
 }
 
 void MidiInReader::unplugHelper(EmuTime::param /*time*/)
 {
-	ScopedLock l(lock);
-	thread.stop();
+	poller.abort();
+	thread.join();
 	file.reset();
 }
 
@@ -66,7 +61,7 @@ const string& MidiInReader::getName() const
 	return name;
 }
 
-string_ref MidiInReader::getDescription() const
+std::string_view MidiInReader::getDescription() const
 {
 	return "MIDI in file reader. Sends data from an input file to the "
 	       "MIDI port it is connected to. The filename is set with "
@@ -74,20 +69,27 @@ string_ref MidiInReader::getDescription() const
 }
 
 
-// Runnable
 void MidiInReader::run()
 {
 	byte buf;
 	if (!file) return;
 	while (true) {
+#ifndef _WIN32
+		if (poller.poll(fileno(file.get()))) {
+			break;
+		}
+#endif
 		size_t num = fread(&buf, 1, 1, file.get());
+		if (poller.aborted()) {
+			break;
+		}
 		if (num != 1) {
 			continue;
 		}
 		assert(isPluggedIn());
 
 		{
-			ScopedLock l(lock);
+			std::lock_guard<std::mutex> lock(mutex);
 			queue.push_back(buf);
 		}
 		eventDistributor.distributeEvent(
@@ -98,23 +100,23 @@ void MidiInReader::run()
 // MidiInDevice
 void MidiInReader::signal(EmuTime::param time)
 {
-	auto connector = static_cast<MidiInConnector*>(getConnector());
-	if (!connector->acceptsData()) {
-		ScopedLock l(lock);
+	auto* conn = static_cast<MidiInConnector*>(getConnector());
+	if (!conn->acceptsData()) {
+		std::lock_guard<std::mutex> lock(mutex);
 		queue.clear();
 		return;
 	}
-	if (!connector->ready()) {
+	if (!conn->ready()) {
 		return;
 	}
 
 	byte data;
 	{
-		ScopedLock l(lock);
+		std::lock_guard<std::mutex> lock(mutex);
 		if (queue.empty()) return;
 		data = queue.pop_front();
 	}
-	connector->recvByte(data, time);
+	conn->recvByte(data, time);
 }
 
 // EventListener
@@ -123,7 +125,7 @@ int MidiInReader::signalEvent(const std::shared_ptr<const Event>& /*event*/)
 	if (isPluggedIn()) {
 		signal(scheduler.getCurrentTime());
 	} else {
-		ScopedLock l(lock);
+		std::lock_guard<std::mutex> lock(mutex);
 		queue.clear();
 	}
 	return 0;
