@@ -1,5 +1,3 @@
-// $Id: MidiOutWindows.cc 12633 2012-06-14 20:41:30Z m9710797 $
-
 #if defined(_WIN32)
 
 #include "Midi_w32.hh"
@@ -7,6 +5,7 @@
 #include "PluggingController.hh"
 #include "PlugException.hh"
 #include "serialize.hh"
+#include <memory>
 
 using std::string;
 
@@ -17,8 +16,7 @@ void MidiOutWindows::registerAll(PluggingController& controller)
 	w32_midiOutInit();
 	unsigned devnum = w32_midiOutGetVFNsNum();
 	for (unsigned i = 0; i < devnum; ++i) {
-		controller.registerPluggable(std::auto_ptr<Pluggable>(
-			new MidiOutWindows(i)));
+		controller.registerPluggable(std::make_unique<MidiOutWindows>(i));
 	}
 }
 
@@ -56,15 +54,15 @@ const string& MidiOutWindows::getName() const
 	return name;
 }
 
-string_ref MidiOutWindows::getDescription() const
+std::string_view MidiOutWindows::getDescription() const
 {
 	return desc;
 }
 
-void MidiOutWindows::recvByte(byte value, EmuTime::param /*time*/)
+void MidiOutWindows::recvMessage(const std::vector<uint8_t>& message, EmuTime::param /*time*/)
 {
 	if (devidx != unsigned(-1)) {
-		w32_midiOutPut(value, devidx);
+		w32_midiOutMsg(message.size(), message.data(), devidx);
 	}
 }
 
