@@ -1,30 +1,25 @@
-// $Id: RS232Connector.cc 12630 2012-06-14 20:17:01Z m9710797 $
-
 #include "RS232Connector.hh"
 #include "RS232Device.hh"
 #include "DummyRS232Device.hh"
 #include "checked_cast.hh"
 #include "serialize.hh"
+#include <memory>
 
 namespace openmsx {
 
-RS232Connector::RS232Connector(PluggingController& pluggingController,
-                               string_ref name)
-	: Connector(pluggingController, name,
-	            std::auto_ptr<Pluggable>(new DummyRS232Device()))
+RS232Connector::RS232Connector(PluggingController& pluggingController_,
+                               std::string name_)
+	: Connector(pluggingController_, std::move(name_),
+	            std::make_unique<DummyRS232Device>())
 {
 }
 
-RS232Connector::~RS232Connector()
-{
-}
-
-const std::string RS232Connector::getDescription() const
+std::string_view RS232Connector::getDescription() const
 {
 	return "Serial RS232 connector";
 }
 
-string_ref RS232Connector::getClass() const
+std::string_view RS232Connector::getClass() const
 {
 	return "RS232";
 }
