@@ -3,10 +3,8 @@
 #include "PlugException.hh"
 #include "EventDistributor.hh"
 #include "Scheduler.hh"
-#include "FilenameSetting.hh"
 #include "FileOperations.hh"
 #include "serialize.hh"
-#include "memory.hh"
 
 namespace openmsx {
 
@@ -14,15 +12,14 @@ RS232Tester::RS232Tester(EventDistributor& eventDistributor_,
                          Scheduler& scheduler_,
                          CommandController& commandController)
 	: eventDistributor(eventDistributor_), scheduler(scheduler_)
-	, thread(this), lock(1)
-	, rs232InputFilenameSetting(make_unique<FilenameSetting>(
+	, rs232InputFilenameSetting(
 	        commandController, "rs232-inputfilename",
 	        "filename of the file where the RS232 input is read from",
-	        "rs232-input"))
-	, rs232OutputFilenameSetting(make_unique<FilenameSetting>(
+	        "rs232-input")
+	, rs232OutputFilenameSetting(
 	        commandController, "rs232-outputfilename",
 	        "filename of the file where the RS232 output is written to",
-	        "rs232-output"))
+	        "rs232-output")
 {
 	eventDistributor.registerEventListener(OPENMSX_RS232_TESTER_EVENT, *this);
 }
@@ -36,19 +33,19 @@ RS232Tester::~RS232Tester()
 void RS232Tester::plugHelper(Connector& connector_, EmuTime::param /*time*/)
 {
 	// output
-	std::string outName = rs232OutputFilenameSetting->getString();
-	FileOperations::openofstream(outFile, outName);
+	std::string_view outName = rs232OutputFilenameSetting.getString();
+	FileOperations::openofstream(outFile, std::string(outName));
 	if (outFile.fail()) {
 		outFile.clear();
-		throw PlugException("Error opening output file: " + outName);
+		throw PlugException("Error opening output file: ", outName);
 	}
 
 	// input
-	std::string inName = rs232InputFilenameSetting->getString();
-	inFile = FileOperations::openFile(inName, "rb");
+	std::string_view inName = rs232InputFilenameSetting.getString();
+	inFile = FileOperations::openFile(std::string(inName), "rb");
 	if (!inFile) {
 		outFile.close();
-		throw PlugException("Error opening input file: " + inName);
+		throw PlugException("Error opening input file: ", inName);
 	}
 
 	auto& rs232Connector = static_cast<RS232Connector&>(connector_);
@@ -58,7 +55,7 @@ void RS232Tester::plugHelper(Connector& connector_, EmuTime::param /*time*/)
 
 	setConnector(&connector_); // base class will do this in a moment,
 	                           // but thread already needs it
-	thread.start();
+	thread = std::thread([this]() { run(); });
 }
 
 void RS232Tester::unplugHelper(EmuTime::param /*time*/)
@@ -67,8 +64,8 @@ void RS232Tester::unplugHelper(EmuTime::param /*time*/)
 	outFile.close();
 
 	// input
-	ScopedLock l(lock);
-	thread.stop();
+	poller.abort();
+	thread.join();
 	inFile.reset();
 }
 
@@ -78,7 +75,7 @@ const std::string& RS232Tester::getName() const
 	return name;
 }
 
-string_ref RS232Tester::getDescription() const
+std::string_view RS232Tester::getDescription() const
 {
 	return	"RS232 tester pluggable. Reads all data from file specified "
 		"with the 'rs-232-inputfilename' setting. Writes all data "
@@ -86,18 +83,25 @@ string_ref RS232Tester::getDescription() const
 		"setting.";
 }
 
-// Runnable
 void RS232Tester::run()
 {
 	byte buf;
 	if (!inFile) return;
 	while (!feof(inFile.get())) {
+#ifndef _WIN32
+		if (poller.poll(fileno(inFile.get()))) {
+			break;
+		}
+#endif
 		size_t num = fread(&buf, 1, 1, inFile.get());
+		if (poller.aborted()) {
+			break;
+		}
 		if (num != 1) {
 			continue;
 		}
 		assert(isPluggedIn());
-		ScopedLock l(lock);
+		std::lock_guard<std::mutex> lock(mutex);
 		queue.push_back(buf);
 		eventDistributor.distributeEvent(
 			std::make_shared<SimpleEvent>(OPENMSX_RS232_TESTER_EVENT));
@@ -107,17 +111,17 @@ void RS232Tester::run()
 // input
 void RS232Tester::signal(EmuTime::param time)
 {
-	auto connector = static_cast<RS232Connector*>(getConnector());
-	if (!connector->acceptsData()) {
-		ScopedLock l(lock);
+	auto* conn = static_cast<RS232Connector*>(getConnector());
+	if (!conn->acceptsData()) {
+		std::lock_guard<std::mutex> lock(mutex);
 		queue.clear();
 		return;
 	}
-	if (!connector->ready()) return;
+	if (!conn->ready()) return;
 
-	ScopedLock l(lock);
+	std::lock_guard<std::mutex> lock(mutex);
 	if (queue.empty()) return;
-	connector->recvByte(queue.pop_front(), time);
+	conn->recvByte(queue.pop_front(), time);
 }
 
 // EventListener
@@ -126,7 +130,7 @@ int RS232Tester::signalEvent(const std::shared_ptr<const Event>& /*event*/)
 	if (isPluggedIn()) {
 		signal(scheduler.getCurrentTime());
 	} else {
-		ScopedLock l(lock);
+		std::lock_guard<std::mutex> lock(mutex);
 		queue.clear();
 	}
 	return 0;
