@@ -10,20 +10,27 @@
 #include "checked_cast.hh"
 
 using std::string;
+using std::string_view;
 
 namespace openmsx {
 
 // class BaseSetting
 
-BaseSetting::BaseSetting(string_ref name_)
-	: name(name_.str())
+BaseSetting::BaseSetting(string_view name)
+	: fullName(name)
+	, baseName(fullName)
+{
+}
+
+BaseSetting::BaseSetting(const TclObject& name)
+	: fullName(name)
+	, baseName(fullName)
 {
 }
 
 void BaseSetting::info(TclObject& result) const
 {
-	result.addListElement(getTypeString());
-	result.addListElement(getDefaultValue());
+	result.addListElement(getTypeString(), getDefaultValue());
 	additionalInfo(result);
 }
 
@@ -31,11 +38,11 @@ void BaseSetting::info(TclObject& result) const
 // class Setting
 
 Setting::Setting(CommandController& commandController_,
-                 string_ref name_, string_ref desc_,
-                 const string& initialValue, SaveSetting save_)
-	: BaseSetting(name_)
+                 string_view name, string_view description_,
+                 const TclObject& initialValue, SaveSetting save_)
+	: BaseSetting(name)
 	, commandController(commandController_)
-	, description(desc_.str())
+	, description(description_)
 	, value(initialValue)
 	, defaultValue(initialValue)
 	, restoreValue(initialValue)
@@ -51,9 +58,9 @@ void Setting::init()
 			.getSettingsConfig().getXMLElement();
 		if (auto* config = settingsConfig.findChild("settings")) {
 			if (auto* elem = config->findChildWithAttribute(
-			                                "setting", "id", getName())) {
+			                                "setting", "id", getBaseName())) {
 				try {
-					setStringDirect(elem->getData());
+					setValueDirect(TclObject(elem->getData()));
 				} catch (MSXException&) {
 					// saved value no longer valid, just keep default
 				}
@@ -74,14 +81,14 @@ Setting::~Setting()
 }
 
 
-string Setting::getDescription() const
+string_view Setting::getDescription() const
 {
 	return description;
 }
 
-void Setting::setString(const string& value)
+void Setting::setValue(const TclObject& newValue)
 {
-	getCommandController().changeSetting(*this, value);
+	getInterpreter().setVariable(getFullNameObj(), newValue);
 }
 
 void Setting::notify() const
@@ -95,27 +102,27 @@ void Setting::notify() const
 	//  - SettingsConfig (keeps values, also of not yet created settings)
 	// This method takes care of the last 3 in this list.
 	Subject<Setting>::notify();
+	TclObject val = getValue();
 	commandController.getCliComm().update(
-		CliComm::SETTING, getName(), getString());
+		CliComm::SETTING, getBaseName(), val.getString());
 
 	// Always keep SettingsConfig in sync.
 	auto& config = getGlobalCommandController().getSettingsConfig().getXMLElement();
 	auto& settings = config.getCreateChild("settings");
-	string value = getString();
-	if (!needLoadSave() || (value == getDefaultValue())) {
+	if (!needLoadSave() || (val == getDefaultValue())) {
 		// remove setting
 		if (auto* elem = settings.findChildWithAttribute(
-				"setting", "id", getName())) {
+				"setting", "id", getBaseName())) {
 			settings.removeChild(*elem);
 		}
 	} else {
 		// add (or overwrite) setting
 		auto& elem = settings.getCreateChildWithAttribute(
-				"setting", "id", getName());
+				"setting", "id", getBaseName());
 		// check for non-saveable value
 		// (mechanism can be generalize later when needed)
-		if (value == dontSaveValue) value = getRestoreValue();
-		elem.setData(value);
+		if (val == dontSaveValue) val = getRestoreValue();
+		elem.setData(string(val.getString()));
 	}
 }
 
@@ -124,7 +131,7 @@ void Setting::notifyPropertyChange() const
 	TclObject result;
 	info(result);
 	commandController.getCliComm().update(
-		CliComm::SETTINGINFO, getName(), result.getString());
+		CliComm::SETTINGINFO, getBaseName(), result.getString());
 }
 
 bool Setting::needLoadSave() const
@@ -136,7 +143,7 @@ bool Setting::needTransfer() const
 	return save != DONT_TRANSFER;
 }
 
-void Setting::setDontSaveValue(const std::string& dontSaveValue_)
+void Setting::setDontSaveValue(const TclObject& dontSaveValue_)
 {
 	dontSaveValue = dontSaveValue_;
 }
@@ -167,30 +174,10 @@ void Setting::additionalInfo(TclObject& /*result*/) const
 	// nothing
 }
 
-void Setting::setRestoreValue(const string& value)
-{
-	restoreValue = value;
-}
 
-
-string Setting::getString() const
+void Setting::setValueDirect(const TclObject& newValue_)
 {
-	return value.getString().str();
-}
-
-string Setting::getDefaultValue() const
-{
-	return defaultValue;
-}
-
-string Setting::getRestoreValue() const
-{
-	return restoreValue;
-}
-
-void Setting::setStringDirect(const string& str)
-{
-	TclObject newValue(str);
+	TclObject newValue = newValue_;
 	checkFunc(newValue);
 	if (newValue != value) {
 		value = newValue;
@@ -209,10 +196,9 @@ void Setting::setStringDirect(const string& str)
 		return;
 	}
 
-	auto& globalController = controller->getGlobalCommandController();
 	// Tcl already makes sure this doesn't result in an endless loop.
 	try {
-		globalController.changeSetting(getName(), getString());
+		getInterpreter().setVariable(getBaseNameObj(), getValue());
 	} catch (MSXException&) {
 		// ignore
 	}
