@@ -1,10 +1,9 @@
 #include "MSXAudio.hh"
-#include "Y8950.hh"
 #include "Y8950Periphery.hh"
 #include "DACSound8U.hh"
 #include "StringOp.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include <memory>
 
 using std::string;
 
@@ -14,19 +13,18 @@ namespace openmsx {
 
 MSXAudio::MSXAudio(const DeviceConfig& config)
 	: MSXDevice(config)
+	, y8950(getName(), config,
+	        config.getChildDataAsInt("sampleram", 256) * 1024,
+	        getCurrentTime(), *this)
 	, dacValue(0x80), dacEnabled(false)
 {
 	string type(StringOp::toLower(config.getChildData("type", "philips")));
 	if (type == "philips") {
-		dac = make_unique<DACSound8U>(
+		dac = std::make_unique<DACSound8U>(
 			getName() + " 8-bit DAC", "MSX-AUDIO 8-bit DAC",
 			config);
 	}
-	int ramSize = config.getChildDataAsInt("sampleram", 256); // size in kb
-	EmuTime::param time = getCurrentTime();
-	y8950 = make_unique<Y8950>(
-		getName(), config, ramSize * 1024, time, *this);
-	powerUp(time);
+	powerUp(getCurrentTime());
 }
 
 MSXAudio::~MSXAudio()
@@ -45,13 +43,13 @@ Y8950Periphery& MSXAudio::createPeriphery(const string& soundDeviceName)
 
 void MSXAudio::powerUp(EmuTime::param time)
 {
-	y8950->clearRam();
+	y8950.clearRam();
 	reset(time);
 }
 
 void MSXAudio::reset(EmuTime::param time)
 {
-	y8950->reset(time);
+	y8950.reset(time);
 	periphery->reset();
 	registerLatch = 0; // TODO check
 }
@@ -59,30 +57,30 @@ void MSXAudio::reset(EmuTime::param time)
 byte MSXAudio::readIO(word port, EmuTime::param time)
 {
 	byte result;
-	if ((port & 0xFF) == 0x0A) {
+	if ((port & 0xE8) == 0x08) {
 		// read DAC
-		result = 255;
+		result = 0xFF;
 	} else {
-		result = (port & 1) ? y8950->readReg(registerLatch, time)
-		                    : y8950->readStatus(time);
+		result = (port & 1) ? y8950.readReg(registerLatch, time)
+		                    : y8950.readStatus(time);
 	}
 	return result;
 }
 
 byte MSXAudio::peekIO(word port, EmuTime::param time) const
 {
-	if ((port & 0xFF) == 0x0A) {
+	if ((port & 0xE8) == 0x08) {
 		// read DAC
-		return 255; // read always returns 255
+		return 0xFF; // read always returns 0xFF
 	} else {
-		return (port & 1) ? y8950->peekReg(registerLatch, time)
-		                  : y8950->peekStatus(time);
+		return (port & 1) ? y8950.peekReg(registerLatch, time)
+		                  : y8950.peekStatus(time);
 	}
 }
 
 void MSXAudio::writeIO(word port, byte value, EmuTime::param time)
 {
-	if ((port & 0xFF) == 0x0A) {
+	if ((port & 0xE8) == 0x08) {
 		dacValue = value;
 		if (dacEnabled) {
 			assert(dac);
@@ -93,7 +91,7 @@ void MSXAudio::writeIO(word port, byte value, EmuTime::param time)
 		registerLatch = value;
 	} else {
 		// 0xC1 or 0xC3
-		y8950->writeReg(registerLatch, value, time);
+		y8950.writeReg(registerLatch, value, time);
 	}
 }
 
@@ -131,11 +129,10 @@ template<typename Archive>
 void MSXAudio::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.serializePolymorphic("periphery", *periphery);
-	ar.serialize("Y8950", *y8950);
-
-	ar.serialize("registerLatch", registerLatch);
-	ar.serialize("dacValue", dacValue);
-	ar.serialize("dacEnabled", dacEnabled);
+	ar.serialize("Y8950",         y8950,
+	             "registerLatch", registerLatch,
+	             "dacValue",      dacValue,
+	             "dacEnabled",    dacEnabled);
 
 	if (ar.isLoader()) {
 		// restore dac status
