@@ -1,32 +1,22 @@
 #include "MSXFmPac.hh"
-#include "RomBlockDebuggable.hh"
-#include "SRAM.hh"
-#include "Rom.hh"
 #include "CacheLine.hh"
 #include "serialize.hh"
-#include "memory.hh"
 
 namespace openmsx {
 
-static const char* const PAC_Header = "PAC2 BACKUP DATA";
+constexpr const char* const PAC_Header = "PAC2 BACKUP DATA";
 
 MSXFmPac::MSXFmPac(const DeviceConfig& config)
-	: MSXMusic(config)
-	, sram(make_unique<SRAM>(
-		getName() + " SRAM", 0x1FFE, config, PAC_Header))
-	, romBlockDebug(make_unique<RomBlockDebuggable>(
-		*this, &bank, 0x4000, 0x4000, 14))
+	: MSXMusicBase(config)
+	, sram(getName() + " SRAM", 0x1FFE, config, PAC_Header)
+	, romBlockDebug(*this, &bank, 0x4000, 0x4000, 14)
 {
 	reset(getCurrentTime());
 }
 
-MSXFmPac::~MSXFmPac()
-{
-}
-
 void MSXFmPac::reset(EmuTime::param time)
 {
-	MSXMusic::reset(time);
+	MSXMusicBase::reset(time);
 	enable = 0;
 	sramEnabled = false;
 	bank = 0;
@@ -37,7 +27,7 @@ void MSXFmPac::reset(EmuTime::param time)
 void MSXFmPac::writeIO(word port, byte value, EmuTime::param time)
 {
 	if (enable & 1) {
-		MSXMusic::writeIO(port, value, time);
+		MSXMusicBase::writeIO(port, value, time);
 	}
 }
 
@@ -52,7 +42,7 @@ byte MSXFmPac::readMem(word address, EmuTime::param /*time*/)
 	default:
 		if (sramEnabled) {
 			if (address < 0x1FFE) {
-				return (*sram)[address];
+				return sram[address];
 			} else if (address == 0x1FFE) {
 				return r1ffe; // always 0x4D
 			} else if (address == 0x1FFF) {
@@ -61,7 +51,7 @@ byte MSXFmPac::readMem(word address, EmuTime::param /*time*/)
 				return 0xFF;
 			}
 		} else {
-			return (*rom)[bank * 0x4000 + address];
+			return rom[bank * 0x4000 + address];
 		}
 	}
 }
@@ -74,14 +64,14 @@ const byte* MSXFmPac::getReadCacheLine(word address) const
 	}
 	if (sramEnabled) {
 		if (address < (0x1FFE & CacheLine::HIGH)) {
-			return &(*sram)[address];
+			return &sram[address];
 		} else if (address == (0x1FFE & CacheLine::HIGH)) {
 			return nullptr;
 		} else {
 			return unmappedRead;
 		}
 	} else {
-		return &(*rom)[bank * 0x4000 + address];
+		return &rom[bank * 0x4000 + address];
 	}
 }
 
@@ -103,11 +93,9 @@ void MSXFmPac::writeMem(word address, byte value, EmuTime::param time)
 			checkSramEnable();
 		}
 		break;
-	case 0x3FF4:
-		writeRegisterPort(value, time);
-		break;
-	case 0x3FF5:
-		writeDataPort(value, time);
+	case 0x3FF4: // address
+	case 0x3FF5: // data
+		writePort(address & 1, value, time);
 		break;
 	case 0x3FF6:
 		enable = value & 0x11;
@@ -120,13 +108,13 @@ void MSXFmPac::writeMem(word address, byte value, EmuTime::param time)
 		byte newBank = value & 0x03;
 		if (bank != newBank) {
 			bank = newBank;
-			invalidateMemCache(0x0000, 0x10000);
+			invalidateDeviceRCache();
 		}
 		break;
 	}
 	default:
 		if (sramEnabled && (address < 0x1FFE)) {
-			sram->write(address, value);
+			sram.write(address, value);
 		}
 	}
 }
@@ -152,7 +140,7 @@ void MSXFmPac::checkSramEnable()
 	bool newEnabled = (r1ffe == 0x4D) && (r1fff == 0x69);
 	if (sramEnabled != newEnabled) {
 		sramEnabled = newEnabled;
-		invalidateMemCache(0x0000, 0x10000);
+		invalidateDeviceRWCache();
 	}
 }
 
@@ -160,12 +148,12 @@ void MSXFmPac::checkSramEnable()
 template<typename Archive>
 void MSXFmPac::serialize(Archive& ar, unsigned version)
 {
-	ar.template serializeInlinedBase<MSXMusic>(*this, version);
-	ar.serialize("sram", *sram);
-	ar.serialize("enable", enable);
-	ar.serialize("bank", bank);
-	ar.serialize("r1ffe", r1ffe);
-	ar.serialize("r1fff", r1fff);
+	ar.template serializeInlinedBase<MSXMusicBase>(*this, version);
+	ar.serialize("sram",   sram,
+	             "enable", enable,
+	             "bank",   bank,
+	             "r1ffe",  r1ffe,
+	             "r1fff",  r1fff);
 	if (ar.isLoader()) {
 		// sramEnabled can be calculated
 		checkSramEnable();
