@@ -8,78 +8,69 @@
 //      low priority.
 
 #include "MSXMoonSound.hh"
-#include "YMF262.hh"
-#include "YMF278.hh"
 #include "Clock.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
-#include "memory.hh"
 
 namespace openmsx {
 
 // The master clock, running at 33.8MHz.
-typedef Clock<33868800> MasterClock;
+using MasterClock = Clock<33868800>;
 
 // Required delay between register select and register read/write.
-static const EmuDuration FM_REG_SELECT_DELAY = MasterClock::duration(56);
+constexpr auto FM_REG_SELECT_DELAY = MasterClock::duration(56);
 // Required delay after register write.
-static const EmuDuration FM_REG_WRITE_DELAY  = MasterClock::duration(56);
+constexpr auto FM_REG_WRITE_DELAY  = MasterClock::duration(56);
 // Datasheet doesn't mention any delay for reads from the FM registers. In fact
 // it says reads from FM registers are not possible while tests on a real
 // YMF278 show they do work (value of the NEW2 bit doesn't matter).
 
 // Required delay between register select and register read/write.
-static const EmuDuration WAVE_REG_SELECT_DELAY = MasterClock::duration(88);
+constexpr auto WAVE_REG_SELECT_DELAY = MasterClock::duration(88);
 // Required delay after register write.
-static const EmuDuration WAVE_REG_WRITE_DELAY  = MasterClock::duration(88);
+constexpr auto WAVE_REG_WRITE_DELAY  = MasterClock::duration(88);
 // Datasheet doesn't mention any delay for register reads (except for reads
 // from register 6, see below). I also couldn't measure any delay on a real
 // YMF278.
 
 // Required delay after memory read.
-static const EmuDuration MEM_READ_DELAY  = MasterClock::duration(38);
+constexpr auto MEM_READ_DELAY  = MasterClock::duration(38);
 // Required delay after memory write (instead of register write delay).
-static const EmuDuration MEM_WRITE_DELAY = MasterClock::duration(28);
+constexpr auto MEM_WRITE_DELAY = MasterClock::duration(28);
 
 // Required delay after instrument load.
 // We pick 10000 cycles, this is approximately 300us (the number given in the
 // datasheet). The exact number of cycles is unknown. But I did some (very
 // rough) tests on real HW, and this number is not too bad (slightly too high
 // but within 2%-4% of real value, needs more detailed tests).
-static const EmuDuration LOAD_DELAY = MasterClock::duration(10000);
+constexpr auto LOAD_DELAY = MasterClock::duration(10000);
 
 
 MSXMoonSound::MSXMoonSound(const DeviceConfig& config)
 	: MSXDevice(config)
-	, ymf262(make_unique<YMF262>(getName() + " FM", config, true))
-	, ymf278(make_unique<YMF278>(
-		getName() + " wave",
-		config.getChildDataAsInt("sampleram", 512), // size in kb
-		config))
+	, ymf262(getName() + " FM", config, true)
+	, ymf278(getName() + " wave",
+	         config.getChildDataAsInt("sampleram", 512), // size in kb
+	         config)
 	, ymf278LoadTime(getCurrentTime())
 	, ymf278BusyTime(getCurrentTime())
 {
 	powerUp(getCurrentTime());
 }
 
-MSXMoonSound::~MSXMoonSound()
-{
-}
-
 void MSXMoonSound::powerUp(EmuTime::param time)
 {
-	ymf278->clearRam();
+	ymf278.clearRam();
 	reset(time);
 }
 
 void MSXMoonSound::reset(EmuTime::param time)
 {
-	ymf262->reset(time);
-	ymf278->reset(time);
+	ymf262.reset(time);
+	ymf278.reset(time);
 
 	opl4latch = 0; // TODO check
 	opl3latch = 0; // TODO check
-	alreadyReadID = false;
 
 	ymf278BusyTime = time;
 	ymf278LoadTime = time;
@@ -110,7 +101,7 @@ byte MSXMoonSound::readIO(word port, EmuTime::param time)
 				// doesn't have any measurable effect on MSX.
 				ymf278BusyTime = time + MEM_READ_DELAY;
 			}
-			result = ymf278->readReg(opl4latch);
+			result = ymf278.readReg(opl4latch);
 			break;
 		default: // unreachable, avoid warning
 			UNREACHABLE; result = 255;
@@ -120,24 +111,11 @@ byte MSXMoonSound::readIO(word port, EmuTime::param time)
 		switch (port & 0x03) {
 		case 0: // read status
 		case 2:
-			result = ymf262->readStatus() |
-			         readYMF278Status(time);
-			if (!alreadyReadID && getNew2()) {
-				// Verified on real YMF278:
-				// Only once after switching NEW2=1, reading
-				// the status register returns '0x02'. This
-				// behavior doesn't re-occur till after a
-				// reset (datasheet confirms this behavior).
-				// Also verified that only bit 1 changes (so
-				// it's not the whole value that is forced to
-				// 0x02, datasheet isn't clear about that).
-				alreadyReadID = true;
-				result |= 0x02;
-			}
+			result = ymf262.readStatus() | readYMF278Status(time);
 			break;
 		case 1:
 		case 3: // read fm register
-			result = ymf262->readReg(opl3latch);
+			result = ymf262.readReg(opl3latch);
 			break;
 		default: // unreachable, avoid warning
 			UNREACHABLE; result = 255;
@@ -156,7 +134,7 @@ byte MSXMoonSound::peekIO(word port, EmuTime::param time) const
 			result = 255;
 			break;
 		case 1: // read wave register
-			result = ymf278->peekReg(opl4latch);
+			result = ymf278.peekReg(opl4latch);
 			break;
 		default: // unreachable, avoid warning
 			UNREACHABLE; result = 255;
@@ -166,15 +144,11 @@ byte MSXMoonSound::peekIO(word port, EmuTime::param time) const
 		switch (port & 0x03) {
 		case 0: // read status
 		case 2:
-			result = ymf262->peekStatus() |
-			         readYMF278Status(time);
-			if (!alreadyReadID && getNew2()) {
-				result |= 0x02;
-			}
+			result = ymf262.peekStatus() | readYMF278Status(time);
 			break;
 		case 1:
 		case 3: // read fm register
-			result = ymf262->peekReg(opl3latch);
+			result = ymf262.peekReg(opl3latch);
 			break;
 		default: // unreachable, avoid warning
 			UNREACHABLE; result = 255;
@@ -209,7 +183,12 @@ void MSXMoonSound::writeIO(word port, byte value, EmuTime::param time)
 					// very briefly and only on R800.
 					ymf278BusyTime = time + WAVE_REG_WRITE_DELAY;
 				}
-				ymf278->writeReg(opl4latch, value, time);
+				if (opl4latch == 0xf8) {
+					ymf262.setMixLevel(value, time);
+				} else if (opl4latch == 0xf9) {
+					ymf278.setMixLevel(value, time);
+				}
+				ymf278.writeReg(opl4latch, value, time);
 				break;
 			default:
 				UNREACHABLE;
@@ -233,7 +212,7 @@ void MSXMoonSound::writeIO(word port, byte value, EmuTime::param time)
 		case 1:
 		case 3: // write fm register
 			ymf278BusyTime = time + FM_REG_WRITE_DELAY;
-			ymf262->writeReg(opl3latch, value, time);
+			ymf262.writeReg(opl3latch, value, time);
 			break;
 		default:
 			UNREACHABLE;
@@ -243,7 +222,7 @@ void MSXMoonSound::writeIO(word port, byte value, EmuTime::param time)
 
 bool MSXMoonSound::getNew2() const
 {
-	return (ymf262->peekReg(0x105) & 0x02) != 0;
+	return (ymf262.peekReg(0x105) & 0x02) != 0;
 }
 
 byte MSXMoonSound::readYMF278Status(EmuTime::param time) const
@@ -257,24 +236,18 @@ byte MSXMoonSound::readYMF278Status(EmuTime::param time) const
 // version 1: initial version
 // version 2: added alreadyReadID
 // version 3: moved loadTime and busyTime from YMF278 to here
+//            removed alreadyReadID
 template<typename Archive>
 void MSXMoonSound::serialize(Archive& ar, unsigned version)
 {
 	ar.template serializeBase<MSXDevice>(*this);
-	ar.serialize("ymf262", *ymf262);
-	ar.serialize("ymf278", *ymf278);
-	ar.serialize("opl3latch", opl3latch);
-	ar.serialize("opl4latch", opl4latch);
-	if (ar.versionAtLeast(version, 2)) {
-		ar.serialize("alreadyReadID", alreadyReadID);
-	} else {
-		assert(ar.isLoader());
-		alreadyReadID = true; // we can't know the actual value, but
-		                      // 'true' is the safest value
-	}
+	ar.serialize("ymf262",    ymf262,
+	             "ymf278",    ymf278,
+	             "opl3latch", opl3latch,
+	             "opl4latch", opl4latch);
 	if (ar.versionAtLeast(version, 3)) {
-		ar.serialize("loadTime", ymf278LoadTime);
-		ar.serialize("busyTime", ymf278BusyTime);
+		ar.serialize("loadTime", ymf278LoadTime,
+		             "busyTime", ymf278BusyTime);
 	} else {
 		assert(ar.isLoader());
 		// For 100% backwards compatibility we should restore these two
