@@ -7,7 +7,7 @@
 #include "RenShaTurbo.hh"
 #include "serialize.hh"
 #include "checked_cast.hh"
-#include "memory.hh"
+#include <memory>
 
 namespace openmsx {
 
@@ -16,17 +16,16 @@ MSXPSG::MSXPSG(const DeviceConfig& config)
 	: MSXDevice(config)
 	, cassette(getMotherBoard().getCassettePort())
 	, renShaTurbo(getMotherBoard().getRenShaTurbo())
+	, selectedPort(0)
 	, prev(255)
-	, keyLayoutBit(config.getChildData("keyboardlayout", "") == "JIS")
+	, keyLayout((config.getChildData("keyboardlayout", {}) == "JIS") ? 0x40 : 0x00)
+	, addressMask(config.getChildDataAsBool("mirrored_registers", true) ? 0x0f : 0xff)
 {
-	selectedPort = 0;
 	ports[0] = &getMotherBoard().getJoystickPort(0);
 	ports[1] = &getMotherBoard().getJoystickPort(1);
+	ay8910 = std::make_unique<AY8910>("PSG", *this, config, getCurrentTime());
 
-	// must come after initialisation of ports
-	EmuTime::param time = getCurrentTime();
-	ay8910 = make_unique<AY8910>("PSG", *this, config, time);
-	reset(time);
+	reset(getCurrentTime());
 }
 
 MSXPSG::~MSXPSG()
@@ -47,9 +46,7 @@ void MSXPSG::powerDown(EmuTime::param /*time*/)
 
 byte MSXPSG::readIO(word /*port*/, EmuTime::param time)
 {
-	byte result = ay8910->readRegister(registerLatch, time);
-	//PRT_DEBUG("PSG read R#"<<(int)registerLatch<<" = "<<(int)result);
-	return result;
+	return ay8910->readRegister(registerLatch, time);
 }
 
 byte MSXPSG::peekIO(word /*port*/, EmuTime::param time) const
@@ -61,10 +58,9 @@ void MSXPSG::writeIO(word port, byte value, EmuTime::param time)
 {
 	switch (port & 0x03) {
 	case 0:
-		registerLatch = value & 0x0F;
+		registerLatch = value & addressMask;
 		break;
 	case 1:
-		//PRT_DEBUG("PSG write R#"<<(int)registerLatch<<" = "<<(int)value);
 		ay8910->writeRegister(registerLatch, value, time);
 		break;
 	}
@@ -82,7 +78,6 @@ byte MSXPSG::readA(EmuTime::param time)
 	joystick &= (pin67| 0xCF);
 
 	byte cassetteInput = cassette.cassetteIn(time) ? 0x80 : 0x00;
-	byte keyLayout = keyLayoutBit ? 0x40 : 0x00;
 	return joystick | keyLayout | cassetteInput;
 }
 
@@ -110,8 +105,8 @@ void MSXPSG::serialize(Archive& ar, unsigned version)
 	if (ar.versionBelow(version, 2)) {
 		assert(ar.isLoader());
 		// in older versions there were always 2 real joytsick ports
-		ar.serialize("joystickportA", *checked_cast<JoystickPort*>(ports[0]));
-		ar.serialize("joystickportB", *checked_cast<JoystickPort*>(ports[1]));
+		ar.serialize("joystickportA", *checked_cast<JoystickPort*>(ports[0]),
+		             "joystickportB", *checked_cast<JoystickPort*>(ports[1]));
 	}
 	ar.serialize("registerLatch", registerLatch);
 	byte portB = prev;
