@@ -1,5 +1,3 @@
-// $Id$
-
 #include "ResampledSoundDevice.hh"
 #include "ResampleTrivial.hh"
 #include "ResampleHQ.hh"
@@ -11,15 +9,18 @@
 #include "EnumSetting.hh"
 #include "unreachable.hh"
 #include <cassert>
+#include <memory>
 
 namespace openmsx {
 
 ResampledSoundDevice::ResampledSoundDevice(
-		MSXMotherBoard& motherBoard, string_ref name,
-		string_ref description, unsigned channels,
-		bool stereo)
-	: SoundDevice(motherBoard.getMSXMixer(), name, description, channels, stereo)
+		MSXMotherBoard& motherBoard, std::string_view name_,
+		std::string_view description_, unsigned channels,
+		unsigned inputSampleRate_, bool stereo_)
+	: SoundDevice(motherBoard.getMSXMixer(), name_, description_,
+	              channels, inputSampleRate_, stereo_)
 	, resampleSetting(motherBoard.getReactor().getGlobalSettings().getResampleSetting())
+	, emuClock(EmuTime::zero())
 {
 	resampleSetting.attach(*this);
 }
@@ -35,13 +36,13 @@ void ResampledSoundDevice::setOutputRate(unsigned /*sampleRate*/)
 	createResampler();
 }
 
-bool ResampledSoundDevice::updateBuffer(unsigned length, int* buffer,
+bool ResampledSoundDevice::updateBuffer(unsigned length, float* buffer,
                                         EmuTime::param time)
 {
 	return algo->generateOutput(buffer, length, time);
 }
 
-bool ResampledSoundDevice::generateInput(int* buffer, unsigned num)
+bool ResampledSoundDevice::generateInput(float* buffer, unsigned num)
 {
 	return mixChannels(buffer, num);
 }
@@ -59,30 +60,38 @@ void ResampledSoundDevice::createResampler()
 	const DynamicClock& hostClock = getHostSampleClock();
 	unsigned outputRate = hostClock.getFreq();
 	unsigned inputRate  = getInputRate() / getEffectiveSpeed();
+	emuClock.reset(hostClock.getTime());
+	emuClock.setFreq(inputRate);
 
 	if (outputRate == inputRate) {
-		algo.reset(new ResampleTrivial(*this));
+		algo = std::make_unique<ResampleTrivial>(*this);
 	} else {
-		switch (resampleSetting.getValue()) {
+		switch (resampleSetting.getEnum()) {
 		case RESAMPLE_HQ:
 			if (!isStereo()) {
-				algo.reset(new ResampleHQ<1>(*this, hostClock, inputRate));
+				algo = std::make_unique<ResampleHQ<1>>(
+					*this, hostClock);
 			} else {
-				algo.reset(new ResampleHQ<2>(*this, hostClock, inputRate));
+				algo = std::make_unique<ResampleHQ<2>>(
+					*this, hostClock);
 			}
 			break;
 		case RESAMPLE_LQ:
 			if (!isStereo()) {
-				algo = ResampleLQ<1>::create(*this, hostClock, inputRate);
+				algo = ResampleLQ<1>::create(
+					*this, hostClock, inputRate);
 			} else {
-				algo = ResampleLQ<2>::create(*this, hostClock, inputRate);
+				algo = ResampleLQ<2>::create(
+					*this, hostClock, inputRate);
 			}
 			break;
 		case RESAMPLE_BLIP:
 			if (!isStereo()) {
-				algo.reset(new ResampleBlip<1>(*this, hostClock, inputRate));
+				algo = std::make_unique<ResampleBlip<1>>(
+					*this, hostClock, inputRate);
 			} else {
-				algo.reset(new ResampleBlip<2>(*this, hostClock, inputRate));
+				algo = std::make_unique<ResampleBlip<2>>(
+					*this, hostClock, inputRate);
 			}
 			break;
 		default:
