@@ -97,47 +97,36 @@
 //-----------------------------------------------------------------------------
 
 #include "SCC.hh"
-#include "SimpleDebuggable.hh"
 #include "DeviceConfig.hh"
-#include "serialize.hh"
+#include "cstd.hh"
 #include "likely.hh"
+#include "outer.hh"
+#include "ranges.hh"
+#include "serialize.hh"
 #include "unreachable.hh"
-#include "memory.hh"
+#include <cmath>
 
 using std::string;
 
 namespace openmsx {
 
-class SCCDebuggable final : public SimpleDebuggable
-{
-public:
-	SCCDebuggable(MSXMotherBoard& motherBoard, SCC& scc);
-	byte read(unsigned address, EmuTime::param time) override;
-	void write(unsigned address, byte value, EmuTime::param time) override;
-private:
-	SCC& scc;
-};
-
+constexpr auto INPUT_RATE = unsigned(cstd::round(3579545.0 / 32));
 
 static string calcDescription(SCC::ChipMode mode)
 {
 	return (mode == SCC::SCC_Real) ? "Konami SCC" : "Konami SCC+";
 }
 
-SCC::SCC(const string& name, const DeviceConfig& config,
+SCC::SCC(const string& name_, const DeviceConfig& config,
          EmuTime::param time, ChipMode mode)
 	: ResampledSoundDevice(
-		config.getMotherBoard(), name, calcDescription(mode), 5)
-	, debuggable(make_unique<SCCDebuggable>(
-		config.getMotherBoard(), *this))
+		config.getMotherBoard(), name_, calcDescription(mode), 5, INPUT_RATE, false)
+	, debuggable(config.getMotherBoard(), getName())
 	, deformTimer(time)
 	, currentChipMode(mode)
 {
 	// Make valgrind happy
-	for (auto& op : orgPeriod) op = 0;
-
-	double input = 3579545.0 / 32;
-	setInputRate(int(input + 0.5));
+	ranges::fill(orgPeriod, 0);
 
 	powerUp(time);
 	registerSound(config);
@@ -162,11 +151,9 @@ void SCC::powerUp(EmuTime::param time)
 	// Initialize ch_enable, deform (initialize this before period)
 	reset(time);
 
-	// Initialize waveform (initialize before volumes)
-	for (unsigned i = 0; i < 5; ++i) {
-		for (unsigned j = 0; j < 32; ++j) {
-			wave[i][j] = ~0;
-		}
+	// Initialize waveforms (initialize before volumes)
+	for (auto& w1 : wave) {
+		ranges::fill(w1, ~0);
 	}
 	// Initialize volume (initialize this before period)
 	for (int i = 0; i < 5; ++i) {
@@ -174,7 +161,7 @@ void SCC::powerUp(EmuTime::param time)
 	}
 	// Actual initial value is difficult to measure, assume zero
 	// (initialize before period)
-	for (auto& p : pos) p = 0;
+	ranges::fill(pos, 0);
 
 	// Initialize period (sets members orgPeriod, period, incr, count, out)
 	for (int i = 0; i < 2 * 5; ++i) {
@@ -353,14 +340,17 @@ void SCC::writeMem(byte address, byte value, EmuTime::param time)
 	}
 }
 
-int SCC::getAmplificationFactor() const
+float SCC::getAmplificationFactorImpl() const
 {
-	return 256;
+	return 1.0f / 128.0f;
 }
 
-inline int SCC::adjust(signed char wav, byte vol)
+inline float SCC::adjust(signed char wav, byte vol)
 {
-	return (int(wav) * vol) >> 4;
+	// The result is an integer value, but we store it as a float because
+	// then we need fewer int->float conversion (compared to converting in
+	// generateChannels()).
+	return float((int(wav) * vol) >> 4);
 }
 
 void SCC::writeWave(unsigned channel, unsigned address, byte value)
@@ -370,13 +360,13 @@ void SCC::writeWave(unsigned channel, unsigned address, byte value)
 	assert((channel != 4) || (currentChipMode == SCC_plusmode));
 
 	if (!readOnly[channel]) {
-		unsigned pos = address & 0x1F;
-		wave[channel][pos] = value;
-		volAdjustedWave[channel][pos] = adjust(value, volume[channel]);
+		unsigned p = address & 0x1F;
+		wave[channel][p] = value;
+		volAdjustedWave[channel][p] = adjust(value, volume[channel]);
 		if ((currentChipMode != SCC_plusmode) && (channel == 3)) {
 			// copy waveform 4 -> waveform 5
-			wave[4][pos] = wave[3][pos];
-			volAdjustedWave[4][pos] = adjust(value, volume[4]);
+			wave[4][p] = wave[3][p];
+			volAdjustedWave[4][p] = adjust(value, volume[4]);
 		}
 	}
 }
@@ -441,16 +431,12 @@ void SCC::setDeformRegHelper(byte value)
 	}
 	switch (value & 0xC0) {
 	case 0x00:
-		for (unsigned i = 0; i < 5; ++i) {
-			rotate[i] = false;
-			readOnly[i] = false;
-		}
+		ranges::fill(rotate, false);
+		ranges::fill(readOnly, false);
 		break;
 	case 0x40:
-		for (unsigned i = 0; i < 5; ++i) {
-			rotate[i] = true;
-			readOnly[i] = true;
-		}
+		ranges::fill(rotate, true);
+		ranges::fill(readOnly, true);
 		break;
 	case 0x80:
 		for (unsigned i = 0; i < 3; ++i) {
@@ -477,53 +463,12 @@ void SCC::setDeformRegHelper(byte value)
 	}
 }
 
-void SCC::generateChannels(int** bufs, unsigned num)
+void SCC::generateChannels(float** bufs, unsigned num)
 {
 	unsigned enable = ch_enable;
 	for (unsigned i = 0; i < 5; ++i, enable >>= 1) {
 		if ((enable & 1) && (volume[i] || out[i])) {
-#ifdef __arm__
-			unsigned dummy;
-			int* buf = bufs[i];
-			asm volatile (
-			"0:\n\t"
-				"ldr	%[T],[%[B]]\n\t"
-				"add	%[T],%[T],%[O]\n\t"
-				"add	%[C],%[C],%[I]\n\t"
-				"str	%[T],[%[B]],#4\n\t"
-				"subs	%[T],%[C],%[PE]\n\t"
-				"bpl	2f\n"
-			"1:\n\t"
-				"cmp	%[B],%[E]\n\t"
-				"bne	0b\n\t"
-				"b	3f\n"
-			"2:\n\t"
-				"adds	%[PO],%[PO],#1\n\t"
-				"subs	%[T],%[T],%[PE]\n\t"
-				"bpl	2b\n\t"
-				"and	%[PO],%[PO],#31\n\t"
-				"add	%[C],%[T],%[PE]\n\t"
-				"ldr	%[O],[%[V],%[PO],LSL #2]\n\t"
-				"b	1b\n"
-			"3:\n\t"
-
-				: [T]  "=&r"  (dummy)
-				, [B]  "=r"   (buf)
-				, [O]  "=r"   (out[i])
-				, [C]  "=r"   (count[i])
-				, [PO] "=r"   (pos[i])
-				:      "[B]"  (buf)
-				,      "[O]"  (out[i])
-				,      "[C]"  (count[i])
-				, [I]  "r"    (incr[i])
-				, [PE] "r"    (period[i] + 1)
-				, [E]  "r"    (&buf[num])
-				,      "[PO]" (pos[i])
-				, [V]  "r"    (volAdjustedWave[i])
-				: "memory", "cc"
-			);
-#else
-			int out2 = out[i];
+			auto out2 = out[i];
 			unsigned count2 = count[i];
 			unsigned pos2 = pos[i];
 			unsigned incr2 = incr[i];
@@ -542,7 +487,6 @@ void SCC::generateChannels(int** bufs, unsigned num)
 			out[i] = out2;
 			count[i] = count2;
 			pos[i] = pos2;
-#endif
 		} else {
 			bufs[i] = nullptr; // channel muted
 			// Update phase counter.
@@ -550,23 +494,23 @@ void SCC::generateChannels(int** bufs, unsigned num)
 			count[i] = newCount % (period[i] + 1);
 			pos[i] = (pos[i] + newCount / (period[i] + 1)) % 32;
 			// Channel stays off until next waveform index.
-			out[i] = 0;
+			out[i] = 0.0f;
 		}
 	}
 }
 
 
-// SimpleDebuggable
+// Debuggable
 
-SCCDebuggable::SCCDebuggable(MSXMotherBoard& motherBoard, SCC& scc_)
-	: SimpleDebuggable(motherBoard, scc_.getName() + " SCC",
+SCC::Debuggable::Debuggable(MSXMotherBoard& motherBoard_, const string& name_)
+	: SimpleDebuggable(motherBoard_, name_ + " SCC",
 	                   "SCC registers in SCC+ format", 0x100)
-	, scc(scc_)
 {
 }
 
-byte SCCDebuggable::read(unsigned address, EmuTime::param time)
+byte SCC::Debuggable::read(unsigned address, EmuTime::param time)
 {
+	auto& scc = OUTER(SCC, debuggable);
 	if (address < 0xA0) {
 		// read wave form 1..5
 		return scc.readWave(address >> 5, address, time);
@@ -581,8 +525,9 @@ byte SCCDebuggable::read(unsigned address, EmuTime::param time)
 	}
 }
 
-void SCCDebuggable::write(unsigned address, byte value, EmuTime::param time)
+void SCC::Debuggable::write(unsigned address, byte value, EmuTime::param time)
 {
+	auto& scc = OUTER(SCC, debuggable);
 	if (address < 0xA0) {
 		// read wave form 1..5
 		scc.writeWave(address >> 5, address, value);
@@ -598,7 +543,7 @@ void SCCDebuggable::write(unsigned address, byte value, EmuTime::param time)
 }
 
 
-static enum_string<SCC::ChipMode> chipModeInfo[] = {
+static std::initializer_list<enum_string<SCC::ChipMode>> chipModeInfo = {
 	{ "Real",       SCC::SCC_Real       },
 	{ "Compatible", SCC::SCC_Compatible },
 	{ "Plus",       SCC::SCC_plusmode   },
@@ -608,12 +553,12 @@ SERIALIZE_ENUM(SCC::ChipMode, chipModeInfo);
 template<typename Archive>
 void SCC::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("mode", currentChipMode);
-	ar.serialize("period", orgPeriod);
-	ar.serialize("volume", volume);
-	ar.serialize("ch_enable", ch_enable);
-	ar.serialize("deformTimer", deformTimer);
-	ar.serialize("deform", deformValue);
+	ar.serialize("mode",        currentChipMode,
+	             "period",      orgPeriod,
+	             "volume",      volume,
+	             "ch_enable",   ch_enable,
+	             "deformTimer", deformTimer,
+	             "deform",      deformValue);
 	// multi-dimensional arrays are not directly support by the
 	// serialization framework, maybe in the future. So for now
 	// manually loop over the channels.
@@ -626,9 +571,9 @@ void SCC::serialize(Archive& ar, unsigned /*version*/)
 	if (ar.isLoader()) {
 		// recalculate volAdjustedWave
 		for (int channel = 0; channel < 5; ++channel) {
-			for (int pos = 0; pos < 32; ++pos) {
-				volAdjustedWave[channel][pos] =
-					adjust(wave[channel][pos], volume[channel]);
+			for (int p = 0; p < 32; ++p) {
+				volAdjustedWave[channel][p] =
+					adjust(wave[channel][p], volume[channel]);
 			}
 		}
 
@@ -649,9 +594,9 @@ void SCC::serialize(Archive& ar, unsigned /*version*/)
 	}
 
 	// call to setFreqVol() modifies these variables, see above
-	ar.serialize("count", count);
-	ar.serialize("pos", pos);
-	ar.serialize("out", out);
+	ar.serialize("count", count,
+	             "pos",   pos,
+	             "out",   out); // note: changed int->float, but no need to bump serialize-version
 }
 INSTANTIATE_SERIALIZE_METHODS(SCC);
 
