@@ -2,42 +2,40 @@
 #include "DeviceConfig.hh"
 #include "CliComm.hh"
 #include "FileContext.hh"
-#include "StringOp.hh"
 #include "MSXException.hh"
 #include "serialize.hh"
 #include <cassert>
 
 namespace openmsx {
 
-SamplePlayer::SamplePlayer(const std::string& name, const std::string& desc,
+constexpr unsigned DUMMY_INPUT_RATE = 44100; // actual rate depends on .wav files
+
+SamplePlayer::SamplePlayer(const std::string& name_, const std::string& desc,
                            const DeviceConfig& config,
                            const std::string& samplesBaseName, unsigned numSamples,
                            const std::string& alternativeName)
-	: ResampledSoundDevice(config.getMotherBoard(), name, desc, 1)
+	: ResampledSoundDevice(config.getMotherBoard(), name_, desc, 1, DUMMY_INPUT_RATE, false)
 {
-	setInputRate(44100); // Initialize with dummy value
-
 	bool alreadyWarned = false;
 	samples.resize(numSamples); // initialize with empty wavs
-	SystemFileContext context;
+	auto context = systemFileContext();
 	for (unsigned i = 0; i < numSamples; ++i) {
 		try {
-			std::string filename = StringOp::Builder() <<
-				samplesBaseName << i << ".wav";
+			std::string filename = strCat(samplesBaseName, i, ".wav");
 			samples[i] = WavData(context.resolve(filename));
 		} catch (MSXException& e1) {
 			try {
 				if (alternativeName.empty()) throw;
-				std::string filename = StringOp::Builder() <<
-					alternativeName << i << ".wav";
+				std::string filename = strCat(
+					alternativeName, i, ".wav");
 				samples[i] = WavData(context.resolve(filename));
 			} catch (MSXException& /*e2*/) {
 				if (!alreadyWarned) {
 					alreadyWarned = true;
 					// print message from the 1st error
 					config.getCliComm().printWarning(
-						"Couldn't read " + name + " sample data: " +
-						e1.getMessage() +
+						"Couldn't read ", name_, " sample data: ",
+						e1.getMessage(),
 						". Continuing without sample data.");
 				}
 			}
@@ -75,13 +73,8 @@ void SamplePlayer::setWavParams()
 	if ((currentSampleNum < samples.size()) &&
 	    samples[currentSampleNum].getSize()) {
 		auto& wav = samples[currentSampleNum];
-		sampBuf = wav.getData();
 		bufferSize = wav.getSize();
 
-		unsigned bits = wav.getBits();
-		assert((bits == 8) || (bits == 16));
-		bits8 = (bits == 8);
-
 		unsigned freq = wav.getFreq();
 		if (freq != getInputRate()) {
 			// this potentially switches resampler, so there might be
@@ -104,20 +97,15 @@ void SamplePlayer::repeat(unsigned sampleNum)
 	}
 }
 
-inline int SamplePlayer::getSample(unsigned index)
-{
-	return bits8
-	     ? (static_cast<const unsigned char*>(sampBuf)[index] - 0x80) * 256
-	     :  static_cast<const short*        >(sampBuf)[index];
-}
-
-void SamplePlayer::generateChannels(int** bufs, unsigned num)
+void SamplePlayer::generateChannels(float** bufs, unsigned num)
 {
 	// Single channel device: replace content of bufs[0] (not add to it).
 	if (!isPlaying()) {
 		bufs[0] = nullptr;
 		return;
 	}
+
+	auto& wav = samples[currentSampleNum];
 	for (unsigned i = 0; i < num; ++i) {
 		if (index >= bufferSize) {
 			if (nextSampleNum != unsigned(-1)) {
@@ -126,12 +114,12 @@ void SamplePlayer::generateChannels(int** bufs, unsigned num)
 				currentSampleNum = unsigned(-1);
 				// fill remaining buffer with zeros
 				do {
-					bufs[0][i++] = 0;
+					bufs[0][i++] = 0.0f;
 				} while (i < num);
 				break;
 			}
 		}
-		bufs[0][i] = 3 * getSample(index++);
+		bufs[0][i] = 3 * wav.getSample(index++);
 	}
 }
 
@@ -143,9 +131,9 @@ void SamplePlayer::doRepeat()
 template<typename Archive>
 void SamplePlayer::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("index", index);
-	ar.serialize("currentSampleNum", currentSampleNum);
-	ar.serialize("nextSampleNum", nextSampleNum);
+	ar.serialize("index",            index,
+	             "currentSampleNum", currentSampleNum,
+	             "nextSampleNum",    nextSampleNum);
 	if (ar.isLoader()) {
 		setWavParams();
 	}
