@@ -9,81 +9,91 @@
 #include "MemBuffer.hh"
 #include "MSXException.hh"
 #include "likely.hh"
+#include "ranges.hh"
+#include "one_of.hh"
 #include "vla.hh"
-#include "memory.hh"
+#include "xrange.hh"
 #include <cassert>
+#include <memory>
 
 using std::string;
 
 namespace openmsx {
 
-MemBuffer<int> mixBufferStorage;
-int* mixBuffer; // 16-byte aligned ptr into mixBufferStorage (for SSE access)
+static MemBuffer<float, SSE2_ALIGNMENT> mixBuffer;
+static unsigned mixBufferSize = 0;
 
 static void allocateMixBuffer(unsigned size)
 {
-	size += 3; // to be able to align
-	if (unlikely(mixBufferStorage.size() < size)) {
-		mixBufferStorage.resize(size);
-		// align at 16-byte boundary
-		auto tmp = reinterpret_cast<size_t>(mixBufferStorage.data());
-		mixBuffer = reinterpret_cast<int*>((tmp + 15) & ~15);
+	if (unlikely(mixBufferSize < size)) {
+		mixBufferSize = size;
+		mixBuffer.resize(mixBufferSize);
 	}
 }
 
-static string makeUnique(MSXMixer& mixer, string_ref name)
+static string makeUnique(MSXMixer& mixer, std::string_view name)
 {
-	string result = name.str();
+	string result(name);
 	if (mixer.findDevice(result)) {
 		unsigned n = 0;
 		do {
-			result = StringOp::Builder() << name << " (" << ++n << ')';
+			result = strCat(name, " (", ++n, ')');
 		} while (mixer.findDevice(result));
 	}
 	return result;
 }
 
-SoundDevice::SoundDevice(MSXMixer& mixer_, string_ref name_,
-			 string_ref description_,
-			 unsigned numChannels_, bool stereo_)
+void SoundDevice::addFill(float*& buf, float val, unsigned num)
+{
+	// Note: in the past we tried to optimize this by always producing
+	// a multiple of 4 output values. In the general case a sounddevice is
+	// allowed to do this, but only at the end of the soundbuffer. This
+	// method can also be called in the middle of a buffer (so multiple
+	// times per buffer), in such case it does go wrong.
+	assert(num > 0);
+	do {
+		*buf++ += val;
+	} while (--num);
+}
+
+SoundDevice::SoundDevice(MSXMixer& mixer_, std::string_view name_, std::string_view description_,
+			 unsigned numChannels_, unsigned inputRate, bool stereo_)
 	: mixer(mixer_)
 	, name(makeUnique(mixer, name_))
-	, description(description_.str())
+	, description(description_)
 	, numChannels(numChannels_)
 	, stereo(stereo_ ? 2 : 1)
 	, numRecordChannels(0)
 	, balanceCenter(true)
 {
 	assert(numChannels <= MAX_CHANNELS);
-	assert(stereo == 1 || stereo == 2);
+	assert(stereo == one_of(1u, 2u));
+
+	setInputRate(inputRate);
 
 	// initially no channels are muted
-	for (unsigned i = 0; i < numChannels; ++i) {
-		channelMuted[i] = false;
-		channelBalance[i] = 0;
-	}
+	ranges::fill(channelMuted, false);
+	ranges::fill(channelBalance, 0);
 }
 
-SoundDevice::~SoundDevice()
-{
-}
+SoundDevice::~SoundDevice() = default;
 
 bool SoundDevice::isStereo() const
 {
 	return stereo == 2 || !balanceCenter;
 }
 
-int SoundDevice::getAmplificationFactor() const
+float SoundDevice::getAmplificationFactorImpl() const
 {
-	return 1;
+	return 1.0f / 32768.0f;
 }
 
 void SoundDevice::registerSound(const DeviceConfig& config)
 {
 	const XMLElement& soundConfig = config.getChild("sound");
-	double volume = soundConfig.getChildDataAsInt("volume") / 32767.0;
+	float volume = soundConfig.getChildDataAsInt("volume") / 32767.0f;
 	int devBalance = 0;
-	string_ref mode = soundConfig.getChildData("mode", "mono");
+	std::string_view mode = soundConfig.getChildData("mode", "mono");
 	if (mode == "mono") {
 		devBalance = 0;
 	} else if (mode == "left") {
@@ -91,11 +101,11 @@ void SoundDevice::registerSound(const DeviceConfig& config)
 	} else if (mode == "right") {
 		devBalance = 100;
 	} else {
-		throw MSXException("balance \"" + mode + "\" illegal");
+		throw MSXException("balance \"", mode, "\" illegal");
 	}
 
 	for (auto& b : soundConfig.getChildren("balance")) {
-		int balance = b->getDataAsInt();
+		int balance = StringOp::stringToInt(b->getData());
 
 		if (!b->hasAttribute("channel")) {
 			devBalance = balance;
@@ -103,9 +113,8 @@ void SoundDevice::registerSound(const DeviceConfig& config)
 		}
 
 		// TODO Support other balances
-		if (balance != 0 && balance != -100 && balance != 100) {
-			throw MSXException(StringOp::Builder() <<
-			                   "balance " << balance << " illegal");
+		if (balance != one_of(0, -100, 100)) {
+			throw MSXException("balance ", balance, " illegal");
 		}
 		if (balance != 0) {
 			balanceCenter = false;
@@ -130,12 +139,25 @@ void SoundDevice::updateStream(EmuTime::param time)
 	mixer.updateStream(time);
 }
 
+void SoundDevice::setSoftwareVolume(float volume, EmuTime::param time)
+{
+	setSoftwareVolume(volume, volume, time);
+}
+
+void SoundDevice::setSoftwareVolume(float left, float right, EmuTime::param time)
+{
+	updateStream(time);
+	softwareVolumeLeft  = left;
+	softwareVolumeRight = right;
+	mixer.updateSoftwareVolume(*this);
+}
+
 void SoundDevice::recordChannel(unsigned channel, const Filename& filename)
 {
 	assert(channel < numChannels);
 	bool wasRecording = writer[channel] != nullptr;
 	if (!filename.empty()) {
-		writer[channel] = make_unique<Wav16Writer>(
+		writer[channel] = std::make_unique<Wav16Writer>(
 			filename, stereo, inputSampleRate);
 	} else {
 		writer[channel].reset();
@@ -164,7 +186,7 @@ void SoundDevice::muteChannel(unsigned channel, bool muted)
 	channelMuted[channel] = muted;
 }
 
-bool SoundDevice::mixChannels(int* dataOut, unsigned samples)
+bool SoundDevice::mixChannels(float* dataOut, unsigned samples)
 {
 #ifdef __SSE2__
 	assert((uintptr_t(dataOut) & 15) == 0); // must be 16-byte aligned
@@ -172,17 +194,18 @@ bool SoundDevice::mixChannels(int* dataOut, unsigned samples)
 	if (samples == 0) return true;
 	unsigned outputStereo = isStereo() ? 2 : 1;
 
-	MemoryOps::MemSet<unsigned> mset;
+	static_assert(sizeof(float) == sizeof(uint32_t));
+	MemoryOps::MemSet<uint32_t> mset;
 	if (numChannels != 1) {
 		// The generateChannels() method of SoundDevices with more than
 		// one channel will _add_ the generated channel data in the
 		// provided buffers. Those with only one channel will directly
 		// replace the content of the buffer. For the former we must
 		// start from a buffer containing all zeros.
-		mset(reinterpret_cast<unsigned*>(dataOut), outputStereo * samples, 0);
+		mset(reinterpret_cast<uint32_t*>(dataOut), outputStereo * samples, 0);
 	}
 
-	VLA(int*, bufs, numChannels);
+	VLA(float*, bufs, numChannels);
 	unsigned separateChannels = 0;
 	unsigned pitch = (samples * stereo + 3) & ~3; // align for SSE access
 	// TODO optimization: All channels with the same balance (according to
@@ -200,7 +223,7 @@ bool SoundDevice::mixChannels(int* dataOut, unsigned samples)
 	}
 	if (separateChannels) {
 		allocateMixBuffer(pitch * separateChannels);
-		mset(reinterpret_cast<unsigned*>(mixBuffer),
+		mset(reinterpret_cast<uint32_t*>(mixBuffer.data()),
 		     pitch * separateChannels, 0);
 		// still need to fill in (some) bufs[i] pointers
 		unsigned count = 0;
@@ -215,12 +238,8 @@ bool SoundDevice::mixChannels(int* dataOut, unsigned samples)
 	generateChannels(bufs, samples);
 
 	if (separateChannels == 0) {
-		for (unsigned i = 0; i < numChannels; ++i) {
-			if (bufs[i]) {
-				return true;
-			}
-		}
-		return false;
+		return ranges::any_of(xrange(numChannels),
+		                      [&bufs](auto i) { return bufs[i]; });
 	}
 
 	// record channels
@@ -228,8 +247,11 @@ bool SoundDevice::mixChannels(int* dataOut, unsigned samples)
 		if (writer[i]) {
 			assert(bufs[i] != dataOut);
 			if (bufs[i]) {
+				auto amp = getAmplificationFactor();
 				writer[i]->write(
-					bufs[i], stereo, samples, getAmplificationFactor());
+					bufs[i], stereo, samples,
+					amp.first,
+					amp.second);
 			} else {
 				writer[i]->writeSilence(stereo, samples);
 			}
@@ -260,10 +282,10 @@ bool SoundDevice::mixChannels(int* dataOut, unsigned samples)
 	if (!balanceCenter) {
 		unsigned i = 0;
 		do {
-			int left0  = 0;
-			int right0 = 0;
-			int left1  = 0;
-			int right1 = 0;
+			float left0  = 0.0f;
+			float right0 = 0.0f;
+			float left1  = 0.0f;
+			float right1 = 0.0f;
 			unsigned j = 0;
 			do {
 				if (mixBalance[j] <= 0) {
@@ -293,10 +315,10 @@ bool SoundDevice::mixChannels(int* dataOut, unsigned samples)
 	unsigned num = samples * stereo;
 	unsigned i = 0;
 	do {
-		int out0 = dataOut[i + 0];
-		int out1 = dataOut[i + 1];
-		int out2 = dataOut[i + 2];
-		int out3 = dataOut[i + 3];
+		auto out0 = dataOut[i + 0];
+		auto out1 = dataOut[i + 1];
+		auto out2 = dataOut[i + 2];
+		auto out3 = dataOut[i + 3];
 		unsigned j = 0;
 		do {
 			out0 += bufs[j][i + 0];
