@@ -75,94 +75,29 @@ chirp 12-..: vokume   0   : silent
 */
 
 #include "VLM5030.hh"
-#include "ResampledSoundDevice.hh"
-#include "Rom.hh"
 #include "DeviceConfig.hh"
 #include "XMLElement.hh"
 #include "FileOperations.hh"
+#include "Math.hh"
+#include "cstd.hh"
+#include "one_of.hh"
+#include "random.hh"
+#include "ranges.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include <cmath>
 #include <cstring>
-#include <cstdlib>
 
 namespace openmsx {
 
-class VLM5030::Impl final : public ResampledSoundDevice
-{
-public:
-	Impl(const std::string& name, const std::string& desc,
-	     const std::string& romFilename, const DeviceConfig& config);
-	~Impl();
-
-	void reset();
-	void writeData(byte data);
-	void writeControl(byte data, EmuTime::param time);
-	bool getBSY(EmuTime::param time);
-
-	template<typename Archive>
-	void serialize(Archive& ar, unsigned version);
-
-private:
-	void setRST(bool pin);
-	void setVCU(bool pin);
-	void setST (bool pin);
-
-	// SoundDevice
-	void generateChannels(int** bufs, unsigned num) override;
-
-	void setupParameter(byte param);
-	int getBits(unsigned sbit, unsigned bits);
-	int parseFrame();
-
-	std::unique_ptr<Rom> rom;
-	int address_mask;
-
-	// state of option paramter
-	int frame_size;
-	int pitch_offset;
-
-	// these contain data describing the current and previous voice frames
-	// these are all used to contain the current state of the sound generation
-	unsigned current_energy;
-	unsigned current_pitch;
-	int current_k[10];
-	int x[10];
-
-	word address;
-	word vcu_addr_h;
-
-	signed_word old_k[10];
-	signed_word new_k[10];
-	signed_word target_k[10];
-	word old_energy;
-	word new_energy;
-	word target_energy;
-	byte old_pitch;
-	byte new_pitch;
-	byte target_pitch;
-
-	byte interp_step;
-	byte interp_count; // number of interp periods
-	byte sample_count; // sample number within interp
-	byte pitch_count;
-
-	byte latch_data;
-	byte parameter;
-	byte phase;
-	bool pin_BSY;
-	bool pin_ST;
-	bool pin_VCU;
-	bool pin_RST;
-};
 
 // interpolator per frame
-static const int FR_SIZE = 4;
+constexpr int FR_SIZE = 4;
 // samples per interpolator
-static const int IP_SIZE_SLOWER = 240 / FR_SIZE;
-static const int IP_SIZE_SLOW   = 200 / FR_SIZE;
-static const int IP_SIZE_NORMAL = 160 / FR_SIZE;
-static const int IP_SIZE_FAST   = 120 / FR_SIZE;
-static const int IP_SIZE_FASTER =  80 / FR_SIZE;
+constexpr int IP_SIZE_SLOWER = 240 / FR_SIZE;
+constexpr int IP_SIZE_SLOW   = 200 / FR_SIZE;
+constexpr int IP_SIZE_NORMAL = 160 / FR_SIZE;
+constexpr int IP_SIZE_FAST   = 120 / FR_SIZE;
+constexpr int IP_SIZE_FASTER =  80 / FR_SIZE;
 
 // phase value
 enum {
@@ -182,7 +117,7 @@ enum {
 //  x   0   0  normal    (00h,04h) : 25.6ms (100%) : 40samplme
 //  0   0   1  fast      (01h)     : 20.2ms  (75%) : 30sample
 //  0   1   x  more fast (02h,03h) : 12.2ms  (50%) : 20sample
-static const int VLM5030_speed_table[8] =
+constexpr int VLM5030_speed_table[8] =
 {
 	IP_SIZE_NORMAL,
 	IP_SIZE_FAST,
@@ -208,7 +143,7 @@ static word energytable[0x20] =
 };
 
 // This is the pitch lookup table
-static const byte pitchtable [0x20] =
+constexpr byte pitchtable [0x20] =
 {
 	1,                               // 0     : random mode
 	22,                              // 1     : start=22
@@ -218,7 +153,7 @@ static const byte pitchtable [0x20] =
 	86, 94, 102,110,118,126          // 26-31 : 8step
 };
 
-static const signed_word K1_table[] = {
+constexpr int16_t K1_table[] = {
 	-24898,  -25672,  -26446,  -27091,  -27736,  -28252,  -28768,  -29155,
 	-29542,  -29929,  -30316,  -30574,  -30832,  -30961,  -31219,  -31348,
 	-31606,  -31735,  -31864,  -31864,  -31993,  -32122,  -32122,  -32251,
@@ -228,32 +163,32 @@ static const signed_word K1_table[] = {
 	     0,   -1935,   -3999,   -6063,   -7998,   -9804,  -11610,  -13416,
 	-15093,  -16642,  -18061,  -19480,  -20770,  -21931,  -22963,  -23995
 };
-static const signed_word K2_table[] = {
+constexpr int16_t K2_table[] = {
 	     0,   -3096,   -6321,   -9417,  -12513,  -15351,  -18061,  -20770,
 	-23092,  -25285,  -27220,  -28897,  -30187,  -31348,  -32122,  -32638,
 	     0,   32638,   32122,   31348,   30187,   28897,   27220,   25285,
 	 23092,   20770,   18061,   15351,   12513,    9417,    6321,    3096
 };
-static const signed_word K3_table[] = {
+constexpr int16_t K3_table[] = {
 	    0,   -3999,   -8127,  -12255,  -16384,  -20383,  -24511,  -28639,
 	32638,   28639,   24511,   20383,   16254,   12255,    8127,    3999
 };
-static const signed_word K5_table[] = {
+constexpr int16_t K5_table[] = {
 	0,   -8127,  -16384,  -24511,   32638,   24511,   16254,    8127
 };
 
-int VLM5030::Impl::getBits(unsigned sbit, unsigned bits)
+int VLM5030::getBits(unsigned sbit, unsigned bits)
 {
 	unsigned offset = address + (sbit / 8);
-	unsigned data = (*rom)[(offset + 0) & address_mask] +
-	                (*rom)[(offset + 1) & address_mask] * 256;
+	unsigned data = rom[(offset + 0) & address_mask] +
+	                rom[(offset + 1) & address_mask] * 256;
 	data >>= (sbit & 7);
 	data &= (0xFF >> (8 - bits));
 	return data;
 }
 
 // get next frame
-int VLM5030::Impl::parseFrame()
+int VLM5030::parseFrame()
 {
 	// remember previous frame
 	old_energy = new_energy;
@@ -262,13 +197,11 @@ int VLM5030::Impl::parseFrame()
 		old_k[i] = new_k[i];
 	}
 	// command byte check
-	byte cmd = (*rom)[address & address_mask];
+	byte cmd = rom[address & address_mask];
 	if (cmd & 0x01) {
 		// extend frame
 		new_energy = new_pitch = 0;
-		for (int i = 0; i <= 9; ++i) {
-			new_k[i] = 0;
-		}
+		ranges::fill(new_k, 0);
 		++address;
 		if (cmd & 0x02) {
 			// end of speech
@@ -301,7 +234,7 @@ int VLM5030::Impl::parseFrame()
 }
 
 // decode and buffering data
-void VLM5030::Impl::generateChannels(int** bufs, unsigned length)
+void VLM5030::generateChannels(float** bufs, unsigned num)
 {
 	// Single channel device: replace content of bufs[0] (not add to it).
 	if (phase == PH_IDLE) {
@@ -312,9 +245,9 @@ void VLM5030::Impl::generateChannels(int** bufs, unsigned length)
 	int buf_count = 0;
 
 	// running
-	if (phase == PH_RUN || phase == PH_STOP) {
+	if (phase == one_of(PH_RUN, PH_STOP)) {
 		// playing speech
-		while (length > 0) {
+		while (num > 0) {
 			int current_val;
 			// check new interpolator or new frame
 			if (sample_count == 0) {
@@ -374,8 +307,8 @@ void VLM5030::Impl::generateChannels(int** bufs, unsigned length)
 				current_val = 0x00;
 			} else if (old_pitch <= 1) {
 				// generate unvoiced samples here
-				current_val = (rand() & 1) ?  int(current_energy)
-				                           : -int(current_energy);
+				current_val = random_bool() ?  int(current_energy)
+				                            : -int(current_energy);
 			} else {
 				// generate voiced samples here
 				current_val = (pitch_count == 0) ? current_energy : 0;
@@ -393,52 +326,51 @@ void VLM5030::Impl::generateChannels(int** bufs, unsigned length)
 			x[0] = u[0];
 
 			// clipping, buffering
-			if (u[0] > 511) {
-				bufs[0][buf_count] =  511 << 6;
-			} else if (u[0] < -511) {
-				bufs[0][buf_count] = -511 << 6;
-			} else {
-				bufs[0][buf_count] = (u[0] << 6);
-			}
+			bufs[0][buf_count] = Math::clip<-511, 511>(u[0]);
 			++buf_count;
 			--sample_count;
 			++pitch_count;
 			if (pitch_count >= current_pitch) {
 				pitch_count = 0;
 			}
-			--length;
+			--num;
 		}
 	// return;
 	}
 phase_stop:
 	switch (phase) {
 	case PH_SETUP:
-		if (sample_count <= length) {
+		if (sample_count <= num) {
 			sample_count = 0;
 			// pin_BSY = true;
 			phase = PH_WAIT;
 		} else {
-			sample_count -= length;
+			sample_count -= num;
 		}
 		break;
 	case PH_END:
-		if (sample_count <= length) {
+		if (sample_count <= num) {
 			sample_count = 0;
 			pin_BSY = false;
 			phase = PH_IDLE;
 		} else {
-			sample_count -= length;
+			sample_count -= num;
 		}
 	}
 	// silent buffering
-	while (length > 0) {
+	while (num > 0) {
 		bufs[0][buf_count++] = 0;
-		--length;
+		--num;
 	}
 }
 
+float VLM5030::getAmplificationFactorImpl() const
+{
+	return 1.0f / (1 << 9);
+}
+
 // setup parameteroption when RST=H
-void VLM5030::Impl::setupParameter(byte param)
+void VLM5030::setupParameter(byte param)
 {
 	// latch parameter value
 	parameter = param;
@@ -465,7 +397,7 @@ void VLM5030::Impl::setupParameter(byte param)
 	}
 }
 
-void VLM5030::Impl::reset()
+void VLM5030::reset()
 {
 	phase = PH_RESET;
 	address = 0;
@@ -487,19 +419,19 @@ void VLM5030::Impl::reset()
 }
 
 // get BSY pin level
-bool VLM5030::Impl::getBSY(EmuTime::param time)
+bool VLM5030::getBSY(EmuTime::param time) const
 {
-	updateStream(time);
+	const_cast<VLM5030*>(this)->updateStream(time);
 	return pin_BSY;
 }
 
 // latch control data
-void VLM5030::Impl::writeData(byte data)
+void VLM5030::writeData(byte data)
 {
 	latch_data = data;
 }
 
-void VLM5030::Impl::writeControl(byte data, EmuTime::param time)
+void VLM5030::writeControl(byte data, EmuTime::param time)
 {
 	updateStream(time);
 	setRST((data & 0x01) != 0);
@@ -508,7 +440,7 @@ void VLM5030::Impl::writeControl(byte data, EmuTime::param time)
 }
 
 // set RST pin level : reset / set table address A8-A15
-void VLM5030::Impl::setRST(bool pin)
+void VLM5030::setRST(bool pin)
 {
 	if (pin_RST) {
 		if (!pin) { // H -> L : latch parameters
@@ -526,14 +458,14 @@ void VLM5030::Impl::setRST(bool pin)
 }
 
 // set VCU pin level : ?? unknown
-void VLM5030::Impl::setVCU(bool pin)
+void VLM5030::setVCU(bool pin)
 {
 	// direct mode / indirect mode
 	pin_VCU = pin;
 }
 
 // set ST pin level  : set table address A0-A7 / start speech
-void VLM5030::Impl::setST(bool pin)
+void VLM5030::setST(bool pin)
 {
 	if (pin_ST == pin) {
 		// pin level unchanged
@@ -554,8 +486,8 @@ void VLM5030::Impl::setST(bool pin)
 			} else {
 				// indirect access mode
 				int table = (latch_data & 0xfe) + ((int(latch_data) & 1) << 8);
-				address = (((*rom)[(table + 0) & address_mask]) << 8) |
-				            (*rom)[(table + 1) & address_mask];
+				address = ((rom[(table + 0) & address_mask]) << 8) |
+				            rom[(table + 1) & address_mask];
 			}
 			// reset process status
 			sample_count = frame_size;
@@ -574,9 +506,8 @@ void VLM5030::Impl::setST(bool pin)
 	}
 }
 
-VLM5030::Impl::Impl(const std::string& name, const std::string& desc,
-                    const std::string& romFilename, const DeviceConfig& config)
-	: ResampledSoundDevice(config.getMotherBoard(), name, desc, 1)
+
+static XMLElement getRomConfig(const std::string& name, const std::string& romFilename)
 {
 	XMLElement voiceROMconfig(name);
 	voiceROMconfig.addAttribute("id", "name");
@@ -584,12 +515,19 @@ VLM5030::Impl::Impl(const std::string& name, const std::string& desc,
 	romElement.addChild( // load by sha1sum
 		"sha1", "4f36d139ee4baa7d5980f765de9895570ee05f40");
 	romElement.addChild( // load by predefined filename in software rom's dir
-		"filename", FileOperations::stripExtension(romFilename) + "_voice.rom");
+		"filename", strCat(FileOperations::stripExtension(romFilename), "_voice.rom"));
 	romElement.addChild( // or hardcoded filename in ditto dir
 		"filename", "keyboardmaster/voice.rom");
-	rom = make_unique<Rom>(
-		name + " ROM", "rom", DeviceConfig(config, voiceROMconfig));
+	return voiceROMconfig;
+}
 
+constexpr auto INPUT_RATE = unsigned(cstd::round(3579545 / 440.0));
+
+VLM5030::VLM5030(const std::string& name_, const std::string& desc,
+                 const std::string& romFilename, const DeviceConfig& config)
+	: ResampledSoundDevice(config.getMotherBoard(), name_, desc, 1, INPUT_RATE, false)
+	, rom(name_ + " ROM", "rom", DeviceConfig(config, getRomConfig(name_, romFilename)))
+{
 	// reset input pins
 	pin_RST = pin_ST = pin_VCU = false;
 	latch_data = 0;
@@ -597,92 +535,50 @@ VLM5030::Impl::Impl(const std::string& name, const std::string& desc,
 	reset();
 	phase = PH_IDLE;
 
-	address_mask = rom->getSize() - 1;
-
-	const int CLOCK_FREQ = 3579545;
-	double input = CLOCK_FREQ / 440.0;
-	setInputRate(int(input + 0.5));
+	address_mask = rom.getSize() - 1;
 
 	registerSound(config);
 }
 
-VLM5030::Impl::~Impl()
+VLM5030::~VLM5030()
 {
 	unregisterSound();
 }
 
 template<typename Archive>
-void VLM5030::Impl::serialize(Archive& ar, unsigned /*version*/)
+void VLM5030::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("address_mask", address_mask);
-	ar.serialize("frame_size", frame_size);
-	ar.serialize("pitch_offset", pitch_offset);
-	ar.serialize("current_energy", current_energy);
-	ar.serialize("current_pitch", current_pitch);
-	ar.serialize("current_k", current_k);
-	ar.serialize("x", x);
-	ar.serialize("address", address);
-	ar.serialize("vcu_addr_h", vcu_addr_h);
-	ar.serialize("old_k", old_k);
-	ar.serialize("new_k", new_k);
-	ar.serialize("target_k", target_k);
-	ar.serialize("old_energy", old_energy);
-	ar.serialize("new_energy", new_energy);
-	ar.serialize("target_energy", target_energy);
-	ar.serialize("old_pitch", old_pitch);
-	ar.serialize("new_pitch", new_pitch);
-	ar.serialize("target_pitch", target_pitch);
-	ar.serialize("interp_step", interp_step);
-	ar.serialize("interp_count", interp_count);
-	ar.serialize("sample_count", sample_count);
-	ar.serialize("pitch_count", pitch_count);
-	ar.serialize("latch_data", latch_data);
-	ar.serialize("parameter", parameter);
-	ar.serialize("phase", phase);
-	ar.serialize("pin_BSY", pin_BSY);
-	ar.serialize("pin_ST", pin_ST);
-	ar.serialize("pin_VCU", pin_VCU);
-	ar.serialize("pin_RST", pin_RST);
+	ar.serialize("address_mask",   address_mask,
+	             "frame_size",     frame_size,
+	             "pitch_offset",   pitch_offset,
+	             "current_energy", current_energy,
+	             "current_pitch",  current_pitch,
+	             "current_k",      current_k,
+	             "x",              x,
+	             "address",        address,
+	             "vcu_addr_h",     vcu_addr_h,
+	             "old_k",          old_k,
+	             "new_k",          new_k,
+	             "target_k",       target_k,
+	             "old_energy",     old_energy,
+	             "new_energy",     new_energy,
+	             "target_energy",  target_energy,
+	             "old_pitch",      old_pitch,
+	             "new_pitch",      new_pitch,
+	             "target_pitch",   target_pitch,
+	             "interp_step",    interp_step,
+	             "interp_count",   interp_count,
+	             "sample_count",   sample_count,
+	             "pitch_count",    pitch_count,
+	             "latch_data",     latch_data,
+	             "parameter",      parameter,
+	             "phase",          phase,
+	             "pin_BSY",        pin_BSY,
+	             "pin_ST",         pin_ST,
+	             "pin_VCU",        pin_VCU,
+	             "pin_RST",        pin_RST);
 }
 
-
-// class VLM5030
-
-VLM5030::VLM5030(const std::string& name, const std::string& desc,
-                 const std::string& romFilename, const DeviceConfig& config)
-	: pimpl(make_unique<Impl>(name, desc, romFilename, config))
-{
-}
-
-VLM5030::~VLM5030()
-{
-}
-
-void VLM5030::reset()
-{
-	pimpl->reset();
-}
-
-void VLM5030::writeData(byte data)
-{
-	pimpl->writeData(data);
-}
-
-void VLM5030::writeControl(byte data, EmuTime::param time)
-{
-	pimpl->writeControl(data, time);
-}
-
-bool VLM5030::getBSY(EmuTime::param time)
-{
-	return pimpl->getBSY(time);
-}
-
-template<typename Archive>
-void VLM5030::serialize(Archive& ar, unsigned version)
-{
-	pimpl->serialize(ar, version);
-}
 INSTANTIATE_SERIALIZE_METHODS(VLM5030);
 
 } // namespace openmsx
