@@ -1,33 +1,31 @@
 #include "WavAudioInput.hh"
 #include "CommandController.hh"
 #include "PlugException.hh"
-#include "FilenameSetting.hh"
 #include "CliComm.hh"
 #include "serialize.hh"
-#include "memory.hh"
 
 using std::string;
 
 namespace openmsx {
 
 WavAudioInput::WavAudioInput(CommandController& commandController)
-	: audioInputFilenameSetting(make_unique<FilenameSetting>(
+	: audioInputFilenameSetting(
 		commandController, "audio-inputfilename",
 		"filename of the file where the sampler reads data from",
-		"audio-input.wav"))
-	, reference(EmuTime::zero)
+		"audio-input.wav")
+	, reference(EmuTime::zero())
 {
-	audioInputFilenameSetting->attach(*this);
+	audioInputFilenameSetting.attach(*this);
 }
 
 WavAudioInput::~WavAudioInput()
 {
-	audioInputFilenameSetting->detach(*this);
+	audioInputFilenameSetting.detach(*this);
 }
 
 void WavAudioInput::loadWave()
 {
-	wav = WavData(audioInputFilenameSetting->getString(), 16, 0);
+	wav = WavData(string(audioInputFilenameSetting.getString()));
 }
 
 const string& WavAudioInput::getName() const
@@ -36,7 +34,7 @@ const string& WavAudioInput::getName() const
 	return name;
 }
 
-string_ref WavAudioInput::getDescription() const
+std::string_view WavAudioInput::getDescription() const
 {
 	return "Read .wav files. Can for example be used as input for "
 		"samplers.";
@@ -49,8 +47,7 @@ void WavAudioInput::plugHelper(Connector& /*connector*/, EmuTime::param time)
 			loadWave();
 		}
 	} catch (MSXException& e) {
-		throw PlugException("Load of wave file failed: " +
-		                    e.getMessage());
+		throw PlugException("Load of wave file failed: ", e.getMessage());
 	}
 	reference = time;
 }
@@ -62,7 +59,7 @@ void WavAudioInput::unplugHelper(EmuTime::param /*time*/)
 void WavAudioInput::update(const Setting& setting)
 {
 	(void)setting;
-	assert(&setting == audioInputFilenameSetting.get());
+	assert(&setting == &audioInputFilenameSetting);
 	try {
 		if (isPluggedIn()) {
 			loadWave();
@@ -70,18 +67,15 @@ void WavAudioInput::update(const Setting& setting)
 	} catch (MSXException& e) {
 		// TODO proper error handling, message should go to console
 		setting.getCommandController().getCliComm().printWarning(
-			"Load of wave file failed: " + e.getMessage());
+			"Load of wave file failed: ", e.getMessage());
 	}
 }
 
-short WavAudioInput::readSample(EmuTime::param time)
+int16_t WavAudioInput::readSample(EmuTime::param time)
 {
 	if (wav.getSize()) {
 		unsigned pos = (time - reference).getTicksAt(wav.getFreq());
-		if (pos < wav.getSize()) {
-			auto buf = static_cast<const short*>(wav.getData());
-			return buf[pos];
-		}
+		return wav.getSample(pos);
 	}
 	return 0;
 }
@@ -91,7 +85,7 @@ void WavAudioInput::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.serialize("reference", reference);
 	if (ar.isLoader()) {
-		update(*audioInputFilenameSetting);
+		update(audioInputFilenameSetting);
 	}
 }
 INSTANTIATE_SERIALIZE_METHODS(WavAudioInput);
