@@ -12,7 +12,7 @@
 #include "StringOp.hh"
 #include "DeviceConfig.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include <memory>
 #include <string>
 
 using std::string;
@@ -44,7 +44,7 @@ public:
 	PanasonicAudioPeriphery(
 		MSXAudio& audio, const DeviceConfig& config,
 		const string& soundDeviceName);
-	~PanasonicAudioPeriphery();
+	~PanasonicAudioPeriphery() override;
 
 	void reset() override;
 
@@ -53,8 +53,8 @@ public:
 
 	byte peekMem(word address, EmuTime::param time) const override;
 	void writeMem(word address, byte value, EmuTime::param time) override;
-	const byte* getReadCacheLine(word start) const override;
-	byte* getWriteCacheLine(word start) const override;
+	const byte* getReadCacheLine(word address) const override;
+	byte* getWriteCacheLine(word address) const override;
 
 	template<typename Archive>
 	void serialize(Archive& ar, unsigned version);
@@ -66,8 +66,8 @@ private:
 
 	MSXAudio& audio;
 	BooleanSetting swSwitch;
-	const std::unique_ptr<Ram> ram;
-	const std::unique_ptr<Rom> rom;
+	Ram ram;
+	Rom rom;
 	byte bankSelect;
 	byte ioPorts;
 };
@@ -117,11 +117,11 @@ void Y8950Periphery::writeMem(word /*address*/, byte /*value*/, EmuTime::param /
 {
 	// nothing
 }
-const byte* Y8950Periphery::getReadCacheLine(word /*start*/) const
+const byte* Y8950Periphery::getReadCacheLine(word /*address*/) const
 {
 	return MSXDevice::unmappedRead;
 }
-byte* Y8950Periphery::getWriteCacheLine(word /*start*/) const
+byte* Y8950Periphery::getWriteCacheLine(word /*address*/) const
 {
 	return MSXDevice::unmappedWrite;
 }
@@ -138,7 +138,7 @@ void MusicModulePeriphery::write(nibble outputs, nibble values,
                                  EmuTime::param time)
 {
 	nibble actual = (outputs & values) | (~outputs & read(time));
-	audio.y8950->setEnabled((actual & 8) != 0, time);
+	audio.y8950.setEnabled((actual & 8) != 0, time);
 	audio.enableDAC((actual & 1) != 0, time);
 }
 
@@ -164,11 +164,9 @@ PanasonicAudioPeriphery::PanasonicAudioPeriphery(
 	           "software of this module must be started or not.",
 	           false)
 	// note: name + " RAM"  already taken by sample RAM
-	, ram(make_unique<Ram>(
-		config, audio.getName() + " mapped RAM",
-		"MSX-AUDIO mapped RAM", 0x1000))
-	, rom(make_unique<Rom>(
-		audio.getName() + " ROM", "MSX-AUDIO ROM", config))
+	, ram(config, audio.getName() + " mapped RAM",
+	      "MSX-AUDIO mapped RAM", 0x1000)
+	, rom(audio.getName() + " ROM", "MSX-AUDIO ROM", config)
 	, ioPorts(0)
 {
 	reset();
@@ -181,7 +179,7 @@ PanasonicAudioPeriphery::~PanasonicAudioPeriphery()
 
 void PanasonicAudioPeriphery::reset()
 {
-	ram->clear(); // TODO check
+	ram.clear(); // TODO check
 	setBank(0);
 	setIOPorts(0); // TODO check: neither IO port ranges active
 }
@@ -190,7 +188,7 @@ void PanasonicAudioPeriphery::write(nibble outputs, nibble values,
                                     EmuTime::param time)
 {
 	nibble actual = (outputs & values) | (~outputs & read(time));
-	audio.y8950->setEnabled(!(actual & 8), time);
+	audio.y8950.setEnabled(!(actual & 8), time);
 }
 
 nibble PanasonicAudioPeriphery::read(EmuTime::param /*time*/)
@@ -202,18 +200,18 @@ nibble PanasonicAudioPeriphery::read(EmuTime::param /*time*/)
 byte PanasonicAudioPeriphery::peekMem(word address, EmuTime::param /*time*/) const
 {
 	if ((bankSelect == 0) && ((address & 0x3FFF) >= 0x3000)) {
-		return (*ram)[(address & 0x3FFF) - 0x3000];
+		return ram[(address & 0x3FFF) - 0x3000];
 	} else {
-		return (*rom)[0x8000 * bankSelect + (address & 0x7FFF)];
+		return rom[0x8000 * bankSelect + (address & 0x7FFF)];
 	}
 }
 
 const byte* PanasonicAudioPeriphery::getReadCacheLine(word address) const
 {
 	if ((bankSelect == 0) && ((address & 0x3FFF) >= 0x3000)) {
-		return &(*ram)[(address & 0x3FFF) - 0x3000];
+		return &ram[(address & 0x3FFF) - 0x3000];
 	} else {
-		return &(*rom)[0x8000 * bankSelect + (address & 0x7FFF)];
+		return &rom[0x8000 * bankSelect + (address & 0x7FFF)];
 	}
 }
 
@@ -227,7 +225,7 @@ void PanasonicAudioPeriphery::writeMem(word address, byte value, EmuTime::param
 	}
 	address &= 0x3FFF;
 	if ((bankSelect == 0) && (address >= 0x3000)) {
-		(*ram)[address - 0x3000] = value;
+		ram[address - 0x3000] = value;
 	}
 }
 
@@ -239,7 +237,7 @@ byte* PanasonicAudioPeriphery::getWriteCacheLine(word address) const
 	}
 	address &= 0x3FFF;
 	if ((bankSelect == 0) && (address >= 0x3000)) {
-		return const_cast<byte*>(&(*ram)[address - 0x3000]);
+		return const_cast<byte*>(&ram[address - 0x3000]);
 	} else {
 		return MSXDevice::unmappedWrite;
 	}
@@ -248,7 +246,7 @@ byte* PanasonicAudioPeriphery::getWriteCacheLine(word address) const
 void PanasonicAudioPeriphery::setBank(byte value)
 {
 	bankSelect = value & 3;
-	audio.getCPU().invalidateMemCache(0x0000, 0x10000);
+	audio.getCPU().invalidateAllSlotsRWCache(0x0000, 0x10000);
 }
 
 void PanasonicAudioPeriphery::setIOPorts(byte value)
@@ -281,8 +279,8 @@ void PanasonicAudioPeriphery::setIOPortsHelper(unsigned base, bool enable)
 template<typename Archive>
 void PanasonicAudioPeriphery::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("ram", *ram);
-	ar.serialize("bankSelect", bankSelect);
+	ar.serialize("ram",        ram,
+	             "bankSelect", bankSelect);
 	byte tmpIoPorts = ioPorts;
 	ar.serialize("ioPorts", tmpIoPorts);
 	if (ar.isLoader()) {
@@ -299,7 +297,7 @@ ToshibaAudioPeriphery::ToshibaAudioPeriphery(MSXAudio& audio_)
 }
 
 void ToshibaAudioPeriphery::write(nibble /*outputs*/, nibble /*values*/,
-                                    EmuTime::param /*time*/)
+                                  EmuTime::param /*time*/)
 {
 	// TODO IO1-IO0 are programmed as output by HX-MU900 software rom
 	//      and it writes periodically the values 1/1/2/2/0/0 to
@@ -315,7 +313,7 @@ nibble ToshibaAudioPeriphery::read(EmuTime::param /*time*/)
 
 void ToshibaAudioPeriphery::setSPOFF(bool value, EmuTime::param time)
 {
-	audio.y8950->setEnabled(!value, time);
+	audio.y8950.setEnabled(!value, time);
 }
 
 
@@ -327,14 +325,14 @@ std::unique_ptr<Y8950Periphery> Y8950PeripheryFactory::create(
 {
 	string type(StringOp::toLower(config.getChildData("type", "philips")));
 	if (type == "philips") {
-		return make_unique<MusicModulePeriphery>(audio);
+		return std::make_unique<MusicModulePeriphery>(audio);
 	} else if (type == "panasonic") {
-		return make_unique<PanasonicAudioPeriphery>(
+		return std::make_unique<PanasonicAudioPeriphery>(
 			audio, config, soundDeviceName);
 	} else if (type == "toshiba") {
-		return make_unique<ToshibaAudioPeriphery>(audio);
+		return std::make_unique<ToshibaAudioPeriphery>(audio);
 	} else {
-		throw MSXException("Unknown MSX-AUDIO type: " + type);
+		throw MSXException("Unknown MSX-AUDIO type: ", type);
 	}
 }
 
