@@ -5,223 +5,49 @@
 ******************************************************************************/
 
 #include "YM2151.hh"
-#include "ResampledSoundDevice.hh"
-#include "EmuTimer.hh"
-#include "IRQHelper.hh"
 #include "DeviceConfig.hh"
+#include "Math.hh"
+#include "cstd.hh"
+#include "ranges.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include <cmath>
 #include <cstring>
 
 namespace openmsx {
 
-class YM2151::Impl final : public ResampledSoundDevice, private EmuTimerCallback
-{
-public:
-	Impl(const std::string& name, const std::string& desc,
-	     const DeviceConfig& config, EmuTime::param time);
-	~Impl();
-	void reset(EmuTime::param time);
-	void writeReg(byte r, byte v, EmuTime::param time);
-	byte readStatus() const;
-
-	template<typename Archive>
-	void serialize(Archive& ar, unsigned version);
-
-private:
-	// a single operator
-	struct YM2151Operator {
-		template<typename Archive>
-		void serialize(Archive& ar, unsigned version);
-
-		int* connect;      // operator output 'direction'
-		int* mem_connect;  // where to put the delayed sample (MEM)
-
-		unsigned phase;    // accumulated operator phase
-		unsigned freq;     // operator frequency count
-		int dt1;           // current DT1 (detune 1 phase inc/decrement) value
-		unsigned mul;      // frequency count multiply
-		unsigned dt1_i;    // DT1 index * 32
-		unsigned dt2;      // current DT2 (detune 2) value
-
-		int mem_value;     // delayed sample (MEM) value
-
-		// channel specific data
-		// note: each operator number 0 contains channel specific data
-		unsigned fb_shift; // feedback shift value for operators 0 in each channel
-		int fb_out_curr;   // operator feedback value (used only by operators 0)
-		int fb_out_prev;   // previous feedback value (used only by operators 0)
-		unsigned kc;       // channel KC (copied to all operators)
-		unsigned kc_i;     // just for speedup
-		unsigned pms;      // channel PMS
-		unsigned ams;      // channel AMS
-
-		unsigned AMmask;   // LFO Amplitude Modulation enable mask
-		unsigned state;    // Envelope state: 4-attack(AR)
-		                   //                 3-decay(D1R)
-		                   //                 2-sustain(D2R)
-		                   //                 1-release(RR)
-		                   //                 0-off
-		unsigned tl;       // Total attenuation Level
-		int volume;        // current envelope attenuation level
-		unsigned d1l;      // envelope switches to sustain state after
-
-		unsigned key;      // 0=last key was KEY OFF, 1=last key was KEY ON
-
-		unsigned ks;       // key scale
-		unsigned ar;       // attack rate
-		unsigned d1r;      // decay rate
-		unsigned d2r;      // sustain rate
-		unsigned rr;       // release rate
-
-		byte eg_sh_ar;     //  (attack state)
-		byte eg_sel_ar;    //  (attack state)
-		byte eg_sh_d1r;    //  (decay state)
-		byte eg_sel_d1r;   //  (decay state)
-		                   // reaching this level
-		byte eg_sh_d2r;    //  (sustain state)
-		byte eg_sel_d2r;   //  (sustain state)
-		byte eg_sh_rr;     //  (release state)
-		byte eg_sel_rr;    //  (release state)
-	};
-
-	void setConnect(YM2151Operator* om1, int cha, int v);
-
-	// SoundDevice
-	void generateChannels(int** bufs, unsigned num) override;
-
-	void callback(byte flag) override;
-	void setStatus(byte flags);
-	void resetStatus(byte flags);
-
-	void initTables();
-	void initChipTables();
-
-	// operator methods
-	void envelopeKONKOFF(YM2151Operator* op, int v);
-	static void refreshEG(YM2151Operator* op);
-	int opCalc(YM2151Operator* op, unsigned env, int pm);
-	int opCalc1(YM2151Operator* op, unsigned env, int pm);
-	inline unsigned volumeCalc(YM2151Operator* op, unsigned AM);
-	inline void keyOn(YM2151Operator* op, unsigned keySet);
-	inline void keyOff(YM2151Operator* op, unsigned keyClear);
-
-	// general chip mehods
-	void chanCalc(unsigned chan);
-	void chan7Calc();
-
-	void advanceEG();
-	void advance();
-
-	bool checkMuteHelper();
-
-	IRQHelper irq;
-
-	// Timers (see EmuTimer class for details about timing)
-	const std::unique_ptr<EmuTimer> timer1;
-	const std::unique_ptr<EmuTimer> timer2;
-
-	YM2151Operator oper[32]; // the 32 operators
-
-	unsigned pan[16];        // channels output masks (0xffffffff = enable)
-
-	unsigned eg_cnt;         // global envelope generator counter
-	unsigned eg_timer;       // global envelope generator counter
-	                         //   works at frequency = chipclock/64/3
-	unsigned lfo_phase;      // accumulated LFO phase (0 to 255)
-	unsigned lfo_timer;      // LFO timer
-	unsigned lfo_overflow;   // LFO generates new output when lfo_timer
-	                         // reaches this value
-	unsigned lfo_counter;    // LFO phase increment counter
-	unsigned lfo_counter_add;// step of lfo_counter
-	unsigned lfa;            // LFO current AM output
-	int lfp;                 // LFO current PM output
-
-	unsigned noise;          // noise enable/period register
-	                         // bit 7 - noise enable, bits 4-0 - noise period
-	unsigned noise_rng;      // 17 bit noise shift register
-	int noise_p;             // current noise 'phase'
-	unsigned noise_f;        // current noise period
-
-	unsigned csm_req;        // CSM  KEY ON / KEY OFF sequence request
-
-	unsigned irq_enable;     // IRQ enable for timer B (bit 3) and timer A
-	                         // (bit 2); bit 7 - CSM mode (keyon to all
-	                         // slots, everytime timer A overflows)
-	unsigned status;         // chip status (BUSY, IRQ Flags)
-
-	// Frequency-deltas to get the closest frequency possible.
-	// There are 11 octaves because of DT2 (max 950 cents over base frequency)
-	// and LFO phase modulation (max 800 cents below AND over base frequency)
-	// Summary:   octave  explanation
-	//             0       note code - LFO PM
-	//             1       note code
-	//             2       note code
-	//             3       note code
-	//             4       note code
-	//             5       note code
-	//             6       note code
-	//             7       note code
-	//             8       note code
-	//             9       note code + DT2 + LFO PM
-	//            10       note code + DT2 + LFO PM
-	unsigned freq[11 * 768]; // 11 octaves, 768 'cents' per octave   // No Save
-
-	// Frequency deltas for DT1. These deltas alter operator frequency
-	// after it has been taken from frequency-deltas table.
-	int dt1_freq[8 * 32];    // 8 DT1 levels, 32 KC values         // No Save
-	unsigned noise_tab[32];  // 17bit Noise Generator periods      // No Save
-
-	int chanout[8];
-	int m2, c1, c2;          // Phase Modulation input for operators 2,3,4
-	int mem;                 // one sample delay memory
-
-	word timer_A_val;
-
-	byte lfo_wsel;           // LFO waveform (0-saw, 1-square, 2-triangle,
-	                         //               3-random noise)
-	byte amd;                // LFO Amplitude Modulation Depth
-	signed char pmd;         // LFO Phase Modulation Depth
-
-	byte test;               // TEST register
-	byte ct;                 // output control pins (bit1-CT2, bit0-CT1)
-
-	byte regs[256];          // only used for serialization ATM
-};
 // TODO void ym2151WritePortCallback(void* ref, unsigned port, byte value);
 
-static const int FREQ_SH  = 16; // 16.16 fixed point (frequency calculations)
+constexpr int FREQ_SH  = 16; // 16.16 fixed point (frequency calculations)
 
-static const int FREQ_MASK = (1 << FREQ_SH) - 1;
+constexpr int FREQ_MASK = (1 << FREQ_SH) - 1;
 
-static const int ENV_BITS = 10;
-static const int ENV_LEN  = 1 << ENV_BITS;
-static const double ENV_STEP = 128.0 / ENV_LEN;
+constexpr int ENV_BITS = 10;
+constexpr int ENV_LEN  = 1 << ENV_BITS;
+constexpr float ENV_STEP = 128.0f / ENV_LEN;
 
-static const int MAX_ATT_INDEX = ENV_LEN - 1; // 1023
-static const int MIN_ATT_INDEX = 0;
+constexpr int MAX_ATT_INDEX = ENV_LEN - 1; // 1023
+constexpr int MIN_ATT_INDEX = 0;
 
-static const unsigned EG_ATT = 4;
-static const unsigned EG_DEC = 3;
-static const unsigned EG_SUS = 2;
-static const unsigned EG_REL = 1;
-static const unsigned EG_OFF = 0;
+constexpr unsigned EG_ATT = 4;
+constexpr unsigned EG_DEC = 3;
+constexpr unsigned EG_SUS = 2;
+constexpr unsigned EG_REL = 1;
+constexpr unsigned EG_OFF = 0;
 
-static const int SIN_BITS = 10;
-static const int SIN_LEN  = 1 << SIN_BITS;
-static const int SIN_MASK = SIN_LEN - 1;
+constexpr int SIN_BITS = 10;
+constexpr int SIN_LEN  = 1 << SIN_BITS;
+constexpr int SIN_MASK = SIN_LEN - 1;
 
-static const int TL_RES_LEN = 256; // 8 bits addressing (real chip)
+constexpr int TL_RES_LEN = 256; // 8 bits addressing (real chip)
 
 // TL_TAB_LEN is calculated as:
 //  13 - sinus amplitude bits     (Y axis)
 //  2  - sinus sign bit           (Y axis)
 // TL_RES_LEN - sinus resolution (X axis)
-static const unsigned TL_TAB_LEN = 13 * 2 * TL_RES_LEN;
+constexpr unsigned TL_TAB_LEN = 13 * 2 * TL_RES_LEN;
 static int tl_tab[TL_TAB_LEN];
 
-static const unsigned ENV_QUIET = TL_TAB_LEN >> 3;
+constexpr unsigned ENV_QUIET = TL_TAB_LEN >> 3;
 
 // sin waveform table in 'decibel' scale
 static unsigned sin_tab[SIN_LEN];
@@ -230,8 +56,8 @@ static unsigned sin_tab[SIN_LEN];
 static unsigned d1l_tab[16];
 
 
-static const unsigned RATE_STEPS = 8;
-static byte eg_inc[19 * RATE_STEPS] = {
+constexpr unsigned RATE_STEPS = 8;
+constexpr byte eg_inc[19 * RATE_STEPS] = {
 
 //cycle:0 1  2 3  4 5  6 7
 
@@ -261,9 +87,9 @@ static byte eg_inc[19 * RATE_STEPS] = {
 };
 
 
-#define O(a) (a*RATE_STEPS)
+static constexpr byte O(int a) { return a * RATE_STEPS; }
 // note that there is no O(17) in this table - it's directly in the code
-static byte eg_rate_select[32 + 64 + 32] = {
+constexpr byte eg_rate_select[32 + 64 + 32] = {
 // Envelope Generator rates (32 + 64 rates + 32 RKS)
 // 32 dummy (infinite time) rates
 O(18),O(18),O(18),O(18),O(18),O(18),O(18),O(18),
@@ -303,53 +129,50 @@ O(16),O(16),O(16),O(16),O(16),O(16),O(16),O(16),
 O(16),O(16),O(16),O(16),O(16),O(16),O(16),O(16),
 O(16),O(16),O(16),O(16),O(16),O(16),O(16),O(16)
 };
-#undef O
 
 // rate  0,    1,    2,   3,   4,   5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15
 // shift 11,   10,   9,   8,   7,   6,  5,  4,  3,  2, 1,  0,  0,  0,  0,  0
 // mask  2047, 1023, 511, 255, 127, 63, 31, 15, 7,  3, 1,  0,  0,  0,  0,  0
-#define O(a) (a*1)
-static byte eg_rate_shift[32 + 64 + 32] = {
+constexpr byte eg_rate_shift[32 + 64 + 32] = {
 // Envelope Generator counter shifts (32 + 64 rates + 32 RKS)
 // 32 infinite time rates
-O(0),O(0),O(0),O(0),O(0),O(0),O(0),O(0),
-O(0),O(0),O(0),O(0),O(0),O(0),O(0),O(0),
-O(0),O(0),O(0),O(0),O(0),O(0),O(0),O(0),
-O(0),O(0),O(0),O(0),O(0),O(0),O(0),O(0),
+   0,  0,  0,  0,  0,  0,  0,  0,
+   0,  0,  0,  0,  0,  0,  0,  0,
+   0,  0,  0,  0,  0,  0,  0,  0,
+   0,  0,  0,  0,  0,  0,  0,  0,
 
 // rates 00-11
-O(11),O(11),O(11),O(11),
-O(10),O(10),O(10),O(10),
-O( 9),O( 9),O( 9),O( 9),
-O( 8),O( 8),O( 8),O( 8),
-O( 7),O( 7),O( 7),O( 7),
-O( 6),O( 6),O( 6),O( 6),
-O( 5),O( 5),O( 5),O( 5),
-O( 4),O( 4),O( 4),O( 4),
-O( 3),O( 3),O( 3),O( 3),
-O( 2),O( 2),O( 2),O( 2),
-O( 1),O( 1),O( 1),O( 1),
-O( 0),O( 0),O( 0),O( 0),
+  11, 11, 11, 11,
+  10, 10, 10, 10,
+   9,  9,  9,  9,
+   8,  8,  8,  8,
+   7,  7,  7,  7,
+   6,  6,  6,  6,
+   5,  5,  5,  5,
+   4,  4,  4,  4,
+   3,  3,  3,  3,
+   2,  2,  2,  2,
+   1,  1,  1,  1,
+   0,  0,  0,  0,
 
 // rate 12
-O( 0),O( 0),O( 0),O( 0),
+   0,  0,  0,  0,
 
 // rate 13
-O( 0),O( 0),O( 0),O( 0),
+   0,  0,  0,  0,
 
 // rate 14
-O( 0),O( 0),O( 0),O( 0),
+   0,  0,  0,  0,
 
 // rate 15
-O( 0),O( 0),O( 0),O( 0),
+   0,  0,  0,  0,
 
 // 32 dummy rates (same as 15 3)
-O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),
-O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),
-O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),
-O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),O( 0)
+   0,  0,  0,  0,  0,  0,  0,  0,
+   0,  0,  0,  0,  0,  0,  0,  0,
+   0,  0,  0,  0,  0,  0,  0,  0,
+   0,  0,  0,  0,  0,  0,  0,  0
 };
-#undef O
 
 // DT2 defines offset in cents from base note
 //
@@ -360,12 +183,12 @@ O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),O( 0),O( 0)
 //
 // DT2=0 DT2=1 DT2=2 DT2=3
 // 0     600   781   950
-static unsigned dt2_tab[4] = { 0, 384, 500, 608 };
+constexpr unsigned dt2_tab[4] = { 0, 384, 500, 608 };
 
 // DT1 defines offset in Hertz from base note
 // This table is converted while initialization...
 // Detune table shown in YM2151 User's Manual is wrong (verified on the real chip)
-static byte dt1_tab[4 * 32] = {
+constexpr byte dt1_tab[4 * 32] = {
 // DT1 = 0
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -383,7 +206,7 @@ static byte dt1_tab[4 * 32] = {
   8, 8, 9,10,11,12,13,14,16,17,19,20,22,22,22,22
 };
 
-static word phaseinc_rom[768] = {
+constexpr word phaseinc_rom[768] = {
 1299,1300,1301,1302,1303,1304,1305,1306,1308,1309,1310,1311,1313,1314,1315,1316,
 1318,1319,1320,1321,1322,1323,1324,1325,1327,1328,1329,1330,1332,1333,1334,1335,
 1337,1338,1339,1340,1341,1342,1343,1344,1346,1347,1348,1349,1351,1352,1353,1354,
@@ -448,7 +271,7 @@ static word phaseinc_rom[768] = {
 // possible that two values: 0x80 and 0x00 might be wrong in this table.
 // To be exact:
 // some 0x80 could be 0x81 as well as some 0x00 could be 0x01.
-static byte lfo_noise_waveform[256] = {
+constexpr byte lfo_noise_waveform[256] = {
 0xFF,0xEE,0xD3,0x80,0x58,0xDA,0x7F,0x94,0x9E,0xE3,0xFA,0x00,0x4D,0xFA,0xFF,0x6A,
 0x7A,0xDE,0x49,0xF6,0x00,0x33,0xBB,0x63,0x91,0x60,0x51,0xFF,0x00,0xD8,0x7F,0xDE,
 0xDC,0x73,0x21,0x85,0xB2,0x9C,0x5D,0x24,0xCD,0x91,0x9E,0x76,0x7F,0x20,0xFB,0xF3,
@@ -470,11 +293,11 @@ static byte lfo_noise_waveform[256] = {
 0xE2,0x4D,0x8A,0xA6,0x46,0x95,0x0F,0x8F,0xF5,0x15,0x97,0x32,0xD4,0x28,0x1E,0x55
 };
 
-void YM2151::Impl::initTables()
+void YM2151::initTables()
 {
 	for (int x = 0; x < TL_RES_LEN; ++x) {
-		double m = (1 << 16) / pow(2, (x + 1) * (ENV_STEP / 4.0) / 8.0);
-		m = floor(m);
+		float m = (1 << 16) / exp2f((x + 1) * (ENV_STEP / 4.0f) / 8.0f);
+		m = floorf(m);
 
 		// we never reach (1 << 16) here due to the (x + 1)
 		// result fits within 16 bits at maximum
@@ -497,37 +320,32 @@ void YM2151::Impl::initTables()
 		}
 	}
 
+	static const float LOG2 = log(2.0);
 	for (int i = 0; i < SIN_LEN; ++i) {
 		// non-standard sinus
-		double m = sin((i * 2 + 1) * M_PI / SIN_LEN); // verified on the real chip
+		float m = sinf((i * 2 + 1) * M_PI / SIN_LEN); // verified on the real chip
 
 		// we never reach zero here due to (i * 2 + 1)
-		double o;
-		if (m > 0.0) { // convert to decibels
-			o = 8 * log( 1.0 / m) / log(2.0);
-		} else {
-			o = 8 * log(-1.0 / m) / log(2.0);
-		}
+		float o = -8.0f * logf(std::abs(m)) / LOG2; // convert to decibels
 		o = o / (ENV_STEP / 4);
 
-		int n = int(2.0 * o);
+		int n = int(2.0f * o);
 		if (n & 1) { // round to closest
 			n = (n >> 1) + 1;
 		} else {
 			n = n >> 1;
 		}
-		sin_tab[i] = n * 2 + (m >= 0.0 ? 0 : 1);
+		sin_tab[i] = n * 2 + (m >= 0.0f ? 0 : 1);
 	}
 
 	// calculate d1l_tab table
 	for (int i = 0; i < 16; ++i) {
 		// every 3 'dB' except for all bits = 1 = 45+48 'dB'
-		double m = unsigned((i != 15 ? i : i + 16) * (4.0 / ENV_STEP));
-		d1l_tab[i] = unsigned(m);
+		d1l_tab[i] = unsigned((i != 15 ? i : i + 16) * (4.0f / ENV_STEP));
 	}
 }
 
-void YM2151::Impl::initChipTables()
+void YM2151::initChipTables()
 {
 	// this loop calculates Hertz values for notes from c-0 to b-7
 	// including 64 'cents' (100/64 that is 1.5625 of real cent) per note
@@ -535,10 +353,10 @@ void YM2151::Impl::initChipTables()
 
 	// real chip works with 10 bits fixed point values (10.10)
 	//   -10 because phaseinc_rom table values are already in 10.10 format
-	double mult = 1 << (FREQ_SH - 10);
+	float mult = 1 << (FREQ_SH - 10);
 
 	for (int i = 0; i < 768; ++i) {
-		double phaseinc = phaseinc_rom[i]; // real chip phase increment
+		float phaseinc = phaseinc_rom[i]; // real chip phase increment
 
 		// octave 2 - reference octave
 		//   adjust to X.10 fixed point
@@ -571,7 +389,7 @@ void YM2151::Impl::initChipTables()
 		for (int i = 0; i < 32; ++i) {
 
 			// calculate phase increment
-			double phaseinc = double(dt1_tab[j * 32 + i]) / (1 << 20) * (SIN_LEN);
+			float phaseinc = float(dt1_tab[j * 32 + i]) / (1 << 20) * (SIN_LEN);
 
 			// positive and negative values
 			dt1_freq[(j + 0) * 32 + i] = int(phaseinc * mult);
@@ -589,7 +407,7 @@ void YM2151::Impl::initChipTables()
 	}
 }
 
-void YM2151::Impl::keyOn(YM2151Operator* op, unsigned keySet) {
+void YM2151::keyOn(YM2151Operator* op, unsigned keySet) {
 	if (!op->key) {
 		op->phase = 0; /* clear phase */
 		op->state = EG_ATT; /* KEY ON = attack */
@@ -604,7 +422,7 @@ void YM2151::Impl::keyOn(YM2151Operator* op, unsigned keySet) {
 	op->key |= keySet;
 }
 
-void YM2151::Impl::keyOff(YM2151Operator* op, unsigned keyClear) {
+void YM2151::keyOff(YM2151Operator* op, unsigned keyClear) {
 	if (op->key) {
 		op->key &= keyClear;
 		if (!op->key) {
@@ -615,7 +433,7 @@ void YM2151::Impl::keyOff(YM2151Operator* op, unsigned keyClear) {
 	}
 }
 
-void YM2151::Impl::envelopeKONKOFF(YM2151Operator* op, int v)
+void YM2151::envelopeKONKOFF(YM2151Operator* op, int v)
 {
 	if (v & 0x08) { // M1
 		keyOn (op + 0, 1);
@@ -639,7 +457,7 @@ void YM2151::Impl::envelopeKONKOFF(YM2151Operator* op, int v)
 	}
 }
 
-void YM2151::Impl::setConnect(YM2151Operator* om1, int cha, int v)
+void YM2151::setConnect(YM2151Operator* om1, int cha, int v)
 {
 	YM2151Operator* om2 = om1 + 1;
 	YM2151Operator* oc1 = om1 + 2;
@@ -727,7 +545,7 @@ void YM2151::Impl::setConnect(YM2151Operator* om1, int cha, int v)
 	}
 }
 
-void YM2151::Impl::refreshEG(YM2151Operator* op)
+void YM2151::refreshEG(YM2151Operator* op)
 {
 	unsigned kc = op->kc;
 
@@ -796,7 +614,7 @@ void YM2151::Impl::refreshEG(YM2151Operator* op)
 	op->eg_sel_rr  = eg_rate_select[op->rr  + v];
 }
 
-void YM2151::Impl::writeReg(byte r, byte v, EmuTime::param time)
+void YM2151::writeReg(byte r, byte v, EmuTime::param time)
 {
 	updateStream(time);
 
@@ -1012,9 +830,11 @@ void YM2151::Impl::writeReg(byte r, byte v, EmuTime::param time)
 	}
 }
 
-YM2151::Impl::Impl(const std::string& name, const std::string& desc,
-                   const DeviceConfig& config, EmuTime::param time)
-	: ResampledSoundDevice(config.getMotherBoard(), name, desc, 8, true)
+constexpr auto INPUT_RATE = unsigned(cstd::round(3579545 / 64.0));
+
+YM2151::YM2151(const std::string& name_, const std::string& desc,
+               const DeviceConfig& config, EmuTime::param time)
+	: ResampledSoundDevice(config.getMotherBoard(), name_, desc, 8, INPUT_RATE, true)
 	, irq(config.getMotherBoard(), getName() + ".IRQ")
 	, timer1(EmuTimer::createOPM_1(config.getScheduler(), *this))
 	, timer2(EmuTimer::createOPM_2(config.getScheduler(), *this))
@@ -1027,29 +847,22 @@ YM2151::Impl::Impl(const std::string& name, const std::string& desc,
 	initTables();
 	initChipTables();
 
-	static const int CLCK_FREQ = 3579545;
-	double input = CLCK_FREQ / 64.0;
-	setInputRate(int(input + 0.5));
-
 	reset(time);
 
 	registerSound(config);
 }
 
-YM2151::Impl::~Impl()
+YM2151::~YM2151()
 {
 	unregisterSound();
 }
 
-bool YM2151::Impl::checkMuteHelper()
+bool YM2151::checkMuteHelper()
 {
-	for (auto& op : oper) {
-		if (op.state != EG_OFF) return false;
-	}
-	return true;
+	return ranges::all_of(oper, [](auto& op) { return op.state == EG_OFF; });
 }
 
-void YM2151::Impl::reset(EmuTime::param time)
+void YM2151::reset(EmuTime::param time)
 {
 	// initialize hardware registers
 	for (auto& op : oper) {
@@ -1073,8 +886,8 @@ void YM2151::Impl::reset(EmuTime::param time)
 	test = 0;
 
 	irq_enable = 0;
-	timer1->setStart(0, time);
-	timer2->setStart(0, time);
+	timer1->setStart(false, time);
+	timer2->setStart(false, time);
 
 	noise     = 0;
 	noise_rng = 0;
@@ -1093,7 +906,7 @@ void YM2151::Impl::reset(EmuTime::param time)
 	irq.reset();
 }
 
-int YM2151::Impl::opCalc(YM2151Operator* OP, unsigned env, int pm)
+int YM2151::opCalc(YM2151Operator* OP, unsigned env, int pm)
 {
 	unsigned p = (env << 3) + sin_tab[(int((OP->phase & ~FREQ_MASK) + (pm << 15)) >> FREQ_SH) & SIN_MASK];
 	if (p >= TL_TAB_LEN) {
@@ -1102,7 +915,7 @@ int YM2151::Impl::opCalc(YM2151Operator* OP, unsigned env, int pm)
 	return tl_tab[p];
 }
 
-int YM2151::Impl::opCalc1(YM2151Operator* OP, unsigned env, int pm)
+int YM2151::opCalc1(YM2151Operator* OP, unsigned env, int pm)
 {
 	int i = (OP->phase & ~FREQ_MASK) + pm;
 	unsigned p = (env << 3) + sin_tab[(i >> FREQ_SH) & SIN_MASK];
@@ -1112,12 +925,12 @@ int YM2151::Impl::opCalc1(YM2151Operator* OP, unsigned env, int pm)
 	return tl_tab[p];
 }
 
-unsigned YM2151::Impl::volumeCalc(YM2151Operator* OP, unsigned AM)
+unsigned YM2151::volumeCalc(YM2151Operator* OP, unsigned AM)
 {
 	return OP->tl + unsigned(OP->volume) + (AM & OP->AMmask);
 }
 
-void YM2151::Impl::chanCalc(unsigned chan)
+void YM2151::chanCalc(unsigned chan)
 {
 	m2 = c1 = c2 = mem = 0;
 	YM2151Operator* op = &oper[chan*4]; // M1
@@ -1163,7 +976,7 @@ void YM2151::Impl::chanCalc(unsigned chan)
 	op->mem_value = mem;
 }
 
-void YM2151::Impl::chan7Calc()
+void YM2151::chan7Calc()
 {
 	m2 = c1 = c2 = mem = 0;
 	YM2151Operator* op = &oper[7 * 4]; // M1
@@ -1423,7 +1236,7 @@ rate 11 1         |
                                  --
 */
 
-void YM2151::Impl::advanceEG()
+void YM2151::advanceEG()
 {
 	if (eg_timer++ != 3) {
 		// envelope generator timer overlfows every 3 samples (on real chip)
@@ -1479,7 +1292,7 @@ void YM2151::Impl::advanceEG()
 	}
 }
 
-void YM2151::Impl::advance()
+void YM2151::advance()
 {
 	// LFO
 	if (test & 2) {
@@ -1637,7 +1450,7 @@ void YM2151::Impl::advance()
 	}
 }
 
-void YM2151::Impl::generateChannels(int** bufs, unsigned num)
+void YM2151::generateChannels(float** bufs, unsigned num)
 {
 	if (checkMuteHelper()) {
 		// TODO update internal state, even if muted
@@ -1658,14 +1471,14 @@ void YM2151::Impl::generateChannels(int** bufs, unsigned num)
 		chan7Calc(); // special case for channel 7
 
 		for (int j = 0; j < 8; ++j) {
-			bufs[j][2 * i + 0] += chanout[j] & pan[2 * j + 0];
-			bufs[j][2 * i + 1] += chanout[j] & pan[2 * j + 1];
+			bufs[j][2 * i + 0] += int(chanout[j] & pan[2 * j + 0]);
+			bufs[j][2 * i + 1] += int(chanout[j] & pan[2 * j + 1]);
 		}
 		advance();
 	}
 }
 
-void YM2151::Impl::callback(byte flag)
+void YM2151::callback(byte flag)
 {
 	if (flag & 0x20) { // Timer 1
 		if (irq_enable & 0x04) {
@@ -1682,12 +1495,12 @@ void YM2151::Impl::callback(byte flag)
 	}
 }
 
-byte YM2151::Impl::readStatus() const
+byte YM2151::readStatus() const
 {
 	return status;
 }
 
-void YM2151::Impl::setStatus(byte flags)
+void YM2151::setStatus(byte flags)
 {
 	status |= flags;
 	if (status) {
@@ -1695,7 +1508,7 @@ void YM2151::Impl::setStatus(byte flags)
 	}
 }
 
-void YM2151::Impl::resetStatus(byte flags)
+void YM2151::resetStatus(byte flags)
 {
 	status &= ~flags;
 	if (!status) {
@@ -1705,83 +1518,83 @@ void YM2151::Impl::resetStatus(byte flags)
 
 
 template<typename Archive>
-void YM2151::Impl::YM2151Operator::serialize(Archive& ar, unsigned /*version*/)
+void YM2151::YM2151Operator::serialize(Archive& a, unsigned /*version*/)
 {
 	//int* connect; // recalculated from regs[0x20-0x27]
 	//int* mem_connect; // recalculated from regs[0x20-0x27]
-	ar.serialize("phase", phase);
-	ar.serialize("freq", freq);
-	ar.serialize("dt1", dt1);
-	ar.serialize("mul", mul);
-	ar.serialize("dt1_i", dt1_i);
-	ar.serialize("dt2", dt2);
-	ar.serialize("mem_value", mem_value);
-	//ar.serialize("fb_shift", fb_shift); // recalculated from regs[0x20-0x27]
-	ar.serialize("fb_out_curr", fb_out_curr);
-	ar.serialize("fb_out_prev", fb_out_prev);
-	ar.serialize("kc", kc);
-	ar.serialize("kc_i", kc_i);
-	ar.serialize("pms", pms);
-	ar.serialize("ams", ams);
-	ar.serialize("AMmask", AMmask);
-	ar.serialize("state", state);
-	ar.serialize("tl", tl);
-	ar.serialize("volume", volume);
-	ar.serialize("d1l", d1l);
-	ar.serialize("key", key);
-	ar.serialize("ks", ks);
-	ar.serialize("ar", this->ar);
-	ar.serialize("d1r", d1r);
-	ar.serialize("d2r", d2r);
-	ar.serialize("rr", rr);
-	ar.serialize("eg_sh_ar", eg_sh_ar);
-	ar.serialize("eg_sel_ar", eg_sel_ar);
-	ar.serialize("eg_sh_d1r", eg_sh_d1r);
-	ar.serialize("eg_sel_d1r", eg_sel_d1r);
-	ar.serialize("eg_sh_d2r", eg_sh_d2r);
-	ar.serialize("eg_sel_d2r", eg_sel_d2r);
-	ar.serialize("eg_sh_rr", eg_sh_rr);
-	ar.serialize("eg_sel_rr", eg_sel_rr);
+	a.serialize("phase",       phase,
+	            "freq",        freq,
+	            "dt1",         dt1,
+	            "mul",         mul,
+	            "dt1_i",       dt1_i,
+	            "dt2",         dt2,
+	            "mem_value",   mem_value,
+	            //"fb_shift",    fb_shift, // recalculated from regs[0x20-0x27]
+	            "fb_out_curr", fb_out_curr,
+	            "fb_out_prev", fb_out_prev,
+	            "kc",          kc,
+	            "kc_i",        kc_i,
+	            "pms",         pms,
+	            "ams",         ams,
+	            "AMmask",      AMmask,
+	            "state",       state,
+	            "tl",          tl,
+	            "volume",      volume,
+	            "d1l",         d1l,
+	            "key",         key,
+	            "ks",          ks,
+	            "ar",          ar,
+	            "d1r",         d1r,
+	            "d2r",         d2r,
+	            "rr",          rr,
+	            "eg_sh_ar",    eg_sh_ar,
+	            "eg_sel_ar",   eg_sel_ar,
+	            "eg_sh_d1r",   eg_sh_d1r,
+	            "eg_sel_d1r",  eg_sel_d1r,
+	            "eg_sh_d2r",   eg_sh_d2r,
+	            "eg_sel_d2r",  eg_sel_d2r,
+	            "eg_sh_rr",    eg_sh_rr,
+	            "eg_sel_rr",   eg_sel_rr);
 };
 
 template<typename Archive>
-void YM2151::Impl::serialize(Archive& ar, unsigned /*version*/)
+void YM2151::serialize(Archive& a, unsigned /*version*/)
 {
-	ar.serialize("irq", irq);
-	ar.serialize("timer1", *timer1);
-	ar.serialize("timer2", *timer2);
-	ar.serialize("operators", oper);
-	//ar.serialize("pan", pan); // recalculated from regs[0x20-0x27]
-	ar.serialize("eg_cnt", eg_cnt);
-	ar.serialize("eg_timer", eg_timer);
-	ar.serialize("lfo_phase", lfo_phase);
-	ar.serialize("lfo_timer", lfo_timer);
-	ar.serialize("lfo_overflow", lfo_overflow);
-	ar.serialize("lfo_counter", lfo_counter);
-	ar.serialize("lfo_counter_add", lfo_counter_add);
-	ar.serialize("lfa", lfa);
-	ar.serialize("lfp", lfp);
-	ar.serialize("noise", noise);
-	ar.serialize("noise_rng", noise_rng);
-	ar.serialize("noise_p", noise_p);
-	ar.serialize("noise_f", noise_f);
-	ar.serialize("csm_req", csm_req);
-	ar.serialize("irq_enable", irq_enable);
-	ar.serialize("status", status);
-	ar.serialize("chanout", chanout);
-	ar.serialize("m2", m2);
-	ar.serialize("c1", c1);
-	ar.serialize("c2", c2);
-	ar.serialize("mem", mem);
-	ar.serialize("timer_A_val", timer_A_val);
-	ar.serialize("lfo_wsel", lfo_wsel);
-	ar.serialize("amd", amd);
-	ar.serialize("pmd", pmd);
-	ar.serialize("test", test);
-	ar.serialize("ct", ct);
-	ar.serialize_blob("registers", regs, sizeof(regs));
+	a.serialize("irq",             irq,
+	            "timer1",          *timer1,
+	            "timer2",          *timer2,
+	            "operators",       oper,
+	            //"pan",             pan, // recalculated from regs[0x20-0x27]
+	            "eg_cnt",          eg_cnt,
+	            "eg_timer",        eg_timer,
+	            "lfo_phase",       lfo_phase,
+	            "lfo_timer",       lfo_timer,
+	            "lfo_overflow",    lfo_overflow,
+	            "lfo_counter",     lfo_counter,
+	            "lfo_counter_add", lfo_counter_add,
+	            "lfa",             lfa,
+	            "lfp",             lfp,
+	            "noise",           noise,
+	            "noise_rng",       noise_rng,
+	            "noise_p",         noise_p,
+	            "noise_f",         noise_f,
+	            "csm_req",         csm_req,
+	            "irq_enable",      irq_enable,
+	            "status",          status,
+	            "chanout",         chanout,
+	            "m2",              m2,
+	            "c1",              c1,
+	            "c2",              c2,
+	            "mem",             mem,
+	            "timer_A_val",     timer_A_val,
+	            "lfo_wsel",        lfo_wsel,
+	            "amd",             amd,
+	            "pmd",             pmd,
+	            "test",            test,
+	            "ct",              ct);
+	a.serialize_blob("registers", regs, sizeof(regs));
 
-	if (ar.isLoader()) {
+	if (a.isLoader()) {
 		// TODO restore more state from registers
 		EmuTime::param time = timer1->getCurrentTime();
 		for (int r = 0x20; r < 0x28; ++r) {
@@ -1789,40 +1602,6 @@ void YM2151::Impl::serialize(Archive& ar, unsigned /*version*/)
 		}
 	}
 }
-
-
-// YM2151
-
-YM2151::YM2151(const std::string& name, const std::string& desc,
-               const DeviceConfig& config, EmuTime::param time)
-	: pimpl(make_unique<Impl>(name, desc, config, time))
-{
-}
-
-YM2151::~YM2151()
-{
-}
-
-void YM2151::reset(EmuTime::param time)
-{
-	pimpl->reset(time);
-}
-
-void YM2151::writeReg(byte r, byte v, EmuTime::param time)
-{
-	pimpl->writeReg(r, v, time);
-}
-
-byte YM2151::readStatus() const
-{
-	return pimpl->readStatus();
-}
-
-template<typename Archive>
-void YM2151::serialize(Archive& ar, unsigned version)
-{
-	pimpl->serialize(ar, version);
-}
 INSTANTIATE_SERIALIZE_METHODS(YM2151);
 
 } // namespace openmsx
