@@ -5,59 +5,338 @@
  */
 
 #include "YM2413Okazaki.hh"
-#include "serialize.hh"
+#include "Math.hh"
+#include "cstd.hh"
 #include "inline.hh"
+#include "one_of.hh"
+#include "ranges.hh"
+#include "serialize.hh"
 #include "unreachable.hh"
 #include <cstring>
 #include <cassert>
+#include <iostream>
 
 namespace openmsx {
 namespace YM2413Okazaki {
 
-// This defines the tables:
-//  - signed char pmTable[8][8]
-//  - int dB2LinTab[DBTABLEN * 2]
-//  - unsigned AR_ADJUST_TABLE[1 << EG_BITS]
-//  - byte tllTable[4][16 * 8]
-//  - unsigned* waveform[2]
-//  - int dphaseDRTable[16][16]
-//  - byte lfo_am_table[LFO_AM_TAB_ELEMENTS]
-//  - unsigned slTable[16]
-//  - byte mlTable[16]
-#include "YM2413OkazakiTable.ii"
+// Number of bits in 'PhaseModulation' fixed point type.
+constexpr int PM_FP_BITS =  8;
 
+// Dynamic range (Accuracy of sin table)
+constexpr int DB_BITS = 8;
+constexpr int DB_MUTE = 1 << DB_BITS;
+constexpr int DBTABLEN = 3 * DB_MUTE; // enough to not have to check for overflow
+
+constexpr double DB_STEP = 48.0 / DB_MUTE;
+constexpr double EG_STEP = 0.375;
+constexpr double TL_STEP = 0.75;
+
+// Size of Sintable ( 8 -- 18 can be used, but 9 recommended.)
+constexpr int PG_BITS = 9;
+constexpr int PG_WIDTH = 1 << PG_BITS;
+constexpr int PG_MASK = PG_WIDTH - 1;
+
+// Phase increment counter
+constexpr int DP_BITS = 18;
+constexpr int DP_BASE_BITS = DP_BITS - PG_BITS;
+
+// Dynamic range of envelope
+constexpr int EG_BITS = 7;
+
+// Bits for linear value
+constexpr int DB2LIN_AMP_BITS = 8;
+constexpr int SLOT_AMP_BITS = DB2LIN_AMP_BITS;
+
+// Bits for Amp modulator
+constexpr int AM_PG_BITS = 8;
+constexpr int AM_PG_WIDTH = 1 << AM_PG_BITS;
+constexpr int AM_DP_BITS = 16;
+constexpr int AM_DP_WIDTH = 1 << AM_DP_BITS;
+constexpr int AM_DP_MASK = AM_DP_WIDTH - 1;
+
+// LFO Amplitude Modulation table (verified on real YM3812)
+// 27 output levels (triangle waveform);
+// 1 level takes one of: 192, 256 or 448 samples
+//
+// Length: 210 elements.
+//  Each of the elements has to be repeated
+//  exactly 64 times (on 64 consecutive samples).
+//  The whole table takes: 64 * 210 = 13440 samples.
+//
+// Verified on real YM3812 (OPL2), but I believe it's the same for YM2413
+// because it closely matches the YM2413 AM parameters:
+//    speed = 3.7Hz
+//    depth = 4.875dB
+// Also this approch can be easily implemented in HW, the previous one (see SVN
+// history) could not.
+constexpr unsigned LFO_AM_TAB_ELEMENTS = 210;
 
 // Extra (derived) constants
-static const EnvPhaseIndex EG_DP_MAX = EnvPhaseIndex(1 << 7);
-static const EnvPhaseIndex EG_DP_INF = EnvPhaseIndex(1 << 8); // as long as it's bigger
+constexpr EnvPhaseIndex EG_DP_MAX = EnvPhaseIndex(1 << 7);
+constexpr EnvPhaseIndex EG_DP_INF = EnvPhaseIndex(1 << 8); // as long as it's bigger
 
+// LFO Phase Modulation table (copied from Burczynski core)
+constexpr signed char pmTable[8][8] =
+{
+	{ 0, 0, 0, 0, 0, 0, 0, 0, }, // FNUM = 000xxxxxx
+	{ 0, 0, 1, 0, 0, 0,-1, 0, }, // FNUM = 001xxxxxx
+	{ 0, 1, 2, 1, 0,-1,-2,-1, }, // FNUM = 010xxxxxx
+	{ 0, 1, 3, 1, 0,-1,-3,-1, }, // FNUM = 011xxxxxx
+	{ 0, 2, 4, 2, 0,-2,-4,-2, }, // FNUM = 100xxxxxx
+	{ 0, 2, 5, 2, 0,-2,-5,-2, }, // FNUM = 101xxxxxx
+	{ 0, 3, 6, 3, 0,-3,-6,-3, }, // FNUM = 110xxxxxx
+	{ 0, 3, 7, 3, 0,-3,-7,-3, }, // FNUM = 111xxxxxx
+};
+
+// LFO Amplitude Modulation table (verified on real YM3812)
+constexpr unsigned char lfo_am_table[LFO_AM_TAB_ELEMENTS] = {
+	0,0,0,0,0,0,0,
+	1,1,1,1,
+	2,2,2,2,
+	3,3,3,3,
+	4,4,4,4,
+	5,5,5,5,
+	6,6,6,6,
+	7,7,7,7,
+	8,8,8,8,
+	9,9,9,9,
+	10,10,10,10,
+	11,11,11,11,
+	12,12,12,12,
+	13,13,13,13,
+	14,14,14,14,
+	15,15,15,15,
+	16,16,16,16,
+	17,17,17,17,
+	18,18,18,18,
+	19,19,19,19,
+	20,20,20,20,
+	21,21,21,21,
+	22,22,22,22,
+	23,23,23,23,
+	24,24,24,24,
+	25,25,25,25,
+	26,26,26,
+	25,25,25,25,
+	24,24,24,24,
+	23,23,23,23,
+	22,22,22,22,
+	21,21,21,21,
+	20,20,20,20,
+	19,19,19,19,
+	18,18,18,18,
+	17,17,17,17,
+	16,16,16,16,
+	15,15,15,15,
+	14,14,14,14,
+	13,13,13,13,
+	12,12,12,12,
+	11,11,11,11,
+	10,10,10,10,
+	9,9,9,9,
+	8,8,8,8,
+	7,7,7,7,
+	6,6,6,6,
+	5,5,5,5,
+	4,4,4,4,
+	3,3,3,3,
+	2,2,2,2,
+	1,1,1,1,
+};
+
+// ML-table
+constexpr uint8_t mlTable[16] = {
+	1,   1*2,  2*2,  3*2,  4*2,  5*2,  6*2,  7*2,
+	8*2, 9*2, 10*2, 10*2, 12*2, 12*2, 15*2, 15*2
+};
+
+// dB to linear table (used by Slot)
+// dB(0 .. DB_MUTE-1) -> linear(0 .. DB2LIN_AMP_WIDTH)
+// indices in range:
+//   [0,        DB_MUTE )    actual values, from max to min
+//   [DB_MUTE,  DBTABLEN)    filled with min val (to allow some overflow in index)
+//   [DBTABLEN, 2*DBTABLEN)  as above but for negative output values
+struct Db2LinTab {
+	int tab[2 * DBTABLEN];
+};
+static constexpr Db2LinTab makeDB2LinTable()
+{
+	Db2LinTab dB2Lin = {};
+
+	for (int i = 0; i < DB_MUTE; ++i) {
+		dB2Lin.tab[i] = int(double((1 << DB2LIN_AMP_BITS) - 1) *
+		                   cstd::pow<5, 3>(10, -double(i) * DB_STEP / 20));
+	}
+	dB2Lin.tab[DB_MUTE - 1] = 0;
+	for (int i = DB_MUTE; i < DBTABLEN; ++i) {
+		dB2Lin.tab[i] = 0;
+	}
+	for (int i = 0; i < DBTABLEN; ++i) {
+		dB2Lin.tab[i + DBTABLEN] = -dB2Lin.tab[i];
+	}
+
+	return dB2Lin;
+}
+constexpr Db2LinTab dB2Lin = makeDB2LinTable();
+
+// Linear to Log curve conversion table (for Attack rate)
+struct ArAdjustTable {
+	unsigned tab[1 << EG_BITS];
+};
+static constexpr ArAdjustTable makeAdjustTable()
+{
+	ArAdjustTable arAdjust = {};
+
+	arAdjust.tab[0] = (1 << EG_BITS) - 1;
+	constexpr double l127 = cstd::log<5, 4>(127.0);
+	for (int i = 1; i < (1 << EG_BITS); ++i) {
+		arAdjust.tab[i] = unsigned(double(1 << EG_BITS) - 1 -
+		         ((1 << EG_BITS) - 1) * cstd::log<5, 4>(double(i)) / l127);
+	}
+
+	return arAdjust;
+}
+constexpr ArAdjustTable arAdjust = makeAdjustTable();
+
+// KSL + TL Table   values are in range [0, 112]
+struct TllTable {
+	uint8_t tab[4][16 * 8];
+};
+static constexpr TllTable makeTllTable()
+{
+	TllTable tll = {};
+
+	// Processed version of Table III-5 from the Application Manual.
+	constexpr unsigned kltable[16] = {
+		0, 24, 32, 37, 40, 43, 45, 47, 48, 50, 51, 52, 53, 54, 55, 56
+	};
+	// Note: KL [0..3] results in {0.0, 1.5, 3.0, 6.0} dB/oct.
+	// This is different from Y8950 and YMF262 which have {0, 3, 1.5, 6}.
+	// (2nd and 3rd elements are swapped). Verified on real YM2413.
+
+	for (unsigned freq = 0; freq < 16 * 8; ++freq) {
+		unsigned fnum = freq & 15;
+		unsigned block = freq / 16;
+		int tmp = 2 * kltable[fnum] - 16 * (7 - block);
+		for (unsigned KL = 0; KL < 4; ++KL) {
+			unsigned t = (tmp <= 0 || KL == 0) ? 0 : (tmp >> (3 - KL));
+			assert(t <= 112);
+			tll.tab[KL][freq] = t;
+		}
+	}
+
+	return tll;
+}
+constexpr TllTable tll = makeTllTable();
+
+// WaveTable for each envelope amp
+//  values are in range [0, DB_MUTE)             (for positive values)
+//                  or  [0, DB_MUTE) + DBTABLEN  (for negative values)
+struct SinTable {
+	unsigned full[PG_WIDTH];
+	unsigned half[PG_WIDTH];
+};
+static constexpr int lin2db(double d)
+{
+	// lin(+0.0 .. +1.0) to dB(DB_MUTE-1 .. 0)
+	return (d == 0)
+		? DB_MUTE - 1
+		: std::min(-int(20.0 * cstd::log10<5, 2>(d) / DB_STEP), DB_MUTE - 1); // 0 - 127
+}
+constexpr SinTable makeSinTable()
+{
+	SinTable sinTable = {};
+
+	for (int i = 0; i < PG_WIDTH / 4; ++i) {
+		sinTable.full[i] = lin2db(cstd::sin<2>(double(2.0 * M_PI) * i / PG_WIDTH));
+	}
+	for (int i = 0; i < PG_WIDTH / 4; ++i) {
+		sinTable.full[PG_WIDTH / 2 - 1 - i] = sinTable.full[i];
+	}
+	for (int i = 0; i < PG_WIDTH / 2; ++i) {
+		sinTable.full[PG_WIDTH / 2 + i] = DBTABLEN + sinTable.full[i];
+	}
+
+	for (int i = 0; i < PG_WIDTH / 2; ++i) {
+		sinTable.half[i] = sinTable.full[i];
+	}
+	for (int i = PG_WIDTH / 2; i < PG_WIDTH; ++i) {
+		sinTable.half[i] = sinTable.full[0];
+	}
+
+	return sinTable;
+}
+constexpr SinTable sinTable = makeSinTable();
+constexpr unsigned const * const waveform[2] = {sinTable.full, sinTable.half};
+
+// Phase incr table for attack, decay and release
+//  note: original code had indices swapped. It also had
+//        a separate table for attack
+//  17.15 fixed point
+struct DphaseDRTable {
+	int tab[16][16];
+};
+static constexpr DphaseDRTable makeDphaseDRTable()
+{
+	DphaseDRTable dphaseDR = {};
+
+	for (unsigned Rks = 0; Rks < 16; ++Rks) {
+		dphaseDR.tab[Rks][0] = 0;
+		for (unsigned DR = 1; DR < 16; ++DR) {
+			unsigned RM = std::min(DR + (Rks >> 2), 15u);
+			unsigned RL = Rks & 3;
+			dphaseDR.tab[Rks][DR] =
+				((RL + 4) << EP_FP_BITS) >> (16 - RM);
+		}
+	}
+
+	return dphaseDR;
+}
+constexpr DphaseDRTable dphaseDR = makeDphaseDRTable();
+
+// Sustain level (17.15 fixed point)
+struct SlTable {
+	unsigned tab[16];
+};
+static constexpr SlTable makeSusLevTable()
+{
+	SlTable sl = {};
+
+	for (int i = 0; i < 16; ++i) {
+		double x = (i == 15) ? 48.0 : (3.0 * i);
+		sl.tab[i] = int(x / EG_STEP) << EP_FP_BITS;
+	}
+
+	return sl;
+}
+constexpr SlTable sl = makeSusLevTable();
 
 //
 // Helper functions
 //
-static inline int EG2DB(int d)
+static constexpr int EG2DB(int d)
 {
 	return d * int(EG_STEP / DB_STEP);
 }
-static inline unsigned TL2EG(unsigned d)
+static constexpr unsigned TL2EG(unsigned d)
 {
 	assert(d < 64); // input is in range [0..63]
 	return d * int(TL_STEP / EG_STEP);
 }
 
-static inline unsigned DB_POS(double x)
+static constexpr unsigned DB_POS(double x)
 {
 	int result = int(x / DB_STEP);
 	assert(0 <= result);
 	assert(result < DB_MUTE);
 	return result;
 }
-static inline unsigned DB_NEG(double x)
+static constexpr unsigned DB_NEG(double x)
 {
 	return DBTABLEN + DB_POS(x);
 }
 
-static inline bool BIT(unsigned s, unsigned b)
+static constexpr bool BIT(unsigned s, unsigned b)
 {
 	return (s >> b) & 1;
 }
@@ -77,7 +356,7 @@ Patch::Patch()
 	setSL(0);
 }
 
-void Patch::initModulator(const byte* data)
+void Patch::initModulator(const uint8_t* data)
 {
 	AMPM = (data[0] >> 6) &  3;
 	EG   = (data[0] >> 5) &  1;
@@ -93,7 +372,7 @@ void Patch::initModulator(const byte* data)
 	RR   = (data[6] >> 0) & 15;
 }
 
-void Patch::initCarrier(const byte* data)
+void Patch::initCarrier(const uint8_t* data)
 {
 	AMPM = (data[1] >> 6) &  3;
 	EG   = (data[1] >> 5) &  1;
@@ -109,35 +388,35 @@ void Patch::initCarrier(const byte* data)
 	RR   = (data[7] >> 0) & 15;
 }
 
-void Patch::setKR(byte value)
+void Patch::setKR(uint8_t value)
 {
 	KR = value ? 8 : 10;
 }
-void Patch::setML(byte value)
+void Patch::setML(uint8_t value)
 {
 	ML = mlTable[value];
 }
-void Patch::setKL(byte value)
+void Patch::setKL(uint8_t value)
 {
-	KL = tllTable[value];
+	KL = tll.tab[value];
 }
-void Patch::setTL(byte value)
+void Patch::setTL(uint8_t value)
 {
 	assert(value < 64);
 	assert(TL2EG(value) < 256);
 	TL = TL2EG(value);
 }
-void Patch::setWF(byte value)
+void Patch::setWF(uint8_t value)
 {
 	WF = waveform[value];
 }
-void Patch::setFB(byte value)
+void Patch::setFB(uint8_t value)
 {
 	FB = value ? 8 - value : 0;
 }
-void Patch::setSL(byte value)
+void Patch::setSL(uint8_t value)
 {
-	SL = slTable[value];
+	SL = sl.tab[value];
 }
 
 
@@ -147,11 +426,11 @@ void Patch::setSL(byte value)
 void Slot::reset()
 {
 	cphase = 0;
-	for (auto& dp : dphase) dp = 0;
+	ranges::fill(dphase, 0);
 	output = 0;
 	feedback = 0;
 	setEnvelopeState(FINISH);
-	dphaseDRTableRks = dphaseDRTable[0];
+	dphaseDRTableRks = dphaseDR.tab[0];
 	tll = 0;
 	sustain = false;
 	volume = TL2EG(0);
@@ -183,7 +462,7 @@ void Slot::updateRKS(unsigned freq)
 {
 	unsigned rks = freq >> patch.KR;
 	assert(rks < 16);
-	dphaseDRTableRks = dphaseDRTable[rks];
+	dphaseDRTableRks = dphaseDR.tab[rks];
 }
 
 void Slot::updateEG()
@@ -297,16 +576,16 @@ void Slot::slotOff()
 {
 	if (state == FINISH) return; // already in off state
 	if (state == ATTACK) {
-		eg_phase = EnvPhaseIndex(AR_ADJUST_TABLE[eg_phase.toInt()]);
+		eg_phase = EnvPhaseIndex(arAdjust.tab[eg_phase.toInt()]);
 	}
 	setEnvelopeState(RELEASE);
 }
 
 
 // Change a rhythm voice
-void Slot::setPatch(Patch& patch)
+void Slot::setPatch(const Patch& newPatch)
 {
-	this->patch = patch; // copy data
+	patch = newPatch; // copy data
 	if ((state == SUSHOLD) && (patch.EG == 0)) {
 		setEnvelopeState(SUSTAIN);
 	}
@@ -386,7 +665,7 @@ void Channel::keyOff()
 // YM2413
 //
 
-static byte inst_data[16 + 3][8] = {
+static uint8_t inst_data[16 + 3][8] = {
 	{ 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 }, // user instrument
 	{ 0x61,0x61,0x1e,0x17,0xf0,0x7f,0x00,0x17 }, // violin
 	{ 0x13,0x41,0x16,0x0e,0xfd,0xf4,0x23,0x23 }, // guitar
@@ -410,6 +689,38 @@ static byte inst_data[16 + 3][8] = {
 
 YM2413::YM2413()
 {
+	if (false) {
+		for (auto& e : dB2Lin.tab) std::cout << e << ' ';
+		std::cout << '\n';
+
+		for (auto& e : arAdjust.tab) std::cout << e << ' ';
+		std::cout << '\n';
+
+		for (int i = 0; i < 4; ++i) {
+			for (int j = 0; j < 16 * 8; ++j) {
+				std::cout << int(tll.tab[i][j]) << ' ';
+			}
+			std::cout << '\n';
+		}
+		std::cout << '\n';
+
+		for (auto& e : sinTable.full) std::cout << e << ' ';
+		std::cout << '\n';
+		for (auto& e : sinTable.half) std::cout << e << ' ';
+		std::cout << '\n';
+
+		for (int i = 0; i < 16; ++i) {
+			for (int j = 0; j < 16; ++j) {
+				std::cout << dphaseDR.tab[i][j] << ' ';
+			}
+			std::cout << '\n';
+		}
+		std::cout << '\n';
+
+		for (auto& e : sl.tab) std::cout << e << ' ';
+		std::cout << '\n';
+	}
+
 	memset(reg, 0, sizeof(reg)); // avoid UMR
 
 	for (unsigned i = 0; i < 16 + 3; ++i) {
@@ -433,6 +744,7 @@ void YM2413::reset()
 	for (unsigned i = 0; i < 0x40; ++i) {
 		writeReg(i, 0);
 	}
+	registerLatch = 0;
 }
 
 // Drum key on
@@ -533,14 +845,14 @@ void YM2413::keyOff_CYM()
 	}
 }
 
-void YM2413::setRhythmFlags(byte old)
+void YM2413::setRhythmFlags(uint8_t old)
 {
 	Channel& ch6 = channels[6];
 	Channel& ch7 = channels[7];
 	Channel& ch8 = channels[8];
 
 	// flags = X | X | mode | BD | SD | TOM | TC | HH
-	byte flags = reg[0x0E];
+	uint8_t flags = reg[0x0E];
 	if ((flags ^ old) & 0x20) {
 		if (flags & 0x20) {
 			// OFF -> ON
@@ -602,7 +914,7 @@ void YM2413::update_key_status()
 }
 
 // Convert Amp(0 to EG_HEIGHT) to Phase(0 to 8PI)
-static inline int wave2_8pi(int e)
+static constexpr int wave2_8pi(int e)
 {
 	int shift = SLOT_AMP_BITS - PG_BITS - 2;
 	if (shift > 0) {
@@ -653,7 +965,7 @@ void Slot::calc_envelope_outline(unsigned& out)
 template <bool HAS_AM, bool FIXED_ENV>
 ALWAYS_INLINE unsigned Slot::calc_envelope(int lfo_am, unsigned fixed_env)
 {
-	assert(!FIXED_ENV || (state == SUSHOLD) || (state == FINISH));
+	assert(!FIXED_ENV || (state == one_of(SUSHOLD, FINISH)));
 
 	if (FIXED_ENV) {
 		unsigned out = fixed_env;
@@ -667,7 +979,7 @@ ALWAYS_INLINE unsigned Slot::calc_envelope(int lfo_am, unsigned fixed_env)
 	} else {
 		unsigned out = eg_phase.toInt(); // in range [0, 128)
 		if (state == ATTACK) {
-			out = AR_ADJUST_TABLE[out]; // [0, 128)
+			out = arAdjust.tab[out]; // [0, 128)
 		}
 		eg_phase += eg_dphase;
 		if (eg_phase >= eg_phase_max) {
@@ -682,7 +994,7 @@ ALWAYS_INLINE unsigned Slot::calc_envelope(int lfo_am, unsigned fixed_env)
 }
 template <bool HAS_AM> unsigned Slot::calc_fixed_env() const
 {
-	assert((state == SUSHOLD) || (state == FINISH));
+	assert(state == one_of(SUSHOLD, FINISH));
 	assert(eg_dphase == EnvPhaseIndex(0));
 	unsigned out = eg_phase.toInt(); // in range [0, 128)
 	out = EG2DB(out + tll); // [0, 480)
@@ -698,7 +1010,7 @@ ALWAYS_INLINE int Slot::calc_slot_car(unsigned lfo_pm, int lfo_am, int fm, unsig
 {
 	int phase = calc_phase(lfo_pm) + wave2_8pi(fm);
 	unsigned egout = calc_envelope<HAS_AM, FIXED_ENV>(lfo_am, fixed_env);
-	int newOutput = dB2LinTab[patch.WF[phase & PG_MASK] + egout];
+	int newOutput = dB2Lin.tab[patch.WF[phase & PG_MASK] + egout];
 	output = (output + newOutput) >> 1;
 	return output;
 }
@@ -713,7 +1025,7 @@ ALWAYS_INLINE int Slot::calc_slot_mod(unsigned lfo_pm, int lfo_am, unsigned fixe
 	if (HAS_FB) {
 		phase += wave2_8pi(feedback) >> patch.FB;
 	}
-	int newOutput = dB2LinTab[patch.WF[phase & PG_MASK] + egout];
+	int newOutput = dB2Lin.tab[patch.WF[phase & PG_MASK] + egout];
 	feedback = (output + newOutput) >> 1;
 	output = newOutput;
 	return feedback;
@@ -724,7 +1036,7 @@ ALWAYS_INLINE int Slot::calc_slot_tom()
 {
 	unsigned phase = calc_phase(0);
 	unsigned egout = calc_envelope<false, false>(0, 0);
-	return dB2LinTab[patch.WF[phase & PG_MASK] + egout];
+	return dB2Lin.tab[patch.WF[phase & PG_MASK] + egout];
 }
 
 // SNARE (ch7 car)
@@ -733,8 +1045,8 @@ ALWAYS_INLINE int Slot::calc_slot_snare(bool noise)
 	unsigned phase = calc_phase(0);
 	unsigned egout = calc_envelope<false, false>(0, 0);
 	return BIT(phase, 7)
-		? dB2LinTab[(noise ? DB_POS(0.0) : DB_POS(15.0)) + egout]
-		: dB2LinTab[(noise ? DB_NEG(0.0) : DB_NEG(15.0)) + egout];
+		? dB2Lin.tab[(noise ? DB_POS(0.0) : DB_POS(15.0)) + egout]
+		: dB2Lin.tab[(noise ? DB_NEG(0.0) : DB_NEG(15.0)) + egout];
 }
 
 // TOP-CYM (ch8 car)
@@ -748,7 +1060,7 @@ ALWAYS_INLINE int Slot::calc_slot_cym(unsigned phase7, unsigned phase8)
 	                   !BIT(phase8, PG_BITS - 5)))
 	               ? DB_NEG(3.0)
 	               : DB_POS(3.0);
-	return dB2LinTab[dbout + egout];
+	return dB2Lin.tab[dbout + egout];
 }
 
 // HI-HAT (ch7 mod)
@@ -762,12 +1074,12 @@ ALWAYS_INLINE int Slot::calc_slot_hat(unsigned phase7, unsigned phase8, bool noi
 	                   !BIT(phase8, PG_BITS - 5)))
 	               ? (noise ? DB_NEG(12.0) : DB_NEG(24.0))
 	               : (noise ? DB_POS(12.0) : DB_POS(24.0));
-	return dB2LinTab[dbout + egout];
+	return dB2Lin.tab[dbout + egout];
 }
 
-int YM2413::getAmplificationFactor() const
+float YM2413::getAmplificationFactor() const
 {
-	return 1 << (15 - DB2LIN_AMP_BITS);
+	return 1.0f / (1 << DB2LIN_AMP_BITS);
 }
 
 bool YM2413::isRhythm() const
@@ -788,7 +1100,7 @@ Patch& YM2413::getPatch(unsigned instrument, bool carrier)
 }
 
 template <unsigned FLAGS>
-ALWAYS_INLINE void YM2413::calcChannel(Channel& ch, int* buf, unsigned num)
+ALWAYS_INLINE void YM2413::calcChannel(Channel& ch, float* buf, unsigned num)
 {
 	// VC++ requires explicit conversion to bool. Compiler bug??
 	const bool HAS_CAR_PM = (FLAGS &  1) != 0;
@@ -835,14 +1147,14 @@ ALWAYS_INLINE void YM2413::calcChannel(Channel& ch, int* buf, unsigned num)
 			lfo_am = lfo_am_table[tmp_am_phase / 64];
 		}
 		int fm = ch.mod.calc_slot_mod<HAS_MOD_AM, HAS_MOD_FB, HAS_MOD_FIXED_ENV>(
-		                      lfo_pm, lfo_am, mod_fixed_env);
+		                      HAS_MOD_PM ? lfo_pm : 0, lfo_am, mod_fixed_env);
 		buf[sample] += ch.car.calc_slot_car<HAS_CAR_AM, HAS_CAR_FIXED_ENV>(
-		                      lfo_pm, lfo_am, fm, car_fixed_env);
+		                      HAS_CAR_PM ? lfo_pm : 0, lfo_am, fm, car_fixed_env);
 		++sample;
 	} while (sample < num);
 }
 
-void YM2413::generateChannels(int* bufs[9 + 5], unsigned num)
+void YM2413::generateChannels(float* bufs[9 + 5], unsigned num)
 {
 	assert(num != 0);
 
@@ -853,10 +1165,8 @@ void YM2413::generateChannels(int* bufs[9 + 5], unsigned num)
 			// Below we choose between 128 specialized versions of
 			// calcChannel(). This allows to move a lot of
 			// conditional code out of the inner-loop.
-			bool carFixedEnv = (ch.car.state == SUSHOLD) ||
-			                   (ch.car.state == FINISH);
-			bool modFixedEnv = (ch.mod.state == SUSHOLD) ||
-			                   (ch.mod.state == FINISH);
+			bool carFixedEnv = ch.car.state == one_of(SUSHOLD, FINISH);
+			bool modFixedEnv = ch.mod.state == one_of(SUSHOLD, FINISH);
 			if (ch.car.state == SETTLE) {
 				modFixedEnv = false;
 			}
@@ -1085,7 +1395,21 @@ void YM2413::generateChannels(int* bufs[9 + 5], unsigned num)
 	}
 }
 
-void YM2413::writeReg(byte r, byte data)
+void YM2413::writePort(bool port, uint8_t value, int /*offset*/)
+{
+	if (port == 0) {
+		registerLatch = value;
+	} else {
+		writeReg(registerLatch & 0x3f, value);
+	}
+}
+
+void YM2413::pokeReg(uint8_t r, uint8_t data)
+{
+	writeReg(r, data);
+}
+
+void YM2413::writeReg(uint8_t r, uint8_t data)
 {
 	assert(r < 0x40);
 
@@ -1228,7 +1552,7 @@ void YM2413::writeReg(byte r, byte data)
 		break;
 	}
 	case 0x0E: {
-		byte old = reg[r];
+		uint8_t old = reg[r];
 		reg[r] = data;
 		setRhythmFlags(old);
 		break;
@@ -1236,7 +1560,7 @@ void YM2413::writeReg(byte r, byte data)
 	case 0x19: case 0x1A: case 0x1B: case 0x1C:
 	case 0x1D: case 0x1E: case 0x1F:
 		r -= 9; // verified on real YM2413
-		// fall-through
+		[[fallthrough]];
 	case 0x10: case 0x11: case 0x12: case 0x13: case 0x14:
 	case 0x15: case 0x16: case 0x17: case 0x18: {
 		reg[r] = data;
@@ -1251,7 +1575,7 @@ void YM2413::writeReg(byte r, byte data)
 	case 0x29: case 0x2A: case 0x2B: case 0x2C:
 	case 0x2D: case 0x2E: case 0x2F:
 		r -= 9; // verified on real YM2413
-		// fall-through
+		[[fallthrough]];
 	case 0x20: case 0x21: case 0x22: case 0x23: case 0x24:
 	case 0x25: case 0x26: case 0x27: case 0x28: {
 		reg[r] = data;
@@ -1272,7 +1596,7 @@ void YM2413::writeReg(byte r, byte data)
 	case 0x39: case 0x3A: case 0x3B: case 0x3C:
 	case 0x3D: case 0x3E: case 0x3F:
 		r -= 9; // verified on real YM2413
-		// fall-through
+		[[fallthrough]];
 	case 0x30: case 0x31: case 0x32: case 0x33: case 0x34:
 	case 0x35: case 0x36: case 0x37: case 0x38: {
 		reg[r] = data;
@@ -1298,14 +1622,14 @@ void YM2413::writeReg(byte r, byte data)
 	}
 }
 
-byte YM2413::peekReg(byte r) const
+uint8_t YM2413::peekReg(uint8_t r) const
 {
 	return reg[r];
 }
 
 } // namespace YM2413Okazaki
 
-static enum_string<YM2413Okazaki::EnvelopeState> envelopeStateInfo[] = {
+static std::initializer_list<enum_string<YM2413Okazaki::EnvelopeState>> envelopeStateInfo = {
 	{ "ATTACK",  YM2413Okazaki::ATTACK  },
 	{ "DECAY",   YM2413Okazaki::DECAY   },
 	{ "SUSHOLD", YM2413Okazaki::SUSHOLD },
@@ -1326,12 +1650,12 @@ namespace YM2413Okazaki {
 template<typename Archive>
 void Slot::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("feedback", feedback);
-	ar.serialize("output", output);
-	ar.serialize("cphase", cphase);
-	ar.serialize("state", state);
-	ar.serialize("eg_phase", eg_phase);
-	ar.serialize("sustain", sustain);
+	ar.serialize("feedback", feedback,
+	             "output",   output,
+	             "cphase",   cphase,
+	             "state",    state,
+	             "eg_phase", eg_phase,
+	             "sustain",  sustain);
 
 	// These are restored by calls to
 	//  updateAll():         eg_dphase, dphaseDRTableRks, tll, dphase
@@ -1346,14 +1670,15 @@ void Slot::serialize(Archive& ar, unsigned /*version*/)
 template<typename Archive>
 void Channel::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("mod", mod);
-	ar.serialize("car", car);
+	ar.serialize("mod", mod,
+	             "car", car);
 }
 
 
 // version 1: initial version
 // version 2: 'registers' are moved here (no longer serialized in base class)
 // version 3: no longer serialize 'user_patch_mod' and 'user_patch_car'
+// version 4: added 'registerLatch'
 template<typename Archive>
 void YM2413::serialize(Archive& ar, unsigned version)
 {
@@ -1363,10 +1688,10 @@ void YM2413::serialize(Archive& ar, unsigned version)
 
 	// no need to serialize patches[]
 	//   patches[0] is restored from registers, the others are read-only
-	ar.serialize("channels", channels);
-	ar.serialize("pm_phase", pm_phase);
-	ar.serialize("am_phase", am_phase);
-	ar.serialize("noise_seed", noise_seed);
+	ar.serialize("channels",   channels,
+	             "pm_phase",   pm_phase,
+	             "am_phase",   am_phase,
+	             "noise_seed", noise_seed);
 
 	if (ar.isLoader()) {
 		patches[0][0].initModulator(&reg[0]);
@@ -1393,6 +1718,11 @@ void YM2413::serialize(Archive& ar, unsigned version)
 		}
 		update_key_status();
 	}
+	if (ar.versionAtLeast(version, 4)) {
+		ar.serialize("registerLatch", registerLatch);
+	} else {
+		// could be restored from MSXMusicBase, worth the effort?
+	}
 }
 
 } // namespace YM2413Okazaki
