@@ -1,17 +1,16 @@
 #include "Base64.hh"
-#include "xrange.hh"
+#include "likely.hh"
 #include <algorithm>
 #include <cassert>
 
 namespace Base64 {
 
 using std::string;
+using openmsx::MemBuffer;
 
-typedef unsigned char byte;
-
-static inline char encode(byte c)
+static inline char encode(uint8_t c)
 {
-	static const char* const base64_chars =
+	static constexpr const char* const base64_chars =
 		"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 		"abcdefghijklmnopqrstuvwxyz"
 		"0123456789+/";
@@ -19,7 +18,7 @@ static inline char encode(byte c)
 	return base64_chars[c];
 }
 
-static inline byte decode(unsigned char c)
+static inline uint8_t decode(uint8_t c)
 {
 	if        ('A' <= c && c <= 'Z') {
 		return c - 'A';
@@ -32,17 +31,16 @@ static inline byte decode(unsigned char c)
 	} else if (c == '/') {
 		return 63;
 	} else {
-		return byte(-1);
+		return uint8_t(-1);
 	}
 }
 
-string encode(const void* input_, size_t inSize)
+string encode(const uint8_t* input, size_t inSize)
 {
-	static const int CHUNKS = 19;
-	static const int IN_CHUNKS  = 3 * CHUNKS;
-	static const int OUT_CHUNKS = 4 * CHUNKS; // 76 chars per line
+	constexpr int CHUNKS = 19;
+	constexpr int IN_CHUNKS  = 3 * CHUNKS;
+	constexpr int OUT_CHUNKS = 4 * CHUNKS; // 76 chars per line
 
-	auto input = static_cast<const byte*>(input_);
 	auto outSize = ((inSize + (IN_CHUNKS - 1)) / IN_CHUNKS) * (OUT_CHUNKS + 1); // overestimation
 	string ret(outSize, 0); // too big
 
@@ -61,11 +59,11 @@ string encode(const void* input_, size_t inSize)
 			input += 3;
 		}
 		if (n) {
-			byte buf3[3] = { 0, 0, 0 };
+			uint8_t buf3[3] = { 0, 0, 0 };
 			for (unsigned i = 0; i < n; ++i) {
 				buf3[i] = input[i];
 			}
-			byte buf4[4];
+			uint8_t buf4[4];
 			buf4[0] =  (buf3[0] & 0xfc) >> 2;
 			buf4[1] = ((buf3[0] & 0x03) << 4) +
 				  ((buf3[1] & 0xf0) >> 4);
@@ -87,18 +85,17 @@ string encode(const void* input_, size_t inSize)
 	return ret;
 }
 
-string decode(const string& input)
+std::pair<MemBuffer<uint8_t>, size_t> decode(std::string_view input)
 {
-	auto inSize = input.size();
-	auto outSize = (inSize * 3 + 3) / 4; // overestimation
-	string ret(outSize, 0); // too big
+	auto outSize = (input.size() * 3 + 3) / 4; // overestimation
+	MemBuffer<uint8_t> ret(outSize); // too big
 
 	unsigned i = 0;
 	size_t out = 0;
-	byte buf4[4];
-	for (auto in : xrange(inSize)) {
-		byte d = decode(input[in]);
-		if (d == byte(-1)) continue;
+	uint8_t buf4[4];
+	for (auto c : input) {
+		uint8_t d = decode(c);
+		if (d == uint8_t(-1)) continue;
 		buf4[i++] = d;
 		if (i == 4) {
 			i = 0;
@@ -111,7 +108,7 @@ string decode(const string& input)
 		for (unsigned j = i; j < 4; ++j) {
 			buf4[j] = 0;
 		}
-		byte buf3[3];
+		uint8_t buf3[3];
 		buf3[0] = ((buf4[0] & 0xff) << 2) + ((buf4[1] & 0x30) >> 4);
 		buf3[1] = ((buf4[1] & 0x0f) << 4) + ((buf4[2] & 0x3c) >> 2);
 		buf3[2] = ((buf4[2] & 0x03) << 6) + ((buf4[3] & 0xff) >> 0);
@@ -122,7 +119,41 @@ string decode(const string& input)
 
 	assert(outSize >= out);
 	ret.resize(out); // shrink to correct size
-	return ret;
+	return std::pair(std::move(ret), out);
+}
+
+bool decode_inplace(std::string_view input, uint8_t* output, size_t outSize)
+{
+	unsigned i = 0;
+	size_t out = 0;
+	uint8_t buf4[4];
+	for (auto c : input) {
+		uint8_t d = decode(c);
+		if (d == uint8_t(-1)) continue;
+		buf4[i++] = d;
+		if (i == 4) {
+			i = 0;
+			if (unlikely((out + 3) > outSize)) return false;
+			output[out++] = char(((buf4[0] & 0xff) << 2) + ((buf4[1] & 0x30) >> 4));
+			output[out++] = char(((buf4[1] & 0x0f) << 4) + ((buf4[2] & 0x3c) >> 2));
+			output[out++] = char(((buf4[2] & 0x03) << 6) + ((buf4[3] & 0xff) >> 0));
+		}
+	}
+	if (i) {
+		for (unsigned j = i; j < 4; ++j) {
+			buf4[j] = 0;
+		}
+		uint8_t buf3[3];
+		buf3[0] = ((buf4[0] & 0xff) << 2) + ((buf4[1] & 0x30) >> 4);
+		buf3[1] = ((buf4[1] & 0x0f) << 4) + ((buf4[2] & 0x3c) >> 2);
+		buf3[2] = ((buf4[2] & 0x03) << 6) + ((buf4[3] & 0xff) >> 0);
+		for (unsigned j = 0; (j < i - 1); ++j) {
+			if (unlikely(out == outSize)) return false;
+			output[out++] = buf3[j];
+		}
+	}
+
+	return out == outSize;
 }
 
 } // namespace Base64
