@@ -1,12 +1,8 @@
 #include "Date.hh"
 #include <sstream>
 #include <iomanip>
-#include <cstdio>
-#include <cstring>
 
-namespace openmsx {
-
-namespace Date {
+namespace openmsx::Date {
 
 const char* const days[7] = {
 	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
@@ -168,6 +164,4 @@ std::string toString(time_t time)
 	return sstr.str();
 }
 
-} // namespace Date
-
-} // namespace openmsx
+} // namespace openmsx::Date
