@@ -1,20 +1,9 @@
 #include "DivModBySame.hh"
+#include "Math.hh"
 #include "uint128.hh"
-#include <cassert>
 
 namespace openmsx {
 
-static uint32_t log2(uint64_t i)
-{
-	uint32_t t = 0;
-	i >>= 1;
-	while (i) {
-		i >>= 1;
-		++t;
-	}
-	return t;
-}
-
 void DivModBySame::setDivisor(uint32_t divisor_)
 {
 	//assert(divisor_ < 0x8000000000000000ull); // when divisor is uint64_t
@@ -35,7 +24,7 @@ void DivModBySame::setDivisor(uint32_t divisor_)
 		// Generate m, s for algorithm 0. Based on: Granlund, T.; Montgomery,
 		// P.L.: "Division by Invariant Integers using Multiplication".
 		// SIGPLAN Notices, Vol. 29, June 1994, page 61.
-		uint32_t l = log2(t) + 1;
+		uint32_t l = Math::log2p1(t);
 		uint64_t j = 0xffffffffffffffffull % t;
 		uint128 k = (uint128(1) << (64 + l)) / (0xffffffffffffffffull - j);
 		uint128 m_low  =  (uint128(1) << (64 + l))      / t;
@@ -46,17 +35,17 @@ void DivModBySame::setDivisor(uint32_t divisor_)
 			--l;
 		}
 		if ((m_high >> 64) == 0) {
-			m = toUint64(m_high);
+			m = low64(m_high);
 			s = l;
 			a = 0;
 		} else {
 			// Generate m, s for algorithm 1. Based on: Magenheimer, D.J.; et al:
 			// "Integer Multiplication and Division on the HP Precision Architecture".
 			// IEEE Transactions on Computers, Vol 37, No. 8, August 1988, page 980.
-			s = log2(t);
-			uint128 m_low =     (uint128(1) << (64 + s)) / t;
-			uint64_t r = toUint64((uint128(1) << (64 + s)) % t);
-			m = toUint64(m_low + ((r <= (t >> 1)) ? 0 : 1));
+			s = Math::log2p1(t) - 1;
+			uint128 m_low2 =   (uint128(1) << (64 + s)) / t;
+			uint64_t r = low64((uint128(1) << (64 + s)) % t);
+			m = low64(m_low2 + ((r <= (t >> 1)) ? 0 : 1));
 			a = m;
 		}
 		// reduce multiplier to smallest possible
