@@ -1,18 +1,16 @@
 #include "SerializeBuffer.hh"
 #include "likely.hh"
-#include <new> // for bad_alloc
 #include <cstdlib>
+#include <utility>
 
 namespace openmsx {
 
 // class OutputBuffer
 
-size_t OutputBuffer::lastSize = 50000; // initial estimate
-
 OutputBuffer::OutputBuffer()
-	: begin(static_cast<byte*>(malloc(lastSize)))
-	, end(begin)
-	, finish(begin + lastSize)
+	: buf(lastSize)
+	, end(buf.data())
+	, finish(buf.data() + lastSize)
 {
 	// We've allocated a buffer with an estimated initial size. This
 	// estimate is based on the largest intermediate size of the previously
@@ -22,9 +20,6 @@ OutputBuffer::OutputBuffer()
 	// performance. If later we discover the buffer is too small, we have
 	// to reallocate (and thus make a copy). In profiling this reallocation
 	// step was noticable.
-	if (!begin) {
-		throw std::bad_alloc();
-	}
 
 	// Slowly drop the estimated required size. This makes sure that when
 	// we've overestimated the size once, we don't forever keep this too
@@ -33,15 +28,10 @@ OutputBuffer::OutputBuffer()
 	lastSize -= lastSize >> 7;
 }
 
-OutputBuffer::~OutputBuffer()
-{
-	free(begin);
-}
-
 #ifdef __GNUC__
-template<size_t LEN> void OutputBuffer::insertN(const void* __restrict data) __restrict
+template<size_t LEN> void OutputBuffer::insertN(const void* __restrict data)
 {
-	byte* newEnd = end + LEN;
+	uint8_t* newEnd = end + LEN;
 	if (likely(newEnd <= finish)) {
 		memcpy(end, data, LEN);
 		end = newEnd;
@@ -50,15 +40,15 @@ template<size_t LEN> void OutputBuffer::insertN(const void* __restrict data) __r
 	}
 }
 // Force template instantiation
-template void OutputBuffer::insertN<1>(const void* __restrict data) __restrict;
-template void OutputBuffer::insertN<2>(const void* __restrict data) __restrict;
-template void OutputBuffer::insertN<4>(const void* __restrict data) __restrict;
-template void OutputBuffer::insertN<8>(const void* __restrict data) __restrict;
+template void OutputBuffer::insertN<1>(const void* __restrict data);
+template void OutputBuffer::insertN<2>(const void* __restrict data);
+template void OutputBuffer::insertN<4>(const void* __restrict data);
+template void OutputBuffer::insertN<8>(const void* __restrict data);
 #endif
 
-void OutputBuffer::insertN(const void* __restrict data, size_t len) __restrict
+void OutputBuffer::insertN(const void* __restrict data, size_t len)
 {
-	byte* newEnd = end + len;
+	uint8_t* newEnd = end + len;
 	if (likely(newEnd <= finish)) {
 		memcpy(end, data, len);
 		end = newEnd;
@@ -67,41 +57,45 @@ void OutputBuffer::insertN(const void* __restrict data, size_t len) __restrict
 	}
 }
 
-byte* OutputBuffer::release(size_t& size)
+MemBuffer<uint8_t> OutputBuffer::release(size_t& size)
 {
-	size = end - begin;
+	size = end - buf.data();
 
 	// Deallocate unused buffer space.
-	byte* result = static_cast<byte*>(realloc(begin, size));
+	buf.resize(size);
 
-	begin = end = finish = nullptr;
+	end = finish = nullptr;
+	return std::move(buf);
+}
+
+void OutputBuffer::grow(size_t len)
+{
+	size_t oldSize = end - buf.data();
+	size_t newSize = std::max(oldSize + len, oldSize + oldSize / 2);
+	buf.resize(newSize);
+	end    = buf.data() + oldSize;
+	finish = buf.data() + newSize;
+}
+
+uint8_t* OutputBuffer::allocateGrow(size_t len)
+{
+	grow(len);
+	auto* result = end;
+	end += len;
 	return result;
 }
 
-void OutputBuffer::insertGrow(const void* __restrict data, size_t len) __restrict
+void OutputBuffer::insertGrow(const void* __restrict data, size_t len)
 {
-	byte* pos = allocateGrow(len);
+	uint8_t* pos = allocateGrow(len);
 	memcpy(pos, data, len);
 }
 
-byte* OutputBuffer::allocateGrow(size_t len) __restrict
-{
-	size_t oldSize = end - begin;
-	size_t newSize = std::max(oldSize + len, oldSize + oldSize / 2);
-	auto newBegin = static_cast<byte*>(realloc(begin, newSize));
-	if (!newBegin) {
-		throw std::bad_alloc();
-	}
-	begin = newBegin;
-	end = begin + oldSize + len;
-	finish = begin + newSize;
-	return begin + oldSize;
-}
 
 
 // class InputBuffer
 
-InputBuffer::InputBuffer(const byte* data, size_t size)
+InputBuffer::InputBuffer(const uint8_t* data, size_t size)
 	: buf(data)
 #ifndef NDEBUG
 	, finish(buf + size)
