@@ -1,4 +1,5 @@
 #include "Display.hh"
+#include "RendererFactory.hh"
 #include "Layer.hh"
 #include "VideoSystem.hh"
 #include "VideoLayer.hh"
@@ -7,19 +8,15 @@
 #include "FileOperations.hh"
 #include "FileContext.hh"
 #include "InputEvents.hh"
-#include "Command.hh"
-#include "InfoTopic.hh"
-#include "OSDGUI.hh"
 #include "CliComm.hh"
-#include "CommandConsole.hh"
 #include "Timer.hh"
-#include "RenderSettings.hh"
 #include "BooleanSetting.hh"
 #include "IntegerSetting.hh"
 #include "EnumSetting.hh"
 #include "Reactor.hh"
 #include "MSXMotherBoard.hh"
 #include "HardwareConfig.hh"
+#include "TclArgParser.hh"
 #include "XMLElement.hh"
 #include "VideoSystemChangeListener.hh"
 #include "CommandException.hh"
@@ -27,10 +24,11 @@
 #include "Version.hh"
 #include "build-info.hh"
 #include "checked_cast.hh"
+#include "outer.hh"
+#include "ranges.hh"
 #include "stl.hh"
 #include "unreachable.hh"
-#include "memory.hh"
-#include <algorithm>
+#include "view.hh"
 #include <cassert>
 
 using std::string;
@@ -38,44 +36,16 @@ using std::vector;
 
 namespace openmsx {
 
-class ScreenShotCmd final : public Command
-{
-public:
-	ScreenShotCmd(CommandController& commandController, Display& display);
-	void execute(array_ref<TclObject> tokens, TclObject& result) override;
-	string help(const vector<string>& tokens) const override;
-	void tabCompletion(vector<string>& tokens) const override;
-private:
-	Display& display;
-};
-
-class FpsInfoTopic final : public InfoTopic
-{
-public:
-	FpsInfoTopic(InfoCommand& openMSXInfoCommand, Display& display);
-	void execute(array_ref<TclObject> tokens,
-	             TclObject& result) const override;
-	string help(const vector<string>& tokens) const override;
-private:
-	Display& display;
-};
-
-
 Display::Display(Reactor& reactor_)
 	: RTSchedulable(reactor_.getRTScheduler())
-	, screenShotCmd(make_unique<ScreenShotCmd>(
-		reactor_.getCommandController(), *this))
-	, fpsInfo(make_unique<FpsInfoTopic>(
-		reactor_.getOpenMSXInfoCommand(), *this))
-	, osdGui(make_unique<OSDGUI>(
-		reactor_.getCommandController(), *this))
+	, screenShotCmd(reactor_.getCommandController())
+	, fpsInfo(reactor_.getOpenMSXInfoCommand())
+	, osdGui(reactor_.getCommandController(), *this)
 	, reactor(reactor_)
-	, renderSettings(make_unique<RenderSettings>(
-		reactor.getCommandController()))
-	, commandConsole(make_unique<CommandConsole>(
-		reactor.getGlobalCommandController(),
-		reactor.getEventDistributor(), *this))
-	, currentRenderer(RendererFactory::UNINITIALIZED)
+	, renderSettings(reactor.getCommandController())
+	, commandConsole(reactor.getGlobalCommandController(),
+	                 reactor.getEventDistributor(), *this)
+	, currentRenderer(RenderSettings::UNINITIALIZED)
 	, switchInProgress(false)
 {
 	frameDurationSum = 0;
@@ -98,17 +68,17 @@ Display::Display(Reactor& reactor_)
 	eventDistributor.registerEventListener(OPENMSX_FOCUS_EVENT,
 			*this);
 #endif
-	renderSettings->getRenderer().attach(*this);
-	renderSettings->getFullScreen().attach(*this);
-	renderSettings->getScaleFactor().attach(*this);
+	renderSettings.getRendererSetting().attach(*this);
+	renderSettings.getFullScreenSetting().attach(*this);
+	renderSettings.getScaleFactorSetting().attach(*this);
 	renderFrozen = false;
 }
 
 Display::~Display()
 {
-	renderSettings->getRenderer().detach(*this);
-	renderSettings->getFullScreen().detach(*this);
-	renderSettings->getScaleFactor().detach(*this);
+	renderSettings.getRendererSetting().detach(*this);
+	renderSettings.getFullScreenSetting().detach(*this);
+	renderSettings.getScaleFactorSetting().detach(*this);
 
 	EventDistributor& eventDistributor = reactor.getEventDistributor();
 #if PLATFORM_ANDROID
@@ -124,9 +94,7 @@ Display::~Display()
 	eventDistributor.unregisterEventListener(OPENMSX_FINISH_FRAME_EVENT,
 			*this);
 
-	PRT_DEBUG("Reset video system...");
 	resetVideoSystem();
-	PRT_DEBUG("Reset video system... DONE!");
 
 	assert(listeners.empty());
 }
@@ -134,9 +102,9 @@ Display::~Display()
 void Display::createVideoSystem()
 {
 	assert(!videoSystem);
-	assert(currentRenderer == RendererFactory::UNINITIALIZED);
+	assert(currentRenderer == RenderSettings::UNINITIALIZED);
 	assert(!switchInProgress);
-	currentRenderer = renderSettings->getRenderer().getEnum();
+	currentRenderer = renderSettings.getRenderer();
 	switchInProgress = true;
 	doRendererSwitch();
 }
@@ -147,10 +115,15 @@ VideoSystem& Display::getVideoSystem()
 	return *videoSystem;
 }
 
+OutputSurface* Display::getOutputSurface()
+{
+	return videoSystem ? videoSystem->getOutputSurface() : nullptr;
+}
+
 void Display::resetVideoSystem()
 {
 	videoSystem.reset();
-	// At this point all layers expect for the Video9000 layer
+	// At this point all layers except for the Video9000 layer
 	// should be gone.
 	//assert(layers.empty());
 }
@@ -168,7 +141,7 @@ void Display::attach(VideoSystemChangeListener& listener)
 
 void Display::detach(VideoSystemChangeListener& listener)
 {
-	listeners.erase(find_unguarded(listeners, &listener));
+	move_pop_back(listeners, rfind_unguarded(listeners, &listener));
 }
 
 Layer* Display::findActiveLayer() const
@@ -202,7 +175,7 @@ Display::Layers::iterator Display::baseLayer()
 
 void Display::executeRT()
 {
-	repaint();
+	videoSystem->repaint();
 }
 
 int Display::signalEvent(const std::shared_ptr<const Event>& event)
@@ -210,7 +183,7 @@ int Display::signalEvent(const std::shared_ptr<const Event>& event)
 	if (event->getType() == OPENMSX_FINISH_FRAME_EVENT) {
 		auto& ffe = checked_cast<const FinishFrameEvent&>(*event);
 		if (ffe.needRender()) {
-			repaint();
+			videoSystem->repaint();
 			reactor.getEventDistributor().distributeEvent(
 				std::make_shared<SimpleEvent>(
 					OPENMSX_FRAME_DRAWN_EVENT));
@@ -218,7 +191,7 @@ int Display::signalEvent(const std::shared_ptr<const Event>& event)
 	} else if (event->getType() == OPENMSX_SWITCH_RENDERER_EVENT) {
 		doRendererSwitch();
 	} else if (event->getType() == OPENMSX_MACHINE_LOADED_EVENT) {
-		setWindowTitle();
+		videoSystem->updateWindowTitle();
 	} else if (event->getType() == OPENMSX_EXPOSE_EVENT) {
 		// Don't render too often, and certainly not when the screen
 		// will anyway soon be rendered.
@@ -238,7 +211,7 @@ int Display::signalEvent(const std::shared_ptr<const Event>& event)
 		//  port discovers that the graphics context is gone.
 		// -When gaining the focus, this repaint does nothing as
 		//  the renderFrozen flag is still false
-		repaint();
+		videoSystem->repaint();
 		auto& focusEvent = checked_cast<const FocusEvent&>(*event);
 		ad_printf("Setting renderFrozen to %d", !focusEvent.getGain());
 		renderFrozen = !focusEvent.getGain();
@@ -246,32 +219,30 @@ int Display::signalEvent(const std::shared_ptr<const Event>& event)
 	return 0;
 }
 
-void Display::setWindowTitle()
+string Display::getWindowTitle()
 {
 	string title = Version::full();
 	if (!Version::RELEASE) {
-		title += " [";
-		title += BUILD_FLAVOUR;
-		title += ']';
+		strAppend(title, " [", BUILD_FLAVOUR, ']');
 	}
 	if (MSXMotherBoard* motherboard = reactor.getMotherBoard()) {
 		if (const HardwareConfig* machine = motherboard->getMachineConfig()) {
 			const XMLElement& config = machine->getConfig();
-			title += " - " +
-			    config.getChild("info").getChildData("manufacturer") + ' ' +
-			    config.getChild("info").getChildData("code");
+			strAppend(title, " - ",
+			          config.getChild("info").getChildData("manufacturer"), ' ',
+			          config.getChild("info").getChildData("code"));
 		}
 	}
-	videoSystem->setWindowTitle(title);
+	return title;
 }
 
 void Display::update(const Setting& setting)
 {
-	if (&setting == &renderSettings->getRenderer()) {
+	if (&setting == &renderSettings.getRendererSetting()) {
 		checkRendererSwitch();
-	} else if (&setting == &renderSettings->getFullScreen()) {
+	} else if (&setting == &renderSettings.getFullScreenSetting()) {
 		checkRendererSwitch();
-	} else if (&setting == &renderSettings->getScaleFactor()) {
+	} else if (&setting == &renderSettings.getScaleFactorSetting()) {
 		checkRendererSwitch();
 	} else {
 		UNREACHABLE;
@@ -286,12 +257,11 @@ void Display::checkRendererSwitch()
 		// queued we don't need to do it again.
 		return;
 	}
-	RendererFactory::RendererID newRenderer =
-		renderSettings->getRenderer().getEnum();
+	auto newRenderer = renderSettings.getRenderer();
 	if ((newRenderer != currentRenderer) ||
 	    !getVideoSystem().checkSettings()) {
 		currentRenderer = newRenderer;
-		// don't do the actualing swithing in the Tcl callback
+		// don't do the actual switching in the Tcl callback
 		// it seems creating and destroying Settings (= Tcl vars)
 		// causes problems???
 		switchInProgress = true;
@@ -311,19 +281,27 @@ void Display::doRendererSwitch()
 			doRendererSwitch2();
 			success = true;
 		} catch (MSXException& e) {
-			string errorMsg = "Couldn't activate renderer " +
-				renderSettings->getRenderer().getString() +
-				": " + e.getMessage();
+			auto& rendererSetting = renderSettings.getRendererSetting();
+			string errorMsg = strCat(
+				"Couldn't activate renderer ",
+				rendererSetting.getString(),
+				": ", e.getMessage());
 			// now try some things that might work against this:
-			if (renderSettings->getRenderer().getEnum() != RendererFactory::SDL) {
+			if (rendererSetting.getEnum() != RenderSettings::SDL) {
 				errorMsg += "\nTrying to switch to SDL renderer instead...";
-				renderSettings->getRenderer().setEnum(RendererFactory::SDL);
-				currentRenderer = RendererFactory::SDL;
+				rendererSetting.setEnum(RenderSettings::SDL);
+				currentRenderer = RenderSettings::SDL;
 			} else {
-				unsigned curval = renderSettings->getScaleFactor().getInt();
-				if (curval == 1) throw MSXException(e.getMessage() + " (and I have no other ideas to try...)"); // give up and die... :(
-				errorMsg += "\nTrying to decrease scale_factor setting from " + StringOp::toString(curval) + " to " + StringOp::toString(curval - 1) + "...";
-				renderSettings->getScaleFactor().setInt(curval - 1);
+				auto& scaleFactorSetting = renderSettings.getScaleFactorSetting();
+				unsigned curval = scaleFactorSetting.getInt();
+				if (curval == 1) {
+					throw MSXException(
+						e.getMessage(),
+						" (and I have no other ideas to try...)"); // give up and die... :(
+				}
+				strAppend(errorMsg, "\nTrying to decrease scale_factor setting from ",
+				          curval, " to ", curval - 1, "...");
+				scaleFactorSetting.setInt(curval - 1);
 			}
 			getCliComm().printWarning(errorMsg);
 		}
@@ -340,7 +318,6 @@ void Display::doRendererSwitch2()
 
 	resetVideoSystem();
 	videoSystem = RendererFactory::createVideoSystem(reactor);
-	setWindowTitle();
 
 	for (auto& l : listeners) {
 		l->postVideoSystemChange();
@@ -399,15 +376,14 @@ void Display::repaintDelayed(uint64_t delta)
 void Display::addLayer(Layer& layer)
 {
 	int z = layer.getZ();
-	auto it = find_if(begin(layers), end(layers),
-		[&](Layer* l) { return l->getZ() > z; });
+	auto it = ranges::find_if(layers, [&](Layer* l) { return l->getZ() > z; });
 	layers.insert(it, &layer);
 	layer.setDisplay(*this);
 }
 
 void Display::removeLayer(Layer& layer)
 {
-	layers.erase(find_unguarded(layers, &layer));
+	layers.erase(rfind_unguarded(layers, &layer));
 }
 
 void Display::updateZ(Layer& layer)
@@ -421,51 +397,34 @@ void Display::updateZ(Layer& layer)
 
 // ScreenShotCmd
 
-ScreenShotCmd::ScreenShotCmd(CommandController& commandController,
-                             Display& display_)
-	: Command(commandController, "screenshot")
-	, display(display_)
+Display::ScreenShotCmd::ScreenShotCmd(CommandController& commandController_)
+	: Command(commandController_, "screenshot")
 {
 }
 
-void ScreenShotCmd::execute(array_ref<TclObject> tokens, TclObject& result)
+void Display::ScreenShotCmd::execute(span<const TclObject> tokens, TclObject& result)
 {
+	std::string_view prefix = "openmsx";
 	bool rawShot = false;
-	bool withOsd = false;
+	bool msxOnly = false;
 	bool doubleSize = false;
-	string_ref prefix = "openmsx";
-	vector<TclObject> arguments;
-	for (unsigned i = 1; i < tokens.size(); ++i) {
-		string_ref tok = tokens[i].getString();
-		if (StringOp::startsWith(tok, '-')) {
-			if (tok == "--") {
-				arguments.insert(end(arguments),
-					std::begin(tokens) + i + 1, std::end(tokens));
-				break;
-			}
-			if (tok == "-prefix") {
-				if (++i == tokens.size()) {
-					throw CommandException("Missing argument");
-				}
-				prefix = tokens[i].getString();
-			} else if (tok == "-raw") {
-				rawShot = true;
-			} else if (tok == "-msxonly") {
-				display.getCliComm().printWarning(
-					"The -msxonly option has been deprecated and will "
-					"be removed in a future release. Instead, use the "
-					"-raw option for the same effect.");
-				rawShot = true;
-			} else if (tok == "-doublesize") {
-				doubleSize = true;
-			} else if (tok == "-with-osd") {
-				withOsd = true;
-			} else {
-				throw CommandException("Invalid option: " + tok);
-			}
-		} else {
-			arguments.push_back(tokens[i]);
-		}
+	bool withOsd = false;
+	ArgsInfo info[] = {
+		valueArg("-prefix", prefix),
+		flagArg("-raw", rawShot),
+		flagArg("-msxonly", msxOnly),
+		flagArg("-doublesize", doubleSize),
+		flagArg("-with-osd", withOsd)
+	};
+	auto arguments = parseTclArgs(getInterpreter(), tokens.subspan(1), info);
+
+	auto& display = OUTER(Display, screenShotCmd);
+	if (msxOnly) {
+		display.getCliComm().printWarning(
+			"The -msxonly option has been deprecated and will "
+			"be removed in a future release. Instead, use the "
+			"-raw option for the same effect.");
+		rawShot = true;
 	}
 	if (doubleSize && !rawShot) {
 		throw CommandException("-doublesize option can only be used in "
@@ -476,7 +435,7 @@ void ScreenShotCmd::execute(array_ref<TclObject> tokens, TclObject& result)
 		                       "combination with -raw");
 	}
 
-	string_ref fname;
+	std::string_view fname;
 	switch (arguments.size()) {
 	case 0:
 		// nothing
@@ -496,7 +455,7 @@ void ScreenShotCmd::execute(array_ref<TclObject> tokens, TclObject& result)
 			display.getVideoSystem().takeScreenShot(filename, withOsd);
 		} catch (MSXException& e) {
 			throw CommandException(
-				"Failed to take screenshot: " + e.getMessage());
+				"Failed to take screenshot: ", e.getMessage());
 		}
 	} else {
 		auto videoLayer = dynamic_cast<VideoLayer*>(
@@ -510,53 +469,50 @@ void ScreenShotCmd::execute(array_ref<TclObject> tokens, TclObject& result)
 			videoLayer->takeRawScreenShot(height, filename);
 		} catch (MSXException& e) {
 			throw CommandException(
-				"Failed to take screenshot: " + e.getMessage());
+				"Failed to take screenshot: ", e.getMessage());
 		}
 	}
 
-	display.getCliComm().printInfo("Screen saved to " + filename);
-	result.setString(filename);
+	display.getCliComm().printInfo("Screen saved to ", filename);
+	result = filename;
 }
 
-string ScreenShotCmd::help(const vector<string>& /*tokens*/) const
+string Display::ScreenShotCmd::help(const vector<string>& /*tokens*/) const
 {
 	// Note: -no-sprites option is implemented in Tcl
-	return
-		"screenshot                   Write screenshot to file \"openmsxNNNN.png\"\n"
-		"screenshot <filename>        Write screenshot to indicated file\n"
-		"screenshot -prefix foo       Write screenshot to file \"fooNNNN.png\"\n"
-		"screenshot -raw              320x240 raw screenshot (of MSX screen only)\n"
-		"screenshot -raw -doublesize  640x480 raw screenshot (of MSX screen only)\n"
-		"screenshot -with-osd         Include OSD elements in the screenshot\n"
-		"screenshot -no-sprites       Don't include sprites in the screenshot\n";
+	return "screenshot                   Write screenshot to file \"openmsxNNNN.png\"\n"
+	       "screenshot <filename>        Write screenshot to indicated file\n"
+	       "screenshot -prefix foo       Write screenshot to file \"fooNNNN.png\"\n"
+	       "screenshot -raw              320x240 raw screenshot (of MSX screen only)\n"
+	       "screenshot -raw -doublesize  640x480 raw screenshot (of MSX screen only)\n"
+	       "screenshot -with-osd         Include OSD elements in the screenshot\n"
+	       "screenshot -no-sprites       Don't include sprites in the screenshot\n";
 }
 
-void ScreenShotCmd::tabCompletion(vector<string>& tokens) const
+void Display::ScreenShotCmd::tabCompletion(vector<string>& tokens) const
 {
-	static const char* const extra[] = {
+	static constexpr const char* const extra[] = {
 		"-prefix", "-raw", "-doublesize", "-with-osd", "-no-sprites",
 	};
-	completeFileName(tokens, UserFileContext(), extra);
+	completeFileName(tokens, userFileContext(), extra);
 }
 
 
 // FpsInfoTopic
 
-FpsInfoTopic::FpsInfoTopic(InfoCommand& openMSXInfoCommand,
-                           Display& display_)
+Display::FpsInfoTopic::FpsInfoTopic(InfoCommand& openMSXInfoCommand)
 	: InfoTopic(openMSXInfoCommand, "fps")
-	, display(display_)
 {
 }
 
-void FpsInfoTopic::execute(array_ref<TclObject> /*tokens*/,
-                           TclObject& result) const
+void Display::FpsInfoTopic::execute(span<const TclObject> /*tokens*/,
+                                    TclObject& result) const
 {
-	double fps = 1000000.0 * Display::NUM_FRAME_DURATIONS / display.frameDurationSum;
-	result.setDouble(fps);
+	auto& display = OUTER(Display, fpsInfo);
+	result = 1000000.0f * Display::NUM_FRAME_DURATIONS / display.frameDurationSum;
 }
 
-string FpsInfoTopic::help(const vector<string>& /*tokens*/) const
+string Display::FpsInfoTopic::help(const vector<string>& /*tokens*/) const
 {
 	return "Returns the current rendering speed in frames per second.";
 }
