@@ -1,10 +1,9 @@
-// $Id: DoubledFrame.cc 12833 2012-08-25 08:29:41Z m9710797 $
-
 #include "DoubledFrame.hh"
+#include <cstdint>
 
 namespace openmsx {
 
-DoubledFrame::DoubledFrame(const SDL_PixelFormat& format)
+DoubledFrame::DoubledFrame(const PixelFormat& format)
 	: FrameSource(format)
 {
 }
@@ -23,12 +22,13 @@ unsigned DoubledFrame::getLineWidth(unsigned line) const
 	return (t >= 0) ? field->getLineWidth(t / 2) : 1;
 }
 
-const void* DoubledFrame::getLineInfo(unsigned line, unsigned& width) const
+const void* DoubledFrame::getLineInfo(
+	unsigned line, unsigned& width, void* buf, unsigned bufWidth) const
 {
-	static const unsigned blackPixel = 0; // both 16bppp and 32bpp
+	static constexpr uint32_t blackPixel = 0; // both 16bppp and 32bpp
 	int t = line - skip;
 	if (t >= 0) {
-		return field->getLineInfo(t / 2, width);
+		return field->getLineInfo(t / 2, width, buf, bufWidth);
 	} else {
 		width = 1;
 		return &blackPixel;
