@@ -38,4 +38,12 @@ OutputSurface* DummyVideoSystem::getOutputSurface()
 	return nullptr;
 }
 
+void DummyVideoSystem::showCursor(bool /*show*/)
+{
+}
+
+void DummyVideoSystem::repaint()
+{
+}
+
 } // namespace openmsx
