@@ -5,64 +5,63 @@
 #include "RenderSettings.hh"
 #include "Scaler.hh"
 #include "ScalerFactory.hh"
-#include "OutputSurface.hh"
-#include "IntegerSetting.hh"
-#include "FloatSetting.hh"
-#include "BooleanSetting.hh"
-#include "EnumSetting.hh"
+#include "SDLOutputSurface.hh"
 #include "Math.hh"
 #include "aligned.hh"
+#include "checked_cast.hh"
+#include "random.hh"
 #include "xrange.hh"
 #include <algorithm>
-#include <random>
 #include <cassert>
+#include <cmath>
 #include <cstdint>
+#include <cstddef>
 #ifdef __SSE2__
 #include <emmintrin.h>
 #endif
 
 namespace openmsx {
 
-static const unsigned NOISE_SHIFT = 8192;
-static const unsigned NOISE_BUF_SIZE = 2 * NOISE_SHIFT;
-SSE_ALIGNED(static signed char noiseBuf[NOISE_BUF_SIZE]);
+constexpr unsigned NOISE_SHIFT = 8192;
+constexpr unsigned NOISE_BUF_SIZE = 2 * NOISE_SHIFT;
+alignas(SSE_ALIGNMENT) static signed char noiseBuf[NOISE_BUF_SIZE];
 
 template <class Pixel>
 void FBPostProcessor<Pixel>::preCalcNoise(float factor)
 {
 	// We skip noise drawing if the factor is 0, so there is no point in
 	// initializing the random data in that case.
-	if (factor == 0) return;
+	if (factor == 0.0f) return;
 
 	// for 32bpp groups of 4 consecutive noiseBuf elements (starting at
 	// 4 element boundaries) must have the same value. Later optimizations
 	// depend on it.
 
-	double scale[4];
+	float scale[4];
 	if (sizeof(Pixel) == 4) {
 		// 32bpp
 		// TODO ATM we compensate for big endian here. A better
 		// alternative is to turn noiseBuf into an array of ints (it's
 		// now bytes) and in the 16bpp code extract R,G,B components
 		// from those ints
-		const Pixel p = Pixel(OPENMSX_BIGENDIAN ? 0x00010203
-		                                        : 0x03020100);
+		const auto p = Pixel(OPENMSX_BIGENDIAN ? 0x00010203
+		                                       : 0x03020100);
 		// TODO we can also fill the array with 'factor' and only set
 		// 'alpha' to 0.0. But PixelOperations doesn't offer a simple
 		// way to get the position of the alpha byte (yet).
-		scale[0] = scale[1] = scale[2] = scale[3] = 0.0;
+		scale[0] = scale[1] = scale[2] = scale[3] = 0.0f;
 		scale[pixelOps.red  (p)] = factor;
 		scale[pixelOps.green(p)] = factor;
 		scale[pixelOps.blue (p)] = factor;
 	} else {
 		// 16bpp
-		scale[0] = (pixelOps.getMaxRed()   / 255.0) * factor;
-		scale[1] = (pixelOps.getMaxGreen() / 255.0) * factor;
-		scale[2] = (pixelOps.getMaxBlue()  / 255.0) * factor;
-		scale[3] = 0.0;
+		scale[0] = (pixelOps.getMaxRed()   / 255.0f) * factor;
+		scale[1] = (pixelOps.getMaxGreen() / 255.0f) * factor;
+		scale[2] = (pixelOps.getMaxBlue()  / 255.0f) * factor;
+		scale[3] = 0.0f;
 	}
 
-	std::minstd_rand generator; // fast (non-cryptographic) random numbers
+	auto& generator = global_urng(); // fast (non-cryptographic) random numbers
 	std::normal_distribution<float> distribution(0.0f, 1.0f);
 	for (unsigned i = 0; i < NOISE_BUF_SIZE; i += 4) {
 		float r = distribution(generator);
@@ -74,7 +73,7 @@ void FBPostProcessor<Pixel>::preCalcNoise(float factor)
 }
 
 #ifdef __SSE2__
-static inline void drawNoiseLineSse2(uint32_t* buf_, signed char* noise, long width)
+static inline void drawNoiseLineSse2(uint32_t* buf_, signed char* noise, size_t width)
 {
 	// To each of the RGBA color components (a value in range [0..255]) we
 	// want to add a signed noise value (in range [-128..127]) and also clip
@@ -89,7 +88,7 @@ static inline void drawNoiseLineSse2(uint32_t* buf_, signed char* noise, long wi
 	//   signed_add_sat(value ^ 128, noise) ^ 128
 	// The follwoing loop does just that, though it processes 64 bytes per
 	// iteration.
-	long x = width * sizeof(uint32_t);
+	ptrdiff_t x = width * sizeof(uint32_t);
 	assert((x & 63) == 0);
 	assert((uintptr_t(buf_) & 15) == 0);
 
@@ -164,7 +163,7 @@ static inline uint32_t addNoise4(uint32_t p, uint32_t n)
 
 template <class Pixel>
 void FBPostProcessor<Pixel>::drawNoiseLine(
-		Pixel* buf, signed char* noise, unsigned long width)
+		Pixel* buf, signed char* noise, size_t width)
 {
 #ifdef __SSE2__
 	if (sizeof(Pixel) == 4) {
@@ -179,14 +178,14 @@ void FBPostProcessor<Pixel>::drawNoiseLine(
 	if (sizeof(Pixel) == 4) {
 		// optimized version for 32bpp
 		auto noise4 = reinterpret_cast<uint32_t*>(noise);
-		for (unsigned i = 0; i < width; ++i) {
+		for (size_t i = 0; i < width; ++i) {
 			buf[i] = addNoise4(buf[i], noise4[i]);
 		}
 	} else {
 		int mr = pixelOps.getMaxRed();
 		int mg = pixelOps.getMaxGreen();
 		int mb = pixelOps.getMaxBlue();
-		for (unsigned i = 0; i < width; ++i) {
+		for (size_t i = 0; i < width; ++i) {
 			Pixel p = buf[i];
 			int r = pixelOps.red(p);
 			int g = pixelOps.green(p);
@@ -206,16 +205,16 @@ void FBPostProcessor<Pixel>::drawNoiseLine(
 }
 
 template <class Pixel>
-void FBPostProcessor<Pixel>::drawNoise(OutputSurface& output)
+void FBPostProcessor<Pixel>::drawNoise(OutputSurface& output_)
 {
-	if (renderSettings.getNoise().getDouble() == 0) return;
+	if (renderSettings.getNoise() == 0.0f) return;
 
-	unsigned height = output.getHeight();
-	unsigned width = output.getWidth();
-	output.lock();
-	for (unsigned y = 0; y < height; ++y) {
-		Pixel* buf = output.getLinePtrDirect<Pixel>(y);
-		drawNoiseLine(buf, &noiseBuf[noiseShift[y]], width);
+	auto& output = checked_cast<SDLOutputSurface&>(output_);
+	auto [w, h] = output.getLogicalSize();
+	auto pixelAccess = output.getDirectPixelAccess();
+	for (int y = 0; y < h; ++y) {
+		auto* buf = pixelAccess.getLinePtr<Pixel>(y);
+		drawNoiseLine(buf, &noiseBuf[noiseShift[y]], w);
 	}
 }
 
@@ -223,7 +222,7 @@ template <class Pixel>
 void FBPostProcessor<Pixel>::update(const Setting& setting)
 {
 	VideoLayer::update(setting);
-	FloatSetting& noiseSetting = renderSettings.getNoise();
+	auto& noiseSetting = renderSettings.getNoiseSetting();
 	if (&setting == &noiseSetting) {
 		preCalcNoise(noiseSetting.getDouble());
 	}
@@ -231,35 +230,35 @@ void FBPostProcessor<Pixel>::update(const Setting& setting)
 
 
 template <class Pixel>
-FBPostProcessor<Pixel>::FBPostProcessor(MSXMotherBoard& motherBoard,
-	Display& display, OutputSurface& screen_, const std::string& videoSource,
-	unsigned maxWidth, unsigned height, bool canDoInterlace)
+FBPostProcessor<Pixel>::FBPostProcessor(MSXMotherBoard& motherBoard_,
+	Display& display_, OutputSurface& screen_, const std::string& videoSource,
+	unsigned maxWidth_, unsigned height_, bool canDoInterlace_)
 	: PostProcessor(
-		motherBoard, display, screen_, videoSource, maxWidth, height,
-		canDoInterlace)
-	, noiseShift(screen.getHeight())
-	, pixelOps(screen.getSDLFormat())
+		motherBoard_, display_, screen_, videoSource, maxWidth_, height_,
+		canDoInterlace_)
+	, noiseShift(screen.getLogicalHeight())
+	, pixelOps(screen.getPixelFormat())
 {
 	scaleAlgorithm = RenderSettings::NO_SCALER;
 	scaleFactor = unsigned(-1);
 
-	FloatSetting& noiseSetting = renderSettings.getNoise();
+	auto& noiseSetting = renderSettings.getNoiseSetting();
 	noiseSetting.attach(*this);
 	preCalcNoise(noiseSetting.getDouble());
-	assert((screen.getWidth() * sizeof(Pixel)) < NOISE_SHIFT);
+	assert((screen.getLogicalWidth() * sizeof(Pixel)) < NOISE_SHIFT);
 }
 
 template <class Pixel>
 FBPostProcessor<Pixel>::~FBPostProcessor()
 {
-	FloatSetting& noiseSetting = renderSettings.getNoise();
-	noiseSetting.detach(*this);
+	renderSettings.getNoiseSetting().detach(*this);
 }
 
 template <class Pixel>
-void FBPostProcessor<Pixel>::paint(OutputSurface& output)
+void FBPostProcessor<Pixel>::paint(OutputSurface& output_)
 {
-	if (renderSettings.getInterleaveBlackFrame().getBoolean()) {
+	auto& output = checked_cast<SDLOutputSurface&>(output_);
+	if (renderSettings.getInterleaveBlackFrame()) {
 		interleaveCount ^= 1;
 		if (interleaveCount) {
 			output.clearScreen();
@@ -270,20 +269,19 @@ void FBPostProcessor<Pixel>::paint(OutputSurface& output)
 	if (!paintFrame) return;
 
 	// New scaler algorithm selected?
-	RenderSettings::ScaleAlgorithm algo =
-		renderSettings.getScaleAlgorithm().getEnum();
-	unsigned factor = renderSettings.getScaleFactor().getInt();
+	auto algo = renderSettings.getScaleAlgorithm();
+	unsigned factor = renderSettings.getScaleFactor();
 	if ((scaleAlgorithm != algo) || (scaleFactor != factor)) {
 		scaleAlgorithm = algo;
 		scaleFactor = factor;
 		currScaler = ScalerFactory<Pixel>::createScaler(
-			PixelOperations<Pixel>(output.getSDLFormat()),
+			PixelOperations<Pixel>(output.getPixelFormat()),
 			renderSettings);
 	}
 
 	// Scale image.
 	const unsigned srcHeight = paintFrame->getHeight();
-	const unsigned dstHeight = output.getHeight();
+	const unsigned dstHeight = output.getLogicalHeight();
 
 	unsigned g = Math::gcd(srcHeight, dstHeight);
 	unsigned srcStep = srcHeight / g;
@@ -311,9 +309,8 @@ void FBPostProcessor<Pixel>::paint(OutputSurface& output)
 		// fill region
 		//fprintf(stderr, "post processing lines %d-%d: %d\n",
 		//	srcStartY, srcEndY, lineWidth );
-		output.lock();
-		double horStretch = renderSettings.getHorizontalStretch().getDouble();
-		unsigned inWidth = unsigned(horStretch + 0.5);
+		float horStretch = renderSettings.getHorizontalStretch();
+		unsigned inWidth = lrintf(horStretch);
 		std::unique_ptr<ScalerOutput<Pixel>> dst(
 			StretchScalerOutputFactory<Pixel>::create(
 				output, pixelOps, inWidth));
@@ -329,15 +326,17 @@ void FBPostProcessor<Pixel>::paint(OutputSurface& output)
 
 	drawNoise(output);
 
-	output.flushFrameBuffer(); // for SDLGL-FBxx
+	output.flushFrameBuffer();
 }
 
 template <class Pixel>
 std::unique_ptr<RawFrame> FBPostProcessor<Pixel>::rotateFrames(
 	std::unique_ptr<RawFrame> finishedFrame, EmuTime::param time)
 {
-	for (auto y : xrange(screen.getHeight())) {
-		noiseShift[y] = rand() & (NOISE_SHIFT - 1) & ~15;
+	auto& generator = global_urng(); // fast (non-cryptographic) random numbers
+	std::uniform_int_distribution<int> distribution(0, NOISE_SHIFT / 16 - 1);
+	for (auto y : xrange(screen.getLogicalHeight())) {
+		noiseShift[y] = distribution(generator) * 16;
 	}
 
 	return PostProcessor::rotateFrames(std::move(finishedFrame), time);
