@@ -2,73 +2,50 @@
 #include "GLContext.hh"
 #include "GLScaler.hh"
 #include "GLScalerFactory.hh"
-#include "IntegerSetting.hh"
 #include "FloatSetting.hh"
-#include "BooleanSetting.hh"
-#include "EnumSetting.hh"
 #include "OutputSurface.hh"
 #include "RawFrame.hh"
 #include "Math.hh"
 #include "InitException.hh"
 #include "gl_transform.hh"
+#include "random.hh"
+#include "ranges.hh"
 #include "stl.hh"
 #include "vla.hh"
-#include <algorithm>
-#include <random>
 #include <cassert>
+#include <cstdint>
 
 using namespace gl;
 
 namespace openmsx {
 
-GLPostProcessor::TextureData::TextureData()
-{
-}
-
-GLPostProcessor::TextureData::TextureData(TextureData&& rhs)
-#if !defined(_MSC_VER)
-	noexcept
-#endif
-	: tex(std::move(rhs.tex))
-	, pbo(std::move(rhs.pbo))
-{
-}
-
 GLPostProcessor::GLPostProcessor(
-	MSXMotherBoard& motherBoard, Display& display,
-	OutputSurface& screen, const std::string& videoSource,
-	unsigned maxWidth, unsigned height_, bool canDoInterlace)
-	: PostProcessor(motherBoard, display, screen,
-	                videoSource, maxWidth, height_, canDoInterlace)
+	MSXMotherBoard& motherBoard_, Display& display_,
+	OutputSurface& screen_, const std::string& videoSource,
+	unsigned maxWidth_, unsigned height_, bool canDoInterlace_)
+	: PostProcessor(motherBoard_, display_, screen_,
+	                videoSource, maxWidth_, height_, canDoInterlace_)
 	, noiseTextureA(true, true) // interpolate + wrap
 	, noiseTextureB(true, true)
 	, height(height_)
 {
-	if (!glewIsSupported("GL_EXT_framebuffer_object")) {
-		throw InitException(
-			"The OpenGL framebuffer object is not supported by "
-			"this glew library. Please upgrade your glew library.\n"
-			"It's also possible (but less likely) your video card "
-			"or video card driver doesn't support framebuffer "
-			"objects.");
-	}
-
 	scaleAlgorithm = static_cast<RenderSettings::ScaleAlgorithm>(-1); // not a valid scaler
 
 	frameCounter = 0;
-	noiseX = 0.0;
-	noiseY = 0.0;
-	preCalcNoise(renderSettings.getNoise().getDouble());
+	noiseX = noiseY = 0.0f;
+	preCalcNoise(renderSettings.getNoise());
+	initBuffers();
 
 	storedFrame = false;
 	for (int i = 0; i < 2; ++i) {
 		colorTex[i].bind();
 		colorTex[i].setInterpolation(true);
+		auto [w, h] = screen.getLogicalSize();
 		glTexImage2D(GL_TEXTURE_2D,     // target
 			     0,                 // level
 			     GL_RGB8,           // internal format
-			     screen.getWidth(), // width
-			     screen.getHeight(),// height
+			     w,                 // width
+			     h,                 // height
 			     0,                 // border
 			     GL_RGB,            // format
 			     GL_UNSIGNED_BYTE,  // type
@@ -84,16 +61,48 @@ GLPostProcessor::GLPostProcessor(
 	monitor3DProg.bindAttribLocation(1, "a_normal");
 	monitor3DProg.bindAttribLocation(2, "a_texCoord");
 	monitor3DProg.link();
-	preCalcMonitor3D(renderSettings.getHorizontalStretch().getDouble());
+	preCalcMonitor3D(renderSettings.getHorizontalStretch());
 
-	renderSettings.getNoise().attach(*this);
-	renderSettings.getHorizontalStretch().attach(*this);
+	renderSettings.getNoiseSetting().attach(*this);
+	renderSettings.getHorizontalStretchSetting().attach(*this);
 }
 
 GLPostProcessor::~GLPostProcessor()
 {
-	renderSettings.getHorizontalStretch().detach(*this);
-	renderSettings.getNoise().detach(*this);
+	renderSettings.getHorizontalStretchSetting().detach(*this);
+	renderSettings.getNoiseSetting().detach(*this);
+}
+
+void GLPostProcessor::initBuffers()
+{
+	const vec2 pos[4] = {
+		vec2(-1, 1), vec2(-1,-1), vec2( 1,-1), vec2( 1, 1)
+	};
+	const vec2 tex[4] = {
+		vec2( 0, 1), vec2( 0, 0), vec2( 1, 0), vec2( 1, 1)
+	};
+	glBindBuffer(GL_ARRAY_BUFFER, vbo[0].get());
+	glBufferData(GL_ARRAY_BUFFER, sizeof(pos), pos, GL_STATIC_DRAW);
+	glBindBuffer(GL_ARRAY_BUFFER, vbo[1].get());
+	glBufferData(GL_ARRAY_BUFFER, sizeof(tex), tex, GL_STATIC_DRAW);
+
+	vao.bind();
+	glBindBuffer(GL_ARRAY_BUFFER, vbo[0].get());
+	glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, nullptr);
+	glEnableVertexAttribArray(0);
+	glBindBuffer(GL_ARRAY_BUFFER, vbo[1].get());
+	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, nullptr);
+	glEnableVertexAttribArray(1);
+	vao.unbind();
+
+	glowVAO.bind();
+	glBindBuffer(GL_ARRAY_BUFFER, vbo[0].get());
+	glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, nullptr);
+	glEnableVertexAttribArray(0);
+	glBindBuffer(GL_ARRAY_BUFFER, vbo[1].get());
+	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, nullptr);
+	glEnableVertexAttribArray(1);
+	glowVAO.unbind();
 }
 
 void GLPostProcessor::createRegions()
@@ -101,7 +110,7 @@ void GLPostProcessor::createRegions()
 	regions.clear();
 
 	const unsigned srcHeight = paintFrame->getHeight();
-	const unsigned dstHeight = screen.getHeight();
+	const unsigned dstHeight = screen.getLogicalHeight();
 
 	unsigned g = Math::gcd(srcHeight, dstHeight);
 	unsigned srcStep = srcHeight / g;
@@ -138,7 +147,7 @@ void GLPostProcessor::createRegions()
 
 void GLPostProcessor::paint(OutputSurface& /*output*/)
 {
-	if (renderSettings.getInterleaveBlackFrame().getBoolean()) {
+	if (renderSettings.getInterleaveBlackFrame()) {
 		interleaveCount ^= 1;
 		if (interleaveCount) {
 			glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
@@ -147,15 +156,16 @@ void GLPostProcessor::paint(OutputSurface& /*output*/)
 		}
 	}
 
-	RenderSettings::DisplayDeform deform =
-		renderSettings.getDisplayDeform().getEnum();
-	double horStretch = renderSettings.getHorizontalStretch().getDouble();
-	int glow = renderSettings.getGlow().getInt();
+	auto deform = renderSettings.getDisplayDeform();
+	float horStretch = renderSettings.getHorizontalStretch();
+	int glow = renderSettings.getGlow();
 	bool renderToTexture = (deform != RenderSettings::DEFORM_NORMAL) ||
-	                       (horStretch != 320.0) ||
-	                       (glow != 0);
+	                       (horStretch != 320.0f) ||
+	                       (glow != 0) ||
+	                       screen.isViewScaled();
 
-	if ((deform == RenderSettings::DEFORM_3D) || !paintFrame) {
+	if ((screen.getViewOffset() != ivec2()) || // any part of the screen not covered by the viewport?
+	    (deform == RenderSettings::DEFORM_3D) || !paintFrame) {
 		glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
 		glClear(GL_COLOR_BUFFER_BIT);
 		if (!paintFrame) {
@@ -164,8 +174,7 @@ void GLPostProcessor::paint(OutputSurface& /*output*/)
 	}
 
 	// New scaler algorithm selected?
-	RenderSettings::ScaleAlgorithm algo =
-		renderSettings.getScaleAlgorithm().getEnum();
+	auto algo = renderSettings.getScaleAlgorithm();
 	if (scaleAlgorithm != algo) {
 		scaleAlgorithm = algo;
 		currScaler = GLScalerFactory::createScaler(renderSettings);
@@ -182,8 +191,9 @@ void GLPostProcessor::paint(OutputSurface& /*output*/)
 		uploadFrame();
 	}
 
+	auto [scrnWidth, scrnHeight] = screen.getLogicalSize();
 	if (renderToTexture) {
-		glViewport(0, 0, screen.getWidth(), screen.getHeight());
+		glViewport(0, 0, scrnWidth, scrnHeight);
 		glBindTexture(GL_TEXTURE_2D, 0);
 		fbo[frameCounter & 1].push();
 	}
@@ -199,8 +209,8 @@ void GLPostProcessor::paint(OutputSurface& /*output*/)
 		                 ? &superImposeTex : nullptr;
 		currScaler->scaleImage(
 			it->second.tex, superImpose,
-			r.srcStartY, r.srcEndY, r.lineWidth,       // src
-			r.dstStartY, r.dstEndY, screen.getWidth(), // dst
+			r.srcStartY, r.srcEndY, r.lineWidth, // src
+			r.dstStartY, r.dstEndY, scrnWidth,   // dst
 			paintFrame->getHeight()); // dst
 		//GLUtil::checkGLError("GLPostProcessor::paint");
 	}
@@ -211,34 +221,35 @@ void GLPostProcessor::paint(OutputSurface& /*output*/)
 	if (renderToTexture) {
 		fbo[frameCounter & 1].pop();
 		colorTex[frameCounter & 1].bind();
-		glViewport(screen.getX(), screen.getY(),
-		           screen.getWidth(), screen.getHeight());
+		auto [x, y] = screen.getViewOffset();
+		auto [w, h] = screen.getViewSize();
+		glViewport(x, y, w, h);
 
 		if (deform == RenderSettings::DEFORM_3D) {
 			drawMonitor3D();
 		} else {
-			float x1 = (320.0f - float(horStretch)) / (2.0f * 320.0f);
-			float x2 = 1.0f - x1;
-
-			static const vec2 pos[4] = {
-				vec2(-1, 1), vec2(-1,-1), vec2( 1,-1), vec2( 1, 1)
-			};
-			vec2 tex[4] = {
-				vec2(x1, 1), vec2(x1, 0), vec2(x2, 0), vec2(x2, 1)
-			};
-
 			gl::context->progTex.activate();
 			glUniform4f(gl::context->unifTexColor,
 			            1.0f, 1.0f, 1.0f, 1.0f);
 			mat4 I;
 			glUniformMatrix4fv(gl::context->unifTexMvp,
 			                   1, GL_FALSE, &I[0][0]);
-			glDisable(GL_BLEND);
-			glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, pos);
-			glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, tex);
-			glEnableVertexAttribArray(0);
-			glEnableVertexAttribArray(1);
+
+			vao.bind();
+			if (horStretch != 320.0f) {
+				float x1 = (320.0f - float(horStretch)) / (2.0f * 320.0f);
+				float x2 = 1.0f - x1;
+				vec2 tex[4] = {
+					vec2(x1, 1), vec2(x1, 0), vec2(x2, 0), vec2(x2, 1)
+				};
+				glBindBuffer(GL_ARRAY_BUFFER, stretchVBO.get());
+				glBufferData(GL_ARRAY_BUFFER, sizeof(tex), tex, GL_STATIC_DRAW);
+			} else {
+				glBindBuffer(GL_ARRAY_BUFFER, vbo[1].get());
+			}
+			glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, nullptr);
 			glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
+			vao.unbind();
 		}
 		storedFrame = true;
 	} else {
@@ -253,16 +264,16 @@ std::unique_ptr<RawFrame> GLPostProcessor::rotateFrames(
 		PostProcessor::rotateFrames(std::move(finishedFrame), time);
 	uploadFrame();
 	++frameCounter;
-	noiseX = double(rand()) / RAND_MAX;
-	noiseY = double(rand()) / RAND_MAX;
+	noiseX = random_float(0.0f, 1.0f);
+	noiseY = random_float(0.0f, 1.0f);
 	return reuseFrame;
 }
 
 void GLPostProcessor::update(const Setting& setting)
 {
 	VideoLayer::update(setting);
-	FloatSetting& noiseSetting = renderSettings.getNoise();
-	FloatSetting& horizontalStretch = renderSettings.getHorizontalStretch();
+	auto& noiseSetting = renderSettings.getNoiseSetting();
+	auto& horizontalStretch = renderSettings.getHorizontalStretchSetting();
 	if (&setting == &noiseSetting) {
 		preCalcNoise(noiseSetting.getDouble());
 	} else if (&setting == &horizontalStretch) {
@@ -286,11 +297,11 @@ void GLPostProcessor::uploadFrame()
 	}
 
 	if (superImposeVideoFrame) {
-		int width  = superImposeVideoFrame->getWidth();
-		int height = superImposeVideoFrame->getHeight();
-		if (superImposeTex.getWidth()  != width ||
-		    superImposeTex.getHeight() != height) {
-			superImposeTex.resize(width, height);
+		int w = superImposeVideoFrame->getWidth();
+		int h = superImposeVideoFrame->getHeight();
+		if (superImposeTex.getWidth()  != w ||
+		    superImposeTex.getHeight() != h) {
+			superImposeTex.resize(w, h);
 			superImposeTex.setInterpolation(true);
 		}
 		superImposeTex.bind();
@@ -299,8 +310,8 @@ void GLPostProcessor::uploadFrame()
 			0,                 // level
 			0,                 // offset x
 			0,                 // offset y
-			width,             // width
-			height,            // height
+			w,                 // width
+			h,                 // height
 			GL_BGRA,           // format
 			GL_UNSIGNED_BYTE,  // type
 			const_cast<RawFrame*>(superImposeVideoFrame)->getLinePtrDirect<unsigned>(0)); // data
@@ -311,17 +322,12 @@ void GLPostProcessor::uploadBlock(
 	unsigned srcStartY, unsigned srcEndY, unsigned lineWidth)
 {
 	// create texture/pbo if needed
-	auto it = find_if(begin(textures), end(textures),
-	                  EqualTupleValue<0>(lineWidth));
+	auto it = ranges::find_if(textures, EqualTupleValue<0>(lineWidth));
 	if (it == end(textures)) {
 		TextureData textureData;
 
 		textureData.tex.resize(lineWidth, height * 2); // *2 for interlace
-
-		if (textureData.pbo.openGLSupported()) {
-			textureData.pbo.setImage(lineWidth, height * 2);
-		}
-
+		textureData.pbo.setImage(lineWidth, height * 2);
 		textures.emplace_back(lineWidth, std::move(textureData));
 		it = end(textures) - 1;
 	}
@@ -333,68 +339,35 @@ void GLPostProcessor::uploadBlock(
 
 	// upload data
 	uint32_t* mapped;
-	if (pbo.openGLSupported()) {
-		pbo.bind();
-		mapped = pbo.mapWrite();
-	} else {
-		mapped = nullptr;
-	}
-	if (mapped) {
-		for (unsigned y = srcStartY; y < srcEndY; ++y) {
-			auto* dest = mapped + y * lineWidth;
-			auto* data = paintFrame->getLinePtr(y, lineWidth, dest);
-			if (data != dest) {
-				memcpy(dest, data, lineWidth * sizeof(uint32_t));
-			}
+	pbo.bind();
+	mapped = pbo.mapWrite();
+	for (unsigned y = srcStartY; y < srcEndY; ++y) {
+		auto* dest = mapped + y * lineWidth;
+		auto* data = paintFrame->getLinePtr(y, lineWidth, dest);
+		if (data != dest) {
+			memcpy(dest, data, lineWidth * sizeof(uint32_t));
 		}
-		pbo.unmap();
+	}
+	pbo.unmap();
 #if defined(__APPLE__)
-		// The nVidia GL driver for the GeForce 8000/9000 series seems to hang
-		// on texture data replacements that are 1 pixel wide and start on a
-		// line number that is a non-zero multiple of 16.
-		if (lineWidth == 1 && srcStartY != 0 && srcStartY % 16 == 0) {
-			srcStartY--;
-		}
+	// The nVidia GL driver for the GeForce 8000/9000 series seems to hang
+	// on texture data replacements that are 1 pixel wide and start on a
+	// line number that is a non-zero multiple of 16.
+	if (lineWidth == 1 && srcStartY != 0 && srcStartY % 16 == 0) {
+		srcStartY--;
+	}
 #endif
-		glTexSubImage2D(
-			GL_TEXTURE_2D,       // target
-			0,                   // level
-			0,                   // offset x
-			srcStartY,           // offset y
-			lineWidth,           // width
-			srcEndY - srcStartY, // height
-			GL_BGRA,             // format
-			GL_UNSIGNED_BYTE,    // type
-			pbo.getOffset(0, srcStartY)); // data
-	}
-	if (pbo.openGLSupported()) {
-		pbo.unbind();
-	}
-	if (!mapped) {
-		glPixelStorei(GL_UNPACK_ROW_LENGTH, paintFrame->getRowLength());
-		unsigned y = srcStartY;
-		unsigned remainingLines = srcEndY - srcStartY;
-		VLA_SSE_ALIGNED(uint32_t, buf, lineWidth);
-		while (remainingLines) {
-			unsigned lines;
-			auto* data = paintFrame->getMultiLinePtr(
-				y, remainingLines, lines, lineWidth, buf);
-			glTexSubImage2D(
-				GL_TEXTURE_2D,     // target
-				0,                 // level
-				0,                 // offset x
-				y,                 // offset y
-				lineWidth,         // width
-				lines,             // height
-				GL_BGRA,           // format
-				GL_UNSIGNED_BYTE,  // type
-				data);             // data
-
-			y += lines;
-			remainingLines -= lines;
-		}
-		glPixelStorei(GL_UNPACK_ROW_LENGTH, 0); // restore default
-	}
+	glTexSubImage2D(
+		GL_TEXTURE_2D,       // target
+		0,                   // level
+		0,                   // offset x
+		srcStartY,           // offset y
+		lineWidth,           // width
+		srcEndY - srcStartY, // height
+		GL_BGRA,             // format
+		GL_UNSIGNED_BYTE,    // type
+		pbo.getOffset(0, srcStartY)); // data
+	pbo.unbind();
 
 	// possibly upload scaler specific data
 	if (currScaler) {
@@ -406,13 +379,6 @@ void GLPostProcessor::drawGlow(int glow)
 {
 	if ((glow == 0) || !storedFrame) return;
 
-	static const vec2 pos[4] = {
-		vec2(-1, 1), vec2(-1,-1), vec2( 1,-1), vec2( 1, 1)
-	};
-	static const vec2 tex[4] = {
-		vec2( 0, 1), vec2( 0, 0), vec2( 1, 0), vec2( 1, 1)
-	};
-
 	gl::context->progTex.activate();
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -421,11 +387,9 @@ void GLPostProcessor::drawGlow(int glow)
 	            1.0f, 1.0f, 1.0f, glow * 31 / 3200.0f);
 	mat4 I;
 	glUniformMatrix4fv(gl::context->unifTexMvp, 1, GL_FALSE, &I[0][0]);
-	glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, pos);
-	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, tex);
-	glEnableVertexAttribArray(0);
-	glEnableVertexAttribArray(1);
+	glowVAO.bind();
 	glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
+	glowVAO.unbind();
 	glDisable(GL_BLEND);
 }
 
@@ -433,7 +397,7 @@ void GLPostProcessor::preCalcNoise(float factor)
 {
 	GLbyte buf1[256 * 256];
 	GLbyte buf2[256 * 256];
-	std::minstd_rand generator; // fast (non-cryptographic) random numbers
+	auto& generator = global_urng(); // fast (non-cryptographic) random numbers
 	std::normal_distribution<float> distribution(0.0f, 1.0f);
 	for (int i = 0; i < 256 * 256; ++i) {
 		float r = distribution(generator);
@@ -446,34 +410,31 @@ void GLPostProcessor::preCalcNoise(float factor)
 	glTexImage2D(
 		GL_TEXTURE_2D,    // target
 		0,                // level
-		GL_LUMINANCE,     // internal format
+		GL_RED,           // internal format
 		256,              // width
 		256,              // height
 		0,                // border
-		GL_LUMINANCE,     // format
+		GL_RED,           // format
 		GL_UNSIGNED_BYTE, // type
 		buf1);            // data
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_G, GL_RED);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_B, GL_RED);
 
 	noiseTextureB.bind();
 	glTexImage2D(
 		GL_TEXTURE_2D,    // target
 		0,                // level
-		GL_LUMINANCE,     // internal format
+		GL_RED,           // internal format
 		256,              // width
 		256,              // height
 		0,                // border
-		GL_LUMINANCE,     // format
+		GL_RED,           // format
 		GL_UNSIGNED_BYTE, // type
 		buf2);            // data
-}
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_G, GL_RED);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_B, GL_RED);
 
-void GLPostProcessor::drawNoise()
-{
-	if (renderSettings.getNoise().getDouble() == 0) return;
-
-	// Rotate and mirror noise texture in consecutive frames to avoid
-	// seeing 'patterns' in the noise.
-	static const vec2 pos[8][4] = {
+	const vec2 pos[8][4] = {
 		{ { -1, -1 }, {  1, -1 }, {  1,  1 }, { -1,  1 } },
 		{ { -1,  1 }, {  1,  1 }, {  1, -1 }, { -1, -1 } },
 		{ { -1,  1 }, { -1, -1 }, {  1, -1 }, {  1,  1 } },
@@ -483,6 +444,16 @@ void GLPostProcessor::drawNoise()
 		{ {  1, -1 }, {  1,  1 }, { -1,  1 }, { -1, -1 } },
 		{ { -1, -1 }, { -1,  1 }, {  1,  1 }, {  1, -1 } }
 	};
+	glBindBuffer(GL_ARRAY_BUFFER, noiseVBO[0].get());
+	glBufferData(GL_ARRAY_BUFFER, sizeof(pos), pos, GL_STATIC_DRAW);
+}
+
+void GLPostProcessor::drawNoise()
+{
+	if (renderSettings.getNoise() == 0.0f) return;
+
+	// Rotate and mirror noise texture in consecutive frames to avoid
+	// seeing 'patterns' in the noise.
 	vec2 noise(noiseX, noiseY);
 	const vec2 tex[4] = {
 		noise + vec2(0.0f, 1.875f),
@@ -490,19 +461,23 @@ void GLPostProcessor::drawNoise()
 		noise + vec2(2.0f, 0.0f  ),
 		noise + vec2(0.0f, 0.0f  )
 	};
+	unsigned seq = frameCounter & 7;
 
 	gl::context->progTex.activate();
-
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_ONE, GL_ONE);
 	glUniform4f(gl::context->unifTexColor, 1.0f, 1.0f, 1.0f, 1.0f);
 	mat4 I;
 	glUniformMatrix4fv(gl::context->unifTexMvp, 1, GL_FALSE, &I[0][0]);
 
-	unsigned seq = frameCounter & 7;
-	glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, pos[seq]);
-	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, tex);
+	noiseVAO.bind();
+	glBindBuffer(GL_ARRAY_BUFFER, noiseVBO[0].get());
+	vec2* base = nullptr;
+	glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, base + seq * 4);
 	glEnableVertexAttribArray(0);
+	glBindBuffer(GL_ARRAY_BUFFER, noiseVBO[1].get());
+	glBufferData(GL_ARRAY_BUFFER, sizeof(tex), tex, GL_STATIC_DRAW);
+	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, nullptr);
 	glEnableVertexAttribArray(1);
 
 	noiseTextureA.bind();
@@ -510,12 +485,15 @@ void GLPostProcessor::drawNoise()
 	glBlendEquation(GL_FUNC_REVERSE_SUBTRACT);
 	noiseTextureB.bind();
 	glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
+	noiseVAO.unbind();
+
 	glBlendEquation(GL_FUNC_ADD); // restore default
+	glDisable(GL_BLEND);
 }
 
-static const int GRID_SIZE = 16;
-static const int GRID_SIZE1 = GRID_SIZE + 1;
-static const int NUM_INDICES = (GRID_SIZE1 * 2 + 2) * GRID_SIZE - 2;
+constexpr int GRID_SIZE = 16;
+constexpr int GRID_SIZE1 = GRID_SIZE + 1;
+constexpr int NUM_INDICES = (GRID_SIZE1 * 2 + 2) * GRID_SIZE - 2;
 struct Vertex {
 	vec3 position;
 	vec3 normal;
@@ -527,7 +505,7 @@ void GLPostProcessor::preCalcMonitor3D(float width)
 	// precalculate vertex-positions, -normals and -texture-coordinates
 	Vertex vertices[GRID_SIZE1][GRID_SIZE1];
 
-	static const float GRID_SIZE2 = float(GRID_SIZE) / 2.0f;
+	constexpr float GRID_SIZE2 = float(GRID_SIZE) / 2.0f;
 	float s = width / 320.0f;
 	float b = (320.0f - width) / (2.0f * 320.0f);
 
@@ -545,9 +523,9 @@ void GLPostProcessor::preCalcMonitor3D(float width)
 	}
 
 	// calculate indices
-	unsigned short indices[NUM_INDICES];
+	uint16_t indices[NUM_INDICES];
 
-	unsigned short* ind = indices;
+	uint16_t* ind = indices;
 	for (int y = 0; y < GRID_SIZE; ++y) {
 		for (int x = 0; x < GRID_SIZE1; ++x) {
 			*ind++ = (y + 0) * GRID_SIZE1 + x;
@@ -567,14 +545,25 @@ void GLPostProcessor::preCalcMonitor3D(float width)
 	}
 
 	// upload calculated values to buffers
+	monitor3DVAO.bind();
 	glBindBuffer(GL_ARRAY_BUFFER, arrayBuffer.get());
 	glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices,
 	             GL_STATIC_DRAW);
+	char* base = nullptr;
+	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, sizeof(Vertex),
+	                      base);
+	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, sizeof(Vertex),
+	                      base + sizeof(vec3));
+	glVertexAttribPointer(2, 2, GL_FLOAT, GL_FALSE, sizeof(Vertex),
+	                      base + sizeof(vec3) + sizeof(vec3));
+	glEnableVertexAttribArray(0);
+	glEnableVertexAttribArray(1);
+	glEnableVertexAttribArray(2);
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
 	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, elementbuffer.get());
 	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(indices), indices,
 	             GL_STATIC_DRAW);
-	glBindBuffer(GL_ARRAY_BUFFER, 0);
-	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
+	monitor3DVAO.unbind();
 
 	// calculate transformation matrices
 	mat4 proj = frustum(-1, 1, -1, 1, 1, 10);
@@ -597,24 +586,9 @@ void GLPostProcessor::preCalcMonitor3D(float width)
 void GLPostProcessor::drawMonitor3D()
 {
 	monitor3DProg.activate();
-
-	char* base = nullptr;
-	glBindBuffer(GL_ARRAY_BUFFER, arrayBuffer.get());
-	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, elementbuffer.get());
-	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, sizeof(Vertex),
-	                      base);
-	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, sizeof(Vertex),
-	                      base + sizeof(vec3));
-	glVertexAttribPointer(2, 2, GL_FLOAT, GL_FALSE, sizeof(Vertex),
-	                      base + sizeof(vec3) + sizeof(vec3));
-	glEnableVertexAttribArray(0);
-	glEnableVertexAttribArray(1);
-	glEnableVertexAttribArray(2);
-
+	monitor3DVAO.bind();
 	glDrawElements(GL_TRIANGLE_STRIP, NUM_INDICES, GL_UNSIGNED_SHORT, nullptr);
-
-	glBindBuffer(GL_ARRAY_BUFFER, 0);
-	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
+	monitor3DVAO.unbind();
 }
 
 } // namespace openmsx
