@@ -7,14 +7,8 @@
 #include "Version.hh"
 #include <iostream>
 #include <vector>
-#include <cstring>
 #include <cstdio>
 
-#ifndef glGetShaderiv
-#error The version of GLEW you have installed is missing some OpenGL 2.0 entry points. \
-       Please upgrade to GLEW 1.3.2 or higher.
-#endif
-
 using std::string;
 using namespace openmsx;
 
@@ -26,7 +20,7 @@ void checkGLError(const string& prefix)
 	GLenum error = glGetError();
 	if (error != GL_NO_ERROR) {
 		string err = (char*)gluErrorString(error);
-		std::cerr << "GL error: " << prefix << ": " << err << std::endl;
+		std::cerr << "GL error: " << prefix << ": " << err << '\n';
 	}
 }
 */
@@ -96,24 +90,16 @@ void ColorTexture::resize(GLsizei width_, GLsizei height_)
 
 // class FrameBufferObject
 
-static GLuint currentId = 0;
-static std::vector<GLuint> stack;
-
-FrameBufferObject::FrameBufferObject()
-	: bufferId(0) // 0 is not a valid openGL name
-{
-}
-
 FrameBufferObject::FrameBufferObject(Texture& texture)
 {
-	glGenFramebuffersEXT(1, &bufferId);
-	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, bufferId);
-	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,
-	                          GL_COLOR_ATTACHMENT0_EXT,
-	                          GL_TEXTURE_2D, texture.textureId, 0);
-	bool success = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT) ==
-	               GL_FRAMEBUFFER_COMPLETE_EXT;
-	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, currentId);
+	glGenFramebuffers(1, &bufferId);
+	push();
+	glFramebufferTexture2D(GL_FRAMEBUFFER,
+	                       GL_COLOR_ATTACHMENT0,
+	                       GL_TEXTURE_2D, texture.textureId, 0);
+	bool success = glCheckFramebufferStatus(GL_FRAMEBUFFER) ==
+	               GL_FRAMEBUFFER_COMPLETE;
+	pop();
 	if (!success) {
 		throw InitException(
 			"Your OpenGL implementation support for "
@@ -123,42 +109,19 @@ FrameBufferObject::FrameBufferObject(Texture& texture)
 
 FrameBufferObject::~FrameBufferObject()
 {
-	// It's ok to delete '0' (it's a NOP), but we anyway have to check
-	// for pop().
-	if (!bufferId) return;
-
-	if (currentId == bufferId) {
-		pop();
-	}
-	glDeleteFramebuffersEXT(1, &bufferId);
+	pop();
+	glDeleteFramebuffers(1, &bufferId);
 }
 
 void FrameBufferObject::push()
 {
-	stack.push_back(currentId);
-	currentId = bufferId;
-	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, currentId);
+	glGetIntegerv(GL_FRAMEBUFFER_BINDING, &previousId);
+	glBindFramebuffer(GL_FRAMEBUFFER, bufferId);
 }
 
 void FrameBufferObject::pop()
 {
-	assert(currentId == bufferId);
-	assert(!stack.empty());
-	currentId = stack.back();
-	stack.pop_back();
-	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, currentId);
-}
-
-
-bool PixelBuffers::enabled = true;
-
-// Utility function used by Shader.
-static string readTextFile(const string& filename)
-{
-	File file(SystemFileContext().resolve(filename));
-	size_t size;
-	const byte* data = file.mmap(size);
-	return string(reinterpret_cast<const char*>(data), size);
+	glBindFramebuffer(GL_FRAMEBUFFER, GLuint(previousId));
 }
 
 
@@ -166,7 +129,7 @@ static string readTextFile(const string& filename)
 
 Shader::Shader(GLenum type, const string& filename)
 {
-	init(type, "", filename);
+	init(type, {}, filename);
 }
 
 Shader::Shader(GLenum type, const string& header, const string& filename)
@@ -177,11 +140,14 @@ Shader::Shader(GLenum type, const string& header, const string& filename)
 void Shader::init(GLenum type, const string& header, const string& filename)
 {
 	// Load shader source.
-	string source = header;
+	string source = "#version 330\n" + header;
 	try {
-		source += readTextFile("shaders/" + filename);
+		File file(systemFileContext().resolve("shaders/" + filename));
+		auto mmap = file.mmap();
+		source.append(reinterpret_cast<const char*>(mmap.data()),
+		              mmap.size());
 	} catch (FileException& e) {
-		std::cerr << "Cannot find shader: " << e.getMessage() << std::endl;
+		std::cerr << "Cannot find shader: " << e.getMessage() << '\n';
 		handle = 0;
 		return;
 	}
@@ -189,7 +155,7 @@ void Shader::init(GLenum type, const string& header, const string& filename)
 	// Allocate shader handle.
 	handle = glCreateShader(type);
 	if (handle == 0) {
-		std::cerr << "Failed to allocate shader" << std::endl;
+		std::cerr << "Failed to allocate shader\n";
 		return;
 	}
 
@@ -258,7 +224,7 @@ void ShaderProgram::allocate()
 {
 	handle = glCreateProgram();
 	if (handle == 0) {
-		std::cerr << "Failed to allocate program" << std::endl;
+		std::cerr << "Failed to allocate program\n";
 	}
 }
 
@@ -327,7 +293,7 @@ void ShaderProgram::activate() const
 }
 
 // only useful for debugging
-void ShaderProgram::validate()
+void ShaderProgram::validate() const
 {
 	glValidateProgram(handle);
 	GLint validateStatus = GL_FALSE;
@@ -338,8 +304,8 @@ void ShaderProgram::validate()
 	VLA(GLchar, infoLog, infoLogLength);
 	glGetProgramInfoLog(handle, infoLogLength, nullptr, infoLog);
 	std::cout << "Validate "
-	          << ((validateStatus == GL_TRUE) ? string("OK") : string("FAIL"))
-	          << ": " << infoLog << std::endl;
+	          << ((validateStatus == GL_TRUE) ? "OK" : "FAIL")
+	          << ": " << infoLog << '\n';
 }
 
 
@@ -355,4 +321,18 @@ BufferObject::~BufferObject()
 	glDeleteBuffers(1, &bufferId); // ok to delete 0-buffer
 }
 
+
+// class VertexArray
+
+VertexArray::VertexArray()
+{
+	glGenVertexArrays(1, &bufferId);
+}
+
+VertexArray::~VertexArray()
+{
+	unbind();
+	glDeleteVertexArrays(1, &bufferId); // ok to delete 0-buffer
+}
+
 } // namespace gl
