@@ -1,10 +1,9 @@
 #include "PNG.hh"
-#include "SDLSurfacePtr.hh"
 #include "MSXException.hh"
 #include "File.hh"
-#include "StringOp.hh"
 #include "build-info.hh"
 #include "Version.hh"
+#include "one_of.hh"
 #include "vla.hh"
 #include "cstdiop.hh"
 #include <cassert>
@@ -15,15 +14,13 @@
 #include <png.h>
 #include <SDL.h>
 
-namespace openmsx {
-namespace PNG {
+namespace openmsx::PNG {
 
 static void handleError(png_structp png_ptr, png_const_charp error_msg)
 {
 	auto operation = reinterpret_cast<const char*>(
 		png_get_error_ptr(png_ptr));
-	throw MSXException(StringOp::Builder() <<
-		"Error while " << operation << " PNG: " << error_msg);
+	throw MSXException("Error while ", operation, " PNG: ", error_msg);
 }
 
 static void handleWarning(png_structp png_ptr, png_const_charp warning_msg)
@@ -31,7 +28,7 @@ static void handleWarning(png_structp png_ptr, png_const_charp warning_msg)
 	auto operation = reinterpret_cast<const char*>(
 		png_get_error_ptr(png_ptr));
 	std::cerr << "Warning while " << operation << " PNG: "
-		<< warning_msg << std::endl;
+		<< warning_msg << '\n';
 }
 
 /*
@@ -74,11 +71,6 @@ imported from SDL_image 1.2.10, file "IMG_png.c", function "IMG_LoadPNG_RW".
 */
 
 struct PNGReadHandle {
-	PNGReadHandle()
-		: ptr(nullptr), info(nullptr)
-	{
-	}
-
 	~PNGReadHandle()
 	{
 		if (ptr) {
@@ -86,8 +78,8 @@ struct PNGReadHandle {
 		}
 	}
 
-	png_structp ptr;
-	png_infop info;
+	png_structp ptr = nullptr;
+	png_infop info = nullptr;
 };
 
 static void readData(png_structp ctx, png_bytep area, png_size_t size)
@@ -149,29 +141,35 @@ SDLSurfacePtr load(const std::string& filename, bool want32bpp)
 		// When the output surface is 16bpp, still produce PNG in BGRA
 		// format because SDL *seems* to be better optimized for this
 		// format (not documented, but I checked SDL-1.2.15 source code).
-		const SDL_PixelFormat& format = *SDL_GetVideoSurface()->format;
-		bool bgr, swapAlpha;
-		if (format.BitsPerPixel < 24) {
-			bgr = true; swapAlpha = false;
-		} else {
-			int r = format.Rshift;
-			int g = format.Gshift;
-			int b = format.Bshift;
-			// Can't trust Ashift in a video surface, but it's safe
-			// to assume Alpha channel is located in the leftover
-			// position.
-			if        (r ==  0 && g ==  8 && b == 16) { // RGBA
-				bgr = false; swapAlpha = false;
-			} else if (r == 16 && g ==  8 && b ==  0) { // BGRA
-				bgr = true;  swapAlpha = false;
-			} else if (r ==  8 && g == 16 && b == 24) { // ARGB
-				bgr = false; swapAlpha = true;
-			} else if (r == 24 && g == 16 && b ==  8) { // ABGR
-				bgr = true;  swapAlpha = true;
-			} else {
-				// Format not directly supported by libpng,
-				// use BGRA and still convert later.
-				bgr = true;  swapAlpha = false;
+		// if (for some reason) the surface is not available yet,
+		// we just skip this
+		bool bgr(true), swapAlpha(false); // default BGRA
+
+		int displayIndex = 0;
+		SDL_DisplayMode currentMode;
+		if (SDL_GetCurrentDisplayMode(displayIndex, &currentMode) == 0) {
+			int bpp;
+			Uint32 Rmask, Gmask, Bmask, Amask;
+			SDL_PixelFormatEnumToMasks(
+				currentMode.format, &bpp, &Rmask, &Gmask, &Bmask, &Amask);
+			if (bpp >= 24) {
+				if        (Rmask == 0x000000FF &&
+				           Gmask == 0x0000FF00 &&
+				           Bmask == 0x00FF0000) { // RGB(A)
+					bgr = false; swapAlpha = false;
+				} else if (Rmask == 0x00FF0000 &&
+				           Gmask == 0x0000FF00 &&
+				           Bmask == 0x000000FF) { // BGR(A)
+					bgr = true;  swapAlpha = false;
+				} else if (Rmask == 0x0000FF00 &&
+				           Gmask == 0x00FF0000 &&
+				           Bmask == 0xFF000000) { // ARGB
+					bgr = false; swapAlpha = true;
+				} else if (Rmask == 0xFF000000 &&
+				           Gmask == 0x00FF0000 &&
+				           Bmask == 0x0000FF00) { // ABGR
+					bgr = true;  swapAlpha = true;
+				}
 			}
 		}
 		if (bgr)       png_set_bgr       (png.ptr);
@@ -188,19 +186,19 @@ SDLSurfacePtr load(const std::string& filename, bool want32bpp)
 		             &color_type, &interlace_type, nullptr, nullptr);
 
 		// Allocate the SDL surface to hold the image.
-		static const unsigned MAX_SIZE = 2048;
+		constexpr unsigned MAX_SIZE = 2048;
 		if (width > MAX_SIZE) {
-			throw MSXException(StringOp::Builder() <<
-				"Attempted to create a surface with excessive width: "
-				<< width << ", max " << MAX_SIZE);
+			throw MSXException(
+				"Attempted to create a surface with excessive width: ",
+				width, ", max ", MAX_SIZE);
 		}
 		if (height > MAX_SIZE) {
-			throw MSXException(StringOp::Builder() <<
-				"Attempted to create a surface with excessive height: "
-				<< height << ", max " << MAX_SIZE);
+			throw MSXException(
+				"Attempted to create a surface with excessive height: ",
+				height, ", max ", MAX_SIZE);
 		}
 		int bpp = png_get_channels(png.ptr, png.info) * 8;
-		assert(bpp == 24 || bpp == 32);
+		assert(bpp == one_of(24, 32));
 		Uint32 redMask, grnMask, bluMask, alpMask;
 		if (OPENMSX_BIGENDIAN) {
 			if (bpp == 32) {
@@ -261,9 +259,9 @@ SDLSurfacePtr load(const std::string& filename, bool want32bpp)
 
 		return surface;
 	} catch (MSXException& e) {
-		throw MSXException(StringOp::Builder() <<
-			"Error while loading PNG file \"" << filename <<
-			"\": " << e.getMessage());
+		throw MSXException(
+			"Error while loading PNG file \"", filename, "\": ",
+			e.getMessage());
 	}
 }
 
@@ -272,11 +270,6 @@ SDLSurfacePtr load(const std::string& filename, bool want32bpp)
 /* heavily modified for openMSX by Joost Damad joost@lumatec.be */
 
 struct PNGWriteHandle {
-	PNGWriteHandle()
-		: ptr(nullptr), info(nullptr)
-	{
-	}
-
 	~PNGWriteHandle()
 	{
 		if (ptr) {
@@ -284,8 +277,8 @@ struct PNGWriteHandle {
 		}
 	}
 
-	png_structp ptr;
-	png_infop info;
+	png_structp ptr = nullptr;
+	png_infop info = nullptr;
 };
 
 static void writeData(png_structp ctx, png_bytep area, png_size_t size)
@@ -332,13 +325,22 @@ static void IMG_SavePNG_RW(int width, int height, const void** row_pointers,
 		text[0].text = const_cast<char*>(version.c_str());
 		text[1].compression = PNG_TEXT_COMPRESSION_NONE;
 		text[1].key  = const_cast<char*>("Creation Time");
+
+		// A buffer size of 20 characters is large enough till the year
+		// 9999. But the compiler doesn't understand calendars and
+		// warns that the snprintf output could be truncated (e.g.
+		// because the year is -2147483647). To silence this warning
+		// (and also to work around the windows _snprintf stuff) we add
+		// some extra buffer space.
+		static constexpr size_t size = (10 + 1 + 8 + 1) + 44;
 		time_t now = time(nullptr);
 		struct tm* tm = localtime(&now);
-		char timeStr[10 + 1 + 8 + 1];
+		char timeStr[size];
 		snprintf(timeStr, sizeof(timeStr), "%04d-%02d-%02d %02d:%02d:%02d",
 				1900 + tm->tm_year, tm->tm_mon + 1, tm->tm_mday,
 				tm->tm_hour, tm->tm_min, tm->tm_sec);
 		text[1].text = timeStr;
+
 		png_set_text(png.ptr, png.info, text, 2);
 
 		png_set_IHDR(png.ptr, png.info, width, height, 8,
@@ -356,52 +358,36 @@ static void IMG_SavePNG_RW(int width, int height, const void** row_pointers,
 		png_write_end(png.ptr, png.info);
 	} catch (MSXException& e) {
 		throw MSXException(
-			"Error while writing PNG file \"" + filename + "\": " +
+			"Error while writing PNG file \"", filename, "\": ",
 			e.getMessage());
 	}
 }
 
-void save(SDL_Surface* surface, const std::string& filename)
+static void save(SDL_Surface* image, const std::string& filename)
 {
-	SDL_PixelFormat frmt24;
-	frmt24.palette = nullptr;
-	frmt24.BitsPerPixel = 24;
-	frmt24.BytesPerPixel = 3;
-	frmt24.Rmask = OPENMSX_BIGENDIAN ? 0xFF0000 : 0x0000FF;
-	frmt24.Gmask = 0x00FF00;
-	frmt24.Bmask = OPENMSX_BIGENDIAN ? 0x0000FF : 0xFF0000;
-	frmt24.Amask = 0;
-	frmt24.Rshift = 0;
-	frmt24.Gshift = 8;
-	frmt24.Bshift = 16;
-	frmt24.Ashift = 0;
-	frmt24.Rloss = 0;
-	frmt24.Gloss = 0;
-	frmt24.Bloss = 0;
-	frmt24.Aloss = 8;
-	frmt24.colorkey = 0;
-	frmt24.alpha = 0;
-	SDLSurfacePtr surf24(SDL_ConvertSurface(surface, &frmt24, 0));
+	SDLAllocFormatPtr frmt24(SDL_AllocFormat(
+		OPENMSX_BIGENDIAN ? SDL_PIXELFORMAT_BGR24 : SDL_PIXELFORMAT_RGB24));
+	SDLSurfacePtr surf24(SDL_ConvertSurface(image, frmt24.get(), 0));
 
 	// Create the array of pointers to image data
-	VLA(const void*, row_pointers, surface->h);
-	for (int i = 0; i < surface->h; ++i) {
+	VLA(const void*, row_pointers, image->h);
+	for (int i = 0; i < image->h; ++i) {
 		row_pointers[i] = surf24.getLinePtr(i);
 	}
 
-	IMG_SavePNG_RW(surface->w, surface->h, row_pointers, filename, true);
+	IMG_SavePNG_RW(image->w, image->h, row_pointers, filename, true);
 }
 
 void save(unsigned width, unsigned height, const void** rowPointers,
-          const SDL_PixelFormat& format, const std::string& filename)
+          const PixelFormat& format, const std::string& filename)
 {
 	// this implementation creates 1 extra copy, can be optimized if required
 	SDLSurfacePtr surface(
-		width, height, format.BitsPerPixel,
-		format.Rmask, format.Gmask, format.Bmask, format.Amask);
+		width, height, format.getBpp(),
+		format.getRmask(), format.getGmask(), format.getBmask(), format.getAmask());
 	for (unsigned y = 0; y < height; ++y) {
 		memcpy(surface.getLinePtr(y),
-		       rowPointers[y], width * format.BytesPerPixel);
+		       rowPointers[y], width * format.getBytesPerPixel());
 	}
 	save(surface.get(), filename);
 }
@@ -418,5 +404,4 @@ void saveGrayscale(unsigned width, unsigned height,
 	IMG_SavePNG_RW(width, height, rowPointers, filename, false);
 }
 
-} // namespace PNG
-} // namespace openmsx
+} // namespace openmsx::PNG
