@@ -11,20 +11,23 @@ TODO:
 #include "RenderSettings.hh"
 #include "VideoSourceSetting.hh"
 #include "IntegerSetting.hh"
-#include "BooleanSetting.hh"
-#include "EnumSetting.hh"
 #include "VDP.hh"
 #include "VDPVRAM.hh"
 #include "SpriteChecker.hh"
 #include "EventDistributor.hh"
 #include "FinishFrameEvent.hh"
 #include "RealTime.hh"
+#include "SpeedManager.hh"
+#include "ThrottleManager.hh"
+#include "GlobalSettings.hh"
 #include "MSXMotherBoard.hh"
 #include "Reactor.hh"
 #include "Timer.hh"
+#include "one_of.hh"
 #include "unreachable.hh"
 #include <algorithm>
 #include <cassert>
+#include <cmath>
 
 namespace openmsx {
 
@@ -65,8 +68,7 @@ void PixelRenderer::draw(
 			displayX - vdp.getHorizontalScrollLow() * 2, displayY,
 			displayWidth, displayHeight
 			);
-		if (vdp.spritesEnabled() &&
-		    !renderSettings.getDisableSprites().getBoolean()) {
+		if (vdp.spritesEnabled() && !renderSettings.getDisableSprites()) {
 			rasterizer->drawSprites(
 				startX, startY,
 				displayX / 2, displayY,
@@ -111,6 +113,10 @@ PixelRenderer::PixelRenderer(VDP& vdp_, Display& display)
 	: vdp(vdp_), vram(vdp.getVRAM())
 	, eventDistributor(vdp.getReactor().getEventDistributor())
 	, realTime(vdp.getMotherBoard().getRealTime())
+	, speedManager(
+		vdp.getReactor().getGlobalSettings().getSpeedManager())
+	, throttleManager(
+		vdp.getReactor().getGlobalSettings().getThrottleManager())
 	, renderSettings(display.getRenderSettings())
 	, videoSourceSetting(vdp.getMotherBoard().getVideoSource())
 	, spriteChecker(vdp.getSpriteChecker())
@@ -126,14 +132,14 @@ PixelRenderer::PixelRenderer(VDP& vdp_, Display& display)
 	frameSkipCounter = 999; // force drawing of frame
 	prevRenderFrame = false;
 
-	renderSettings.getMaxFrameSkip().attach(*this);
-	renderSettings.getMinFrameSkip().attach(*this);
+	renderSettings.getMaxFrameSkipSetting().attach(*this);
+	renderSettings.getMinFrameSkipSetting().attach(*this);
 }
 
 PixelRenderer::~PixelRenderer()
 {
-	renderSettings.getMinFrameSkip().detach(*this);
-	renderSettings.getMaxFrameSkip().detach(*this);
+	renderSettings.getMinFrameSkipSetting().detach(*this);
+	renderSettings.getMaxFrameSkipSetting().detach(*this);
 }
 
 PostProcessor* PixelRenderer::getPostProcessor() const
@@ -164,39 +170,46 @@ void PixelRenderer::frameStart(EmuTime::param time)
 		frameSkipCounter = 999;
 		renderFrame = false;
 		prevRenderFrame = false;
+		paintFrame = false;
 		return;
 	}
+
 	prevRenderFrame = renderFrame;
-	if (vdp.isInterlaced() && renderSettings.getDeinterlace().getBoolean() &&
-	    vdp.getEvenOdd() && vdp.isEvenOddEnabled()) {
-		// deinterlaced odd frame, do same as even frame
-	} else {
-		if (frameSkipCounter <
-		              renderSettings.getMinFrameSkip().getInt()) {
-			++frameSkipCounter;
-			renderFrame = false;
-		} else if (frameSkipCounter >=
-		              renderSettings.getMaxFrameSkip().getInt()) {
-			frameSkipCounter = 0;
-			renderFrame = true;
+	if (vdp.isInterlaced() && renderSettings.getDeinterlace()
+			&& vdp.getEvenOdd() && vdp.isEvenOddEnabled()) {
+		// Deinterlaced odd frame: do same as even frame.
+		paintFrame = prevRenderFrame;
+	} else if (throttleManager.isThrottled()) {
+		// Note: min/maxFrameSkip control the number of skipped frames, but
+		//       for every series of skipped frames there is also one painted
+		//       frame, so our boundary checks are offset by one.
+		int counter = int(frameSkipCounter);
+		if (counter < renderSettings.getMinFrameSkip()) {
+			paintFrame = false;
+		} else if (counter > renderSettings.getMaxFrameSkip()) {
+			paintFrame = true;
 		} else {
-			++frameSkipCounter;
-			if (rasterizer->isRecording()) {
-				renderFrame = true;
-			} else {
-				renderFrame = realTime.timeLeft(
-					unsigned(finishFrameDuration), time);
-			}
-			if (renderFrame) {
-				frameSkipCounter = 0;
-			}
+			paintFrame = realTime.timeLeft(
+				unsigned(finishFrameDuration), time);
 		}
+		frameSkipCounter += 1.0f / float(speedManager.getSpeed());
+	} else  {
+		// We need to render a frame every now and then,
+		// to show the user what is happening.
+		paintFrame = (Timer::getTime() - lastPaintTime) >= 100000; // 10 fps
 	}
-	if (!renderFrame) return;
+
+	if (paintFrame) {
+		frameSkipCounter = std::remainder(frameSkipCounter, 1.0f);
+	} else if (!rasterizer->isRecording()) {
+		renderFrame = false;
+		return;
+	}
+	renderFrame = true;
 
 	rasterizer->frameStart(time);
 
-	accuracy = renderSettings.getAccuracy().getEnum();
+	accuracy = renderSettings.getAccuracy();
 
 	nextX = 0;
 	nextY = 0;
@@ -207,7 +220,6 @@ void PixelRenderer::frameStart(EmuTime::param time)
 
 void PixelRenderer::frameEnd(EmuTime::param time)
 {
-	bool skipEvent = !renderFrame;
 	if (renderFrame) {
 		// Render changes from this last frame.
 		sync(time, true);
@@ -217,23 +229,28 @@ void PixelRenderer::frameEnd(EmuTime::param time)
 		rasterizer->frameEnd();
 		auto time2 = Timer::getTime();
 		auto current = time2 - time1;
-		const double ALPHA = 0.2;
+		const float ALPHA = 0.2f;
 		finishFrameDuration = finishFrameDuration * (1 - ALPHA) +
 		                      current * ALPHA;
 
-		if (vdp.isInterlaced() && vdp.isEvenOddEnabled() &&
-		    renderSettings.getDeinterlace().getBoolean() &&
-		    !prevRenderFrame) {
-			// dont send event in deinterlace mode when
-			// previous frame was not rendered
-			skipEvent = true;
+		if (vdp.isInterlaced() && vdp.isEvenOddEnabled()
+				&& renderSettings.getDeinterlace() && !prevRenderFrame) {
+			// Don't paint in deinterlace mode when previous frame
+			// was not rendered.
+			paintFrame = false;
+		}
+		if (paintFrame) {
+			lastPaintTime = time2;
 		}
 	}
-	eventDistributor.distributeEvent(
-		std::make_shared<FinishFrameEvent>(
-			rasterizer->getPostProcessor()->getVideoSource(),
-			videoSourceSetting.getSource(),
-			skipEvent));
+	if (vdp.getMotherBoard().isActive() &&
+	    !vdp.getMotherBoard().isFastForwarding()) {
+		eventDistributor.distributeEvent(
+			std::make_shared<FinishFrameEvent>(
+				rasterizer->getPostProcessor()->getVideoSource(),
+				videoSourceSetting.getSource(),
+				!paintFrame));
+	}
 }
 
 void PixelRenderer::updateHorizontalScrollLow(
@@ -321,7 +338,7 @@ void PixelRenderer::updatePalette(
 		DisplayMode mode = vdp.getDisplayMode();
 		if (mode.getBase() == DisplayMode::GRAPHIC5) {
 			int bgColor = vdp.getBackgroundColor();
-			if (index == (bgColor & 3) || (index == (bgColor >> 2))) {
+			if (index == one_of(bgColor & 3, bgColor >> 2)) {
 				sync(time);
 			}
 		} else if (mode.getByte() != DisplayMode::GRAPHIC7) {
@@ -467,6 +484,10 @@ inline bool PixelRenderer::checkSync(int offset, EmuTime::param time)
 		return false;
 	case DisplayMode::GRAPHIC4:
 	case DisplayMode::GRAPHIC5: {
+		if (vdp.isFastBlinkEnabled()) {
+			// TODO could be improved
+			return true;
+		}
 		// Is the address inside the visual page(s)?
 		// TODO: Also look at which lines are touched inside pages.
 		int visiblePage = vram.nameTable.getMask()
@@ -604,13 +625,11 @@ void PixelRenderer::renderUntil(EmuTime::param time)
 
 void PixelRenderer::update(const Setting& setting)
 {
-	if (&setting == &renderSettings.getMinFrameSkip()
-	|| &setting == &renderSettings.getMaxFrameSkip() ) {
-		// Force drawing of frame.
-		frameSkipCounter = 999;
-	} else {
-		UNREACHABLE;
-	}
+	assert(&setting == one_of(&renderSettings.getMinFrameSkipSetting(),
+	                          &renderSettings.getMaxFrameSkipSetting()));
+	(void)setting;
+	// Force drawing of frame.
+	frameSkipCounter = 999;
 }
 
 } // namespace openmsx
