@@ -7,7 +7,6 @@
 #include "SuperImposedFrame.hh"
 #include "PNG.hh"
 #include "RenderSettings.hh"
-#include "BooleanSetting.hh"
 #include "RawFrame.hh"
 #include "AviRecorder.hh"
 #include "CliComm.hh"
@@ -16,22 +15,22 @@
 #include "EventDistributor.hh"
 #include "FinishFrameEvent.hh"
 #include "CommandException.hh"
-#include "MemoryOps.hh"
+#include "MemBuffer.hh"
 #include "vla.hh"
-#include "memory.hh"
 #include "likely.hh"
 #include "build-info.hh"
 #include <algorithm>
 #include <cassert>
 #include <cstdint>
+#include <memory>
 
 namespace openmsx {
 
-PostProcessor::PostProcessor(MSXMotherBoard& motherBoard,
+PostProcessor::PostProcessor(MSXMotherBoard& motherBoard_,
 	Display& display_, OutputSurface& screen_, const std::string& videoSource,
 	unsigned maxWidth_, unsigned height_, bool canDoInterlace_)
-	: VideoLayer(motherBoard, videoSource)
-	, Schedulable(motherBoard.getScheduler())
+	: VideoLayer(motherBoard_, videoSource)
+	, Schedulable(motherBoard_.getScheduler())
 	, renderSettings(display_.getRenderSettings())
 	, screen(screen_)
 	, paintFrame(nullptr)
@@ -44,18 +43,18 @@ PostProcessor::PostProcessor(MSXMotherBoard& motherBoard,
 	, height(height_)
 	, display(display_)
 	, canDoInterlace(canDoInterlace_)
-	, lastRotate(motherBoard.getCurrentTime())
-	, eventDistributor(motherBoard.getReactor().getEventDistributor())
+	, lastRotate(motherBoard_.getCurrentTime())
+	, eventDistributor(motherBoard_.getReactor().getEventDistributor())
 {
 	if (canDoInterlace) {
-		deinterlacedFrame = make_unique<DeinterlacedFrame>(
-			screen.getSDLFormat());
-		interlacedFrame   = make_unique<DoubledFrame>(
-			screen.getSDLFormat());
+		deinterlacedFrame = std::make_unique<DeinterlacedFrame>(
+			screen.getPixelFormat());
+		interlacedFrame   = std::make_unique<DoubledFrame>(
+			screen.getPixelFormat());
 		deflicker = Deflicker::create(
-			screen.getSDLFormat(), lastFrames);
+			screen.getPixelFormat(), lastFrames);
 		superImposedFrame = SuperImposedFrame::create(
-			screen.getSDLFormat());
+			screen.getPixelFormat());
 	} else {
 		// Laserdisc always produces non-interlaced frames, so we don't
 		// need lastFrames[1..3], deinterlacedFrame and
@@ -94,7 +93,7 @@ unsigned PostProcessor::getLineWidth(
 std::unique_ptr<RawFrame> PostProcessor::rotateFrames(
 	std::unique_ptr<RawFrame> finishedFrame, EmuTime::param time)
 {
-	if (renderSettings.getInterleaveBlackFrame().getBoolean()) {
+	if (renderSettings.getInterleaveBlackFrame()) {
 		auto delta = time - lastRotate; // time between last two calls
 		auto middle = time + delta / 2; // estimate for middle between now
 		                                // and next call
@@ -110,13 +109,13 @@ std::unique_ptr<RawFrame> PostProcessor::rotateFrames(
 	auto currType = finishedFrame->getField();
 	if (canDoInterlace) {
 		if (currType != FrameSource::FIELD_NONINTERLACED) {
-			if (renderSettings.getDeinterlace().getBoolean()) {
+			if (renderSettings.getDeinterlace()) {
 				doDeinterlace = true;
 				numRequired = 2;
 			} else {
 				doInterlace = true;
 			}
-		} else if (renderSettings.getDeflicker().getBoolean()) {
+		} else if (renderSettings.getDeflicker()) {
 			doDeflicker = true;
 			numRequired = 4;
 		}
@@ -195,8 +194,8 @@ std::unique_ptr<RawFrame> PostProcessor::rotateFrames(
 	// Return recycled frame to the caller
 	if (canDoInterlace) {
 		if (unlikely(!recycleFrame)) {
-			recycleFrame = make_unique<RawFrame>(
-				screen.getSDLFormat(), maxWidth, height);
+			recycleFrame = std::make_unique<RawFrame>(
+				screen.getPixelFormat(), maxWidth, height);
 		}
 		return recycleFrame;
 	} else {
@@ -204,7 +203,7 @@ std::unique_ptr<RawFrame> PostProcessor::rotateFrames(
 	}
 }
 
-void PostProcessor::executeUntil(EmuTime::param /*time*/, int /*userData*/)
+void PostProcessor::executeUntil(EmuTime::param /*time*/)
 {
 	// insert fake end of frame event
 	eventDistributor.distributeEvent(
@@ -212,29 +211,30 @@ void PostProcessor::executeUntil(EmuTime::param /*time*/, int /*userData*/)
 			getVideoSource(), getVideoSourceSetting(), false));
 }
 
-void PostProcessor::getScaledFrame(unsigned height, const void** lines,
-                                   std::vector<void*>& workBuffer)
+using WorkBuffer = std::vector<MemBuffer<char, SSE2_ALIGNMENT>>;
+static void getScaledFrame(FrameSource& paintFrame, unsigned bpp,
+                           unsigned height, const void** lines,
+                           WorkBuffer& workBuffer)
 {
 	unsigned width = (height == 240) ? 320 : 640;
-	unsigned pitch = width * ((getBpp() == 32) ? 4 : 2);
+	unsigned pitch = width * ((bpp == 32) ? 4 : 2);
 	const void* line = nullptr;
 	void* work = nullptr;
 	for (unsigned i = 0; i < height; ++i) {
 		if (line == work) {
 			// If work buffer was used in previous iteration,
 			// then allocate a new one.
-			work = MemoryOps::mallocAligned(16, pitch);
-			workBuffer.push_back(work);
+			work = workBuffer.emplace_back(pitch).data();
 		}
 #if HAVE_32BPP
-		if (getBpp() == 32) {
+		if (bpp == 32) {
 			// 32bpp
 			auto* work2 = static_cast<uint32_t*>(work);
 			if (height == 240) {
-				line = paintFrame->getLinePtr320_240(i, work2);
+				line = paintFrame.getLinePtr320_240(i, work2);
 			} else {
 				assert (height == 480);
-				line = paintFrame->getLinePtr640_480(i, work2);
+				line = paintFrame.getLinePtr640_480(i, work2);
 			}
 		} else
 #endif
@@ -243,10 +243,10 @@ void PostProcessor::getScaledFrame(unsigned height, const void** lines,
 			// 15bpp or 16bpp
 			auto* work2 = static_cast<uint16_t*>(work);
 			if (height == 240) {
-				line = paintFrame->getLinePtr320_240(i, work2);
+				line = paintFrame.getLinePtr320_240(i, work2);
 			} else {
 				assert (height == 480);
-				line = paintFrame->getLinePtr640_480(i, work2);
+				line = paintFrame.getLinePtr640_480(i, work2);
 			}
 #endif
 		}
@@ -254,27 +254,22 @@ void PostProcessor::getScaledFrame(unsigned height, const void** lines,
 	}
 }
 
-void PostProcessor::takeRawScreenShot(unsigned height, const std::string& filename)
+void PostProcessor::takeRawScreenShot(unsigned height2, const std::string& filename)
 {
 	if (!paintFrame) {
 		throw CommandException("TODO");
 	}
 
-	VLA(const void*, lines, height);
-	std::vector<void*> workBuffer;
-	getScaledFrame(height, lines, workBuffer);
-
-	unsigned width = (height == 240) ? 320 : 640;
-	PNG::save(width, height, lines, paintFrame->getSDLPixelFormat(), filename);
-
-	for (void* p : workBuffer) {
-		MemoryOps::freeAligned(p);
-	}
+	VLA(const void*, lines, height2);
+	WorkBuffer workBuffer;
+	getScaledFrame(*paintFrame, getBpp(), height2, lines, workBuffer);
+	unsigned width = (height2 == 240) ? 320 : 640;
+	PNG::save(width, height2, lines, paintFrame->getPixelFormat(), filename);
 }
 
 unsigned PostProcessor::getBpp() const
 {
-	return screen.getSDLFormat().BitsPerPixel;
+	return screen.getPixelFormat().getBpp();
 }
 
 } // namespace openmsx
