@@ -2,52 +2,52 @@
 #include "VDP.hh"
 #include "VDPVRAM.hh"
 #include "RawFrame.hh"
-#include "CharacterConverter.hh"
-#include "BitmapConverter.hh"
-#include "SpriteConverter.hh"
-#include "MSXMotherBoard.hh"
 #include "Display.hh"
 #include "Renderer.hh"
 #include "RenderSettings.hh"
 #include "PostProcessor.hh"
-#include "FloatSetting.hh"
-#include "StringSetting.hh"
 #include "MemoryOps.hh"
-#include "VisibleSurface.hh"
-#include "memory.hh"
+#include "OutputSurface.hh"
+#include "one_of.hh"
 #include "build-info.hh"
 #include "components.hh"
 #include <algorithm>
 #include <cassert>
 #include <cstdint>
+#include <memory>
+
+using namespace gl;
 
 namespace openmsx {
 
 /** VDP ticks between start of line and start of left border.
   */
-static const int TICKS_LEFT_BORDER = 100 + 102;
+constexpr int TICKS_LEFT_BORDER = 100 + 102;
 
 /** The middle of the visible (display + borders) part of a line,
   * expressed in VDP ticks since the start of the line.
   * TODO: Move this to a central location?
   */
-static const int TICKS_VISIBLE_MIDDLE =
+constexpr int TICKS_VISIBLE_MIDDLE =
 	TICKS_LEFT_BORDER + (VDP::TICKS_PER_LINE - TICKS_LEFT_BORDER - 27) / 2;
 
-template <class Pixel>
-inline int SDLRasterizer<Pixel>::translateX(int absoluteX, bool narrow)
+/** Translate from absolute VDP coordinates to screen coordinates:
+  * Note: In reality, there are only 569.5 visible pixels on a line.
+  *       Because it looks better, the borders are extended to 640.
+  * @param absoluteX Absolute VDP coordinate.
+  * @param narrow Is this a narrow (512 pixels wide) display mode?
+  */
+static inline int translateX(int absoluteX, bool narrow)
 {
 	int maxX = narrow ? 640 : 320;
 	if (absoluteX == VDP::TICKS_PER_LINE) return maxX;
 
 	// Note: The ROUND_MASK forces the ticks to a pixel (2-tick) boundary.
 	//       If this is not done, rounding errors will occur.
-	//       This is especially tricky because division of a negative number
-	//       is rounded towards zero instead of down.
 	const int ROUND_MASK = narrow ? ~1 : ~3;
 	int screenX =
-		((absoluteX & ROUND_MASK) - (TICKS_VISIBLE_MIDDLE & ROUND_MASK))
-		/ (narrow ? 2 : 4)
+		((absoluteX - (TICKS_VISIBLE_MIDDLE & ROUND_MASK))
+		>> (narrow ? 1 : 2))
 		+ maxX / 2;
 	return std::max(screenX, 0);
 }
@@ -60,29 +60,26 @@ inline void SDLRasterizer<Pixel>::renderBitmapLine(Pixel* buf, unsigned vramLine
 		const byte* vramPtr1;
 		vram.bitmapCacheWindow.getReadAreaPlanar(
 			vramLine * 256, 256, vramPtr0, vramPtr1);
-		bitmapConverter->convertLinePlanar(buf, vramPtr0, vramPtr1);
+		bitmapConverter.convertLinePlanar(buf, vramPtr0, vramPtr1);
 	} else {
 		const byte* vramPtr =
 			vram.bitmapCacheWindow.getReadArea(vramLine * 128, 128);
-		bitmapConverter->convertLine(buf, vramPtr);
+		bitmapConverter.convertLine(buf, vramPtr);
 	}
 }
 
 template <class Pixel>
 SDLRasterizer<Pixel>::SDLRasterizer(
-		VDP& vdp_, Display& display, VisibleSurface& screen_,
+		VDP& vdp_, Display& display, OutputSurface& screen_,
 		std::unique_ptr<PostProcessor> postProcessor_)
 	: vdp(vdp_), vram(vdp.getVRAM())
 	, screen(screen_)
 	, postProcessor(std::move(postProcessor_))
-	, workFrame(make_unique<RawFrame>(screen.getSDLFormat(), 640, 240))
+	, workFrame(std::make_unique<RawFrame>(screen.getPixelFormat(), 640, 240))
 	, renderSettings(display.getRenderSettings())
-	, characterConverter(make_unique<CharacterConverter<Pixel>>(
-		vdp, palFg, palBg))
-	, bitmapConverter(make_unique<BitmapConverter<Pixel>>(
-		palFg, PALETTE256, V9958_COLORS))
-	, spriteConverter(make_unique<SpriteConverter<Pixel>>(
-		vdp.getSpriteChecker()))
+	, characterConverter(vdp, palFg, palBg)
+	, bitmapConverter(palFg, PALETTE256, V9958_COLORS)
+	, spriteConverter(vdp.getSpriteChecker())
 {
 	// Init the palette.
 	precalcPalette();
@@ -95,19 +92,19 @@ SDLRasterizer<Pixel>::SDLRasterizer(
 		}
 	}
 
-	renderSettings.getGamma()      .attach(*this);
-	renderSettings.getBrightness() .attach(*this);
-	renderSettings.getContrast()   .attach(*this);
-	renderSettings.getColorMatrix().attach(*this);
+	renderSettings.getGammaSetting()      .attach(*this);
+	renderSettings.getBrightnessSetting() .attach(*this);
+	renderSettings.getContrastSetting()   .attach(*this);
+	renderSettings.getColorMatrixSetting().attach(*this);
 }
 
 template <class Pixel>
 SDLRasterizer<Pixel>::~SDLRasterizer()
 {
-	renderSettings.getColorMatrix().detach(*this);
-	renderSettings.getGamma()      .detach(*this);
-	renderSettings.getBrightness() .detach(*this);
-	renderSettings.getContrast()   .detach(*this);
+	renderSettings.getColorMatrixSetting().detach(*this);
+	renderSettings.getGammaSetting()      .detach(*this);
+	renderSettings.getBrightnessSetting() .detach(*this);
+	renderSettings.getContrastSetting()   .detach(*this);
 }
 
 template <class Pixel>
@@ -129,7 +126,7 @@ void SDLRasterizer<Pixel>::reset()
 {
 	// Init renderer state.
 	setDisplayMode(vdp.getDisplayMode());
-	spriteConverter->setTransparency(vdp.getTransparency());
+	spriteConverter.setTransparency(vdp.getTransparency());
 
 	resetPalette();
 }
@@ -225,15 +222,15 @@ template <class Pixel>
 void SDLRasterizer<Pixel>::setDisplayMode(DisplayMode mode)
 {
 	if (mode.isBitmapMode()) {
-		bitmapConverter->setDisplayMode(mode);
+		bitmapConverter.setDisplayMode(mode);
 	} else {
-		characterConverter->setDisplayMode(mode);
+		characterConverter.setDisplayMode(mode);
 	}
 	precalcColorIndex0(mode, vdp.getTransparency(),
 	                   vdp.isSuperimposing(), vdp.getBackgroundColor());
-	spriteConverter->setDisplayMode(mode);
-	spriteConverter->setPalette(mode.getByte() == DisplayMode::GRAPHIC7
-	                            ? palGraphic7Sprites : palBg);
+	spriteConverter.setDisplayMode(mode);
+	spriteConverter.setPalette(mode.getByte() == DisplayMode::GRAPHIC7
+	                           ? palGraphic7Sprites : palBg);
 
 	borderSettingChanged();
 }
@@ -246,7 +243,7 @@ void SDLRasterizer<Pixel>::setPalette(int index, int grb)
 	palFg[index     ] = newColor;
 	palFg[index + 16] = newColor;
 	palBg[index     ] = newColor;
-	bitmapConverter->palette16Changed();
+	bitmapConverter.palette16Changed();
 
 	precalcColorIndex0(vdp.getDisplayMode(), vdp.getTransparency(),
 	                   vdp.isSuperimposing(), vdp.getBackgroundColor());
@@ -284,7 +281,7 @@ void SDLRasterizer<Pixel>::setBorderMask(bool /*masked*/)
 template <class Pixel>
 void SDLRasterizer<Pixel>::setTransparency(bool enabled)
 {
-	spriteConverter->setTransparency(enabled);
+	spriteConverter.setTransparency(enabled);
 	precalcColorIndex0(vdp.getDisplayMode(), enabled,
 	                   vdp.isSuperimposing(), vdp.getBackgroundColor());
 }
@@ -294,14 +291,13 @@ void SDLRasterizer<Pixel>::precalcPalette()
 {
 	if (vdp.isMSX1VDP()) {
 		// Fixed palette.
-		for (int i = 0; i < 16; i++) {
-			const byte* rgb = vdp.hasToshibaPalette() ? Renderer::TOSHIBA_PALETTE[i] : Renderer::TMS99X8A_PALETTE[i];
-			double dr = rgb[0] / 255.0;
-			double dg = rgb[1] / 255.0;
-			double db = rgb[2] / 255.0;
-			renderSettings.transformRGB(dr, dg, db);
+		const auto palette = vdp.getMSX1Palette();
+		for (int i = 0; i < 16; ++i) {
+			const auto rgb = palette[i];
 			palFg[i] = palFg[i + 16] = palBg[i] =
-				screen.mapKeyedRGB<Pixel>(dr, dg, db);
+				screen.mapKeyedRGB<Pixel>(
+					renderSettings.transformRGB(
+						vec3(rgb[0], rgb[1], rgb[2]) / 255.0f));
 		}
 	} else {
 		if (vdp.hasYJK()) {
@@ -310,26 +306,24 @@ void SDLRasterizer<Pixel>::precalcPalette()
 				// Most users use the "normal" monitor type; making this a
 				// special case speeds up palette precalculation a lot.
 				int intensity[32];
-				for (int i = 0; i < 32; i++) {
+				for (int i = 0; i < 32; ++i) {
 					intensity[i] =
 						int(255 * renderSettings.transformComponent(i / 31.0));
 				}
-				for (int rgb = 0; rgb < (1 << 15); rgb++) {
-					V9958_COLORS[rgb] = screen.mapKeyedRGB<Pixel>(
-						intensity[(rgb >> 10)     ],
+				for (int rgb = 0; rgb < (1 << 15); ++rgb) {
+					V9958_COLORS[rgb] = screen.mapKeyedRGB255<Pixel>(ivec3(
+						intensity[(rgb >> 10) & 31],
 						intensity[(rgb >>  5) & 31],
-						intensity[ rgb        & 31]);
+						intensity[(rgb >>  0) & 31]));
 				}
 			} else {
-				for (int r5 = 0; r5 < 32; r5++) {
-					for (int g5 = 0; g5 < 32; g5++) {
-						for (int b5 = 0; b5 < 32; b5++) {
-							double dr = r5 / 31.0;
-							double dg = g5 / 31.0;
-							double db = b5 / 31.0;
-							renderSettings.transformRGB(dr, dg, db);
-							V9958_COLORS[(r5<<10) + (g5<<5) + b5] =
-								screen.mapKeyedRGB<Pixel>(dr, dg, db);
+				for (int r = 0; r < 32; ++r) {
+					for (int g = 0; g < 32; ++g) {
+						for (int b = 0; b < 32; ++b) {
+							V9958_COLORS[(r << 10) + (g << 5) + b] =
+								screen.mapKeyedRGB<Pixel>(
+									renderSettings.transformRGB(
+										vec3(r, g, b) / 31.0f));
 						}
 					}
 				}
@@ -337,14 +331,14 @@ void SDLRasterizer<Pixel>::precalcPalette()
 			// Precalculate palette for V9938 colors.
 			// Based on comparing red and green gradients, using palette and
 			// YJK, in SCREEN11 on a real turbo R.
-			for (int r3 = 0; r3 < 8; r3++) {
+			for (int r3 = 0; r3 < 8; ++r3) {
 				int r5 = (r3 << 2) | (r3 >> 1);
-				for (int g3 = 0; g3 < 8; g3++) {
+				for (int g3 = 0; g3 < 8; ++g3) {
 					int g5 = (g3 << 2) | (g3 >> 1);
-					for (int b3 = 0; b3 < 8; b3++) {
+					for (int b3 = 0; b3 < 8; ++b3) {
 						int b5 = (b3 << 2) | (b3 >> 1);
 						V9938_COLORS[r3][g3][b3] =
-							V9958_COLORS[(r5<<10) + (g5<<5) + b5];
+							V9958_COLORS[(r5 << 10) + (g5 << 5) + b5];
 					}
 				}
 			}
@@ -352,44 +346,43 @@ void SDLRasterizer<Pixel>::precalcPalette()
 			// Precalculate palette for V9938 colors.
 			if (renderSettings.isColorMatrixIdentity()) {
 				int intensity[8];
-				for (int i = 0; i < 8; i++) {
+				for (int i = 0; i < 8; ++i) {
 					intensity[i] =
-						int(255 * renderSettings.transformComponent(i / 7.0));
+						int(255 * renderSettings.transformComponent(i / 7.0f));
 				}
-				for (int r3 = 0; r3 < 8; r3++) {
-					for (int g3 = 0; g3 < 8; g3++) {
-						for (int b3 = 0; b3 < 8; b3++) {
-							V9938_COLORS[r3][g3][b3] =
-								screen.mapKeyedRGB<Pixel>(
-									intensity[r3], intensity[g3], intensity[b3]
-									);
+				for (int r = 0; r < 8; ++r) {
+					for (int g = 0; g < 8; ++g) {
+						for (int b = 0; b < 8; ++b) {
+							V9938_COLORS[r][g][b] =
+								screen.mapKeyedRGB255<Pixel>(ivec3(
+									intensity[r],
+									intensity[g],
+									intensity[b]));
 						}
 					}
 				}
 			} else {
-				for (int r3 = 0; r3 < 8; r3++) {
-					for (int g3 = 0; g3 < 8; g3++) {
-						for (int b3 = 0; b3 < 8; b3++) {
-							double dr = r3 / 7.0;
-							double dg = g3 / 7.0;
-							double db = b3 / 7.0;
-							renderSettings.transformRGB(dr, dg, db);
-							V9938_COLORS[r3][g3][b3] =
-								screen.mapKeyedRGB<Pixel>(dr, dg, db);
+				for (int r = 0; r < 8; ++r) {
+					for (int g = 0; g < 8; ++g) {
+						for (int b = 0; b < 8; ++b) {
+							V9938_COLORS[r][g][b] =
+								screen.mapKeyedRGB<Pixel>(
+									renderSettings.transformRGB(
+										vec3(r, g, b) / 7.0f));;
 						}
 					}
 				}
 			}
 		}
 		// Precalculate Graphic 7 bitmap palette.
-		for (int i = 0; i < 256; i++) {
+		for (int i = 0; i < 256; ++i) {
 			PALETTE256[i] = V9938_COLORS
 				[(i & 0x1C) >> 2]
 				[(i & 0xE0) >> 5]
 				[(i & 0x03) == 3 ? 7 : (i & 0x03) * 2];
 		}
 		// Precalculate Graphic 7 sprite palette.
-		for (int i = 0; i < 16; i++) {
+		for (int i = 0; i < 16; ++i) {
 			uint16_t grb = Renderer::GRAPHIC7_SPRITE_PALETTE[i];
 			palGraphic7Sprites[i] =
 				V9938_COLORS[(grb >> 4) & 7][grb >> 8][grb & 7];
@@ -414,7 +407,7 @@ void SDLRasterizer<Pixel>::precalcColorIndex0(DisplayMode mode,
 
 		if (palFg[0] != c) {
 			palFg[0] = c;
-			bitmapConverter->palette16Changed();
+			bitmapConverter.palette16Changed();
 		}
 	} else {
 		// TODO: superimposing
@@ -422,7 +415,7 @@ void SDLRasterizer<Pixel>::precalcColorIndex0(DisplayMode mode,
 		    (palFg[16] != palBg[tpIndex &  3])) {
 			palFg[ 0] = palBg[tpIndex >> 2];
 			palFg[16] = palBg[tpIndex &  3];
-			bitmapConverter->palette16Changed();
+			bitmapConverter.palette16Changed();
 		}
 	}
 }
@@ -431,14 +424,13 @@ template <class Pixel>
 void SDLRasterizer<Pixel>::getBorderColors(Pixel& border0, Pixel& border1)
 {
 	DisplayMode mode = vdp.getDisplayMode();
-	byte modeBase = mode.getBase();
 	int bgColor = vdp.getBackgroundColor();
-	if (modeBase == DisplayMode::GRAPHIC5) {
+	if (mode.getBase() == DisplayMode::GRAPHIC5) {
 		// border in SCREEN6 has separate color for even and odd pixels.
 		// TODO odd/even swapped?
 		border0 = palBg[(bgColor & 0x0C) >> 2];
 		border1 = palBg[(bgColor & 0x03) >> 0];
-	} else if (modeBase == DisplayMode::GRAPHIC7) {
+	} else if (mode.getByte() == DisplayMode::GRAPHIC7) {
 		border0 = border1 = PALETTE256[bgColor];
 	} else {
 		if (!bgColor && vdp.isSuperimposing()) {
@@ -502,7 +494,7 @@ void SDLRasterizer<Pixel>::drawDisplay(
 	// drawBorder() (for the right border). And the value we set there is
 	// anyway the same as the one we would set here.
 	DisplayMode mode = vdp.getDisplayMode();
-	unsigned lineWidth = mode.getLineWidth();
+	int lineWidth = mode.getLineWidth();
 	if (lineWidth == 256) {
 		int endX = displayX + displayWidth;
 		displayX /= 2;
@@ -553,14 +545,19 @@ void SDLRasterizer<Pixel>::drawDisplay(
 	}
 
 	if (mode.isBitmapMode()) {
-		// Which bits in the name mask determine the page?
-		int pageMaskOdd = (mode.isPlanar() ? 0x000 : 0x200) |
-		                  vdp.getEvenOddMask();
-		int pageMaskEven = vdp.isMultiPageScrolling()
-		                 ? (pageMaskOdd & ~0x100)
-		                 : pageMaskOdd;
-
 		for (int y = screenY; y < screenLimitY; y++) {
+			// Which bits in the name mask determine the page?
+			// TODO optimize this?
+			//   Calculating pageMaskOdd/Even is a non-trivial amount
+			//   of work. We used to do this per frame (more or less)
+			//   but now do it per line. Per-line is actually only
+			//   needed when vdp.isFastBlinkEnabled() is true.
+			//   Idea: can be cheaply calculated incrementally.
+			int pageMaskOdd = (mode.isPlanar() ? 0x000 : 0x200) |
+				vdp.getEvenOddMask(y);
+			int pageMaskEven = vdp.isMultiPageScrolling()
+				? (pageMaskOdd & ~0x100)
+				: pageMaskOdd;
 			const int vramLine[2] = {
 				(vram.nameTable.getMask() >> 7) & (pageMaskEven | displayY),
 				(vram.nameTable.getMask() >> 7) & (pageMaskOdd  | displayY)
@@ -572,7 +569,9 @@ void SDLRasterizer<Pixel>::drawDisplay(
 			           + leftBackground + displayX;
 			int firstPageWidth = pageBorder - displayX;
 			if (firstPageWidth > 0) {
-				if ((displayX + hScroll) == 0) {
+				if (((displayX + hScroll) == 0) &&
+				    (firstPageWidth == lineWidth)) {
+					// fast-path, directly render to destination
 					renderBitmapLine(dst, vramLine[scrollPage1]);
 				} else {
 					lineInBuf = vramLine[scrollPage1];
@@ -603,11 +602,11 @@ void SDLRasterizer<Pixel>::drawDisplay(
 
 			Pixel* dst = workFrame->getLinePtrDirect<Pixel>(y)
 			           + leftBackground + displayX;
-			if (displayX == 0) {
-				characterConverter->convertLine(dst, displayY);
+			if ((displayX == 0) && (displayWidth == lineWidth)){
+				characterConverter.convertLine(dst, displayY);
 			} else {
 				Pixel buf[512];
-				characterConverter->convertLine(buf, displayY);
+				characterConverter.convertLine(buf, displayY);
 				const Pixel* src = buf + displayX;
 				memcpy(dst, src, displayWidth * sizeof(Pixel));
 			}
@@ -649,26 +648,26 @@ void SDLRasterizer<Pixel>::drawSprites(
 	if (spriteMode == 1) {
 		for (int y = fromY; y < limitY; y++, screenY++) {
 			Pixel* pixelPtr = workFrame->getLinePtrDirect<Pixel>(screenY) + screenX;
-			spriteConverter->drawMode1(y, displayX, displayLimitX, pixelPtr);
+			spriteConverter.drawMode1(y, displayX, displayLimitX, pixelPtr);
 		}
 	} else {
 		byte mode = vdp.getDisplayMode().getByte();
 		if (mode == DisplayMode::GRAPHIC5) {
 			for (int y = fromY; y < limitY; y++, screenY++) {
 				Pixel* pixelPtr = workFrame->getLinePtrDirect<Pixel>(screenY) + screenX;
-				spriteConverter->template drawMode2<DisplayMode::GRAPHIC5>(
+				spriteConverter.template drawMode2<DisplayMode::GRAPHIC5>(
 					y, displayX, displayLimitX, pixelPtr);
 			}
 		} else if (mode == DisplayMode::GRAPHIC6) {
 			for (int y = fromY; y < limitY; y++, screenY++) {
 				Pixel* pixelPtr = workFrame->getLinePtrDirect<Pixel>(screenY) + screenX;
-				spriteConverter->template drawMode2<DisplayMode::GRAPHIC6>(
+				spriteConverter.template drawMode2<DisplayMode::GRAPHIC6>(
 					y, displayX, displayLimitX, pixelPtr);
 			}
 		} else {
 			for (int y = fromY; y < limitY; y++, screenY++) {
 				Pixel* pixelPtr = workFrame->getLinePtrDirect<Pixel>(screenY) + screenX;
-				spriteConverter->template drawMode2<DisplayMode::GRAPHIC4>(
+				spriteConverter.template drawMode2<DisplayMode::GRAPHIC4>(
 					y, displayX, displayLimitX, pixelPtr);
 			}
 		}
@@ -684,10 +683,10 @@ bool SDLRasterizer<Pixel>::isRecording() const
 template <class Pixel>
 void SDLRasterizer<Pixel>::update(const Setting& setting)
 {
-	if ((&setting == &renderSettings.getGamma()) ||
-	    (&setting == &renderSettings.getBrightness()) ||
-	    (&setting == &renderSettings.getContrast()) ||
-	    (&setting == &renderSettings.getColorMatrix())) {
+	if (&setting == one_of(&renderSettings.getGammaSetting(),
+	                       &renderSettings.getBrightnessSetting(),
+	                       &renderSettings.getContrastSetting(),
+	                       &renderSettings.getColorMatrixSetting())) {
 		precalcPalette();
 		resetPalette();
 	}
