@@ -1,26 +1,26 @@
 #include "SuperImposedFrame.hh"
 #include "PixelOperations.hh"
 #include "LineScalers.hh"
-#include "memory.hh"
 #include "unreachable.hh"
 #include "vla.hh"
 #include "build-info.hh"
 #include <algorithm>
 #include <cstdint>
+#include <memory>
 
 namespace openmsx {
 
 template <typename Pixel>
-class SuperImposedFrameImpl : public SuperImposedFrame
+class SuperImposedFrameImpl final : public SuperImposedFrame
 {
 public:
-	SuperImposedFrameImpl(const SDL_PixelFormat& format);
+	explicit SuperImposedFrameImpl(const PixelFormat& format);
 
 private:
-	virtual unsigned getLineWidth(unsigned line) const;
-	virtual const void* getLineInfo(
+	unsigned getLineWidth(unsigned line) const override;
+	const void* getLineInfo(
 		unsigned line, unsigned& width,
-		void* buf, unsigned bufWidth) const;
+		void* buf, unsigned bufWidth) const override;
 
 	PixelOperations<Pixel> pixelOps;
 };
@@ -29,22 +29,22 @@ private:
 // class SuperImposedFrame
 
 std::unique_ptr<SuperImposedFrame> SuperImposedFrame::create(
-	const SDL_PixelFormat& format)
+	const PixelFormat& format)
 {
 #if HAVE_16BPP
-	if (format.BitsPerPixel == 15 || format.BitsPerPixel == 16) {
-		return make_unique<SuperImposedFrameImpl<uint16_t>>(format);
+	if (format.getBytesPerPixel() == 2) {
+		return std::make_unique<SuperImposedFrameImpl<uint16_t>>(format);
 	}
 #endif
 #if HAVE_32BPP
-	if (format.BitsPerPixel == 32) {
-		return make_unique<SuperImposedFrameImpl<uint32_t>>(format);
+	if (format.getBytesPerPixel() == 4) {
+		return std::make_unique<SuperImposedFrameImpl<uint32_t>>(format);
 	}
 #endif
 	UNREACHABLE; return nullptr; // avoid warning
 }
 
-SuperImposedFrame::SuperImposedFrame(const SDL_PixelFormat& format)
+SuperImposedFrame::SuperImposedFrame(const PixelFormat& format)
 	: FrameSource(format)
 {
 }
@@ -62,7 +62,7 @@ void SuperImposedFrame::init(
 
 template <typename Pixel>
 SuperImposedFrameImpl<Pixel>::SuperImposedFrameImpl(
-		const SDL_PixelFormat& format)
+		const PixelFormat& format)
 	: SuperImposedFrame(format)
 	, pixelOps(format)
 {
@@ -80,7 +80,7 @@ unsigned SuperImposedFrameImpl<Pixel>::getLineWidth(unsigned line) const
 
 template <typename Pixel>
 const void* SuperImposedFrameImpl<Pixel>::getLineInfo(
-	unsigned line, unsigned& width, void* tBuf_, unsigned bufWidth) const
+	unsigned line, unsigned& width, void* buf, unsigned bufWidth) const
 {
 	unsigned tNum = (getHeight() == top   ->getHeight()) ? line : line / 2;
 	unsigned bNum = (getHeight() == bottom->getHeight()) ? line : line / 2;
@@ -89,7 +89,7 @@ const void* SuperImposedFrameImpl<Pixel>::getLineInfo(
 	width = std::max(tWidth, bWidth);  // as wide as the widest source
 	width = std::min(width, bufWidth); // but no wider than the output buffer
 
-	auto* tBuf = static_cast<Pixel*>(tBuf_);
+	auto* tBuf = static_cast<Pixel*>(buf);
 	VLA_SSE_ALIGNED(Pixel, bBuf, width);
 	auto* tLine = top   ->getLinePtr(tNum, width, tBuf);
 	auto* bLine = bottom->getLinePtr(bNum, width, bBuf);
