@@ -2,9 +2,8 @@
 #include "SpriteChecker.hh"
 #include "Renderer.hh"
 #include "Math.hh"
-#include "SimpleDebuggable.hh"
+#include "outer.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include <algorithm>
 #include <cstring>
 
@@ -12,8 +11,6 @@ namespace openmsx {
 
 // class VRAMWindow
 
-DummyVRAMOBserver VRAMWindow::dummyObserver;
-
 VRAMWindow::VRAMWindow(Ram& vram)
 	: data(&vram[0])
 {
@@ -41,75 +38,57 @@ VRAMWindow::VRAMWindow(Ram& vram)
  *   size of this debuggable would have to be 256kB to be able to access the
  *   complete extended VRAM in interleaved mode.
  */
-class LogicalVRAMDebuggable final : public SimpleDebuggable
-{
-public:
-	explicit LogicalVRAMDebuggable(VDP& vdp);
-	byte read(unsigned address, EmuTime::param time) override;
-	void write(unsigned address, byte value, EmuTime::param time) override;
-private:
-	unsigned transform(unsigned address);
-	VDP& vdp;
-};
-
-
-LogicalVRAMDebuggable::LogicalVRAMDebuggable(VDP& vdp_)
+VDPVRAM::LogicalVRAMDebuggable::LogicalVRAMDebuggable(VDP& vdp_)
 	: SimpleDebuggable(vdp_.getMotherBoard(), vdp_.getName() == "VDP" ? "VRAM" :
 			vdp_.getName() + " VRAM",
 			"CPU view on video RAM given the current display mode.",
 			 128 * 1024) // always 128kB
-	, vdp(vdp_)
 {
 }
 
-unsigned LogicalVRAMDebuggable::transform(unsigned address)
+unsigned VDPVRAM::LogicalVRAMDebuggable::transform(unsigned address)
 {
-	return vdp.getDisplayMode().isPlanar()
+	auto& vram = OUTER(VDPVRAM, logicalVRAMDebug);
+	return vram.vdp.getDisplayMode().isPlanar()
 	     ? ((address << 16) | (address >> 1)) & 0x1FFFF
 	     : address;
 }
 
-byte LogicalVRAMDebuggable::read(unsigned address, EmuTime::param time)
+byte VDPVRAM::LogicalVRAMDebuggable::read(unsigned address, EmuTime::param time)
 {
-	return vdp.getVRAM().cpuRead(transform(address), time);
+	auto& vram = OUTER(VDPVRAM, logicalVRAMDebug);
+	return vram.cpuRead(transform(address), time);
 }
 
-void LogicalVRAMDebuggable::write(unsigned address, byte value, EmuTime::param time)
+void VDPVRAM::LogicalVRAMDebuggable::write(
+	unsigned address, byte value, EmuTime::param time)
 {
-	vdp.getVRAM().cpuWrite(transform(address), value, time);
+	auto& vram = OUTER(VDPVRAM, logicalVRAMDebug);
+	vram.cpuWrite(transform(address), value, time);
 }
 
 
 // class PhysicalVRAMDebuggable
 
-class PhysicalVRAMDebuggable final : public SimpleDebuggable
-{
-public:
-	PhysicalVRAMDebuggable(VDP& vdp, VDPVRAM& vram, unsigned actualSize);
-	byte read(unsigned address, EmuTime::param time) override;
-	void write(unsigned address, byte value, EmuTime::param time) override;
-private:
-	VDPVRAM& vram;
-};
-
-
-PhysicalVRAMDebuggable::PhysicalVRAMDebuggable(VDP& vdp,
-				VDPVRAM& vram_, unsigned actualSize)
-	: SimpleDebuggable(vdp.getMotherBoard(), vdp.getName() == "VDP" ?
-			"physical VRAM" : "physical " + vdp.getName() + " VRAM",
+VDPVRAM::PhysicalVRAMDebuggable::PhysicalVRAMDebuggable(
+		VDP& vdp_, unsigned actualSize_)
+	: SimpleDebuggable(vdp_.getMotherBoard(), vdp_.getName() == "VDP" ?
+	                   "physical VRAM" : strCat("physical ", vdp_.getName(), " VRAM"),
 	                   "VDP-screen-mode-independent view on the video RAM.",
-	                   actualSize)
-	, vram(vram_) // vdp.getVRAM() doesn't work yet
+	                   actualSize_)
 {
 }
 
-byte PhysicalVRAMDebuggable::read(unsigned address, EmuTime::param time)
+byte VDPVRAM::PhysicalVRAMDebuggable::read(unsigned address, EmuTime::param time)
 {
+	auto& vram = OUTER(VDPVRAM, physicalVRAMDebug);
 	return vram.cpuRead(address, time);
 }
 
-void PhysicalVRAMDebuggable::write(unsigned address, byte value, EmuTime::param time)
+void VDPVRAM::PhysicalVRAMDebuggable::write(
+	unsigned address, byte value, EmuTime::param time)
 {
+	auto& vram = OUTER(VDPVRAM, physicalVRAMDebug);
 	vram.cpuWrite(address, value, time);
 }
 
@@ -128,11 +107,11 @@ static unsigned bufferSize(unsigned size)
 
 VDPVRAM::VDPVRAM(VDP& vdp_, unsigned size, EmuTime::param time)
 	: vdp(vdp_)
-	, data(vdp_.getDeviceConfig2(), bufferSize(size))
-	, logicalVRAMDebug (make_unique<LogicalVRAMDebuggable>(vdp))
-	, physicalVRAMDebug(make_unique<PhysicalVRAMDebuggable>(vdp, *this, size))
+	, data(*vdp_.getDeviceConfig2().getXML(), bufferSize(size))
+	, logicalVRAMDebug (vdp)
+	, physicalVRAMDebug(vdp, size)
 	#ifdef DEBUG
-	, vramTime(EmuTime::zero)
+	, vramTime(EmuTime::zero())
 	#endif
 	, actualSize(size)
 	, cmdReadWindow(data)
@@ -153,11 +132,7 @@ VDPVRAM::VDPVRAM(VDP& vdp_, unsigned size, EmuTime::param time)
 	// Whole VRAM is cachable.
 	// Because this window has no observer, any EmuTime can be passed.
 	// TODO: Move this to cache registration.
-	bitmapCacheWindow.setMask(0x1FFFF, ~0u << 17, EmuTime::zero);
-}
-
-VDPVRAM::~VDPVRAM()
-{
+	bitmapCacheWindow.setMask(0x1FFFF, ~0u << 17, EmuTime::zero());
 }
 
 void VDPVRAM::clear()
@@ -173,10 +148,10 @@ void VDPVRAM::clear()
 	}
 }
 
-void VDPVRAM::updateDisplayMode(DisplayMode mode, EmuTime::param time)
+void VDPVRAM::updateDisplayMode(DisplayMode mode, bool cmdBit, EmuTime::param time)
 {
 	assert(vdp.isInsideFrame(time));
-	cmdEngine->updateDisplayMode(mode, time);
+	cmdEngine->updateDisplayMode(mode, cmdBit, time);
 	renderer->updateDisplayMode(mode, time);
 	spriteChecker->updateDisplayMode(mode, time);
 }
@@ -202,9 +177,9 @@ void VDPVRAM::setSizeMask(EmuTime::param time)
 	sizeMask = (
 		  vrMode
 		// VR = 1: 64K address space, CAS0/1 is determined by A16
-		? (Math::powerOfTwo(actualSize) - 1) | (1u << 16)
+		? (Math::ceil2(actualSize) - 1) | (1u << 16)
 		// VR = 0: 16K address space, CAS0/1 is determined by A14
-		: (std::min(Math::powerOfTwo(actualSize), 16384u) - 1) | (1u << 14)
+		: (std::min(Math::ceil2(actualSize), 16384u) - 1) | (1u << 14)
 		) | (1u << 17); // CASX (expansion RAM) is always relevant
 
 	cmdReadWindow.setSizeMask(sizeMask, time);
@@ -248,9 +223,9 @@ void VDPVRAM::updateVRMode(bool newVRmode, EmuTime::param time)
 	}
 }
 
-void VDPVRAM::setRenderer(Renderer* renderer, EmuTime::param time)
+void VDPVRAM::setRenderer(Renderer* newRenderer, EmuTime::param time)
 {
-	this->renderer = renderer;
+	renderer = newRenderer;
 
 	bitmapVisibleWindow.resetObserver();
 	// Set up bitmapVisibleWindow to full VRAM.
@@ -334,9 +309,9 @@ void VDPVRAM::change4k8kMapping(bool mapping8k)
 template<typename Archive>
 void VRAMWindow::serialize(Archive& ar, unsigned /*version*/)
 {
-	ar.serialize("baseAddr",  baseAddr);
-	ar.serialize("baseMask",  origBaseMask);
-	ar.serialize("indexMask", indexMask);
+	ar.serialize("baseAddr",  baseAddr,
+	             "baseMask",  origBaseMask,
+	             "indexMask", indexMask);
 	if (ar.isLoader()) {
 		effectiveBaseMask = origBaseMask & sizeMask;
 		combiMask = ~effectiveBaseMask | indexMask;
@@ -353,17 +328,17 @@ void VDPVRAM::serialize(Archive& ar, unsigned /*version*/)
 	}
 
 	ar.serialize_blob("data", &data[0], actualSize);
-	ar.serialize("cmdReadWindow",       cmdReadWindow);
-	ar.serialize("cmdWriteWindow",      cmdWriteWindow);
-	ar.serialize("nameTable",           nameTable);
+	ar.serialize("cmdReadWindow",       cmdReadWindow,
+	             "cmdWriteWindow",      cmdWriteWindow,
+	             "nameTable",           nameTable,
 	// TODO: Find a way of changing the line below to "colorTable",
 	// without breaking backwards compatibility
-	ar.serialize("colourTable",         colorTable);
-	ar.serialize("patternTable",        patternTable);
-	ar.serialize("bitmapVisibleWindow", bitmapVisibleWindow);
-	ar.serialize("bitmapCacheWindow",   bitmapCacheWindow);
-	ar.serialize("spriteAttribTable",   spriteAttribTable);
-	ar.serialize("spritePatternTable",  spritePatternTable);
+	             "colourTable",         colorTable,
+	             "patternTable",        patternTable,
+	             "bitmapVisibleWindow", bitmapVisibleWindow,
+	             "bitmapCacheWindow",   bitmapCacheWindow,
+	             "spriteAttribTable",   spriteAttribTable,
+	             "spritePatternTable",  spritePatternTable);
 }
 INSTANTIATE_SERIALIZE_METHODS(VDPVRAM);
 
