@@ -3,11 +3,10 @@
 #include "Reactor.hh"
 #include "GlobalSettings.hh"
 #include "BooleanSetting.hh"
-#include "VideoSourceSetting.hh"
 #include "MSXEventDistributor.hh"
 #include "MSXMotherBoard.hh"
 #include "Event.hh"
-#include "memory.hh"
+#include "one_of.hh"
 
 namespace openmsx {
 
@@ -16,8 +15,7 @@ VideoLayer::VideoLayer(MSXMotherBoard& motherBoard_,
 	: motherBoard(motherBoard_)
 	, display(motherBoard.getReactor().getDisplay())
 	, videoSourceSetting(motherBoard.getVideoSource())
-	, videoSourceActivator(make_unique<VideoSourceActivator>(
-		videoSourceSetting, videoSource_))
+	, videoSourceActivator(videoSourceSetting, videoSource_)
 	, powerSetting(motherBoard.getReactor().getGlobalSettings().getPowerSetting())
 	, video9000Source(0)
 	, activeVideo9000(INACTIVE)
@@ -33,18 +31,16 @@ VideoLayer::VideoLayer(MSXMotherBoard& motherBoard_,
 
 VideoLayer::~VideoLayer()
 {
-	PRT_DEBUG("Destructing VideoLayer...");
 	motherBoard.getMSXEventDistributor().unregisterEventListener(*this);
 	powerSetting.detach(*this);
 	videoSourceSetting.detach(*this);
 
 	display.removeLayer(*this);
-	PRT_DEBUG("Destructing VideoLayer... DONE!");
 }
 
 int VideoLayer::getVideoSource() const
 {
-	return videoSourceActivator->getID();
+	return videoSourceActivator.getID();
 }
 int VideoLayer::getVideoSourceSetting() const
 {
@@ -69,22 +65,20 @@ void VideoLayer::calcZ()
 
 void VideoLayer::calcCoverage()
 {
-	Coverage coverage;
-
+	Coverage cov;
 	if (!powerSetting.getBoolean() || !motherBoard.isActive()) {
-		coverage = COVER_NONE;
+		cov = COVER_NONE;
 	} else {
-		coverage = COVER_FULL;
+		cov = COVER_FULL;
 	}
-
-	setCoverage(coverage);
+	setCoverage(cov);
 }
 
-void VideoLayer::signalEvent(const std::shared_ptr<const Event>& event,
-                             EmuTime::param /*time*/)
+void VideoLayer::signalMSXEvent(const std::shared_ptr<const Event>& event,
+                                EmuTime::param /*time*/)
 {
-	if ((event->getType() == OPENMSX_MACHINE_ACTIVATED) ||
-	    (event->getType() == OPENMSX_MACHINE_DEACTIVATED)) {
+	if (event->getType() == one_of(OPENMSX_MACHINE_ACTIVATED,
+		                       OPENMSX_MACHINE_DEACTIVATED)) {
 		calcCoverage();
 	}
 }
