@@ -3,9 +3,9 @@
 #include "ZMBVEncoder.hh"
 #include "FrameSource.hh"
 #include "PixelOperations.hh"
-#include "unreachable.hh"
 #include "endian.hh"
-#include <algorithm>
+#include "ranges.hh"
+#include "unreachable.hh"
 #include <cassert>
 #include <cstdlib>
 #include <cstring>
@@ -13,38 +13,38 @@
 
 namespace openmsx {
 
-static const uint8_t DBZV_VERSION_HIGH = 0;
-static const uint8_t DBZV_VERSION_LOW = 1;
-static const uint8_t COMPRESSION_ZLIB = 1;
-static const unsigned MAX_VECTOR = 16;
-static const unsigned BLOCK_WIDTH  = MAX_VECTOR;
-static const unsigned BLOCK_HEIGHT = MAX_VECTOR;
-static const unsigned FLAG_KEYFRAME = 0x01;
+constexpr uint8_t DBZV_VERSION_HIGH = 0;
+constexpr uint8_t DBZV_VERSION_LOW = 1;
+constexpr uint8_t COMPRESSION_ZLIB = 1;
+constexpr unsigned MAX_VECTOR = 16;
+constexpr unsigned BLOCK_WIDTH  = MAX_VECTOR;
+constexpr unsigned BLOCK_HEIGHT = MAX_VECTOR;
+constexpr unsigned FLAG_KEYFRAME = 0x01;
 
 struct CodecVector {
-	double cost() const {
-		double c = sqrt(double(x * x + y * y));
+	float cost() const {
+		float c = sqrtf(float(x * x + y * y));
 		if ((x == 0) || (y == 0)) {
 			// no penalty for purely horizontal/vertical offset
-			c *= 1.0;
+			c *= 1.0f;
 		} else if (abs(x) == abs(y)) {
 			// small penalty for pure diagonal
-			c *= 2.0;
+			c *= 2.0f;
 		} else {
 			// bigger penalty for 'random' direction
-			c *= 4.0;
+			c *= 4.0f;
 		}
 		return c;
 	}
 	int x;
 	int y;
 };
-bool operator<(const CodecVector& l, const CodecVector& r)
+static inline bool operator<(const CodecVector& l, const CodecVector& r)
 {
 	return l.cost() < r.cost();
 }
 
-static const unsigned VECTOR_TAB_SIZE =
+constexpr unsigned VECTOR_TAB_SIZE =
 	1 +                                       // center
 	8 * MAX_VECTOR +                          // horizontal, vertial, diagonal
 	MAX_VECTOR * MAX_VECTOR - 2 * MAX_VECTOR; // rest (only MAX_VECTOR/2)
@@ -59,8 +59,6 @@ struct KeyframeHeader {
 	uint8_t blockheight;
 };
 
-const char* ZMBVEncoder::CODEC_4CC = "ZMBV";
-
 
 static inline void writePixel(
 	const PixelOperations<uint16_t>& pixelOps,
@@ -86,47 +84,34 @@ static void createVectorTable()
 {
 	unsigned p = 0;
 	// center
-	vectorTable[p].x = 0;
-	vectorTable[p].y = 0;
+	vectorTable[p] = {0, 0};
 	p += 1;
 	// horizontal, vertial, diagonal
 	for (int i = 1; i <= int(MAX_VECTOR); ++i) {
-		vectorTable[p + 0].x =  i;
-		vectorTable[p + 0].y =  0;
-		vectorTable[p + 1].x = -i;
-		vectorTable[p + 1].y =  0;
-		vectorTable[p + 2].x =  0;
-		vectorTable[p + 2].y =  i;
-		vectorTable[p + 3].x =  0;
-		vectorTable[p + 3].y = -i;
-		vectorTable[p + 4].x =  i;
-		vectorTable[p + 4].y =  i;
-		vectorTable[p + 5].x = -i;
-		vectorTable[p + 5].y =  i;
-		vectorTable[p + 6].x =  i;
-		vectorTable[p + 6].y = -i;
-		vectorTable[p + 7].x = -i;
-		vectorTable[p + 7].y = -i;
+		vectorTable[p + 0] = { i, 0};
+		vectorTable[p + 1] = {-i, 0};
+		vectorTable[p + 2] = { 0, i};
+		vectorTable[p + 3] = { 0,-i};
+		vectorTable[p + 4] = { i, i};
+		vectorTable[p + 5] = {-i, i};
+		vectorTable[p + 6] = { i,-i};
+		vectorTable[p + 7] = {-i,-i};
 		p += 8;
 	}
 	// rest
 	for (int y = 1; y <= int(MAX_VECTOR / 2); ++y) {
 		for (int x = 1; x <= int(MAX_VECTOR / 2); ++x) {
 			if (x == y) continue; // already have diagonal
-			vectorTable[p + 0].x =  x;
-			vectorTable[p + 0].y =  y;
-			vectorTable[p + 1].x = -x;
-			vectorTable[p + 1].y =  y;
-			vectorTable[p + 2].x =  x;
-			vectorTable[p + 2].y = -y;
-			vectorTable[p + 3].x = -x;
-			vectorTable[p + 3].y = -y;
+			vectorTable[p + 0] = { x, y};
+			vectorTable[p + 1] = {-x, y};
+			vectorTable[p + 2] = { x,-y};
+			vectorTable[p + 3] = {-x,-y};
 			p += 4;
 		}
 	}
 	assert(p == VECTOR_TAB_SIZE);
 
-	std::sort(&vectorTable[0], &vectorTable[VECTOR_TAB_SIZE]);
+	ranges::sort(vectorTable);
 }
 
 ZMBVEncoder::ZMBVEncoder(unsigned width_, unsigned height_, unsigned bpp)
@@ -157,10 +142,6 @@ ZMBVEncoder::ZMBVEncoder(unsigned width_, unsigned height_, unsigned bpp)
 	// Level 6 seems a good compromise between size/speed for THIS test.
 }
 
-ZMBVEncoder::~ZMBVEncoder()
-{
-}
-
 void ZMBVEncoder::setupBuffers(unsigned bpp)
 {
 	switch (bpp) {
@@ -206,7 +187,7 @@ void ZMBVEncoder::setupBuffers(unsigned bpp)
 	}
 }
 
-unsigned ZMBVEncoder::neededSize()
+unsigned ZMBVEncoder::neededSize() const
 {
 	unsigned f = pixelSize;
 	f = f * width * height + 2 * (1 + (width / 8)) * (1 + (height / 8)) + 1024;
@@ -249,7 +230,7 @@ template<class P>
 void ZMBVEncoder::addXorBlock(
 	const PixelOperations<P>& pixelOps, int vx, int vy, unsigned offset, unsigned& workUsed)
 {
-	typedef typename Endian::Little<P>::type LE_P;
+	using LE_P = typename Endian::Little<P>::type;
 
 	auto* pold = &(reinterpret_cast<P*>(oldframe.data()))[offset + (vy * pitch) + vx];
 	auto* pnew = &(reinterpret_cast<P*>(newframe.data()))[offset];
@@ -265,13 +246,16 @@ void ZMBVEncoder::addXorBlock(
 }
 
 template<class P>
-void ZMBVEncoder::addXorFrame(const SDL_PixelFormat& pixelFormat, unsigned& workUsed)
+void ZMBVEncoder::addXorFrame(const PixelFormat& pixelFormat, unsigned& workUsed)
 {
 	PixelOperations<P> pixelOps(pixelFormat);
 	auto* vectors = reinterpret_cast<int8_t*>(&work[workUsed]);
 
+	unsigned xblocks = width / BLOCK_WIDTH;
+	unsigned yblocks = height / BLOCK_HEIGHT;
+	unsigned blockcount = xblocks * yblocks;
+
 	// Align the following xor data on 4 byte boundary
-	auto blockcount = unsigned(blockOffsets.size());
 	workUsed = (workUsed + blockcount * 2 + 3) & ~3;
 
 	int bestvx = 0;
@@ -306,9 +290,9 @@ void ZMBVEncoder::addXorFrame(const SDL_PixelFormat& pixelFormat, unsigned& work
 }
 
 template<class P>
-void ZMBVEncoder::addFullFrame(const SDL_PixelFormat& pixelFormat, unsigned& workUsed)
+void ZMBVEncoder::addFullFrame(const PixelFormat& pixelFormat, unsigned& workUsed)
 {
-	typedef typename Endian::Little<P>::type LE_P;
+	using LE_P = typename Endian::Little<P>::type;
 
 	PixelOperations<P> pixelOps(pixelFormat);
 	auto* readFrame =
@@ -324,16 +308,18 @@ void ZMBVEncoder::addFullFrame(const SDL_PixelFormat& pixelFormat, unsigned& wor
 	}
 }
 
-const void* ZMBVEncoder::getScaledLine(FrameSource* frame, unsigned y, void* buf_)
+const void* ZMBVEncoder::getScaledLine(FrameSource* frame, unsigned y, void* workBuf_) const
 {
 #if HAVE_32BPP
 	if (pixelSize == 4) { // 32bpp
-		auto* buf = static_cast<uint32_t*>(buf_);
+		auto* workBuf = static_cast<uint32_t*>(workBuf_);
 		switch (height) {
 		case 240:
-			return frame->getLinePtr320_240(y, buf);
+			return frame->getLinePtr320_240(y, workBuf);
 		case 480:
-			return frame->getLinePtr640_480(y, buf);
+			return frame->getLinePtr640_480(y, workBuf);
+		case 720:
+			return frame->getLinePtr960_720(y, workBuf);
 		default:
 			UNREACHABLE;
 		}
@@ -341,12 +327,14 @@ const void* ZMBVEncoder::getScaledLine(FrameSource* frame, unsigned y, void* buf
 #endif
 #if HAVE_16BPP
 	if (pixelSize == 2) { // 15bpp or 16bpp
-		auto* buf = static_cast<uint16_t*>(buf_);
+		auto* workBuf = static_cast<uint16_t*>(workBuf_);
 		switch (height) {
 		case 240:
-			return frame->getLinePtr320_240(y, buf);
+			return frame->getLinePtr320_240(y, workBuf);
 		case 480:
-			return frame->getLinePtr640_480(y, buf);
+			return frame->getLinePtr640_480(y, workBuf);
+		case 720:
+			return frame->getLinePtr960_720(y, workBuf);
 		default:
 			UNREACHABLE;
 		}
@@ -398,12 +386,12 @@ void ZMBVEncoder::compressFrame(bool keyFrame, FrameSource* frame,
 		switch (pixelSize) {
 #if HAVE_16BPP
 		case 2:
-			addFullFrame<uint16_t>(frame->getSDLPixelFormat(), workUsed);
+			addFullFrame<uint16_t>(frame->getPixelFormat(), workUsed);
 			break;
 #endif
 #if HAVE_32BPP
 		case 4:
-			addFullFrame<uint32_t>(frame->getSDLPixelFormat(), workUsed);
+			addFullFrame<uint32_t>(frame->getPixelFormat(), workUsed);
 			break;
 #endif
 		default:
@@ -414,12 +402,12 @@ void ZMBVEncoder::compressFrame(bool keyFrame, FrameSource* frame,
 		switch (pixelSize) {
 #if HAVE_16BPP
 		case 2:
-			addXorFrame<uint16_t>(frame->getSDLPixelFormat(), workUsed);
+			addXorFrame<uint16_t>(frame->getPixelFormat(), workUsed);
 			break;
 #endif
 #if HAVE_32BPP
 		case 4:
-			addXorFrame<uint32_t>(frame->getSDLPixelFormat(), workUsed);
+			addXorFrame<uint32_t>(frame->getPixelFormat(), workUsed);
 			break;
 #endif
 		default:
@@ -434,7 +422,8 @@ void ZMBVEncoder::compressFrame(bool keyFrame, FrameSource* frame,
 	zstream.next_out = static_cast<Bytef*>(writeBuf + writeDone);
 	zstream.avail_out = outputSize - writeDone;
 	zstream.total_out = 0;
-	deflate(&zstream, Z_SYNC_FLUSH);
+	auto r = deflate(&zstream, Z_SYNC_FLUSH);
+	assert(r == Z_OK); (void)r;
 
 	buffer = output.data();
 	written = writeDone + zstream.total_out;
