@@ -1,34 +1,33 @@
-// $Id:$
-
 #include "DirectScalerOutput.hh"
-#include "OutputSurface.hh"
+#include "SDLOutputSurface.hh"
 #include "MemoryOps.hh"
 #include "build-info.hh"
+#include <cstdint>
 
 namespace openmsx {
 
 template<typename Pixel>
-DirectScalerOutput<Pixel>::DirectScalerOutput(OutputSurface& output_)
-	: output(output_)
+DirectScalerOutput<Pixel>::DirectScalerOutput(SDLOutputSurface& output_)
+	: output(output_), pixelAccess(output.getDirectPixelAccess())
 {
 }
 
 template<typename Pixel>
 unsigned DirectScalerOutput<Pixel>::getWidth()  const
 {
-	return output.getWidth();
+	return output.getLogicalWidth();
 }
 
 template<typename Pixel>
 unsigned DirectScalerOutput<Pixel>::getHeight() const
 {
-	return output.getHeight();
+	return output.getLogicalHeight();
 }
 
 template<typename Pixel>
 Pixel* DirectScalerOutput<Pixel>::acquireLine(unsigned y)
 {
-	return output.getLinePtrDirect<Pixel>(y);
+	return pixelAccess.getLinePtr<Pixel>(y);
 }
 
 template<typename Pixel>
@@ -40,18 +39,18 @@ void DirectScalerOutput<Pixel>::releaseLine(unsigned /*y*/, Pixel* /*buf*/)
 template<typename Pixel>
 void DirectScalerOutput<Pixel>::fillLine(unsigned y, Pixel color)
 {
-	Pixel* dstLine = output.getLinePtrDirect<Pixel>(y);
-	MemoryOps::MemSet<Pixel, MemoryOps::STREAMING> memset;
-	memset(dstLine, output.getWidth(), color);
+	auto* dstLine = pixelAccess.getLinePtr<Pixel>(y);
+	MemoryOps::MemSet<Pixel> memset;
+	memset(dstLine, output.getLogicalWidth(), color);
 }
 
 
 // Force template instantiation.
 #if HAVE_16BPP
-template class DirectScalerOutput<word>;
+template class DirectScalerOutput<uint16_t>;
 #endif
 #if HAVE_32BPP
-template class DirectScalerOutput<unsigned>;
+template class DirectScalerOutput<uint32_t>;
 #endif
 
 } // namespace openmsx
