@@ -1,5 +1,3 @@
-// $Id: HQ3xLiteScaler.cc 12123 2011-04-17 20:30:07Z m9710797 $
-
 /*
 Original code: Copyright (C) 2003 MaxSt ( maxst@hiend3d.com )
 openMSX adaptation by Wouter Vermaelen
@@ -13,9 +11,9 @@ Visit the HiEnd3D site for info:
 #include "HQ3xLiteScaler.hh"
 #include "HQCommon.hh"
 #include "LineScalers.hh"
-#include "openmsx.hh"
 #include "unreachable.hh"
 #include "build-info.hh"
+#include <cstdint>
 
 namespace openmsx {
 
@@ -23,7 +21,8 @@ template <typename Pixel> struct HQLite_1x1on3x3
 {
 	void operator()(const Pixel* in0, const Pixel* in1, const Pixel* in2,
 	                Pixel* out0, Pixel* out1, Pixel* out2,
-	                unsigned srcWidth, unsigned* edgeBuf, EdgeHQLite edgeOp);
+	                unsigned srcWidth, unsigned* edgeBuf, EdgeHQLite edgeOp)
+	               __restrict;
 };
 
 template <typename Pixel>
@@ -102,7 +101,7 @@ void HQ3xLiteScaler<Pixel>::scale2x1to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_2on3<Pixel> > postScale(pixelOps);
+	PolyScale<Pixel, Scale_2on3<Pixel>> postScale(pixelOps);
 	EdgeHQLite edgeOp;
 	doHQScale3<Pixel>(HQLite_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -114,7 +113,7 @@ void HQ3xLiteScaler<Pixel>::scale1x1to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_1on1<Pixel> > postScale;
+	PolyScale<Pixel, Scale_1on1<Pixel>> postScale;
 	EdgeHQLite edgeOp;
 	doHQScale3<Pixel>(HQLite_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -126,7 +125,7 @@ void HQ3xLiteScaler<Pixel>::scale4x1to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_4on3<Pixel> > postScale(pixelOps);
+	PolyScale<Pixel, Scale_4on3<Pixel>> postScale(pixelOps);
 	EdgeHQLite edgeOp;
 	doHQScale3<Pixel>(HQLite_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -138,7 +137,7 @@ void HQ3xLiteScaler<Pixel>::scale2x1to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_2on1<Pixel> > postScale(pixelOps);
+	PolyScale<Pixel, Scale_2on1<Pixel>> postScale(pixelOps);
 	EdgeHQLite edgeOp;
 	doHQScale3<Pixel>(HQLite_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -150,7 +149,7 @@ void HQ3xLiteScaler<Pixel>::scale8x1to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_8on3<Pixel> > postScale(pixelOps);
+	PolyScale<Pixel, Scale_8on3<Pixel>> postScale(pixelOps);
 	EdgeHQLite edgeOp;
 	doHQScale3<Pixel>(HQLite_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -162,7 +161,7 @@ void HQ3xLiteScaler<Pixel>::scale4x1to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_4on1<Pixel> > postScale(pixelOps);
+	PolyScale<Pixel, Scale_4on1<Pixel>> postScale(pixelOps);
 	EdgeHQLite edgeOp;
 	doHQScale3<Pixel>(HQLite_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -171,10 +170,10 @@ void HQ3xLiteScaler<Pixel>::scale4x1to3x3(FrameSource& src,
 
 // Force template instantiation.
 #if HAVE_16BPP
-template class HQ3xLiteScaler<word>;
+template class HQ3xLiteScaler<uint16_t>;
 #endif
 #if HAVE_32BPP
-template class HQ3xLiteScaler<unsigned>;
+template class HQ3xLiteScaler<uint32_t>;
 #endif
 
 } // namespace openmsx
