@@ -1,5 +1,3 @@
-// $Id: HQ3xScaler.cc 12123 2011-04-17 20:30:07Z m9710797 $
-
 /*
 Original code: Copyright (C) 2003 MaxSt ( maxst@hiend3d.com )
 openMSX adaptation by Wouter Vermaelen
@@ -13,9 +11,9 @@ Visit the HiEnd3D site for info:
 #include "HQ3xScaler.hh"
 #include "HQCommon.hh"
 #include "LineScalers.hh"
-#include "openmsx.hh"
 #include "unreachable.hh"
 #include "build-info.hh"
+#include <cstdint>
 
 namespace openmsx {
 
@@ -23,7 +21,8 @@ template <typename Pixel> struct HQ_1x1on3x3
 {
 	void operator()(const Pixel* in0, const Pixel* in1, const Pixel* in2,
 	                Pixel* out0, Pixel* out1, Pixel* out2,
-	                unsigned srcWidth, unsigned* edgeBuf, EdgeHQ edgeOp);
+	                unsigned srcWidth, unsigned* edgeBuf, EdgeHQ edgeOp)
+	               __restrict;
 };
 
 template <typename Pixel>
@@ -105,7 +104,7 @@ void HQ3xScaler<Pixel>::scale2x1to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_2on3<Pixel> > postScale(pixelOps);
+	PolyScale<Pixel, Scale_2on3<Pixel>> postScale(pixelOps);
 	EdgeHQ edgeOp = createEdgeHQ(pixelOps);
 	doHQScale3<Pixel>(HQ_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -117,7 +116,7 @@ void HQ3xScaler<Pixel>::scale1x1to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_1on1<Pixel> > postScale;
+	PolyScale<Pixel, Scale_1on1<Pixel>> postScale;
 	EdgeHQ edgeOp = createEdgeHQ(pixelOps);
 	doHQScale3<Pixel>(HQ_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -129,7 +128,7 @@ void HQ3xScaler<Pixel>::scale4x1to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_4on3<Pixel> > postScale(pixelOps);
+	PolyScale<Pixel, Scale_4on3<Pixel>> postScale(pixelOps);
 	EdgeHQ edgeOp = createEdgeHQ(pixelOps);
 	doHQScale3<Pixel>(HQ_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -141,7 +140,7 @@ void HQ3xScaler<Pixel>::scale2x1to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_2on1<Pixel> > postScale(pixelOps);
+	PolyScale<Pixel, Scale_2on1<Pixel>> postScale(pixelOps);
 	EdgeHQ edgeOp = createEdgeHQ(pixelOps);
 	doHQScale3<Pixel>(HQ_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -153,7 +152,7 @@ void HQ3xScaler<Pixel>::scale8x1to9x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_8on3<Pixel> > postScale(pixelOps);
+	PolyScale<Pixel, Scale_8on3<Pixel>> postScale(pixelOps);
 	EdgeHQ edgeOp = createEdgeHQ(pixelOps);
 	doHQScale3<Pixel>(HQ_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -165,7 +164,7 @@ void HQ3xScaler<Pixel>::scale4x1to3x3(FrameSource& src,
 	unsigned srcStartY, unsigned srcEndY, unsigned srcWidth,
 	ScalerOutput<Pixel>& dst, unsigned dstStartY, unsigned dstEndY)
 {
-	PolyScale<Pixel, Scale_4on1<Pixel> > postScale(pixelOps);
+	PolyScale<Pixel, Scale_4on1<Pixel>> postScale(pixelOps);
 	EdgeHQ edgeOp = createEdgeHQ(pixelOps);
 	doHQScale3<Pixel>(HQ_1x1on3x3<Pixel>(), edgeOp, postScale,
 	                  src, srcStartY, srcEndY, srcWidth,
@@ -174,10 +173,10 @@ void HQ3xScaler<Pixel>::scale4x1to3x3(FrameSource& src,
 
 // Force template instantiation.
 #if HAVE_16BPP
-template class HQ3xScaler<word>;
+template class HQ3xScaler<uint16_t>;
 #endif
 #if HAVE_32BPP
-template class HQ3xScaler<unsigned>;
+template class HQ3xScaler<uint32_t>;
 #endif
 
 } // namespace openmsx
