@@ -7,7 +7,6 @@
 #include "SaI3xScaler.hh"
 #include "FrameSource.hh"
 #include "ScalerOutput.hh"
-#include "aligned.hh"
 #include "vla.hh"
 #include "build-info.hh"
 #include <cassert>
@@ -32,7 +31,7 @@ void SaI3xScaler<Pixel>::scaleBlank1to3(
 	                  ? dstEndY : dstEndY - 3;
 	unsigned srcY = srcStartY, dstY = dstStartY;
 	for (/* */; dstY < stopDstY; srcY += 1, dstY += 3) {
-		Pixel color = src.getLineColor<Pixel>(srcY);
+		auto color = src.getLineColor<Pixel>(srcY);
 		for (int i = 0; i < 3; ++i) {
 			dst.fillLine(dstY + i, color);
 		}
@@ -52,8 +51,8 @@ inline Pixel SaI3xScaler<Pixel>::blend(Pixel p1, Pixel p2)
 	return pixelOps.template blend<1, 1>(p1, p2);
 }
 
-static const unsigned redblueMask = 0xF81F;
-static const unsigned greenMask = 0x7E0;
+constexpr unsigned redblueMask = 0xF81F;
+constexpr unsigned greenMask = 0x7E0;
 
 // TODO use PixelOperations::lerp()
 template <typename Pixel>
@@ -146,8 +145,8 @@ public:
 
 template <unsigned X, unsigned OLD, unsigned NEW>
 struct Round {
-	static_assert(OLD > NEW, "!");
-	static const unsigned result =
+	static_assert(OLD > NEW);
+	static constexpr unsigned result =
 		(X >> (OLD - NEW)) + ((X >> (OLD - NEW - 1)) & 1);
 };
 
