@@ -4,44 +4,22 @@
 #include "V9990VRAM.hh"
 #include "V9990CmdEngine.hh"
 #include "V9990Renderer.hh"
-#include "SimpleDebuggable.hh"
 #include "Reactor.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
-#include "memory.hh"
 #include <cassert>
 #include <cstring>
+#include <memory>
 
 namespace openmsx {
 
-class V9990RegDebug final : public SimpleDebuggable
-{
-public:
-	explicit V9990RegDebug(V9990& v9990);
-	byte read(unsigned address) override;
-	void write(unsigned address, byte value, EmuTime::param time) override;
-private:
-	V9990& v9990;
-};
-
-class V9990PalDebug final : public SimpleDebuggable
-{
-public:
-	explicit V9990PalDebug(V9990& v9990);
-	byte read(unsigned address) override;
-	void write(unsigned address, byte value, EmuTime::param time) override;
-private:
-	V9990& v9990;
-};
-
-
-static const byte ALLOW_READ  = 1;
-static const byte ALLOW_WRITE = 2;
-static const byte NO_ACCESS = 0;
-static const byte RD_ONLY   = ALLOW_READ;
-static const byte WR_ONLY   = ALLOW_WRITE;
-static const byte RD_WR     = ALLOW_READ | ALLOW_WRITE;
-static const byte regAccess[64] = {
+constexpr byte ALLOW_READ  = 1;
+constexpr byte ALLOW_WRITE = 2;
+constexpr byte NO_ACCESS = 0;
+constexpr byte RD_ONLY   = ALLOW_READ;
+constexpr byte WR_ONLY   = ALLOW_WRITE;
+constexpr byte RD_WR     = ALLOW_READ | ALLOW_WRITE;
+constexpr byte regAccess[64] = {
 	WR_ONLY, WR_ONLY, WR_ONLY,          // VRAM Write Address
 	WR_ONLY, WR_ONLY, WR_ONLY,          // VRAM Read Address
 	RD_WR, RD_WR,                       // Screen Mode
@@ -75,13 +53,18 @@ static const byte regAccess[64] = {
 
 V9990::V9990(const DeviceConfig& config)
 	: MSXDevice(config)
-	, Schedulable(MSXDevice::getScheduler())
-	, v9990RegDebug(make_unique<V9990RegDebug>(*this))
-	, v9990PalDebug(make_unique<V9990PalDebug>(*this))
+	, syncVSync(*this)
+	, syncDisplayStart(*this)
+	, syncVScan(*this)
+	, syncHScan(*this)
+	, syncSetMode(*this)
+	, syncCmdEnd(*this)
+	, v9990RegDebug(*this)
+	, v9990PalDebug(*this)
 	, irq(getMotherBoard(), getName() + ".IRQ")
 	, display(getReactor().getDisplay())
-	, frameStartTime(Schedulable::getCurrentTime())
-	, hScanSyncTime(Schedulable::getCurrentTime())
+	, frameStartTime(getCurrentTime())
+	, hScanSyncTime(getCurrentTime())
 	, pendingIRQs(0)
 	, externalVideoSource(false)
 {
@@ -98,11 +81,11 @@ V9990::V9990(const DeviceConfig& config)
 	}
 
 	// create VRAM
-	EmuTime::param time = Schedulable::getCurrentTime();
-	vram = make_unique<V9990VRAM>(*this, time);
+	EmuTime::param time = getCurrentTime();
+	vram = std::make_unique<V9990VRAM>(*this, time);
 
 	// create Command Engine
-	cmdEngine = make_unique<V9990CmdEngine>(
+	cmdEngine = std::make_unique<V9990CmdEngine>(
 		*this, time, display.getRenderSettings());
 	vram->setCmdEngine(*cmdEngine);
 
@@ -143,11 +126,12 @@ void V9990::powerUp(EmuTime::param time)
 
 void V9990::reset(EmuTime::param time)
 {
-	removeSyncPoint(V9990_VSYNC);
-	removeSyncPoint(V9990_DISPLAY_START);
-	removeSyncPoint(V9990_VSCAN);
-	removeSyncPoint(V9990_HSCAN);
-	removeSyncPoint(V9990_SET_MODE);
+	syncVSync       .removeSyncPoint();
+	syncDisplayStart.removeSyncPoint();
+	syncVScan       .removeSyncPoint();
+	syncHScan       .removeSyncPoint();
+	syncSetMode     .removeSyncPoint();
+	syncCmdEnd      .removeSyncPoint();
 
 	// Clear registers / ports
 	memset(regs, 0, sizeof(regs));
@@ -394,7 +378,7 @@ void V9990::writeIO(word port, byte val, EmuTime::param time)
 			// TODO investigate: does switching overscan mode
 			//      happen at next line or next frame
 			status = (status & 0xFB) | ((val & 1) << 2);
-			syncAtNextLine(V9990_SET_MODE, time);
+			syncAtNextLine(syncSetMode, time);
 
 			bool newSystemReset = (val & 2) != 0;
 			if (newSystemReset != systemReset) {
@@ -433,42 +417,55 @@ void V9990::writeIO(word port, byte val, EmuTime::param time)
 // Schedulable
 // -------------------------------------------------------------------------
 
-void V9990::executeUntil(EmuTime::param time, int userData)
+void V9990::execVSync(EmuTime::param time)
 {
-	switch (userData)  {
-	case V9990_VSYNC:
-		// Transition from one frame to the next
-		renderer->frameEnd(time);
-		frameStart(time);
-		break;
+	// Transition from one frame to the next
+	renderer->frameEnd(time);
+	frameStart(time);
+}
 
-	case V9990_DISPLAY_START:
-		if (displayEnabled) {
-			renderer->updateDisplayEnabled(true, time);
+void V9990::execDisplayStart(EmuTime::param time)
+{
+	if (displayEnabled) {
+		renderer->updateDisplayEnabled(true, time);
+	}
+	isDisplayArea = true;
+}
+
+void V9990::execVScan(EmuTime::param time)
+{
+	if (isDisplayEnabled()) {
+		renderer->updateDisplayEnabled(false, time);
+	}
+	isDisplayArea = false;
+	raiseIRQ(VER_IRQ);
+}
+
+void V9990::execHScan()
+{
+	raiseIRQ(HOR_IRQ);
+}
+
+void V9990::execSetMode(EmuTime::param time)
+{
+	calcDisplayMode();
+	renderer->setDisplayMode(getDisplayMode(), time);
+	renderer->setColorMode(getColorMode(), time);
+}
+
+void V9990::execCheckCmdEnd(EmuTime::param time)
+{
+	cmdEngine->sync(time);
+	scheduleCmdEnd(time); // in case of underestimation
+}
+
+void V9990::scheduleCmdEnd(EmuTime::param time)
+{
+	if (regs[INTERRUPT_0] & 4) {
+		auto next = cmdEngine->estimateCmdEnd();
+		if (next > time) {
+			syncCmdEnd.setSyncPoint(next);
 		}
-		isDisplayArea = true;
-		break;
-
-	case V9990_VSCAN:
-		if (isDisplayEnabled()) {
-			renderer->updateDisplayEnabled(false, time);
-		}
-		isDisplayArea = false;
-		raiseIRQ(VER_IRQ);
-		break;
-
-	case V9990_HSCAN:
-		raiseIRQ(HOR_IRQ);
-		break;
-
-	case V9990_SET_MODE:
-		calcDisplayMode();
-		renderer->setDisplayMode(getDisplayMode(), time);
-		renderer->setColorMode(getColorMode(), time);
-		break;
-
-	default:
-		UNREACHABLE;
 	}
 }
 
@@ -483,51 +480,53 @@ void V9990::preVideoSystemChange()
 
 void V9990::postVideoSystemChange()
 {
-	EmuTime::param time = Schedulable::getCurrentTime();
+	EmuTime::param time = getCurrentTime();
 	createRenderer(time);
 	renderer->frameStart(time);
 }
 
 // -------------------------------------------------------------------------
-// V9990RegDebug
+// RegDebug
 // -------------------------------------------------------------------------
 
-V9990RegDebug::V9990RegDebug(V9990& v9990_)
+V9990::RegDebug::RegDebug(V9990& v9990_)
 	: SimpleDebuggable(v9990_.getMotherBoard(),
 	                   v9990_.getName() + " regs", "V9990 registers", 0x40)
-	, v9990(v9990_)
 {
 }
 
-byte V9990RegDebug::read(unsigned address)
+byte V9990::RegDebug::read(unsigned address)
 {
+	auto& v9990 = OUTER(V9990, v9990RegDebug);
 	return v9990.regs[address];
 }
 
-void V9990RegDebug::write(unsigned address, byte value, EmuTime::param time)
+void V9990::RegDebug::write(unsigned address, byte value, EmuTime::param time)
 {
+	auto& v9990 = OUTER(V9990, v9990RegDebug);
 	v9990.writeRegister(address, value, time);
 }
 
 // -------------------------------------------------------------------------
-// V9990PalDebug
+// PalDebug
 // -------------------------------------------------------------------------
 
-V9990PalDebug::V9990PalDebug(V9990& v9990_)
+V9990::PalDebug::PalDebug(V9990& v9990_)
 	: SimpleDebuggable(v9990_.getMotherBoard(),
 	                   v9990_.getName() + " palette",
 	                   "V9990 palette (format is R, G, B, 0).", 0x100)
-	, v9990(v9990_)
 {
 }
 
-byte V9990PalDebug::read(unsigned address)
+byte V9990::PalDebug::read(unsigned address)
 {
+	auto& v9990 = OUTER(V9990, v9990PalDebug);
 	return v9990.palette[address];
 }
 
-void V9990PalDebug::write(unsigned address, byte value, EmuTime::param time)
+void V9990::PalDebug::write(unsigned address, byte value, EmuTime::param time)
 {
+	auto& v9990 = OUTER(V9990, v9990PalDebug);
 	v9990.writePaletteRegister(address, value, time);
 }
 
@@ -571,19 +570,19 @@ byte V9990::readRegister(byte reg, EmuTime::param time) const
 	return result;
 }
 
-void V9990::syncAtNextLine(V9990SyncType type, EmuTime::param time)
+void V9990::syncAtNextLine(SyncBase& type, EmuTime::param time)
 {
 	int line = getUCTicksThisFrame(time) / V9990DisplayTiming::UC_TICKS_PER_LINE;
 	int ticks = (line + 1) * V9990DisplayTiming::UC_TICKS_PER_LINE;
 	EmuTime nextTime = frameStartTime + ticks;
-	setSyncPoint(nextTime, type);
+	type.setSyncPoint(nextTime);
 }
 
 void V9990::writeRegister(byte reg, byte val, EmuTime::param time)
 {
 	// Found this table by writing 0xFF to a register and reading
 	// back the value (only works for read/write registers)
-	static const byte regWriteMask[32] = {
+	static constexpr byte regWriteMask[32] = {
 		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
 		0xFF, 0x87, 0xFF, 0x83, 0x0F, 0xFF, 0xFF, 0xFF,
 		0xFF, 0xFF, 0xDF, 0x07, 0xFF, 0xFF, 0xC1, 0x07,
@@ -597,6 +596,9 @@ void V9990::writeRegister(byte reg, byte val, EmuTime::param time)
 	}
 	if (reg >= CMD_PARAM_SRC_ADDRESS_0) {
 		cmdEngine->setCmdReg(reg, val, time);
+		if (reg == CMD_PARAM_OPCODE) {
+			scheduleCmdEnd(time);
+		}
 		return;
 	}
 
@@ -612,7 +614,7 @@ void V9990::writeRegister(byte reg, byte val, EmuTime::param time)
 		case SCREEN_MODE_0:
 		case SCREEN_MODE_1:
 			// TODO verify this on real V9990
-			syncAtNextLine(V9990_SET_MODE, time);
+			syncAtNextLine(syncSetMode, time);
 			break;
 		case PALETTE_CONTROL:
 			renderer->setColorMode(getColorMode(val), time);
@@ -654,12 +656,15 @@ void V9990::writeRegister(byte reg, byte val, EmuTime::param time)
 			// update read buffer immediately after read pointer changes. TODO: timing?
 			vramReadBuffer = vram->readVRAMCPU(vramReadPtr, time);
 			break;
+		case SCREEN_MODE_0:
+		case SCREEN_MODE_1:
+		case CONTROL:
+			// These influence the command timing
+			scheduleCmdEnd(time);
+			break;
 		case INTERRUPT_0:
-			if (pendingIRQs & val) {
-				irq.set();
-			} else {
-				irq.reset();
-			}
+			irq.set((pendingIRQs & val) != 0);
+			scheduleCmdEnd(time);
 			break;
 		case INTERRUPT_1:
 		case INTERRUPT_2:
@@ -723,19 +728,16 @@ void V9990::frameStart(EmuTime::param time)
 	frameStartTime.reset(time);
 
 	// schedule next VSYNC
-	setSyncPoint(
-		frameStartTime + V9990DisplayTiming::getUCTicksPerFrame(palTiming),
-		V9990_VSYNC);
+	syncVSync.setSyncPoint(
+		frameStartTime + V9990DisplayTiming::getUCTicksPerFrame(palTiming));
 
 	// schedule DISPLAY_START
-	setSyncPoint(
-	    frameStartTime + getTopBorder() * V9990DisplayTiming::UC_TICKS_PER_LINE,
-	    V9990_DISPLAY_START);
+	syncDisplayStart.setSyncPoint(
+	    frameStartTime + getTopBorder() * V9990DisplayTiming::UC_TICKS_PER_LINE);
 
 	// schedule VSCAN
-	setSyncPoint(
-	    frameStartTime + getBottomBorder() * V9990DisplayTiming::UC_TICKS_PER_LINE,
-	    V9990_VSCAN);
+	syncVScan.setSyncPoint(
+	    frameStartTime + getBottomBorder() * V9990DisplayTiming::UC_TICKS_PER_LINE);
 
 	renderer->frameStart(time);
 }
@@ -786,31 +788,25 @@ void V9990::setVerticalTiming()
 
 V9990ColorMode V9990::getColorMode(byte pal_ctrl) const
 {
-	V9990ColorMode mode = INVALID_COLOR_MODE;
-
 	if (!(regs[SCREEN_MODE_0] & 0x80)) {
-		mode = BP4;
+		return BP4;
 	} else {
 		switch (regs[SCREEN_MODE_0] & 0x03) {
-			case 0x00: mode = BP2; break;
-			case 0x01: mode = BP4; break;
+			case 0x00: return BP2;
+			case 0x01: return BP4;
 			case 0x02:
 				switch (pal_ctrl & 0xC0) {
-					case 0x00: mode = BP6; break;
-					case 0x40: mode = BD8; break;
-					case 0x80: mode = BYJK; break;
-					case 0xC0: mode = BYUV; break;
-					default: UNREACHABLE;
+					case 0x00: return BP6;
+					case 0x40: return BD8;
+					case 0x80: return BYJK;
+					case 0xC0: return BYUV;
 				}
 				break;
-			case 0x03: mode = BD16; break;
-			default: UNREACHABLE;
+			case 0x03: return BD16;
 		}
 	}
-
-	// TODO Check
-	if (mode == INVALID_COLOR_MODE) mode = BP4;
-	return mode;
+	UNREACHABLE;
+	return INVALID_COLOR_MODE;
 }
 
 V9990ColorMode V9990::getColorMode() const
@@ -829,7 +825,7 @@ void V9990::calcDisplayMode()
 			mode = P2;
 			break;
 		case 0x80:
-			if(status & 0x04) { // MCLK timing
+			if (status & 0x04) { // MCLK timing
 				switch(regs[SCREEN_MODE_0] & 0x30) {
 				case 0x00: mode = B0; break;
 				case 0x10: mode = B2; break;
@@ -861,7 +857,7 @@ void V9990::scheduleHscan(EmuTime::param time)
 {
 	// remove pending HSCAN, if any
 	if (hScanSyncTime > time) {
-		removeSyncPoint(V9990_HSCAN);
+		syncHScan.removeSyncPoint();
 		hScanSyncTime = time;
 	}
 
@@ -887,10 +883,10 @@ void V9990::scheduleHscan(EmuTime::param time)
 	}
 
 	hScanSyncTime = frameStartTime + offset;
-	setSyncPoint(hScanSyncTime, V9990_HSCAN);
+	syncHScan.setSyncPoint(hScanSyncTime);
 }
 
-static enum_string<V9990DisplayMode> displayModeInfo[] = {
+static std::initializer_list<enum_string<V9990DisplayMode>> displayModeInfo = {
 	{ "INVALID", INVALID_DISPLAY_MODE },
 	{ "P1", P1 }, { "P2", P2 },
 	{ "B0", B0 }, { "B1", B1 }, { "B2", B2 }, { "B3", B3 },
@@ -898,32 +894,48 @@ static enum_string<V9990DisplayMode> displayModeInfo[] = {
 };
 SERIALIZE_ENUM(V9990DisplayMode, displayModeInfo);
 
-// version 1:  initial version
-// version 2:  added systemReset
-// version 3:  added vramReadPtr, vramWritePtr, vramReadBuffer
+// version 1: initial version
+// version 2: added systemReset
+// version 3: added vramReadPtr, vramWritePtr, vramReadBuffer
+// version 4: removed 'userData' from Schedulable
+// version 5: added syncCmdEnd
 template<typename Archive>
 void V9990::serialize(Archive& ar, unsigned version)
 {
 	ar.template serializeBase<MSXDevice>(*this);
-	ar.template serializeBase<Schedulable>(*this);
 
-	ar.serialize("vram", *vram);
-	ar.serialize("cmdEngine", *cmdEngine);
-	ar.serialize("irq", irq);
-	ar.serialize("frameStartTime", frameStartTime);
-	ar.serialize("hScanSyncTime", hScanSyncTime);
-	ar.serialize("displayMode", mode);
+	if (ar.versionAtLeast(version, 4)) {
+		ar.serialize("syncVSync",        syncVSync,
+		             "syncDisplayStart", syncDisplayStart,
+		             "syncVScan",        syncVScan,
+		             "syncHScan",        syncHScan,
+		             "syncSetMode",      syncSetMode);
+	} else {
+		Schedulable::restoreOld(ar,
+			{&syncVSync, &syncDisplayStart, &syncVScan,
+			 &syncHScan, &syncSetMode});
+	}
+	if (ar.versionAtLeast(version, 5)) {
+		ar.serialize("syncCmdEnd", syncCmdEnd);
+	}
+
+	ar.serialize("displayMode", mode); // must be deserialized before cmdEngine (because it's used to restore some derived state in cmdEngine)
+	ar.serialize("vram",           *vram,
+	             "cmdEngine",      *cmdEngine,
+	             "irq",            irq,
+	             "frameStartTime", frameStartTime,
+	             "hScanSyncTime",  hScanSyncTime);
 	ar.serialize_blob("palette", palette, sizeof(palette));
-	ar.serialize("status", status);
-	ar.serialize("pendingIRQs", pendingIRQs);
+	ar.serialize("status",      status,
+	             "pendingIRQs", pendingIRQs);
 	ar.serialize_blob("registers", regs, sizeof(regs));
-	ar.serialize("regSelect", regSelect);
-	ar.serialize("palTiming", palTiming);
-	ar.serialize("interlaced", interlaced);
-	ar.serialize("isDisplayArea", isDisplayArea);
-	ar.serialize("displayEnabled", displayEnabled);
-	ar.serialize("scrollAYHigh", scrollAYHigh);
-	ar.serialize("scrollBYHigh", scrollBYHigh);
+	ar.serialize("regSelect",      regSelect,
+	             "palTiming",      palTiming,
+	             "interlaced",     interlaced,
+	             "isDisplayArea",  isDisplayArea,
+	             "displayEnabled", displayEnabled,
+	             "scrollAYHigh",   scrollAYHigh,
+	             "scrollBYHigh",   scrollBYHigh);
 
 	if (ar.versionBelow(version, 2)) {
 		systemReset = false;
@@ -934,11 +946,11 @@ void V9990::serialize(Archive& ar, unsigned version)
 	if (ar.versionBelow(version, 3)) {
 		vramReadPtr = getVRAMAddr(VRAM_READ_ADDRESS_0);
 		vramWritePtr = getVRAMAddr(VRAM_WRITE_ADDRESS_0);
-		vramReadBuffer = vram->readVRAMCPU(vramReadPtr, Schedulable::getCurrentTime());
+		vramReadBuffer = vram->readVRAMCPU(vramReadPtr, getCurrentTime());
 	} else {
-		ar.serialize("vramReadPtr", vramReadPtr);
-		ar.serialize("vramWritePtr", vramWritePtr);
-		ar.serialize("vramReadBuffer", vramReadBuffer);
+		ar.serialize("vramReadPtr",    vramReadPtr,
+		             "vramWritePtr",   vramWritePtr,
+		             "vramReadBuffer", vramReadBuffer);
 	}
 
 	// No need to serialize 'externalVideoSource', it will be restored when
@@ -956,7 +968,7 @@ void V9990::serialize(Archive& ar, unsigned version)
 		setHorizontalTiming();
 		setVerticalTiming();
 
-		renderer->reset(Schedulable::getCurrentTime());
+		renderer->reset(getCurrentTime());
 	}
 }
 INSTANTIATE_SERIALIZE_METHODS(V9990);
