@@ -15,9 +15,9 @@
 #include "Reactor.hh"
 #include "RenderSettings.hh"
 #include "IntegerSetting.hh"
-#include "BooleanSetting.hh"
-#include "EnumSetting.hh"
+#include "one_of.hh"
 #include "unreachable.hh"
+#include <cassert>
 
 namespace openmsx {
 
@@ -37,14 +37,14 @@ V9990PixelRenderer::V9990PixelRenderer(V9990& vdp_)
 
 	reset(vdp.getMotherBoard().getCurrentTime());
 
-	renderSettings.getMaxFrameSkip().attach(*this);
-	renderSettings.getMinFrameSkip().attach(*this);
+	renderSettings.getMaxFrameSkipSetting().attach(*this);
+	renderSettings.getMinFrameSkipSetting().attach(*this);
 }
 
 V9990PixelRenderer::~V9990PixelRenderer()
 {
-	renderSettings.getMaxFrameSkip().detach(*this);
-	renderSettings.getMinFrameSkip().detach(*this);
+	renderSettings.getMaxFrameSkipSetting().detach(*this);
+	renderSettings.getMinFrameSkipSetting().detach(*this);
 }
 
 PostProcessor* V9990PixelRenderer::getPostProcessor() const
@@ -70,16 +70,14 @@ void V9990PixelRenderer::frameStart(EmuTime::param time)
 		return;
 	}
 	prevDrawFrame = drawFrame;
-	if (vdp.isInterlaced() && renderSettings.getDeinterlace().getBoolean() &&
+	if (vdp.isInterlaced() && renderSettings.getDeinterlace() &&
 	    vdp.getEvenOdd() && vdp.isEvenOddEnabled()) {
 		// deinterlaced odd frame, do same as even frame
 	} else {
-		if (frameSkipCounter <
-		              renderSettings.getMinFrameSkip().getInt()) {
+		if (frameSkipCounter < renderSettings.getMinFrameSkip()) {
 			++frameSkipCounter;
 			drawFrame = false;
-		} else if (frameSkipCounter >=
-		              renderSettings.getMaxFrameSkip().getInt()) {
+		} else if (frameSkipCounter >= renderSettings.getMaxFrameSkip()) {
 			frameSkipCounter = 0;
 			drawFrame = true;
 		} else {
@@ -97,7 +95,7 @@ void V9990PixelRenderer::frameStart(EmuTime::param time)
 	}
 	if (!drawFrame) return;
 
-	accuracy = renderSettings.getAccuracy().getEnum();
+	accuracy = renderSettings.getAccuracy();
 	lastX = 0;
 	lastY = 0;
 	verticalOffsetA = verticalOffsetB = vdp.getTopBorder();
@@ -118,12 +116,12 @@ void V9990PixelRenderer::frameEnd(EmuTime::param time)
 		rasterizer->frameEnd(time);
 		auto time2 = Timer::getTime();
 		auto current = time2 - time1;
-		const double ALPHA = 0.2;
+		const float ALPHA = 0.2f;
 		finishFrameDuration = finishFrameDuration * (1 - ALPHA) +
 		                      current * ALPHA;
 
 		if (vdp.isInterlaced() && vdp.isEvenOddEnabled() &&
-		    renderSettings.getDeinterlace().getBoolean() &&
+		    renderSettings.getDeinterlace() &&
 		    !prevDrawFrame) {
 			// dont send event in deinterlace mode when
 			// previous frame was not rendered
@@ -131,11 +129,14 @@ void V9990PixelRenderer::frameEnd(EmuTime::param time)
 		}
 
 	}
-	eventDistributor.distributeEvent(
-		std::make_shared<FinishFrameEvent>(
-			rasterizer->getPostProcessor()->getVideoSource(),
-			videoSourceSetting.getSource(),
-			skipEvent));
+	if (vdp.getMotherBoard().isActive() &&
+	    !vdp.getMotherBoard().isFastForwarding()) {
+		eventDistributor.distributeEvent(
+			std::make_shared<FinishFrameEvent>(
+				rasterizer->getPostProcessor()->getVideoSource(),
+				videoSourceSetting.getSource(),
+				skipEvent));
+	}
 }
 
 void V9990PixelRenderer::sync(EmuTime::param time, bool force)
@@ -318,13 +319,11 @@ void V9990PixelRenderer::updateScrollBYLow(EmuTime::param time)
 
 void V9990PixelRenderer::update(const Setting& setting)
 {
-	if (&setting == &renderSettings.getMinFrameSkip() ||
-	    &setting == &renderSettings.getMaxFrameSkip()) {
-		// Force drawing of frame
-		frameSkipCounter = 999;
-	} else {
-		UNREACHABLE;
-	}
+	assert(&setting == one_of(&renderSettings.getMinFrameSkipSetting(),
+	                          &renderSettings.getMaxFrameSkipSetting()));
+	(void)setting;
+	// Force drawing of frame
+	frameSkipCounter = 999;
 }
 
 } // namespace openmsx
