@@ -2,22 +2,16 @@
 #include "V9990.hh"
 #include "RawFrame.hh"
 #include "PostProcessor.hh"
-#include "V9990BitmapConverter.hh"
-#include "V9990P1Converter.hh"
-#include "V9990P2Converter.hh"
-#include "BooleanSetting.hh"
-#include "FloatSetting.hh"
-#include "StringSetting.hh"
-#include "MSXMotherBoard.hh"
 #include "Display.hh"
-#include "VisibleSurface.hh"
+#include "OutputSurface.hh"
 #include "RenderSettings.hh"
 #include "MemoryOps.hh"
-#include "memory.hh"
 #include "build-info.hh"
 #include "components.hh"
+#include "one_of.hh"
 #include <algorithm>
 #include <cstdint>
+#include <memory>
 
 using std::min;
 using std::max;
@@ -26,36 +20,35 @@ namespace openmsx {
 
 template <class Pixel>
 V9990SDLRasterizer<Pixel>::V9990SDLRasterizer(
-		V9990& vdp_, Display& display, VisibleSurface& screen_,
+		V9990& vdp_, Display& display, OutputSurface& screen_,
 		std::unique_ptr<PostProcessor> postProcessor_)
 	: vdp(vdp_), vram(vdp.getVRAM())
 	, screen(screen_)
-	, workFrame(make_unique<RawFrame>(screen.getSDLFormat(), 1280, 240))
+	, workFrame(std::make_unique<RawFrame>(screen.getPixelFormat(), 1280, 240))
 	, renderSettings(display.getRenderSettings())
 	, displayMode(P1) // dummy value
 	, colorMode(PP)   //   avoid UMR
 	, postProcessor(std::move(postProcessor_))
-	, bitmapConverter(make_unique<V9990BitmapConverter<Pixel>>(
-		vdp, palette64, palette256, palette32768))
-	, p1Converter(make_unique<V9990P1Converter<Pixel>>(vdp, palette64))
-	, p2Converter(make_unique<V9990P2Converter<Pixel>>(vdp, palette64))
+	, bitmapConverter(vdp, palette64, palette64_32768, palette256, palette256_32768, palette32768)
+	, p1Converter(vdp, palette64)
+	, p2Converter(vdp, palette64)
 {
 	// Fill palettes
 	preCalcPalettes();
 
-	renderSettings.getGamma()      .attach(*this);
-	renderSettings.getBrightness() .attach(*this);
-	renderSettings.getContrast()   .attach(*this);
-	renderSettings.getColorMatrix().attach(*this);
+	renderSettings.getGammaSetting()      .attach(*this);
+	renderSettings.getBrightnessSetting() .attach(*this);
+	renderSettings.getContrastSetting()   .attach(*this);
+	renderSettings.getColorMatrixSetting().attach(*this);
 }
 
 template <class Pixel>
 V9990SDLRasterizer<Pixel>::~V9990SDLRasterizer()
 {
-	renderSettings.getColorMatrix().detach(*this);
-	renderSettings.getGamma()      .detach(*this);
-	renderSettings.getBrightness() .detach(*this);
-	renderSettings.getContrast()   .detach(*this);
+	renderSettings.getColorMatrixSetting().detach(*this);
+	renderSettings.getGammaSetting()      .detach(*this);
+	renderSettings.getBrightnessSetting() .detach(*this);
+	renderSettings.getContrastSetting()   .detach(*this);
 }
 
 template <class Pixel>
@@ -114,14 +107,14 @@ template <class Pixel>
 void V9990SDLRasterizer<Pixel>::setDisplayMode(V9990DisplayMode mode)
 {
 	displayMode = mode;
-	bitmapConverter->setColorMode(colorMode, displayMode);
+	bitmapConverter.setColorMode(colorMode, displayMode);
 }
 
 template <class Pixel>
 void V9990SDLRasterizer<Pixel>::setColorMode(V9990ColorMode mode)
 {
 	colorMode = mode;
-	bitmapConverter->setColorMode(colorMode, displayMode);
+	bitmapConverter.setColorMode(colorMode, displayMode);
 }
 
 template <class Pixel>
@@ -196,7 +189,7 @@ void V9990SDLRasterizer<Pixel>::drawDisplay(
 
 	if ((toX > fromX) && (toY > fromY)) {
 		bool drawSprites = vdp.spritesEnabled() &&
-			!renderSettings.getDisableSprites().getBoolean();
+			!renderSettings.getDisableSprites();
 
 		displayX = V9990::UCtoX(displayX, displayMode);
 		int displayWidth  = toX - fromX;
@@ -229,9 +222,9 @@ void V9990SDLRasterizer<Pixel>::drawP1Mode(
 {
 	while (displayHeight--) {
 		Pixel* pixelPtr = workFrame->getLinePtrDirect<Pixel>(fromY) + fromX;
-		p1Converter->convertLine(pixelPtr, displayX, displayWidth,
-		                         displayY, displayYA, displayYB,
-					 drawSprites);
+		p1Converter.convertLine(pixelPtr, displayX, displayWidth,
+		                        displayY, displayYA, displayYB,
+		                        drawSprites);
 		workFrame->setLineWidth(fromY, 320);
 		++fromY;
 		++displayY;
@@ -247,8 +240,8 @@ void V9990SDLRasterizer<Pixel>::drawP2Mode(
 {
 	while (displayHeight--) {
 		Pixel* pixelPtr = workFrame->getLinePtrDirect<Pixel>(fromY) + fromX;
-		p2Converter->convertLine(pixelPtr, displayX, displayWidth,
-		                         displayY, displayYA, drawSprites);
+		p2Converter.convertLine(pixelPtr, displayX, displayWidth,
+		                        displayY, displayYA, drawSprites);
 		workFrame->setLineWidth(fromY, 640);
 		++fromY;
 		++displayY;
@@ -285,8 +278,8 @@ void V9990SDLRasterizer<Pixel>::drawBxMode(
 		// plus 3 pixels cannot go beyond the end of the buffer.
 		unsigned y = scrollYBase + ((displayYA + scrollY) & rollMask);
 		Pixel* pixelPtr = workFrame->getLinePtrDirect<Pixel>(fromY) + fromX;
-		bitmapConverter->convertLine(pixelPtr, x, y, displayWidth,
-		                             cursorY, drawSprites);
+		bitmapConverter.convertLine(pixelPtr, x, y, displayWidth,
+		                            cursorY, drawSprites);
 		workFrame->setLineWidth(fromY, vdp.getLineWidth());
 		++fromY;
 		displayYA += lineStep;
@@ -299,15 +292,28 @@ template <class Pixel>
 void V9990SDLRasterizer<Pixel>::preCalcPalettes()
 {
 	// the 32768 color palette
-	for (int g = 0; g < 32; ++g) {
-		for (int r = 0; r < 32; ++r) {
-			for (int b = 0; b < 32; ++b) {
-				double dr = r / 31.0;
-				double dg = g / 31.0;
-				double db = b / 31.0;
-				renderSettings.transformRGB(dr, dg, db);
-				palette32768[(g << 10) + (r << 5) + b] =
-					screen.mapRGB(dr, dg, db);
+	if (renderSettings.isColorMatrixIdentity()) {
+		// Most users use the "normal" monitor type; making this a
+		// special case speeds up palette precalculation a lot.
+		int intensity[32];
+		for (int i = 0; i < 32; ++i) {
+			intensity[i] =
+				int(255 * renderSettings.transformComponent(i / 31.0f));
+		}
+		for (int grb = 0; grb < (1 << 15); ++grb) {
+			palette32768[grb] = screen.mapKeyedRGB255<Pixel>(gl::ivec3(
+				intensity[(grb >>  5) & 31],
+				intensity[(grb >> 10) & 31],
+				intensity[(grb >>  0) & 31]));
+		}
+	} else {
+		for (int g = 0; g < 32; ++g) {
+			for (int r = 0; r < 32; ++r) {
+				for (int b = 0; b < 32; ++b) {
+					palette32768[(g << 10) + (r << 5) + b] =
+						screen.mapRGB(renderSettings.transformRGB(
+							gl::vec3(r, g, b) / 31.0f));
+				}
 			}
 		}
 	}
@@ -318,10 +324,10 @@ void V9990SDLRasterizer<Pixel>::preCalcPalettes()
 	for (int g = 0; g < 8; ++g) {
 		for (int r = 0; r < 8; ++r) {
 			for (int b = 0; b < 4; ++b) {
-				palette256[(g << 5) + (r << 2) + b] =
-					palette32768[(mapRG[g] << 10) +
-					             (mapRG[r] <<  5) +
-					              mapB [b]];
+				auto idx256 = (g << 5) | (r << 2) | b;
+				auto idx32768 = (mapRG[g] << 10) | (mapRG[r] << 5) | mapB[b];
+				palette256_32768[idx256] = idx32768;
+				palette256[idx256] = palette32768[idx32768];
 			}
 		}
 	}
@@ -333,10 +339,10 @@ template <class Pixel>
 void V9990SDLRasterizer<Pixel>::setPalette(int index,
                                            byte r, byte g, byte b, bool ys)
 {
+	auto idx32768 = ((g & 31) << 10) | ((r & 31) << 5) | ((b & 31) << 0);
+	palette64_32768[index & 63] = idx32768; // TODO what with ys?
 	palette64[index & 63] = ys ? screen.getKeyColor<Pixel>()
-	                           : palette32768[((g & 31) << 10) +
-	                                          ((r & 31) <<  5) +
-	                                          ((b & 31) <<  0)];
+	                           : palette32768[idx32768];
 }
 
 template <class Pixel>
@@ -351,6 +357,7 @@ void V9990SDLRasterizer<Pixel>::resetPalette()
 	}
 	palette256[0] = vdp.isSuperimposing() ? screen.getKeyColor<Pixel>()
 	                                      : palette32768[0];
+	// TODO what with palette256_32768[0]?
 }
 
 template <class Pixel>
@@ -368,10 +375,10 @@ bool V9990SDLRasterizer<Pixel>::isRecording() const
 template <class Pixel>
 void V9990SDLRasterizer<Pixel>::update(const Setting& setting)
 {
-	if ((&setting == &renderSettings.getGamma()) ||
-	    (&setting == &renderSettings.getBrightness()) ||
-	    (&setting == &renderSettings.getContrast()) ||
-	    (&setting == &renderSettings.getColorMatrix())) {
+	if (&setting == one_of(&renderSettings.getGammaSetting(),
+	                       &renderSettings.getBrightnessSetting(),
+	                       &renderSettings.getContrastSetting(),
+	                       &renderSettings.getColorMatrixSetting())) {
 		preCalcPalettes();
 	}
 }
